/* Generated by Cython 0.29.26 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": [],
        "name": "tables.indexesextension",
        "sources": [
            "tables/indexesextension.pyx"
        ]
    },
    "module_name": "tables.indexesextension"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_26"
#define CYTHON_HEX_VERSION 0x001D1AF0
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0 || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL (PY_VERSION_HEX < 0x030B00A1)
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS (PY_VERSION_HEX >= 0x030600B1)
  #endif
  #ifndef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_MAJOR_VERSION < 3
    #include "longintrepr.h"
  #endif
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_DefaultClassType PyType_Type
#if PY_VERSION_HEX >= 0x030B00A1
    static CYTHON_INLINE PyCodeObject* __Pyx_PyCode_New(int a, int k, int l, int s, int f,
                                                    PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                                    PyObject *fv, PyObject *cell, PyObject* fn,
                                                    PyObject *name, int fline, PyObject *lnos) {
        PyObject *kwds=NULL, *argcount=NULL, *posonlyargcount=NULL, *kwonlyargcount=NULL;
        PyObject *nlocals=NULL, *stacksize=NULL, *flags=NULL, *replace=NULL, *call_result=NULL, *empty=NULL;
        const char *fn_cstr=NULL;
        const char *name_cstr=NULL;
        PyCodeObject* co=NULL;
        PyObject *type, *value, *traceback;
        PyErr_Fetch(&type, &value, &traceback);
        if (!(kwds=PyDict_New())) goto end;
        if (!(argcount=PyLong_FromLong(a))) goto end;
        if (PyDict_SetItemString(kwds, "co_argcount", argcount) != 0) goto end;
        if (!(posonlyargcount=PyLong_FromLong(0))) goto end;
        if (PyDict_SetItemString(kwds, "co_posonlyargcount", posonlyargcount) != 0) goto end;
        if (!(kwonlyargcount=PyLong_FromLong(k))) goto end;
        if (PyDict_SetItemString(kwds, "co_kwonlyargcount", kwonlyargcount) != 0) goto end;
        if (!(nlocals=PyLong_FromLong(l))) goto end;
        if (PyDict_SetItemString(kwds, "co_nlocals", nlocals) != 0) goto end;
        if (!(stacksize=PyLong_FromLong(s))) goto end;
        if (PyDict_SetItemString(kwds, "co_stacksize", stacksize) != 0) goto end;
        if (!(flags=PyLong_FromLong(f))) goto end;
        if (PyDict_SetItemString(kwds, "co_flags", flags) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_code", code) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_consts", c) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_names", n) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_varnames", v) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_freevars", fv) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_cellvars", cell) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_linetable", lnos) != 0) goto end;
        if (!(fn_cstr=PyUnicode_AsUTF8AndSize(fn, NULL))) goto end;
        if (!(name_cstr=PyUnicode_AsUTF8AndSize(name, NULL))) goto end;
        if (!(co = PyCode_NewEmpty(fn_cstr, name_cstr, fline))) goto end;
        if (!(replace = PyObject_GetAttrString((PyObject*)co, "replace"))) goto cleanup_code_too;
        if (!(empty = PyTuple_New(0))) goto cleanup_code_too; // unfortunately __pyx_empty_tuple isn't available here
        if (!(call_result = PyObject_Call(replace, empty, kwds))) goto cleanup_code_too;
        Py_XDECREF((PyObject*)co);
        co = (PyCodeObject*)call_result;
        call_result = NULL;
        if (0) {
            cleanup_code_too:
            Py_XDECREF((PyObject*)co);
            co = NULL;
        }
        end:
        Py_XDECREF(kwds);
        Py_XDECREF(argcount);
        Py_XDECREF(posonlyargcount);
        Py_XDECREF(kwonlyargcount);
        Py_XDECREF(nlocals);
        Py_XDECREF(stacksize);
        Py_XDECREF(replace);
        Py_XDECREF(call_result);
        Py_XDECREF(empty);
        if (type) {
            PyErr_Restore(type, value, traceback);
        }
        return co;
    }
#else
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #if defined(PyUnicode_IS_READY)
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #else
  #define __Pyx_PyUnicode_READY(op)       (0)
  #endif
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #if defined(PyUnicode_IS_READY) && defined(PyUnicode_GET_SIZE)
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
  #endif
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsHash_t
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? ((void)(klass), PyMethod_New(func, self)) : __Pyx_NewRef(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__; (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__tables__indexesextension
#define __PYX_HAVE_API__tables__indexesextension
/* Early includes */
#include <string.h>
#include <stdio.h>
#include "numpy/arrayobject.h"
#include "numpy/ndarrayobject.h"
#include "numpy/ndarraytypes.h"
#include "numpy/arrayscalars.h"
#include "numpy/ufuncobject.h"

    /* NumPy API declarations from "numpy/__init__.pxd" */
    
#include "time.h"
#include "hdf5.h"
#include "H5ATTR.h"
#include "H5ARRAY.h"
#include "utils.h"
#include "typeconv.h"
#include "blosc_filter.h"
#include <stdlib.h>
#include "H5ARRAY-opt.h"
#include "idx-opt.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;

/* Header.proto */
#if !defined(CYTHON_CCOMPLEX)
  #if defined(__cplusplus)
    #define CYTHON_CCOMPLEX 1
  #elif defined(_Complex_I)
    #define CYTHON_CCOMPLEX 1
  #else
    #define CYTHON_CCOMPLEX 0
  #endif
#endif
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #include <complex>
  #else
    #include <complex.h>
  #endif
#endif
#if CYTHON_CCOMPLEX && !defined(__cplusplus) && defined(__sun__) && defined(__GNUC__)
  #undef _Complex_I
  #define _Complex_I 1.0fj
#endif


static const char *__pyx_f[] = {
  "tables/indexesextension.pyx",
  "stringsource",
  "__init__.pxd",
  "type.pxd",
  "tables/hdf5extension.pxd",
  "tables/lrucacheextension.pxd",
};
/* NoFastGil.proto */
#define __Pyx_PyGILState_Ensure PyGILState_Ensure
#define __Pyx_PyGILState_Release PyGILState_Release
#define __Pyx_FastGIL_Remember()
#define __Pyx_FastGIL_Forget()
#define __Pyx_FastGilFuncInit()

/* ForceInitThreads.proto */
#ifndef __PYX_FORCE_INIT_THREADS
  #define __PYX_FORCE_INIT_THREADS 0
#endif


/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":690
 * # in Cython to enable them only on the right systems.
 * 
 * ctypedef npy_int8       int8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 */
typedef npy_int8 __pyx_t_5numpy_int8_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":691
 * 
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t
 */
typedef npy_int16 __pyx_t_5numpy_int16_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":692
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int64      int64_t
 * #ctypedef npy_int96      int96_t
 */
typedef npy_int32 __pyx_t_5numpy_int32_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":693
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_int96      int96_t
 * #ctypedef npy_int128     int128_t
 */
typedef npy_int64 __pyx_t_5numpy_int64_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":697
 * #ctypedef npy_int128     int128_t
 * 
 * ctypedef npy_uint8      uint8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 */
typedef npy_uint8 __pyx_t_5numpy_uint8_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":698
 * 
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t
 */
typedef npy_uint16 __pyx_t_5numpy_uint16_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":699
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint64     uint64_t
 * #ctypedef npy_uint96     uint96_t
 */
typedef npy_uint32 __pyx_t_5numpy_uint32_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":700
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_uint96     uint96_t
 * #ctypedef npy_uint128    uint128_t
 */
typedef npy_uint64 __pyx_t_5numpy_uint64_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":704
 * #ctypedef npy_uint128    uint128_t
 * 
 * ctypedef npy_float32    float32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_float64    float64_t
 * #ctypedef npy_float80    float80_t
 */
typedef npy_float32 __pyx_t_5numpy_float32_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":705
 * 
 * ctypedef npy_float32    float32_t
 * ctypedef npy_float64    float64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_float80    float80_t
 * #ctypedef npy_float128   float128_t
 */
typedef npy_float64 __pyx_t_5numpy_float64_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":714
 * # The int types are mapped a bit surprising --
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t
 */
typedef npy_long __pyx_t_5numpy_int_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":715
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   longlong_t
 * 
 */
typedef npy_longlong __pyx_t_5numpy_long_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":716
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_ulong      uint_t
 */
typedef npy_longlong __pyx_t_5numpy_longlong_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":718
 * ctypedef npy_longlong   longlong_t
 * 
 * ctypedef npy_ulong      uint_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t
 */
typedef npy_ulong __pyx_t_5numpy_uint_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":719
 * 
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 */
typedef npy_ulonglong __pyx_t_5numpy_ulong_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":720
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_intp       intp_t
 */
typedef npy_ulonglong __pyx_t_5numpy_ulonglong_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":722
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 * ctypedef npy_intp       intp_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uintp      uintp_t
 * 
 */
typedef npy_intp __pyx_t_5numpy_intp_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":723
 * 
 * ctypedef npy_intp       intp_t
 * ctypedef npy_uintp      uintp_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_double     float_t
 */
typedef npy_uintp __pyx_t_5numpy_uintp_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":725
 * ctypedef npy_uintp      uintp_t
 * 
 * ctypedef npy_double     float_t             # <<<<<<<<<<<<<<
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t
 */
typedef npy_double __pyx_t_5numpy_float_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":726
 * 
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longdouble longdouble_t
 * 
 */
typedef npy_double __pyx_t_5numpy_double_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":727
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cfloat      cfloat_t
 */
typedef npy_longdouble __pyx_t_5numpy_longdouble_t;

/* "tables/indexesextension.pyx":42
 * 
 * # These two types are defined in npy_common.h but not in cython's numpy.pxd
 * ctypedef unsigned char npy_bool             # <<<<<<<<<<<<<<
 * ctypedef npy_uint16 npy_float16
 * 
 */
typedef unsigned char __pyx_t_6tables_16indexesextension_npy_bool;

/* "tables/indexesextension.pyx":43
 * # These two types are defined in npy_common.h but not in cython's numpy.pxd
 * ctypedef unsigned char npy_bool
 * ctypedef npy_uint16 npy_float16             # <<<<<<<<<<<<<<
 * 
 * from libc.stdlib cimport malloc, free
 */
typedef npy_uint16 __pyx_t_6tables_16indexesextension_npy_float16;
/* Declarations.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< float > __pyx_t_float_complex;
  #else
    typedef float _Complex __pyx_t_float_complex;
  #endif
#else
    typedef struct { float real, imag; } __pyx_t_float_complex;
#endif
static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float, float);

/* Declarations.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< double > __pyx_t_double_complex;
  #else
    typedef double _Complex __pyx_t_double_complex;
  #endif
#else
    typedef struct { double real, imag; } __pyx_t_double_complex;
#endif
static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double, double);


/*--- Type declarations ---*/
struct __pyx_obj_6tables_13hdf5extension_Node;
struct __pyx_obj_6tables_13hdf5extension_Leaf;
struct __pyx_obj_6tables_13hdf5extension_Array;
struct __pyx_obj_6tables_17lrucacheextension_NodeCache;
struct __pyx_obj_6tables_17lrucacheextension_BaseCache;
struct __pyx_obj_6tables_17lrucacheextension_ObjectNode;
struct __pyx_obj_6tables_17lrucacheextension_ObjectCache;
struct __pyx_obj_6tables_17lrucacheextension_NumCache;
struct __pyx_obj_6tables_16indexesextension_Index;
struct __pyx_obj_6tables_16indexesextension_CacheArray;
struct __pyx_obj_6tables_16indexesextension_IndexArray;
struct __pyx_obj_6tables_16indexesextension_LastRowArray;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":729
 * ctypedef npy_longdouble longdouble_t
 * 
 * ctypedef npy_cfloat      cfloat_t             # <<<<<<<<<<<<<<
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t
 */
typedef npy_cfloat __pyx_t_5numpy_cfloat_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":730
 * 
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t             # <<<<<<<<<<<<<<
 * ctypedef npy_clongdouble clongdouble_t
 * 
 */
typedef npy_cdouble __pyx_t_5numpy_cdouble_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":731
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cdouble     complex_t
 */
typedef npy_clongdouble __pyx_t_5numpy_clongdouble_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":733
 * ctypedef npy_clongdouble clongdouble_t
 * 
 * ctypedef npy_cdouble     complex_t             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 */
typedef npy_cdouble __pyx_t_5numpy_complex_t;

/* "hdf5extension.pxd":16
 * 
 * # Declaration of instance variables for shared classes
 * cdef class Node:             # <<<<<<<<<<<<<<
 *   cdef object name
 *   cdef hid_t  parent_id
 */
struct __pyx_obj_6tables_13hdf5extension_Node {
  PyObject_HEAD
  PyObject *name;
  hid_t parent_id;
};


/* "hdf5extension.pxd":20
 *   cdef hid_t  parent_id
 * 
 * cdef class Leaf(Node):             # <<<<<<<<<<<<<<
 *   cdef hid_t   dataset_id
 *   cdef hid_t   type_id
 */
struct __pyx_obj_6tables_13hdf5extension_Leaf {
  struct __pyx_obj_6tables_13hdf5extension_Node __pyx_base;
  struct __pyx_vtabstruct_6tables_13hdf5extension_Leaf *__pyx_vtab;
  hid_t dataset_id;
  hid_t type_id;
  hid_t base_type_id;
  hid_t disk_type_id;
  hsize_t *dims;
};


/* "hdf5extension.pxd":29
 *   cdef _convert_time64(self, ndarray nparr, int sense)
 * 
 * cdef class Array(Leaf):             # <<<<<<<<<<<<<<
 *   cdef int      rank
 *   cdef hsize_t *maxdims
 */
struct __pyx_obj_6tables_13hdf5extension_Array {
  struct __pyx_obj_6tables_13hdf5extension_Leaf __pyx_base;
  int rank;
  hsize_t *maxdims;
  hsize_t *dims_chunk;
};


/* "lrucacheextension.pxd":15
 * # Declaration of instance variables for shared classes
 * # The NodeCache class is useful for caching general objects (like Nodes).
 * cdef class NodeCache:             # <<<<<<<<<<<<<<
 *   cdef readonly long nslots
 *   cdef long nextslot
 */
struct __pyx_obj_6tables_17lrucacheextension_NodeCache {
  PyObject_HEAD
  struct __pyx_vtabstruct_6tables_17lrucacheextension_NodeCache *__pyx_vtab;
  long nslots;
  long nextslot;
  PyObject *nodes;
  PyObject *paths;
};


/* "lrucacheextension.pxd":25
 * 
 * # Base class for other caches
 * cdef class BaseCache:             # <<<<<<<<<<<<<<
 *   cdef int iscachedisabled, incsetcount
 *   cdef long setcount, getcount, containscount
 */
struct __pyx_obj_6tables_17lrucacheextension_BaseCache {
  PyObject_HEAD
  struct __pyx_vtabstruct_6tables_17lrucacheextension_BaseCache *__pyx_vtab;
  int iscachedisabled;
  int incsetcount;
  long setcount;
  long getcount;
  long containscount;
  long disablecyclecount;
  long disableeverycycles;
  long enablecyclecount;
  long enableeverycycles;
  double nprobes;
  double hitratio;
  long seqn_;
  long nextslot;
  long nslots;
  long *ratimes;
  double lowesthr;
  PyArrayObject *atimes;
  PyObject *name;
};


/* "lrucacheextension.pxd":42
 * 
 * #  Helper class for ObjectCache
 * cdef class ObjectNode:             # <<<<<<<<<<<<<<
 *   cdef object key, obj
 *   cdef long nslot
 */
struct __pyx_obj_6tables_17lrucacheextension_ObjectNode {
  PyObject_HEAD
  PyObject *key;
  PyObject *obj;
  long nslot;
};


/* "lrucacheextension.pxd":48
 * 
 * # The ObjectCache class is useful for general python objects
 * cdef class ObjectCache(BaseCache):             # <<<<<<<<<<<<<<
 *   cdef long maxcachesize, cachesize, maxobjsize
 *   cdef long *rsizes
 */
struct __pyx_obj_6tables_17lrucacheextension_ObjectCache {
  struct __pyx_obj_6tables_17lrucacheextension_BaseCache __pyx_base;
  long maxcachesize;
  long cachesize;
  long maxobjsize;
  long *rsizes;
  PyArrayObject *sizes;
  PyObject *__pyx___list;
  PyObject *__pyx___dict;
  struct __pyx_obj_6tables_17lrucacheextension_ObjectNode *mrunode;
};


/* "lrucacheextension.pxd":63
 * 
 * # The NumCache class is useful for caching numerical data in an efficient way
 * cdef class NumCache(BaseCache):             # <<<<<<<<<<<<<<
 *   cdef long itemsize, slotsize
 *   cdef ndarray cacheobj, keys
 */
struct __pyx_obj_6tables_17lrucacheextension_NumCache {
  struct __pyx_obj_6tables_17lrucacheextension_BaseCache __pyx_base;
  long itemsize;
  long slotsize;
  PyArrayObject *cacheobj;
  PyArrayObject *keys;
  void *rcache;
  PY_LONG_LONG *rkeys;
  PyObject *__pyx___dict;
};


/* "tables/indexesextension.pyx":555
 * 
 * 
 * cdef class Index:             # <<<<<<<<<<<<<<
 *   pass
 * 
 */
struct __pyx_obj_6tables_16indexesextension_Index {
  PyObject_HEAD
};


/* "tables/indexesextension.pyx":559
 * 
 * 
 * cdef class CacheArray(Array):             # <<<<<<<<<<<<<<
 *   """Container for keeping index caches of 1st and 2nd level."""
 * 
 */
struct __pyx_obj_6tables_16indexesextension_CacheArray {
  struct __pyx_obj_6tables_13hdf5extension_Array __pyx_base;
  hid_t mem_space_id;
};


/* "tables/indexesextension.pyx":589
 * 
 * 
 * cdef class IndexArray(Array):             # <<<<<<<<<<<<<<
 *   """Container for keeping sorted and indices values."""
 * 
 */
struct __pyx_obj_6tables_16indexesextension_IndexArray {
  struct __pyx_obj_6tables_13hdf5extension_Array __pyx_base;
  void *rbufst;
  void *rbufln;
  void *rbufrv;
  void *rbufbc;
  void *rbuflb;
  hid_t mem_space_id;
  int l_chunksize;
  int l_slicesize;
  int nbounds;
  int indsize;
  struct __pyx_obj_6tables_16indexesextension_CacheArray *bounds_ext;
  struct __pyx_obj_6tables_17lrucacheextension_NumCache *boundscache;
  struct __pyx_obj_6tables_17lrucacheextension_NumCache *sortedcache;
  PyArrayObject *bufferbc;
  PyArrayObject *bufferlb;
};


/* "tables/indexesextension.pyx":1493
 * 
 * 
 * cdef class LastRowArray(Array):             # <<<<<<<<<<<<<<
 *   """
 *   Container for keeping sorted and indices values of last rows of an index.
 */
struct __pyx_obj_6tables_16indexesextension_LastRowArray {
  struct __pyx_obj_6tables_13hdf5extension_Array __pyx_base;
};



/* "hdf5extension.pxd":20
 *   cdef hid_t  parent_id
 * 
 * cdef class Leaf(Node):             # <<<<<<<<<<<<<<
 *   cdef hid_t   dataset_id
 *   cdef hid_t   type_id
 */

struct __pyx_vtabstruct_6tables_13hdf5extension_Leaf {
  PyObject *(*_get_type_ids)(struct __pyx_obj_6tables_13hdf5extension_Leaf *);
  PyObject *(*_convert_time64)(struct __pyx_obj_6tables_13hdf5extension_Leaf *, PyArrayObject *, int);
};
static struct __pyx_vtabstruct_6tables_13hdf5extension_Leaf *__pyx_vtabptr_6tables_13hdf5extension_Leaf;


/* "hdf5extension.pxd":29
 *   cdef _convert_time64(self, ndarray nparr, int sense)
 * 
 * cdef class Array(Leaf):             # <<<<<<<<<<<<<<
 *   cdef int      rank
 *   cdef hsize_t *maxdims
 */

struct __pyx_vtabstruct_6tables_13hdf5extension_Array {
  struct __pyx_vtabstruct_6tables_13hdf5extension_Leaf __pyx_base;
};
static struct __pyx_vtabstruct_6tables_13hdf5extension_Array *__pyx_vtabptr_6tables_13hdf5extension_Array;


/* "lrucacheextension.pxd":15
 * # Declaration of instance variables for shared classes
 * # The NodeCache class is useful for caching general objects (like Nodes).
 * cdef class NodeCache:             # <<<<<<<<<<<<<<
 *   cdef readonly long nslots
 *   cdef long nextslot
 */

struct __pyx_vtabstruct_6tables_17lrucacheextension_NodeCache {
  PyObject *(*setitem)(struct __pyx_obj_6tables_17lrucacheextension_NodeCache *, PyObject *, PyObject *);
  long (*getslot)(struct __pyx_obj_6tables_17lrucacheextension_NodeCache *, PyObject *);
  PyObject *(*cpop)(struct __pyx_obj_6tables_17lrucacheextension_NodeCache *, PyObject *);
};
static struct __pyx_vtabstruct_6tables_17lrucacheextension_NodeCache *__pyx_vtabptr_6tables_17lrucacheextension_NodeCache;


/* "lrucacheextension.pxd":25
 * 
 * # Base class for other caches
 * cdef class BaseCache:             # <<<<<<<<<<<<<<
 *   cdef int iscachedisabled, incsetcount
 *   cdef long setcount, getcount, containscount
 */

struct __pyx_vtabstruct_6tables_17lrucacheextension_BaseCache {
  int (*checkhitratio)(struct __pyx_obj_6tables_17lrucacheextension_BaseCache *);
  int (*couldenablecache_)(struct __pyx_obj_6tables_17lrucacheextension_BaseCache *);
  long (*incseqn)(struct __pyx_obj_6tables_17lrucacheextension_BaseCache *);
};
static struct __pyx_vtabstruct_6tables_17lrucacheextension_BaseCache *__pyx_vtabptr_6tables_17lrucacheextension_BaseCache;


/* "lrucacheextension.pxd":48
 * 
 * # The ObjectCache class is useful for general python objects
 * cdef class ObjectCache(BaseCache):             # <<<<<<<<<<<<<<
 *   cdef long maxcachesize, cachesize, maxobjsize
 *   cdef long *rsizes
 */

struct __pyx_vtabstruct_6tables_17lrucacheextension_ObjectCache {
  struct __pyx_vtabstruct_6tables_17lrucacheextension_BaseCache __pyx_base;
  PyObject *(*removeslot_)(struct __pyx_obj_6tables_17lrucacheextension_ObjectCache *, long);
  PyObject *(*clearcache_)(struct __pyx_obj_6tables_17lrucacheextension_ObjectCache *);
  PyObject *(*updateslot_)(struct __pyx_obj_6tables_17lrucacheextension_ObjectCache *, long, long, PyObject *, PyObject *);
  long (*setitem_)(struct __pyx_obj_6tables_17lrucacheextension_ObjectCache *, PyObject *, PyObject *, long);
  long (*getslot_)(struct __pyx_obj_6tables_17lrucacheextension_ObjectCache *, PyObject *);
  PyObject *(*getitem_)(struct __pyx_obj_6tables_17lrucacheextension_ObjectCache *, long);
};
static struct __pyx_vtabstruct_6tables_17lrucacheextension_ObjectCache *__pyx_vtabptr_6tables_17lrucacheextension_ObjectCache;


/* "lrucacheextension.pxd":63
 * 
 * # The NumCache class is useful for caching numerical data in an efficient way
 * cdef class NumCache(BaseCache):             # <<<<<<<<<<<<<<
 *   cdef long itemsize, slotsize
 *   cdef ndarray cacheobj, keys
 */

struct __pyx_vtabstruct_6tables_17lrucacheextension_NumCache {
  struct __pyx_vtabstruct_6tables_17lrucacheextension_BaseCache __pyx_base;
  void *(*getaddrslot_)(struct __pyx_obj_6tables_17lrucacheextension_NumCache *, long);
  long (*setitem_)(struct __pyx_obj_6tables_17lrucacheextension_NumCache *, PY_LONG_LONG, void *, long);
  long (*setitem1_)(struct __pyx_obj_6tables_17lrucacheextension_NumCache *, PY_LONG_LONG);
  long (*getslot_)(struct __pyx_obj_6tables_17lrucacheextension_NumCache *, PY_LONG_LONG);
  PyObject *(*getitem_)(struct __pyx_obj_6tables_17lrucacheextension_NumCache *, long, void *, long);
  void *(*getitem1_)(struct __pyx_obj_6tables_17lrucacheextension_NumCache *, long);
};
static struct __pyx_vtabstruct_6tables_17lrucacheextension_NumCache *__pyx_vtabptr_6tables_17lrucacheextension_NumCache;


/* "tables/indexesextension.pyx":559
 * 
 * 
 * cdef class CacheArray(Array):             # <<<<<<<<<<<<<<
 *   """Container for keeping index caches of 1st and 2nd level."""
 * 
 */

struct __pyx_vtabstruct_6tables_16indexesextension_CacheArray {
  struct __pyx_vtabstruct_6tables_13hdf5extension_Array __pyx_base;
  PyObject *(*initread)(struct __pyx_obj_6tables_16indexesextension_CacheArray *, int);
  PyObject *(*read_slice)(struct __pyx_obj_6tables_16indexesextension_CacheArray *, hsize_t, hsize_t, hsize_t, void *);
};
static struct __pyx_vtabstruct_6tables_16indexesextension_CacheArray *__pyx_vtabptr_6tables_16indexesextension_CacheArray;


/* "tables/indexesextension.pyx":589
 * 
 * 
 * cdef class IndexArray(Array):             # <<<<<<<<<<<<<<
 *   """Container for keeping sorted and indices values."""
 * 
 */

struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray {
  struct __pyx_vtabstruct_6tables_13hdf5extension_Array __pyx_base;
  void *(*_g_read_sorted_slice)(struct __pyx_obj_6tables_16indexesextension_IndexArray *, hsize_t, hsize_t, hsize_t);
  void *(*get_lru_bounds)(struct __pyx_obj_6tables_16indexesextension_IndexArray *, int, int);
  void *(*get_lru_sorted)(struct __pyx_obj_6tables_16indexesextension_IndexArray *, int, int, int, int);
};
static struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *__pyx_vtabptr_6tables_16indexesextension_IndexArray;


/* "tables/indexesextension.pyx":1493
 * 
 * 
 * cdef class LastRowArray(Array):             # <<<<<<<<<<<<<<
 *   """
 *   Container for keeping sorted and indices values of last rows of an index.
 */

struct __pyx_vtabstruct_6tables_16indexesextension_LastRowArray {
  struct __pyx_vtabstruct_6tables_13hdf5extension_Array __pyx_base;
};
static struct __pyx_vtabstruct_6tables_16indexesextension_LastRowArray *__pyx_vtabptr_6tables_16indexesextension_LastRowArray;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* None.proto */
static CYTHON_INLINE long __Pyx_div_long(long, long);

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
#define __Pyx_GetModuleGlobalNameUncached(var, name)  {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
#if CYTHON_FAST_PYCALL
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif // CYTHON_FAST_PYCALL
#endif

/* PyObjectCall2Args.proto */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* SliceObject.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(
        PyObject* obj, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o, n, NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value);
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* DictGetItem.proto */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key);
#define __Pyx_PyObject_Dict_GetItem(obj, name)\
    (likely(PyDict_CheckExact(obj)) ?\
     __Pyx_PyDict_GetItem(obj, name) : PyObject_GetItem(obj, name))
#else
#define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#define __Pyx_PyObject_Dict_GetItem(obj, name)  PyObject_GetItem(obj, name)
#endif

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* None.proto */
static CYTHON_INLINE int __Pyx_div_int(int, int);

/* UnaryNegOverflows.proto */
#define UNARY_NEG_WOULD_OVERFLOW(x)\
        (((x) < 0) & ((unsigned long)(x) == 0-(unsigned long)(x)))

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* CallNextTpDealloc.proto */
static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc);

/* CallNextTpTraverse.proto */
static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse);

/* CallNextTpClear.proto */
static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_dealloc);

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto
#define __PYX_HAVE_RT_ImportType_proto
enum __Pyx_ImportType_CheckSize {
   __Pyx_ImportType_CheckSize_Error = 0,
   __Pyx_ImportType_CheckSize_Warn = 1,
   __Pyx_ImportType_CheckSize_Ignore = 2
};
static PyTypeObject *__Pyx_ImportType(PyObject* module, const char *module_name, const char *class_name, size_t size, enum __Pyx_ImportType_CheckSize check_size);
#endif

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyObject *dict);

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* GCCDiagnostics.proto */
#if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* RealImag.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #define __Pyx_CREAL(z) ((z).real())
    #define __Pyx_CIMAG(z) ((z).imag())
  #else
    #define __Pyx_CREAL(z) (__real__(z))
    #define __Pyx_CIMAG(z) (__imag__(z))
  #endif
#else
    #define __Pyx_CREAL(z) ((z).real)
    #define __Pyx_CIMAG(z) ((z).imag)
#endif
#if defined(__cplusplus) && CYTHON_CCOMPLEX\
        && (defined(_WIN32) || defined(__clang__) || (defined(__GNUC__) && (__GNUC__ >= 5 || __GNUC__ == 4 && __GNUC_MINOR__ >= 4 )) || __cplusplus >= 201103)
    #define __Pyx_SET_CREAL(z,x) ((z).real(x))
    #define __Pyx_SET_CIMAG(z,y) ((z).imag(y))
#else
    #define __Pyx_SET_CREAL(z,x) __Pyx_CREAL(z) = (x)
    #define __Pyx_SET_CIMAG(z,y) __Pyx_CIMAG(z) = (y)
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq_float(a, b)   ((a)==(b))
    #define __Pyx_c_sum_float(a, b)  ((a)+(b))
    #define __Pyx_c_diff_float(a, b) ((a)-(b))
    #define __Pyx_c_prod_float(a, b) ((a)*(b))
    #define __Pyx_c_quot_float(a, b) ((a)/(b))
    #define __Pyx_c_neg_float(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_float(z) ((z)==(float)0)
    #define __Pyx_c_conj_float(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (::std::abs(z))
        #define __Pyx_c_pow_float(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_float(z) ((z)==0)
    #define __Pyx_c_conj_float(z)    (conjf(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (cabsf(z))
        #define __Pyx_c_pow_float(a, b)  (cpowf(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex);
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex);
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex, __pyx_t_float_complex);
    #endif
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq_double(a, b)   ((a)==(b))
    #define __Pyx_c_sum_double(a, b)  ((a)+(b))
    #define __Pyx_c_diff_double(a, b) ((a)-(b))
    #define __Pyx_c_prod_double(a, b) ((a)*(b))
    #define __Pyx_c_quot_double(a, b) ((a)/(b))
    #define __Pyx_c_neg_double(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_double(z) ((z)==(double)0)
    #define __Pyx_c_conj_double(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (::std::abs(z))
        #define __Pyx_c_pow_double(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_double(z) ((z)==0)
    #define __Pyx_c_conj_double(z)    (conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (cabs(z))
        #define __Pyx_c_pow_double(a, b)  (cpow(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex);
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex);
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex, __pyx_t_double_complex);
    #endif
#endif

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE hsize_t __Pyx_PyInt_As_hsize_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE npy_uint32 __Pyx_PyInt_As_npy_uint32(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE npy_int64 __Pyx_PyInt_As_npy_int64(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE npy_uint64 __Pyx_PyInt_As_npy_uint64(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static PyObject *__pyx_f_6tables_16indexesextension_10CacheArray_initread(struct __pyx_obj_6tables_16indexesextension_CacheArray *__pyx_v_self, int __pyx_v_nbounds); /* proto*/
static PyObject *__pyx_f_6tables_16indexesextension_10CacheArray_read_slice(struct __pyx_obj_6tables_16indexesextension_CacheArray *__pyx_v_self, hsize_t __pyx_v_nrow, hsize_t __pyx_v_start, hsize_t __pyx_v_stop, void *__pyx_v_rbuf); /* proto*/
static void *__pyx_f_6tables_16indexesextension_10IndexArray__g_read_sorted_slice(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, hsize_t __pyx_v_irow, hsize_t __pyx_v_start, hsize_t __pyx_v_stop); /* proto*/
static void *__pyx_f_6tables_16indexesextension_10IndexArray_get_lru_bounds(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, int __pyx_v_nrow, int __pyx_v_nbounds); /* proto*/
static void *__pyx_f_6tables_16indexesextension_10IndexArray_get_lru_sorted(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, int __pyx_v_nrow, int __pyx_v_ncs, int __pyx_v_nchunk, int __pyx_v_cs); /* proto*/

/* Module declarations from 'cython' */

/* Module declarations from 'cpython.buffer' */

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'cpython' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'cpython.mem' */

/* Module declarations from 'numpy' */

/* Module declarations from 'numpy' */
static PyTypeObject *__pyx_ptype_5numpy_dtype = 0;
static PyTypeObject *__pyx_ptype_5numpy_flatiter = 0;
static PyTypeObject *__pyx_ptype_5numpy_broadcast = 0;
static PyTypeObject *__pyx_ptype_5numpy_ndarray = 0;
static PyTypeObject *__pyx_ptype_5numpy_generic = 0;
static PyTypeObject *__pyx_ptype_5numpy_number = 0;
static PyTypeObject *__pyx_ptype_5numpy_integer = 0;
static PyTypeObject *__pyx_ptype_5numpy_signedinteger = 0;
static PyTypeObject *__pyx_ptype_5numpy_unsignedinteger = 0;
static PyTypeObject *__pyx_ptype_5numpy_inexact = 0;
static PyTypeObject *__pyx_ptype_5numpy_floating = 0;
static PyTypeObject *__pyx_ptype_5numpy_complexfloating = 0;
static PyTypeObject *__pyx_ptype_5numpy_flexible = 0;
static PyTypeObject *__pyx_ptype_5numpy_character = 0;
static PyTypeObject *__pyx_ptype_5numpy_ufunc = 0;
static CYTHON_INLINE int __pyx_f_5numpy_import_array(void); /*proto*/

/* Module declarations from 'tables.definitions' */

/* Module declarations from 'tables.hdf5extension' */
static PyTypeObject *__pyx_ptype_6tables_13hdf5extension_Node = 0;
static PyTypeObject *__pyx_ptype_6tables_13hdf5extension_Leaf = 0;
static PyTypeObject *__pyx_ptype_6tables_13hdf5extension_Array = 0;

/* Module declarations from 'libc.stdlib' */

/* Module declarations from 'tables.lrucacheextension' */
static PyTypeObject *__pyx_ptype_6tables_17lrucacheextension_NodeCache = 0;
static PyTypeObject *__pyx_ptype_6tables_17lrucacheextension_BaseCache = 0;
static PyTypeObject *__pyx_ptype_6tables_17lrucacheextension_ObjectNode = 0;
static PyTypeObject *__pyx_ptype_6tables_17lrucacheextension_ObjectCache = 0;
static PyTypeObject *__pyx_ptype_6tables_17lrucacheextension_NumCache = 0;

/* Module declarations from 'tables.indexesextension' */
static PyTypeObject *__pyx_ptype_6tables_16indexesextension_Index = 0;
static PyTypeObject *__pyx_ptype_6tables_16indexesextension_CacheArray = 0;
static PyTypeObject *__pyx_ptype_6tables_16indexesextension_IndexArray = 0;
static PyTypeObject *__pyx_ptype_6tables_16indexesextension_LastRowArray = 0;
static CYTHON_INLINE void __pyx_f_6tables_16indexesextension_swap_bytes(char *, char *, size_t); /*proto*/
static void __pyx_f_6tables_16indexesextension__keysort_string(char *, size_t, char *, size_t, size_t); /*proto*/
static CYTHON_INLINE int __pyx_fuse_0__pyx_f_6tables_16indexesextension_less_than(npy_int8 *, npy_int8 *); /*proto*/
static CYTHON_INLINE int __pyx_fuse_1__pyx_f_6tables_16indexesextension_less_than(npy_int16 *, npy_int16 *); /*proto*/
static CYTHON_INLINE int __pyx_fuse_2__pyx_f_6tables_16indexesextension_less_than(npy_int32 *, npy_int32 *); /*proto*/
static CYTHON_INLINE int __pyx_fuse_3__pyx_f_6tables_16indexesextension_less_than(npy_int64 *, npy_int64 *); /*proto*/
static CYTHON_INLINE int __pyx_fuse_4__pyx_f_6tables_16indexesextension_less_than(npy_uint8 *, npy_uint8 *); /*proto*/
static CYTHON_INLINE int __pyx_fuse_5__pyx_f_6tables_16indexesextension_less_than(npy_uint16 *, npy_uint16 *); /*proto*/
static CYTHON_INLINE int __pyx_fuse_6__pyx_f_6tables_16indexesextension_less_than(npy_uint32 *, npy_uint32 *); /*proto*/
static CYTHON_INLINE int __pyx_fuse_7__pyx_f_6tables_16indexesextension_less_than(npy_uint64 *, npy_uint64 *); /*proto*/
static CYTHON_INLINE int __pyx_fuse_8__pyx_f_6tables_16indexesextension_less_than(npy_float32 *, npy_float32 *); /*proto*/
static CYTHON_INLINE int __pyx_fuse_9__pyx_f_6tables_16indexesextension_less_than(npy_float64 *, npy_float64 *); /*proto*/
static CYTHON_INLINE int __pyx_fuse_10__pyx_f_6tables_16indexesextension_less_than(npy_longdouble *, npy_longdouble *); /*proto*/
static void __pyx_fuse_0__pyx_f_6tables_16indexesextension__keysort(npy_int8 *, char *, size_t, size_t); /*proto*/
static void __pyx_fuse_1__pyx_f_6tables_16indexesextension__keysort(npy_int16 *, char *, size_t, size_t); /*proto*/
static void __pyx_fuse_2__pyx_f_6tables_16indexesextension__keysort(npy_int32 *, char *, size_t, size_t); /*proto*/
static void __pyx_fuse_3__pyx_f_6tables_16indexesextension__keysort(npy_int64 *, char *, size_t, size_t); /*proto*/
static void __pyx_fuse_4__pyx_f_6tables_16indexesextension__keysort(npy_uint8 *, char *, size_t, size_t); /*proto*/
static void __pyx_fuse_5__pyx_f_6tables_16indexesextension__keysort(npy_uint16 *, char *, size_t, size_t); /*proto*/
static void __pyx_fuse_6__pyx_f_6tables_16indexesextension__keysort(npy_uint32 *, char *, size_t, size_t); /*proto*/
static void __pyx_fuse_7__pyx_f_6tables_16indexesextension__keysort(npy_uint64 *, char *, size_t, size_t); /*proto*/
static void __pyx_fuse_8__pyx_f_6tables_16indexesextension__keysort(npy_float32 *, char *, size_t, size_t); /*proto*/
static void __pyx_fuse_9__pyx_f_6tables_16indexesextension__keysort(npy_float64 *, char *, size_t, size_t); /*proto*/
static void __pyx_fuse_10__pyx_f_6tables_16indexesextension__keysort(npy_longdouble *, char *, size_t, size_t); /*proto*/
static PyObject *__pyx_f_6tables_16indexesextension___pyx_unpickle_Index__set_state(struct __pyx_obj_6tables_16indexesextension_Index *, PyObject *); /*proto*/
#define __Pyx_MODULE_NAME "tables.indexesextension"
extern int __pyx_module_is_main_tables__indexesextension;
int __pyx_module_is_main_tables__indexesextension = 0;

/* Implementation of 'tables.indexesextension' */
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_super;
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_ImportError;
static const char __pyx_k_a[] = "a";
static const char __pyx_k_x[] = "x";
static const char __pyx_k_hi[] = "hi";
static const char __pyx_k_lo[] = "lo";
static const char __pyx_k_idx[] = "idx";
static const char __pyx_k_mid[] = "mid";
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_atom[] = "atom";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_irow[] = "irow";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_size[] = "size";
static const char __pyx_k_stop[] = "stop";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_Index[] = "Index";
static const char __pyx_k_dtype[] = "dtype";
static const char __pyx_k_empty[] = "empty";
static const char __pyx_k_item1[] = "item1";
static const char __pyx_k_item2[] = "item2";
static const char __pyx_k_nrows[] = "nrows";
static const char __pyx_k_numpy[] = "numpy";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_shape[] = "shape";
static const char __pyx_k_start[] = "start";
static const char __pyx_k_super[] = "super";
static const char __pyx_k_array1[] = "array1";
static const char __pyx_k_array2[] = "array2";
static const char __pyx_k_bounds[] = "bounds";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_params[] = "params";
static const char __pyx_k_pickle[] = "pickle";
static const char __pyx_k_ranges[] = "ranges";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_sorted[] = "sorted";
static const char __pyx_k_starts[] = "starts";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_v_file[] = "_v_file";
static const char __pyx_k_elsize1[] = "elsize1";
static const char __pyx_k_elsize2[] = "elsize2";
static const char __pyx_k_g_close[] = "_g_close";
static const char __pyx_k_keysort[] = "keysort";
static const char __pyx_k_lengths[] = "lengths";
static const char __pyx_k_rvcache[] = "rvcache";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_itemsize[] = "itemsize";
static const char __pyx_k_pyx_type[] = "__pyx_type";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_type_num[] = "type_num";
static const char __pyx_k_v_parent[] = "_v_parent";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_chunksize[] = "chunksize";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_slicesize[] = "slicesize";
static const char __pyx_k_CacheArray[] = "CacheArray";
static const char __pyx_k_IndexArray[] = "IndexArray";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_exceptions[] = "exceptions";
static const char __pyx_k_pyx_result[] = "__pyx_result";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_ImportError[] = "ImportError";
static const char __pyx_k_PickleError[] = "PickleError";
static const char __pyx_k_bisect_left[] = "_bisect_left";
static const char __pyx_k_HDF5ExtError[] = "HDF5ExtError";
static const char __pyx_k_LastRowArray[] = "LastRowArray";
static const char __pyx_k_bisect_right[] = "_bisect_right";
static const char __pyx_k_pyx_checksum[] = "__pyx_checksum";
static const char __pyx_k_stringsource[] = "stringsource";
static const char __pyx_k_v_chunkshape[] = "_v_chunkshape";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_BOUNDS_MAX_SIZE[] = "BOUNDS_MAX_SIZE";
static const char __pyx_k_SORTED_MAX_SIZE[] = "SORTED_MAX_SIZE";
static const char __pyx_k_pyx_PickleError[] = "__pyx_PickleError";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_opt_search_types[] = "opt_search_types";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_pyx_unpickle_Index[] = "__pyx_unpickle_Index";
static const char __pyx_k_non_opt_types_bounds[] = "non-opt types bounds";
static const char __pyx_k_Unknown_array_datatype[] = "Unknown array datatype";
static const char __pyx_k_tables_indexesextension[] = "tables.indexesextension";
static const char __pyx_k_tables_indexesextension_pyx[] = "tables/indexesextension.pyx";
static const char __pyx_k_Problems_reading_the_array_data[] = "Problems reading the array data.";
static const char __pyx_k_Problems_reading_the_index_data[] = "Problems reading the index data in Last Row.";
static const char __pyx_k_numpy_core_multiarray_failed_to[] = "numpy.core.multiarray failed to import";
static const char __pyx_k_Incompatible_checksums_s_vs_0xd4[] = "Incompatible checksums (%s vs 0xd41d8cd = ())";
static const char __pyx_k_Problems_initializing_the_bounds[] = "Problems initializing the bounds array data.";
static const char __pyx_k_Problems_reading_the_bounds_arra[] = "Problems reading the bounds array data.";
static const char __pyx_k_Problems_reading_the_index_indic[] = "Problems reading the index indices.";
static const char __pyx_k_cython_interface_for_keeping_ind[] = "cython interface for keeping indexes classes.\n\nClasses (type extensions):\n\n    IndexArray\n    CacheArray\n    LastRowArray\n\nFunctions:\n\n    keysort\n\nMisc variables:\n\n";
static const char __pyx_k_numpy_core_umath_failed_to_impor[] = "numpy.core.umath failed to import";
static const char __pyx_k_self_dims_self_dims_chunk_self_m[] = "self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling";
static const char __pyx_k_Problems_reading_the_index_data_2[] = "Problems reading the index data.";
static const char __pyx_k_self_dims_self_dims_chunk_self_m_2[] = "self.dims,self.dims_chunk,self.maxdims,self.rbufbc,self.rbuflb,self.rbufln,self.rbufrv,self.rbufst cannot be converted to a Python object for pickling";
static PyObject *__pyx_n_s_BOUNDS_MAX_SIZE;
static PyObject *__pyx_n_s_CacheArray;
static PyObject *__pyx_n_s_HDF5ExtError;
static PyObject *__pyx_n_s_ImportError;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0xd4;
static PyObject *__pyx_n_s_Index;
static PyObject *__pyx_n_s_IndexArray;
static PyObject *__pyx_n_s_LastRowArray;
static PyObject *__pyx_n_s_PickleError;
static PyObject *__pyx_kp_s_Problems_initializing_the_bounds;
static PyObject *__pyx_kp_s_Problems_reading_the_array_data;
static PyObject *__pyx_kp_s_Problems_reading_the_bounds_arra;
static PyObject *__pyx_kp_s_Problems_reading_the_index_data;
static PyObject *__pyx_kp_s_Problems_reading_the_index_data_2;
static PyObject *__pyx_kp_s_Problems_reading_the_index_indic;
static PyObject *__pyx_n_s_SORTED_MAX_SIZE;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_kp_s_Unknown_array_datatype;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_n_s_a;
static PyObject *__pyx_n_s_array1;
static PyObject *__pyx_n_s_array2;
static PyObject *__pyx_n_s_atom;
static PyObject *__pyx_n_s_bisect_left;
static PyObject *__pyx_n_s_bisect_right;
static PyObject *__pyx_n_s_bounds;
static PyObject *__pyx_n_s_chunksize;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_dict;
static PyObject *__pyx_n_s_dtype;
static PyObject *__pyx_n_s_elsize1;
static PyObject *__pyx_n_s_elsize2;
static PyObject *__pyx_n_s_empty;
static PyObject *__pyx_n_s_exceptions;
static PyObject *__pyx_n_s_g_close;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_hi;
static PyObject *__pyx_n_s_idx;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_irow;
static PyObject *__pyx_n_s_item1;
static PyObject *__pyx_n_s_item2;
static PyObject *__pyx_n_s_itemsize;
static PyObject *__pyx_n_s_keysort;
static PyObject *__pyx_n_s_lengths;
static PyObject *__pyx_n_s_lo;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_mid;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_new;
static PyObject *__pyx_kp_s_non_opt_types_bounds;
static PyObject *__pyx_n_s_nrows;
static PyObject *__pyx_n_s_numpy;
static PyObject *__pyx_kp_s_numpy_core_multiarray_failed_to;
static PyObject *__pyx_kp_s_numpy_core_umath_failed_to_impor;
static PyObject *__pyx_n_s_opt_search_types;
static PyObject *__pyx_n_s_params;
static PyObject *__pyx_n_s_pickle;
static PyObject *__pyx_n_s_pyx_PickleError;
static PyObject *__pyx_n_s_pyx_checksum;
static PyObject *__pyx_n_s_pyx_result;
static PyObject *__pyx_n_s_pyx_state;
static PyObject *__pyx_n_s_pyx_type;
static PyObject *__pyx_n_s_pyx_unpickle_Index;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_ranges;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_rvcache;
static PyObject *__pyx_kp_s_self_dims_self_dims_chunk_self_m;
static PyObject *__pyx_kp_s_self_dims_self_dims_chunk_self_m_2;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_shape;
static PyObject *__pyx_n_s_size;
static PyObject *__pyx_n_s_slicesize;
static PyObject *__pyx_n_s_sorted;
static PyObject *__pyx_n_s_start;
static PyObject *__pyx_n_s_starts;
static PyObject *__pyx_n_s_stop;
static PyObject *__pyx_kp_s_stringsource;
static PyObject *__pyx_n_s_super;
static PyObject *__pyx_n_s_tables_indexesextension;
static PyObject *__pyx_kp_s_tables_indexesextension_pyx;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_type_num;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_n_s_v_chunkshape;
static PyObject *__pyx_n_s_v_file;
static PyObject *__pyx_n_s_v_parent;
static PyObject *__pyx_n_s_x;
static PyObject *__pyx_pf_6tables_16indexesextension_keysort(CYTHON_UNUSED PyObject *__pyx_self, PyArrayObject *__pyx_v_array1, PyArrayObject *__pyx_v_array2); /* proto */
static PyObject *__pyx_pf_6tables_16indexesextension_2_bisect_left(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_a, PyObject *__pyx_v_x, int __pyx_v_hi); /* proto */
static PyObject *__pyx_pf_6tables_16indexesextension_4_bisect_right(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_a, PyObject *__pyx_v_x, int __pyx_v_hi); /* proto */
static PyObject *__pyx_pf_6tables_16indexesextension_5Index___reduce_cython__(struct __pyx_obj_6tables_16indexesextension_Index *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_16indexesextension_5Index_2__setstate_cython__(struct __pyx_obj_6tables_16indexesextension_Index *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6tables_16indexesextension_10CacheArray__g_close(struct __pyx_obj_6tables_16indexesextension_CacheArray *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_16indexesextension_10CacheArray_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_16indexesextension_CacheArray *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_16indexesextension_10CacheArray_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_16indexesextension_CacheArray *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray__read_index_slice(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, hsize_t __pyx_v_irow, hsize_t __pyx_v_start, hsize_t __pyx_v_stop, PyArrayObject *__pyx_v_idx); /* proto */
static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_2_init_sorted_slice(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, PyObject *__pyx_v_index); /* proto */
static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_4_read_sorted_slice(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, hsize_t __pyx_v_irow, hsize_t __pyx_v_start, hsize_t __pyx_v_stop); /* proto */
static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_6_search_bin_na_b(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, long __pyx_v_item1, long __pyx_v_item2); /* proto */
static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_8_search_bin_na_ub(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, long __pyx_v_item1, long __pyx_v_item2); /* proto */
static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_10_search_bin_na_s(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, long __pyx_v_item1, long __pyx_v_item2); /* proto */
static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_12_search_bin_na_us(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, long __pyx_v_item1, long __pyx_v_item2); /* proto */
static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_14_search_bin_na_i(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, long __pyx_v_item1, long __pyx_v_item2); /* proto */
static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_16_search_bin_na_ui(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, npy_uint32 __pyx_v_item1, npy_uint32 __pyx_v_item2); /* proto */
static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_18_search_bin_na_ll(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, npy_int64 __pyx_v_item1, npy_int64 __pyx_v_item2); /* proto */
static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_20_search_bin_na_ull(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, npy_uint64 __pyx_v_item1, npy_uint64 __pyx_v_item2); /* proto */
static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_22_search_bin_na_e(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, npy_float64 __pyx_v_item1, npy_float64 __pyx_v_item2); /* proto */
static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_24_search_bin_na_f(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, npy_float64 __pyx_v_item1, npy_float64 __pyx_v_item2); /* proto */
static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_26_search_bin_na_d(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, npy_float64 __pyx_v_item1, npy_float64 __pyx_v_item2); /* proto */
static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_28_search_bin_na_g(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, npy_longdouble __pyx_v_item1, npy_longdouble __pyx_v_item2); /* proto */
static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_30_g_close(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_32__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_34__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6tables_16indexesextension_12LastRowArray__read_index_slice(struct __pyx_obj_6tables_16indexesextension_LastRowArray *__pyx_v_self, hsize_t __pyx_v_start, hsize_t __pyx_v_stop, PyArrayObject *__pyx_v_idx); /* proto */
static PyObject *__pyx_pf_6tables_16indexesextension_12LastRowArray_2_read_sorted_slice(struct __pyx_obj_6tables_16indexesextension_LastRowArray *__pyx_v_self, struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_sorted, hsize_t __pyx_v_start, hsize_t __pyx_v_stop); /* proto */
static PyObject *__pyx_pf_6tables_16indexesextension_12LastRowArray_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_16indexesextension_LastRowArray *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_16indexesextension_12LastRowArray_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_16indexesextension_LastRowArray *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6tables_16indexesextension_6__pyx_unpickle_Index(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_6tables_16indexesextension_Index(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6tables_16indexesextension_CacheArray(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6tables_16indexesextension_IndexArray(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6tables_16indexesextension_LastRowArray(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_222419149;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_slice__4;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_codeobj__12;
static PyObject *__pyx_codeobj__14;
static PyObject *__pyx_codeobj__16;
static PyObject *__pyx_codeobj__18;
/* Late includes */

/* "tables/indexesextension.pyx":144
 * DEF SMALL_QUICKSORT = 15
 * 
 * def keysort(ndarray array1, ndarray array2):             # <<<<<<<<<<<<<<
 *     """Sort array1 in-place. array2 is also sorted following the array1 order.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_16indexesextension_1keysort(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6tables_16indexesextension_keysort[] = "Sort array1 in-place. array2 is also sorted following the array1 order.\n\n    array1 can be of any type, except complex or string.  array2 may be made of\n    elements on any size.\n\n    ";
static PyMethodDef __pyx_mdef_6tables_16indexesextension_1keysort = {"keysort", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6tables_16indexesextension_1keysort, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6tables_16indexesextension_keysort};
static PyObject *__pyx_pw_6tables_16indexesextension_1keysort(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyArrayObject *__pyx_v_array1 = 0;
  PyArrayObject *__pyx_v_array2 = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("keysort (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_array1,&__pyx_n_s_array2,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_array1)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_array2)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("keysort", 1, 2, 2, 1); __PYX_ERR(0, 144, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "keysort") < 0)) __PYX_ERR(0, 144, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_array1 = ((PyArrayObject *)values[0]);
    __pyx_v_array2 = ((PyArrayObject *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("keysort", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 144, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("tables.indexesextension.keysort", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_array1), __pyx_ptype_5numpy_ndarray, 1, "array1", 0))) __PYX_ERR(0, 144, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_array2), __pyx_ptype_5numpy_ndarray, 1, "array2", 0))) __PYX_ERR(0, 144, __pyx_L1_error)
  __pyx_r = __pyx_pf_6tables_16indexesextension_keysort(__pyx_self, __pyx_v_array1, __pyx_v_array2);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_16indexesextension_keysort(CYTHON_UNUSED PyObject *__pyx_self, PyArrayObject *__pyx_v_array1, PyArrayObject *__pyx_v_array2) {
  size_t __pyx_v_size;
  size_t __pyx_v_elsize1;
  size_t __pyx_v_elsize2;
  int __pyx_v_type_num;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("keysort", 0);

  /* "tables/indexesextension.pyx":151
 * 
 *     """
 *     cdef size_t size = cnp.PyArray_SIZE(array1)             # <<<<<<<<<<<<<<
 *     cdef size_t elsize1 = cnp.PyArray_ITEMSIZE(array1)
 *     cdef size_t elsize2 = cnp.PyArray_ITEMSIZE(array2)
 */
  __pyx_v_size = PyArray_SIZE(__pyx_v_array1);

  /* "tables/indexesextension.pyx":152
 *     """
 *     cdef size_t size = cnp.PyArray_SIZE(array1)
 *     cdef size_t elsize1 = cnp.PyArray_ITEMSIZE(array1)             # <<<<<<<<<<<<<<
 *     cdef size_t elsize2 = cnp.PyArray_ITEMSIZE(array2)
 *     cdef int type_num = cnp.PyArray_TYPE(array1)
 */
  __pyx_v_elsize1 = PyArray_ITEMSIZE(__pyx_v_array1);

  /* "tables/indexesextension.pyx":153
 *     cdef size_t size = cnp.PyArray_SIZE(array1)
 *     cdef size_t elsize1 = cnp.PyArray_ITEMSIZE(array1)
 *     cdef size_t elsize2 = cnp.PyArray_ITEMSIZE(array2)             # <<<<<<<<<<<<<<
 *     cdef int type_num = cnp.PyArray_TYPE(array1)
 * 
 */
  __pyx_v_elsize2 = PyArray_ITEMSIZE(__pyx_v_array2);

  /* "tables/indexesextension.pyx":154
 *     cdef size_t elsize1 = cnp.PyArray_ITEMSIZE(array1)
 *     cdef size_t elsize2 = cnp.PyArray_ITEMSIZE(array2)
 *     cdef int type_num = cnp.PyArray_TYPE(array1)             # <<<<<<<<<<<<<<
 * 
 *     # floating types
 */
  __pyx_v_type_num = PyArray_TYPE(__pyx_v_array1);

  /* "tables/indexesextension.pyx":157
 * 
 *     # floating types
 *     if type_num == cnp.NPY_FLOAT16:             # <<<<<<<<<<<<<<
 *         _keysort[npy_float16](<npy_float16*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     elif type_num == cnp.NPY_FLOAT32:
 */
  switch (__pyx_v_type_num) {
    case NPY_FLOAT16:

    /* "tables/indexesextension.pyx":158
 *     # floating types
 *     if type_num == cnp.NPY_FLOAT16:
 *         _keysort[npy_float16](<npy_float16*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)             # <<<<<<<<<<<<<<
 *     elif type_num == cnp.NPY_FLOAT32:
 *         _keysort[npy_float32](<npy_float32*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 */
    __pyx_fuse_5__pyx_f_6tables_16indexesextension__keysort(((__pyx_t_6tables_16indexesextension_npy_float16 *)PyArray_DATA(__pyx_v_array1)), PyArray_BYTES(__pyx_v_array2), __pyx_v_elsize2, __pyx_v_size);

    /* "tables/indexesextension.pyx":157
 * 
 *     # floating types
 *     if type_num == cnp.NPY_FLOAT16:             # <<<<<<<<<<<<<<
 *         _keysort[npy_float16](<npy_float16*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     elif type_num == cnp.NPY_FLOAT32:
 */
    break;
    case NPY_FLOAT32:

    /* "tables/indexesextension.pyx":160
 *         _keysort[npy_float16](<npy_float16*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     elif type_num == cnp.NPY_FLOAT32:
 *         _keysort[npy_float32](<npy_float32*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)             # <<<<<<<<<<<<<<
 *     elif type_num == cnp.NPY_FLOAT64:
 *         _keysort[npy_float64](<npy_float64*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 */
    __pyx_fuse_8__pyx_f_6tables_16indexesextension__keysort(((npy_float32 *)PyArray_DATA(__pyx_v_array1)), PyArray_BYTES(__pyx_v_array2), __pyx_v_elsize2, __pyx_v_size);

    /* "tables/indexesextension.pyx":159
 *     if type_num == cnp.NPY_FLOAT16:
 *         _keysort[npy_float16](<npy_float16*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     elif type_num == cnp.NPY_FLOAT32:             # <<<<<<<<<<<<<<
 *         _keysort[npy_float32](<npy_float32*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     elif type_num == cnp.NPY_FLOAT64:
 */
    break;
    case NPY_FLOAT64:

    /* "tables/indexesextension.pyx":162
 *         _keysort[npy_float32](<npy_float32*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     elif type_num == cnp.NPY_FLOAT64:
 *         _keysort[npy_float64](<npy_float64*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)             # <<<<<<<<<<<<<<
 *     elif type_num == cnp.NPY_LONGDOUBLE:
 *         _keysort[npy_longdouble](<npy_longdouble*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 */
    __pyx_fuse_9__pyx_f_6tables_16indexesextension__keysort(((npy_float64 *)PyArray_DATA(__pyx_v_array1)), PyArray_BYTES(__pyx_v_array2), __pyx_v_elsize2, __pyx_v_size);

    /* "tables/indexesextension.pyx":161
 *     elif type_num == cnp.NPY_FLOAT32:
 *         _keysort[npy_float32](<npy_float32*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     elif type_num == cnp.NPY_FLOAT64:             # <<<<<<<<<<<<<<
 *         _keysort[npy_float64](<npy_float64*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     elif type_num == cnp.NPY_LONGDOUBLE:
 */
    break;
    case NPY_LONGDOUBLE:

    /* "tables/indexesextension.pyx":164
 *         _keysort[npy_float64](<npy_float64*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     elif type_num == cnp.NPY_LONGDOUBLE:
 *         _keysort[npy_longdouble](<npy_longdouble*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)             # <<<<<<<<<<<<<<
 *     # signed integer types
 *     elif type_num == cnp.NPY_INT8:
 */
    __pyx_fuse_10__pyx_f_6tables_16indexesextension__keysort(((npy_longdouble *)PyArray_DATA(__pyx_v_array1)), PyArray_BYTES(__pyx_v_array2), __pyx_v_elsize2, __pyx_v_size);

    /* "tables/indexesextension.pyx":163
 *     elif type_num == cnp.NPY_FLOAT64:
 *         _keysort[npy_float64](<npy_float64*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     elif type_num == cnp.NPY_LONGDOUBLE:             # <<<<<<<<<<<<<<
 *         _keysort[npy_longdouble](<npy_longdouble*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     # signed integer types
 */
    break;
    case NPY_INT8:

    /* "tables/indexesextension.pyx":167
 *     # signed integer types
 *     elif type_num == cnp.NPY_INT8:
 *         _keysort[npy_int8](<npy_int8*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)             # <<<<<<<<<<<<<<
 *     elif type_num == cnp.NPY_INT16:
 *         _keysort[npy_int16](<npy_int16*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 */
    __pyx_fuse_0__pyx_f_6tables_16indexesextension__keysort(((npy_int8 *)PyArray_DATA(__pyx_v_array1)), PyArray_BYTES(__pyx_v_array2), __pyx_v_elsize2, __pyx_v_size);

    /* "tables/indexesextension.pyx":166
 *         _keysort[npy_longdouble](<npy_longdouble*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     # signed integer types
 *     elif type_num == cnp.NPY_INT8:             # <<<<<<<<<<<<<<
 *         _keysort[npy_int8](<npy_int8*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     elif type_num == cnp.NPY_INT16:
 */
    break;
    case NPY_INT16:

    /* "tables/indexesextension.pyx":169
 *         _keysort[npy_int8](<npy_int8*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     elif type_num == cnp.NPY_INT16:
 *         _keysort[npy_int16](<npy_int16*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)             # <<<<<<<<<<<<<<
 *     elif type_num == cnp.NPY_INT32:
 *         _keysort[npy_int32](<npy_int32*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 */
    __pyx_fuse_1__pyx_f_6tables_16indexesextension__keysort(((npy_int16 *)PyArray_DATA(__pyx_v_array1)), PyArray_BYTES(__pyx_v_array2), __pyx_v_elsize2, __pyx_v_size);

    /* "tables/indexesextension.pyx":168
 *     elif type_num == cnp.NPY_INT8:
 *         _keysort[npy_int8](<npy_int8*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     elif type_num == cnp.NPY_INT16:             # <<<<<<<<<<<<<<
 *         _keysort[npy_int16](<npy_int16*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     elif type_num == cnp.NPY_INT32:
 */
    break;
    case NPY_INT32:

    /* "tables/indexesextension.pyx":171
 *         _keysort[npy_int16](<npy_int16*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     elif type_num == cnp.NPY_INT32:
 *         _keysort[npy_int32](<npy_int32*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)             # <<<<<<<<<<<<<<
 *     elif type_num == cnp.NPY_INT64:
 *         _keysort[npy_int64](<npy_int64*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 */
    __pyx_fuse_2__pyx_f_6tables_16indexesextension__keysort(((npy_int32 *)PyArray_DATA(__pyx_v_array1)), PyArray_BYTES(__pyx_v_array2), __pyx_v_elsize2, __pyx_v_size);

    /* "tables/indexesextension.pyx":170
 *     elif type_num == cnp.NPY_INT16:
 *         _keysort[npy_int16](<npy_int16*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     elif type_num == cnp.NPY_INT32:             # <<<<<<<<<<<<<<
 *         _keysort[npy_int32](<npy_int32*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     elif type_num == cnp.NPY_INT64:
 */
    break;
    case NPY_INT64:

    /* "tables/indexesextension.pyx":173
 *         _keysort[npy_int32](<npy_int32*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     elif type_num == cnp.NPY_INT64:
 *         _keysort[npy_int64](<npy_int64*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)             # <<<<<<<<<<<<<<
 *     # unsigned integer types
 *     elif type_num == cnp.NPY_UINT8:
 */
    __pyx_fuse_3__pyx_f_6tables_16indexesextension__keysort(((npy_int64 *)PyArray_DATA(__pyx_v_array1)), PyArray_BYTES(__pyx_v_array2), __pyx_v_elsize2, __pyx_v_size);

    /* "tables/indexesextension.pyx":172
 *     elif type_num == cnp.NPY_INT32:
 *         _keysort[npy_int32](<npy_int32*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     elif type_num == cnp.NPY_INT64:             # <<<<<<<<<<<<<<
 *         _keysort[npy_int64](<npy_int64*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     # unsigned integer types
 */
    break;
    case NPY_UINT8:

    /* "tables/indexesextension.pyx":176
 *     # unsigned integer types
 *     elif type_num == cnp.NPY_UINT8:
 *         _keysort[npy_uint8](<npy_uint8*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)             # <<<<<<<<<<<<<<
 *     elif type_num == cnp.NPY_UINT16:
 *         _keysort[npy_uint16](<npy_uint16*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 */
    __pyx_fuse_4__pyx_f_6tables_16indexesextension__keysort(((npy_uint8 *)PyArray_DATA(__pyx_v_array1)), PyArray_BYTES(__pyx_v_array2), __pyx_v_elsize2, __pyx_v_size);

    /* "tables/indexesextension.pyx":175
 *         _keysort[npy_int64](<npy_int64*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     # unsigned integer types
 *     elif type_num == cnp.NPY_UINT8:             # <<<<<<<<<<<<<<
 *         _keysort[npy_uint8](<npy_uint8*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     elif type_num == cnp.NPY_UINT16:
 */
    break;
    case NPY_UINT16:

    /* "tables/indexesextension.pyx":178
 *         _keysort[npy_uint8](<npy_uint8*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     elif type_num == cnp.NPY_UINT16:
 *         _keysort[npy_uint16](<npy_uint16*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)             # <<<<<<<<<<<<<<
 *     elif type_num == cnp.NPY_UINT32:
 *         _keysort[npy_uint32](<npy_uint32*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 */
    __pyx_fuse_5__pyx_f_6tables_16indexesextension__keysort(((npy_uint16 *)PyArray_DATA(__pyx_v_array1)), PyArray_BYTES(__pyx_v_array2), __pyx_v_elsize2, __pyx_v_size);

    /* "tables/indexesextension.pyx":177
 *     elif type_num == cnp.NPY_UINT8:
 *         _keysort[npy_uint8](<npy_uint8*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     elif type_num == cnp.NPY_UINT16:             # <<<<<<<<<<<<<<
 *         _keysort[npy_uint16](<npy_uint16*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     elif type_num == cnp.NPY_UINT32:
 */
    break;
    case NPY_UINT32:

    /* "tables/indexesextension.pyx":180
 *         _keysort[npy_uint16](<npy_uint16*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     elif type_num == cnp.NPY_UINT32:
 *         _keysort[npy_uint32](<npy_uint32*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)             # <<<<<<<<<<<<<<
 *     elif type_num == cnp.NPY_UINT64:
 *         _keysort[npy_uint64](<npy_uint64*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 */
    __pyx_fuse_6__pyx_f_6tables_16indexesextension__keysort(((npy_uint32 *)PyArray_DATA(__pyx_v_array1)), PyArray_BYTES(__pyx_v_array2), __pyx_v_elsize2, __pyx_v_size);

    /* "tables/indexesextension.pyx":179
 *     elif type_num == cnp.NPY_UINT16:
 *         _keysort[npy_uint16](<npy_uint16*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     elif type_num == cnp.NPY_UINT32:             # <<<<<<<<<<<<<<
 *         _keysort[npy_uint32](<npy_uint32*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     elif type_num == cnp.NPY_UINT64:
 */
    break;
    case NPY_UINT64:

    /* "tables/indexesextension.pyx":182
 *         _keysort[npy_uint32](<npy_uint32*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     elif type_num == cnp.NPY_UINT64:
 *         _keysort[npy_uint64](<npy_uint64*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)             # <<<<<<<<<<<<<<
 *     # other
 *     elif type_num == cnp.NPY_BOOL:
 */
    __pyx_fuse_7__pyx_f_6tables_16indexesextension__keysort(((npy_uint64 *)PyArray_DATA(__pyx_v_array1)), PyArray_BYTES(__pyx_v_array2), __pyx_v_elsize2, __pyx_v_size);

    /* "tables/indexesextension.pyx":181
 *     elif type_num == cnp.NPY_UINT32:
 *         _keysort[npy_uint32](<npy_uint32*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     elif type_num == cnp.NPY_UINT64:             # <<<<<<<<<<<<<<
 *         _keysort[npy_uint64](<npy_uint64*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     # other
 */
    break;
    case NPY_BOOL:

    /* "tables/indexesextension.pyx":185
 *     # other
 *     elif type_num == cnp.NPY_BOOL:
 *         _keysort[npy_bool](<npy_bool*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)             # <<<<<<<<<<<<<<
 *     elif type_num == cnp.NPY_STRING:
 *         _keysort_string(PyArray_BYTES(array1), elsize1, PyArray_BYTES(array2), elsize2, size)
 */
    __pyx_fuse_4__pyx_f_6tables_16indexesextension__keysort(((__pyx_t_6tables_16indexesextension_npy_bool *)PyArray_DATA(__pyx_v_array1)), PyArray_BYTES(__pyx_v_array2), __pyx_v_elsize2, __pyx_v_size);

    /* "tables/indexesextension.pyx":184
 *         _keysort[npy_uint64](<npy_uint64*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     # other
 *     elif type_num == cnp.NPY_BOOL:             # <<<<<<<<<<<<<<
 *         _keysort[npy_bool](<npy_bool*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     elif type_num == cnp.NPY_STRING:
 */
    break;
    case NPY_STRING:

    /* "tables/indexesextension.pyx":187
 *         _keysort[npy_bool](<npy_bool*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     elif type_num == cnp.NPY_STRING:
 *         _keysort_string(PyArray_BYTES(array1), elsize1, PyArray_BYTES(array2), elsize2, size)             # <<<<<<<<<<<<<<
 *     else:
 *         raise ValueError("Unknown array datatype")
 */
    __pyx_f_6tables_16indexesextension__keysort_string(PyArray_BYTES(__pyx_v_array1), __pyx_v_elsize1, PyArray_BYTES(__pyx_v_array2), __pyx_v_elsize2, __pyx_v_size);

    /* "tables/indexesextension.pyx":186
 *     elif type_num == cnp.NPY_BOOL:
 *         _keysort[npy_bool](<npy_bool*>PyArray_DATA(array1), PyArray_BYTES(array2), elsize2, size)
 *     elif type_num == cnp.NPY_STRING:             # <<<<<<<<<<<<<<
 *         _keysort_string(PyArray_BYTES(array1), elsize1, PyArray_BYTES(array2), elsize2, size)
 *     else:
 */
    break;
    default:

    /* "tables/indexesextension.pyx":189
 *         _keysort_string(PyArray_BYTES(array1), elsize1, PyArray_BYTES(array2), elsize2, size)
 *     else:
 *         raise ValueError("Unknown array datatype")             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 189, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 189, __pyx_L1_error)
    break;
  }

  /* "tables/indexesextension.pyx":144
 * DEF SMALL_QUICKSORT = 15
 * 
 * def keysort(ndarray array1, ndarray array2):             # <<<<<<<<<<<<<<
 *     """Sort array1 in-place. array2 is also sorted following the array1 order.
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("tables.indexesextension.keysort", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/indexesextension.pyx":192
 * 
 * 
 * cdef inline void swap_bytes(char *x, char *y, size_t n) nogil:             # <<<<<<<<<<<<<<
 *     if n == 8:
 *         (<npy_int64*>x)[0], (<npy_int64*>y)[0] = (<npy_int64*>y)[0], (<npy_int64*>x)[0]
 */

static CYTHON_INLINE void __pyx_f_6tables_16indexesextension_swap_bytes(char *__pyx_v_x, char *__pyx_v_y, size_t __pyx_v_n) {
  size_t __pyx_v_i;
  npy_int64 __pyx_t_1;
  npy_int64 __pyx_t_2;
  npy_int32 __pyx_t_3;
  npy_int32 __pyx_t_4;
  npy_int16 __pyx_t_5;
  npy_int16 __pyx_t_6;
  size_t __pyx_t_7;
  size_t __pyx_t_8;
  size_t __pyx_t_9;
  char __pyx_t_10;
  char __pyx_t_11;

  /* "tables/indexesextension.pyx":193
 * 
 * cdef inline void swap_bytes(char *x, char *y, size_t n) nogil:
 *     if n == 8:             # <<<<<<<<<<<<<<
 *         (<npy_int64*>x)[0], (<npy_int64*>y)[0] = (<npy_int64*>y)[0], (<npy_int64*>x)[0]
 *     elif n == 4:
 */
  switch (__pyx_v_n) {
    case 8:

    /* "tables/indexesextension.pyx":194
 * cdef inline void swap_bytes(char *x, char *y, size_t n) nogil:
 *     if n == 8:
 *         (<npy_int64*>x)[0], (<npy_int64*>y)[0] = (<npy_int64*>y)[0], (<npy_int64*>x)[0]             # <<<<<<<<<<<<<<
 *     elif n == 4:
 *         (<npy_int32*>x)[0], (<npy_int32*>y)[0] = (<npy_int32*>y)[0], (<npy_int32*>x)[0]
 */
    __pyx_t_1 = (((npy_int64 *)__pyx_v_y)[0]);
    __pyx_t_2 = (((npy_int64 *)__pyx_v_x)[0]);
    (((npy_int64 *)__pyx_v_x)[0]) = __pyx_t_1;
    (((npy_int64 *)__pyx_v_y)[0]) = __pyx_t_2;

    /* "tables/indexesextension.pyx":193
 * 
 * cdef inline void swap_bytes(char *x, char *y, size_t n) nogil:
 *     if n == 8:             # <<<<<<<<<<<<<<
 *         (<npy_int64*>x)[0], (<npy_int64*>y)[0] = (<npy_int64*>y)[0], (<npy_int64*>x)[0]
 *     elif n == 4:
 */
    break;
    case 4:

    /* "tables/indexesextension.pyx":196
 *         (<npy_int64*>x)[0], (<npy_int64*>y)[0] = (<npy_int64*>y)[0], (<npy_int64*>x)[0]
 *     elif n == 4:
 *         (<npy_int32*>x)[0], (<npy_int32*>y)[0] = (<npy_int32*>y)[0], (<npy_int32*>x)[0]             # <<<<<<<<<<<<<<
 *     elif n == 2:
 *         (<npy_int16*>x)[0], (<npy_int16*>y)[0] = (<npy_int16*>y)[0], (<npy_int16*>x)[0]
 */
    __pyx_t_3 = (((npy_int32 *)__pyx_v_y)[0]);
    __pyx_t_4 = (((npy_int32 *)__pyx_v_x)[0]);
    (((npy_int32 *)__pyx_v_x)[0]) = __pyx_t_3;
    (((npy_int32 *)__pyx_v_y)[0]) = __pyx_t_4;

    /* "tables/indexesextension.pyx":195
 *     if n == 8:
 *         (<npy_int64*>x)[0], (<npy_int64*>y)[0] = (<npy_int64*>y)[0], (<npy_int64*>x)[0]
 *     elif n == 4:             # <<<<<<<<<<<<<<
 *         (<npy_int32*>x)[0], (<npy_int32*>y)[0] = (<npy_int32*>y)[0], (<npy_int32*>x)[0]
 *     elif n == 2:
 */
    break;
    case 2:

    /* "tables/indexesextension.pyx":198
 *         (<npy_int32*>x)[0], (<npy_int32*>y)[0] = (<npy_int32*>y)[0], (<npy_int32*>x)[0]
 *     elif n == 2:
 *         (<npy_int16*>x)[0], (<npy_int16*>y)[0] = (<npy_int16*>y)[0], (<npy_int16*>x)[0]             # <<<<<<<<<<<<<<
 *     else:
 *         for i in range(n):
 */
    __pyx_t_5 = (((npy_int16 *)__pyx_v_y)[0]);
    __pyx_t_6 = (((npy_int16 *)__pyx_v_x)[0]);
    (((npy_int16 *)__pyx_v_x)[0]) = __pyx_t_5;
    (((npy_int16 *)__pyx_v_y)[0]) = __pyx_t_6;

    /* "tables/indexesextension.pyx":197
 *     elif n == 4:
 *         (<npy_int32*>x)[0], (<npy_int32*>y)[0] = (<npy_int32*>y)[0], (<npy_int32*>x)[0]
 *     elif n == 2:             # <<<<<<<<<<<<<<
 *         (<npy_int16*>x)[0], (<npy_int16*>y)[0] = (<npy_int16*>y)[0], (<npy_int16*>x)[0]
 *     else:
 */
    break;
    default:

    /* "tables/indexesextension.pyx":200
 *         (<npy_int16*>x)[0], (<npy_int16*>y)[0] = (<npy_int16*>y)[0], (<npy_int16*>x)[0]
 *     else:
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             x[i], y[i] = y[i], x[i]
 * 
 */
    __pyx_t_7 = __pyx_v_n;
    __pyx_t_8 = __pyx_t_7;
    for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
      __pyx_v_i = __pyx_t_9;

      /* "tables/indexesextension.pyx":201
 *     else:
 *         for i in range(n):
 *             x[i], y[i] = y[i], x[i]             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __pyx_t_10 = (__pyx_v_y[__pyx_v_i]);
      __pyx_t_11 = (__pyx_v_x[__pyx_v_i]);
      (__pyx_v_x[__pyx_v_i]) = __pyx_t_10;
      (__pyx_v_y[__pyx_v_i]) = __pyx_t_11;
    }
    break;
  }

  /* "tables/indexesextension.pyx":192
 * 
 * 
 * cdef inline void swap_bytes(char *x, char *y, size_t n) nogil:             # <<<<<<<<<<<<<<
 *     if n == 8:
 *         (<npy_int64*>x)[0], (<npy_int64*>y)[0] = (<npy_int64*>y)[0], (<npy_int64*>x)[0]
 */

  /* function exit code */
}

/* "tables/indexesextension.pyx":204
 * 
 * 
 * cdef inline int less_than(number_type* a, number_type* b) nogil:             # <<<<<<<<<<<<<<
 *     if number_type in floating_type:
 *         return a[0] < b[0] or (b[0] != b[0] and a[0] == a[0])
 */

static CYTHON_INLINE int __pyx_fuse_0__pyx_f_6tables_16indexesextension_less_than(npy_int8 *__pyx_v_a, npy_int8 *__pyx_v_b) {
  int __pyx_r;

  /* "tables/indexesextension.pyx":208
 *         return a[0] < b[0] or (b[0] != b[0] and a[0] == a[0])
 *     else:
 *         return a[0] < b[0]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((__pyx_v_a[0]) < (__pyx_v_b[0]));
  goto __pyx_L0;

  /* "tables/indexesextension.pyx":204
 * 
 * 
 * cdef inline int less_than(number_type* a, number_type* b) nogil:             # <<<<<<<<<<<<<<
 *     if number_type in floating_type:
 *         return a[0] < b[0] or (b[0] != b[0] and a[0] == a[0])
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_1__pyx_f_6tables_16indexesextension_less_than(npy_int16 *__pyx_v_a, npy_int16 *__pyx_v_b) {
  int __pyx_r;

  /* "tables/indexesextension.pyx":208
 *         return a[0] < b[0] or (b[0] != b[0] and a[0] == a[0])
 *     else:
 *         return a[0] < b[0]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((__pyx_v_a[0]) < (__pyx_v_b[0]));
  goto __pyx_L0;

  /* "tables/indexesextension.pyx":204
 * 
 * 
 * cdef inline int less_than(number_type* a, number_type* b) nogil:             # <<<<<<<<<<<<<<
 *     if number_type in floating_type:
 *         return a[0] < b[0] or (b[0] != b[0] and a[0] == a[0])
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_2__pyx_f_6tables_16indexesextension_less_than(npy_int32 *__pyx_v_a, npy_int32 *__pyx_v_b) {
  int __pyx_r;

  /* "tables/indexesextension.pyx":208
 *         return a[0] < b[0] or (b[0] != b[0] and a[0] == a[0])
 *     else:
 *         return a[0] < b[0]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((__pyx_v_a[0]) < (__pyx_v_b[0]));
  goto __pyx_L0;

  /* "tables/indexesextension.pyx":204
 * 
 * 
 * cdef inline int less_than(number_type* a, number_type* b) nogil:             # <<<<<<<<<<<<<<
 *     if number_type in floating_type:
 *         return a[0] < b[0] or (b[0] != b[0] and a[0] == a[0])
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_3__pyx_f_6tables_16indexesextension_less_than(npy_int64 *__pyx_v_a, npy_int64 *__pyx_v_b) {
  int __pyx_r;

  /* "tables/indexesextension.pyx":208
 *         return a[0] < b[0] or (b[0] != b[0] and a[0] == a[0])
 *     else:
 *         return a[0] < b[0]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((__pyx_v_a[0]) < (__pyx_v_b[0]));
  goto __pyx_L0;

  /* "tables/indexesextension.pyx":204
 * 
 * 
 * cdef inline int less_than(number_type* a, number_type* b) nogil:             # <<<<<<<<<<<<<<
 *     if number_type in floating_type:
 *         return a[0] < b[0] or (b[0] != b[0] and a[0] == a[0])
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_4__pyx_f_6tables_16indexesextension_less_than(npy_uint8 *__pyx_v_a, npy_uint8 *__pyx_v_b) {
  int __pyx_r;

  /* "tables/indexesextension.pyx":208
 *         return a[0] < b[0] or (b[0] != b[0] and a[0] == a[0])
 *     else:
 *         return a[0] < b[0]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((__pyx_v_a[0]) < (__pyx_v_b[0]));
  goto __pyx_L0;

  /* "tables/indexesextension.pyx":204
 * 
 * 
 * cdef inline int less_than(number_type* a, number_type* b) nogil:             # <<<<<<<<<<<<<<
 *     if number_type in floating_type:
 *         return a[0] < b[0] or (b[0] != b[0] and a[0] == a[0])
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_5__pyx_f_6tables_16indexesextension_less_than(npy_uint16 *__pyx_v_a, npy_uint16 *__pyx_v_b) {
  int __pyx_r;

  /* "tables/indexesextension.pyx":208
 *         return a[0] < b[0] or (b[0] != b[0] and a[0] == a[0])
 *     else:
 *         return a[0] < b[0]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((__pyx_v_a[0]) < (__pyx_v_b[0]));
  goto __pyx_L0;

  /* "tables/indexesextension.pyx":204
 * 
 * 
 * cdef inline int less_than(number_type* a, number_type* b) nogil:             # <<<<<<<<<<<<<<
 *     if number_type in floating_type:
 *         return a[0] < b[0] or (b[0] != b[0] and a[0] == a[0])
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_6__pyx_f_6tables_16indexesextension_less_than(npy_uint32 *__pyx_v_a, npy_uint32 *__pyx_v_b) {
  int __pyx_r;

  /* "tables/indexesextension.pyx":208
 *         return a[0] < b[0] or (b[0] != b[0] and a[0] == a[0])
 *     else:
 *         return a[0] < b[0]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((__pyx_v_a[0]) < (__pyx_v_b[0]));
  goto __pyx_L0;

  /* "tables/indexesextension.pyx":204
 * 
 * 
 * cdef inline int less_than(number_type* a, number_type* b) nogil:             # <<<<<<<<<<<<<<
 *     if number_type in floating_type:
 *         return a[0] < b[0] or (b[0] != b[0] and a[0] == a[0])
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_7__pyx_f_6tables_16indexesextension_less_than(npy_uint64 *__pyx_v_a, npy_uint64 *__pyx_v_b) {
  int __pyx_r;

  /* "tables/indexesextension.pyx":208
 *         return a[0] < b[0] or (b[0] != b[0] and a[0] == a[0])
 *     else:
 *         return a[0] < b[0]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((__pyx_v_a[0]) < (__pyx_v_b[0]));
  goto __pyx_L0;

  /* "tables/indexesextension.pyx":204
 * 
 * 
 * cdef inline int less_than(number_type* a, number_type* b) nogil:             # <<<<<<<<<<<<<<
 *     if number_type in floating_type:
 *         return a[0] < b[0] or (b[0] != b[0] and a[0] == a[0])
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_8__pyx_f_6tables_16indexesextension_less_than(npy_float32 *__pyx_v_a, npy_float32 *__pyx_v_b) {
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "tables/indexesextension.pyx":206
 * cdef inline int less_than(number_type* a, number_type* b) nogil:
 *     if number_type in floating_type:
 *         return a[0] < b[0] or (b[0] != b[0] and a[0] == a[0])             # <<<<<<<<<<<<<<
 *     else:
 *         return a[0] < b[0]
 */
  __pyx_t_2 = ((__pyx_v_a[0]) < (__pyx_v_b[0]));
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_b[0]) != (__pyx_v_b[0]));
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_a[0]) == (__pyx_v_a[0]));
  __pyx_t_1 = __pyx_t_2;
  __pyx_L3_bool_binop_done:;
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "tables/indexesextension.pyx":204
 * 
 * 
 * cdef inline int less_than(number_type* a, number_type* b) nogil:             # <<<<<<<<<<<<<<
 *     if number_type in floating_type:
 *         return a[0] < b[0] or (b[0] != b[0] and a[0] == a[0])
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_9__pyx_f_6tables_16indexesextension_less_than(npy_float64 *__pyx_v_a, npy_float64 *__pyx_v_b) {
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "tables/indexesextension.pyx":206
 * cdef inline int less_than(number_type* a, number_type* b) nogil:
 *     if number_type in floating_type:
 *         return a[0] < b[0] or (b[0] != b[0] and a[0] == a[0])             # <<<<<<<<<<<<<<
 *     else:
 *         return a[0] < b[0]
 */
  __pyx_t_2 = ((__pyx_v_a[0]) < (__pyx_v_b[0]));
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_b[0]) != (__pyx_v_b[0]));
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_a[0]) == (__pyx_v_a[0]));
  __pyx_t_1 = __pyx_t_2;
  __pyx_L3_bool_binop_done:;
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "tables/indexesextension.pyx":204
 * 
 * 
 * cdef inline int less_than(number_type* a, number_type* b) nogil:             # <<<<<<<<<<<<<<
 *     if number_type in floating_type:
 *         return a[0] < b[0] or (b[0] != b[0] and a[0] == a[0])
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_10__pyx_f_6tables_16indexesextension_less_than(npy_longdouble *__pyx_v_a, npy_longdouble *__pyx_v_b) {
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "tables/indexesextension.pyx":206
 * cdef inline int less_than(number_type* a, number_type* b) nogil:
 *     if number_type in floating_type:
 *         return a[0] < b[0] or (b[0] != b[0] and a[0] == a[0])             # <<<<<<<<<<<<<<
 *     else:
 *         return a[0] < b[0]
 */
  __pyx_t_2 = ((__pyx_v_a[0]) < (__pyx_v_b[0]));
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_b[0]) != (__pyx_v_b[0]));
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_a[0]) == (__pyx_v_a[0]));
  __pyx_t_1 = __pyx_t_2;
  __pyx_L3_bool_binop_done:;
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "tables/indexesextension.pyx":204
 * 
 * 
 * cdef inline int less_than(number_type* a, number_type* b) nogil:             # <<<<<<<<<<<<<<
 *     if number_type in floating_type:
 *         return a[0] < b[0] or (b[0] != b[0] and a[0] == a[0])
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "tables/indexesextension.pyx":212
 * 
 * @cython.cdivision(True)
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:             # <<<<<<<<<<<<<<
 *     cdef number_type *pl = start1
 *     cdef number_type *pr = start1 + (n - 1)
 */

static void __pyx_fuse_0__pyx_f_6tables_16indexesextension__keysort(npy_int8 *__pyx_v_start1, char *__pyx_v_start2, size_t __pyx_v_elsize2, size_t __pyx_v_n) {
  npy_int8 *__pyx_v_pl;
  npy_int8 *__pyx_v_pr;
  char *__pyx_v_ipl;
  char *__pyx_v_ipr;
  npy_int8 __pyx_v_vp;
  char *__pyx_v_ivp;
  npy_int8 *__pyx_v_stack[0x64];
  npy_int8 **__pyx_v_sptr;
  char *__pyx_v_istack[0x64];
  char **__pyx_v_isptr;
  CYTHON_UNUSED size_t __pyx_v_stack_index;
  npy_int8 *__pyx_v_pm;
  npy_int8 *__pyx_v_pi;
  npy_int8 *__pyx_v_pj;
  npy_int8 *__pyx_v_pt;
  char *__pyx_v_ipm;
  char *__pyx_v_ipi;
  char *__pyx_v_ipj;
  char *__pyx_v_ipt;
  int __pyx_t_1;
  npy_int8 __pyx_t_2;
  npy_int8 __pyx_t_3;
  int __pyx_t_4;

  /* "tables/indexesextension.pyx":213
 * @cython.cdivision(True)
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:
 *     cdef number_type *pl = start1             # <<<<<<<<<<<<<<
 *     cdef number_type *pr = start1 + (n - 1)
 * 
 */
  __pyx_v_pl = __pyx_v_start1;

  /* "tables/indexesextension.pyx":214
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:
 *     cdef number_type *pl = start1
 *     cdef number_type *pr = start1 + (n - 1)             # <<<<<<<<<<<<<<
 * 
 *     cdef char *ipl = start2
 */
  __pyx_v_pr = (__pyx_v_start1 + (__pyx_v_n - 1));

  /* "tables/indexesextension.pyx":216
 *     cdef number_type *pr = start1 + (n - 1)
 * 
 *     cdef char *ipl = start2             # <<<<<<<<<<<<<<
 *     cdef char *ipr = start2 + (n - 1) * elsize2
 * 
 */
  __pyx_v_ipl = __pyx_v_start2;

  /* "tables/indexesextension.pyx":217
 * 
 *     cdef char *ipl = start2
 *     cdef char *ipr = start2 + (n - 1) * elsize2             # <<<<<<<<<<<<<<
 * 
 *     cdef number_type vp
 */
  __pyx_v_ipr = (__pyx_v_start2 + ((__pyx_v_n - 1) * __pyx_v_elsize2));

  /* "tables/indexesextension.pyx":220
 * 
 *     cdef number_type vp
 *     cdef char *ivp = <char *> malloc(elsize2)             # <<<<<<<<<<<<<<
 * 
 *     cdef number_type *stack[PYA_QS_STACK]
 */
  __pyx_v_ivp = ((char *)malloc(__pyx_v_elsize2));

  /* "tables/indexesextension.pyx":223
 * 
 *     cdef number_type *stack[PYA_QS_STACK]
 *     cdef number_type **sptr = stack             # <<<<<<<<<<<<<<
 * 
 *     cdef char *istack[PYA_QS_STACK]
 */
  __pyx_v_sptr = __pyx_v_stack;

  /* "tables/indexesextension.pyx":226
 * 
 *     cdef char *istack[PYA_QS_STACK]
 *     cdef char **isptr = istack             # <<<<<<<<<<<<<<
 * 
 *     cdef size_t stack_index = 0
 */
  __pyx_v_isptr = __pyx_v_istack;

  /* "tables/indexesextension.pyx":228
 *     cdef char **isptr = istack
 * 
 *     cdef size_t stack_index = 0             # <<<<<<<<<<<<<<
 * 
 *     cdef number_type *pm
 */
  __pyx_v_stack_index = 0;

  /* "tables/indexesextension.pyx":239
 *     cdef char *ipt
 * 
 *     while True:             # <<<<<<<<<<<<<<
 *         while pr - pl > SMALL_QUICKSORT:
 *             pm  = pl + ((pr - pl) >> 1)
 */
  while (1) {

    /* "tables/indexesextension.pyx":240
 * 
 *     while True:
 *         while pr - pl > SMALL_QUICKSORT:             # <<<<<<<<<<<<<<
 *             pm  = pl + ((pr - pl) >> 1)
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 */
    while (1) {
      __pyx_t_1 = (((__pyx_v_pr - __pyx_v_pl) > 15) != 0);
      if (!__pyx_t_1) break;

      /* "tables/indexesextension.pyx":241
 *     while True:
 *         while pr - pl > SMALL_QUICKSORT:
 *             pm  = pl + ((pr - pl) >> 1)             # <<<<<<<<<<<<<<
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 * 
 */
      __pyx_v_pm = (__pyx_v_pl + ((__pyx_v_pr - __pyx_v_pl) >> 1));

      /* "tables/indexesextension.pyx":242
 *         while pr - pl > SMALL_QUICKSORT:
 *             pm  = pl + ((pr - pl) >> 1)
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2             # <<<<<<<<<<<<<<
 * 
 *             if less_than(pm, pl):
 */
      __pyx_v_ipm = (__pyx_v_ipl + ((((__pyx_v_ipr - __pyx_v_ipl) / __pyx_v_elsize2) >> 1) * __pyx_v_elsize2));

      /* "tables/indexesextension.pyx":244
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      __pyx_t_1 = (__pyx_fuse_0__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pm, __pyx_v_pl) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":245
 * 
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 */
        __pyx_t_2 = (__pyx_v_pl[0]);
        __pyx_t_3 = (__pyx_v_pm[0]);
        (__pyx_v_pm[0]) = __pyx_t_2;
        (__pyx_v_pl[0]) = __pyx_t_3;

        /* "tables/indexesextension.pyx":246
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             if less_than(pr, pm):
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipl, __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":244
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      }

      /* "tables/indexesextension.pyx":248
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 *             if less_than(pr, pm):             # <<<<<<<<<<<<<<
 *                 pr[0], pm[0] =  pm[0], pr[0]
 *                 swap_bytes(ipr, ipm, elsize2)
 */
      __pyx_t_1 = (__pyx_fuse_0__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pr, __pyx_v_pm) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":249
 * 
 *             if less_than(pr, pm):
 *                 pr[0], pm[0] =  pm[0], pr[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipr, ipm, elsize2)
 * 
 */
        __pyx_t_3 = (__pyx_v_pm[0]);
        __pyx_t_2 = (__pyx_v_pr[0]);
        (__pyx_v_pr[0]) = __pyx_t_3;
        (__pyx_v_pm[0]) = __pyx_t_2;

        /* "tables/indexesextension.pyx":250
 *             if less_than(pr, pm):
 *                 pr[0], pm[0] =  pm[0], pr[0]
 *                 swap_bytes(ipr, ipm, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             if less_than(pm, pl):
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipr, __pyx_v_ipm, __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":248
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 *             if less_than(pr, pm):             # <<<<<<<<<<<<<<
 *                 pr[0], pm[0] =  pm[0], pr[0]
 *                 swap_bytes(ipr, ipm, elsize2)
 */
      }

      /* "tables/indexesextension.pyx":252
 *                 swap_bytes(ipr, ipm, elsize2)
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      __pyx_t_1 = (__pyx_fuse_0__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pm, __pyx_v_pl) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":253
 * 
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 */
        __pyx_t_2 = (__pyx_v_pl[0]);
        __pyx_t_3 = (__pyx_v_pm[0]);
        (__pyx_v_pm[0]) = __pyx_t_2;
        (__pyx_v_pl[0]) = __pyx_t_3;

        /* "tables/indexesextension.pyx":254
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             vp = pm[0]
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipl, __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":252
 *                 swap_bytes(ipr, ipm, elsize2)
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      }

      /* "tables/indexesextension.pyx":256
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 *             vp = pm[0]             # <<<<<<<<<<<<<<
 * 
 *             pi = pl
 */
      __pyx_v_vp = (__pyx_v_pm[0]);

      /* "tables/indexesextension.pyx":258
 *             vp = pm[0]
 * 
 *             pi = pl             # <<<<<<<<<<<<<<
 *             ipi = ipl
 * 
 */
      __pyx_v_pi = __pyx_v_pl;

      /* "tables/indexesextension.pyx":259
 * 
 *             pi = pl
 *             ipi = ipl             # <<<<<<<<<<<<<<
 * 
 *             pj = pr - 1
 */
      __pyx_v_ipi = __pyx_v_ipl;

      /* "tables/indexesextension.pyx":261
 *             ipi = ipl
 * 
 *             pj = pr - 1             # <<<<<<<<<<<<<<
 *             ipj = ipr - elsize2
 * 
 */
      __pyx_v_pj = (__pyx_v_pr - 1);

      /* "tables/indexesextension.pyx":262
 * 
 *             pj = pr - 1
 *             ipj = ipr - elsize2             # <<<<<<<<<<<<<<
 * 
 *             pm[0], pj[0] = pj[0], pm[0]
 */
      __pyx_v_ipj = (__pyx_v_ipr - __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":264
 *             ipj = ipr - elsize2
 * 
 *             pm[0], pj[0] = pj[0], pm[0]             # <<<<<<<<<<<<<<
 *             swap_bytes(ipm, ipj, elsize2)
 * 
 */
      __pyx_t_3 = (__pyx_v_pj[0]);
      __pyx_t_2 = (__pyx_v_pm[0]);
      (__pyx_v_pm[0]) = __pyx_t_3;
      (__pyx_v_pj[0]) = __pyx_t_2;

      /* "tables/indexesextension.pyx":265
 * 
 *             pm[0], pj[0] = pj[0], pm[0]
 *             swap_bytes(ipm, ipj, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             while True:
 */
      __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipj, __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":267
 *             swap_bytes(ipm, ipj, elsize2)
 * 
 *             while True:             # <<<<<<<<<<<<<<
 *                 pi += 1
 *                 ipi += elsize2
 */
      while (1) {

        /* "tables/indexesextension.pyx":268
 * 
 *             while True:
 *                 pi += 1             # <<<<<<<<<<<<<<
 *                 ipi += elsize2
 *                 while less_than(pi, &vp):
 */
        __pyx_v_pi = (__pyx_v_pi + 1);

        /* "tables/indexesextension.pyx":269
 *             while True:
 *                 pi += 1
 *                 ipi += elsize2             # <<<<<<<<<<<<<<
 *                 while less_than(pi, &vp):
 *                     pi += 1
 */
        __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":270
 *                 pi += 1
 *                 ipi += elsize2
 *                 while less_than(pi, &vp):             # <<<<<<<<<<<<<<
 *                     pi += 1
 *                     ipi += elsize2
 */
        while (1) {
          __pyx_t_1 = (__pyx_fuse_0__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pi, (&__pyx_v_vp)) != 0);
          if (!__pyx_t_1) break;

          /* "tables/indexesextension.pyx":271
 *                 ipi += elsize2
 *                 while less_than(pi, &vp):
 *                     pi += 1             # <<<<<<<<<<<<<<
 *                     ipi += elsize2
 * 
 */
          __pyx_v_pi = (__pyx_v_pi + 1);

          /* "tables/indexesextension.pyx":272
 *                 while less_than(pi, &vp):
 *                     pi += 1
 *                     ipi += elsize2             # <<<<<<<<<<<<<<
 * 
 *                 pj -= 1
 */
          __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_elsize2);
        }

        /* "tables/indexesextension.pyx":274
 *                     ipi += elsize2
 * 
 *                 pj -= 1             # <<<<<<<<<<<<<<
 *                 ipj -= elsize2
 *                 while less_than(&vp, pj):
 */
        __pyx_v_pj = (__pyx_v_pj - 1);

        /* "tables/indexesextension.pyx":275
 * 
 *                 pj -= 1
 *                 ipj -= elsize2             # <<<<<<<<<<<<<<
 *                 while less_than(&vp, pj):
 *                     pj -= 1
 */
        __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":276
 *                 pj -= 1
 *                 ipj -= elsize2
 *                 while less_than(&vp, pj):             # <<<<<<<<<<<<<<
 *                     pj -= 1
 *                     ipj -= elsize2
 */
        while (1) {
          __pyx_t_1 = (__pyx_fuse_0__pyx_f_6tables_16indexesextension_less_than((&__pyx_v_vp), __pyx_v_pj) != 0);
          if (!__pyx_t_1) break;

          /* "tables/indexesextension.pyx":277
 *                 ipj -= elsize2
 *                 while less_than(&vp, pj):
 *                     pj -= 1             # <<<<<<<<<<<<<<
 *                     ipj -= elsize2
 * 
 */
          __pyx_v_pj = (__pyx_v_pj - 1);

          /* "tables/indexesextension.pyx":278
 *                 while less_than(&vp, pj):
 *                     pj -= 1
 *                     ipj -= elsize2             # <<<<<<<<<<<<<<
 * 
 *                 if pi >= pj:
 */
          __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_elsize2);
        }

        /* "tables/indexesextension.pyx":280
 *                     ipj -= elsize2
 * 
 *                 if pi >= pj:             # <<<<<<<<<<<<<<
 *                     break
 * 
 */
        __pyx_t_1 = ((__pyx_v_pi >= __pyx_v_pj) != 0);
        if (__pyx_t_1) {

          /* "tables/indexesextension.pyx":281
 * 
 *                 if pi >= pj:
 *                     break             # <<<<<<<<<<<<<<
 * 
 *                 pi[0], pj[0] = pj[0], pi[0]
 */
          goto __pyx_L11_break;

          /* "tables/indexesextension.pyx":280
 *                     ipj -= elsize2
 * 
 *                 if pi >= pj:             # <<<<<<<<<<<<<<
 *                     break
 * 
 */
        }

        /* "tables/indexesextension.pyx":283
 *                     break
 * 
 *                 pi[0], pj[0] = pj[0], pi[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipi, ipj, elsize2)
 * 
 */
        __pyx_t_2 = (__pyx_v_pj[0]);
        __pyx_t_3 = (__pyx_v_pi[0]);
        (__pyx_v_pi[0]) = __pyx_t_2;
        (__pyx_v_pj[0]) = __pyx_t_3;

        /* "tables/indexesextension.pyx":284
 * 
 *                 pi[0], pj[0] = pj[0], pi[0]
 *                 swap_bytes(ipi, ipj, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             pi[0], (pr-1)[0] = (pr-1)[0], pi[0]
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipi, __pyx_v_ipj, __pyx_v_elsize2);
      }
      __pyx_L11_break:;

      /* "tables/indexesextension.pyx":286
 *                 swap_bytes(ipi, ipj, elsize2)
 * 
 *             pi[0], (pr-1)[0] = (pr-1)[0], pi[0]             # <<<<<<<<<<<<<<
 *             swap_bytes(ipi, ipr-elsize2, elsize2)
 * 
 */
      __pyx_t_3 = ((__pyx_v_pr - 1)[0]);
      __pyx_t_2 = (__pyx_v_pi[0]);
      (__pyx_v_pi[0]) = __pyx_t_3;
      ((__pyx_v_pr - 1)[0]) = __pyx_t_2;

      /* "tables/indexesextension.pyx":287
 * 
 *             pi[0], (pr-1)[0] = (pr-1)[0], pi[0]
 *             swap_bytes(ipi, ipr-elsize2, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             # push largest partition on stack and proceed with the other
 */
      __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipi, (__pyx_v_ipr - __pyx_v_elsize2), __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":290
 * 
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):             # <<<<<<<<<<<<<<
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr
 */
      __pyx_t_1 = (((__pyx_v_pi - __pyx_v_pl) < (__pyx_v_pr - __pyx_v_pi)) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":291
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):
 *                 sptr[0] = pi + 1             # <<<<<<<<<<<<<<
 *                 sptr[1] = pr
 *                 sptr += 2
 */
        (__pyx_v_sptr[0]) = (__pyx_v_pi + 1);

        /* "tables/indexesextension.pyx":292
 *             if (pi - pl) < (pr - pi):
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr             # <<<<<<<<<<<<<<
 *                 sptr += 2
 * 
 */
        (__pyx_v_sptr[1]) = __pyx_v_pr;

        /* "tables/indexesextension.pyx":293
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr
 *                 sptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 isptr[0] = ipi + elsize2
 */
        __pyx_v_sptr = (__pyx_v_sptr + 2);

        /* "tables/indexesextension.pyx":295
 *                 sptr += 2
 * 
 *                 isptr[0] = ipi + elsize2             # <<<<<<<<<<<<<<
 *                 isptr[1] = ipr
 *                 isptr += 2
 */
        (__pyx_v_isptr[0]) = (__pyx_v_ipi + __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":296
 * 
 *                 isptr[0] = ipi + elsize2
 *                 isptr[1] = ipr             # <<<<<<<<<<<<<<
 *                 isptr += 2
 * 
 */
        (__pyx_v_isptr[1]) = __pyx_v_ipr;

        /* "tables/indexesextension.pyx":297
 *                 isptr[0] = ipi + elsize2
 *                 isptr[1] = ipr
 *                 isptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 pr = pi - 1
 */
        __pyx_v_isptr = (__pyx_v_isptr + 2);

        /* "tables/indexesextension.pyx":299
 *                 isptr += 2
 * 
 *                 pr = pi - 1             # <<<<<<<<<<<<<<
 *                 ipr = ipi - elsize2
 *             else:
 */
        __pyx_v_pr = (__pyx_v_pi - 1);

        /* "tables/indexesextension.pyx":300
 * 
 *                 pr = pi - 1
 *                 ipr = ipi - elsize2             # <<<<<<<<<<<<<<
 *             else:
 *                 sptr[0] = pl
 */
        __pyx_v_ipr = (__pyx_v_ipi - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":290
 * 
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):             # <<<<<<<<<<<<<<
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr
 */
        goto __pyx_L17;
      }

      /* "tables/indexesextension.pyx":302
 *                 ipr = ipi - elsize2
 *             else:
 *                 sptr[0] = pl             # <<<<<<<<<<<<<<
 *                 sptr[1] = pi - 1
 *                 sptr += 2
 */
      /*else*/ {
        (__pyx_v_sptr[0]) = __pyx_v_pl;

        /* "tables/indexesextension.pyx":303
 *             else:
 *                 sptr[0] = pl
 *                 sptr[1] = pi - 1             # <<<<<<<<<<<<<<
 *                 sptr += 2
 * 
 */
        (__pyx_v_sptr[1]) = (__pyx_v_pi - 1);

        /* "tables/indexesextension.pyx":304
 *                 sptr[0] = pl
 *                 sptr[1] = pi - 1
 *                 sptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 isptr[0] = ipl
 */
        __pyx_v_sptr = (__pyx_v_sptr + 2);

        /* "tables/indexesextension.pyx":306
 *                 sptr += 2
 * 
 *                 isptr[0] = ipl             # <<<<<<<<<<<<<<
 *                 isptr[1] = ipi - elsize2
 *                 isptr += 2
 */
        (__pyx_v_isptr[0]) = __pyx_v_ipl;

        /* "tables/indexesextension.pyx":307
 * 
 *                 isptr[0] = ipl
 *                 isptr[1] = ipi - elsize2             # <<<<<<<<<<<<<<
 *                 isptr += 2
 * 
 */
        (__pyx_v_isptr[1]) = (__pyx_v_ipi - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":308
 *                 isptr[0] = ipl
 *                 isptr[1] = ipi - elsize2
 *                 isptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 pl = pi + 1
 */
        __pyx_v_isptr = (__pyx_v_isptr + 2);

        /* "tables/indexesextension.pyx":310
 *                 isptr += 2
 * 
 *                 pl = pi + 1             # <<<<<<<<<<<<<<
 *                 ipl = ipi + elsize2
 * 
 */
        __pyx_v_pl = (__pyx_v_pi + 1);

        /* "tables/indexesextension.pyx":311
 * 
 *                 pl = pi + 1
 *                 ipl = ipi + elsize2             # <<<<<<<<<<<<<<
 * 
 *         pi = pl + 1
 */
        __pyx_v_ipl = (__pyx_v_ipi + __pyx_v_elsize2);
      }
      __pyx_L17:;
    }

    /* "tables/indexesextension.pyx":313
 *                 ipl = ipi + elsize2
 * 
 *         pi = pl + 1             # <<<<<<<<<<<<<<
 *         ipi = ipl + elsize2
 *         while pi <= pr:
 */
    __pyx_v_pi = (__pyx_v_pl + 1);

    /* "tables/indexesextension.pyx":314
 * 
 *         pi = pl + 1
 *         ipi = ipl + elsize2             # <<<<<<<<<<<<<<
 *         while pi <= pr:
 *             vp = pi[0]
 */
    __pyx_v_ipi = (__pyx_v_ipl + __pyx_v_elsize2);

    /* "tables/indexesextension.pyx":315
 *         pi = pl + 1
 *         ipi = ipl + elsize2
 *         while pi <= pr:             # <<<<<<<<<<<<<<
 *             vp = pi[0]
 *             memcpy(ivp, ipi, elsize2)
 */
    while (1) {
      __pyx_t_1 = ((__pyx_v_pi <= __pyx_v_pr) != 0);
      if (!__pyx_t_1) break;

      /* "tables/indexesextension.pyx":316
 *         ipi = ipl + elsize2
 *         while pi <= pr:
 *             vp = pi[0]             # <<<<<<<<<<<<<<
 *             memcpy(ivp, ipi, elsize2)
 * 
 */
      __pyx_v_vp = (__pyx_v_pi[0]);

      /* "tables/indexesextension.pyx":317
 *         while pi <= pr:
 *             vp = pi[0]
 *             memcpy(ivp, ipi, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             pj = pi
 */
      (void)(memcpy(__pyx_v_ivp, __pyx_v_ipi, __pyx_v_elsize2));

      /* "tables/indexesextension.pyx":319
 *             memcpy(ivp, ipi, elsize2)
 * 
 *             pj = pi             # <<<<<<<<<<<<<<
 *             pt = pi - 1
 * 
 */
      __pyx_v_pj = __pyx_v_pi;

      /* "tables/indexesextension.pyx":320
 * 
 *             pj = pi
 *             pt = pi - 1             # <<<<<<<<<<<<<<
 * 
 *             ipj = ipi
 */
      __pyx_v_pt = (__pyx_v_pi - 1);

      /* "tables/indexesextension.pyx":322
 *             pt = pi - 1
 * 
 *             ipj = ipi             # <<<<<<<<<<<<<<
 *             ipt = ipi - elsize2
 * 
 */
      __pyx_v_ipj = __pyx_v_ipi;

      /* "tables/indexesextension.pyx":323
 * 
 *             ipj = ipi
 *             ipt = ipi - elsize2             # <<<<<<<<<<<<<<
 * 
 *             while pj > pl and less_than(&vp, pt):
 */
      __pyx_v_ipt = (__pyx_v_ipi - __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":325
 *             ipt = ipi - elsize2
 * 
 *             while pj > pl and less_than(&vp, pt):             # <<<<<<<<<<<<<<
 *                 pj[0] = pt[0]
 *                 pj -= 1
 */
      while (1) {
        __pyx_t_4 = ((__pyx_v_pj > __pyx_v_pl) != 0);
        if (__pyx_t_4) {
        } else {
          __pyx_t_1 = __pyx_t_4;
          goto __pyx_L22_bool_binop_done;
        }
        __pyx_t_4 = (__pyx_fuse_0__pyx_f_6tables_16indexesextension_less_than((&__pyx_v_vp), __pyx_v_pt) != 0);
        __pyx_t_1 = __pyx_t_4;
        __pyx_L22_bool_binop_done:;
        if (!__pyx_t_1) break;

        /* "tables/indexesextension.pyx":326
 * 
 *             while pj > pl and less_than(&vp, pt):
 *                 pj[0] = pt[0]             # <<<<<<<<<<<<<<
 *                 pj -= 1
 *                 pt -= 1
 */
        (__pyx_v_pj[0]) = (__pyx_v_pt[0]);

        /* "tables/indexesextension.pyx":327
 *             while pj > pl and less_than(&vp, pt):
 *                 pj[0] = pt[0]
 *                 pj -= 1             # <<<<<<<<<<<<<<
 *                 pt -= 1
 * 
 */
        __pyx_v_pj = (__pyx_v_pj - 1);

        /* "tables/indexesextension.pyx":328
 *                 pj[0] = pt[0]
 *                 pj -= 1
 *                 pt -= 1             # <<<<<<<<<<<<<<
 * 
 *                 memcpy(ipj, ipt, elsize2)
 */
        __pyx_v_pt = (__pyx_v_pt - 1);

        /* "tables/indexesextension.pyx":330
 *                 pt -= 1
 * 
 *                 memcpy(ipj, ipt, elsize2)             # <<<<<<<<<<<<<<
 *                 ipj -= elsize2
 *                 ipt -= elsize2
 */
        (void)(memcpy(__pyx_v_ipj, __pyx_v_ipt, __pyx_v_elsize2));

        /* "tables/indexesextension.pyx":331
 * 
 *                 memcpy(ipj, ipt, elsize2)
 *                 ipj -= elsize2             # <<<<<<<<<<<<<<
 *                 ipt -= elsize2
 * 
 */
        __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":332
 *                 memcpy(ipj, ipt, elsize2)
 *                 ipj -= elsize2
 *                 ipt -= elsize2             # <<<<<<<<<<<<<<
 * 
 *             pj[0] = vp
 */
        __pyx_v_ipt = (__pyx_v_ipt - __pyx_v_elsize2);
      }

      /* "tables/indexesextension.pyx":334
 *                 ipt -= elsize2
 * 
 *             pj[0] = vp             # <<<<<<<<<<<<<<
 *             memcpy(ipj, ivp, elsize2)
 * 
 */
      (__pyx_v_pj[0]) = __pyx_v_vp;

      /* "tables/indexesextension.pyx":335
 * 
 *             pj[0] = vp
 *             memcpy(ipj, ivp, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             pi += 1
 */
      (void)(memcpy(__pyx_v_ipj, __pyx_v_ivp, __pyx_v_elsize2));

      /* "tables/indexesextension.pyx":337
 *             memcpy(ipj, ivp, elsize2)
 * 
 *             pi += 1             # <<<<<<<<<<<<<<
 *             ipi += elsize2
 * 
 */
      __pyx_v_pi = (__pyx_v_pi + 1);

      /* "tables/indexesextension.pyx":338
 * 
 *             pi += 1
 *             ipi += elsize2             # <<<<<<<<<<<<<<
 * 
 *         if sptr == stack:
 */
      __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_elsize2);
    }

    /* "tables/indexesextension.pyx":340
 *             ipi += elsize2
 * 
 *         if sptr == stack:             # <<<<<<<<<<<<<<
 *             break
 * 
 */
    __pyx_t_1 = ((__pyx_v_sptr == __pyx_v_stack) != 0);
    if (__pyx_t_1) {

      /* "tables/indexesextension.pyx":341
 * 
 *         if sptr == stack:
 *             break             # <<<<<<<<<<<<<<
 * 
 *         sptr -= 2
 */
      goto __pyx_L4_break;

      /* "tables/indexesextension.pyx":340
 *             ipi += elsize2
 * 
 *         if sptr == stack:             # <<<<<<<<<<<<<<
 *             break
 * 
 */
    }

    /* "tables/indexesextension.pyx":343
 *             break
 * 
 *         sptr -= 2             # <<<<<<<<<<<<<<
 *         pl = sptr[0]
 *         pr = sptr[1]
 */
    __pyx_v_sptr = (__pyx_v_sptr - 2);

    /* "tables/indexesextension.pyx":344
 * 
 *         sptr -= 2
 *         pl = sptr[0]             # <<<<<<<<<<<<<<
 *         pr = sptr[1]
 * 
 */
    __pyx_v_pl = (__pyx_v_sptr[0]);

    /* "tables/indexesextension.pyx":345
 *         sptr -= 2
 *         pl = sptr[0]
 *         pr = sptr[1]             # <<<<<<<<<<<<<<
 * 
 *         isptr -= 2
 */
    __pyx_v_pr = (__pyx_v_sptr[1]);

    /* "tables/indexesextension.pyx":347
 *         pr = sptr[1]
 * 
 *         isptr -= 2             # <<<<<<<<<<<<<<
 *         ipl = isptr[0]
 *         ipr = isptr[1]
 */
    __pyx_v_isptr = (__pyx_v_isptr - 2);

    /* "tables/indexesextension.pyx":348
 * 
 *         isptr -= 2
 *         ipl = isptr[0]             # <<<<<<<<<<<<<<
 *         ipr = isptr[1]
 * 
 */
    __pyx_v_ipl = (__pyx_v_isptr[0]);

    /* "tables/indexesextension.pyx":349
 *         isptr -= 2
 *         ipl = isptr[0]
 *         ipr = isptr[1]             # <<<<<<<<<<<<<<
 * 
 *     free(ivp)
 */
    __pyx_v_ipr = (__pyx_v_isptr[1]);
  }
  __pyx_L4_break:;

  /* "tables/indexesextension.pyx":351
 *         ipr = isptr[1]
 * 
 *     free(ivp)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  free(__pyx_v_ivp);

  /* "tables/indexesextension.pyx":212
 * 
 * @cython.cdivision(True)
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:             # <<<<<<<<<<<<<<
 *     cdef number_type *pl = start1
 *     cdef number_type *pr = start1 + (n - 1)
 */

  /* function exit code */
}

static void __pyx_fuse_1__pyx_f_6tables_16indexesextension__keysort(npy_int16 *__pyx_v_start1, char *__pyx_v_start2, size_t __pyx_v_elsize2, size_t __pyx_v_n) {
  npy_int16 *__pyx_v_pl;
  npy_int16 *__pyx_v_pr;
  char *__pyx_v_ipl;
  char *__pyx_v_ipr;
  npy_int16 __pyx_v_vp;
  char *__pyx_v_ivp;
  npy_int16 *__pyx_v_stack[0x64];
  npy_int16 **__pyx_v_sptr;
  char *__pyx_v_istack[0x64];
  char **__pyx_v_isptr;
  CYTHON_UNUSED size_t __pyx_v_stack_index;
  npy_int16 *__pyx_v_pm;
  npy_int16 *__pyx_v_pi;
  npy_int16 *__pyx_v_pj;
  npy_int16 *__pyx_v_pt;
  char *__pyx_v_ipm;
  char *__pyx_v_ipi;
  char *__pyx_v_ipj;
  char *__pyx_v_ipt;
  int __pyx_t_1;
  npy_int16 __pyx_t_2;
  npy_int16 __pyx_t_3;
  int __pyx_t_4;

  /* "tables/indexesextension.pyx":213
 * @cython.cdivision(True)
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:
 *     cdef number_type *pl = start1             # <<<<<<<<<<<<<<
 *     cdef number_type *pr = start1 + (n - 1)
 * 
 */
  __pyx_v_pl = __pyx_v_start1;

  /* "tables/indexesextension.pyx":214
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:
 *     cdef number_type *pl = start1
 *     cdef number_type *pr = start1 + (n - 1)             # <<<<<<<<<<<<<<
 * 
 *     cdef char *ipl = start2
 */
  __pyx_v_pr = (__pyx_v_start1 + (__pyx_v_n - 1));

  /* "tables/indexesextension.pyx":216
 *     cdef number_type *pr = start1 + (n - 1)
 * 
 *     cdef char *ipl = start2             # <<<<<<<<<<<<<<
 *     cdef char *ipr = start2 + (n - 1) * elsize2
 * 
 */
  __pyx_v_ipl = __pyx_v_start2;

  /* "tables/indexesextension.pyx":217
 * 
 *     cdef char *ipl = start2
 *     cdef char *ipr = start2 + (n - 1) * elsize2             # <<<<<<<<<<<<<<
 * 
 *     cdef number_type vp
 */
  __pyx_v_ipr = (__pyx_v_start2 + ((__pyx_v_n - 1) * __pyx_v_elsize2));

  /* "tables/indexesextension.pyx":220
 * 
 *     cdef number_type vp
 *     cdef char *ivp = <char *> malloc(elsize2)             # <<<<<<<<<<<<<<
 * 
 *     cdef number_type *stack[PYA_QS_STACK]
 */
  __pyx_v_ivp = ((char *)malloc(__pyx_v_elsize2));

  /* "tables/indexesextension.pyx":223
 * 
 *     cdef number_type *stack[PYA_QS_STACK]
 *     cdef number_type **sptr = stack             # <<<<<<<<<<<<<<
 * 
 *     cdef char *istack[PYA_QS_STACK]
 */
  __pyx_v_sptr = __pyx_v_stack;

  /* "tables/indexesextension.pyx":226
 * 
 *     cdef char *istack[PYA_QS_STACK]
 *     cdef char **isptr = istack             # <<<<<<<<<<<<<<
 * 
 *     cdef size_t stack_index = 0
 */
  __pyx_v_isptr = __pyx_v_istack;

  /* "tables/indexesextension.pyx":228
 *     cdef char **isptr = istack
 * 
 *     cdef size_t stack_index = 0             # <<<<<<<<<<<<<<
 * 
 *     cdef number_type *pm
 */
  __pyx_v_stack_index = 0;

  /* "tables/indexesextension.pyx":239
 *     cdef char *ipt
 * 
 *     while True:             # <<<<<<<<<<<<<<
 *         while pr - pl > SMALL_QUICKSORT:
 *             pm  = pl + ((pr - pl) >> 1)
 */
  while (1) {

    /* "tables/indexesextension.pyx":240
 * 
 *     while True:
 *         while pr - pl > SMALL_QUICKSORT:             # <<<<<<<<<<<<<<
 *             pm  = pl + ((pr - pl) >> 1)
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 */
    while (1) {
      __pyx_t_1 = (((__pyx_v_pr - __pyx_v_pl) > 15) != 0);
      if (!__pyx_t_1) break;

      /* "tables/indexesextension.pyx":241
 *     while True:
 *         while pr - pl > SMALL_QUICKSORT:
 *             pm  = pl + ((pr - pl) >> 1)             # <<<<<<<<<<<<<<
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 * 
 */
      __pyx_v_pm = (__pyx_v_pl + ((__pyx_v_pr - __pyx_v_pl) >> 1));

      /* "tables/indexesextension.pyx":242
 *         while pr - pl > SMALL_QUICKSORT:
 *             pm  = pl + ((pr - pl) >> 1)
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2             # <<<<<<<<<<<<<<
 * 
 *             if less_than(pm, pl):
 */
      __pyx_v_ipm = (__pyx_v_ipl + ((((__pyx_v_ipr - __pyx_v_ipl) / __pyx_v_elsize2) >> 1) * __pyx_v_elsize2));

      /* "tables/indexesextension.pyx":244
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      __pyx_t_1 = (__pyx_fuse_1__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pm, __pyx_v_pl) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":245
 * 
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 */
        __pyx_t_2 = (__pyx_v_pl[0]);
        __pyx_t_3 = (__pyx_v_pm[0]);
        (__pyx_v_pm[0]) = __pyx_t_2;
        (__pyx_v_pl[0]) = __pyx_t_3;

        /* "tables/indexesextension.pyx":246
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             if less_than(pr, pm):
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipl, __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":244
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      }

      /* "tables/indexesextension.pyx":248
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 *             if less_than(pr, pm):             # <<<<<<<<<<<<<<
 *                 pr[0], pm[0] =  pm[0], pr[0]
 *                 swap_bytes(ipr, ipm, elsize2)
 */
      __pyx_t_1 = (__pyx_fuse_1__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pr, __pyx_v_pm) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":249
 * 
 *             if less_than(pr, pm):
 *                 pr[0], pm[0] =  pm[0], pr[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipr, ipm, elsize2)
 * 
 */
        __pyx_t_3 = (__pyx_v_pm[0]);
        __pyx_t_2 = (__pyx_v_pr[0]);
        (__pyx_v_pr[0]) = __pyx_t_3;
        (__pyx_v_pm[0]) = __pyx_t_2;

        /* "tables/indexesextension.pyx":250
 *             if less_than(pr, pm):
 *                 pr[0], pm[0] =  pm[0], pr[0]
 *                 swap_bytes(ipr, ipm, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             if less_than(pm, pl):
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipr, __pyx_v_ipm, __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":248
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 *             if less_than(pr, pm):             # <<<<<<<<<<<<<<
 *                 pr[0], pm[0] =  pm[0], pr[0]
 *                 swap_bytes(ipr, ipm, elsize2)
 */
      }

      /* "tables/indexesextension.pyx":252
 *                 swap_bytes(ipr, ipm, elsize2)
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      __pyx_t_1 = (__pyx_fuse_1__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pm, __pyx_v_pl) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":253
 * 
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 */
        __pyx_t_2 = (__pyx_v_pl[0]);
        __pyx_t_3 = (__pyx_v_pm[0]);
        (__pyx_v_pm[0]) = __pyx_t_2;
        (__pyx_v_pl[0]) = __pyx_t_3;

        /* "tables/indexesextension.pyx":254
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             vp = pm[0]
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipl, __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":252
 *                 swap_bytes(ipr, ipm, elsize2)
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      }

      /* "tables/indexesextension.pyx":256
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 *             vp = pm[0]             # <<<<<<<<<<<<<<
 * 
 *             pi = pl
 */
      __pyx_v_vp = (__pyx_v_pm[0]);

      /* "tables/indexesextension.pyx":258
 *             vp = pm[0]
 * 
 *             pi = pl             # <<<<<<<<<<<<<<
 *             ipi = ipl
 * 
 */
      __pyx_v_pi = __pyx_v_pl;

      /* "tables/indexesextension.pyx":259
 * 
 *             pi = pl
 *             ipi = ipl             # <<<<<<<<<<<<<<
 * 
 *             pj = pr - 1
 */
      __pyx_v_ipi = __pyx_v_ipl;

      /* "tables/indexesextension.pyx":261
 *             ipi = ipl
 * 
 *             pj = pr - 1             # <<<<<<<<<<<<<<
 *             ipj = ipr - elsize2
 * 
 */
      __pyx_v_pj = (__pyx_v_pr - 1);

      /* "tables/indexesextension.pyx":262
 * 
 *             pj = pr - 1
 *             ipj = ipr - elsize2             # <<<<<<<<<<<<<<
 * 
 *             pm[0], pj[0] = pj[0], pm[0]
 */
      __pyx_v_ipj = (__pyx_v_ipr - __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":264
 *             ipj = ipr - elsize2
 * 
 *             pm[0], pj[0] = pj[0], pm[0]             # <<<<<<<<<<<<<<
 *             swap_bytes(ipm, ipj, elsize2)
 * 
 */
      __pyx_t_3 = (__pyx_v_pj[0]);
      __pyx_t_2 = (__pyx_v_pm[0]);
      (__pyx_v_pm[0]) = __pyx_t_3;
      (__pyx_v_pj[0]) = __pyx_t_2;

      /* "tables/indexesextension.pyx":265
 * 
 *             pm[0], pj[0] = pj[0], pm[0]
 *             swap_bytes(ipm, ipj, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             while True:
 */
      __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipj, __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":267
 *             swap_bytes(ipm, ipj, elsize2)
 * 
 *             while True:             # <<<<<<<<<<<<<<
 *                 pi += 1
 *                 ipi += elsize2
 */
      while (1) {

        /* "tables/indexesextension.pyx":268
 * 
 *             while True:
 *                 pi += 1             # <<<<<<<<<<<<<<
 *                 ipi += elsize2
 *                 while less_than(pi, &vp):
 */
        __pyx_v_pi = (__pyx_v_pi + 1);

        /* "tables/indexesextension.pyx":269
 *             while True:
 *                 pi += 1
 *                 ipi += elsize2             # <<<<<<<<<<<<<<
 *                 while less_than(pi, &vp):
 *                     pi += 1
 */
        __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":270
 *                 pi += 1
 *                 ipi += elsize2
 *                 while less_than(pi, &vp):             # <<<<<<<<<<<<<<
 *                     pi += 1
 *                     ipi += elsize2
 */
        while (1) {
          __pyx_t_1 = (__pyx_fuse_1__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pi, (&__pyx_v_vp)) != 0);
          if (!__pyx_t_1) break;

          /* "tables/indexesextension.pyx":271
 *                 ipi += elsize2
 *                 while less_than(pi, &vp):
 *                     pi += 1             # <<<<<<<<<<<<<<
 *                     ipi += elsize2
 * 
 */
          __pyx_v_pi = (__pyx_v_pi + 1);

          /* "tables/indexesextension.pyx":272
 *                 while less_than(pi, &vp):
 *                     pi += 1
 *                     ipi += elsize2             # <<<<<<<<<<<<<<
 * 
 *                 pj -= 1
 */
          __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_elsize2);
        }

        /* "tables/indexesextension.pyx":274
 *                     ipi += elsize2
 * 
 *                 pj -= 1             # <<<<<<<<<<<<<<
 *                 ipj -= elsize2
 *                 while less_than(&vp, pj):
 */
        __pyx_v_pj = (__pyx_v_pj - 1);

        /* "tables/indexesextension.pyx":275
 * 
 *                 pj -= 1
 *                 ipj -= elsize2             # <<<<<<<<<<<<<<
 *                 while less_than(&vp, pj):
 *                     pj -= 1
 */
        __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":276
 *                 pj -= 1
 *                 ipj -= elsize2
 *                 while less_than(&vp, pj):             # <<<<<<<<<<<<<<
 *                     pj -= 1
 *                     ipj -= elsize2
 */
        while (1) {
          __pyx_t_1 = (__pyx_fuse_1__pyx_f_6tables_16indexesextension_less_than((&__pyx_v_vp), __pyx_v_pj) != 0);
          if (!__pyx_t_1) break;

          /* "tables/indexesextension.pyx":277
 *                 ipj -= elsize2
 *                 while less_than(&vp, pj):
 *                     pj -= 1             # <<<<<<<<<<<<<<
 *                     ipj -= elsize2
 * 
 */
          __pyx_v_pj = (__pyx_v_pj - 1);

          /* "tables/indexesextension.pyx":278
 *                 while less_than(&vp, pj):
 *                     pj -= 1
 *                     ipj -= elsize2             # <<<<<<<<<<<<<<
 * 
 *                 if pi >= pj:
 */
          __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_elsize2);
        }

        /* "tables/indexesextension.pyx":280
 *                     ipj -= elsize2
 * 
 *                 if pi >= pj:             # <<<<<<<<<<<<<<
 *                     break
 * 
 */
        __pyx_t_1 = ((__pyx_v_pi >= __pyx_v_pj) != 0);
        if (__pyx_t_1) {

          /* "tables/indexesextension.pyx":281
 * 
 *                 if pi >= pj:
 *                     break             # <<<<<<<<<<<<<<
 * 
 *                 pi[0], pj[0] = pj[0], pi[0]
 */
          goto __pyx_L11_break;

          /* "tables/indexesextension.pyx":280
 *                     ipj -= elsize2
 * 
 *                 if pi >= pj:             # <<<<<<<<<<<<<<
 *                     break
 * 
 */
        }

        /* "tables/indexesextension.pyx":283
 *                     break
 * 
 *                 pi[0], pj[0] = pj[0], pi[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipi, ipj, elsize2)
 * 
 */
        __pyx_t_2 = (__pyx_v_pj[0]);
        __pyx_t_3 = (__pyx_v_pi[0]);
        (__pyx_v_pi[0]) = __pyx_t_2;
        (__pyx_v_pj[0]) = __pyx_t_3;

        /* "tables/indexesextension.pyx":284
 * 
 *                 pi[0], pj[0] = pj[0], pi[0]
 *                 swap_bytes(ipi, ipj, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             pi[0], (pr-1)[0] = (pr-1)[0], pi[0]
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipi, __pyx_v_ipj, __pyx_v_elsize2);
      }
      __pyx_L11_break:;

      /* "tables/indexesextension.pyx":286
 *                 swap_bytes(ipi, ipj, elsize2)
 * 
 *             pi[0], (pr-1)[0] = (pr-1)[0], pi[0]             # <<<<<<<<<<<<<<
 *             swap_bytes(ipi, ipr-elsize2, elsize2)
 * 
 */
      __pyx_t_3 = ((__pyx_v_pr - 1)[0]);
      __pyx_t_2 = (__pyx_v_pi[0]);
      (__pyx_v_pi[0]) = __pyx_t_3;
      ((__pyx_v_pr - 1)[0]) = __pyx_t_2;

      /* "tables/indexesextension.pyx":287
 * 
 *             pi[0], (pr-1)[0] = (pr-1)[0], pi[0]
 *             swap_bytes(ipi, ipr-elsize2, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             # push largest partition on stack and proceed with the other
 */
      __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipi, (__pyx_v_ipr - __pyx_v_elsize2), __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":290
 * 
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):             # <<<<<<<<<<<<<<
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr
 */
      __pyx_t_1 = (((__pyx_v_pi - __pyx_v_pl) < (__pyx_v_pr - __pyx_v_pi)) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":291
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):
 *                 sptr[0] = pi + 1             # <<<<<<<<<<<<<<
 *                 sptr[1] = pr
 *                 sptr += 2
 */
        (__pyx_v_sptr[0]) = (__pyx_v_pi + 1);

        /* "tables/indexesextension.pyx":292
 *             if (pi - pl) < (pr - pi):
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr             # <<<<<<<<<<<<<<
 *                 sptr += 2
 * 
 */
        (__pyx_v_sptr[1]) = __pyx_v_pr;

        /* "tables/indexesextension.pyx":293
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr
 *                 sptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 isptr[0] = ipi + elsize2
 */
        __pyx_v_sptr = (__pyx_v_sptr + 2);

        /* "tables/indexesextension.pyx":295
 *                 sptr += 2
 * 
 *                 isptr[0] = ipi + elsize2             # <<<<<<<<<<<<<<
 *                 isptr[1] = ipr
 *                 isptr += 2
 */
        (__pyx_v_isptr[0]) = (__pyx_v_ipi + __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":296
 * 
 *                 isptr[0] = ipi + elsize2
 *                 isptr[1] = ipr             # <<<<<<<<<<<<<<
 *                 isptr += 2
 * 
 */
        (__pyx_v_isptr[1]) = __pyx_v_ipr;

        /* "tables/indexesextension.pyx":297
 *                 isptr[0] = ipi + elsize2
 *                 isptr[1] = ipr
 *                 isptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 pr = pi - 1
 */
        __pyx_v_isptr = (__pyx_v_isptr + 2);

        /* "tables/indexesextension.pyx":299
 *                 isptr += 2
 * 
 *                 pr = pi - 1             # <<<<<<<<<<<<<<
 *                 ipr = ipi - elsize2
 *             else:
 */
        __pyx_v_pr = (__pyx_v_pi - 1);

        /* "tables/indexesextension.pyx":300
 * 
 *                 pr = pi - 1
 *                 ipr = ipi - elsize2             # <<<<<<<<<<<<<<
 *             else:
 *                 sptr[0] = pl
 */
        __pyx_v_ipr = (__pyx_v_ipi - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":290
 * 
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):             # <<<<<<<<<<<<<<
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr
 */
        goto __pyx_L17;
      }

      /* "tables/indexesextension.pyx":302
 *                 ipr = ipi - elsize2
 *             else:
 *                 sptr[0] = pl             # <<<<<<<<<<<<<<
 *                 sptr[1] = pi - 1
 *                 sptr += 2
 */
      /*else*/ {
        (__pyx_v_sptr[0]) = __pyx_v_pl;

        /* "tables/indexesextension.pyx":303
 *             else:
 *                 sptr[0] = pl
 *                 sptr[1] = pi - 1             # <<<<<<<<<<<<<<
 *                 sptr += 2
 * 
 */
        (__pyx_v_sptr[1]) = (__pyx_v_pi - 1);

        /* "tables/indexesextension.pyx":304
 *                 sptr[0] = pl
 *                 sptr[1] = pi - 1
 *                 sptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 isptr[0] = ipl
 */
        __pyx_v_sptr = (__pyx_v_sptr + 2);

        /* "tables/indexesextension.pyx":306
 *                 sptr += 2
 * 
 *                 isptr[0] = ipl             # <<<<<<<<<<<<<<
 *                 isptr[1] = ipi - elsize2
 *                 isptr += 2
 */
        (__pyx_v_isptr[0]) = __pyx_v_ipl;

        /* "tables/indexesextension.pyx":307
 * 
 *                 isptr[0] = ipl
 *                 isptr[1] = ipi - elsize2             # <<<<<<<<<<<<<<
 *                 isptr += 2
 * 
 */
        (__pyx_v_isptr[1]) = (__pyx_v_ipi - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":308
 *                 isptr[0] = ipl
 *                 isptr[1] = ipi - elsize2
 *                 isptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 pl = pi + 1
 */
        __pyx_v_isptr = (__pyx_v_isptr + 2);

        /* "tables/indexesextension.pyx":310
 *                 isptr += 2
 * 
 *                 pl = pi + 1             # <<<<<<<<<<<<<<
 *                 ipl = ipi + elsize2
 * 
 */
        __pyx_v_pl = (__pyx_v_pi + 1);

        /* "tables/indexesextension.pyx":311
 * 
 *                 pl = pi + 1
 *                 ipl = ipi + elsize2             # <<<<<<<<<<<<<<
 * 
 *         pi = pl + 1
 */
        __pyx_v_ipl = (__pyx_v_ipi + __pyx_v_elsize2);
      }
      __pyx_L17:;
    }

    /* "tables/indexesextension.pyx":313
 *                 ipl = ipi + elsize2
 * 
 *         pi = pl + 1             # <<<<<<<<<<<<<<
 *         ipi = ipl + elsize2
 *         while pi <= pr:
 */
    __pyx_v_pi = (__pyx_v_pl + 1);

    /* "tables/indexesextension.pyx":314
 * 
 *         pi = pl + 1
 *         ipi = ipl + elsize2             # <<<<<<<<<<<<<<
 *         while pi <= pr:
 *             vp = pi[0]
 */
    __pyx_v_ipi = (__pyx_v_ipl + __pyx_v_elsize2);

    /* "tables/indexesextension.pyx":315
 *         pi = pl + 1
 *         ipi = ipl + elsize2
 *         while pi <= pr:             # <<<<<<<<<<<<<<
 *             vp = pi[0]
 *             memcpy(ivp, ipi, elsize2)
 */
    while (1) {
      __pyx_t_1 = ((__pyx_v_pi <= __pyx_v_pr) != 0);
      if (!__pyx_t_1) break;

      /* "tables/indexesextension.pyx":316
 *         ipi = ipl + elsize2
 *         while pi <= pr:
 *             vp = pi[0]             # <<<<<<<<<<<<<<
 *             memcpy(ivp, ipi, elsize2)
 * 
 */
      __pyx_v_vp = (__pyx_v_pi[0]);

      /* "tables/indexesextension.pyx":317
 *         while pi <= pr:
 *             vp = pi[0]
 *             memcpy(ivp, ipi, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             pj = pi
 */
      (void)(memcpy(__pyx_v_ivp, __pyx_v_ipi, __pyx_v_elsize2));

      /* "tables/indexesextension.pyx":319
 *             memcpy(ivp, ipi, elsize2)
 * 
 *             pj = pi             # <<<<<<<<<<<<<<
 *             pt = pi - 1
 * 
 */
      __pyx_v_pj = __pyx_v_pi;

      /* "tables/indexesextension.pyx":320
 * 
 *             pj = pi
 *             pt = pi - 1             # <<<<<<<<<<<<<<
 * 
 *             ipj = ipi
 */
      __pyx_v_pt = (__pyx_v_pi - 1);

      /* "tables/indexesextension.pyx":322
 *             pt = pi - 1
 * 
 *             ipj = ipi             # <<<<<<<<<<<<<<
 *             ipt = ipi - elsize2
 * 
 */
      __pyx_v_ipj = __pyx_v_ipi;

      /* "tables/indexesextension.pyx":323
 * 
 *             ipj = ipi
 *             ipt = ipi - elsize2             # <<<<<<<<<<<<<<
 * 
 *             while pj > pl and less_than(&vp, pt):
 */
      __pyx_v_ipt = (__pyx_v_ipi - __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":325
 *             ipt = ipi - elsize2
 * 
 *             while pj > pl and less_than(&vp, pt):             # <<<<<<<<<<<<<<
 *                 pj[0] = pt[0]
 *                 pj -= 1
 */
      while (1) {
        __pyx_t_4 = ((__pyx_v_pj > __pyx_v_pl) != 0);
        if (__pyx_t_4) {
        } else {
          __pyx_t_1 = __pyx_t_4;
          goto __pyx_L22_bool_binop_done;
        }
        __pyx_t_4 = (__pyx_fuse_1__pyx_f_6tables_16indexesextension_less_than((&__pyx_v_vp), __pyx_v_pt) != 0);
        __pyx_t_1 = __pyx_t_4;
        __pyx_L22_bool_binop_done:;
        if (!__pyx_t_1) break;

        /* "tables/indexesextension.pyx":326
 * 
 *             while pj > pl and less_than(&vp, pt):
 *                 pj[0] = pt[0]             # <<<<<<<<<<<<<<
 *                 pj -= 1
 *                 pt -= 1
 */
        (__pyx_v_pj[0]) = (__pyx_v_pt[0]);

        /* "tables/indexesextension.pyx":327
 *             while pj > pl and less_than(&vp, pt):
 *                 pj[0] = pt[0]
 *                 pj -= 1             # <<<<<<<<<<<<<<
 *                 pt -= 1
 * 
 */
        __pyx_v_pj = (__pyx_v_pj - 1);

        /* "tables/indexesextension.pyx":328
 *                 pj[0] = pt[0]
 *                 pj -= 1
 *                 pt -= 1             # <<<<<<<<<<<<<<
 * 
 *                 memcpy(ipj, ipt, elsize2)
 */
        __pyx_v_pt = (__pyx_v_pt - 1);

        /* "tables/indexesextension.pyx":330
 *                 pt -= 1
 * 
 *                 memcpy(ipj, ipt, elsize2)             # <<<<<<<<<<<<<<
 *                 ipj -= elsize2
 *                 ipt -= elsize2
 */
        (void)(memcpy(__pyx_v_ipj, __pyx_v_ipt, __pyx_v_elsize2));

        /* "tables/indexesextension.pyx":331
 * 
 *                 memcpy(ipj, ipt, elsize2)
 *                 ipj -= elsize2             # <<<<<<<<<<<<<<
 *                 ipt -= elsize2
 * 
 */
        __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":332
 *                 memcpy(ipj, ipt, elsize2)
 *                 ipj -= elsize2
 *                 ipt -= elsize2             # <<<<<<<<<<<<<<
 * 
 *             pj[0] = vp
 */
        __pyx_v_ipt = (__pyx_v_ipt - __pyx_v_elsize2);
      }

      /* "tables/indexesextension.pyx":334
 *                 ipt -= elsize2
 * 
 *             pj[0] = vp             # <<<<<<<<<<<<<<
 *             memcpy(ipj, ivp, elsize2)
 * 
 */
      (__pyx_v_pj[0]) = __pyx_v_vp;

      /* "tables/indexesextension.pyx":335
 * 
 *             pj[0] = vp
 *             memcpy(ipj, ivp, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             pi += 1
 */
      (void)(memcpy(__pyx_v_ipj, __pyx_v_ivp, __pyx_v_elsize2));

      /* "tables/indexesextension.pyx":337
 *             memcpy(ipj, ivp, elsize2)
 * 
 *             pi += 1             # <<<<<<<<<<<<<<
 *             ipi += elsize2
 * 
 */
      __pyx_v_pi = (__pyx_v_pi + 1);

      /* "tables/indexesextension.pyx":338
 * 
 *             pi += 1
 *             ipi += elsize2             # <<<<<<<<<<<<<<
 * 
 *         if sptr == stack:
 */
      __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_elsize2);
    }

    /* "tables/indexesextension.pyx":340
 *             ipi += elsize2
 * 
 *         if sptr == stack:             # <<<<<<<<<<<<<<
 *             break
 * 
 */
    __pyx_t_1 = ((__pyx_v_sptr == __pyx_v_stack) != 0);
    if (__pyx_t_1) {

      /* "tables/indexesextension.pyx":341
 * 
 *         if sptr == stack:
 *             break             # <<<<<<<<<<<<<<
 * 
 *         sptr -= 2
 */
      goto __pyx_L4_break;

      /* "tables/indexesextension.pyx":340
 *             ipi += elsize2
 * 
 *         if sptr == stack:             # <<<<<<<<<<<<<<
 *             break
 * 
 */
    }

    /* "tables/indexesextension.pyx":343
 *             break
 * 
 *         sptr -= 2             # <<<<<<<<<<<<<<
 *         pl = sptr[0]
 *         pr = sptr[1]
 */
    __pyx_v_sptr = (__pyx_v_sptr - 2);

    /* "tables/indexesextension.pyx":344
 * 
 *         sptr -= 2
 *         pl = sptr[0]             # <<<<<<<<<<<<<<
 *         pr = sptr[1]
 * 
 */
    __pyx_v_pl = (__pyx_v_sptr[0]);

    /* "tables/indexesextension.pyx":345
 *         sptr -= 2
 *         pl = sptr[0]
 *         pr = sptr[1]             # <<<<<<<<<<<<<<
 * 
 *         isptr -= 2
 */
    __pyx_v_pr = (__pyx_v_sptr[1]);

    /* "tables/indexesextension.pyx":347
 *         pr = sptr[1]
 * 
 *         isptr -= 2             # <<<<<<<<<<<<<<
 *         ipl = isptr[0]
 *         ipr = isptr[1]
 */
    __pyx_v_isptr = (__pyx_v_isptr - 2);

    /* "tables/indexesextension.pyx":348
 * 
 *         isptr -= 2
 *         ipl = isptr[0]             # <<<<<<<<<<<<<<
 *         ipr = isptr[1]
 * 
 */
    __pyx_v_ipl = (__pyx_v_isptr[0]);

    /* "tables/indexesextension.pyx":349
 *         isptr -= 2
 *         ipl = isptr[0]
 *         ipr = isptr[1]             # <<<<<<<<<<<<<<
 * 
 *     free(ivp)
 */
    __pyx_v_ipr = (__pyx_v_isptr[1]);
  }
  __pyx_L4_break:;

  /* "tables/indexesextension.pyx":351
 *         ipr = isptr[1]
 * 
 *     free(ivp)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  free(__pyx_v_ivp);

  /* "tables/indexesextension.pyx":212
 * 
 * @cython.cdivision(True)
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:             # <<<<<<<<<<<<<<
 *     cdef number_type *pl = start1
 *     cdef number_type *pr = start1 + (n - 1)
 */

  /* function exit code */
}

static void __pyx_fuse_2__pyx_f_6tables_16indexesextension__keysort(npy_int32 *__pyx_v_start1, char *__pyx_v_start2, size_t __pyx_v_elsize2, size_t __pyx_v_n) {
  npy_int32 *__pyx_v_pl;
  npy_int32 *__pyx_v_pr;
  char *__pyx_v_ipl;
  char *__pyx_v_ipr;
  npy_int32 __pyx_v_vp;
  char *__pyx_v_ivp;
  npy_int32 *__pyx_v_stack[0x64];
  npy_int32 **__pyx_v_sptr;
  char *__pyx_v_istack[0x64];
  char **__pyx_v_isptr;
  CYTHON_UNUSED size_t __pyx_v_stack_index;
  npy_int32 *__pyx_v_pm;
  npy_int32 *__pyx_v_pi;
  npy_int32 *__pyx_v_pj;
  npy_int32 *__pyx_v_pt;
  char *__pyx_v_ipm;
  char *__pyx_v_ipi;
  char *__pyx_v_ipj;
  char *__pyx_v_ipt;
  int __pyx_t_1;
  npy_int32 __pyx_t_2;
  npy_int32 __pyx_t_3;
  int __pyx_t_4;

  /* "tables/indexesextension.pyx":213
 * @cython.cdivision(True)
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:
 *     cdef number_type *pl = start1             # <<<<<<<<<<<<<<
 *     cdef number_type *pr = start1 + (n - 1)
 * 
 */
  __pyx_v_pl = __pyx_v_start1;

  /* "tables/indexesextension.pyx":214
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:
 *     cdef number_type *pl = start1
 *     cdef number_type *pr = start1 + (n - 1)             # <<<<<<<<<<<<<<
 * 
 *     cdef char *ipl = start2
 */
  __pyx_v_pr = (__pyx_v_start1 + (__pyx_v_n - 1));

  /* "tables/indexesextension.pyx":216
 *     cdef number_type *pr = start1 + (n - 1)
 * 
 *     cdef char *ipl = start2             # <<<<<<<<<<<<<<
 *     cdef char *ipr = start2 + (n - 1) * elsize2
 * 
 */
  __pyx_v_ipl = __pyx_v_start2;

  /* "tables/indexesextension.pyx":217
 * 
 *     cdef char *ipl = start2
 *     cdef char *ipr = start2 + (n - 1) * elsize2             # <<<<<<<<<<<<<<
 * 
 *     cdef number_type vp
 */
  __pyx_v_ipr = (__pyx_v_start2 + ((__pyx_v_n - 1) * __pyx_v_elsize2));

  /* "tables/indexesextension.pyx":220
 * 
 *     cdef number_type vp
 *     cdef char *ivp = <char *> malloc(elsize2)             # <<<<<<<<<<<<<<
 * 
 *     cdef number_type *stack[PYA_QS_STACK]
 */
  __pyx_v_ivp = ((char *)malloc(__pyx_v_elsize2));

  /* "tables/indexesextension.pyx":223
 * 
 *     cdef number_type *stack[PYA_QS_STACK]
 *     cdef number_type **sptr = stack             # <<<<<<<<<<<<<<
 * 
 *     cdef char *istack[PYA_QS_STACK]
 */
  __pyx_v_sptr = __pyx_v_stack;

  /* "tables/indexesextension.pyx":226
 * 
 *     cdef char *istack[PYA_QS_STACK]
 *     cdef char **isptr = istack             # <<<<<<<<<<<<<<
 * 
 *     cdef size_t stack_index = 0
 */
  __pyx_v_isptr = __pyx_v_istack;

  /* "tables/indexesextension.pyx":228
 *     cdef char **isptr = istack
 * 
 *     cdef size_t stack_index = 0             # <<<<<<<<<<<<<<
 * 
 *     cdef number_type *pm
 */
  __pyx_v_stack_index = 0;

  /* "tables/indexesextension.pyx":239
 *     cdef char *ipt
 * 
 *     while True:             # <<<<<<<<<<<<<<
 *         while pr - pl > SMALL_QUICKSORT:
 *             pm  = pl + ((pr - pl) >> 1)
 */
  while (1) {

    /* "tables/indexesextension.pyx":240
 * 
 *     while True:
 *         while pr - pl > SMALL_QUICKSORT:             # <<<<<<<<<<<<<<
 *             pm  = pl + ((pr - pl) >> 1)
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 */
    while (1) {
      __pyx_t_1 = (((__pyx_v_pr - __pyx_v_pl) > 15) != 0);
      if (!__pyx_t_1) break;

      /* "tables/indexesextension.pyx":241
 *     while True:
 *         while pr - pl > SMALL_QUICKSORT:
 *             pm  = pl + ((pr - pl) >> 1)             # <<<<<<<<<<<<<<
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 * 
 */
      __pyx_v_pm = (__pyx_v_pl + ((__pyx_v_pr - __pyx_v_pl) >> 1));

      /* "tables/indexesextension.pyx":242
 *         while pr - pl > SMALL_QUICKSORT:
 *             pm  = pl + ((pr - pl) >> 1)
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2             # <<<<<<<<<<<<<<
 * 
 *             if less_than(pm, pl):
 */
      __pyx_v_ipm = (__pyx_v_ipl + ((((__pyx_v_ipr - __pyx_v_ipl) / __pyx_v_elsize2) >> 1) * __pyx_v_elsize2));

      /* "tables/indexesextension.pyx":244
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      __pyx_t_1 = (__pyx_fuse_2__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pm, __pyx_v_pl) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":245
 * 
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 */
        __pyx_t_2 = (__pyx_v_pl[0]);
        __pyx_t_3 = (__pyx_v_pm[0]);
        (__pyx_v_pm[0]) = __pyx_t_2;
        (__pyx_v_pl[0]) = __pyx_t_3;

        /* "tables/indexesextension.pyx":246
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             if less_than(pr, pm):
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipl, __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":244
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      }

      /* "tables/indexesextension.pyx":248
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 *             if less_than(pr, pm):             # <<<<<<<<<<<<<<
 *                 pr[0], pm[0] =  pm[0], pr[0]
 *                 swap_bytes(ipr, ipm, elsize2)
 */
      __pyx_t_1 = (__pyx_fuse_2__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pr, __pyx_v_pm) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":249
 * 
 *             if less_than(pr, pm):
 *                 pr[0], pm[0] =  pm[0], pr[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipr, ipm, elsize2)
 * 
 */
        __pyx_t_3 = (__pyx_v_pm[0]);
        __pyx_t_2 = (__pyx_v_pr[0]);
        (__pyx_v_pr[0]) = __pyx_t_3;
        (__pyx_v_pm[0]) = __pyx_t_2;

        /* "tables/indexesextension.pyx":250
 *             if less_than(pr, pm):
 *                 pr[0], pm[0] =  pm[0], pr[0]
 *                 swap_bytes(ipr, ipm, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             if less_than(pm, pl):
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipr, __pyx_v_ipm, __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":248
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 *             if less_than(pr, pm):             # <<<<<<<<<<<<<<
 *                 pr[0], pm[0] =  pm[0], pr[0]
 *                 swap_bytes(ipr, ipm, elsize2)
 */
      }

      /* "tables/indexesextension.pyx":252
 *                 swap_bytes(ipr, ipm, elsize2)
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      __pyx_t_1 = (__pyx_fuse_2__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pm, __pyx_v_pl) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":253
 * 
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 */
        __pyx_t_2 = (__pyx_v_pl[0]);
        __pyx_t_3 = (__pyx_v_pm[0]);
        (__pyx_v_pm[0]) = __pyx_t_2;
        (__pyx_v_pl[0]) = __pyx_t_3;

        /* "tables/indexesextension.pyx":254
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             vp = pm[0]
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipl, __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":252
 *                 swap_bytes(ipr, ipm, elsize2)
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      }

      /* "tables/indexesextension.pyx":256
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 *             vp = pm[0]             # <<<<<<<<<<<<<<
 * 
 *             pi = pl
 */
      __pyx_v_vp = (__pyx_v_pm[0]);

      /* "tables/indexesextension.pyx":258
 *             vp = pm[0]
 * 
 *             pi = pl             # <<<<<<<<<<<<<<
 *             ipi = ipl
 * 
 */
      __pyx_v_pi = __pyx_v_pl;

      /* "tables/indexesextension.pyx":259
 * 
 *             pi = pl
 *             ipi = ipl             # <<<<<<<<<<<<<<
 * 
 *             pj = pr - 1
 */
      __pyx_v_ipi = __pyx_v_ipl;

      /* "tables/indexesextension.pyx":261
 *             ipi = ipl
 * 
 *             pj = pr - 1             # <<<<<<<<<<<<<<
 *             ipj = ipr - elsize2
 * 
 */
      __pyx_v_pj = (__pyx_v_pr - 1);

      /* "tables/indexesextension.pyx":262
 * 
 *             pj = pr - 1
 *             ipj = ipr - elsize2             # <<<<<<<<<<<<<<
 * 
 *             pm[0], pj[0] = pj[0], pm[0]
 */
      __pyx_v_ipj = (__pyx_v_ipr - __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":264
 *             ipj = ipr - elsize2
 * 
 *             pm[0], pj[0] = pj[0], pm[0]             # <<<<<<<<<<<<<<
 *             swap_bytes(ipm, ipj, elsize2)
 * 
 */
      __pyx_t_3 = (__pyx_v_pj[0]);
      __pyx_t_2 = (__pyx_v_pm[0]);
      (__pyx_v_pm[0]) = __pyx_t_3;
      (__pyx_v_pj[0]) = __pyx_t_2;

      /* "tables/indexesextension.pyx":265
 * 
 *             pm[0], pj[0] = pj[0], pm[0]
 *             swap_bytes(ipm, ipj, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             while True:
 */
      __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipj, __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":267
 *             swap_bytes(ipm, ipj, elsize2)
 * 
 *             while True:             # <<<<<<<<<<<<<<
 *                 pi += 1
 *                 ipi += elsize2
 */
      while (1) {

        /* "tables/indexesextension.pyx":268
 * 
 *             while True:
 *                 pi += 1             # <<<<<<<<<<<<<<
 *                 ipi += elsize2
 *                 while less_than(pi, &vp):
 */
        __pyx_v_pi = (__pyx_v_pi + 1);

        /* "tables/indexesextension.pyx":269
 *             while True:
 *                 pi += 1
 *                 ipi += elsize2             # <<<<<<<<<<<<<<
 *                 while less_than(pi, &vp):
 *                     pi += 1
 */
        __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":270
 *                 pi += 1
 *                 ipi += elsize2
 *                 while less_than(pi, &vp):             # <<<<<<<<<<<<<<
 *                     pi += 1
 *                     ipi += elsize2
 */
        while (1) {
          __pyx_t_1 = (__pyx_fuse_2__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pi, (&__pyx_v_vp)) != 0);
          if (!__pyx_t_1) break;

          /* "tables/indexesextension.pyx":271
 *                 ipi += elsize2
 *                 while less_than(pi, &vp):
 *                     pi += 1             # <<<<<<<<<<<<<<
 *                     ipi += elsize2
 * 
 */
          __pyx_v_pi = (__pyx_v_pi + 1);

          /* "tables/indexesextension.pyx":272
 *                 while less_than(pi, &vp):
 *                     pi += 1
 *                     ipi += elsize2             # <<<<<<<<<<<<<<
 * 
 *                 pj -= 1
 */
          __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_elsize2);
        }

        /* "tables/indexesextension.pyx":274
 *                     ipi += elsize2
 * 
 *                 pj -= 1             # <<<<<<<<<<<<<<
 *                 ipj -= elsize2
 *                 while less_than(&vp, pj):
 */
        __pyx_v_pj = (__pyx_v_pj - 1);

        /* "tables/indexesextension.pyx":275
 * 
 *                 pj -= 1
 *                 ipj -= elsize2             # <<<<<<<<<<<<<<
 *                 while less_than(&vp, pj):
 *                     pj -= 1
 */
        __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":276
 *                 pj -= 1
 *                 ipj -= elsize2
 *                 while less_than(&vp, pj):             # <<<<<<<<<<<<<<
 *                     pj -= 1
 *                     ipj -= elsize2
 */
        while (1) {
          __pyx_t_1 = (__pyx_fuse_2__pyx_f_6tables_16indexesextension_less_than((&__pyx_v_vp), __pyx_v_pj) != 0);
          if (!__pyx_t_1) break;

          /* "tables/indexesextension.pyx":277
 *                 ipj -= elsize2
 *                 while less_than(&vp, pj):
 *                     pj -= 1             # <<<<<<<<<<<<<<
 *                     ipj -= elsize2
 * 
 */
          __pyx_v_pj = (__pyx_v_pj - 1);

          /* "tables/indexesextension.pyx":278
 *                 while less_than(&vp, pj):
 *                     pj -= 1
 *                     ipj -= elsize2             # <<<<<<<<<<<<<<
 * 
 *                 if pi >= pj:
 */
          __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_elsize2);
        }

        /* "tables/indexesextension.pyx":280
 *                     ipj -= elsize2
 * 
 *                 if pi >= pj:             # <<<<<<<<<<<<<<
 *                     break
 * 
 */
        __pyx_t_1 = ((__pyx_v_pi >= __pyx_v_pj) != 0);
        if (__pyx_t_1) {

          /* "tables/indexesextension.pyx":281
 * 
 *                 if pi >= pj:
 *                     break             # <<<<<<<<<<<<<<
 * 
 *                 pi[0], pj[0] = pj[0], pi[0]
 */
          goto __pyx_L11_break;

          /* "tables/indexesextension.pyx":280
 *                     ipj -= elsize2
 * 
 *                 if pi >= pj:             # <<<<<<<<<<<<<<
 *                     break
 * 
 */
        }

        /* "tables/indexesextension.pyx":283
 *                     break
 * 
 *                 pi[0], pj[0] = pj[0], pi[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipi, ipj, elsize2)
 * 
 */
        __pyx_t_2 = (__pyx_v_pj[0]);
        __pyx_t_3 = (__pyx_v_pi[0]);
        (__pyx_v_pi[0]) = __pyx_t_2;
        (__pyx_v_pj[0]) = __pyx_t_3;

        /* "tables/indexesextension.pyx":284
 * 
 *                 pi[0], pj[0] = pj[0], pi[0]
 *                 swap_bytes(ipi, ipj, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             pi[0], (pr-1)[0] = (pr-1)[0], pi[0]
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipi, __pyx_v_ipj, __pyx_v_elsize2);
      }
      __pyx_L11_break:;

      /* "tables/indexesextension.pyx":286
 *                 swap_bytes(ipi, ipj, elsize2)
 * 
 *             pi[0], (pr-1)[0] = (pr-1)[0], pi[0]             # <<<<<<<<<<<<<<
 *             swap_bytes(ipi, ipr-elsize2, elsize2)
 * 
 */
      __pyx_t_3 = ((__pyx_v_pr - 1)[0]);
      __pyx_t_2 = (__pyx_v_pi[0]);
      (__pyx_v_pi[0]) = __pyx_t_3;
      ((__pyx_v_pr - 1)[0]) = __pyx_t_2;

      /* "tables/indexesextension.pyx":287
 * 
 *             pi[0], (pr-1)[0] = (pr-1)[0], pi[0]
 *             swap_bytes(ipi, ipr-elsize2, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             # push largest partition on stack and proceed with the other
 */
      __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipi, (__pyx_v_ipr - __pyx_v_elsize2), __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":290
 * 
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):             # <<<<<<<<<<<<<<
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr
 */
      __pyx_t_1 = (((__pyx_v_pi - __pyx_v_pl) < (__pyx_v_pr - __pyx_v_pi)) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":291
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):
 *                 sptr[0] = pi + 1             # <<<<<<<<<<<<<<
 *                 sptr[1] = pr
 *                 sptr += 2
 */
        (__pyx_v_sptr[0]) = (__pyx_v_pi + 1);

        /* "tables/indexesextension.pyx":292
 *             if (pi - pl) < (pr - pi):
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr             # <<<<<<<<<<<<<<
 *                 sptr += 2
 * 
 */
        (__pyx_v_sptr[1]) = __pyx_v_pr;

        /* "tables/indexesextension.pyx":293
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr
 *                 sptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 isptr[0] = ipi + elsize2
 */
        __pyx_v_sptr = (__pyx_v_sptr + 2);

        /* "tables/indexesextension.pyx":295
 *                 sptr += 2
 * 
 *                 isptr[0] = ipi + elsize2             # <<<<<<<<<<<<<<
 *                 isptr[1] = ipr
 *                 isptr += 2
 */
        (__pyx_v_isptr[0]) = (__pyx_v_ipi + __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":296
 * 
 *                 isptr[0] = ipi + elsize2
 *                 isptr[1] = ipr             # <<<<<<<<<<<<<<
 *                 isptr += 2
 * 
 */
        (__pyx_v_isptr[1]) = __pyx_v_ipr;

        /* "tables/indexesextension.pyx":297
 *                 isptr[0] = ipi + elsize2
 *                 isptr[1] = ipr
 *                 isptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 pr = pi - 1
 */
        __pyx_v_isptr = (__pyx_v_isptr + 2);

        /* "tables/indexesextension.pyx":299
 *                 isptr += 2
 * 
 *                 pr = pi - 1             # <<<<<<<<<<<<<<
 *                 ipr = ipi - elsize2
 *             else:
 */
        __pyx_v_pr = (__pyx_v_pi - 1);

        /* "tables/indexesextension.pyx":300
 * 
 *                 pr = pi - 1
 *                 ipr = ipi - elsize2             # <<<<<<<<<<<<<<
 *             else:
 *                 sptr[0] = pl
 */
        __pyx_v_ipr = (__pyx_v_ipi - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":290
 * 
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):             # <<<<<<<<<<<<<<
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr
 */
        goto __pyx_L17;
      }

      /* "tables/indexesextension.pyx":302
 *                 ipr = ipi - elsize2
 *             else:
 *                 sptr[0] = pl             # <<<<<<<<<<<<<<
 *                 sptr[1] = pi - 1
 *                 sptr += 2
 */
      /*else*/ {
        (__pyx_v_sptr[0]) = __pyx_v_pl;

        /* "tables/indexesextension.pyx":303
 *             else:
 *                 sptr[0] = pl
 *                 sptr[1] = pi - 1             # <<<<<<<<<<<<<<
 *                 sptr += 2
 * 
 */
        (__pyx_v_sptr[1]) = (__pyx_v_pi - 1);

        /* "tables/indexesextension.pyx":304
 *                 sptr[0] = pl
 *                 sptr[1] = pi - 1
 *                 sptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 isptr[0] = ipl
 */
        __pyx_v_sptr = (__pyx_v_sptr + 2);

        /* "tables/indexesextension.pyx":306
 *                 sptr += 2
 * 
 *                 isptr[0] = ipl             # <<<<<<<<<<<<<<
 *                 isptr[1] = ipi - elsize2
 *                 isptr += 2
 */
        (__pyx_v_isptr[0]) = __pyx_v_ipl;

        /* "tables/indexesextension.pyx":307
 * 
 *                 isptr[0] = ipl
 *                 isptr[1] = ipi - elsize2             # <<<<<<<<<<<<<<
 *                 isptr += 2
 * 
 */
        (__pyx_v_isptr[1]) = (__pyx_v_ipi - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":308
 *                 isptr[0] = ipl
 *                 isptr[1] = ipi - elsize2
 *                 isptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 pl = pi + 1
 */
        __pyx_v_isptr = (__pyx_v_isptr + 2);

        /* "tables/indexesextension.pyx":310
 *                 isptr += 2
 * 
 *                 pl = pi + 1             # <<<<<<<<<<<<<<
 *                 ipl = ipi + elsize2
 * 
 */
        __pyx_v_pl = (__pyx_v_pi + 1);

        /* "tables/indexesextension.pyx":311
 * 
 *                 pl = pi + 1
 *                 ipl = ipi + elsize2             # <<<<<<<<<<<<<<
 * 
 *         pi = pl + 1
 */
        __pyx_v_ipl = (__pyx_v_ipi + __pyx_v_elsize2);
      }
      __pyx_L17:;
    }

    /* "tables/indexesextension.pyx":313
 *                 ipl = ipi + elsize2
 * 
 *         pi = pl + 1             # <<<<<<<<<<<<<<
 *         ipi = ipl + elsize2
 *         while pi <= pr:
 */
    __pyx_v_pi = (__pyx_v_pl + 1);

    /* "tables/indexesextension.pyx":314
 * 
 *         pi = pl + 1
 *         ipi = ipl + elsize2             # <<<<<<<<<<<<<<
 *         while pi <= pr:
 *             vp = pi[0]
 */
    __pyx_v_ipi = (__pyx_v_ipl + __pyx_v_elsize2);

    /* "tables/indexesextension.pyx":315
 *         pi = pl + 1
 *         ipi = ipl + elsize2
 *         while pi <= pr:             # <<<<<<<<<<<<<<
 *             vp = pi[0]
 *             memcpy(ivp, ipi, elsize2)
 */
    while (1) {
      __pyx_t_1 = ((__pyx_v_pi <= __pyx_v_pr) != 0);
      if (!__pyx_t_1) break;

      /* "tables/indexesextension.pyx":316
 *         ipi = ipl + elsize2
 *         while pi <= pr:
 *             vp = pi[0]             # <<<<<<<<<<<<<<
 *             memcpy(ivp, ipi, elsize2)
 * 
 */
      __pyx_v_vp = (__pyx_v_pi[0]);

      /* "tables/indexesextension.pyx":317
 *         while pi <= pr:
 *             vp = pi[0]
 *             memcpy(ivp, ipi, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             pj = pi
 */
      (void)(memcpy(__pyx_v_ivp, __pyx_v_ipi, __pyx_v_elsize2));

      /* "tables/indexesextension.pyx":319
 *             memcpy(ivp, ipi, elsize2)
 * 
 *             pj = pi             # <<<<<<<<<<<<<<
 *             pt = pi - 1
 * 
 */
      __pyx_v_pj = __pyx_v_pi;

      /* "tables/indexesextension.pyx":320
 * 
 *             pj = pi
 *             pt = pi - 1             # <<<<<<<<<<<<<<
 * 
 *             ipj = ipi
 */
      __pyx_v_pt = (__pyx_v_pi - 1);

      /* "tables/indexesextension.pyx":322
 *             pt = pi - 1
 * 
 *             ipj = ipi             # <<<<<<<<<<<<<<
 *             ipt = ipi - elsize2
 * 
 */
      __pyx_v_ipj = __pyx_v_ipi;

      /* "tables/indexesextension.pyx":323
 * 
 *             ipj = ipi
 *             ipt = ipi - elsize2             # <<<<<<<<<<<<<<
 * 
 *             while pj > pl and less_than(&vp, pt):
 */
      __pyx_v_ipt = (__pyx_v_ipi - __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":325
 *             ipt = ipi - elsize2
 * 
 *             while pj > pl and less_than(&vp, pt):             # <<<<<<<<<<<<<<
 *                 pj[0] = pt[0]
 *                 pj -= 1
 */
      while (1) {
        __pyx_t_4 = ((__pyx_v_pj > __pyx_v_pl) != 0);
        if (__pyx_t_4) {
        } else {
          __pyx_t_1 = __pyx_t_4;
          goto __pyx_L22_bool_binop_done;
        }
        __pyx_t_4 = (__pyx_fuse_2__pyx_f_6tables_16indexesextension_less_than((&__pyx_v_vp), __pyx_v_pt) != 0);
        __pyx_t_1 = __pyx_t_4;
        __pyx_L22_bool_binop_done:;
        if (!__pyx_t_1) break;

        /* "tables/indexesextension.pyx":326
 * 
 *             while pj > pl and less_than(&vp, pt):
 *                 pj[0] = pt[0]             # <<<<<<<<<<<<<<
 *                 pj -= 1
 *                 pt -= 1
 */
        (__pyx_v_pj[0]) = (__pyx_v_pt[0]);

        /* "tables/indexesextension.pyx":327
 *             while pj > pl and less_than(&vp, pt):
 *                 pj[0] = pt[0]
 *                 pj -= 1             # <<<<<<<<<<<<<<
 *                 pt -= 1
 * 
 */
        __pyx_v_pj = (__pyx_v_pj - 1);

        /* "tables/indexesextension.pyx":328
 *                 pj[0] = pt[0]
 *                 pj -= 1
 *                 pt -= 1             # <<<<<<<<<<<<<<
 * 
 *                 memcpy(ipj, ipt, elsize2)
 */
        __pyx_v_pt = (__pyx_v_pt - 1);

        /* "tables/indexesextension.pyx":330
 *                 pt -= 1
 * 
 *                 memcpy(ipj, ipt, elsize2)             # <<<<<<<<<<<<<<
 *                 ipj -= elsize2
 *                 ipt -= elsize2
 */
        (void)(memcpy(__pyx_v_ipj, __pyx_v_ipt, __pyx_v_elsize2));

        /* "tables/indexesextension.pyx":331
 * 
 *                 memcpy(ipj, ipt, elsize2)
 *                 ipj -= elsize2             # <<<<<<<<<<<<<<
 *                 ipt -= elsize2
 * 
 */
        __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":332
 *                 memcpy(ipj, ipt, elsize2)
 *                 ipj -= elsize2
 *                 ipt -= elsize2             # <<<<<<<<<<<<<<
 * 
 *             pj[0] = vp
 */
        __pyx_v_ipt = (__pyx_v_ipt - __pyx_v_elsize2);
      }

      /* "tables/indexesextension.pyx":334
 *                 ipt -= elsize2
 * 
 *             pj[0] = vp             # <<<<<<<<<<<<<<
 *             memcpy(ipj, ivp, elsize2)
 * 
 */
      (__pyx_v_pj[0]) = __pyx_v_vp;

      /* "tables/indexesextension.pyx":335
 * 
 *             pj[0] = vp
 *             memcpy(ipj, ivp, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             pi += 1
 */
      (void)(memcpy(__pyx_v_ipj, __pyx_v_ivp, __pyx_v_elsize2));

      /* "tables/indexesextension.pyx":337
 *             memcpy(ipj, ivp, elsize2)
 * 
 *             pi += 1             # <<<<<<<<<<<<<<
 *             ipi += elsize2
 * 
 */
      __pyx_v_pi = (__pyx_v_pi + 1);

      /* "tables/indexesextension.pyx":338
 * 
 *             pi += 1
 *             ipi += elsize2             # <<<<<<<<<<<<<<
 * 
 *         if sptr == stack:
 */
      __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_elsize2);
    }

    /* "tables/indexesextension.pyx":340
 *             ipi += elsize2
 * 
 *         if sptr == stack:             # <<<<<<<<<<<<<<
 *             break
 * 
 */
    __pyx_t_1 = ((__pyx_v_sptr == __pyx_v_stack) != 0);
    if (__pyx_t_1) {

      /* "tables/indexesextension.pyx":341
 * 
 *         if sptr == stack:
 *             break             # <<<<<<<<<<<<<<
 * 
 *         sptr -= 2
 */
      goto __pyx_L4_break;

      /* "tables/indexesextension.pyx":340
 *             ipi += elsize2
 * 
 *         if sptr == stack:             # <<<<<<<<<<<<<<
 *             break
 * 
 */
    }

    /* "tables/indexesextension.pyx":343
 *             break
 * 
 *         sptr -= 2             # <<<<<<<<<<<<<<
 *         pl = sptr[0]
 *         pr = sptr[1]
 */
    __pyx_v_sptr = (__pyx_v_sptr - 2);

    /* "tables/indexesextension.pyx":344
 * 
 *         sptr -= 2
 *         pl = sptr[0]             # <<<<<<<<<<<<<<
 *         pr = sptr[1]
 * 
 */
    __pyx_v_pl = (__pyx_v_sptr[0]);

    /* "tables/indexesextension.pyx":345
 *         sptr -= 2
 *         pl = sptr[0]
 *         pr = sptr[1]             # <<<<<<<<<<<<<<
 * 
 *         isptr -= 2
 */
    __pyx_v_pr = (__pyx_v_sptr[1]);

    /* "tables/indexesextension.pyx":347
 *         pr = sptr[1]
 * 
 *         isptr -= 2             # <<<<<<<<<<<<<<
 *         ipl = isptr[0]
 *         ipr = isptr[1]
 */
    __pyx_v_isptr = (__pyx_v_isptr - 2);

    /* "tables/indexesextension.pyx":348
 * 
 *         isptr -= 2
 *         ipl = isptr[0]             # <<<<<<<<<<<<<<
 *         ipr = isptr[1]
 * 
 */
    __pyx_v_ipl = (__pyx_v_isptr[0]);

    /* "tables/indexesextension.pyx":349
 *         isptr -= 2
 *         ipl = isptr[0]
 *         ipr = isptr[1]             # <<<<<<<<<<<<<<
 * 
 *     free(ivp)
 */
    __pyx_v_ipr = (__pyx_v_isptr[1]);
  }
  __pyx_L4_break:;

  /* "tables/indexesextension.pyx":351
 *         ipr = isptr[1]
 * 
 *     free(ivp)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  free(__pyx_v_ivp);

  /* "tables/indexesextension.pyx":212
 * 
 * @cython.cdivision(True)
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:             # <<<<<<<<<<<<<<
 *     cdef number_type *pl = start1
 *     cdef number_type *pr = start1 + (n - 1)
 */

  /* function exit code */
}

static void __pyx_fuse_3__pyx_f_6tables_16indexesextension__keysort(npy_int64 *__pyx_v_start1, char *__pyx_v_start2, size_t __pyx_v_elsize2, size_t __pyx_v_n) {
  npy_int64 *__pyx_v_pl;
  npy_int64 *__pyx_v_pr;
  char *__pyx_v_ipl;
  char *__pyx_v_ipr;
  npy_int64 __pyx_v_vp;
  char *__pyx_v_ivp;
  npy_int64 *__pyx_v_stack[0x64];
  npy_int64 **__pyx_v_sptr;
  char *__pyx_v_istack[0x64];
  char **__pyx_v_isptr;
  CYTHON_UNUSED size_t __pyx_v_stack_index;
  npy_int64 *__pyx_v_pm;
  npy_int64 *__pyx_v_pi;
  npy_int64 *__pyx_v_pj;
  npy_int64 *__pyx_v_pt;
  char *__pyx_v_ipm;
  char *__pyx_v_ipi;
  char *__pyx_v_ipj;
  char *__pyx_v_ipt;
  int __pyx_t_1;
  npy_int64 __pyx_t_2;
  npy_int64 __pyx_t_3;
  int __pyx_t_4;

  /* "tables/indexesextension.pyx":213
 * @cython.cdivision(True)
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:
 *     cdef number_type *pl = start1             # <<<<<<<<<<<<<<
 *     cdef number_type *pr = start1 + (n - 1)
 * 
 */
  __pyx_v_pl = __pyx_v_start1;

  /* "tables/indexesextension.pyx":214
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:
 *     cdef number_type *pl = start1
 *     cdef number_type *pr = start1 + (n - 1)             # <<<<<<<<<<<<<<
 * 
 *     cdef char *ipl = start2
 */
  __pyx_v_pr = (__pyx_v_start1 + (__pyx_v_n - 1));

  /* "tables/indexesextension.pyx":216
 *     cdef number_type *pr = start1 + (n - 1)
 * 
 *     cdef char *ipl = start2             # <<<<<<<<<<<<<<
 *     cdef char *ipr = start2 + (n - 1) * elsize2
 * 
 */
  __pyx_v_ipl = __pyx_v_start2;

  /* "tables/indexesextension.pyx":217
 * 
 *     cdef char *ipl = start2
 *     cdef char *ipr = start2 + (n - 1) * elsize2             # <<<<<<<<<<<<<<
 * 
 *     cdef number_type vp
 */
  __pyx_v_ipr = (__pyx_v_start2 + ((__pyx_v_n - 1) * __pyx_v_elsize2));

  /* "tables/indexesextension.pyx":220
 * 
 *     cdef number_type vp
 *     cdef char *ivp = <char *> malloc(elsize2)             # <<<<<<<<<<<<<<
 * 
 *     cdef number_type *stack[PYA_QS_STACK]
 */
  __pyx_v_ivp = ((char *)malloc(__pyx_v_elsize2));

  /* "tables/indexesextension.pyx":223
 * 
 *     cdef number_type *stack[PYA_QS_STACK]
 *     cdef number_type **sptr = stack             # <<<<<<<<<<<<<<
 * 
 *     cdef char *istack[PYA_QS_STACK]
 */
  __pyx_v_sptr = __pyx_v_stack;

  /* "tables/indexesextension.pyx":226
 * 
 *     cdef char *istack[PYA_QS_STACK]
 *     cdef char **isptr = istack             # <<<<<<<<<<<<<<
 * 
 *     cdef size_t stack_index = 0
 */
  __pyx_v_isptr = __pyx_v_istack;

  /* "tables/indexesextension.pyx":228
 *     cdef char **isptr = istack
 * 
 *     cdef size_t stack_index = 0             # <<<<<<<<<<<<<<
 * 
 *     cdef number_type *pm
 */
  __pyx_v_stack_index = 0;

  /* "tables/indexesextension.pyx":239
 *     cdef char *ipt
 * 
 *     while True:             # <<<<<<<<<<<<<<
 *         while pr - pl > SMALL_QUICKSORT:
 *             pm  = pl + ((pr - pl) >> 1)
 */
  while (1) {

    /* "tables/indexesextension.pyx":240
 * 
 *     while True:
 *         while pr - pl > SMALL_QUICKSORT:             # <<<<<<<<<<<<<<
 *             pm  = pl + ((pr - pl) >> 1)
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 */
    while (1) {
      __pyx_t_1 = (((__pyx_v_pr - __pyx_v_pl) > 15) != 0);
      if (!__pyx_t_1) break;

      /* "tables/indexesextension.pyx":241
 *     while True:
 *         while pr - pl > SMALL_QUICKSORT:
 *             pm  = pl + ((pr - pl) >> 1)             # <<<<<<<<<<<<<<
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 * 
 */
      __pyx_v_pm = (__pyx_v_pl + ((__pyx_v_pr - __pyx_v_pl) >> 1));

      /* "tables/indexesextension.pyx":242
 *         while pr - pl > SMALL_QUICKSORT:
 *             pm  = pl + ((pr - pl) >> 1)
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2             # <<<<<<<<<<<<<<
 * 
 *             if less_than(pm, pl):
 */
      __pyx_v_ipm = (__pyx_v_ipl + ((((__pyx_v_ipr - __pyx_v_ipl) / __pyx_v_elsize2) >> 1) * __pyx_v_elsize2));

      /* "tables/indexesextension.pyx":244
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      __pyx_t_1 = (__pyx_fuse_3__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pm, __pyx_v_pl) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":245
 * 
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 */
        __pyx_t_2 = (__pyx_v_pl[0]);
        __pyx_t_3 = (__pyx_v_pm[0]);
        (__pyx_v_pm[0]) = __pyx_t_2;
        (__pyx_v_pl[0]) = __pyx_t_3;

        /* "tables/indexesextension.pyx":246
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             if less_than(pr, pm):
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipl, __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":244
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      }

      /* "tables/indexesextension.pyx":248
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 *             if less_than(pr, pm):             # <<<<<<<<<<<<<<
 *                 pr[0], pm[0] =  pm[0], pr[0]
 *                 swap_bytes(ipr, ipm, elsize2)
 */
      __pyx_t_1 = (__pyx_fuse_3__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pr, __pyx_v_pm) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":249
 * 
 *             if less_than(pr, pm):
 *                 pr[0], pm[0] =  pm[0], pr[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipr, ipm, elsize2)
 * 
 */
        __pyx_t_3 = (__pyx_v_pm[0]);
        __pyx_t_2 = (__pyx_v_pr[0]);
        (__pyx_v_pr[0]) = __pyx_t_3;
        (__pyx_v_pm[0]) = __pyx_t_2;

        /* "tables/indexesextension.pyx":250
 *             if less_than(pr, pm):
 *                 pr[0], pm[0] =  pm[0], pr[0]
 *                 swap_bytes(ipr, ipm, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             if less_than(pm, pl):
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipr, __pyx_v_ipm, __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":248
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 *             if less_than(pr, pm):             # <<<<<<<<<<<<<<
 *                 pr[0], pm[0] =  pm[0], pr[0]
 *                 swap_bytes(ipr, ipm, elsize2)
 */
      }

      /* "tables/indexesextension.pyx":252
 *                 swap_bytes(ipr, ipm, elsize2)
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      __pyx_t_1 = (__pyx_fuse_3__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pm, __pyx_v_pl) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":253
 * 
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 */
        __pyx_t_2 = (__pyx_v_pl[0]);
        __pyx_t_3 = (__pyx_v_pm[0]);
        (__pyx_v_pm[0]) = __pyx_t_2;
        (__pyx_v_pl[0]) = __pyx_t_3;

        /* "tables/indexesextension.pyx":254
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             vp = pm[0]
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipl, __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":252
 *                 swap_bytes(ipr, ipm, elsize2)
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      }

      /* "tables/indexesextension.pyx":256
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 *             vp = pm[0]             # <<<<<<<<<<<<<<
 * 
 *             pi = pl
 */
      __pyx_v_vp = (__pyx_v_pm[0]);

      /* "tables/indexesextension.pyx":258
 *             vp = pm[0]
 * 
 *             pi = pl             # <<<<<<<<<<<<<<
 *             ipi = ipl
 * 
 */
      __pyx_v_pi = __pyx_v_pl;

      /* "tables/indexesextension.pyx":259
 * 
 *             pi = pl
 *             ipi = ipl             # <<<<<<<<<<<<<<
 * 
 *             pj = pr - 1
 */
      __pyx_v_ipi = __pyx_v_ipl;

      /* "tables/indexesextension.pyx":261
 *             ipi = ipl
 * 
 *             pj = pr - 1             # <<<<<<<<<<<<<<
 *             ipj = ipr - elsize2
 * 
 */
      __pyx_v_pj = (__pyx_v_pr - 1);

      /* "tables/indexesextension.pyx":262
 * 
 *             pj = pr - 1
 *             ipj = ipr - elsize2             # <<<<<<<<<<<<<<
 * 
 *             pm[0], pj[0] = pj[0], pm[0]
 */
      __pyx_v_ipj = (__pyx_v_ipr - __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":264
 *             ipj = ipr - elsize2
 * 
 *             pm[0], pj[0] = pj[0], pm[0]             # <<<<<<<<<<<<<<
 *             swap_bytes(ipm, ipj, elsize2)
 * 
 */
      __pyx_t_3 = (__pyx_v_pj[0]);
      __pyx_t_2 = (__pyx_v_pm[0]);
      (__pyx_v_pm[0]) = __pyx_t_3;
      (__pyx_v_pj[0]) = __pyx_t_2;

      /* "tables/indexesextension.pyx":265
 * 
 *             pm[0], pj[0] = pj[0], pm[0]
 *             swap_bytes(ipm, ipj, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             while True:
 */
      __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipj, __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":267
 *             swap_bytes(ipm, ipj, elsize2)
 * 
 *             while True:             # <<<<<<<<<<<<<<
 *                 pi += 1
 *                 ipi += elsize2
 */
      while (1) {

        /* "tables/indexesextension.pyx":268
 * 
 *             while True:
 *                 pi += 1             # <<<<<<<<<<<<<<
 *                 ipi += elsize2
 *                 while less_than(pi, &vp):
 */
        __pyx_v_pi = (__pyx_v_pi + 1);

        /* "tables/indexesextension.pyx":269
 *             while True:
 *                 pi += 1
 *                 ipi += elsize2             # <<<<<<<<<<<<<<
 *                 while less_than(pi, &vp):
 *                     pi += 1
 */
        __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":270
 *                 pi += 1
 *                 ipi += elsize2
 *                 while less_than(pi, &vp):             # <<<<<<<<<<<<<<
 *                     pi += 1
 *                     ipi += elsize2
 */
        while (1) {
          __pyx_t_1 = (__pyx_fuse_3__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pi, (&__pyx_v_vp)) != 0);
          if (!__pyx_t_1) break;

          /* "tables/indexesextension.pyx":271
 *                 ipi += elsize2
 *                 while less_than(pi, &vp):
 *                     pi += 1             # <<<<<<<<<<<<<<
 *                     ipi += elsize2
 * 
 */
          __pyx_v_pi = (__pyx_v_pi + 1);

          /* "tables/indexesextension.pyx":272
 *                 while less_than(pi, &vp):
 *                     pi += 1
 *                     ipi += elsize2             # <<<<<<<<<<<<<<
 * 
 *                 pj -= 1
 */
          __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_elsize2);
        }

        /* "tables/indexesextension.pyx":274
 *                     ipi += elsize2
 * 
 *                 pj -= 1             # <<<<<<<<<<<<<<
 *                 ipj -= elsize2
 *                 while less_than(&vp, pj):
 */
        __pyx_v_pj = (__pyx_v_pj - 1);

        /* "tables/indexesextension.pyx":275
 * 
 *                 pj -= 1
 *                 ipj -= elsize2             # <<<<<<<<<<<<<<
 *                 while less_than(&vp, pj):
 *                     pj -= 1
 */
        __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":276
 *                 pj -= 1
 *                 ipj -= elsize2
 *                 while less_than(&vp, pj):             # <<<<<<<<<<<<<<
 *                     pj -= 1
 *                     ipj -= elsize2
 */
        while (1) {
          __pyx_t_1 = (__pyx_fuse_3__pyx_f_6tables_16indexesextension_less_than((&__pyx_v_vp), __pyx_v_pj) != 0);
          if (!__pyx_t_1) break;

          /* "tables/indexesextension.pyx":277
 *                 ipj -= elsize2
 *                 while less_than(&vp, pj):
 *                     pj -= 1             # <<<<<<<<<<<<<<
 *                     ipj -= elsize2
 * 
 */
          __pyx_v_pj = (__pyx_v_pj - 1);

          /* "tables/indexesextension.pyx":278
 *                 while less_than(&vp, pj):
 *                     pj -= 1
 *                     ipj -= elsize2             # <<<<<<<<<<<<<<
 * 
 *                 if pi >= pj:
 */
          __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_elsize2);
        }

        /* "tables/indexesextension.pyx":280
 *                     ipj -= elsize2
 * 
 *                 if pi >= pj:             # <<<<<<<<<<<<<<
 *                     break
 * 
 */
        __pyx_t_1 = ((__pyx_v_pi >= __pyx_v_pj) != 0);
        if (__pyx_t_1) {

          /* "tables/indexesextension.pyx":281
 * 
 *                 if pi >= pj:
 *                     break             # <<<<<<<<<<<<<<
 * 
 *                 pi[0], pj[0] = pj[0], pi[0]
 */
          goto __pyx_L11_break;

          /* "tables/indexesextension.pyx":280
 *                     ipj -= elsize2
 * 
 *                 if pi >= pj:             # <<<<<<<<<<<<<<
 *                     break
 * 
 */
        }

        /* "tables/indexesextension.pyx":283
 *                     break
 * 
 *                 pi[0], pj[0] = pj[0], pi[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipi, ipj, elsize2)
 * 
 */
        __pyx_t_2 = (__pyx_v_pj[0]);
        __pyx_t_3 = (__pyx_v_pi[0]);
        (__pyx_v_pi[0]) = __pyx_t_2;
        (__pyx_v_pj[0]) = __pyx_t_3;

        /* "tables/indexesextension.pyx":284
 * 
 *                 pi[0], pj[0] = pj[0], pi[0]
 *                 swap_bytes(ipi, ipj, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             pi[0], (pr-1)[0] = (pr-1)[0], pi[0]
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipi, __pyx_v_ipj, __pyx_v_elsize2);
      }
      __pyx_L11_break:;

      /* "tables/indexesextension.pyx":286
 *                 swap_bytes(ipi, ipj, elsize2)
 * 
 *             pi[0], (pr-1)[0] = (pr-1)[0], pi[0]             # <<<<<<<<<<<<<<
 *             swap_bytes(ipi, ipr-elsize2, elsize2)
 * 
 */
      __pyx_t_3 = ((__pyx_v_pr - 1)[0]);
      __pyx_t_2 = (__pyx_v_pi[0]);
      (__pyx_v_pi[0]) = __pyx_t_3;
      ((__pyx_v_pr - 1)[0]) = __pyx_t_2;

      /* "tables/indexesextension.pyx":287
 * 
 *             pi[0], (pr-1)[0] = (pr-1)[0], pi[0]
 *             swap_bytes(ipi, ipr-elsize2, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             # push largest partition on stack and proceed with the other
 */
      __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipi, (__pyx_v_ipr - __pyx_v_elsize2), __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":290
 * 
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):             # <<<<<<<<<<<<<<
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr
 */
      __pyx_t_1 = (((__pyx_v_pi - __pyx_v_pl) < (__pyx_v_pr - __pyx_v_pi)) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":291
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):
 *                 sptr[0] = pi + 1             # <<<<<<<<<<<<<<
 *                 sptr[1] = pr
 *                 sptr += 2
 */
        (__pyx_v_sptr[0]) = (__pyx_v_pi + 1);

        /* "tables/indexesextension.pyx":292
 *             if (pi - pl) < (pr - pi):
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr             # <<<<<<<<<<<<<<
 *                 sptr += 2
 * 
 */
        (__pyx_v_sptr[1]) = __pyx_v_pr;

        /* "tables/indexesextension.pyx":293
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr
 *                 sptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 isptr[0] = ipi + elsize2
 */
        __pyx_v_sptr = (__pyx_v_sptr + 2);

        /* "tables/indexesextension.pyx":295
 *                 sptr += 2
 * 
 *                 isptr[0] = ipi + elsize2             # <<<<<<<<<<<<<<
 *                 isptr[1] = ipr
 *                 isptr += 2
 */
        (__pyx_v_isptr[0]) = (__pyx_v_ipi + __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":296
 * 
 *                 isptr[0] = ipi + elsize2
 *                 isptr[1] = ipr             # <<<<<<<<<<<<<<
 *                 isptr += 2
 * 
 */
        (__pyx_v_isptr[1]) = __pyx_v_ipr;

        /* "tables/indexesextension.pyx":297
 *                 isptr[0] = ipi + elsize2
 *                 isptr[1] = ipr
 *                 isptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 pr = pi - 1
 */
        __pyx_v_isptr = (__pyx_v_isptr + 2);

        /* "tables/indexesextension.pyx":299
 *                 isptr += 2
 * 
 *                 pr = pi - 1             # <<<<<<<<<<<<<<
 *                 ipr = ipi - elsize2
 *             else:
 */
        __pyx_v_pr = (__pyx_v_pi - 1);

        /* "tables/indexesextension.pyx":300
 * 
 *                 pr = pi - 1
 *                 ipr = ipi - elsize2             # <<<<<<<<<<<<<<
 *             else:
 *                 sptr[0] = pl
 */
        __pyx_v_ipr = (__pyx_v_ipi - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":290
 * 
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):             # <<<<<<<<<<<<<<
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr
 */
        goto __pyx_L17;
      }

      /* "tables/indexesextension.pyx":302
 *                 ipr = ipi - elsize2
 *             else:
 *                 sptr[0] = pl             # <<<<<<<<<<<<<<
 *                 sptr[1] = pi - 1
 *                 sptr += 2
 */
      /*else*/ {
        (__pyx_v_sptr[0]) = __pyx_v_pl;

        /* "tables/indexesextension.pyx":303
 *             else:
 *                 sptr[0] = pl
 *                 sptr[1] = pi - 1             # <<<<<<<<<<<<<<
 *                 sptr += 2
 * 
 */
        (__pyx_v_sptr[1]) = (__pyx_v_pi - 1);

        /* "tables/indexesextension.pyx":304
 *                 sptr[0] = pl
 *                 sptr[1] = pi - 1
 *                 sptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 isptr[0] = ipl
 */
        __pyx_v_sptr = (__pyx_v_sptr + 2);

        /* "tables/indexesextension.pyx":306
 *                 sptr += 2
 * 
 *                 isptr[0] = ipl             # <<<<<<<<<<<<<<
 *                 isptr[1] = ipi - elsize2
 *                 isptr += 2
 */
        (__pyx_v_isptr[0]) = __pyx_v_ipl;

        /* "tables/indexesextension.pyx":307
 * 
 *                 isptr[0] = ipl
 *                 isptr[1] = ipi - elsize2             # <<<<<<<<<<<<<<
 *                 isptr += 2
 * 
 */
        (__pyx_v_isptr[1]) = (__pyx_v_ipi - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":308
 *                 isptr[0] = ipl
 *                 isptr[1] = ipi - elsize2
 *                 isptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 pl = pi + 1
 */
        __pyx_v_isptr = (__pyx_v_isptr + 2);

        /* "tables/indexesextension.pyx":310
 *                 isptr += 2
 * 
 *                 pl = pi + 1             # <<<<<<<<<<<<<<
 *                 ipl = ipi + elsize2
 * 
 */
        __pyx_v_pl = (__pyx_v_pi + 1);

        /* "tables/indexesextension.pyx":311
 * 
 *                 pl = pi + 1
 *                 ipl = ipi + elsize2             # <<<<<<<<<<<<<<
 * 
 *         pi = pl + 1
 */
        __pyx_v_ipl = (__pyx_v_ipi + __pyx_v_elsize2);
      }
      __pyx_L17:;
    }

    /* "tables/indexesextension.pyx":313
 *                 ipl = ipi + elsize2
 * 
 *         pi = pl + 1             # <<<<<<<<<<<<<<
 *         ipi = ipl + elsize2
 *         while pi <= pr:
 */
    __pyx_v_pi = (__pyx_v_pl + 1);

    /* "tables/indexesextension.pyx":314
 * 
 *         pi = pl + 1
 *         ipi = ipl + elsize2             # <<<<<<<<<<<<<<
 *         while pi <= pr:
 *             vp = pi[0]
 */
    __pyx_v_ipi = (__pyx_v_ipl + __pyx_v_elsize2);

    /* "tables/indexesextension.pyx":315
 *         pi = pl + 1
 *         ipi = ipl + elsize2
 *         while pi <= pr:             # <<<<<<<<<<<<<<
 *             vp = pi[0]
 *             memcpy(ivp, ipi, elsize2)
 */
    while (1) {
      __pyx_t_1 = ((__pyx_v_pi <= __pyx_v_pr) != 0);
      if (!__pyx_t_1) break;

      /* "tables/indexesextension.pyx":316
 *         ipi = ipl + elsize2
 *         while pi <= pr:
 *             vp = pi[0]             # <<<<<<<<<<<<<<
 *             memcpy(ivp, ipi, elsize2)
 * 
 */
      __pyx_v_vp = (__pyx_v_pi[0]);

      /* "tables/indexesextension.pyx":317
 *         while pi <= pr:
 *             vp = pi[0]
 *             memcpy(ivp, ipi, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             pj = pi
 */
      (void)(memcpy(__pyx_v_ivp, __pyx_v_ipi, __pyx_v_elsize2));

      /* "tables/indexesextension.pyx":319
 *             memcpy(ivp, ipi, elsize2)
 * 
 *             pj = pi             # <<<<<<<<<<<<<<
 *             pt = pi - 1
 * 
 */
      __pyx_v_pj = __pyx_v_pi;

      /* "tables/indexesextension.pyx":320
 * 
 *             pj = pi
 *             pt = pi - 1             # <<<<<<<<<<<<<<
 * 
 *             ipj = ipi
 */
      __pyx_v_pt = (__pyx_v_pi - 1);

      /* "tables/indexesextension.pyx":322
 *             pt = pi - 1
 * 
 *             ipj = ipi             # <<<<<<<<<<<<<<
 *             ipt = ipi - elsize2
 * 
 */
      __pyx_v_ipj = __pyx_v_ipi;

      /* "tables/indexesextension.pyx":323
 * 
 *             ipj = ipi
 *             ipt = ipi - elsize2             # <<<<<<<<<<<<<<
 * 
 *             while pj > pl and less_than(&vp, pt):
 */
      __pyx_v_ipt = (__pyx_v_ipi - __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":325
 *             ipt = ipi - elsize2
 * 
 *             while pj > pl and less_than(&vp, pt):             # <<<<<<<<<<<<<<
 *                 pj[0] = pt[0]
 *                 pj -= 1
 */
      while (1) {
        __pyx_t_4 = ((__pyx_v_pj > __pyx_v_pl) != 0);
        if (__pyx_t_4) {
        } else {
          __pyx_t_1 = __pyx_t_4;
          goto __pyx_L22_bool_binop_done;
        }
        __pyx_t_4 = (__pyx_fuse_3__pyx_f_6tables_16indexesextension_less_than((&__pyx_v_vp), __pyx_v_pt) != 0);
        __pyx_t_1 = __pyx_t_4;
        __pyx_L22_bool_binop_done:;
        if (!__pyx_t_1) break;

        /* "tables/indexesextension.pyx":326
 * 
 *             while pj > pl and less_than(&vp, pt):
 *                 pj[0] = pt[0]             # <<<<<<<<<<<<<<
 *                 pj -= 1
 *                 pt -= 1
 */
        (__pyx_v_pj[0]) = (__pyx_v_pt[0]);

        /* "tables/indexesextension.pyx":327
 *             while pj > pl and less_than(&vp, pt):
 *                 pj[0] = pt[0]
 *                 pj -= 1             # <<<<<<<<<<<<<<
 *                 pt -= 1
 * 
 */
        __pyx_v_pj = (__pyx_v_pj - 1);

        /* "tables/indexesextension.pyx":328
 *                 pj[0] = pt[0]
 *                 pj -= 1
 *                 pt -= 1             # <<<<<<<<<<<<<<
 * 
 *                 memcpy(ipj, ipt, elsize2)
 */
        __pyx_v_pt = (__pyx_v_pt - 1);

        /* "tables/indexesextension.pyx":330
 *                 pt -= 1
 * 
 *                 memcpy(ipj, ipt, elsize2)             # <<<<<<<<<<<<<<
 *                 ipj -= elsize2
 *                 ipt -= elsize2
 */
        (void)(memcpy(__pyx_v_ipj, __pyx_v_ipt, __pyx_v_elsize2));

        /* "tables/indexesextension.pyx":331
 * 
 *                 memcpy(ipj, ipt, elsize2)
 *                 ipj -= elsize2             # <<<<<<<<<<<<<<
 *                 ipt -= elsize2
 * 
 */
        __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":332
 *                 memcpy(ipj, ipt, elsize2)
 *                 ipj -= elsize2
 *                 ipt -= elsize2             # <<<<<<<<<<<<<<
 * 
 *             pj[0] = vp
 */
        __pyx_v_ipt = (__pyx_v_ipt - __pyx_v_elsize2);
      }

      /* "tables/indexesextension.pyx":334
 *                 ipt -= elsize2
 * 
 *             pj[0] = vp             # <<<<<<<<<<<<<<
 *             memcpy(ipj, ivp, elsize2)
 * 
 */
      (__pyx_v_pj[0]) = __pyx_v_vp;

      /* "tables/indexesextension.pyx":335
 * 
 *             pj[0] = vp
 *             memcpy(ipj, ivp, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             pi += 1
 */
      (void)(memcpy(__pyx_v_ipj, __pyx_v_ivp, __pyx_v_elsize2));

      /* "tables/indexesextension.pyx":337
 *             memcpy(ipj, ivp, elsize2)
 * 
 *             pi += 1             # <<<<<<<<<<<<<<
 *             ipi += elsize2
 * 
 */
      __pyx_v_pi = (__pyx_v_pi + 1);

      /* "tables/indexesextension.pyx":338
 * 
 *             pi += 1
 *             ipi += elsize2             # <<<<<<<<<<<<<<
 * 
 *         if sptr == stack:
 */
      __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_elsize2);
    }

    /* "tables/indexesextension.pyx":340
 *             ipi += elsize2
 * 
 *         if sptr == stack:             # <<<<<<<<<<<<<<
 *             break
 * 
 */
    __pyx_t_1 = ((__pyx_v_sptr == __pyx_v_stack) != 0);
    if (__pyx_t_1) {

      /* "tables/indexesextension.pyx":341
 * 
 *         if sptr == stack:
 *             break             # <<<<<<<<<<<<<<
 * 
 *         sptr -= 2
 */
      goto __pyx_L4_break;

      /* "tables/indexesextension.pyx":340
 *             ipi += elsize2
 * 
 *         if sptr == stack:             # <<<<<<<<<<<<<<
 *             break
 * 
 */
    }

    /* "tables/indexesextension.pyx":343
 *             break
 * 
 *         sptr -= 2             # <<<<<<<<<<<<<<
 *         pl = sptr[0]
 *         pr = sptr[1]
 */
    __pyx_v_sptr = (__pyx_v_sptr - 2);

    /* "tables/indexesextension.pyx":344
 * 
 *         sptr -= 2
 *         pl = sptr[0]             # <<<<<<<<<<<<<<
 *         pr = sptr[1]
 * 
 */
    __pyx_v_pl = (__pyx_v_sptr[0]);

    /* "tables/indexesextension.pyx":345
 *         sptr -= 2
 *         pl = sptr[0]
 *         pr = sptr[1]             # <<<<<<<<<<<<<<
 * 
 *         isptr -= 2
 */
    __pyx_v_pr = (__pyx_v_sptr[1]);

    /* "tables/indexesextension.pyx":347
 *         pr = sptr[1]
 * 
 *         isptr -= 2             # <<<<<<<<<<<<<<
 *         ipl = isptr[0]
 *         ipr = isptr[1]
 */
    __pyx_v_isptr = (__pyx_v_isptr - 2);

    /* "tables/indexesextension.pyx":348
 * 
 *         isptr -= 2
 *         ipl = isptr[0]             # <<<<<<<<<<<<<<
 *         ipr = isptr[1]
 * 
 */
    __pyx_v_ipl = (__pyx_v_isptr[0]);

    /* "tables/indexesextension.pyx":349
 *         isptr -= 2
 *         ipl = isptr[0]
 *         ipr = isptr[1]             # <<<<<<<<<<<<<<
 * 
 *     free(ivp)
 */
    __pyx_v_ipr = (__pyx_v_isptr[1]);
  }
  __pyx_L4_break:;

  /* "tables/indexesextension.pyx":351
 *         ipr = isptr[1]
 * 
 *     free(ivp)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  free(__pyx_v_ivp);

  /* "tables/indexesextension.pyx":212
 * 
 * @cython.cdivision(True)
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:             # <<<<<<<<<<<<<<
 *     cdef number_type *pl = start1
 *     cdef number_type *pr = start1 + (n - 1)
 */

  /* function exit code */
}

static void __pyx_fuse_4__pyx_f_6tables_16indexesextension__keysort(npy_uint8 *__pyx_v_start1, char *__pyx_v_start2, size_t __pyx_v_elsize2, size_t __pyx_v_n) {
  npy_uint8 *__pyx_v_pl;
  npy_uint8 *__pyx_v_pr;
  char *__pyx_v_ipl;
  char *__pyx_v_ipr;
  npy_uint8 __pyx_v_vp;
  char *__pyx_v_ivp;
  npy_uint8 *__pyx_v_stack[0x64];
  npy_uint8 **__pyx_v_sptr;
  char *__pyx_v_istack[0x64];
  char **__pyx_v_isptr;
  CYTHON_UNUSED size_t __pyx_v_stack_index;
  npy_uint8 *__pyx_v_pm;
  npy_uint8 *__pyx_v_pi;
  npy_uint8 *__pyx_v_pj;
  npy_uint8 *__pyx_v_pt;
  char *__pyx_v_ipm;
  char *__pyx_v_ipi;
  char *__pyx_v_ipj;
  char *__pyx_v_ipt;
  int __pyx_t_1;
  npy_uint8 __pyx_t_2;
  npy_uint8 __pyx_t_3;
  int __pyx_t_4;

  /* "tables/indexesextension.pyx":213
 * @cython.cdivision(True)
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:
 *     cdef number_type *pl = start1             # <<<<<<<<<<<<<<
 *     cdef number_type *pr = start1 + (n - 1)
 * 
 */
  __pyx_v_pl = __pyx_v_start1;

  /* "tables/indexesextension.pyx":214
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:
 *     cdef number_type *pl = start1
 *     cdef number_type *pr = start1 + (n - 1)             # <<<<<<<<<<<<<<
 * 
 *     cdef char *ipl = start2
 */
  __pyx_v_pr = (__pyx_v_start1 + (__pyx_v_n - 1));

  /* "tables/indexesextension.pyx":216
 *     cdef number_type *pr = start1 + (n - 1)
 * 
 *     cdef char *ipl = start2             # <<<<<<<<<<<<<<
 *     cdef char *ipr = start2 + (n - 1) * elsize2
 * 
 */
  __pyx_v_ipl = __pyx_v_start2;

  /* "tables/indexesextension.pyx":217
 * 
 *     cdef char *ipl = start2
 *     cdef char *ipr = start2 + (n - 1) * elsize2             # <<<<<<<<<<<<<<
 * 
 *     cdef number_type vp
 */
  __pyx_v_ipr = (__pyx_v_start2 + ((__pyx_v_n - 1) * __pyx_v_elsize2));

  /* "tables/indexesextension.pyx":220
 * 
 *     cdef number_type vp
 *     cdef char *ivp = <char *> malloc(elsize2)             # <<<<<<<<<<<<<<
 * 
 *     cdef number_type *stack[PYA_QS_STACK]
 */
  __pyx_v_ivp = ((char *)malloc(__pyx_v_elsize2));

  /* "tables/indexesextension.pyx":223
 * 
 *     cdef number_type *stack[PYA_QS_STACK]
 *     cdef number_type **sptr = stack             # <<<<<<<<<<<<<<
 * 
 *     cdef char *istack[PYA_QS_STACK]
 */
  __pyx_v_sptr = __pyx_v_stack;

  /* "tables/indexesextension.pyx":226
 * 
 *     cdef char *istack[PYA_QS_STACK]
 *     cdef char **isptr = istack             # <<<<<<<<<<<<<<
 * 
 *     cdef size_t stack_index = 0
 */
  __pyx_v_isptr = __pyx_v_istack;

  /* "tables/indexesextension.pyx":228
 *     cdef char **isptr = istack
 * 
 *     cdef size_t stack_index = 0             # <<<<<<<<<<<<<<
 * 
 *     cdef number_type *pm
 */
  __pyx_v_stack_index = 0;

  /* "tables/indexesextension.pyx":239
 *     cdef char *ipt
 * 
 *     while True:             # <<<<<<<<<<<<<<
 *         while pr - pl > SMALL_QUICKSORT:
 *             pm  = pl + ((pr - pl) >> 1)
 */
  while (1) {

    /* "tables/indexesextension.pyx":240
 * 
 *     while True:
 *         while pr - pl > SMALL_QUICKSORT:             # <<<<<<<<<<<<<<
 *             pm  = pl + ((pr - pl) >> 1)
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 */
    while (1) {
      __pyx_t_1 = (((__pyx_v_pr - __pyx_v_pl) > 15) != 0);
      if (!__pyx_t_1) break;

      /* "tables/indexesextension.pyx":241
 *     while True:
 *         while pr - pl > SMALL_QUICKSORT:
 *             pm  = pl + ((pr - pl) >> 1)             # <<<<<<<<<<<<<<
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 * 
 */
      __pyx_v_pm = (__pyx_v_pl + ((__pyx_v_pr - __pyx_v_pl) >> 1));

      /* "tables/indexesextension.pyx":242
 *         while pr - pl > SMALL_QUICKSORT:
 *             pm  = pl + ((pr - pl) >> 1)
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2             # <<<<<<<<<<<<<<
 * 
 *             if less_than(pm, pl):
 */
      __pyx_v_ipm = (__pyx_v_ipl + ((((__pyx_v_ipr - __pyx_v_ipl) / __pyx_v_elsize2) >> 1) * __pyx_v_elsize2));

      /* "tables/indexesextension.pyx":244
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      __pyx_t_1 = (__pyx_fuse_4__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pm, __pyx_v_pl) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":245
 * 
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 */
        __pyx_t_2 = (__pyx_v_pl[0]);
        __pyx_t_3 = (__pyx_v_pm[0]);
        (__pyx_v_pm[0]) = __pyx_t_2;
        (__pyx_v_pl[0]) = __pyx_t_3;

        /* "tables/indexesextension.pyx":246
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             if less_than(pr, pm):
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipl, __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":244
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      }

      /* "tables/indexesextension.pyx":248
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 *             if less_than(pr, pm):             # <<<<<<<<<<<<<<
 *                 pr[0], pm[0] =  pm[0], pr[0]
 *                 swap_bytes(ipr, ipm, elsize2)
 */
      __pyx_t_1 = (__pyx_fuse_4__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pr, __pyx_v_pm) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":249
 * 
 *             if less_than(pr, pm):
 *                 pr[0], pm[0] =  pm[0], pr[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipr, ipm, elsize2)
 * 
 */
        __pyx_t_3 = (__pyx_v_pm[0]);
        __pyx_t_2 = (__pyx_v_pr[0]);
        (__pyx_v_pr[0]) = __pyx_t_3;
        (__pyx_v_pm[0]) = __pyx_t_2;

        /* "tables/indexesextension.pyx":250
 *             if less_than(pr, pm):
 *                 pr[0], pm[0] =  pm[0], pr[0]
 *                 swap_bytes(ipr, ipm, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             if less_than(pm, pl):
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipr, __pyx_v_ipm, __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":248
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 *             if less_than(pr, pm):             # <<<<<<<<<<<<<<
 *                 pr[0], pm[0] =  pm[0], pr[0]
 *                 swap_bytes(ipr, ipm, elsize2)
 */
      }

      /* "tables/indexesextension.pyx":252
 *                 swap_bytes(ipr, ipm, elsize2)
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      __pyx_t_1 = (__pyx_fuse_4__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pm, __pyx_v_pl) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":253
 * 
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 */
        __pyx_t_2 = (__pyx_v_pl[0]);
        __pyx_t_3 = (__pyx_v_pm[0]);
        (__pyx_v_pm[0]) = __pyx_t_2;
        (__pyx_v_pl[0]) = __pyx_t_3;

        /* "tables/indexesextension.pyx":254
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             vp = pm[0]
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipl, __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":252
 *                 swap_bytes(ipr, ipm, elsize2)
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      }

      /* "tables/indexesextension.pyx":256
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 *             vp = pm[0]             # <<<<<<<<<<<<<<
 * 
 *             pi = pl
 */
      __pyx_v_vp = (__pyx_v_pm[0]);

      /* "tables/indexesextension.pyx":258
 *             vp = pm[0]
 * 
 *             pi = pl             # <<<<<<<<<<<<<<
 *             ipi = ipl
 * 
 */
      __pyx_v_pi = __pyx_v_pl;

      /* "tables/indexesextension.pyx":259
 * 
 *             pi = pl
 *             ipi = ipl             # <<<<<<<<<<<<<<
 * 
 *             pj = pr - 1
 */
      __pyx_v_ipi = __pyx_v_ipl;

      /* "tables/indexesextension.pyx":261
 *             ipi = ipl
 * 
 *             pj = pr - 1             # <<<<<<<<<<<<<<
 *             ipj = ipr - elsize2
 * 
 */
      __pyx_v_pj = (__pyx_v_pr - 1);

      /* "tables/indexesextension.pyx":262
 * 
 *             pj = pr - 1
 *             ipj = ipr - elsize2             # <<<<<<<<<<<<<<
 * 
 *             pm[0], pj[0] = pj[0], pm[0]
 */
      __pyx_v_ipj = (__pyx_v_ipr - __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":264
 *             ipj = ipr - elsize2
 * 
 *             pm[0], pj[0] = pj[0], pm[0]             # <<<<<<<<<<<<<<
 *             swap_bytes(ipm, ipj, elsize2)
 * 
 */
      __pyx_t_3 = (__pyx_v_pj[0]);
      __pyx_t_2 = (__pyx_v_pm[0]);
      (__pyx_v_pm[0]) = __pyx_t_3;
      (__pyx_v_pj[0]) = __pyx_t_2;

      /* "tables/indexesextension.pyx":265
 * 
 *             pm[0], pj[0] = pj[0], pm[0]
 *             swap_bytes(ipm, ipj, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             while True:
 */
      __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipj, __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":267
 *             swap_bytes(ipm, ipj, elsize2)
 * 
 *             while True:             # <<<<<<<<<<<<<<
 *                 pi += 1
 *                 ipi += elsize2
 */
      while (1) {

        /* "tables/indexesextension.pyx":268
 * 
 *             while True:
 *                 pi += 1             # <<<<<<<<<<<<<<
 *                 ipi += elsize2
 *                 while less_than(pi, &vp):
 */
        __pyx_v_pi = (__pyx_v_pi + 1);

        /* "tables/indexesextension.pyx":269
 *             while True:
 *                 pi += 1
 *                 ipi += elsize2             # <<<<<<<<<<<<<<
 *                 while less_than(pi, &vp):
 *                     pi += 1
 */
        __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":270
 *                 pi += 1
 *                 ipi += elsize2
 *                 while less_than(pi, &vp):             # <<<<<<<<<<<<<<
 *                     pi += 1
 *                     ipi += elsize2
 */
        while (1) {
          __pyx_t_1 = (__pyx_fuse_4__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pi, (&__pyx_v_vp)) != 0);
          if (!__pyx_t_1) break;

          /* "tables/indexesextension.pyx":271
 *                 ipi += elsize2
 *                 while less_than(pi, &vp):
 *                     pi += 1             # <<<<<<<<<<<<<<
 *                     ipi += elsize2
 * 
 */
          __pyx_v_pi = (__pyx_v_pi + 1);

          /* "tables/indexesextension.pyx":272
 *                 while less_than(pi, &vp):
 *                     pi += 1
 *                     ipi += elsize2             # <<<<<<<<<<<<<<
 * 
 *                 pj -= 1
 */
          __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_elsize2);
        }

        /* "tables/indexesextension.pyx":274
 *                     ipi += elsize2
 * 
 *                 pj -= 1             # <<<<<<<<<<<<<<
 *                 ipj -= elsize2
 *                 while less_than(&vp, pj):
 */
        __pyx_v_pj = (__pyx_v_pj - 1);

        /* "tables/indexesextension.pyx":275
 * 
 *                 pj -= 1
 *                 ipj -= elsize2             # <<<<<<<<<<<<<<
 *                 while less_than(&vp, pj):
 *                     pj -= 1
 */
        __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":276
 *                 pj -= 1
 *                 ipj -= elsize2
 *                 while less_than(&vp, pj):             # <<<<<<<<<<<<<<
 *                     pj -= 1
 *                     ipj -= elsize2
 */
        while (1) {
          __pyx_t_1 = (__pyx_fuse_4__pyx_f_6tables_16indexesextension_less_than((&__pyx_v_vp), __pyx_v_pj) != 0);
          if (!__pyx_t_1) break;

          /* "tables/indexesextension.pyx":277
 *                 ipj -= elsize2
 *                 while less_than(&vp, pj):
 *                     pj -= 1             # <<<<<<<<<<<<<<
 *                     ipj -= elsize2
 * 
 */
          __pyx_v_pj = (__pyx_v_pj - 1);

          /* "tables/indexesextension.pyx":278
 *                 while less_than(&vp, pj):
 *                     pj -= 1
 *                     ipj -= elsize2             # <<<<<<<<<<<<<<
 * 
 *                 if pi >= pj:
 */
          __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_elsize2);
        }

        /* "tables/indexesextension.pyx":280
 *                     ipj -= elsize2
 * 
 *                 if pi >= pj:             # <<<<<<<<<<<<<<
 *                     break
 * 
 */
        __pyx_t_1 = ((__pyx_v_pi >= __pyx_v_pj) != 0);
        if (__pyx_t_1) {

          /* "tables/indexesextension.pyx":281
 * 
 *                 if pi >= pj:
 *                     break             # <<<<<<<<<<<<<<
 * 
 *                 pi[0], pj[0] = pj[0], pi[0]
 */
          goto __pyx_L11_break;

          /* "tables/indexesextension.pyx":280
 *                     ipj -= elsize2
 * 
 *                 if pi >= pj:             # <<<<<<<<<<<<<<
 *                     break
 * 
 */
        }

        /* "tables/indexesextension.pyx":283
 *                     break
 * 
 *                 pi[0], pj[0] = pj[0], pi[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipi, ipj, elsize2)
 * 
 */
        __pyx_t_2 = (__pyx_v_pj[0]);
        __pyx_t_3 = (__pyx_v_pi[0]);
        (__pyx_v_pi[0]) = __pyx_t_2;
        (__pyx_v_pj[0]) = __pyx_t_3;

        /* "tables/indexesextension.pyx":284
 * 
 *                 pi[0], pj[0] = pj[0], pi[0]
 *                 swap_bytes(ipi, ipj, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             pi[0], (pr-1)[0] = (pr-1)[0], pi[0]
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipi, __pyx_v_ipj, __pyx_v_elsize2);
      }
      __pyx_L11_break:;

      /* "tables/indexesextension.pyx":286
 *                 swap_bytes(ipi, ipj, elsize2)
 * 
 *             pi[0], (pr-1)[0] = (pr-1)[0], pi[0]             # <<<<<<<<<<<<<<
 *             swap_bytes(ipi, ipr-elsize2, elsize2)
 * 
 */
      __pyx_t_3 = ((__pyx_v_pr - 1)[0]);
      __pyx_t_2 = (__pyx_v_pi[0]);
      (__pyx_v_pi[0]) = __pyx_t_3;
      ((__pyx_v_pr - 1)[0]) = __pyx_t_2;

      /* "tables/indexesextension.pyx":287
 * 
 *             pi[0], (pr-1)[0] = (pr-1)[0], pi[0]
 *             swap_bytes(ipi, ipr-elsize2, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             # push largest partition on stack and proceed with the other
 */
      __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipi, (__pyx_v_ipr - __pyx_v_elsize2), __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":290
 * 
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):             # <<<<<<<<<<<<<<
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr
 */
      __pyx_t_1 = (((__pyx_v_pi - __pyx_v_pl) < (__pyx_v_pr - __pyx_v_pi)) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":291
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):
 *                 sptr[0] = pi + 1             # <<<<<<<<<<<<<<
 *                 sptr[1] = pr
 *                 sptr += 2
 */
        (__pyx_v_sptr[0]) = (__pyx_v_pi + 1);

        /* "tables/indexesextension.pyx":292
 *             if (pi - pl) < (pr - pi):
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr             # <<<<<<<<<<<<<<
 *                 sptr += 2
 * 
 */
        (__pyx_v_sptr[1]) = __pyx_v_pr;

        /* "tables/indexesextension.pyx":293
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr
 *                 sptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 isptr[0] = ipi + elsize2
 */
        __pyx_v_sptr = (__pyx_v_sptr + 2);

        /* "tables/indexesextension.pyx":295
 *                 sptr += 2
 * 
 *                 isptr[0] = ipi + elsize2             # <<<<<<<<<<<<<<
 *                 isptr[1] = ipr
 *                 isptr += 2
 */
        (__pyx_v_isptr[0]) = (__pyx_v_ipi + __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":296
 * 
 *                 isptr[0] = ipi + elsize2
 *                 isptr[1] = ipr             # <<<<<<<<<<<<<<
 *                 isptr += 2
 * 
 */
        (__pyx_v_isptr[1]) = __pyx_v_ipr;

        /* "tables/indexesextension.pyx":297
 *                 isptr[0] = ipi + elsize2
 *                 isptr[1] = ipr
 *                 isptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 pr = pi - 1
 */
        __pyx_v_isptr = (__pyx_v_isptr + 2);

        /* "tables/indexesextension.pyx":299
 *                 isptr += 2
 * 
 *                 pr = pi - 1             # <<<<<<<<<<<<<<
 *                 ipr = ipi - elsize2
 *             else:
 */
        __pyx_v_pr = (__pyx_v_pi - 1);

        /* "tables/indexesextension.pyx":300
 * 
 *                 pr = pi - 1
 *                 ipr = ipi - elsize2             # <<<<<<<<<<<<<<
 *             else:
 *                 sptr[0] = pl
 */
        __pyx_v_ipr = (__pyx_v_ipi - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":290
 * 
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):             # <<<<<<<<<<<<<<
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr
 */
        goto __pyx_L17;
      }

      /* "tables/indexesextension.pyx":302
 *                 ipr = ipi - elsize2
 *             else:
 *                 sptr[0] = pl             # <<<<<<<<<<<<<<
 *                 sptr[1] = pi - 1
 *                 sptr += 2
 */
      /*else*/ {
        (__pyx_v_sptr[0]) = __pyx_v_pl;

        /* "tables/indexesextension.pyx":303
 *             else:
 *                 sptr[0] = pl
 *                 sptr[1] = pi - 1             # <<<<<<<<<<<<<<
 *                 sptr += 2
 * 
 */
        (__pyx_v_sptr[1]) = (__pyx_v_pi - 1);

        /* "tables/indexesextension.pyx":304
 *                 sptr[0] = pl
 *                 sptr[1] = pi - 1
 *                 sptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 isptr[0] = ipl
 */
        __pyx_v_sptr = (__pyx_v_sptr + 2);

        /* "tables/indexesextension.pyx":306
 *                 sptr += 2
 * 
 *                 isptr[0] = ipl             # <<<<<<<<<<<<<<
 *                 isptr[1] = ipi - elsize2
 *                 isptr += 2
 */
        (__pyx_v_isptr[0]) = __pyx_v_ipl;

        /* "tables/indexesextension.pyx":307
 * 
 *                 isptr[0] = ipl
 *                 isptr[1] = ipi - elsize2             # <<<<<<<<<<<<<<
 *                 isptr += 2
 * 
 */
        (__pyx_v_isptr[1]) = (__pyx_v_ipi - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":308
 *                 isptr[0] = ipl
 *                 isptr[1] = ipi - elsize2
 *                 isptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 pl = pi + 1
 */
        __pyx_v_isptr = (__pyx_v_isptr + 2);

        /* "tables/indexesextension.pyx":310
 *                 isptr += 2
 * 
 *                 pl = pi + 1             # <<<<<<<<<<<<<<
 *                 ipl = ipi + elsize2
 * 
 */
        __pyx_v_pl = (__pyx_v_pi + 1);

        /* "tables/indexesextension.pyx":311
 * 
 *                 pl = pi + 1
 *                 ipl = ipi + elsize2             # <<<<<<<<<<<<<<
 * 
 *         pi = pl + 1
 */
        __pyx_v_ipl = (__pyx_v_ipi + __pyx_v_elsize2);
      }
      __pyx_L17:;
    }

    /* "tables/indexesextension.pyx":313
 *                 ipl = ipi + elsize2
 * 
 *         pi = pl + 1             # <<<<<<<<<<<<<<
 *         ipi = ipl + elsize2
 *         while pi <= pr:
 */
    __pyx_v_pi = (__pyx_v_pl + 1);

    /* "tables/indexesextension.pyx":314
 * 
 *         pi = pl + 1
 *         ipi = ipl + elsize2             # <<<<<<<<<<<<<<
 *         while pi <= pr:
 *             vp = pi[0]
 */
    __pyx_v_ipi = (__pyx_v_ipl + __pyx_v_elsize2);

    /* "tables/indexesextension.pyx":315
 *         pi = pl + 1
 *         ipi = ipl + elsize2
 *         while pi <= pr:             # <<<<<<<<<<<<<<
 *             vp = pi[0]
 *             memcpy(ivp, ipi, elsize2)
 */
    while (1) {
      __pyx_t_1 = ((__pyx_v_pi <= __pyx_v_pr) != 0);
      if (!__pyx_t_1) break;

      /* "tables/indexesextension.pyx":316
 *         ipi = ipl + elsize2
 *         while pi <= pr:
 *             vp = pi[0]             # <<<<<<<<<<<<<<
 *             memcpy(ivp, ipi, elsize2)
 * 
 */
      __pyx_v_vp = (__pyx_v_pi[0]);

      /* "tables/indexesextension.pyx":317
 *         while pi <= pr:
 *             vp = pi[0]
 *             memcpy(ivp, ipi, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             pj = pi
 */
      (void)(memcpy(__pyx_v_ivp, __pyx_v_ipi, __pyx_v_elsize2));

      /* "tables/indexesextension.pyx":319
 *             memcpy(ivp, ipi, elsize2)
 * 
 *             pj = pi             # <<<<<<<<<<<<<<
 *             pt = pi - 1
 * 
 */
      __pyx_v_pj = __pyx_v_pi;

      /* "tables/indexesextension.pyx":320
 * 
 *             pj = pi
 *             pt = pi - 1             # <<<<<<<<<<<<<<
 * 
 *             ipj = ipi
 */
      __pyx_v_pt = (__pyx_v_pi - 1);

      /* "tables/indexesextension.pyx":322
 *             pt = pi - 1
 * 
 *             ipj = ipi             # <<<<<<<<<<<<<<
 *             ipt = ipi - elsize2
 * 
 */
      __pyx_v_ipj = __pyx_v_ipi;

      /* "tables/indexesextension.pyx":323
 * 
 *             ipj = ipi
 *             ipt = ipi - elsize2             # <<<<<<<<<<<<<<
 * 
 *             while pj > pl and less_than(&vp, pt):
 */
      __pyx_v_ipt = (__pyx_v_ipi - __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":325
 *             ipt = ipi - elsize2
 * 
 *             while pj > pl and less_than(&vp, pt):             # <<<<<<<<<<<<<<
 *                 pj[0] = pt[0]
 *                 pj -= 1
 */
      while (1) {
        __pyx_t_4 = ((__pyx_v_pj > __pyx_v_pl) != 0);
        if (__pyx_t_4) {
        } else {
          __pyx_t_1 = __pyx_t_4;
          goto __pyx_L22_bool_binop_done;
        }
        __pyx_t_4 = (__pyx_fuse_4__pyx_f_6tables_16indexesextension_less_than((&__pyx_v_vp), __pyx_v_pt) != 0);
        __pyx_t_1 = __pyx_t_4;
        __pyx_L22_bool_binop_done:;
        if (!__pyx_t_1) break;

        /* "tables/indexesextension.pyx":326
 * 
 *             while pj > pl and less_than(&vp, pt):
 *                 pj[0] = pt[0]             # <<<<<<<<<<<<<<
 *                 pj -= 1
 *                 pt -= 1
 */
        (__pyx_v_pj[0]) = (__pyx_v_pt[0]);

        /* "tables/indexesextension.pyx":327
 *             while pj > pl and less_than(&vp, pt):
 *                 pj[0] = pt[0]
 *                 pj -= 1             # <<<<<<<<<<<<<<
 *                 pt -= 1
 * 
 */
        __pyx_v_pj = (__pyx_v_pj - 1);

        /* "tables/indexesextension.pyx":328
 *                 pj[0] = pt[0]
 *                 pj -= 1
 *                 pt -= 1             # <<<<<<<<<<<<<<
 * 
 *                 memcpy(ipj, ipt, elsize2)
 */
        __pyx_v_pt = (__pyx_v_pt - 1);

        /* "tables/indexesextension.pyx":330
 *                 pt -= 1
 * 
 *                 memcpy(ipj, ipt, elsize2)             # <<<<<<<<<<<<<<
 *                 ipj -= elsize2
 *                 ipt -= elsize2
 */
        (void)(memcpy(__pyx_v_ipj, __pyx_v_ipt, __pyx_v_elsize2));

        /* "tables/indexesextension.pyx":331
 * 
 *                 memcpy(ipj, ipt, elsize2)
 *                 ipj -= elsize2             # <<<<<<<<<<<<<<
 *                 ipt -= elsize2
 * 
 */
        __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":332
 *                 memcpy(ipj, ipt, elsize2)
 *                 ipj -= elsize2
 *                 ipt -= elsize2             # <<<<<<<<<<<<<<
 * 
 *             pj[0] = vp
 */
        __pyx_v_ipt = (__pyx_v_ipt - __pyx_v_elsize2);
      }

      /* "tables/indexesextension.pyx":334
 *                 ipt -= elsize2
 * 
 *             pj[0] = vp             # <<<<<<<<<<<<<<
 *             memcpy(ipj, ivp, elsize2)
 * 
 */
      (__pyx_v_pj[0]) = __pyx_v_vp;

      /* "tables/indexesextension.pyx":335
 * 
 *             pj[0] = vp
 *             memcpy(ipj, ivp, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             pi += 1
 */
      (void)(memcpy(__pyx_v_ipj, __pyx_v_ivp, __pyx_v_elsize2));

      /* "tables/indexesextension.pyx":337
 *             memcpy(ipj, ivp, elsize2)
 * 
 *             pi += 1             # <<<<<<<<<<<<<<
 *             ipi += elsize2
 * 
 */
      __pyx_v_pi = (__pyx_v_pi + 1);

      /* "tables/indexesextension.pyx":338
 * 
 *             pi += 1
 *             ipi += elsize2             # <<<<<<<<<<<<<<
 * 
 *         if sptr == stack:
 */
      __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_elsize2);
    }

    /* "tables/indexesextension.pyx":340
 *             ipi += elsize2
 * 
 *         if sptr == stack:             # <<<<<<<<<<<<<<
 *             break
 * 
 */
    __pyx_t_1 = ((__pyx_v_sptr == __pyx_v_stack) != 0);
    if (__pyx_t_1) {

      /* "tables/indexesextension.pyx":341
 * 
 *         if sptr == stack:
 *             break             # <<<<<<<<<<<<<<
 * 
 *         sptr -= 2
 */
      goto __pyx_L4_break;

      /* "tables/indexesextension.pyx":340
 *             ipi += elsize2
 * 
 *         if sptr == stack:             # <<<<<<<<<<<<<<
 *             break
 * 
 */
    }

    /* "tables/indexesextension.pyx":343
 *             break
 * 
 *         sptr -= 2             # <<<<<<<<<<<<<<
 *         pl = sptr[0]
 *         pr = sptr[1]
 */
    __pyx_v_sptr = (__pyx_v_sptr - 2);

    /* "tables/indexesextension.pyx":344
 * 
 *         sptr -= 2
 *         pl = sptr[0]             # <<<<<<<<<<<<<<
 *         pr = sptr[1]
 * 
 */
    __pyx_v_pl = (__pyx_v_sptr[0]);

    /* "tables/indexesextension.pyx":345
 *         sptr -= 2
 *         pl = sptr[0]
 *         pr = sptr[1]             # <<<<<<<<<<<<<<
 * 
 *         isptr -= 2
 */
    __pyx_v_pr = (__pyx_v_sptr[1]);

    /* "tables/indexesextension.pyx":347
 *         pr = sptr[1]
 * 
 *         isptr -= 2             # <<<<<<<<<<<<<<
 *         ipl = isptr[0]
 *         ipr = isptr[1]
 */
    __pyx_v_isptr = (__pyx_v_isptr - 2);

    /* "tables/indexesextension.pyx":348
 * 
 *         isptr -= 2
 *         ipl = isptr[0]             # <<<<<<<<<<<<<<
 *         ipr = isptr[1]
 * 
 */
    __pyx_v_ipl = (__pyx_v_isptr[0]);

    /* "tables/indexesextension.pyx":349
 *         isptr -= 2
 *         ipl = isptr[0]
 *         ipr = isptr[1]             # <<<<<<<<<<<<<<
 * 
 *     free(ivp)
 */
    __pyx_v_ipr = (__pyx_v_isptr[1]);
  }
  __pyx_L4_break:;

  /* "tables/indexesextension.pyx":351
 *         ipr = isptr[1]
 * 
 *     free(ivp)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  free(__pyx_v_ivp);

  /* "tables/indexesextension.pyx":212
 * 
 * @cython.cdivision(True)
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:             # <<<<<<<<<<<<<<
 *     cdef number_type *pl = start1
 *     cdef number_type *pr = start1 + (n - 1)
 */

  /* function exit code */
}

static void __pyx_fuse_5__pyx_f_6tables_16indexesextension__keysort(npy_uint16 *__pyx_v_start1, char *__pyx_v_start2, size_t __pyx_v_elsize2, size_t __pyx_v_n) {
  npy_uint16 *__pyx_v_pl;
  npy_uint16 *__pyx_v_pr;
  char *__pyx_v_ipl;
  char *__pyx_v_ipr;
  npy_uint16 __pyx_v_vp;
  char *__pyx_v_ivp;
  npy_uint16 *__pyx_v_stack[0x64];
  npy_uint16 **__pyx_v_sptr;
  char *__pyx_v_istack[0x64];
  char **__pyx_v_isptr;
  CYTHON_UNUSED size_t __pyx_v_stack_index;
  npy_uint16 *__pyx_v_pm;
  npy_uint16 *__pyx_v_pi;
  npy_uint16 *__pyx_v_pj;
  npy_uint16 *__pyx_v_pt;
  char *__pyx_v_ipm;
  char *__pyx_v_ipi;
  char *__pyx_v_ipj;
  char *__pyx_v_ipt;
  int __pyx_t_1;
  npy_uint16 __pyx_t_2;
  npy_uint16 __pyx_t_3;
  int __pyx_t_4;

  /* "tables/indexesextension.pyx":213
 * @cython.cdivision(True)
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:
 *     cdef number_type *pl = start1             # <<<<<<<<<<<<<<
 *     cdef number_type *pr = start1 + (n - 1)
 * 
 */
  __pyx_v_pl = __pyx_v_start1;

  /* "tables/indexesextension.pyx":214
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:
 *     cdef number_type *pl = start1
 *     cdef number_type *pr = start1 + (n - 1)             # <<<<<<<<<<<<<<
 * 
 *     cdef char *ipl = start2
 */
  __pyx_v_pr = (__pyx_v_start1 + (__pyx_v_n - 1));

  /* "tables/indexesextension.pyx":216
 *     cdef number_type *pr = start1 + (n - 1)
 * 
 *     cdef char *ipl = start2             # <<<<<<<<<<<<<<
 *     cdef char *ipr = start2 + (n - 1) * elsize2
 * 
 */
  __pyx_v_ipl = __pyx_v_start2;

  /* "tables/indexesextension.pyx":217
 * 
 *     cdef char *ipl = start2
 *     cdef char *ipr = start2 + (n - 1) * elsize2             # <<<<<<<<<<<<<<
 * 
 *     cdef number_type vp
 */
  __pyx_v_ipr = (__pyx_v_start2 + ((__pyx_v_n - 1) * __pyx_v_elsize2));

  /* "tables/indexesextension.pyx":220
 * 
 *     cdef number_type vp
 *     cdef char *ivp = <char *> malloc(elsize2)             # <<<<<<<<<<<<<<
 * 
 *     cdef number_type *stack[PYA_QS_STACK]
 */
  __pyx_v_ivp = ((char *)malloc(__pyx_v_elsize2));

  /* "tables/indexesextension.pyx":223
 * 
 *     cdef number_type *stack[PYA_QS_STACK]
 *     cdef number_type **sptr = stack             # <<<<<<<<<<<<<<
 * 
 *     cdef char *istack[PYA_QS_STACK]
 */
  __pyx_v_sptr = __pyx_v_stack;

  /* "tables/indexesextension.pyx":226
 * 
 *     cdef char *istack[PYA_QS_STACK]
 *     cdef char **isptr = istack             # <<<<<<<<<<<<<<
 * 
 *     cdef size_t stack_index = 0
 */
  __pyx_v_isptr = __pyx_v_istack;

  /* "tables/indexesextension.pyx":228
 *     cdef char **isptr = istack
 * 
 *     cdef size_t stack_index = 0             # <<<<<<<<<<<<<<
 * 
 *     cdef number_type *pm
 */
  __pyx_v_stack_index = 0;

  /* "tables/indexesextension.pyx":239
 *     cdef char *ipt
 * 
 *     while True:             # <<<<<<<<<<<<<<
 *         while pr - pl > SMALL_QUICKSORT:
 *             pm  = pl + ((pr - pl) >> 1)
 */
  while (1) {

    /* "tables/indexesextension.pyx":240
 * 
 *     while True:
 *         while pr - pl > SMALL_QUICKSORT:             # <<<<<<<<<<<<<<
 *             pm  = pl + ((pr - pl) >> 1)
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 */
    while (1) {
      __pyx_t_1 = (((__pyx_v_pr - __pyx_v_pl) > 15) != 0);
      if (!__pyx_t_1) break;

      /* "tables/indexesextension.pyx":241
 *     while True:
 *         while pr - pl > SMALL_QUICKSORT:
 *             pm  = pl + ((pr - pl) >> 1)             # <<<<<<<<<<<<<<
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 * 
 */
      __pyx_v_pm = (__pyx_v_pl + ((__pyx_v_pr - __pyx_v_pl) >> 1));

      /* "tables/indexesextension.pyx":242
 *         while pr - pl > SMALL_QUICKSORT:
 *             pm  = pl + ((pr - pl) >> 1)
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2             # <<<<<<<<<<<<<<
 * 
 *             if less_than(pm, pl):
 */
      __pyx_v_ipm = (__pyx_v_ipl + ((((__pyx_v_ipr - __pyx_v_ipl) / __pyx_v_elsize2) >> 1) * __pyx_v_elsize2));

      /* "tables/indexesextension.pyx":244
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      __pyx_t_1 = (__pyx_fuse_5__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pm, __pyx_v_pl) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":245
 * 
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 */
        __pyx_t_2 = (__pyx_v_pl[0]);
        __pyx_t_3 = (__pyx_v_pm[0]);
        (__pyx_v_pm[0]) = __pyx_t_2;
        (__pyx_v_pl[0]) = __pyx_t_3;

        /* "tables/indexesextension.pyx":246
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             if less_than(pr, pm):
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipl, __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":244
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      }

      /* "tables/indexesextension.pyx":248
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 *             if less_than(pr, pm):             # <<<<<<<<<<<<<<
 *                 pr[0], pm[0] =  pm[0], pr[0]
 *                 swap_bytes(ipr, ipm, elsize2)
 */
      __pyx_t_1 = (__pyx_fuse_5__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pr, __pyx_v_pm) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":249
 * 
 *             if less_than(pr, pm):
 *                 pr[0], pm[0] =  pm[0], pr[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipr, ipm, elsize2)
 * 
 */
        __pyx_t_3 = (__pyx_v_pm[0]);
        __pyx_t_2 = (__pyx_v_pr[0]);
        (__pyx_v_pr[0]) = __pyx_t_3;
        (__pyx_v_pm[0]) = __pyx_t_2;

        /* "tables/indexesextension.pyx":250
 *             if less_than(pr, pm):
 *                 pr[0], pm[0] =  pm[0], pr[0]
 *                 swap_bytes(ipr, ipm, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             if less_than(pm, pl):
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipr, __pyx_v_ipm, __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":248
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 *             if less_than(pr, pm):             # <<<<<<<<<<<<<<
 *                 pr[0], pm[0] =  pm[0], pr[0]
 *                 swap_bytes(ipr, ipm, elsize2)
 */
      }

      /* "tables/indexesextension.pyx":252
 *                 swap_bytes(ipr, ipm, elsize2)
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      __pyx_t_1 = (__pyx_fuse_5__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pm, __pyx_v_pl) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":253
 * 
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 */
        __pyx_t_2 = (__pyx_v_pl[0]);
        __pyx_t_3 = (__pyx_v_pm[0]);
        (__pyx_v_pm[0]) = __pyx_t_2;
        (__pyx_v_pl[0]) = __pyx_t_3;

        /* "tables/indexesextension.pyx":254
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             vp = pm[0]
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipl, __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":252
 *                 swap_bytes(ipr, ipm, elsize2)
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      }

      /* "tables/indexesextension.pyx":256
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 *             vp = pm[0]             # <<<<<<<<<<<<<<
 * 
 *             pi = pl
 */
      __pyx_v_vp = (__pyx_v_pm[0]);

      /* "tables/indexesextension.pyx":258
 *             vp = pm[0]
 * 
 *             pi = pl             # <<<<<<<<<<<<<<
 *             ipi = ipl
 * 
 */
      __pyx_v_pi = __pyx_v_pl;

      /* "tables/indexesextension.pyx":259
 * 
 *             pi = pl
 *             ipi = ipl             # <<<<<<<<<<<<<<
 * 
 *             pj = pr - 1
 */
      __pyx_v_ipi = __pyx_v_ipl;

      /* "tables/indexesextension.pyx":261
 *             ipi = ipl
 * 
 *             pj = pr - 1             # <<<<<<<<<<<<<<
 *             ipj = ipr - elsize2
 * 
 */
      __pyx_v_pj = (__pyx_v_pr - 1);

      /* "tables/indexesextension.pyx":262
 * 
 *             pj = pr - 1
 *             ipj = ipr - elsize2             # <<<<<<<<<<<<<<
 * 
 *             pm[0], pj[0] = pj[0], pm[0]
 */
      __pyx_v_ipj = (__pyx_v_ipr - __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":264
 *             ipj = ipr - elsize2
 * 
 *             pm[0], pj[0] = pj[0], pm[0]             # <<<<<<<<<<<<<<
 *             swap_bytes(ipm, ipj, elsize2)
 * 
 */
      __pyx_t_3 = (__pyx_v_pj[0]);
      __pyx_t_2 = (__pyx_v_pm[0]);
      (__pyx_v_pm[0]) = __pyx_t_3;
      (__pyx_v_pj[0]) = __pyx_t_2;

      /* "tables/indexesextension.pyx":265
 * 
 *             pm[0], pj[0] = pj[0], pm[0]
 *             swap_bytes(ipm, ipj, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             while True:
 */
      __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipj, __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":267
 *             swap_bytes(ipm, ipj, elsize2)
 * 
 *             while True:             # <<<<<<<<<<<<<<
 *                 pi += 1
 *                 ipi += elsize2
 */
      while (1) {

        /* "tables/indexesextension.pyx":268
 * 
 *             while True:
 *                 pi += 1             # <<<<<<<<<<<<<<
 *                 ipi += elsize2
 *                 while less_than(pi, &vp):
 */
        __pyx_v_pi = (__pyx_v_pi + 1);

        /* "tables/indexesextension.pyx":269
 *             while True:
 *                 pi += 1
 *                 ipi += elsize2             # <<<<<<<<<<<<<<
 *                 while less_than(pi, &vp):
 *                     pi += 1
 */
        __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":270
 *                 pi += 1
 *                 ipi += elsize2
 *                 while less_than(pi, &vp):             # <<<<<<<<<<<<<<
 *                     pi += 1
 *                     ipi += elsize2
 */
        while (1) {
          __pyx_t_1 = (__pyx_fuse_5__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pi, (&__pyx_v_vp)) != 0);
          if (!__pyx_t_1) break;

          /* "tables/indexesextension.pyx":271
 *                 ipi += elsize2
 *                 while less_than(pi, &vp):
 *                     pi += 1             # <<<<<<<<<<<<<<
 *                     ipi += elsize2
 * 
 */
          __pyx_v_pi = (__pyx_v_pi + 1);

          /* "tables/indexesextension.pyx":272
 *                 while less_than(pi, &vp):
 *                     pi += 1
 *                     ipi += elsize2             # <<<<<<<<<<<<<<
 * 
 *                 pj -= 1
 */
          __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_elsize2);
        }

        /* "tables/indexesextension.pyx":274
 *                     ipi += elsize2
 * 
 *                 pj -= 1             # <<<<<<<<<<<<<<
 *                 ipj -= elsize2
 *                 while less_than(&vp, pj):
 */
        __pyx_v_pj = (__pyx_v_pj - 1);

        /* "tables/indexesextension.pyx":275
 * 
 *                 pj -= 1
 *                 ipj -= elsize2             # <<<<<<<<<<<<<<
 *                 while less_than(&vp, pj):
 *                     pj -= 1
 */
        __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":276
 *                 pj -= 1
 *                 ipj -= elsize2
 *                 while less_than(&vp, pj):             # <<<<<<<<<<<<<<
 *                     pj -= 1
 *                     ipj -= elsize2
 */
        while (1) {
          __pyx_t_1 = (__pyx_fuse_5__pyx_f_6tables_16indexesextension_less_than((&__pyx_v_vp), __pyx_v_pj) != 0);
          if (!__pyx_t_1) break;

          /* "tables/indexesextension.pyx":277
 *                 ipj -= elsize2
 *                 while less_than(&vp, pj):
 *                     pj -= 1             # <<<<<<<<<<<<<<
 *                     ipj -= elsize2
 * 
 */
          __pyx_v_pj = (__pyx_v_pj - 1);

          /* "tables/indexesextension.pyx":278
 *                 while less_than(&vp, pj):
 *                     pj -= 1
 *                     ipj -= elsize2             # <<<<<<<<<<<<<<
 * 
 *                 if pi >= pj:
 */
          __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_elsize2);
        }

        /* "tables/indexesextension.pyx":280
 *                     ipj -= elsize2
 * 
 *                 if pi >= pj:             # <<<<<<<<<<<<<<
 *                     break
 * 
 */
        __pyx_t_1 = ((__pyx_v_pi >= __pyx_v_pj) != 0);
        if (__pyx_t_1) {

          /* "tables/indexesextension.pyx":281
 * 
 *                 if pi >= pj:
 *                     break             # <<<<<<<<<<<<<<
 * 
 *                 pi[0], pj[0] = pj[0], pi[0]
 */
          goto __pyx_L11_break;

          /* "tables/indexesextension.pyx":280
 *                     ipj -= elsize2
 * 
 *                 if pi >= pj:             # <<<<<<<<<<<<<<
 *                     break
 * 
 */
        }

        /* "tables/indexesextension.pyx":283
 *                     break
 * 
 *                 pi[0], pj[0] = pj[0], pi[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipi, ipj, elsize2)
 * 
 */
        __pyx_t_2 = (__pyx_v_pj[0]);
        __pyx_t_3 = (__pyx_v_pi[0]);
        (__pyx_v_pi[0]) = __pyx_t_2;
        (__pyx_v_pj[0]) = __pyx_t_3;

        /* "tables/indexesextension.pyx":284
 * 
 *                 pi[0], pj[0] = pj[0], pi[0]
 *                 swap_bytes(ipi, ipj, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             pi[0], (pr-1)[0] = (pr-1)[0], pi[0]
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipi, __pyx_v_ipj, __pyx_v_elsize2);
      }
      __pyx_L11_break:;

      /* "tables/indexesextension.pyx":286
 *                 swap_bytes(ipi, ipj, elsize2)
 * 
 *             pi[0], (pr-1)[0] = (pr-1)[0], pi[0]             # <<<<<<<<<<<<<<
 *             swap_bytes(ipi, ipr-elsize2, elsize2)
 * 
 */
      __pyx_t_3 = ((__pyx_v_pr - 1)[0]);
      __pyx_t_2 = (__pyx_v_pi[0]);
      (__pyx_v_pi[0]) = __pyx_t_3;
      ((__pyx_v_pr - 1)[0]) = __pyx_t_2;

      /* "tables/indexesextension.pyx":287
 * 
 *             pi[0], (pr-1)[0] = (pr-1)[0], pi[0]
 *             swap_bytes(ipi, ipr-elsize2, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             # push largest partition on stack and proceed with the other
 */
      __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipi, (__pyx_v_ipr - __pyx_v_elsize2), __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":290
 * 
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):             # <<<<<<<<<<<<<<
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr
 */
      __pyx_t_1 = (((__pyx_v_pi - __pyx_v_pl) < (__pyx_v_pr - __pyx_v_pi)) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":291
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):
 *                 sptr[0] = pi + 1             # <<<<<<<<<<<<<<
 *                 sptr[1] = pr
 *                 sptr += 2
 */
        (__pyx_v_sptr[0]) = (__pyx_v_pi + 1);

        /* "tables/indexesextension.pyx":292
 *             if (pi - pl) < (pr - pi):
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr             # <<<<<<<<<<<<<<
 *                 sptr += 2
 * 
 */
        (__pyx_v_sptr[1]) = __pyx_v_pr;

        /* "tables/indexesextension.pyx":293
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr
 *                 sptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 isptr[0] = ipi + elsize2
 */
        __pyx_v_sptr = (__pyx_v_sptr + 2);

        /* "tables/indexesextension.pyx":295
 *                 sptr += 2
 * 
 *                 isptr[0] = ipi + elsize2             # <<<<<<<<<<<<<<
 *                 isptr[1] = ipr
 *                 isptr += 2
 */
        (__pyx_v_isptr[0]) = (__pyx_v_ipi + __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":296
 * 
 *                 isptr[0] = ipi + elsize2
 *                 isptr[1] = ipr             # <<<<<<<<<<<<<<
 *                 isptr += 2
 * 
 */
        (__pyx_v_isptr[1]) = __pyx_v_ipr;

        /* "tables/indexesextension.pyx":297
 *                 isptr[0] = ipi + elsize2
 *                 isptr[1] = ipr
 *                 isptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 pr = pi - 1
 */
        __pyx_v_isptr = (__pyx_v_isptr + 2);

        /* "tables/indexesextension.pyx":299
 *                 isptr += 2
 * 
 *                 pr = pi - 1             # <<<<<<<<<<<<<<
 *                 ipr = ipi - elsize2
 *             else:
 */
        __pyx_v_pr = (__pyx_v_pi - 1);

        /* "tables/indexesextension.pyx":300
 * 
 *                 pr = pi - 1
 *                 ipr = ipi - elsize2             # <<<<<<<<<<<<<<
 *             else:
 *                 sptr[0] = pl
 */
        __pyx_v_ipr = (__pyx_v_ipi - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":290
 * 
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):             # <<<<<<<<<<<<<<
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr
 */
        goto __pyx_L17;
      }

      /* "tables/indexesextension.pyx":302
 *                 ipr = ipi - elsize2
 *             else:
 *                 sptr[0] = pl             # <<<<<<<<<<<<<<
 *                 sptr[1] = pi - 1
 *                 sptr += 2
 */
      /*else*/ {
        (__pyx_v_sptr[0]) = __pyx_v_pl;

        /* "tables/indexesextension.pyx":303
 *             else:
 *                 sptr[0] = pl
 *                 sptr[1] = pi - 1             # <<<<<<<<<<<<<<
 *                 sptr += 2
 * 
 */
        (__pyx_v_sptr[1]) = (__pyx_v_pi - 1);

        /* "tables/indexesextension.pyx":304
 *                 sptr[0] = pl
 *                 sptr[1] = pi - 1
 *                 sptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 isptr[0] = ipl
 */
        __pyx_v_sptr = (__pyx_v_sptr + 2);

        /* "tables/indexesextension.pyx":306
 *                 sptr += 2
 * 
 *                 isptr[0] = ipl             # <<<<<<<<<<<<<<
 *                 isptr[1] = ipi - elsize2
 *                 isptr += 2
 */
        (__pyx_v_isptr[0]) = __pyx_v_ipl;

        /* "tables/indexesextension.pyx":307
 * 
 *                 isptr[0] = ipl
 *                 isptr[1] = ipi - elsize2             # <<<<<<<<<<<<<<
 *                 isptr += 2
 * 
 */
        (__pyx_v_isptr[1]) = (__pyx_v_ipi - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":308
 *                 isptr[0] = ipl
 *                 isptr[1] = ipi - elsize2
 *                 isptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 pl = pi + 1
 */
        __pyx_v_isptr = (__pyx_v_isptr + 2);

        /* "tables/indexesextension.pyx":310
 *                 isptr += 2
 * 
 *                 pl = pi + 1             # <<<<<<<<<<<<<<
 *                 ipl = ipi + elsize2
 * 
 */
        __pyx_v_pl = (__pyx_v_pi + 1);

        /* "tables/indexesextension.pyx":311
 * 
 *                 pl = pi + 1
 *                 ipl = ipi + elsize2             # <<<<<<<<<<<<<<
 * 
 *         pi = pl + 1
 */
        __pyx_v_ipl = (__pyx_v_ipi + __pyx_v_elsize2);
      }
      __pyx_L17:;
    }

    /* "tables/indexesextension.pyx":313
 *                 ipl = ipi + elsize2
 * 
 *         pi = pl + 1             # <<<<<<<<<<<<<<
 *         ipi = ipl + elsize2
 *         while pi <= pr:
 */
    __pyx_v_pi = (__pyx_v_pl + 1);

    /* "tables/indexesextension.pyx":314
 * 
 *         pi = pl + 1
 *         ipi = ipl + elsize2             # <<<<<<<<<<<<<<
 *         while pi <= pr:
 *             vp = pi[0]
 */
    __pyx_v_ipi = (__pyx_v_ipl + __pyx_v_elsize2);

    /* "tables/indexesextension.pyx":315
 *         pi = pl + 1
 *         ipi = ipl + elsize2
 *         while pi <= pr:             # <<<<<<<<<<<<<<
 *             vp = pi[0]
 *             memcpy(ivp, ipi, elsize2)
 */
    while (1) {
      __pyx_t_1 = ((__pyx_v_pi <= __pyx_v_pr) != 0);
      if (!__pyx_t_1) break;

      /* "tables/indexesextension.pyx":316
 *         ipi = ipl + elsize2
 *         while pi <= pr:
 *             vp = pi[0]             # <<<<<<<<<<<<<<
 *             memcpy(ivp, ipi, elsize2)
 * 
 */
      __pyx_v_vp = (__pyx_v_pi[0]);

      /* "tables/indexesextension.pyx":317
 *         while pi <= pr:
 *             vp = pi[0]
 *             memcpy(ivp, ipi, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             pj = pi
 */
      (void)(memcpy(__pyx_v_ivp, __pyx_v_ipi, __pyx_v_elsize2));

      /* "tables/indexesextension.pyx":319
 *             memcpy(ivp, ipi, elsize2)
 * 
 *             pj = pi             # <<<<<<<<<<<<<<
 *             pt = pi - 1
 * 
 */
      __pyx_v_pj = __pyx_v_pi;

      /* "tables/indexesextension.pyx":320
 * 
 *             pj = pi
 *             pt = pi - 1             # <<<<<<<<<<<<<<
 * 
 *             ipj = ipi
 */
      __pyx_v_pt = (__pyx_v_pi - 1);

      /* "tables/indexesextension.pyx":322
 *             pt = pi - 1
 * 
 *             ipj = ipi             # <<<<<<<<<<<<<<
 *             ipt = ipi - elsize2
 * 
 */
      __pyx_v_ipj = __pyx_v_ipi;

      /* "tables/indexesextension.pyx":323
 * 
 *             ipj = ipi
 *             ipt = ipi - elsize2             # <<<<<<<<<<<<<<
 * 
 *             while pj > pl and less_than(&vp, pt):
 */
      __pyx_v_ipt = (__pyx_v_ipi - __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":325
 *             ipt = ipi - elsize2
 * 
 *             while pj > pl and less_than(&vp, pt):             # <<<<<<<<<<<<<<
 *                 pj[0] = pt[0]
 *                 pj -= 1
 */
      while (1) {
        __pyx_t_4 = ((__pyx_v_pj > __pyx_v_pl) != 0);
        if (__pyx_t_4) {
        } else {
          __pyx_t_1 = __pyx_t_4;
          goto __pyx_L22_bool_binop_done;
        }
        __pyx_t_4 = (__pyx_fuse_5__pyx_f_6tables_16indexesextension_less_than((&__pyx_v_vp), __pyx_v_pt) != 0);
        __pyx_t_1 = __pyx_t_4;
        __pyx_L22_bool_binop_done:;
        if (!__pyx_t_1) break;

        /* "tables/indexesextension.pyx":326
 * 
 *             while pj > pl and less_than(&vp, pt):
 *                 pj[0] = pt[0]             # <<<<<<<<<<<<<<
 *                 pj -= 1
 *                 pt -= 1
 */
        (__pyx_v_pj[0]) = (__pyx_v_pt[0]);

        /* "tables/indexesextension.pyx":327
 *             while pj > pl and less_than(&vp, pt):
 *                 pj[0] = pt[0]
 *                 pj -= 1             # <<<<<<<<<<<<<<
 *                 pt -= 1
 * 
 */
        __pyx_v_pj = (__pyx_v_pj - 1);

        /* "tables/indexesextension.pyx":328
 *                 pj[0] = pt[0]
 *                 pj -= 1
 *                 pt -= 1             # <<<<<<<<<<<<<<
 * 
 *                 memcpy(ipj, ipt, elsize2)
 */
        __pyx_v_pt = (__pyx_v_pt - 1);

        /* "tables/indexesextension.pyx":330
 *                 pt -= 1
 * 
 *                 memcpy(ipj, ipt, elsize2)             # <<<<<<<<<<<<<<
 *                 ipj -= elsize2
 *                 ipt -= elsize2
 */
        (void)(memcpy(__pyx_v_ipj, __pyx_v_ipt, __pyx_v_elsize2));

        /* "tables/indexesextension.pyx":331
 * 
 *                 memcpy(ipj, ipt, elsize2)
 *                 ipj -= elsize2             # <<<<<<<<<<<<<<
 *                 ipt -= elsize2
 * 
 */
        __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":332
 *                 memcpy(ipj, ipt, elsize2)
 *                 ipj -= elsize2
 *                 ipt -= elsize2             # <<<<<<<<<<<<<<
 * 
 *             pj[0] = vp
 */
        __pyx_v_ipt = (__pyx_v_ipt - __pyx_v_elsize2);
      }

      /* "tables/indexesextension.pyx":334
 *                 ipt -= elsize2
 * 
 *             pj[0] = vp             # <<<<<<<<<<<<<<
 *             memcpy(ipj, ivp, elsize2)
 * 
 */
      (__pyx_v_pj[0]) = __pyx_v_vp;

      /* "tables/indexesextension.pyx":335
 * 
 *             pj[0] = vp
 *             memcpy(ipj, ivp, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             pi += 1
 */
      (void)(memcpy(__pyx_v_ipj, __pyx_v_ivp, __pyx_v_elsize2));

      /* "tables/indexesextension.pyx":337
 *             memcpy(ipj, ivp, elsize2)
 * 
 *             pi += 1             # <<<<<<<<<<<<<<
 *             ipi += elsize2
 * 
 */
      __pyx_v_pi = (__pyx_v_pi + 1);

      /* "tables/indexesextension.pyx":338
 * 
 *             pi += 1
 *             ipi += elsize2             # <<<<<<<<<<<<<<
 * 
 *         if sptr == stack:
 */
      __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_elsize2);
    }

    /* "tables/indexesextension.pyx":340
 *             ipi += elsize2
 * 
 *         if sptr == stack:             # <<<<<<<<<<<<<<
 *             break
 * 
 */
    __pyx_t_1 = ((__pyx_v_sptr == __pyx_v_stack) != 0);
    if (__pyx_t_1) {

      /* "tables/indexesextension.pyx":341
 * 
 *         if sptr == stack:
 *             break             # <<<<<<<<<<<<<<
 * 
 *         sptr -= 2
 */
      goto __pyx_L4_break;

      /* "tables/indexesextension.pyx":340
 *             ipi += elsize2
 * 
 *         if sptr == stack:             # <<<<<<<<<<<<<<
 *             break
 * 
 */
    }

    /* "tables/indexesextension.pyx":343
 *             break
 * 
 *         sptr -= 2             # <<<<<<<<<<<<<<
 *         pl = sptr[0]
 *         pr = sptr[1]
 */
    __pyx_v_sptr = (__pyx_v_sptr - 2);

    /* "tables/indexesextension.pyx":344
 * 
 *         sptr -= 2
 *         pl = sptr[0]             # <<<<<<<<<<<<<<
 *         pr = sptr[1]
 * 
 */
    __pyx_v_pl = (__pyx_v_sptr[0]);

    /* "tables/indexesextension.pyx":345
 *         sptr -= 2
 *         pl = sptr[0]
 *         pr = sptr[1]             # <<<<<<<<<<<<<<
 * 
 *         isptr -= 2
 */
    __pyx_v_pr = (__pyx_v_sptr[1]);

    /* "tables/indexesextension.pyx":347
 *         pr = sptr[1]
 * 
 *         isptr -= 2             # <<<<<<<<<<<<<<
 *         ipl = isptr[0]
 *         ipr = isptr[1]
 */
    __pyx_v_isptr = (__pyx_v_isptr - 2);

    /* "tables/indexesextension.pyx":348
 * 
 *         isptr -= 2
 *         ipl = isptr[0]             # <<<<<<<<<<<<<<
 *         ipr = isptr[1]
 * 
 */
    __pyx_v_ipl = (__pyx_v_isptr[0]);

    /* "tables/indexesextension.pyx":349
 *         isptr -= 2
 *         ipl = isptr[0]
 *         ipr = isptr[1]             # <<<<<<<<<<<<<<
 * 
 *     free(ivp)
 */
    __pyx_v_ipr = (__pyx_v_isptr[1]);
  }
  __pyx_L4_break:;

  /* "tables/indexesextension.pyx":351
 *         ipr = isptr[1]
 * 
 *     free(ivp)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  free(__pyx_v_ivp);

  /* "tables/indexesextension.pyx":212
 * 
 * @cython.cdivision(True)
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:             # <<<<<<<<<<<<<<
 *     cdef number_type *pl = start1
 *     cdef number_type *pr = start1 + (n - 1)
 */

  /* function exit code */
}

static void __pyx_fuse_6__pyx_f_6tables_16indexesextension__keysort(npy_uint32 *__pyx_v_start1, char *__pyx_v_start2, size_t __pyx_v_elsize2, size_t __pyx_v_n) {
  npy_uint32 *__pyx_v_pl;
  npy_uint32 *__pyx_v_pr;
  char *__pyx_v_ipl;
  char *__pyx_v_ipr;
  npy_uint32 __pyx_v_vp;
  char *__pyx_v_ivp;
  npy_uint32 *__pyx_v_stack[0x64];
  npy_uint32 **__pyx_v_sptr;
  char *__pyx_v_istack[0x64];
  char **__pyx_v_isptr;
  CYTHON_UNUSED size_t __pyx_v_stack_index;
  npy_uint32 *__pyx_v_pm;
  npy_uint32 *__pyx_v_pi;
  npy_uint32 *__pyx_v_pj;
  npy_uint32 *__pyx_v_pt;
  char *__pyx_v_ipm;
  char *__pyx_v_ipi;
  char *__pyx_v_ipj;
  char *__pyx_v_ipt;
  int __pyx_t_1;
  npy_uint32 __pyx_t_2;
  npy_uint32 __pyx_t_3;
  int __pyx_t_4;

  /* "tables/indexesextension.pyx":213
 * @cython.cdivision(True)
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:
 *     cdef number_type *pl = start1             # <<<<<<<<<<<<<<
 *     cdef number_type *pr = start1 + (n - 1)
 * 
 */
  __pyx_v_pl = __pyx_v_start1;

  /* "tables/indexesextension.pyx":214
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:
 *     cdef number_type *pl = start1
 *     cdef number_type *pr = start1 + (n - 1)             # <<<<<<<<<<<<<<
 * 
 *     cdef char *ipl = start2
 */
  __pyx_v_pr = (__pyx_v_start1 + (__pyx_v_n - 1));

  /* "tables/indexesextension.pyx":216
 *     cdef number_type *pr = start1 + (n - 1)
 * 
 *     cdef char *ipl = start2             # <<<<<<<<<<<<<<
 *     cdef char *ipr = start2 + (n - 1) * elsize2
 * 
 */
  __pyx_v_ipl = __pyx_v_start2;

  /* "tables/indexesextension.pyx":217
 * 
 *     cdef char *ipl = start2
 *     cdef char *ipr = start2 + (n - 1) * elsize2             # <<<<<<<<<<<<<<
 * 
 *     cdef number_type vp
 */
  __pyx_v_ipr = (__pyx_v_start2 + ((__pyx_v_n - 1) * __pyx_v_elsize2));

  /* "tables/indexesextension.pyx":220
 * 
 *     cdef number_type vp
 *     cdef char *ivp = <char *> malloc(elsize2)             # <<<<<<<<<<<<<<
 * 
 *     cdef number_type *stack[PYA_QS_STACK]
 */
  __pyx_v_ivp = ((char *)malloc(__pyx_v_elsize2));

  /* "tables/indexesextension.pyx":223
 * 
 *     cdef number_type *stack[PYA_QS_STACK]
 *     cdef number_type **sptr = stack             # <<<<<<<<<<<<<<
 * 
 *     cdef char *istack[PYA_QS_STACK]
 */
  __pyx_v_sptr = __pyx_v_stack;

  /* "tables/indexesextension.pyx":226
 * 
 *     cdef char *istack[PYA_QS_STACK]
 *     cdef char **isptr = istack             # <<<<<<<<<<<<<<
 * 
 *     cdef size_t stack_index = 0
 */
  __pyx_v_isptr = __pyx_v_istack;

  /* "tables/indexesextension.pyx":228
 *     cdef char **isptr = istack
 * 
 *     cdef size_t stack_index = 0             # <<<<<<<<<<<<<<
 * 
 *     cdef number_type *pm
 */
  __pyx_v_stack_index = 0;

  /* "tables/indexesextension.pyx":239
 *     cdef char *ipt
 * 
 *     while True:             # <<<<<<<<<<<<<<
 *         while pr - pl > SMALL_QUICKSORT:
 *             pm  = pl + ((pr - pl) >> 1)
 */
  while (1) {

    /* "tables/indexesextension.pyx":240
 * 
 *     while True:
 *         while pr - pl > SMALL_QUICKSORT:             # <<<<<<<<<<<<<<
 *             pm  = pl + ((pr - pl) >> 1)
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 */
    while (1) {
      __pyx_t_1 = (((__pyx_v_pr - __pyx_v_pl) > 15) != 0);
      if (!__pyx_t_1) break;

      /* "tables/indexesextension.pyx":241
 *     while True:
 *         while pr - pl > SMALL_QUICKSORT:
 *             pm  = pl + ((pr - pl) >> 1)             # <<<<<<<<<<<<<<
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 * 
 */
      __pyx_v_pm = (__pyx_v_pl + ((__pyx_v_pr - __pyx_v_pl) >> 1));

      /* "tables/indexesextension.pyx":242
 *         while pr - pl > SMALL_QUICKSORT:
 *             pm  = pl + ((pr - pl) >> 1)
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2             # <<<<<<<<<<<<<<
 * 
 *             if less_than(pm, pl):
 */
      __pyx_v_ipm = (__pyx_v_ipl + ((((__pyx_v_ipr - __pyx_v_ipl) / __pyx_v_elsize2) >> 1) * __pyx_v_elsize2));

      /* "tables/indexesextension.pyx":244
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      __pyx_t_1 = (__pyx_fuse_6__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pm, __pyx_v_pl) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":245
 * 
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 */
        __pyx_t_2 = (__pyx_v_pl[0]);
        __pyx_t_3 = (__pyx_v_pm[0]);
        (__pyx_v_pm[0]) = __pyx_t_2;
        (__pyx_v_pl[0]) = __pyx_t_3;

        /* "tables/indexesextension.pyx":246
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             if less_than(pr, pm):
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipl, __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":244
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      }

      /* "tables/indexesextension.pyx":248
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 *             if less_than(pr, pm):             # <<<<<<<<<<<<<<
 *                 pr[0], pm[0] =  pm[0], pr[0]
 *                 swap_bytes(ipr, ipm, elsize2)
 */
      __pyx_t_1 = (__pyx_fuse_6__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pr, __pyx_v_pm) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":249
 * 
 *             if less_than(pr, pm):
 *                 pr[0], pm[0] =  pm[0], pr[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipr, ipm, elsize2)
 * 
 */
        __pyx_t_3 = (__pyx_v_pm[0]);
        __pyx_t_2 = (__pyx_v_pr[0]);
        (__pyx_v_pr[0]) = __pyx_t_3;
        (__pyx_v_pm[0]) = __pyx_t_2;

        /* "tables/indexesextension.pyx":250
 *             if less_than(pr, pm):
 *                 pr[0], pm[0] =  pm[0], pr[0]
 *                 swap_bytes(ipr, ipm, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             if less_than(pm, pl):
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipr, __pyx_v_ipm, __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":248
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 *             if less_than(pr, pm):             # <<<<<<<<<<<<<<
 *                 pr[0], pm[0] =  pm[0], pr[0]
 *                 swap_bytes(ipr, ipm, elsize2)
 */
      }

      /* "tables/indexesextension.pyx":252
 *                 swap_bytes(ipr, ipm, elsize2)
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      __pyx_t_1 = (__pyx_fuse_6__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pm, __pyx_v_pl) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":253
 * 
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 */
        __pyx_t_2 = (__pyx_v_pl[0]);
        __pyx_t_3 = (__pyx_v_pm[0]);
        (__pyx_v_pm[0]) = __pyx_t_2;
        (__pyx_v_pl[0]) = __pyx_t_3;

        /* "tables/indexesextension.pyx":254
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             vp = pm[0]
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipl, __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":252
 *                 swap_bytes(ipr, ipm, elsize2)
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      }

      /* "tables/indexesextension.pyx":256
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 *             vp = pm[0]             # <<<<<<<<<<<<<<
 * 
 *             pi = pl
 */
      __pyx_v_vp = (__pyx_v_pm[0]);

      /* "tables/indexesextension.pyx":258
 *             vp = pm[0]
 * 
 *             pi = pl             # <<<<<<<<<<<<<<
 *             ipi = ipl
 * 
 */
      __pyx_v_pi = __pyx_v_pl;

      /* "tables/indexesextension.pyx":259
 * 
 *             pi = pl
 *             ipi = ipl             # <<<<<<<<<<<<<<
 * 
 *             pj = pr - 1
 */
      __pyx_v_ipi = __pyx_v_ipl;

      /* "tables/indexesextension.pyx":261
 *             ipi = ipl
 * 
 *             pj = pr - 1             # <<<<<<<<<<<<<<
 *             ipj = ipr - elsize2
 * 
 */
      __pyx_v_pj = (__pyx_v_pr - 1);

      /* "tables/indexesextension.pyx":262
 * 
 *             pj = pr - 1
 *             ipj = ipr - elsize2             # <<<<<<<<<<<<<<
 * 
 *             pm[0], pj[0] = pj[0], pm[0]
 */
      __pyx_v_ipj = (__pyx_v_ipr - __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":264
 *             ipj = ipr - elsize2
 * 
 *             pm[0], pj[0] = pj[0], pm[0]             # <<<<<<<<<<<<<<
 *             swap_bytes(ipm, ipj, elsize2)
 * 
 */
      __pyx_t_3 = (__pyx_v_pj[0]);
      __pyx_t_2 = (__pyx_v_pm[0]);
      (__pyx_v_pm[0]) = __pyx_t_3;
      (__pyx_v_pj[0]) = __pyx_t_2;

      /* "tables/indexesextension.pyx":265
 * 
 *             pm[0], pj[0] = pj[0], pm[0]
 *             swap_bytes(ipm, ipj, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             while True:
 */
      __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipj, __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":267
 *             swap_bytes(ipm, ipj, elsize2)
 * 
 *             while True:             # <<<<<<<<<<<<<<
 *                 pi += 1
 *                 ipi += elsize2
 */
      while (1) {

        /* "tables/indexesextension.pyx":268
 * 
 *             while True:
 *                 pi += 1             # <<<<<<<<<<<<<<
 *                 ipi += elsize2
 *                 while less_than(pi, &vp):
 */
        __pyx_v_pi = (__pyx_v_pi + 1);

        /* "tables/indexesextension.pyx":269
 *             while True:
 *                 pi += 1
 *                 ipi += elsize2             # <<<<<<<<<<<<<<
 *                 while less_than(pi, &vp):
 *                     pi += 1
 */
        __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":270
 *                 pi += 1
 *                 ipi += elsize2
 *                 while less_than(pi, &vp):             # <<<<<<<<<<<<<<
 *                     pi += 1
 *                     ipi += elsize2
 */
        while (1) {
          __pyx_t_1 = (__pyx_fuse_6__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pi, (&__pyx_v_vp)) != 0);
          if (!__pyx_t_1) break;

          /* "tables/indexesextension.pyx":271
 *                 ipi += elsize2
 *                 while less_than(pi, &vp):
 *                     pi += 1             # <<<<<<<<<<<<<<
 *                     ipi += elsize2
 * 
 */
          __pyx_v_pi = (__pyx_v_pi + 1);

          /* "tables/indexesextension.pyx":272
 *                 while less_than(pi, &vp):
 *                     pi += 1
 *                     ipi += elsize2             # <<<<<<<<<<<<<<
 * 
 *                 pj -= 1
 */
          __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_elsize2);
        }

        /* "tables/indexesextension.pyx":274
 *                     ipi += elsize2
 * 
 *                 pj -= 1             # <<<<<<<<<<<<<<
 *                 ipj -= elsize2
 *                 while less_than(&vp, pj):
 */
        __pyx_v_pj = (__pyx_v_pj - 1);

        /* "tables/indexesextension.pyx":275
 * 
 *                 pj -= 1
 *                 ipj -= elsize2             # <<<<<<<<<<<<<<
 *                 while less_than(&vp, pj):
 *                     pj -= 1
 */
        __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":276
 *                 pj -= 1
 *                 ipj -= elsize2
 *                 while less_than(&vp, pj):             # <<<<<<<<<<<<<<
 *                     pj -= 1
 *                     ipj -= elsize2
 */
        while (1) {
          __pyx_t_1 = (__pyx_fuse_6__pyx_f_6tables_16indexesextension_less_than((&__pyx_v_vp), __pyx_v_pj) != 0);
          if (!__pyx_t_1) break;

          /* "tables/indexesextension.pyx":277
 *                 ipj -= elsize2
 *                 while less_than(&vp, pj):
 *                     pj -= 1             # <<<<<<<<<<<<<<
 *                     ipj -= elsize2
 * 
 */
          __pyx_v_pj = (__pyx_v_pj - 1);

          /* "tables/indexesextension.pyx":278
 *                 while less_than(&vp, pj):
 *                     pj -= 1
 *                     ipj -= elsize2             # <<<<<<<<<<<<<<
 * 
 *                 if pi >= pj:
 */
          __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_elsize2);
        }

        /* "tables/indexesextension.pyx":280
 *                     ipj -= elsize2
 * 
 *                 if pi >= pj:             # <<<<<<<<<<<<<<
 *                     break
 * 
 */
        __pyx_t_1 = ((__pyx_v_pi >= __pyx_v_pj) != 0);
        if (__pyx_t_1) {

          /* "tables/indexesextension.pyx":281
 * 
 *                 if pi >= pj:
 *                     break             # <<<<<<<<<<<<<<
 * 
 *                 pi[0], pj[0] = pj[0], pi[0]
 */
          goto __pyx_L11_break;

          /* "tables/indexesextension.pyx":280
 *                     ipj -= elsize2
 * 
 *                 if pi >= pj:             # <<<<<<<<<<<<<<
 *                     break
 * 
 */
        }

        /* "tables/indexesextension.pyx":283
 *                     break
 * 
 *                 pi[0], pj[0] = pj[0], pi[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipi, ipj, elsize2)
 * 
 */
        __pyx_t_2 = (__pyx_v_pj[0]);
        __pyx_t_3 = (__pyx_v_pi[0]);
        (__pyx_v_pi[0]) = __pyx_t_2;
        (__pyx_v_pj[0]) = __pyx_t_3;

        /* "tables/indexesextension.pyx":284
 * 
 *                 pi[0], pj[0] = pj[0], pi[0]
 *                 swap_bytes(ipi, ipj, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             pi[0], (pr-1)[0] = (pr-1)[0], pi[0]
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipi, __pyx_v_ipj, __pyx_v_elsize2);
      }
      __pyx_L11_break:;

      /* "tables/indexesextension.pyx":286
 *                 swap_bytes(ipi, ipj, elsize2)
 * 
 *             pi[0], (pr-1)[0] = (pr-1)[0], pi[0]             # <<<<<<<<<<<<<<
 *             swap_bytes(ipi, ipr-elsize2, elsize2)
 * 
 */
      __pyx_t_3 = ((__pyx_v_pr - 1)[0]);
      __pyx_t_2 = (__pyx_v_pi[0]);
      (__pyx_v_pi[0]) = __pyx_t_3;
      ((__pyx_v_pr - 1)[0]) = __pyx_t_2;

      /* "tables/indexesextension.pyx":287
 * 
 *             pi[0], (pr-1)[0] = (pr-1)[0], pi[0]
 *             swap_bytes(ipi, ipr-elsize2, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             # push largest partition on stack and proceed with the other
 */
      __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipi, (__pyx_v_ipr - __pyx_v_elsize2), __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":290
 * 
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):             # <<<<<<<<<<<<<<
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr
 */
      __pyx_t_1 = (((__pyx_v_pi - __pyx_v_pl) < (__pyx_v_pr - __pyx_v_pi)) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":291
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):
 *                 sptr[0] = pi + 1             # <<<<<<<<<<<<<<
 *                 sptr[1] = pr
 *                 sptr += 2
 */
        (__pyx_v_sptr[0]) = (__pyx_v_pi + 1);

        /* "tables/indexesextension.pyx":292
 *             if (pi - pl) < (pr - pi):
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr             # <<<<<<<<<<<<<<
 *                 sptr += 2
 * 
 */
        (__pyx_v_sptr[1]) = __pyx_v_pr;

        /* "tables/indexesextension.pyx":293
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr
 *                 sptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 isptr[0] = ipi + elsize2
 */
        __pyx_v_sptr = (__pyx_v_sptr + 2);

        /* "tables/indexesextension.pyx":295
 *                 sptr += 2
 * 
 *                 isptr[0] = ipi + elsize2             # <<<<<<<<<<<<<<
 *                 isptr[1] = ipr
 *                 isptr += 2
 */
        (__pyx_v_isptr[0]) = (__pyx_v_ipi + __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":296
 * 
 *                 isptr[0] = ipi + elsize2
 *                 isptr[1] = ipr             # <<<<<<<<<<<<<<
 *                 isptr += 2
 * 
 */
        (__pyx_v_isptr[1]) = __pyx_v_ipr;

        /* "tables/indexesextension.pyx":297
 *                 isptr[0] = ipi + elsize2
 *                 isptr[1] = ipr
 *                 isptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 pr = pi - 1
 */
        __pyx_v_isptr = (__pyx_v_isptr + 2);

        /* "tables/indexesextension.pyx":299
 *                 isptr += 2
 * 
 *                 pr = pi - 1             # <<<<<<<<<<<<<<
 *                 ipr = ipi - elsize2
 *             else:
 */
        __pyx_v_pr = (__pyx_v_pi - 1);

        /* "tables/indexesextension.pyx":300
 * 
 *                 pr = pi - 1
 *                 ipr = ipi - elsize2             # <<<<<<<<<<<<<<
 *             else:
 *                 sptr[0] = pl
 */
        __pyx_v_ipr = (__pyx_v_ipi - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":290
 * 
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):             # <<<<<<<<<<<<<<
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr
 */
        goto __pyx_L17;
      }

      /* "tables/indexesextension.pyx":302
 *                 ipr = ipi - elsize2
 *             else:
 *                 sptr[0] = pl             # <<<<<<<<<<<<<<
 *                 sptr[1] = pi - 1
 *                 sptr += 2
 */
      /*else*/ {
        (__pyx_v_sptr[0]) = __pyx_v_pl;

        /* "tables/indexesextension.pyx":303
 *             else:
 *                 sptr[0] = pl
 *                 sptr[1] = pi - 1             # <<<<<<<<<<<<<<
 *                 sptr += 2
 * 
 */
        (__pyx_v_sptr[1]) = (__pyx_v_pi - 1);

        /* "tables/indexesextension.pyx":304
 *                 sptr[0] = pl
 *                 sptr[1] = pi - 1
 *                 sptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 isptr[0] = ipl
 */
        __pyx_v_sptr = (__pyx_v_sptr + 2);

        /* "tables/indexesextension.pyx":306
 *                 sptr += 2
 * 
 *                 isptr[0] = ipl             # <<<<<<<<<<<<<<
 *                 isptr[1] = ipi - elsize2
 *                 isptr += 2
 */
        (__pyx_v_isptr[0]) = __pyx_v_ipl;

        /* "tables/indexesextension.pyx":307
 * 
 *                 isptr[0] = ipl
 *                 isptr[1] = ipi - elsize2             # <<<<<<<<<<<<<<
 *                 isptr += 2
 * 
 */
        (__pyx_v_isptr[1]) = (__pyx_v_ipi - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":308
 *                 isptr[0] = ipl
 *                 isptr[1] = ipi - elsize2
 *                 isptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 pl = pi + 1
 */
        __pyx_v_isptr = (__pyx_v_isptr + 2);

        /* "tables/indexesextension.pyx":310
 *                 isptr += 2
 * 
 *                 pl = pi + 1             # <<<<<<<<<<<<<<
 *                 ipl = ipi + elsize2
 * 
 */
        __pyx_v_pl = (__pyx_v_pi + 1);

        /* "tables/indexesextension.pyx":311
 * 
 *                 pl = pi + 1
 *                 ipl = ipi + elsize2             # <<<<<<<<<<<<<<
 * 
 *         pi = pl + 1
 */
        __pyx_v_ipl = (__pyx_v_ipi + __pyx_v_elsize2);
      }
      __pyx_L17:;
    }

    /* "tables/indexesextension.pyx":313
 *                 ipl = ipi + elsize2
 * 
 *         pi = pl + 1             # <<<<<<<<<<<<<<
 *         ipi = ipl + elsize2
 *         while pi <= pr:
 */
    __pyx_v_pi = (__pyx_v_pl + 1);

    /* "tables/indexesextension.pyx":314
 * 
 *         pi = pl + 1
 *         ipi = ipl + elsize2             # <<<<<<<<<<<<<<
 *         while pi <= pr:
 *             vp = pi[0]
 */
    __pyx_v_ipi = (__pyx_v_ipl + __pyx_v_elsize2);

    /* "tables/indexesextension.pyx":315
 *         pi = pl + 1
 *         ipi = ipl + elsize2
 *         while pi <= pr:             # <<<<<<<<<<<<<<
 *             vp = pi[0]
 *             memcpy(ivp, ipi, elsize2)
 */
    while (1) {
      __pyx_t_1 = ((__pyx_v_pi <= __pyx_v_pr) != 0);
      if (!__pyx_t_1) break;

      /* "tables/indexesextension.pyx":316
 *         ipi = ipl + elsize2
 *         while pi <= pr:
 *             vp = pi[0]             # <<<<<<<<<<<<<<
 *             memcpy(ivp, ipi, elsize2)
 * 
 */
      __pyx_v_vp = (__pyx_v_pi[0]);

      /* "tables/indexesextension.pyx":317
 *         while pi <= pr:
 *             vp = pi[0]
 *             memcpy(ivp, ipi, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             pj = pi
 */
      (void)(memcpy(__pyx_v_ivp, __pyx_v_ipi, __pyx_v_elsize2));

      /* "tables/indexesextension.pyx":319
 *             memcpy(ivp, ipi, elsize2)
 * 
 *             pj = pi             # <<<<<<<<<<<<<<
 *             pt = pi - 1
 * 
 */
      __pyx_v_pj = __pyx_v_pi;

      /* "tables/indexesextension.pyx":320
 * 
 *             pj = pi
 *             pt = pi - 1             # <<<<<<<<<<<<<<
 * 
 *             ipj = ipi
 */
      __pyx_v_pt = (__pyx_v_pi - 1);

      /* "tables/indexesextension.pyx":322
 *             pt = pi - 1
 * 
 *             ipj = ipi             # <<<<<<<<<<<<<<
 *             ipt = ipi - elsize2
 * 
 */
      __pyx_v_ipj = __pyx_v_ipi;

      /* "tables/indexesextension.pyx":323
 * 
 *             ipj = ipi
 *             ipt = ipi - elsize2             # <<<<<<<<<<<<<<
 * 
 *             while pj > pl and less_than(&vp, pt):
 */
      __pyx_v_ipt = (__pyx_v_ipi - __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":325
 *             ipt = ipi - elsize2
 * 
 *             while pj > pl and less_than(&vp, pt):             # <<<<<<<<<<<<<<
 *                 pj[0] = pt[0]
 *                 pj -= 1
 */
      while (1) {
        __pyx_t_4 = ((__pyx_v_pj > __pyx_v_pl) != 0);
        if (__pyx_t_4) {
        } else {
          __pyx_t_1 = __pyx_t_4;
          goto __pyx_L22_bool_binop_done;
        }
        __pyx_t_4 = (__pyx_fuse_6__pyx_f_6tables_16indexesextension_less_than((&__pyx_v_vp), __pyx_v_pt) != 0);
        __pyx_t_1 = __pyx_t_4;
        __pyx_L22_bool_binop_done:;
        if (!__pyx_t_1) break;

        /* "tables/indexesextension.pyx":326
 * 
 *             while pj > pl and less_than(&vp, pt):
 *                 pj[0] = pt[0]             # <<<<<<<<<<<<<<
 *                 pj -= 1
 *                 pt -= 1
 */
        (__pyx_v_pj[0]) = (__pyx_v_pt[0]);

        /* "tables/indexesextension.pyx":327
 *             while pj > pl and less_than(&vp, pt):
 *                 pj[0] = pt[0]
 *                 pj -= 1             # <<<<<<<<<<<<<<
 *                 pt -= 1
 * 
 */
        __pyx_v_pj = (__pyx_v_pj - 1);

        /* "tables/indexesextension.pyx":328
 *                 pj[0] = pt[0]
 *                 pj -= 1
 *                 pt -= 1             # <<<<<<<<<<<<<<
 * 
 *                 memcpy(ipj, ipt, elsize2)
 */
        __pyx_v_pt = (__pyx_v_pt - 1);

        /* "tables/indexesextension.pyx":330
 *                 pt -= 1
 * 
 *                 memcpy(ipj, ipt, elsize2)             # <<<<<<<<<<<<<<
 *                 ipj -= elsize2
 *                 ipt -= elsize2
 */
        (void)(memcpy(__pyx_v_ipj, __pyx_v_ipt, __pyx_v_elsize2));

        /* "tables/indexesextension.pyx":331
 * 
 *                 memcpy(ipj, ipt, elsize2)
 *                 ipj -= elsize2             # <<<<<<<<<<<<<<
 *                 ipt -= elsize2
 * 
 */
        __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":332
 *                 memcpy(ipj, ipt, elsize2)
 *                 ipj -= elsize2
 *                 ipt -= elsize2             # <<<<<<<<<<<<<<
 * 
 *             pj[0] = vp
 */
        __pyx_v_ipt = (__pyx_v_ipt - __pyx_v_elsize2);
      }

      /* "tables/indexesextension.pyx":334
 *                 ipt -= elsize2
 * 
 *             pj[0] = vp             # <<<<<<<<<<<<<<
 *             memcpy(ipj, ivp, elsize2)
 * 
 */
      (__pyx_v_pj[0]) = __pyx_v_vp;

      /* "tables/indexesextension.pyx":335
 * 
 *             pj[0] = vp
 *             memcpy(ipj, ivp, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             pi += 1
 */
      (void)(memcpy(__pyx_v_ipj, __pyx_v_ivp, __pyx_v_elsize2));

      /* "tables/indexesextension.pyx":337
 *             memcpy(ipj, ivp, elsize2)
 * 
 *             pi += 1             # <<<<<<<<<<<<<<
 *             ipi += elsize2
 * 
 */
      __pyx_v_pi = (__pyx_v_pi + 1);

      /* "tables/indexesextension.pyx":338
 * 
 *             pi += 1
 *             ipi += elsize2             # <<<<<<<<<<<<<<
 * 
 *         if sptr == stack:
 */
      __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_elsize2);
    }

    /* "tables/indexesextension.pyx":340
 *             ipi += elsize2
 * 
 *         if sptr == stack:             # <<<<<<<<<<<<<<
 *             break
 * 
 */
    __pyx_t_1 = ((__pyx_v_sptr == __pyx_v_stack) != 0);
    if (__pyx_t_1) {

      /* "tables/indexesextension.pyx":341
 * 
 *         if sptr == stack:
 *             break             # <<<<<<<<<<<<<<
 * 
 *         sptr -= 2
 */
      goto __pyx_L4_break;

      /* "tables/indexesextension.pyx":340
 *             ipi += elsize2
 * 
 *         if sptr == stack:             # <<<<<<<<<<<<<<
 *             break
 * 
 */
    }

    /* "tables/indexesextension.pyx":343
 *             break
 * 
 *         sptr -= 2             # <<<<<<<<<<<<<<
 *         pl = sptr[0]
 *         pr = sptr[1]
 */
    __pyx_v_sptr = (__pyx_v_sptr - 2);

    /* "tables/indexesextension.pyx":344
 * 
 *         sptr -= 2
 *         pl = sptr[0]             # <<<<<<<<<<<<<<
 *         pr = sptr[1]
 * 
 */
    __pyx_v_pl = (__pyx_v_sptr[0]);

    /* "tables/indexesextension.pyx":345
 *         sptr -= 2
 *         pl = sptr[0]
 *         pr = sptr[1]             # <<<<<<<<<<<<<<
 * 
 *         isptr -= 2
 */
    __pyx_v_pr = (__pyx_v_sptr[1]);

    /* "tables/indexesextension.pyx":347
 *         pr = sptr[1]
 * 
 *         isptr -= 2             # <<<<<<<<<<<<<<
 *         ipl = isptr[0]
 *         ipr = isptr[1]
 */
    __pyx_v_isptr = (__pyx_v_isptr - 2);

    /* "tables/indexesextension.pyx":348
 * 
 *         isptr -= 2
 *         ipl = isptr[0]             # <<<<<<<<<<<<<<
 *         ipr = isptr[1]
 * 
 */
    __pyx_v_ipl = (__pyx_v_isptr[0]);

    /* "tables/indexesextension.pyx":349
 *         isptr -= 2
 *         ipl = isptr[0]
 *         ipr = isptr[1]             # <<<<<<<<<<<<<<
 * 
 *     free(ivp)
 */
    __pyx_v_ipr = (__pyx_v_isptr[1]);
  }
  __pyx_L4_break:;

  /* "tables/indexesextension.pyx":351
 *         ipr = isptr[1]
 * 
 *     free(ivp)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  free(__pyx_v_ivp);

  /* "tables/indexesextension.pyx":212
 * 
 * @cython.cdivision(True)
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:             # <<<<<<<<<<<<<<
 *     cdef number_type *pl = start1
 *     cdef number_type *pr = start1 + (n - 1)
 */

  /* function exit code */
}

static void __pyx_fuse_7__pyx_f_6tables_16indexesextension__keysort(npy_uint64 *__pyx_v_start1, char *__pyx_v_start2, size_t __pyx_v_elsize2, size_t __pyx_v_n) {
  npy_uint64 *__pyx_v_pl;
  npy_uint64 *__pyx_v_pr;
  char *__pyx_v_ipl;
  char *__pyx_v_ipr;
  npy_uint64 __pyx_v_vp;
  char *__pyx_v_ivp;
  npy_uint64 *__pyx_v_stack[0x64];
  npy_uint64 **__pyx_v_sptr;
  char *__pyx_v_istack[0x64];
  char **__pyx_v_isptr;
  CYTHON_UNUSED size_t __pyx_v_stack_index;
  npy_uint64 *__pyx_v_pm;
  npy_uint64 *__pyx_v_pi;
  npy_uint64 *__pyx_v_pj;
  npy_uint64 *__pyx_v_pt;
  char *__pyx_v_ipm;
  char *__pyx_v_ipi;
  char *__pyx_v_ipj;
  char *__pyx_v_ipt;
  int __pyx_t_1;
  npy_uint64 __pyx_t_2;
  npy_uint64 __pyx_t_3;
  int __pyx_t_4;

  /* "tables/indexesextension.pyx":213
 * @cython.cdivision(True)
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:
 *     cdef number_type *pl = start1             # <<<<<<<<<<<<<<
 *     cdef number_type *pr = start1 + (n - 1)
 * 
 */
  __pyx_v_pl = __pyx_v_start1;

  /* "tables/indexesextension.pyx":214
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:
 *     cdef number_type *pl = start1
 *     cdef number_type *pr = start1 + (n - 1)             # <<<<<<<<<<<<<<
 * 
 *     cdef char *ipl = start2
 */
  __pyx_v_pr = (__pyx_v_start1 + (__pyx_v_n - 1));

  /* "tables/indexesextension.pyx":216
 *     cdef number_type *pr = start1 + (n - 1)
 * 
 *     cdef char *ipl = start2             # <<<<<<<<<<<<<<
 *     cdef char *ipr = start2 + (n - 1) * elsize2
 * 
 */
  __pyx_v_ipl = __pyx_v_start2;

  /* "tables/indexesextension.pyx":217
 * 
 *     cdef char *ipl = start2
 *     cdef char *ipr = start2 + (n - 1) * elsize2             # <<<<<<<<<<<<<<
 * 
 *     cdef number_type vp
 */
  __pyx_v_ipr = (__pyx_v_start2 + ((__pyx_v_n - 1) * __pyx_v_elsize2));

  /* "tables/indexesextension.pyx":220
 * 
 *     cdef number_type vp
 *     cdef char *ivp = <char *> malloc(elsize2)             # <<<<<<<<<<<<<<
 * 
 *     cdef number_type *stack[PYA_QS_STACK]
 */
  __pyx_v_ivp = ((char *)malloc(__pyx_v_elsize2));

  /* "tables/indexesextension.pyx":223
 * 
 *     cdef number_type *stack[PYA_QS_STACK]
 *     cdef number_type **sptr = stack             # <<<<<<<<<<<<<<
 * 
 *     cdef char *istack[PYA_QS_STACK]
 */
  __pyx_v_sptr = __pyx_v_stack;

  /* "tables/indexesextension.pyx":226
 * 
 *     cdef char *istack[PYA_QS_STACK]
 *     cdef char **isptr = istack             # <<<<<<<<<<<<<<
 * 
 *     cdef size_t stack_index = 0
 */
  __pyx_v_isptr = __pyx_v_istack;

  /* "tables/indexesextension.pyx":228
 *     cdef char **isptr = istack
 * 
 *     cdef size_t stack_index = 0             # <<<<<<<<<<<<<<
 * 
 *     cdef number_type *pm
 */
  __pyx_v_stack_index = 0;

  /* "tables/indexesextension.pyx":239
 *     cdef char *ipt
 * 
 *     while True:             # <<<<<<<<<<<<<<
 *         while pr - pl > SMALL_QUICKSORT:
 *             pm  = pl + ((pr - pl) >> 1)
 */
  while (1) {

    /* "tables/indexesextension.pyx":240
 * 
 *     while True:
 *         while pr - pl > SMALL_QUICKSORT:             # <<<<<<<<<<<<<<
 *             pm  = pl + ((pr - pl) >> 1)
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 */
    while (1) {
      __pyx_t_1 = (((__pyx_v_pr - __pyx_v_pl) > 15) != 0);
      if (!__pyx_t_1) break;

      /* "tables/indexesextension.pyx":241
 *     while True:
 *         while pr - pl > SMALL_QUICKSORT:
 *             pm  = pl + ((pr - pl) >> 1)             # <<<<<<<<<<<<<<
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 * 
 */
      __pyx_v_pm = (__pyx_v_pl + ((__pyx_v_pr - __pyx_v_pl) >> 1));

      /* "tables/indexesextension.pyx":242
 *         while pr - pl > SMALL_QUICKSORT:
 *             pm  = pl + ((pr - pl) >> 1)
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2             # <<<<<<<<<<<<<<
 * 
 *             if less_than(pm, pl):
 */
      __pyx_v_ipm = (__pyx_v_ipl + ((((__pyx_v_ipr - __pyx_v_ipl) / __pyx_v_elsize2) >> 1) * __pyx_v_elsize2));

      /* "tables/indexesextension.pyx":244
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      __pyx_t_1 = (__pyx_fuse_7__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pm, __pyx_v_pl) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":245
 * 
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 */
        __pyx_t_2 = (__pyx_v_pl[0]);
        __pyx_t_3 = (__pyx_v_pm[0]);
        (__pyx_v_pm[0]) = __pyx_t_2;
        (__pyx_v_pl[0]) = __pyx_t_3;

        /* "tables/indexesextension.pyx":246
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             if less_than(pr, pm):
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipl, __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":244
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      }

      /* "tables/indexesextension.pyx":248
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 *             if less_than(pr, pm):             # <<<<<<<<<<<<<<
 *                 pr[0], pm[0] =  pm[0], pr[0]
 *                 swap_bytes(ipr, ipm, elsize2)
 */
      __pyx_t_1 = (__pyx_fuse_7__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pr, __pyx_v_pm) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":249
 * 
 *             if less_than(pr, pm):
 *                 pr[0], pm[0] =  pm[0], pr[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipr, ipm, elsize2)
 * 
 */
        __pyx_t_3 = (__pyx_v_pm[0]);
        __pyx_t_2 = (__pyx_v_pr[0]);
        (__pyx_v_pr[0]) = __pyx_t_3;
        (__pyx_v_pm[0]) = __pyx_t_2;

        /* "tables/indexesextension.pyx":250
 *             if less_than(pr, pm):
 *                 pr[0], pm[0] =  pm[0], pr[0]
 *                 swap_bytes(ipr, ipm, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             if less_than(pm, pl):
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipr, __pyx_v_ipm, __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":248
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 *             if less_than(pr, pm):             # <<<<<<<<<<<<<<
 *                 pr[0], pm[0] =  pm[0], pr[0]
 *                 swap_bytes(ipr, ipm, elsize2)
 */
      }

      /* "tables/indexesextension.pyx":252
 *                 swap_bytes(ipr, ipm, elsize2)
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      __pyx_t_1 = (__pyx_fuse_7__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pm, __pyx_v_pl) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":253
 * 
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 */
        __pyx_t_2 = (__pyx_v_pl[0]);
        __pyx_t_3 = (__pyx_v_pm[0]);
        (__pyx_v_pm[0]) = __pyx_t_2;
        (__pyx_v_pl[0]) = __pyx_t_3;

        /* "tables/indexesextension.pyx":254
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             vp = pm[0]
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipl, __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":252
 *                 swap_bytes(ipr, ipm, elsize2)
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      }

      /* "tables/indexesextension.pyx":256
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 *             vp = pm[0]             # <<<<<<<<<<<<<<
 * 
 *             pi = pl
 */
      __pyx_v_vp = (__pyx_v_pm[0]);

      /* "tables/indexesextension.pyx":258
 *             vp = pm[0]
 * 
 *             pi = pl             # <<<<<<<<<<<<<<
 *             ipi = ipl
 * 
 */
      __pyx_v_pi = __pyx_v_pl;

      /* "tables/indexesextension.pyx":259
 * 
 *             pi = pl
 *             ipi = ipl             # <<<<<<<<<<<<<<
 * 
 *             pj = pr - 1
 */
      __pyx_v_ipi = __pyx_v_ipl;

      /* "tables/indexesextension.pyx":261
 *             ipi = ipl
 * 
 *             pj = pr - 1             # <<<<<<<<<<<<<<
 *             ipj = ipr - elsize2
 * 
 */
      __pyx_v_pj = (__pyx_v_pr - 1);

      /* "tables/indexesextension.pyx":262
 * 
 *             pj = pr - 1
 *             ipj = ipr - elsize2             # <<<<<<<<<<<<<<
 * 
 *             pm[0], pj[0] = pj[0], pm[0]
 */
      __pyx_v_ipj = (__pyx_v_ipr - __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":264
 *             ipj = ipr - elsize2
 * 
 *             pm[0], pj[0] = pj[0], pm[0]             # <<<<<<<<<<<<<<
 *             swap_bytes(ipm, ipj, elsize2)
 * 
 */
      __pyx_t_3 = (__pyx_v_pj[0]);
      __pyx_t_2 = (__pyx_v_pm[0]);
      (__pyx_v_pm[0]) = __pyx_t_3;
      (__pyx_v_pj[0]) = __pyx_t_2;

      /* "tables/indexesextension.pyx":265
 * 
 *             pm[0], pj[0] = pj[0], pm[0]
 *             swap_bytes(ipm, ipj, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             while True:
 */
      __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipj, __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":267
 *             swap_bytes(ipm, ipj, elsize2)
 * 
 *             while True:             # <<<<<<<<<<<<<<
 *                 pi += 1
 *                 ipi += elsize2
 */
      while (1) {

        /* "tables/indexesextension.pyx":268
 * 
 *             while True:
 *                 pi += 1             # <<<<<<<<<<<<<<
 *                 ipi += elsize2
 *                 while less_than(pi, &vp):
 */
        __pyx_v_pi = (__pyx_v_pi + 1);

        /* "tables/indexesextension.pyx":269
 *             while True:
 *                 pi += 1
 *                 ipi += elsize2             # <<<<<<<<<<<<<<
 *                 while less_than(pi, &vp):
 *                     pi += 1
 */
        __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":270
 *                 pi += 1
 *                 ipi += elsize2
 *                 while less_than(pi, &vp):             # <<<<<<<<<<<<<<
 *                     pi += 1
 *                     ipi += elsize2
 */
        while (1) {
          __pyx_t_1 = (__pyx_fuse_7__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pi, (&__pyx_v_vp)) != 0);
          if (!__pyx_t_1) break;

          /* "tables/indexesextension.pyx":271
 *                 ipi += elsize2
 *                 while less_than(pi, &vp):
 *                     pi += 1             # <<<<<<<<<<<<<<
 *                     ipi += elsize2
 * 
 */
          __pyx_v_pi = (__pyx_v_pi + 1);

          /* "tables/indexesextension.pyx":272
 *                 while less_than(pi, &vp):
 *                     pi += 1
 *                     ipi += elsize2             # <<<<<<<<<<<<<<
 * 
 *                 pj -= 1
 */
          __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_elsize2);
        }

        /* "tables/indexesextension.pyx":274
 *                     ipi += elsize2
 * 
 *                 pj -= 1             # <<<<<<<<<<<<<<
 *                 ipj -= elsize2
 *                 while less_than(&vp, pj):
 */
        __pyx_v_pj = (__pyx_v_pj - 1);

        /* "tables/indexesextension.pyx":275
 * 
 *                 pj -= 1
 *                 ipj -= elsize2             # <<<<<<<<<<<<<<
 *                 while less_than(&vp, pj):
 *                     pj -= 1
 */
        __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":276
 *                 pj -= 1
 *                 ipj -= elsize2
 *                 while less_than(&vp, pj):             # <<<<<<<<<<<<<<
 *                     pj -= 1
 *                     ipj -= elsize2
 */
        while (1) {
          __pyx_t_1 = (__pyx_fuse_7__pyx_f_6tables_16indexesextension_less_than((&__pyx_v_vp), __pyx_v_pj) != 0);
          if (!__pyx_t_1) break;

          /* "tables/indexesextension.pyx":277
 *                 ipj -= elsize2
 *                 while less_than(&vp, pj):
 *                     pj -= 1             # <<<<<<<<<<<<<<
 *                     ipj -= elsize2
 * 
 */
          __pyx_v_pj = (__pyx_v_pj - 1);

          /* "tables/indexesextension.pyx":278
 *                 while less_than(&vp, pj):
 *                     pj -= 1
 *                     ipj -= elsize2             # <<<<<<<<<<<<<<
 * 
 *                 if pi >= pj:
 */
          __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_elsize2);
        }

        /* "tables/indexesextension.pyx":280
 *                     ipj -= elsize2
 * 
 *                 if pi >= pj:             # <<<<<<<<<<<<<<
 *                     break
 * 
 */
        __pyx_t_1 = ((__pyx_v_pi >= __pyx_v_pj) != 0);
        if (__pyx_t_1) {

          /* "tables/indexesextension.pyx":281
 * 
 *                 if pi >= pj:
 *                     break             # <<<<<<<<<<<<<<
 * 
 *                 pi[0], pj[0] = pj[0], pi[0]
 */
          goto __pyx_L11_break;

          /* "tables/indexesextension.pyx":280
 *                     ipj -= elsize2
 * 
 *                 if pi >= pj:             # <<<<<<<<<<<<<<
 *                     break
 * 
 */
        }

        /* "tables/indexesextension.pyx":283
 *                     break
 * 
 *                 pi[0], pj[0] = pj[0], pi[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipi, ipj, elsize2)
 * 
 */
        __pyx_t_2 = (__pyx_v_pj[0]);
        __pyx_t_3 = (__pyx_v_pi[0]);
        (__pyx_v_pi[0]) = __pyx_t_2;
        (__pyx_v_pj[0]) = __pyx_t_3;

        /* "tables/indexesextension.pyx":284
 * 
 *                 pi[0], pj[0] = pj[0], pi[0]
 *                 swap_bytes(ipi, ipj, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             pi[0], (pr-1)[0] = (pr-1)[0], pi[0]
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipi, __pyx_v_ipj, __pyx_v_elsize2);
      }
      __pyx_L11_break:;

      /* "tables/indexesextension.pyx":286
 *                 swap_bytes(ipi, ipj, elsize2)
 * 
 *             pi[0], (pr-1)[0] = (pr-1)[0], pi[0]             # <<<<<<<<<<<<<<
 *             swap_bytes(ipi, ipr-elsize2, elsize2)
 * 
 */
      __pyx_t_3 = ((__pyx_v_pr - 1)[0]);
      __pyx_t_2 = (__pyx_v_pi[0]);
      (__pyx_v_pi[0]) = __pyx_t_3;
      ((__pyx_v_pr - 1)[0]) = __pyx_t_2;

      /* "tables/indexesextension.pyx":287
 * 
 *             pi[0], (pr-1)[0] = (pr-1)[0], pi[0]
 *             swap_bytes(ipi, ipr-elsize2, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             # push largest partition on stack and proceed with the other
 */
      __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipi, (__pyx_v_ipr - __pyx_v_elsize2), __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":290
 * 
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):             # <<<<<<<<<<<<<<
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr
 */
      __pyx_t_1 = (((__pyx_v_pi - __pyx_v_pl) < (__pyx_v_pr - __pyx_v_pi)) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":291
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):
 *                 sptr[0] = pi + 1             # <<<<<<<<<<<<<<
 *                 sptr[1] = pr
 *                 sptr += 2
 */
        (__pyx_v_sptr[0]) = (__pyx_v_pi + 1);

        /* "tables/indexesextension.pyx":292
 *             if (pi - pl) < (pr - pi):
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr             # <<<<<<<<<<<<<<
 *                 sptr += 2
 * 
 */
        (__pyx_v_sptr[1]) = __pyx_v_pr;

        /* "tables/indexesextension.pyx":293
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr
 *                 sptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 isptr[0] = ipi + elsize2
 */
        __pyx_v_sptr = (__pyx_v_sptr + 2);

        /* "tables/indexesextension.pyx":295
 *                 sptr += 2
 * 
 *                 isptr[0] = ipi + elsize2             # <<<<<<<<<<<<<<
 *                 isptr[1] = ipr
 *                 isptr += 2
 */
        (__pyx_v_isptr[0]) = (__pyx_v_ipi + __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":296
 * 
 *                 isptr[0] = ipi + elsize2
 *                 isptr[1] = ipr             # <<<<<<<<<<<<<<
 *                 isptr += 2
 * 
 */
        (__pyx_v_isptr[1]) = __pyx_v_ipr;

        /* "tables/indexesextension.pyx":297
 *                 isptr[0] = ipi + elsize2
 *                 isptr[1] = ipr
 *                 isptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 pr = pi - 1
 */
        __pyx_v_isptr = (__pyx_v_isptr + 2);

        /* "tables/indexesextension.pyx":299
 *                 isptr += 2
 * 
 *                 pr = pi - 1             # <<<<<<<<<<<<<<
 *                 ipr = ipi - elsize2
 *             else:
 */
        __pyx_v_pr = (__pyx_v_pi - 1);

        /* "tables/indexesextension.pyx":300
 * 
 *                 pr = pi - 1
 *                 ipr = ipi - elsize2             # <<<<<<<<<<<<<<
 *             else:
 *                 sptr[0] = pl
 */
        __pyx_v_ipr = (__pyx_v_ipi - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":290
 * 
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):             # <<<<<<<<<<<<<<
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr
 */
        goto __pyx_L17;
      }

      /* "tables/indexesextension.pyx":302
 *                 ipr = ipi - elsize2
 *             else:
 *                 sptr[0] = pl             # <<<<<<<<<<<<<<
 *                 sptr[1] = pi - 1
 *                 sptr += 2
 */
      /*else*/ {
        (__pyx_v_sptr[0]) = __pyx_v_pl;

        /* "tables/indexesextension.pyx":303
 *             else:
 *                 sptr[0] = pl
 *                 sptr[1] = pi - 1             # <<<<<<<<<<<<<<
 *                 sptr += 2
 * 
 */
        (__pyx_v_sptr[1]) = (__pyx_v_pi - 1);

        /* "tables/indexesextension.pyx":304
 *                 sptr[0] = pl
 *                 sptr[1] = pi - 1
 *                 sptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 isptr[0] = ipl
 */
        __pyx_v_sptr = (__pyx_v_sptr + 2);

        /* "tables/indexesextension.pyx":306
 *                 sptr += 2
 * 
 *                 isptr[0] = ipl             # <<<<<<<<<<<<<<
 *                 isptr[1] = ipi - elsize2
 *                 isptr += 2
 */
        (__pyx_v_isptr[0]) = __pyx_v_ipl;

        /* "tables/indexesextension.pyx":307
 * 
 *                 isptr[0] = ipl
 *                 isptr[1] = ipi - elsize2             # <<<<<<<<<<<<<<
 *                 isptr += 2
 * 
 */
        (__pyx_v_isptr[1]) = (__pyx_v_ipi - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":308
 *                 isptr[0] = ipl
 *                 isptr[1] = ipi - elsize2
 *                 isptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 pl = pi + 1
 */
        __pyx_v_isptr = (__pyx_v_isptr + 2);

        /* "tables/indexesextension.pyx":310
 *                 isptr += 2
 * 
 *                 pl = pi + 1             # <<<<<<<<<<<<<<
 *                 ipl = ipi + elsize2
 * 
 */
        __pyx_v_pl = (__pyx_v_pi + 1);

        /* "tables/indexesextension.pyx":311
 * 
 *                 pl = pi + 1
 *                 ipl = ipi + elsize2             # <<<<<<<<<<<<<<
 * 
 *         pi = pl + 1
 */
        __pyx_v_ipl = (__pyx_v_ipi + __pyx_v_elsize2);
      }
      __pyx_L17:;
    }

    /* "tables/indexesextension.pyx":313
 *                 ipl = ipi + elsize2
 * 
 *         pi = pl + 1             # <<<<<<<<<<<<<<
 *         ipi = ipl + elsize2
 *         while pi <= pr:
 */
    __pyx_v_pi = (__pyx_v_pl + 1);

    /* "tables/indexesextension.pyx":314
 * 
 *         pi = pl + 1
 *         ipi = ipl + elsize2             # <<<<<<<<<<<<<<
 *         while pi <= pr:
 *             vp = pi[0]
 */
    __pyx_v_ipi = (__pyx_v_ipl + __pyx_v_elsize2);

    /* "tables/indexesextension.pyx":315
 *         pi = pl + 1
 *         ipi = ipl + elsize2
 *         while pi <= pr:             # <<<<<<<<<<<<<<
 *             vp = pi[0]
 *             memcpy(ivp, ipi, elsize2)
 */
    while (1) {
      __pyx_t_1 = ((__pyx_v_pi <= __pyx_v_pr) != 0);
      if (!__pyx_t_1) break;

      /* "tables/indexesextension.pyx":316
 *         ipi = ipl + elsize2
 *         while pi <= pr:
 *             vp = pi[0]             # <<<<<<<<<<<<<<
 *             memcpy(ivp, ipi, elsize2)
 * 
 */
      __pyx_v_vp = (__pyx_v_pi[0]);

      /* "tables/indexesextension.pyx":317
 *         while pi <= pr:
 *             vp = pi[0]
 *             memcpy(ivp, ipi, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             pj = pi
 */
      (void)(memcpy(__pyx_v_ivp, __pyx_v_ipi, __pyx_v_elsize2));

      /* "tables/indexesextension.pyx":319
 *             memcpy(ivp, ipi, elsize2)
 * 
 *             pj = pi             # <<<<<<<<<<<<<<
 *             pt = pi - 1
 * 
 */
      __pyx_v_pj = __pyx_v_pi;

      /* "tables/indexesextension.pyx":320
 * 
 *             pj = pi
 *             pt = pi - 1             # <<<<<<<<<<<<<<
 * 
 *             ipj = ipi
 */
      __pyx_v_pt = (__pyx_v_pi - 1);

      /* "tables/indexesextension.pyx":322
 *             pt = pi - 1
 * 
 *             ipj = ipi             # <<<<<<<<<<<<<<
 *             ipt = ipi - elsize2
 * 
 */
      __pyx_v_ipj = __pyx_v_ipi;

      /* "tables/indexesextension.pyx":323
 * 
 *             ipj = ipi
 *             ipt = ipi - elsize2             # <<<<<<<<<<<<<<
 * 
 *             while pj > pl and less_than(&vp, pt):
 */
      __pyx_v_ipt = (__pyx_v_ipi - __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":325
 *             ipt = ipi - elsize2
 * 
 *             while pj > pl and less_than(&vp, pt):             # <<<<<<<<<<<<<<
 *                 pj[0] = pt[0]
 *                 pj -= 1
 */
      while (1) {
        __pyx_t_4 = ((__pyx_v_pj > __pyx_v_pl) != 0);
        if (__pyx_t_4) {
        } else {
          __pyx_t_1 = __pyx_t_4;
          goto __pyx_L22_bool_binop_done;
        }
        __pyx_t_4 = (__pyx_fuse_7__pyx_f_6tables_16indexesextension_less_than((&__pyx_v_vp), __pyx_v_pt) != 0);
        __pyx_t_1 = __pyx_t_4;
        __pyx_L22_bool_binop_done:;
        if (!__pyx_t_1) break;

        /* "tables/indexesextension.pyx":326
 * 
 *             while pj > pl and less_than(&vp, pt):
 *                 pj[0] = pt[0]             # <<<<<<<<<<<<<<
 *                 pj -= 1
 *                 pt -= 1
 */
        (__pyx_v_pj[0]) = (__pyx_v_pt[0]);

        /* "tables/indexesextension.pyx":327
 *             while pj > pl and less_than(&vp, pt):
 *                 pj[0] = pt[0]
 *                 pj -= 1             # <<<<<<<<<<<<<<
 *                 pt -= 1
 * 
 */
        __pyx_v_pj = (__pyx_v_pj - 1);

        /* "tables/indexesextension.pyx":328
 *                 pj[0] = pt[0]
 *                 pj -= 1
 *                 pt -= 1             # <<<<<<<<<<<<<<
 * 
 *                 memcpy(ipj, ipt, elsize2)
 */
        __pyx_v_pt = (__pyx_v_pt - 1);

        /* "tables/indexesextension.pyx":330
 *                 pt -= 1
 * 
 *                 memcpy(ipj, ipt, elsize2)             # <<<<<<<<<<<<<<
 *                 ipj -= elsize2
 *                 ipt -= elsize2
 */
        (void)(memcpy(__pyx_v_ipj, __pyx_v_ipt, __pyx_v_elsize2));

        /* "tables/indexesextension.pyx":331
 * 
 *                 memcpy(ipj, ipt, elsize2)
 *                 ipj -= elsize2             # <<<<<<<<<<<<<<
 *                 ipt -= elsize2
 * 
 */
        __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":332
 *                 memcpy(ipj, ipt, elsize2)
 *                 ipj -= elsize2
 *                 ipt -= elsize2             # <<<<<<<<<<<<<<
 * 
 *             pj[0] = vp
 */
        __pyx_v_ipt = (__pyx_v_ipt - __pyx_v_elsize2);
      }

      /* "tables/indexesextension.pyx":334
 *                 ipt -= elsize2
 * 
 *             pj[0] = vp             # <<<<<<<<<<<<<<
 *             memcpy(ipj, ivp, elsize2)
 * 
 */
      (__pyx_v_pj[0]) = __pyx_v_vp;

      /* "tables/indexesextension.pyx":335
 * 
 *             pj[0] = vp
 *             memcpy(ipj, ivp, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             pi += 1
 */
      (void)(memcpy(__pyx_v_ipj, __pyx_v_ivp, __pyx_v_elsize2));

      /* "tables/indexesextension.pyx":337
 *             memcpy(ipj, ivp, elsize2)
 * 
 *             pi += 1             # <<<<<<<<<<<<<<
 *             ipi += elsize2
 * 
 */
      __pyx_v_pi = (__pyx_v_pi + 1);

      /* "tables/indexesextension.pyx":338
 * 
 *             pi += 1
 *             ipi += elsize2             # <<<<<<<<<<<<<<
 * 
 *         if sptr == stack:
 */
      __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_elsize2);
    }

    /* "tables/indexesextension.pyx":340
 *             ipi += elsize2
 * 
 *         if sptr == stack:             # <<<<<<<<<<<<<<
 *             break
 * 
 */
    __pyx_t_1 = ((__pyx_v_sptr == __pyx_v_stack) != 0);
    if (__pyx_t_1) {

      /* "tables/indexesextension.pyx":341
 * 
 *         if sptr == stack:
 *             break             # <<<<<<<<<<<<<<
 * 
 *         sptr -= 2
 */
      goto __pyx_L4_break;

      /* "tables/indexesextension.pyx":340
 *             ipi += elsize2
 * 
 *         if sptr == stack:             # <<<<<<<<<<<<<<
 *             break
 * 
 */
    }

    /* "tables/indexesextension.pyx":343
 *             break
 * 
 *         sptr -= 2             # <<<<<<<<<<<<<<
 *         pl = sptr[0]
 *         pr = sptr[1]
 */
    __pyx_v_sptr = (__pyx_v_sptr - 2);

    /* "tables/indexesextension.pyx":344
 * 
 *         sptr -= 2
 *         pl = sptr[0]             # <<<<<<<<<<<<<<
 *         pr = sptr[1]
 * 
 */
    __pyx_v_pl = (__pyx_v_sptr[0]);

    /* "tables/indexesextension.pyx":345
 *         sptr -= 2
 *         pl = sptr[0]
 *         pr = sptr[1]             # <<<<<<<<<<<<<<
 * 
 *         isptr -= 2
 */
    __pyx_v_pr = (__pyx_v_sptr[1]);

    /* "tables/indexesextension.pyx":347
 *         pr = sptr[1]
 * 
 *         isptr -= 2             # <<<<<<<<<<<<<<
 *         ipl = isptr[0]
 *         ipr = isptr[1]
 */
    __pyx_v_isptr = (__pyx_v_isptr - 2);

    /* "tables/indexesextension.pyx":348
 * 
 *         isptr -= 2
 *         ipl = isptr[0]             # <<<<<<<<<<<<<<
 *         ipr = isptr[1]
 * 
 */
    __pyx_v_ipl = (__pyx_v_isptr[0]);

    /* "tables/indexesextension.pyx":349
 *         isptr -= 2
 *         ipl = isptr[0]
 *         ipr = isptr[1]             # <<<<<<<<<<<<<<
 * 
 *     free(ivp)
 */
    __pyx_v_ipr = (__pyx_v_isptr[1]);
  }
  __pyx_L4_break:;

  /* "tables/indexesextension.pyx":351
 *         ipr = isptr[1]
 * 
 *     free(ivp)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  free(__pyx_v_ivp);

  /* "tables/indexesextension.pyx":212
 * 
 * @cython.cdivision(True)
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:             # <<<<<<<<<<<<<<
 *     cdef number_type *pl = start1
 *     cdef number_type *pr = start1 + (n - 1)
 */

  /* function exit code */
}

static void __pyx_fuse_8__pyx_f_6tables_16indexesextension__keysort(npy_float32 *__pyx_v_start1, char *__pyx_v_start2, size_t __pyx_v_elsize2, size_t __pyx_v_n) {
  npy_float32 *__pyx_v_pl;
  npy_float32 *__pyx_v_pr;
  char *__pyx_v_ipl;
  char *__pyx_v_ipr;
  npy_float32 __pyx_v_vp;
  char *__pyx_v_ivp;
  npy_float32 *__pyx_v_stack[0x64];
  npy_float32 **__pyx_v_sptr;
  char *__pyx_v_istack[0x64];
  char **__pyx_v_isptr;
  CYTHON_UNUSED size_t __pyx_v_stack_index;
  npy_float32 *__pyx_v_pm;
  npy_float32 *__pyx_v_pi;
  npy_float32 *__pyx_v_pj;
  npy_float32 *__pyx_v_pt;
  char *__pyx_v_ipm;
  char *__pyx_v_ipi;
  char *__pyx_v_ipj;
  char *__pyx_v_ipt;
  int __pyx_t_1;
  npy_float32 __pyx_t_2;
  npy_float32 __pyx_t_3;
  int __pyx_t_4;

  /* "tables/indexesextension.pyx":213
 * @cython.cdivision(True)
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:
 *     cdef number_type *pl = start1             # <<<<<<<<<<<<<<
 *     cdef number_type *pr = start1 + (n - 1)
 * 
 */
  __pyx_v_pl = __pyx_v_start1;

  /* "tables/indexesextension.pyx":214
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:
 *     cdef number_type *pl = start1
 *     cdef number_type *pr = start1 + (n - 1)             # <<<<<<<<<<<<<<
 * 
 *     cdef char *ipl = start2
 */
  __pyx_v_pr = (__pyx_v_start1 + (__pyx_v_n - 1));

  /* "tables/indexesextension.pyx":216
 *     cdef number_type *pr = start1 + (n - 1)
 * 
 *     cdef char *ipl = start2             # <<<<<<<<<<<<<<
 *     cdef char *ipr = start2 + (n - 1) * elsize2
 * 
 */
  __pyx_v_ipl = __pyx_v_start2;

  /* "tables/indexesextension.pyx":217
 * 
 *     cdef char *ipl = start2
 *     cdef char *ipr = start2 + (n - 1) * elsize2             # <<<<<<<<<<<<<<
 * 
 *     cdef number_type vp
 */
  __pyx_v_ipr = (__pyx_v_start2 + ((__pyx_v_n - 1) * __pyx_v_elsize2));

  /* "tables/indexesextension.pyx":220
 * 
 *     cdef number_type vp
 *     cdef char *ivp = <char *> malloc(elsize2)             # <<<<<<<<<<<<<<
 * 
 *     cdef number_type *stack[PYA_QS_STACK]
 */
  __pyx_v_ivp = ((char *)malloc(__pyx_v_elsize2));

  /* "tables/indexesextension.pyx":223
 * 
 *     cdef number_type *stack[PYA_QS_STACK]
 *     cdef number_type **sptr = stack             # <<<<<<<<<<<<<<
 * 
 *     cdef char *istack[PYA_QS_STACK]
 */
  __pyx_v_sptr = __pyx_v_stack;

  /* "tables/indexesextension.pyx":226
 * 
 *     cdef char *istack[PYA_QS_STACK]
 *     cdef char **isptr = istack             # <<<<<<<<<<<<<<
 * 
 *     cdef size_t stack_index = 0
 */
  __pyx_v_isptr = __pyx_v_istack;

  /* "tables/indexesextension.pyx":228
 *     cdef char **isptr = istack
 * 
 *     cdef size_t stack_index = 0             # <<<<<<<<<<<<<<
 * 
 *     cdef number_type *pm
 */
  __pyx_v_stack_index = 0;

  /* "tables/indexesextension.pyx":239
 *     cdef char *ipt
 * 
 *     while True:             # <<<<<<<<<<<<<<
 *         while pr - pl > SMALL_QUICKSORT:
 *             pm  = pl + ((pr - pl) >> 1)
 */
  while (1) {

    /* "tables/indexesextension.pyx":240
 * 
 *     while True:
 *         while pr - pl > SMALL_QUICKSORT:             # <<<<<<<<<<<<<<
 *             pm  = pl + ((pr - pl) >> 1)
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 */
    while (1) {
      __pyx_t_1 = (((__pyx_v_pr - __pyx_v_pl) > 15) != 0);
      if (!__pyx_t_1) break;

      /* "tables/indexesextension.pyx":241
 *     while True:
 *         while pr - pl > SMALL_QUICKSORT:
 *             pm  = pl + ((pr - pl) >> 1)             # <<<<<<<<<<<<<<
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 * 
 */
      __pyx_v_pm = (__pyx_v_pl + ((__pyx_v_pr - __pyx_v_pl) >> 1));

      /* "tables/indexesextension.pyx":242
 *         while pr - pl > SMALL_QUICKSORT:
 *             pm  = pl + ((pr - pl) >> 1)
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2             # <<<<<<<<<<<<<<
 * 
 *             if less_than(pm, pl):
 */
      __pyx_v_ipm = (__pyx_v_ipl + ((((__pyx_v_ipr - __pyx_v_ipl) / __pyx_v_elsize2) >> 1) * __pyx_v_elsize2));

      /* "tables/indexesextension.pyx":244
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      __pyx_t_1 = (__pyx_fuse_8__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pm, __pyx_v_pl) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":245
 * 
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 */
        __pyx_t_2 = (__pyx_v_pl[0]);
        __pyx_t_3 = (__pyx_v_pm[0]);
        (__pyx_v_pm[0]) = __pyx_t_2;
        (__pyx_v_pl[0]) = __pyx_t_3;

        /* "tables/indexesextension.pyx":246
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             if less_than(pr, pm):
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipl, __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":244
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      }

      /* "tables/indexesextension.pyx":248
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 *             if less_than(pr, pm):             # <<<<<<<<<<<<<<
 *                 pr[0], pm[0] =  pm[0], pr[0]
 *                 swap_bytes(ipr, ipm, elsize2)
 */
      __pyx_t_1 = (__pyx_fuse_8__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pr, __pyx_v_pm) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":249
 * 
 *             if less_than(pr, pm):
 *                 pr[0], pm[0] =  pm[0], pr[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipr, ipm, elsize2)
 * 
 */
        __pyx_t_3 = (__pyx_v_pm[0]);
        __pyx_t_2 = (__pyx_v_pr[0]);
        (__pyx_v_pr[0]) = __pyx_t_3;
        (__pyx_v_pm[0]) = __pyx_t_2;

        /* "tables/indexesextension.pyx":250
 *             if less_than(pr, pm):
 *                 pr[0], pm[0] =  pm[0], pr[0]
 *                 swap_bytes(ipr, ipm, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             if less_than(pm, pl):
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipr, __pyx_v_ipm, __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":248
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 *             if less_than(pr, pm):             # <<<<<<<<<<<<<<
 *                 pr[0], pm[0] =  pm[0], pr[0]
 *                 swap_bytes(ipr, ipm, elsize2)
 */
      }

      /* "tables/indexesextension.pyx":252
 *                 swap_bytes(ipr, ipm, elsize2)
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      __pyx_t_1 = (__pyx_fuse_8__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pm, __pyx_v_pl) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":253
 * 
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 */
        __pyx_t_2 = (__pyx_v_pl[0]);
        __pyx_t_3 = (__pyx_v_pm[0]);
        (__pyx_v_pm[0]) = __pyx_t_2;
        (__pyx_v_pl[0]) = __pyx_t_3;

        /* "tables/indexesextension.pyx":254
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             vp = pm[0]
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipl, __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":252
 *                 swap_bytes(ipr, ipm, elsize2)
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      }

      /* "tables/indexesextension.pyx":256
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 *             vp = pm[0]             # <<<<<<<<<<<<<<
 * 
 *             pi = pl
 */
      __pyx_v_vp = (__pyx_v_pm[0]);

      /* "tables/indexesextension.pyx":258
 *             vp = pm[0]
 * 
 *             pi = pl             # <<<<<<<<<<<<<<
 *             ipi = ipl
 * 
 */
      __pyx_v_pi = __pyx_v_pl;

      /* "tables/indexesextension.pyx":259
 * 
 *             pi = pl
 *             ipi = ipl             # <<<<<<<<<<<<<<
 * 
 *             pj = pr - 1
 */
      __pyx_v_ipi = __pyx_v_ipl;

      /* "tables/indexesextension.pyx":261
 *             ipi = ipl
 * 
 *             pj = pr - 1             # <<<<<<<<<<<<<<
 *             ipj = ipr - elsize2
 * 
 */
      __pyx_v_pj = (__pyx_v_pr - 1);

      /* "tables/indexesextension.pyx":262
 * 
 *             pj = pr - 1
 *             ipj = ipr - elsize2             # <<<<<<<<<<<<<<
 * 
 *             pm[0], pj[0] = pj[0], pm[0]
 */
      __pyx_v_ipj = (__pyx_v_ipr - __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":264
 *             ipj = ipr - elsize2
 * 
 *             pm[0], pj[0] = pj[0], pm[0]             # <<<<<<<<<<<<<<
 *             swap_bytes(ipm, ipj, elsize2)
 * 
 */
      __pyx_t_3 = (__pyx_v_pj[0]);
      __pyx_t_2 = (__pyx_v_pm[0]);
      (__pyx_v_pm[0]) = __pyx_t_3;
      (__pyx_v_pj[0]) = __pyx_t_2;

      /* "tables/indexesextension.pyx":265
 * 
 *             pm[0], pj[0] = pj[0], pm[0]
 *             swap_bytes(ipm, ipj, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             while True:
 */
      __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipj, __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":267
 *             swap_bytes(ipm, ipj, elsize2)
 * 
 *             while True:             # <<<<<<<<<<<<<<
 *                 pi += 1
 *                 ipi += elsize2
 */
      while (1) {

        /* "tables/indexesextension.pyx":268
 * 
 *             while True:
 *                 pi += 1             # <<<<<<<<<<<<<<
 *                 ipi += elsize2
 *                 while less_than(pi, &vp):
 */
        __pyx_v_pi = (__pyx_v_pi + 1);

        /* "tables/indexesextension.pyx":269
 *             while True:
 *                 pi += 1
 *                 ipi += elsize2             # <<<<<<<<<<<<<<
 *                 while less_than(pi, &vp):
 *                     pi += 1
 */
        __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":270
 *                 pi += 1
 *                 ipi += elsize2
 *                 while less_than(pi, &vp):             # <<<<<<<<<<<<<<
 *                     pi += 1
 *                     ipi += elsize2
 */
        while (1) {
          __pyx_t_1 = (__pyx_fuse_8__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pi, (&__pyx_v_vp)) != 0);
          if (!__pyx_t_1) break;

          /* "tables/indexesextension.pyx":271
 *                 ipi += elsize2
 *                 while less_than(pi, &vp):
 *                     pi += 1             # <<<<<<<<<<<<<<
 *                     ipi += elsize2
 * 
 */
          __pyx_v_pi = (__pyx_v_pi + 1);

          /* "tables/indexesextension.pyx":272
 *                 while less_than(pi, &vp):
 *                     pi += 1
 *                     ipi += elsize2             # <<<<<<<<<<<<<<
 * 
 *                 pj -= 1
 */
          __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_elsize2);
        }

        /* "tables/indexesextension.pyx":274
 *                     ipi += elsize2
 * 
 *                 pj -= 1             # <<<<<<<<<<<<<<
 *                 ipj -= elsize2
 *                 while less_than(&vp, pj):
 */
        __pyx_v_pj = (__pyx_v_pj - 1);

        /* "tables/indexesextension.pyx":275
 * 
 *                 pj -= 1
 *                 ipj -= elsize2             # <<<<<<<<<<<<<<
 *                 while less_than(&vp, pj):
 *                     pj -= 1
 */
        __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":276
 *                 pj -= 1
 *                 ipj -= elsize2
 *                 while less_than(&vp, pj):             # <<<<<<<<<<<<<<
 *                     pj -= 1
 *                     ipj -= elsize2
 */
        while (1) {
          __pyx_t_1 = (__pyx_fuse_8__pyx_f_6tables_16indexesextension_less_than((&__pyx_v_vp), __pyx_v_pj) != 0);
          if (!__pyx_t_1) break;

          /* "tables/indexesextension.pyx":277
 *                 ipj -= elsize2
 *                 while less_than(&vp, pj):
 *                     pj -= 1             # <<<<<<<<<<<<<<
 *                     ipj -= elsize2
 * 
 */
          __pyx_v_pj = (__pyx_v_pj - 1);

          /* "tables/indexesextension.pyx":278
 *                 while less_than(&vp, pj):
 *                     pj -= 1
 *                     ipj -= elsize2             # <<<<<<<<<<<<<<
 * 
 *                 if pi >= pj:
 */
          __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_elsize2);
        }

        /* "tables/indexesextension.pyx":280
 *                     ipj -= elsize2
 * 
 *                 if pi >= pj:             # <<<<<<<<<<<<<<
 *                     break
 * 
 */
        __pyx_t_1 = ((__pyx_v_pi >= __pyx_v_pj) != 0);
        if (__pyx_t_1) {

          /* "tables/indexesextension.pyx":281
 * 
 *                 if pi >= pj:
 *                     break             # <<<<<<<<<<<<<<
 * 
 *                 pi[0], pj[0] = pj[0], pi[0]
 */
          goto __pyx_L11_break;

          /* "tables/indexesextension.pyx":280
 *                     ipj -= elsize2
 * 
 *                 if pi >= pj:             # <<<<<<<<<<<<<<
 *                     break
 * 
 */
        }

        /* "tables/indexesextension.pyx":283
 *                     break
 * 
 *                 pi[0], pj[0] = pj[0], pi[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipi, ipj, elsize2)
 * 
 */
        __pyx_t_2 = (__pyx_v_pj[0]);
        __pyx_t_3 = (__pyx_v_pi[0]);
        (__pyx_v_pi[0]) = __pyx_t_2;
        (__pyx_v_pj[0]) = __pyx_t_3;

        /* "tables/indexesextension.pyx":284
 * 
 *                 pi[0], pj[0] = pj[0], pi[0]
 *                 swap_bytes(ipi, ipj, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             pi[0], (pr-1)[0] = (pr-1)[0], pi[0]
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipi, __pyx_v_ipj, __pyx_v_elsize2);
      }
      __pyx_L11_break:;

      /* "tables/indexesextension.pyx":286
 *                 swap_bytes(ipi, ipj, elsize2)
 * 
 *             pi[0], (pr-1)[0] = (pr-1)[0], pi[0]             # <<<<<<<<<<<<<<
 *             swap_bytes(ipi, ipr-elsize2, elsize2)
 * 
 */
      __pyx_t_3 = ((__pyx_v_pr - 1)[0]);
      __pyx_t_2 = (__pyx_v_pi[0]);
      (__pyx_v_pi[0]) = __pyx_t_3;
      ((__pyx_v_pr - 1)[0]) = __pyx_t_2;

      /* "tables/indexesextension.pyx":287
 * 
 *             pi[0], (pr-1)[0] = (pr-1)[0], pi[0]
 *             swap_bytes(ipi, ipr-elsize2, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             # push largest partition on stack and proceed with the other
 */
      __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipi, (__pyx_v_ipr - __pyx_v_elsize2), __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":290
 * 
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):             # <<<<<<<<<<<<<<
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr
 */
      __pyx_t_1 = (((__pyx_v_pi - __pyx_v_pl) < (__pyx_v_pr - __pyx_v_pi)) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":291
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):
 *                 sptr[0] = pi + 1             # <<<<<<<<<<<<<<
 *                 sptr[1] = pr
 *                 sptr += 2
 */
        (__pyx_v_sptr[0]) = (__pyx_v_pi + 1);

        /* "tables/indexesextension.pyx":292
 *             if (pi - pl) < (pr - pi):
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr             # <<<<<<<<<<<<<<
 *                 sptr += 2
 * 
 */
        (__pyx_v_sptr[1]) = __pyx_v_pr;

        /* "tables/indexesextension.pyx":293
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr
 *                 sptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 isptr[0] = ipi + elsize2
 */
        __pyx_v_sptr = (__pyx_v_sptr + 2);

        /* "tables/indexesextension.pyx":295
 *                 sptr += 2
 * 
 *                 isptr[0] = ipi + elsize2             # <<<<<<<<<<<<<<
 *                 isptr[1] = ipr
 *                 isptr += 2
 */
        (__pyx_v_isptr[0]) = (__pyx_v_ipi + __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":296
 * 
 *                 isptr[0] = ipi + elsize2
 *                 isptr[1] = ipr             # <<<<<<<<<<<<<<
 *                 isptr += 2
 * 
 */
        (__pyx_v_isptr[1]) = __pyx_v_ipr;

        /* "tables/indexesextension.pyx":297
 *                 isptr[0] = ipi + elsize2
 *                 isptr[1] = ipr
 *                 isptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 pr = pi - 1
 */
        __pyx_v_isptr = (__pyx_v_isptr + 2);

        /* "tables/indexesextension.pyx":299
 *                 isptr += 2
 * 
 *                 pr = pi - 1             # <<<<<<<<<<<<<<
 *                 ipr = ipi - elsize2
 *             else:
 */
        __pyx_v_pr = (__pyx_v_pi - 1);

        /* "tables/indexesextension.pyx":300
 * 
 *                 pr = pi - 1
 *                 ipr = ipi - elsize2             # <<<<<<<<<<<<<<
 *             else:
 *                 sptr[0] = pl
 */
        __pyx_v_ipr = (__pyx_v_ipi - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":290
 * 
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):             # <<<<<<<<<<<<<<
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr
 */
        goto __pyx_L17;
      }

      /* "tables/indexesextension.pyx":302
 *                 ipr = ipi - elsize2
 *             else:
 *                 sptr[0] = pl             # <<<<<<<<<<<<<<
 *                 sptr[1] = pi - 1
 *                 sptr += 2
 */
      /*else*/ {
        (__pyx_v_sptr[0]) = __pyx_v_pl;

        /* "tables/indexesextension.pyx":303
 *             else:
 *                 sptr[0] = pl
 *                 sptr[1] = pi - 1             # <<<<<<<<<<<<<<
 *                 sptr += 2
 * 
 */
        (__pyx_v_sptr[1]) = (__pyx_v_pi - 1);

        /* "tables/indexesextension.pyx":304
 *                 sptr[0] = pl
 *                 sptr[1] = pi - 1
 *                 sptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 isptr[0] = ipl
 */
        __pyx_v_sptr = (__pyx_v_sptr + 2);

        /* "tables/indexesextension.pyx":306
 *                 sptr += 2
 * 
 *                 isptr[0] = ipl             # <<<<<<<<<<<<<<
 *                 isptr[1] = ipi - elsize2
 *                 isptr += 2
 */
        (__pyx_v_isptr[0]) = __pyx_v_ipl;

        /* "tables/indexesextension.pyx":307
 * 
 *                 isptr[0] = ipl
 *                 isptr[1] = ipi - elsize2             # <<<<<<<<<<<<<<
 *                 isptr += 2
 * 
 */
        (__pyx_v_isptr[1]) = (__pyx_v_ipi - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":308
 *                 isptr[0] = ipl
 *                 isptr[1] = ipi - elsize2
 *                 isptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 pl = pi + 1
 */
        __pyx_v_isptr = (__pyx_v_isptr + 2);

        /* "tables/indexesextension.pyx":310
 *                 isptr += 2
 * 
 *                 pl = pi + 1             # <<<<<<<<<<<<<<
 *                 ipl = ipi + elsize2
 * 
 */
        __pyx_v_pl = (__pyx_v_pi + 1);

        /* "tables/indexesextension.pyx":311
 * 
 *                 pl = pi + 1
 *                 ipl = ipi + elsize2             # <<<<<<<<<<<<<<
 * 
 *         pi = pl + 1
 */
        __pyx_v_ipl = (__pyx_v_ipi + __pyx_v_elsize2);
      }
      __pyx_L17:;
    }

    /* "tables/indexesextension.pyx":313
 *                 ipl = ipi + elsize2
 * 
 *         pi = pl + 1             # <<<<<<<<<<<<<<
 *         ipi = ipl + elsize2
 *         while pi <= pr:
 */
    __pyx_v_pi = (__pyx_v_pl + 1);

    /* "tables/indexesextension.pyx":314
 * 
 *         pi = pl + 1
 *         ipi = ipl + elsize2             # <<<<<<<<<<<<<<
 *         while pi <= pr:
 *             vp = pi[0]
 */
    __pyx_v_ipi = (__pyx_v_ipl + __pyx_v_elsize2);

    /* "tables/indexesextension.pyx":315
 *         pi = pl + 1
 *         ipi = ipl + elsize2
 *         while pi <= pr:             # <<<<<<<<<<<<<<
 *             vp = pi[0]
 *             memcpy(ivp, ipi, elsize2)
 */
    while (1) {
      __pyx_t_1 = ((__pyx_v_pi <= __pyx_v_pr) != 0);
      if (!__pyx_t_1) break;

      /* "tables/indexesextension.pyx":316
 *         ipi = ipl + elsize2
 *         while pi <= pr:
 *             vp = pi[0]             # <<<<<<<<<<<<<<
 *             memcpy(ivp, ipi, elsize2)
 * 
 */
      __pyx_v_vp = (__pyx_v_pi[0]);

      /* "tables/indexesextension.pyx":317
 *         while pi <= pr:
 *             vp = pi[0]
 *             memcpy(ivp, ipi, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             pj = pi
 */
      (void)(memcpy(__pyx_v_ivp, __pyx_v_ipi, __pyx_v_elsize2));

      /* "tables/indexesextension.pyx":319
 *             memcpy(ivp, ipi, elsize2)
 * 
 *             pj = pi             # <<<<<<<<<<<<<<
 *             pt = pi - 1
 * 
 */
      __pyx_v_pj = __pyx_v_pi;

      /* "tables/indexesextension.pyx":320
 * 
 *             pj = pi
 *             pt = pi - 1             # <<<<<<<<<<<<<<
 * 
 *             ipj = ipi
 */
      __pyx_v_pt = (__pyx_v_pi - 1);

      /* "tables/indexesextension.pyx":322
 *             pt = pi - 1
 * 
 *             ipj = ipi             # <<<<<<<<<<<<<<
 *             ipt = ipi - elsize2
 * 
 */
      __pyx_v_ipj = __pyx_v_ipi;

      /* "tables/indexesextension.pyx":323
 * 
 *             ipj = ipi
 *             ipt = ipi - elsize2             # <<<<<<<<<<<<<<
 * 
 *             while pj > pl and less_than(&vp, pt):
 */
      __pyx_v_ipt = (__pyx_v_ipi - __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":325
 *             ipt = ipi - elsize2
 * 
 *             while pj > pl and less_than(&vp, pt):             # <<<<<<<<<<<<<<
 *                 pj[0] = pt[0]
 *                 pj -= 1
 */
      while (1) {
        __pyx_t_4 = ((__pyx_v_pj > __pyx_v_pl) != 0);
        if (__pyx_t_4) {
        } else {
          __pyx_t_1 = __pyx_t_4;
          goto __pyx_L22_bool_binop_done;
        }
        __pyx_t_4 = (__pyx_fuse_8__pyx_f_6tables_16indexesextension_less_than((&__pyx_v_vp), __pyx_v_pt) != 0);
        __pyx_t_1 = __pyx_t_4;
        __pyx_L22_bool_binop_done:;
        if (!__pyx_t_1) break;

        /* "tables/indexesextension.pyx":326
 * 
 *             while pj > pl and less_than(&vp, pt):
 *                 pj[0] = pt[0]             # <<<<<<<<<<<<<<
 *                 pj -= 1
 *                 pt -= 1
 */
        (__pyx_v_pj[0]) = (__pyx_v_pt[0]);

        /* "tables/indexesextension.pyx":327
 *             while pj > pl and less_than(&vp, pt):
 *                 pj[0] = pt[0]
 *                 pj -= 1             # <<<<<<<<<<<<<<
 *                 pt -= 1
 * 
 */
        __pyx_v_pj = (__pyx_v_pj - 1);

        /* "tables/indexesextension.pyx":328
 *                 pj[0] = pt[0]
 *                 pj -= 1
 *                 pt -= 1             # <<<<<<<<<<<<<<
 * 
 *                 memcpy(ipj, ipt, elsize2)
 */
        __pyx_v_pt = (__pyx_v_pt - 1);

        /* "tables/indexesextension.pyx":330
 *                 pt -= 1
 * 
 *                 memcpy(ipj, ipt, elsize2)             # <<<<<<<<<<<<<<
 *                 ipj -= elsize2
 *                 ipt -= elsize2
 */
        (void)(memcpy(__pyx_v_ipj, __pyx_v_ipt, __pyx_v_elsize2));

        /* "tables/indexesextension.pyx":331
 * 
 *                 memcpy(ipj, ipt, elsize2)
 *                 ipj -= elsize2             # <<<<<<<<<<<<<<
 *                 ipt -= elsize2
 * 
 */
        __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":332
 *                 memcpy(ipj, ipt, elsize2)
 *                 ipj -= elsize2
 *                 ipt -= elsize2             # <<<<<<<<<<<<<<
 * 
 *             pj[0] = vp
 */
        __pyx_v_ipt = (__pyx_v_ipt - __pyx_v_elsize2);
      }

      /* "tables/indexesextension.pyx":334
 *                 ipt -= elsize2
 * 
 *             pj[0] = vp             # <<<<<<<<<<<<<<
 *             memcpy(ipj, ivp, elsize2)
 * 
 */
      (__pyx_v_pj[0]) = __pyx_v_vp;

      /* "tables/indexesextension.pyx":335
 * 
 *             pj[0] = vp
 *             memcpy(ipj, ivp, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             pi += 1
 */
      (void)(memcpy(__pyx_v_ipj, __pyx_v_ivp, __pyx_v_elsize2));

      /* "tables/indexesextension.pyx":337
 *             memcpy(ipj, ivp, elsize2)
 * 
 *             pi += 1             # <<<<<<<<<<<<<<
 *             ipi += elsize2
 * 
 */
      __pyx_v_pi = (__pyx_v_pi + 1);

      /* "tables/indexesextension.pyx":338
 * 
 *             pi += 1
 *             ipi += elsize2             # <<<<<<<<<<<<<<
 * 
 *         if sptr == stack:
 */
      __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_elsize2);
    }

    /* "tables/indexesextension.pyx":340
 *             ipi += elsize2
 * 
 *         if sptr == stack:             # <<<<<<<<<<<<<<
 *             break
 * 
 */
    __pyx_t_1 = ((__pyx_v_sptr == __pyx_v_stack) != 0);
    if (__pyx_t_1) {

      /* "tables/indexesextension.pyx":341
 * 
 *         if sptr == stack:
 *             break             # <<<<<<<<<<<<<<
 * 
 *         sptr -= 2
 */
      goto __pyx_L4_break;

      /* "tables/indexesextension.pyx":340
 *             ipi += elsize2
 * 
 *         if sptr == stack:             # <<<<<<<<<<<<<<
 *             break
 * 
 */
    }

    /* "tables/indexesextension.pyx":343
 *             break
 * 
 *         sptr -= 2             # <<<<<<<<<<<<<<
 *         pl = sptr[0]
 *         pr = sptr[1]
 */
    __pyx_v_sptr = (__pyx_v_sptr - 2);

    /* "tables/indexesextension.pyx":344
 * 
 *         sptr -= 2
 *         pl = sptr[0]             # <<<<<<<<<<<<<<
 *         pr = sptr[1]
 * 
 */
    __pyx_v_pl = (__pyx_v_sptr[0]);

    /* "tables/indexesextension.pyx":345
 *         sptr -= 2
 *         pl = sptr[0]
 *         pr = sptr[1]             # <<<<<<<<<<<<<<
 * 
 *         isptr -= 2
 */
    __pyx_v_pr = (__pyx_v_sptr[1]);

    /* "tables/indexesextension.pyx":347
 *         pr = sptr[1]
 * 
 *         isptr -= 2             # <<<<<<<<<<<<<<
 *         ipl = isptr[0]
 *         ipr = isptr[1]
 */
    __pyx_v_isptr = (__pyx_v_isptr - 2);

    /* "tables/indexesextension.pyx":348
 * 
 *         isptr -= 2
 *         ipl = isptr[0]             # <<<<<<<<<<<<<<
 *         ipr = isptr[1]
 * 
 */
    __pyx_v_ipl = (__pyx_v_isptr[0]);

    /* "tables/indexesextension.pyx":349
 *         isptr -= 2
 *         ipl = isptr[0]
 *         ipr = isptr[1]             # <<<<<<<<<<<<<<
 * 
 *     free(ivp)
 */
    __pyx_v_ipr = (__pyx_v_isptr[1]);
  }
  __pyx_L4_break:;

  /* "tables/indexesextension.pyx":351
 *         ipr = isptr[1]
 * 
 *     free(ivp)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  free(__pyx_v_ivp);

  /* "tables/indexesextension.pyx":212
 * 
 * @cython.cdivision(True)
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:             # <<<<<<<<<<<<<<
 *     cdef number_type *pl = start1
 *     cdef number_type *pr = start1 + (n - 1)
 */

  /* function exit code */
}

static void __pyx_fuse_9__pyx_f_6tables_16indexesextension__keysort(npy_float64 *__pyx_v_start1, char *__pyx_v_start2, size_t __pyx_v_elsize2, size_t __pyx_v_n) {
  npy_float64 *__pyx_v_pl;
  npy_float64 *__pyx_v_pr;
  char *__pyx_v_ipl;
  char *__pyx_v_ipr;
  npy_float64 __pyx_v_vp;
  char *__pyx_v_ivp;
  npy_float64 *__pyx_v_stack[0x64];
  npy_float64 **__pyx_v_sptr;
  char *__pyx_v_istack[0x64];
  char **__pyx_v_isptr;
  CYTHON_UNUSED size_t __pyx_v_stack_index;
  npy_float64 *__pyx_v_pm;
  npy_float64 *__pyx_v_pi;
  npy_float64 *__pyx_v_pj;
  npy_float64 *__pyx_v_pt;
  char *__pyx_v_ipm;
  char *__pyx_v_ipi;
  char *__pyx_v_ipj;
  char *__pyx_v_ipt;
  int __pyx_t_1;
  npy_float64 __pyx_t_2;
  npy_float64 __pyx_t_3;
  int __pyx_t_4;

  /* "tables/indexesextension.pyx":213
 * @cython.cdivision(True)
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:
 *     cdef number_type *pl = start1             # <<<<<<<<<<<<<<
 *     cdef number_type *pr = start1 + (n - 1)
 * 
 */
  __pyx_v_pl = __pyx_v_start1;

  /* "tables/indexesextension.pyx":214
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:
 *     cdef number_type *pl = start1
 *     cdef number_type *pr = start1 + (n - 1)             # <<<<<<<<<<<<<<
 * 
 *     cdef char *ipl = start2
 */
  __pyx_v_pr = (__pyx_v_start1 + (__pyx_v_n - 1));

  /* "tables/indexesextension.pyx":216
 *     cdef number_type *pr = start1 + (n - 1)
 * 
 *     cdef char *ipl = start2             # <<<<<<<<<<<<<<
 *     cdef char *ipr = start2 + (n - 1) * elsize2
 * 
 */
  __pyx_v_ipl = __pyx_v_start2;

  /* "tables/indexesextension.pyx":217
 * 
 *     cdef char *ipl = start2
 *     cdef char *ipr = start2 + (n - 1) * elsize2             # <<<<<<<<<<<<<<
 * 
 *     cdef number_type vp
 */
  __pyx_v_ipr = (__pyx_v_start2 + ((__pyx_v_n - 1) * __pyx_v_elsize2));

  /* "tables/indexesextension.pyx":220
 * 
 *     cdef number_type vp
 *     cdef char *ivp = <char *> malloc(elsize2)             # <<<<<<<<<<<<<<
 * 
 *     cdef number_type *stack[PYA_QS_STACK]
 */
  __pyx_v_ivp = ((char *)malloc(__pyx_v_elsize2));

  /* "tables/indexesextension.pyx":223
 * 
 *     cdef number_type *stack[PYA_QS_STACK]
 *     cdef number_type **sptr = stack             # <<<<<<<<<<<<<<
 * 
 *     cdef char *istack[PYA_QS_STACK]
 */
  __pyx_v_sptr = __pyx_v_stack;

  /* "tables/indexesextension.pyx":226
 * 
 *     cdef char *istack[PYA_QS_STACK]
 *     cdef char **isptr = istack             # <<<<<<<<<<<<<<
 * 
 *     cdef size_t stack_index = 0
 */
  __pyx_v_isptr = __pyx_v_istack;

  /* "tables/indexesextension.pyx":228
 *     cdef char **isptr = istack
 * 
 *     cdef size_t stack_index = 0             # <<<<<<<<<<<<<<
 * 
 *     cdef number_type *pm
 */
  __pyx_v_stack_index = 0;

  /* "tables/indexesextension.pyx":239
 *     cdef char *ipt
 * 
 *     while True:             # <<<<<<<<<<<<<<
 *         while pr - pl > SMALL_QUICKSORT:
 *             pm  = pl + ((pr - pl) >> 1)
 */
  while (1) {

    /* "tables/indexesextension.pyx":240
 * 
 *     while True:
 *         while pr - pl > SMALL_QUICKSORT:             # <<<<<<<<<<<<<<
 *             pm  = pl + ((pr - pl) >> 1)
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 */
    while (1) {
      __pyx_t_1 = (((__pyx_v_pr - __pyx_v_pl) > 15) != 0);
      if (!__pyx_t_1) break;

      /* "tables/indexesextension.pyx":241
 *     while True:
 *         while pr - pl > SMALL_QUICKSORT:
 *             pm  = pl + ((pr - pl) >> 1)             # <<<<<<<<<<<<<<
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 * 
 */
      __pyx_v_pm = (__pyx_v_pl + ((__pyx_v_pr - __pyx_v_pl) >> 1));

      /* "tables/indexesextension.pyx":242
 *         while pr - pl > SMALL_QUICKSORT:
 *             pm  = pl + ((pr - pl) >> 1)
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2             # <<<<<<<<<<<<<<
 * 
 *             if less_than(pm, pl):
 */
      __pyx_v_ipm = (__pyx_v_ipl + ((((__pyx_v_ipr - __pyx_v_ipl) / __pyx_v_elsize2) >> 1) * __pyx_v_elsize2));

      /* "tables/indexesextension.pyx":244
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      __pyx_t_1 = (__pyx_fuse_9__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pm, __pyx_v_pl) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":245
 * 
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 */
        __pyx_t_2 = (__pyx_v_pl[0]);
        __pyx_t_3 = (__pyx_v_pm[0]);
        (__pyx_v_pm[0]) = __pyx_t_2;
        (__pyx_v_pl[0]) = __pyx_t_3;

        /* "tables/indexesextension.pyx":246
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             if less_than(pr, pm):
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipl, __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":244
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      }

      /* "tables/indexesextension.pyx":248
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 *             if less_than(pr, pm):             # <<<<<<<<<<<<<<
 *                 pr[0], pm[0] =  pm[0], pr[0]
 *                 swap_bytes(ipr, ipm, elsize2)
 */
      __pyx_t_1 = (__pyx_fuse_9__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pr, __pyx_v_pm) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":249
 * 
 *             if less_than(pr, pm):
 *                 pr[0], pm[0] =  pm[0], pr[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipr, ipm, elsize2)
 * 
 */
        __pyx_t_3 = (__pyx_v_pm[0]);
        __pyx_t_2 = (__pyx_v_pr[0]);
        (__pyx_v_pr[0]) = __pyx_t_3;
        (__pyx_v_pm[0]) = __pyx_t_2;

        /* "tables/indexesextension.pyx":250
 *             if less_than(pr, pm):
 *                 pr[0], pm[0] =  pm[0], pr[0]
 *                 swap_bytes(ipr, ipm, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             if less_than(pm, pl):
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipr, __pyx_v_ipm, __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":248
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 *             if less_than(pr, pm):             # <<<<<<<<<<<<<<
 *                 pr[0], pm[0] =  pm[0], pr[0]
 *                 swap_bytes(ipr, ipm, elsize2)
 */
      }

      /* "tables/indexesextension.pyx":252
 *                 swap_bytes(ipr, ipm, elsize2)
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      __pyx_t_1 = (__pyx_fuse_9__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pm, __pyx_v_pl) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":253
 * 
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 */
        __pyx_t_2 = (__pyx_v_pl[0]);
        __pyx_t_3 = (__pyx_v_pm[0]);
        (__pyx_v_pm[0]) = __pyx_t_2;
        (__pyx_v_pl[0]) = __pyx_t_3;

        /* "tables/indexesextension.pyx":254
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             vp = pm[0]
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipl, __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":252
 *                 swap_bytes(ipr, ipm, elsize2)
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      }

      /* "tables/indexesextension.pyx":256
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 *             vp = pm[0]             # <<<<<<<<<<<<<<
 * 
 *             pi = pl
 */
      __pyx_v_vp = (__pyx_v_pm[0]);

      /* "tables/indexesextension.pyx":258
 *             vp = pm[0]
 * 
 *             pi = pl             # <<<<<<<<<<<<<<
 *             ipi = ipl
 * 
 */
      __pyx_v_pi = __pyx_v_pl;

      /* "tables/indexesextension.pyx":259
 * 
 *             pi = pl
 *             ipi = ipl             # <<<<<<<<<<<<<<
 * 
 *             pj = pr - 1
 */
      __pyx_v_ipi = __pyx_v_ipl;

      /* "tables/indexesextension.pyx":261
 *             ipi = ipl
 * 
 *             pj = pr - 1             # <<<<<<<<<<<<<<
 *             ipj = ipr - elsize2
 * 
 */
      __pyx_v_pj = (__pyx_v_pr - 1);

      /* "tables/indexesextension.pyx":262
 * 
 *             pj = pr - 1
 *             ipj = ipr - elsize2             # <<<<<<<<<<<<<<
 * 
 *             pm[0], pj[0] = pj[0], pm[0]
 */
      __pyx_v_ipj = (__pyx_v_ipr - __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":264
 *             ipj = ipr - elsize2
 * 
 *             pm[0], pj[0] = pj[0], pm[0]             # <<<<<<<<<<<<<<
 *             swap_bytes(ipm, ipj, elsize2)
 * 
 */
      __pyx_t_3 = (__pyx_v_pj[0]);
      __pyx_t_2 = (__pyx_v_pm[0]);
      (__pyx_v_pm[0]) = __pyx_t_3;
      (__pyx_v_pj[0]) = __pyx_t_2;

      /* "tables/indexesextension.pyx":265
 * 
 *             pm[0], pj[0] = pj[0], pm[0]
 *             swap_bytes(ipm, ipj, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             while True:
 */
      __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipj, __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":267
 *             swap_bytes(ipm, ipj, elsize2)
 * 
 *             while True:             # <<<<<<<<<<<<<<
 *                 pi += 1
 *                 ipi += elsize2
 */
      while (1) {

        /* "tables/indexesextension.pyx":268
 * 
 *             while True:
 *                 pi += 1             # <<<<<<<<<<<<<<
 *                 ipi += elsize2
 *                 while less_than(pi, &vp):
 */
        __pyx_v_pi = (__pyx_v_pi + 1);

        /* "tables/indexesextension.pyx":269
 *             while True:
 *                 pi += 1
 *                 ipi += elsize2             # <<<<<<<<<<<<<<
 *                 while less_than(pi, &vp):
 *                     pi += 1
 */
        __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":270
 *                 pi += 1
 *                 ipi += elsize2
 *                 while less_than(pi, &vp):             # <<<<<<<<<<<<<<
 *                     pi += 1
 *                     ipi += elsize2
 */
        while (1) {
          __pyx_t_1 = (__pyx_fuse_9__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pi, (&__pyx_v_vp)) != 0);
          if (!__pyx_t_1) break;

          /* "tables/indexesextension.pyx":271
 *                 ipi += elsize2
 *                 while less_than(pi, &vp):
 *                     pi += 1             # <<<<<<<<<<<<<<
 *                     ipi += elsize2
 * 
 */
          __pyx_v_pi = (__pyx_v_pi + 1);

          /* "tables/indexesextension.pyx":272
 *                 while less_than(pi, &vp):
 *                     pi += 1
 *                     ipi += elsize2             # <<<<<<<<<<<<<<
 * 
 *                 pj -= 1
 */
          __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_elsize2);
        }

        /* "tables/indexesextension.pyx":274
 *                     ipi += elsize2
 * 
 *                 pj -= 1             # <<<<<<<<<<<<<<
 *                 ipj -= elsize2
 *                 while less_than(&vp, pj):
 */
        __pyx_v_pj = (__pyx_v_pj - 1);

        /* "tables/indexesextension.pyx":275
 * 
 *                 pj -= 1
 *                 ipj -= elsize2             # <<<<<<<<<<<<<<
 *                 while less_than(&vp, pj):
 *                     pj -= 1
 */
        __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":276
 *                 pj -= 1
 *                 ipj -= elsize2
 *                 while less_than(&vp, pj):             # <<<<<<<<<<<<<<
 *                     pj -= 1
 *                     ipj -= elsize2
 */
        while (1) {
          __pyx_t_1 = (__pyx_fuse_9__pyx_f_6tables_16indexesextension_less_than((&__pyx_v_vp), __pyx_v_pj) != 0);
          if (!__pyx_t_1) break;

          /* "tables/indexesextension.pyx":277
 *                 ipj -= elsize2
 *                 while less_than(&vp, pj):
 *                     pj -= 1             # <<<<<<<<<<<<<<
 *                     ipj -= elsize2
 * 
 */
          __pyx_v_pj = (__pyx_v_pj - 1);

          /* "tables/indexesextension.pyx":278
 *                 while less_than(&vp, pj):
 *                     pj -= 1
 *                     ipj -= elsize2             # <<<<<<<<<<<<<<
 * 
 *                 if pi >= pj:
 */
          __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_elsize2);
        }

        /* "tables/indexesextension.pyx":280
 *                     ipj -= elsize2
 * 
 *                 if pi >= pj:             # <<<<<<<<<<<<<<
 *                     break
 * 
 */
        __pyx_t_1 = ((__pyx_v_pi >= __pyx_v_pj) != 0);
        if (__pyx_t_1) {

          /* "tables/indexesextension.pyx":281
 * 
 *                 if pi >= pj:
 *                     break             # <<<<<<<<<<<<<<
 * 
 *                 pi[0], pj[0] = pj[0], pi[0]
 */
          goto __pyx_L11_break;

          /* "tables/indexesextension.pyx":280
 *                     ipj -= elsize2
 * 
 *                 if pi >= pj:             # <<<<<<<<<<<<<<
 *                     break
 * 
 */
        }

        /* "tables/indexesextension.pyx":283
 *                     break
 * 
 *                 pi[0], pj[0] = pj[0], pi[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipi, ipj, elsize2)
 * 
 */
        __pyx_t_2 = (__pyx_v_pj[0]);
        __pyx_t_3 = (__pyx_v_pi[0]);
        (__pyx_v_pi[0]) = __pyx_t_2;
        (__pyx_v_pj[0]) = __pyx_t_3;

        /* "tables/indexesextension.pyx":284
 * 
 *                 pi[0], pj[0] = pj[0], pi[0]
 *                 swap_bytes(ipi, ipj, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             pi[0], (pr-1)[0] = (pr-1)[0], pi[0]
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipi, __pyx_v_ipj, __pyx_v_elsize2);
      }
      __pyx_L11_break:;

      /* "tables/indexesextension.pyx":286
 *                 swap_bytes(ipi, ipj, elsize2)
 * 
 *             pi[0], (pr-1)[0] = (pr-1)[0], pi[0]             # <<<<<<<<<<<<<<
 *             swap_bytes(ipi, ipr-elsize2, elsize2)
 * 
 */
      __pyx_t_3 = ((__pyx_v_pr - 1)[0]);
      __pyx_t_2 = (__pyx_v_pi[0]);
      (__pyx_v_pi[0]) = __pyx_t_3;
      ((__pyx_v_pr - 1)[0]) = __pyx_t_2;

      /* "tables/indexesextension.pyx":287
 * 
 *             pi[0], (pr-1)[0] = (pr-1)[0], pi[0]
 *             swap_bytes(ipi, ipr-elsize2, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             # push largest partition on stack and proceed with the other
 */
      __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipi, (__pyx_v_ipr - __pyx_v_elsize2), __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":290
 * 
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):             # <<<<<<<<<<<<<<
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr
 */
      __pyx_t_1 = (((__pyx_v_pi - __pyx_v_pl) < (__pyx_v_pr - __pyx_v_pi)) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":291
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):
 *                 sptr[0] = pi + 1             # <<<<<<<<<<<<<<
 *                 sptr[1] = pr
 *                 sptr += 2
 */
        (__pyx_v_sptr[0]) = (__pyx_v_pi + 1);

        /* "tables/indexesextension.pyx":292
 *             if (pi - pl) < (pr - pi):
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr             # <<<<<<<<<<<<<<
 *                 sptr += 2
 * 
 */
        (__pyx_v_sptr[1]) = __pyx_v_pr;

        /* "tables/indexesextension.pyx":293
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr
 *                 sptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 isptr[0] = ipi + elsize2
 */
        __pyx_v_sptr = (__pyx_v_sptr + 2);

        /* "tables/indexesextension.pyx":295
 *                 sptr += 2
 * 
 *                 isptr[0] = ipi + elsize2             # <<<<<<<<<<<<<<
 *                 isptr[1] = ipr
 *                 isptr += 2
 */
        (__pyx_v_isptr[0]) = (__pyx_v_ipi + __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":296
 * 
 *                 isptr[0] = ipi + elsize2
 *                 isptr[1] = ipr             # <<<<<<<<<<<<<<
 *                 isptr += 2
 * 
 */
        (__pyx_v_isptr[1]) = __pyx_v_ipr;

        /* "tables/indexesextension.pyx":297
 *                 isptr[0] = ipi + elsize2
 *                 isptr[1] = ipr
 *                 isptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 pr = pi - 1
 */
        __pyx_v_isptr = (__pyx_v_isptr + 2);

        /* "tables/indexesextension.pyx":299
 *                 isptr += 2
 * 
 *                 pr = pi - 1             # <<<<<<<<<<<<<<
 *                 ipr = ipi - elsize2
 *             else:
 */
        __pyx_v_pr = (__pyx_v_pi - 1);

        /* "tables/indexesextension.pyx":300
 * 
 *                 pr = pi - 1
 *                 ipr = ipi - elsize2             # <<<<<<<<<<<<<<
 *             else:
 *                 sptr[0] = pl
 */
        __pyx_v_ipr = (__pyx_v_ipi - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":290
 * 
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):             # <<<<<<<<<<<<<<
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr
 */
        goto __pyx_L17;
      }

      /* "tables/indexesextension.pyx":302
 *                 ipr = ipi - elsize2
 *             else:
 *                 sptr[0] = pl             # <<<<<<<<<<<<<<
 *                 sptr[1] = pi - 1
 *                 sptr += 2
 */
      /*else*/ {
        (__pyx_v_sptr[0]) = __pyx_v_pl;

        /* "tables/indexesextension.pyx":303
 *             else:
 *                 sptr[0] = pl
 *                 sptr[1] = pi - 1             # <<<<<<<<<<<<<<
 *                 sptr += 2
 * 
 */
        (__pyx_v_sptr[1]) = (__pyx_v_pi - 1);

        /* "tables/indexesextension.pyx":304
 *                 sptr[0] = pl
 *                 sptr[1] = pi - 1
 *                 sptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 isptr[0] = ipl
 */
        __pyx_v_sptr = (__pyx_v_sptr + 2);

        /* "tables/indexesextension.pyx":306
 *                 sptr += 2
 * 
 *                 isptr[0] = ipl             # <<<<<<<<<<<<<<
 *                 isptr[1] = ipi - elsize2
 *                 isptr += 2
 */
        (__pyx_v_isptr[0]) = __pyx_v_ipl;

        /* "tables/indexesextension.pyx":307
 * 
 *                 isptr[0] = ipl
 *                 isptr[1] = ipi - elsize2             # <<<<<<<<<<<<<<
 *                 isptr += 2
 * 
 */
        (__pyx_v_isptr[1]) = (__pyx_v_ipi - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":308
 *                 isptr[0] = ipl
 *                 isptr[1] = ipi - elsize2
 *                 isptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 pl = pi + 1
 */
        __pyx_v_isptr = (__pyx_v_isptr + 2);

        /* "tables/indexesextension.pyx":310
 *                 isptr += 2
 * 
 *                 pl = pi + 1             # <<<<<<<<<<<<<<
 *                 ipl = ipi + elsize2
 * 
 */
        __pyx_v_pl = (__pyx_v_pi + 1);

        /* "tables/indexesextension.pyx":311
 * 
 *                 pl = pi + 1
 *                 ipl = ipi + elsize2             # <<<<<<<<<<<<<<
 * 
 *         pi = pl + 1
 */
        __pyx_v_ipl = (__pyx_v_ipi + __pyx_v_elsize2);
      }
      __pyx_L17:;
    }

    /* "tables/indexesextension.pyx":313
 *                 ipl = ipi + elsize2
 * 
 *         pi = pl + 1             # <<<<<<<<<<<<<<
 *         ipi = ipl + elsize2
 *         while pi <= pr:
 */
    __pyx_v_pi = (__pyx_v_pl + 1);

    /* "tables/indexesextension.pyx":314
 * 
 *         pi = pl + 1
 *         ipi = ipl + elsize2             # <<<<<<<<<<<<<<
 *         while pi <= pr:
 *             vp = pi[0]
 */
    __pyx_v_ipi = (__pyx_v_ipl + __pyx_v_elsize2);

    /* "tables/indexesextension.pyx":315
 *         pi = pl + 1
 *         ipi = ipl + elsize2
 *         while pi <= pr:             # <<<<<<<<<<<<<<
 *             vp = pi[0]
 *             memcpy(ivp, ipi, elsize2)
 */
    while (1) {
      __pyx_t_1 = ((__pyx_v_pi <= __pyx_v_pr) != 0);
      if (!__pyx_t_1) break;

      /* "tables/indexesextension.pyx":316
 *         ipi = ipl + elsize2
 *         while pi <= pr:
 *             vp = pi[0]             # <<<<<<<<<<<<<<
 *             memcpy(ivp, ipi, elsize2)
 * 
 */
      __pyx_v_vp = (__pyx_v_pi[0]);

      /* "tables/indexesextension.pyx":317
 *         while pi <= pr:
 *             vp = pi[0]
 *             memcpy(ivp, ipi, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             pj = pi
 */
      (void)(memcpy(__pyx_v_ivp, __pyx_v_ipi, __pyx_v_elsize2));

      /* "tables/indexesextension.pyx":319
 *             memcpy(ivp, ipi, elsize2)
 * 
 *             pj = pi             # <<<<<<<<<<<<<<
 *             pt = pi - 1
 * 
 */
      __pyx_v_pj = __pyx_v_pi;

      /* "tables/indexesextension.pyx":320
 * 
 *             pj = pi
 *             pt = pi - 1             # <<<<<<<<<<<<<<
 * 
 *             ipj = ipi
 */
      __pyx_v_pt = (__pyx_v_pi - 1);

      /* "tables/indexesextension.pyx":322
 *             pt = pi - 1
 * 
 *             ipj = ipi             # <<<<<<<<<<<<<<
 *             ipt = ipi - elsize2
 * 
 */
      __pyx_v_ipj = __pyx_v_ipi;

      /* "tables/indexesextension.pyx":323
 * 
 *             ipj = ipi
 *             ipt = ipi - elsize2             # <<<<<<<<<<<<<<
 * 
 *             while pj > pl and less_than(&vp, pt):
 */
      __pyx_v_ipt = (__pyx_v_ipi - __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":325
 *             ipt = ipi - elsize2
 * 
 *             while pj > pl and less_than(&vp, pt):             # <<<<<<<<<<<<<<
 *                 pj[0] = pt[0]
 *                 pj -= 1
 */
      while (1) {
        __pyx_t_4 = ((__pyx_v_pj > __pyx_v_pl) != 0);
        if (__pyx_t_4) {
        } else {
          __pyx_t_1 = __pyx_t_4;
          goto __pyx_L22_bool_binop_done;
        }
        __pyx_t_4 = (__pyx_fuse_9__pyx_f_6tables_16indexesextension_less_than((&__pyx_v_vp), __pyx_v_pt) != 0);
        __pyx_t_1 = __pyx_t_4;
        __pyx_L22_bool_binop_done:;
        if (!__pyx_t_1) break;

        /* "tables/indexesextension.pyx":326
 * 
 *             while pj > pl and less_than(&vp, pt):
 *                 pj[0] = pt[0]             # <<<<<<<<<<<<<<
 *                 pj -= 1
 *                 pt -= 1
 */
        (__pyx_v_pj[0]) = (__pyx_v_pt[0]);

        /* "tables/indexesextension.pyx":327
 *             while pj > pl and less_than(&vp, pt):
 *                 pj[0] = pt[0]
 *                 pj -= 1             # <<<<<<<<<<<<<<
 *                 pt -= 1
 * 
 */
        __pyx_v_pj = (__pyx_v_pj - 1);

        /* "tables/indexesextension.pyx":328
 *                 pj[0] = pt[0]
 *                 pj -= 1
 *                 pt -= 1             # <<<<<<<<<<<<<<
 * 
 *                 memcpy(ipj, ipt, elsize2)
 */
        __pyx_v_pt = (__pyx_v_pt - 1);

        /* "tables/indexesextension.pyx":330
 *                 pt -= 1
 * 
 *                 memcpy(ipj, ipt, elsize2)             # <<<<<<<<<<<<<<
 *                 ipj -= elsize2
 *                 ipt -= elsize2
 */
        (void)(memcpy(__pyx_v_ipj, __pyx_v_ipt, __pyx_v_elsize2));

        /* "tables/indexesextension.pyx":331
 * 
 *                 memcpy(ipj, ipt, elsize2)
 *                 ipj -= elsize2             # <<<<<<<<<<<<<<
 *                 ipt -= elsize2
 * 
 */
        __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":332
 *                 memcpy(ipj, ipt, elsize2)
 *                 ipj -= elsize2
 *                 ipt -= elsize2             # <<<<<<<<<<<<<<
 * 
 *             pj[0] = vp
 */
        __pyx_v_ipt = (__pyx_v_ipt - __pyx_v_elsize2);
      }

      /* "tables/indexesextension.pyx":334
 *                 ipt -= elsize2
 * 
 *             pj[0] = vp             # <<<<<<<<<<<<<<
 *             memcpy(ipj, ivp, elsize2)
 * 
 */
      (__pyx_v_pj[0]) = __pyx_v_vp;

      /* "tables/indexesextension.pyx":335
 * 
 *             pj[0] = vp
 *             memcpy(ipj, ivp, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             pi += 1
 */
      (void)(memcpy(__pyx_v_ipj, __pyx_v_ivp, __pyx_v_elsize2));

      /* "tables/indexesextension.pyx":337
 *             memcpy(ipj, ivp, elsize2)
 * 
 *             pi += 1             # <<<<<<<<<<<<<<
 *             ipi += elsize2
 * 
 */
      __pyx_v_pi = (__pyx_v_pi + 1);

      /* "tables/indexesextension.pyx":338
 * 
 *             pi += 1
 *             ipi += elsize2             # <<<<<<<<<<<<<<
 * 
 *         if sptr == stack:
 */
      __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_elsize2);
    }

    /* "tables/indexesextension.pyx":340
 *             ipi += elsize2
 * 
 *         if sptr == stack:             # <<<<<<<<<<<<<<
 *             break
 * 
 */
    __pyx_t_1 = ((__pyx_v_sptr == __pyx_v_stack) != 0);
    if (__pyx_t_1) {

      /* "tables/indexesextension.pyx":341
 * 
 *         if sptr == stack:
 *             break             # <<<<<<<<<<<<<<
 * 
 *         sptr -= 2
 */
      goto __pyx_L4_break;

      /* "tables/indexesextension.pyx":340
 *             ipi += elsize2
 * 
 *         if sptr == stack:             # <<<<<<<<<<<<<<
 *             break
 * 
 */
    }

    /* "tables/indexesextension.pyx":343
 *             break
 * 
 *         sptr -= 2             # <<<<<<<<<<<<<<
 *         pl = sptr[0]
 *         pr = sptr[1]
 */
    __pyx_v_sptr = (__pyx_v_sptr - 2);

    /* "tables/indexesextension.pyx":344
 * 
 *         sptr -= 2
 *         pl = sptr[0]             # <<<<<<<<<<<<<<
 *         pr = sptr[1]
 * 
 */
    __pyx_v_pl = (__pyx_v_sptr[0]);

    /* "tables/indexesextension.pyx":345
 *         sptr -= 2
 *         pl = sptr[0]
 *         pr = sptr[1]             # <<<<<<<<<<<<<<
 * 
 *         isptr -= 2
 */
    __pyx_v_pr = (__pyx_v_sptr[1]);

    /* "tables/indexesextension.pyx":347
 *         pr = sptr[1]
 * 
 *         isptr -= 2             # <<<<<<<<<<<<<<
 *         ipl = isptr[0]
 *         ipr = isptr[1]
 */
    __pyx_v_isptr = (__pyx_v_isptr - 2);

    /* "tables/indexesextension.pyx":348
 * 
 *         isptr -= 2
 *         ipl = isptr[0]             # <<<<<<<<<<<<<<
 *         ipr = isptr[1]
 * 
 */
    __pyx_v_ipl = (__pyx_v_isptr[0]);

    /* "tables/indexesextension.pyx":349
 *         isptr -= 2
 *         ipl = isptr[0]
 *         ipr = isptr[1]             # <<<<<<<<<<<<<<
 * 
 *     free(ivp)
 */
    __pyx_v_ipr = (__pyx_v_isptr[1]);
  }
  __pyx_L4_break:;

  /* "tables/indexesextension.pyx":351
 *         ipr = isptr[1]
 * 
 *     free(ivp)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  free(__pyx_v_ivp);

  /* "tables/indexesextension.pyx":212
 * 
 * @cython.cdivision(True)
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:             # <<<<<<<<<<<<<<
 *     cdef number_type *pl = start1
 *     cdef number_type *pr = start1 + (n - 1)
 */

  /* function exit code */
}

static void __pyx_fuse_10__pyx_f_6tables_16indexesextension__keysort(npy_longdouble *__pyx_v_start1, char *__pyx_v_start2, size_t __pyx_v_elsize2, size_t __pyx_v_n) {
  npy_longdouble *__pyx_v_pl;
  npy_longdouble *__pyx_v_pr;
  char *__pyx_v_ipl;
  char *__pyx_v_ipr;
  npy_longdouble __pyx_v_vp;
  char *__pyx_v_ivp;
  npy_longdouble *__pyx_v_stack[0x64];
  npy_longdouble **__pyx_v_sptr;
  char *__pyx_v_istack[0x64];
  char **__pyx_v_isptr;
  CYTHON_UNUSED size_t __pyx_v_stack_index;
  npy_longdouble *__pyx_v_pm;
  npy_longdouble *__pyx_v_pi;
  npy_longdouble *__pyx_v_pj;
  npy_longdouble *__pyx_v_pt;
  char *__pyx_v_ipm;
  char *__pyx_v_ipi;
  char *__pyx_v_ipj;
  char *__pyx_v_ipt;
  int __pyx_t_1;
  npy_longdouble __pyx_t_2;
  npy_longdouble __pyx_t_3;
  int __pyx_t_4;

  /* "tables/indexesextension.pyx":213
 * @cython.cdivision(True)
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:
 *     cdef number_type *pl = start1             # <<<<<<<<<<<<<<
 *     cdef number_type *pr = start1 + (n - 1)
 * 
 */
  __pyx_v_pl = __pyx_v_start1;

  /* "tables/indexesextension.pyx":214
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:
 *     cdef number_type *pl = start1
 *     cdef number_type *pr = start1 + (n - 1)             # <<<<<<<<<<<<<<
 * 
 *     cdef char *ipl = start2
 */
  __pyx_v_pr = (__pyx_v_start1 + (__pyx_v_n - 1));

  /* "tables/indexesextension.pyx":216
 *     cdef number_type *pr = start1 + (n - 1)
 * 
 *     cdef char *ipl = start2             # <<<<<<<<<<<<<<
 *     cdef char *ipr = start2 + (n - 1) * elsize2
 * 
 */
  __pyx_v_ipl = __pyx_v_start2;

  /* "tables/indexesextension.pyx":217
 * 
 *     cdef char *ipl = start2
 *     cdef char *ipr = start2 + (n - 1) * elsize2             # <<<<<<<<<<<<<<
 * 
 *     cdef number_type vp
 */
  __pyx_v_ipr = (__pyx_v_start2 + ((__pyx_v_n - 1) * __pyx_v_elsize2));

  /* "tables/indexesextension.pyx":220
 * 
 *     cdef number_type vp
 *     cdef char *ivp = <char *> malloc(elsize2)             # <<<<<<<<<<<<<<
 * 
 *     cdef number_type *stack[PYA_QS_STACK]
 */
  __pyx_v_ivp = ((char *)malloc(__pyx_v_elsize2));

  /* "tables/indexesextension.pyx":223
 * 
 *     cdef number_type *stack[PYA_QS_STACK]
 *     cdef number_type **sptr = stack             # <<<<<<<<<<<<<<
 * 
 *     cdef char *istack[PYA_QS_STACK]
 */
  __pyx_v_sptr = __pyx_v_stack;

  /* "tables/indexesextension.pyx":226
 * 
 *     cdef char *istack[PYA_QS_STACK]
 *     cdef char **isptr = istack             # <<<<<<<<<<<<<<
 * 
 *     cdef size_t stack_index = 0
 */
  __pyx_v_isptr = __pyx_v_istack;

  /* "tables/indexesextension.pyx":228
 *     cdef char **isptr = istack
 * 
 *     cdef size_t stack_index = 0             # <<<<<<<<<<<<<<
 * 
 *     cdef number_type *pm
 */
  __pyx_v_stack_index = 0;

  /* "tables/indexesextension.pyx":239
 *     cdef char *ipt
 * 
 *     while True:             # <<<<<<<<<<<<<<
 *         while pr - pl > SMALL_QUICKSORT:
 *             pm  = pl + ((pr - pl) >> 1)
 */
  while (1) {

    /* "tables/indexesextension.pyx":240
 * 
 *     while True:
 *         while pr - pl > SMALL_QUICKSORT:             # <<<<<<<<<<<<<<
 *             pm  = pl + ((pr - pl) >> 1)
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 */
    while (1) {
      __pyx_t_1 = (((__pyx_v_pr - __pyx_v_pl) > 15) != 0);
      if (!__pyx_t_1) break;

      /* "tables/indexesextension.pyx":241
 *     while True:
 *         while pr - pl > SMALL_QUICKSORT:
 *             pm  = pl + ((pr - pl) >> 1)             # <<<<<<<<<<<<<<
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 * 
 */
      __pyx_v_pm = (__pyx_v_pl + ((__pyx_v_pr - __pyx_v_pl) >> 1));

      /* "tables/indexesextension.pyx":242
 *         while pr - pl > SMALL_QUICKSORT:
 *             pm  = pl + ((pr - pl) >> 1)
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2             # <<<<<<<<<<<<<<
 * 
 *             if less_than(pm, pl):
 */
      __pyx_v_ipm = (__pyx_v_ipl + ((((__pyx_v_ipr - __pyx_v_ipl) / __pyx_v_elsize2) >> 1) * __pyx_v_elsize2));

      /* "tables/indexesextension.pyx":244
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      __pyx_t_1 = (__pyx_fuse_10__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pm, __pyx_v_pl) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":245
 * 
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 */
        __pyx_t_2 = (__pyx_v_pl[0]);
        __pyx_t_3 = (__pyx_v_pm[0]);
        (__pyx_v_pm[0]) = __pyx_t_2;
        (__pyx_v_pl[0]) = __pyx_t_3;

        /* "tables/indexesextension.pyx":246
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             if less_than(pr, pm):
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipl, __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":244
 *             ipm  = ipl + ((ipr - ipl)//elsize2 >> 1)*elsize2
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      }

      /* "tables/indexesextension.pyx":248
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 *             if less_than(pr, pm):             # <<<<<<<<<<<<<<
 *                 pr[0], pm[0] =  pm[0], pr[0]
 *                 swap_bytes(ipr, ipm, elsize2)
 */
      __pyx_t_1 = (__pyx_fuse_10__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pr, __pyx_v_pm) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":249
 * 
 *             if less_than(pr, pm):
 *                 pr[0], pm[0] =  pm[0], pr[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipr, ipm, elsize2)
 * 
 */
        __pyx_t_3 = (__pyx_v_pm[0]);
        __pyx_t_2 = (__pyx_v_pr[0]);
        (__pyx_v_pr[0]) = __pyx_t_3;
        (__pyx_v_pm[0]) = __pyx_t_2;

        /* "tables/indexesextension.pyx":250
 *             if less_than(pr, pm):
 *                 pr[0], pm[0] =  pm[0], pr[0]
 *                 swap_bytes(ipr, ipm, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             if less_than(pm, pl):
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipr, __pyx_v_ipm, __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":248
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 *             if less_than(pr, pm):             # <<<<<<<<<<<<<<
 *                 pr[0], pm[0] =  pm[0], pr[0]
 *                 swap_bytes(ipr, ipm, elsize2)
 */
      }

      /* "tables/indexesextension.pyx":252
 *                 swap_bytes(ipr, ipm, elsize2)
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      __pyx_t_1 = (__pyx_fuse_10__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pm, __pyx_v_pl) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":253
 * 
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 */
        __pyx_t_2 = (__pyx_v_pl[0]);
        __pyx_t_3 = (__pyx_v_pm[0]);
        (__pyx_v_pm[0]) = __pyx_t_2;
        (__pyx_v_pl[0]) = __pyx_t_3;

        /* "tables/indexesextension.pyx":254
 *             if less_than(pm, pl):
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             vp = pm[0]
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipl, __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":252
 *                 swap_bytes(ipr, ipm, elsize2)
 * 
 *             if less_than(pm, pl):             # <<<<<<<<<<<<<<
 *                 pm[0], pl[0] =  pl[0], pm[0]
 *                 swap_bytes(ipm, ipl, elsize2)
 */
      }

      /* "tables/indexesextension.pyx":256
 *                 swap_bytes(ipm, ipl, elsize2)
 * 
 *             vp = pm[0]             # <<<<<<<<<<<<<<
 * 
 *             pi = pl
 */
      __pyx_v_vp = (__pyx_v_pm[0]);

      /* "tables/indexesextension.pyx":258
 *             vp = pm[0]
 * 
 *             pi = pl             # <<<<<<<<<<<<<<
 *             ipi = ipl
 * 
 */
      __pyx_v_pi = __pyx_v_pl;

      /* "tables/indexesextension.pyx":259
 * 
 *             pi = pl
 *             ipi = ipl             # <<<<<<<<<<<<<<
 * 
 *             pj = pr - 1
 */
      __pyx_v_ipi = __pyx_v_ipl;

      /* "tables/indexesextension.pyx":261
 *             ipi = ipl
 * 
 *             pj = pr - 1             # <<<<<<<<<<<<<<
 *             ipj = ipr - elsize2
 * 
 */
      __pyx_v_pj = (__pyx_v_pr - 1);

      /* "tables/indexesextension.pyx":262
 * 
 *             pj = pr - 1
 *             ipj = ipr - elsize2             # <<<<<<<<<<<<<<
 * 
 *             pm[0], pj[0] = pj[0], pm[0]
 */
      __pyx_v_ipj = (__pyx_v_ipr - __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":264
 *             ipj = ipr - elsize2
 * 
 *             pm[0], pj[0] = pj[0], pm[0]             # <<<<<<<<<<<<<<
 *             swap_bytes(ipm, ipj, elsize2)
 * 
 */
      __pyx_t_3 = (__pyx_v_pj[0]);
      __pyx_t_2 = (__pyx_v_pm[0]);
      (__pyx_v_pm[0]) = __pyx_t_3;
      (__pyx_v_pj[0]) = __pyx_t_2;

      /* "tables/indexesextension.pyx":265
 * 
 *             pm[0], pj[0] = pj[0], pm[0]
 *             swap_bytes(ipm, ipj, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             while True:
 */
      __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipj, __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":267
 *             swap_bytes(ipm, ipj, elsize2)
 * 
 *             while True:             # <<<<<<<<<<<<<<
 *                 pi += 1
 *                 ipi += elsize2
 */
      while (1) {

        /* "tables/indexesextension.pyx":268
 * 
 *             while True:
 *                 pi += 1             # <<<<<<<<<<<<<<
 *                 ipi += elsize2
 *                 while less_than(pi, &vp):
 */
        __pyx_v_pi = (__pyx_v_pi + 1);

        /* "tables/indexesextension.pyx":269
 *             while True:
 *                 pi += 1
 *                 ipi += elsize2             # <<<<<<<<<<<<<<
 *                 while less_than(pi, &vp):
 *                     pi += 1
 */
        __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":270
 *                 pi += 1
 *                 ipi += elsize2
 *                 while less_than(pi, &vp):             # <<<<<<<<<<<<<<
 *                     pi += 1
 *                     ipi += elsize2
 */
        while (1) {
          __pyx_t_1 = (__pyx_fuse_10__pyx_f_6tables_16indexesextension_less_than(__pyx_v_pi, (&__pyx_v_vp)) != 0);
          if (!__pyx_t_1) break;

          /* "tables/indexesextension.pyx":271
 *                 ipi += elsize2
 *                 while less_than(pi, &vp):
 *                     pi += 1             # <<<<<<<<<<<<<<
 *                     ipi += elsize2
 * 
 */
          __pyx_v_pi = (__pyx_v_pi + 1);

          /* "tables/indexesextension.pyx":272
 *                 while less_than(pi, &vp):
 *                     pi += 1
 *                     ipi += elsize2             # <<<<<<<<<<<<<<
 * 
 *                 pj -= 1
 */
          __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_elsize2);
        }

        /* "tables/indexesextension.pyx":274
 *                     ipi += elsize2
 * 
 *                 pj -= 1             # <<<<<<<<<<<<<<
 *                 ipj -= elsize2
 *                 while less_than(&vp, pj):
 */
        __pyx_v_pj = (__pyx_v_pj - 1);

        /* "tables/indexesextension.pyx":275
 * 
 *                 pj -= 1
 *                 ipj -= elsize2             # <<<<<<<<<<<<<<
 *                 while less_than(&vp, pj):
 *                     pj -= 1
 */
        __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":276
 *                 pj -= 1
 *                 ipj -= elsize2
 *                 while less_than(&vp, pj):             # <<<<<<<<<<<<<<
 *                     pj -= 1
 *                     ipj -= elsize2
 */
        while (1) {
          __pyx_t_1 = (__pyx_fuse_10__pyx_f_6tables_16indexesextension_less_than((&__pyx_v_vp), __pyx_v_pj) != 0);
          if (!__pyx_t_1) break;

          /* "tables/indexesextension.pyx":277
 *                 ipj -= elsize2
 *                 while less_than(&vp, pj):
 *                     pj -= 1             # <<<<<<<<<<<<<<
 *                     ipj -= elsize2
 * 
 */
          __pyx_v_pj = (__pyx_v_pj - 1);

          /* "tables/indexesextension.pyx":278
 *                 while less_than(&vp, pj):
 *                     pj -= 1
 *                     ipj -= elsize2             # <<<<<<<<<<<<<<
 * 
 *                 if pi >= pj:
 */
          __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_elsize2);
        }

        /* "tables/indexesextension.pyx":280
 *                     ipj -= elsize2
 * 
 *                 if pi >= pj:             # <<<<<<<<<<<<<<
 *                     break
 * 
 */
        __pyx_t_1 = ((__pyx_v_pi >= __pyx_v_pj) != 0);
        if (__pyx_t_1) {

          /* "tables/indexesextension.pyx":281
 * 
 *                 if pi >= pj:
 *                     break             # <<<<<<<<<<<<<<
 * 
 *                 pi[0], pj[0] = pj[0], pi[0]
 */
          goto __pyx_L11_break;

          /* "tables/indexesextension.pyx":280
 *                     ipj -= elsize2
 * 
 *                 if pi >= pj:             # <<<<<<<<<<<<<<
 *                     break
 * 
 */
        }

        /* "tables/indexesextension.pyx":283
 *                     break
 * 
 *                 pi[0], pj[0] = pj[0], pi[0]             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipi, ipj, elsize2)
 * 
 */
        __pyx_t_2 = (__pyx_v_pj[0]);
        __pyx_t_3 = (__pyx_v_pi[0]);
        (__pyx_v_pi[0]) = __pyx_t_2;
        (__pyx_v_pj[0]) = __pyx_t_3;

        /* "tables/indexesextension.pyx":284
 * 
 *                 pi[0], pj[0] = pj[0], pi[0]
 *                 swap_bytes(ipi, ipj, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             pi[0], (pr-1)[0] = (pr-1)[0], pi[0]
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipi, __pyx_v_ipj, __pyx_v_elsize2);
      }
      __pyx_L11_break:;

      /* "tables/indexesextension.pyx":286
 *                 swap_bytes(ipi, ipj, elsize2)
 * 
 *             pi[0], (pr-1)[0] = (pr-1)[0], pi[0]             # <<<<<<<<<<<<<<
 *             swap_bytes(ipi, ipr-elsize2, elsize2)
 * 
 */
      __pyx_t_3 = ((__pyx_v_pr - 1)[0]);
      __pyx_t_2 = (__pyx_v_pi[0]);
      (__pyx_v_pi[0]) = __pyx_t_3;
      ((__pyx_v_pr - 1)[0]) = __pyx_t_2;

      /* "tables/indexesextension.pyx":287
 * 
 *             pi[0], (pr-1)[0] = (pr-1)[0], pi[0]
 *             swap_bytes(ipi, ipr-elsize2, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             # push largest partition on stack and proceed with the other
 */
      __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipi, (__pyx_v_ipr - __pyx_v_elsize2), __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":290
 * 
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):             # <<<<<<<<<<<<<<
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr
 */
      __pyx_t_1 = (((__pyx_v_pi - __pyx_v_pl) < (__pyx_v_pr - __pyx_v_pi)) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":291
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):
 *                 sptr[0] = pi + 1             # <<<<<<<<<<<<<<
 *                 sptr[1] = pr
 *                 sptr += 2
 */
        (__pyx_v_sptr[0]) = (__pyx_v_pi + 1);

        /* "tables/indexesextension.pyx":292
 *             if (pi - pl) < (pr - pi):
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr             # <<<<<<<<<<<<<<
 *                 sptr += 2
 * 
 */
        (__pyx_v_sptr[1]) = __pyx_v_pr;

        /* "tables/indexesextension.pyx":293
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr
 *                 sptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 isptr[0] = ipi + elsize2
 */
        __pyx_v_sptr = (__pyx_v_sptr + 2);

        /* "tables/indexesextension.pyx":295
 *                 sptr += 2
 * 
 *                 isptr[0] = ipi + elsize2             # <<<<<<<<<<<<<<
 *                 isptr[1] = ipr
 *                 isptr += 2
 */
        (__pyx_v_isptr[0]) = (__pyx_v_ipi + __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":296
 * 
 *                 isptr[0] = ipi + elsize2
 *                 isptr[1] = ipr             # <<<<<<<<<<<<<<
 *                 isptr += 2
 * 
 */
        (__pyx_v_isptr[1]) = __pyx_v_ipr;

        /* "tables/indexesextension.pyx":297
 *                 isptr[0] = ipi + elsize2
 *                 isptr[1] = ipr
 *                 isptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 pr = pi - 1
 */
        __pyx_v_isptr = (__pyx_v_isptr + 2);

        /* "tables/indexesextension.pyx":299
 *                 isptr += 2
 * 
 *                 pr = pi - 1             # <<<<<<<<<<<<<<
 *                 ipr = ipi - elsize2
 *             else:
 */
        __pyx_v_pr = (__pyx_v_pi - 1);

        /* "tables/indexesextension.pyx":300
 * 
 *                 pr = pi - 1
 *                 ipr = ipi - elsize2             # <<<<<<<<<<<<<<
 *             else:
 *                 sptr[0] = pl
 */
        __pyx_v_ipr = (__pyx_v_ipi - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":290
 * 
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):             # <<<<<<<<<<<<<<
 *                 sptr[0] = pi + 1
 *                 sptr[1] = pr
 */
        goto __pyx_L17;
      }

      /* "tables/indexesextension.pyx":302
 *                 ipr = ipi - elsize2
 *             else:
 *                 sptr[0] = pl             # <<<<<<<<<<<<<<
 *                 sptr[1] = pi - 1
 *                 sptr += 2
 */
      /*else*/ {
        (__pyx_v_sptr[0]) = __pyx_v_pl;

        /* "tables/indexesextension.pyx":303
 *             else:
 *                 sptr[0] = pl
 *                 sptr[1] = pi - 1             # <<<<<<<<<<<<<<
 *                 sptr += 2
 * 
 */
        (__pyx_v_sptr[1]) = (__pyx_v_pi - 1);

        /* "tables/indexesextension.pyx":304
 *                 sptr[0] = pl
 *                 sptr[1] = pi - 1
 *                 sptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 isptr[0] = ipl
 */
        __pyx_v_sptr = (__pyx_v_sptr + 2);

        /* "tables/indexesextension.pyx":306
 *                 sptr += 2
 * 
 *                 isptr[0] = ipl             # <<<<<<<<<<<<<<
 *                 isptr[1] = ipi - elsize2
 *                 isptr += 2
 */
        (__pyx_v_isptr[0]) = __pyx_v_ipl;

        /* "tables/indexesextension.pyx":307
 * 
 *                 isptr[0] = ipl
 *                 isptr[1] = ipi - elsize2             # <<<<<<<<<<<<<<
 *                 isptr += 2
 * 
 */
        (__pyx_v_isptr[1]) = (__pyx_v_ipi - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":308
 *                 isptr[0] = ipl
 *                 isptr[1] = ipi - elsize2
 *                 isptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 pl = pi + 1
 */
        __pyx_v_isptr = (__pyx_v_isptr + 2);

        /* "tables/indexesextension.pyx":310
 *                 isptr += 2
 * 
 *                 pl = pi + 1             # <<<<<<<<<<<<<<
 *                 ipl = ipi + elsize2
 * 
 */
        __pyx_v_pl = (__pyx_v_pi + 1);

        /* "tables/indexesextension.pyx":311
 * 
 *                 pl = pi + 1
 *                 ipl = ipi + elsize2             # <<<<<<<<<<<<<<
 * 
 *         pi = pl + 1
 */
        __pyx_v_ipl = (__pyx_v_ipi + __pyx_v_elsize2);
      }
      __pyx_L17:;
    }

    /* "tables/indexesextension.pyx":313
 *                 ipl = ipi + elsize2
 * 
 *         pi = pl + 1             # <<<<<<<<<<<<<<
 *         ipi = ipl + elsize2
 *         while pi <= pr:
 */
    __pyx_v_pi = (__pyx_v_pl + 1);

    /* "tables/indexesextension.pyx":314
 * 
 *         pi = pl + 1
 *         ipi = ipl + elsize2             # <<<<<<<<<<<<<<
 *         while pi <= pr:
 *             vp = pi[0]
 */
    __pyx_v_ipi = (__pyx_v_ipl + __pyx_v_elsize2);

    /* "tables/indexesextension.pyx":315
 *         pi = pl + 1
 *         ipi = ipl + elsize2
 *         while pi <= pr:             # <<<<<<<<<<<<<<
 *             vp = pi[0]
 *             memcpy(ivp, ipi, elsize2)
 */
    while (1) {
      __pyx_t_1 = ((__pyx_v_pi <= __pyx_v_pr) != 0);
      if (!__pyx_t_1) break;

      /* "tables/indexesextension.pyx":316
 *         ipi = ipl + elsize2
 *         while pi <= pr:
 *             vp = pi[0]             # <<<<<<<<<<<<<<
 *             memcpy(ivp, ipi, elsize2)
 * 
 */
      __pyx_v_vp = (__pyx_v_pi[0]);

      /* "tables/indexesextension.pyx":317
 *         while pi <= pr:
 *             vp = pi[0]
 *             memcpy(ivp, ipi, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             pj = pi
 */
      (void)(memcpy(__pyx_v_ivp, __pyx_v_ipi, __pyx_v_elsize2));

      /* "tables/indexesextension.pyx":319
 *             memcpy(ivp, ipi, elsize2)
 * 
 *             pj = pi             # <<<<<<<<<<<<<<
 *             pt = pi - 1
 * 
 */
      __pyx_v_pj = __pyx_v_pi;

      /* "tables/indexesextension.pyx":320
 * 
 *             pj = pi
 *             pt = pi - 1             # <<<<<<<<<<<<<<
 * 
 *             ipj = ipi
 */
      __pyx_v_pt = (__pyx_v_pi - 1);

      /* "tables/indexesextension.pyx":322
 *             pt = pi - 1
 * 
 *             ipj = ipi             # <<<<<<<<<<<<<<
 *             ipt = ipi - elsize2
 * 
 */
      __pyx_v_ipj = __pyx_v_ipi;

      /* "tables/indexesextension.pyx":323
 * 
 *             ipj = ipi
 *             ipt = ipi - elsize2             # <<<<<<<<<<<<<<
 * 
 *             while pj > pl and less_than(&vp, pt):
 */
      __pyx_v_ipt = (__pyx_v_ipi - __pyx_v_elsize2);

      /* "tables/indexesextension.pyx":325
 *             ipt = ipi - elsize2
 * 
 *             while pj > pl and less_than(&vp, pt):             # <<<<<<<<<<<<<<
 *                 pj[0] = pt[0]
 *                 pj -= 1
 */
      while (1) {
        __pyx_t_4 = ((__pyx_v_pj > __pyx_v_pl) != 0);
        if (__pyx_t_4) {
        } else {
          __pyx_t_1 = __pyx_t_4;
          goto __pyx_L22_bool_binop_done;
        }
        __pyx_t_4 = (__pyx_fuse_10__pyx_f_6tables_16indexesextension_less_than((&__pyx_v_vp), __pyx_v_pt) != 0);
        __pyx_t_1 = __pyx_t_4;
        __pyx_L22_bool_binop_done:;
        if (!__pyx_t_1) break;

        /* "tables/indexesextension.pyx":326
 * 
 *             while pj > pl and less_than(&vp, pt):
 *                 pj[0] = pt[0]             # <<<<<<<<<<<<<<
 *                 pj -= 1
 *                 pt -= 1
 */
        (__pyx_v_pj[0]) = (__pyx_v_pt[0]);

        /* "tables/indexesextension.pyx":327
 *             while pj > pl and less_than(&vp, pt):
 *                 pj[0] = pt[0]
 *                 pj -= 1             # <<<<<<<<<<<<<<
 *                 pt -= 1
 * 
 */
        __pyx_v_pj = (__pyx_v_pj - 1);

        /* "tables/indexesextension.pyx":328
 *                 pj[0] = pt[0]
 *                 pj -= 1
 *                 pt -= 1             # <<<<<<<<<<<<<<
 * 
 *                 memcpy(ipj, ipt, elsize2)
 */
        __pyx_v_pt = (__pyx_v_pt - 1);

        /* "tables/indexesextension.pyx":330
 *                 pt -= 1
 * 
 *                 memcpy(ipj, ipt, elsize2)             # <<<<<<<<<<<<<<
 *                 ipj -= elsize2
 *                 ipt -= elsize2
 */
        (void)(memcpy(__pyx_v_ipj, __pyx_v_ipt, __pyx_v_elsize2));

        /* "tables/indexesextension.pyx":331
 * 
 *                 memcpy(ipj, ipt, elsize2)
 *                 ipj -= elsize2             # <<<<<<<<<<<<<<
 *                 ipt -= elsize2
 * 
 */
        __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_elsize2);

        /* "tables/indexesextension.pyx":332
 *                 memcpy(ipj, ipt, elsize2)
 *                 ipj -= elsize2
 *                 ipt -= elsize2             # <<<<<<<<<<<<<<
 * 
 *             pj[0] = vp
 */
        __pyx_v_ipt = (__pyx_v_ipt - __pyx_v_elsize2);
      }

      /* "tables/indexesextension.pyx":334
 *                 ipt -= elsize2
 * 
 *             pj[0] = vp             # <<<<<<<<<<<<<<
 *             memcpy(ipj, ivp, elsize2)
 * 
 */
      (__pyx_v_pj[0]) = __pyx_v_vp;

      /* "tables/indexesextension.pyx":335
 * 
 *             pj[0] = vp
 *             memcpy(ipj, ivp, elsize2)             # <<<<<<<<<<<<<<
 * 
 *             pi += 1
 */
      (void)(memcpy(__pyx_v_ipj, __pyx_v_ivp, __pyx_v_elsize2));

      /* "tables/indexesextension.pyx":337
 *             memcpy(ipj, ivp, elsize2)
 * 
 *             pi += 1             # <<<<<<<<<<<<<<
 *             ipi += elsize2
 * 
 */
      __pyx_v_pi = (__pyx_v_pi + 1);

      /* "tables/indexesextension.pyx":338
 * 
 *             pi += 1
 *             ipi += elsize2             # <<<<<<<<<<<<<<
 * 
 *         if sptr == stack:
 */
      __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_elsize2);
    }

    /* "tables/indexesextension.pyx":340
 *             ipi += elsize2
 * 
 *         if sptr == stack:             # <<<<<<<<<<<<<<
 *             break
 * 
 */
    __pyx_t_1 = ((__pyx_v_sptr == __pyx_v_stack) != 0);
    if (__pyx_t_1) {

      /* "tables/indexesextension.pyx":341
 * 
 *         if sptr == stack:
 *             break             # <<<<<<<<<<<<<<
 * 
 *         sptr -= 2
 */
      goto __pyx_L4_break;

      /* "tables/indexesextension.pyx":340
 *             ipi += elsize2
 * 
 *         if sptr == stack:             # <<<<<<<<<<<<<<
 *             break
 * 
 */
    }

    /* "tables/indexesextension.pyx":343
 *             break
 * 
 *         sptr -= 2             # <<<<<<<<<<<<<<
 *         pl = sptr[0]
 *         pr = sptr[1]
 */
    __pyx_v_sptr = (__pyx_v_sptr - 2);

    /* "tables/indexesextension.pyx":344
 * 
 *         sptr -= 2
 *         pl = sptr[0]             # <<<<<<<<<<<<<<
 *         pr = sptr[1]
 * 
 */
    __pyx_v_pl = (__pyx_v_sptr[0]);

    /* "tables/indexesextension.pyx":345
 *         sptr -= 2
 *         pl = sptr[0]
 *         pr = sptr[1]             # <<<<<<<<<<<<<<
 * 
 *         isptr -= 2
 */
    __pyx_v_pr = (__pyx_v_sptr[1]);

    /* "tables/indexesextension.pyx":347
 *         pr = sptr[1]
 * 
 *         isptr -= 2             # <<<<<<<<<<<<<<
 *         ipl = isptr[0]
 *         ipr = isptr[1]
 */
    __pyx_v_isptr = (__pyx_v_isptr - 2);

    /* "tables/indexesextension.pyx":348
 * 
 *         isptr -= 2
 *         ipl = isptr[0]             # <<<<<<<<<<<<<<
 *         ipr = isptr[1]
 * 
 */
    __pyx_v_ipl = (__pyx_v_isptr[0]);

    /* "tables/indexesextension.pyx":349
 *         isptr -= 2
 *         ipl = isptr[0]
 *         ipr = isptr[1]             # <<<<<<<<<<<<<<
 * 
 *     free(ivp)
 */
    __pyx_v_ipr = (__pyx_v_isptr[1]);
  }
  __pyx_L4_break:;

  /* "tables/indexesextension.pyx":351
 *         ipr = isptr[1]
 * 
 *     free(ivp)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  free(__pyx_v_ivp);

  /* "tables/indexesextension.pyx":212
 * 
 * @cython.cdivision(True)
 * cdef void _keysort(number_type* start1, char* start2, size_t elsize2, size_t n) nogil:             # <<<<<<<<<<<<<<
 *     cdef number_type *pl = start1
 *     cdef number_type *pr = start1 + (n - 1)
 */

  /* function exit code */
}

/* "tables/indexesextension.pyx":355
 * 
 * @cython.cdivision(True)
 * cdef void _keysort_string(char* start1, size_t ss, char* start2, size_t ts, size_t n) nogil:             # <<<<<<<<<<<<<<
 *     cdef char *pl = start1
 *     cdef char *pr = start1 + (n - 1) * ss
 */

static void __pyx_f_6tables_16indexesextension__keysort_string(char *__pyx_v_start1, size_t __pyx_v_ss, char *__pyx_v_start2, size_t __pyx_v_ts, size_t __pyx_v_n) {
  char *__pyx_v_pl;
  char *__pyx_v_pr;
  char *__pyx_v_ipl;
  char *__pyx_v_ipr;
  char *__pyx_v_vp;
  char *__pyx_v_ivp;
  char *__pyx_v_stack[0x64];
  char **__pyx_v_sptr;
  char *__pyx_v_istack[0x64];
  char **__pyx_v_isptr;
  CYTHON_UNUSED size_t __pyx_v_stack_index;
  char *__pyx_v_pm;
  char *__pyx_v_pi;
  char *__pyx_v_pj;
  char *__pyx_v_pt;
  char *__pyx_v_ipm;
  char *__pyx_v_ipi;
  char *__pyx_v_ipj;
  char *__pyx_v_ipt;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "tables/indexesextension.pyx":356
 * @cython.cdivision(True)
 * cdef void _keysort_string(char* start1, size_t ss, char* start2, size_t ts, size_t n) nogil:
 *     cdef char *pl = start1             # <<<<<<<<<<<<<<
 *     cdef char *pr = start1 + (n - 1) * ss
 * 
 */
  __pyx_v_pl = __pyx_v_start1;

  /* "tables/indexesextension.pyx":357
 * cdef void _keysort_string(char* start1, size_t ss, char* start2, size_t ts, size_t n) nogil:
 *     cdef char *pl = start1
 *     cdef char *pr = start1 + (n - 1) * ss             # <<<<<<<<<<<<<<
 * 
 *     cdef char *ipl = start2
 */
  __pyx_v_pr = (__pyx_v_start1 + ((__pyx_v_n - 1) * __pyx_v_ss));

  /* "tables/indexesextension.pyx":359
 *     cdef char *pr = start1 + (n - 1) * ss
 * 
 *     cdef char *ipl = start2             # <<<<<<<<<<<<<<
 *     cdef char *ipr = start2 + (n - 1) * ts
 * 
 */
  __pyx_v_ipl = __pyx_v_start2;

  /* "tables/indexesextension.pyx":360
 * 
 *     cdef char *ipl = start2
 *     cdef char *ipr = start2 + (n - 1) * ts             # <<<<<<<<<<<<<<
 * 
 *     cdef char *vp = <char *>malloc(ss)
 */
  __pyx_v_ipr = (__pyx_v_start2 + ((__pyx_v_n - 1) * __pyx_v_ts));

  /* "tables/indexesextension.pyx":362
 *     cdef char *ipr = start2 + (n - 1) * ts
 * 
 *     cdef char *vp = <char *>malloc(ss)             # <<<<<<<<<<<<<<
 *     cdef char *ivp = <char *>malloc(ts)
 * 
 */
  __pyx_v_vp = ((char *)malloc(__pyx_v_ss));

  /* "tables/indexesextension.pyx":363
 * 
 *     cdef char *vp = <char *>malloc(ss)
 *     cdef char *ivp = <char *>malloc(ts)             # <<<<<<<<<<<<<<
 * 
 *     cdef char *stack[PYA_QS_STACK]
 */
  __pyx_v_ivp = ((char *)malloc(__pyx_v_ts));

  /* "tables/indexesextension.pyx":366
 * 
 *     cdef char *stack[PYA_QS_STACK]
 *     cdef char **sptr = stack             # <<<<<<<<<<<<<<
 * 
 *     cdef char *istack[PYA_QS_STACK]
 */
  __pyx_v_sptr = __pyx_v_stack;

  /* "tables/indexesextension.pyx":369
 * 
 *     cdef char *istack[PYA_QS_STACK]
 *     cdef char **isptr = istack             # <<<<<<<<<<<<<<
 * 
 *     cdef size_t stack_index = 0
 */
  __pyx_v_isptr = __pyx_v_istack;

  /* "tables/indexesextension.pyx":371
 *     cdef char **isptr = istack
 * 
 *     cdef size_t stack_index = 0             # <<<<<<<<<<<<<<
 * 
 *     cdef char *pm
 */
  __pyx_v_stack_index = 0;

  /* "tables/indexesextension.pyx":383
 *     cdef char *ipt
 * 
 *     while True:             # <<<<<<<<<<<<<<
 *         while pr - pl > SMALL_QUICKSORT * ss:
 *             pm  = pl + ((pr - pl)//ss >> 1)*ss
 */
  while (1) {

    /* "tables/indexesextension.pyx":384
 * 
 *     while True:
 *         while pr - pl > SMALL_QUICKSORT * ss:             # <<<<<<<<<<<<<<
 *             pm  = pl + ((pr - pl)//ss >> 1)*ss
 *             ipm  = ipl + ((ipr - ipl)//ts >> 1)*ts
 */
    while (1) {
      __pyx_t_1 = (((__pyx_v_pr - __pyx_v_pl) > (15 * __pyx_v_ss)) != 0);
      if (!__pyx_t_1) break;

      /* "tables/indexesextension.pyx":385
 *     while True:
 *         while pr - pl > SMALL_QUICKSORT * ss:
 *             pm  = pl + ((pr - pl)//ss >> 1)*ss             # <<<<<<<<<<<<<<
 *             ipm  = ipl + ((ipr - ipl)//ts >> 1)*ts
 * 
 */
      __pyx_v_pm = (__pyx_v_pl + ((((__pyx_v_pr - __pyx_v_pl) / __pyx_v_ss) >> 1) * __pyx_v_ss));

      /* "tables/indexesextension.pyx":386
 *         while pr - pl > SMALL_QUICKSORT * ss:
 *             pm  = pl + ((pr - pl)//ss >> 1)*ss
 *             ipm  = ipl + ((ipr - ipl)//ts >> 1)*ts             # <<<<<<<<<<<<<<
 * 
 *             if strncmp(pm, pl, ss) < 0:
 */
      __pyx_v_ipm = (__pyx_v_ipl + ((((__pyx_v_ipr - __pyx_v_ipl) / __pyx_v_ts) >> 1) * __pyx_v_ts));

      /* "tables/indexesextension.pyx":388
 *             ipm  = ipl + ((ipr - ipl)//ts >> 1)*ts
 * 
 *             if strncmp(pm, pl, ss) < 0:             # <<<<<<<<<<<<<<
 *                 swap_bytes(pm, pl, ss)
 *                 swap_bytes(ipm, ipl, ts)
 */
      __pyx_t_1 = ((strncmp(__pyx_v_pm, __pyx_v_pl, __pyx_v_ss) < 0) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":389
 * 
 *             if strncmp(pm, pl, ss) < 0:
 *                 swap_bytes(pm, pl, ss)             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipm, ipl, ts)
 * 
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_pm, __pyx_v_pl, __pyx_v_ss);

        /* "tables/indexesextension.pyx":390
 *             if strncmp(pm, pl, ss) < 0:
 *                 swap_bytes(pm, pl, ss)
 *                 swap_bytes(ipm, ipl, ts)             # <<<<<<<<<<<<<<
 * 
 *             if strncmp(pr, pm, ss) < 0:
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipl, __pyx_v_ts);

        /* "tables/indexesextension.pyx":388
 *             ipm  = ipl + ((ipr - ipl)//ts >> 1)*ts
 * 
 *             if strncmp(pm, pl, ss) < 0:             # <<<<<<<<<<<<<<
 *                 swap_bytes(pm, pl, ss)
 *                 swap_bytes(ipm, ipl, ts)
 */
      }

      /* "tables/indexesextension.pyx":392
 *                 swap_bytes(ipm, ipl, ts)
 * 
 *             if strncmp(pr, pm, ss) < 0:             # <<<<<<<<<<<<<<
 *                 swap_bytes(pr, pm, ss)
 *                 swap_bytes(ipr, ipm, ts)
 */
      __pyx_t_1 = ((strncmp(__pyx_v_pr, __pyx_v_pm, __pyx_v_ss) < 0) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":393
 * 
 *             if strncmp(pr, pm, ss) < 0:
 *                 swap_bytes(pr, pm, ss)             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipr, ipm, ts)
 * 
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_pr, __pyx_v_pm, __pyx_v_ss);

        /* "tables/indexesextension.pyx":394
 *             if strncmp(pr, pm, ss) < 0:
 *                 swap_bytes(pr, pm, ss)
 *                 swap_bytes(ipr, ipm, ts)             # <<<<<<<<<<<<<<
 * 
 *             if strncmp(pm, pl, ss) < 0:
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipr, __pyx_v_ipm, __pyx_v_ts);

        /* "tables/indexesextension.pyx":392
 *                 swap_bytes(ipm, ipl, ts)
 * 
 *             if strncmp(pr, pm, ss) < 0:             # <<<<<<<<<<<<<<
 *                 swap_bytes(pr, pm, ss)
 *                 swap_bytes(ipr, ipm, ts)
 */
      }

      /* "tables/indexesextension.pyx":396
 *                 swap_bytes(ipr, ipm, ts)
 * 
 *             if strncmp(pm, pl, ss) < 0:             # <<<<<<<<<<<<<<
 *                 swap_bytes(pm, pl, ss)
 *                 swap_bytes(ipm, ipl, ts)
 */
      __pyx_t_1 = ((strncmp(__pyx_v_pm, __pyx_v_pl, __pyx_v_ss) < 0) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":397
 * 
 *             if strncmp(pm, pl, ss) < 0:
 *                 swap_bytes(pm, pl, ss)             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipm, ipl, ts)
 * 
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_pm, __pyx_v_pl, __pyx_v_ss);

        /* "tables/indexesextension.pyx":398
 *             if strncmp(pm, pl, ss) < 0:
 *                 swap_bytes(pm, pl, ss)
 *                 swap_bytes(ipm, ipl, ts)             # <<<<<<<<<<<<<<
 * 
 *             memcpy(vp, pm, ss)
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipl, __pyx_v_ts);

        /* "tables/indexesextension.pyx":396
 *                 swap_bytes(ipr, ipm, ts)
 * 
 *             if strncmp(pm, pl, ss) < 0:             # <<<<<<<<<<<<<<
 *                 swap_bytes(pm, pl, ss)
 *                 swap_bytes(ipm, ipl, ts)
 */
      }

      /* "tables/indexesextension.pyx":400
 *                 swap_bytes(ipm, ipl, ts)
 * 
 *             memcpy(vp, pm, ss)             # <<<<<<<<<<<<<<
 * 
 *             pi = pl
 */
      (void)(memcpy(__pyx_v_vp, __pyx_v_pm, __pyx_v_ss));

      /* "tables/indexesextension.pyx":402
 *             memcpy(vp, pm, ss)
 * 
 *             pi = pl             # <<<<<<<<<<<<<<
 *             ipi = ipl
 * 
 */
      __pyx_v_pi = __pyx_v_pl;

      /* "tables/indexesextension.pyx":403
 * 
 *             pi = pl
 *             ipi = ipl             # <<<<<<<<<<<<<<
 * 
 *             pj = pr - ss
 */
      __pyx_v_ipi = __pyx_v_ipl;

      /* "tables/indexesextension.pyx":405
 *             ipi = ipl
 * 
 *             pj = pr - ss             # <<<<<<<<<<<<<<
 *             ipj = ipr - ts
 * 
 */
      __pyx_v_pj = (__pyx_v_pr - __pyx_v_ss);

      /* "tables/indexesextension.pyx":406
 * 
 *             pj = pr - ss
 *             ipj = ipr - ts             # <<<<<<<<<<<<<<
 * 
 *             swap_bytes(pm, pj, ss)
 */
      __pyx_v_ipj = (__pyx_v_ipr - __pyx_v_ts);

      /* "tables/indexesextension.pyx":408
 *             ipj = ipr - ts
 * 
 *             swap_bytes(pm, pj, ss)             # <<<<<<<<<<<<<<
 *             swap_bytes(ipm, ipj, ts)
 * 
 */
      __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_pm, __pyx_v_pj, __pyx_v_ss);

      /* "tables/indexesextension.pyx":409
 * 
 *             swap_bytes(pm, pj, ss)
 *             swap_bytes(ipm, ipj, ts)             # <<<<<<<<<<<<<<
 * 
 *             while True:
 */
      __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipm, __pyx_v_ipj, __pyx_v_ts);

      /* "tables/indexesextension.pyx":411
 *             swap_bytes(ipm, ipj, ts)
 * 
 *             while True:             # <<<<<<<<<<<<<<
 *                 pi += ss
 *                 ipi += ts
 */
      while (1) {

        /* "tables/indexesextension.pyx":412
 * 
 *             while True:
 *                 pi += ss             # <<<<<<<<<<<<<<
 *                 ipi += ts
 *                 while strncmp(pi, vp, ss) < 0:
 */
        __pyx_v_pi = (__pyx_v_pi + __pyx_v_ss);

        /* "tables/indexesextension.pyx":413
 *             while True:
 *                 pi += ss
 *                 ipi += ts             # <<<<<<<<<<<<<<
 *                 while strncmp(pi, vp, ss) < 0:
 *                     pi += ss
 */
        __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_ts);

        /* "tables/indexesextension.pyx":414
 *                 pi += ss
 *                 ipi += ts
 *                 while strncmp(pi, vp, ss) < 0:             # <<<<<<<<<<<<<<
 *                     pi += ss
 *                     ipi += ts
 */
        while (1) {
          __pyx_t_1 = ((strncmp(__pyx_v_pi, __pyx_v_vp, __pyx_v_ss) < 0) != 0);
          if (!__pyx_t_1) break;

          /* "tables/indexesextension.pyx":415
 *                 ipi += ts
 *                 while strncmp(pi, vp, ss) < 0:
 *                     pi += ss             # <<<<<<<<<<<<<<
 *                     ipi += ts
 * 
 */
          __pyx_v_pi = (__pyx_v_pi + __pyx_v_ss);

          /* "tables/indexesextension.pyx":416
 *                 while strncmp(pi, vp, ss) < 0:
 *                     pi += ss
 *                     ipi += ts             # <<<<<<<<<<<<<<
 * 
 *                 pj -= ss
 */
          __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_ts);
        }

        /* "tables/indexesextension.pyx":418
 *                     ipi += ts
 * 
 *                 pj -= ss             # <<<<<<<<<<<<<<
 *                 ipj -= ts
 *                 while strncmp(vp, pj, ss) < 0:
 */
        __pyx_v_pj = (__pyx_v_pj - __pyx_v_ss);

        /* "tables/indexesextension.pyx":419
 * 
 *                 pj -= ss
 *                 ipj -= ts             # <<<<<<<<<<<<<<
 *                 while strncmp(vp, pj, ss) < 0:
 *                     pj -= ss
 */
        __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_ts);

        /* "tables/indexesextension.pyx":420
 *                 pj -= ss
 *                 ipj -= ts
 *                 while strncmp(vp, pj, ss) < 0:             # <<<<<<<<<<<<<<
 *                     pj -= ss
 *                     ipj -= ts
 */
        while (1) {
          __pyx_t_1 = ((strncmp(__pyx_v_vp, __pyx_v_pj, __pyx_v_ss) < 0) != 0);
          if (!__pyx_t_1) break;

          /* "tables/indexesextension.pyx":421
 *                 ipj -= ts
 *                 while strncmp(vp, pj, ss) < 0:
 *                     pj -= ss             # <<<<<<<<<<<<<<
 *                     ipj -= ts
 * 
 */
          __pyx_v_pj = (__pyx_v_pj - __pyx_v_ss);

          /* "tables/indexesextension.pyx":422
 *                 while strncmp(vp, pj, ss) < 0:
 *                     pj -= ss
 *                     ipj -= ts             # <<<<<<<<<<<<<<
 * 
 *                 if pi >= pj:
 */
          __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_ts);
        }

        /* "tables/indexesextension.pyx":424
 *                     ipj -= ts
 * 
 *                 if pi >= pj:             # <<<<<<<<<<<<<<
 *                     break
 * 
 */
        __pyx_t_1 = ((__pyx_v_pi >= __pyx_v_pj) != 0);
        if (__pyx_t_1) {

          /* "tables/indexesextension.pyx":425
 * 
 *                 if pi >= pj:
 *                     break             # <<<<<<<<<<<<<<
 * 
 *                 swap_bytes(pi, pj, ss)
 */
          goto __pyx_L11_break;

          /* "tables/indexesextension.pyx":424
 *                     ipj -= ts
 * 
 *                 if pi >= pj:             # <<<<<<<<<<<<<<
 *                     break
 * 
 */
        }

        /* "tables/indexesextension.pyx":427
 *                     break
 * 
 *                 swap_bytes(pi, pj, ss)             # <<<<<<<<<<<<<<
 *                 swap_bytes(ipi, ipj, ts)
 * 
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_pi, __pyx_v_pj, __pyx_v_ss);

        /* "tables/indexesextension.pyx":428
 * 
 *                 swap_bytes(pi, pj, ss)
 *                 swap_bytes(ipi, ipj, ts)             # <<<<<<<<<<<<<<
 * 
 *             swap_bytes(pi, pr-ss, ss)
 */
        __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipi, __pyx_v_ipj, __pyx_v_ts);
      }
      __pyx_L11_break:;

      /* "tables/indexesextension.pyx":430
 *                 swap_bytes(ipi, ipj, ts)
 * 
 *             swap_bytes(pi, pr-ss, ss)             # <<<<<<<<<<<<<<
 *             swap_bytes(ipi, ipr-ts, ts)
 * 
 */
      __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_pi, (__pyx_v_pr - __pyx_v_ss), __pyx_v_ss);

      /* "tables/indexesextension.pyx":431
 * 
 *             swap_bytes(pi, pr-ss, ss)
 *             swap_bytes(ipi, ipr-ts, ts)             # <<<<<<<<<<<<<<
 * 
 *             # push largest partition on stack and proceed with the other
 */
      __pyx_f_6tables_16indexesextension_swap_bytes(__pyx_v_ipi, (__pyx_v_ipr - __pyx_v_ts), __pyx_v_ts);

      /* "tables/indexesextension.pyx":434
 * 
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):             # <<<<<<<<<<<<<<
 *                 sptr[0] = pi + ss
 *                 sptr[1] = pr
 */
      __pyx_t_1 = (((__pyx_v_pi - __pyx_v_pl) < (__pyx_v_pr - __pyx_v_pi)) != 0);
      if (__pyx_t_1) {

        /* "tables/indexesextension.pyx":435
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):
 *                 sptr[0] = pi + ss             # <<<<<<<<<<<<<<
 *                 sptr[1] = pr
 *                 sptr += 2
 */
        (__pyx_v_sptr[0]) = (__pyx_v_pi + __pyx_v_ss);

        /* "tables/indexesextension.pyx":436
 *             if (pi - pl) < (pr - pi):
 *                 sptr[0] = pi + ss
 *                 sptr[1] = pr             # <<<<<<<<<<<<<<
 *                 sptr += 2
 * 
 */
        (__pyx_v_sptr[1]) = __pyx_v_pr;

        /* "tables/indexesextension.pyx":437
 *                 sptr[0] = pi + ss
 *                 sptr[1] = pr
 *                 sptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 isptr[0] = ipi + ts
 */
        __pyx_v_sptr = (__pyx_v_sptr + 2);

        /* "tables/indexesextension.pyx":439
 *                 sptr += 2
 * 
 *                 isptr[0] = ipi + ts             # <<<<<<<<<<<<<<
 *                 isptr[1] = ipr
 *                 isptr += 2
 */
        (__pyx_v_isptr[0]) = (__pyx_v_ipi + __pyx_v_ts);

        /* "tables/indexesextension.pyx":440
 * 
 *                 isptr[0] = ipi + ts
 *                 isptr[1] = ipr             # <<<<<<<<<<<<<<
 *                 isptr += 2
 * 
 */
        (__pyx_v_isptr[1]) = __pyx_v_ipr;

        /* "tables/indexesextension.pyx":441
 *                 isptr[0] = ipi + ts
 *                 isptr[1] = ipr
 *                 isptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 pr = pi - ss
 */
        __pyx_v_isptr = (__pyx_v_isptr + 2);

        /* "tables/indexesextension.pyx":443
 *                 isptr += 2
 * 
 *                 pr = pi - ss             # <<<<<<<<<<<<<<
 *                 ipr = ipi - ts
 *             else:
 */
        __pyx_v_pr = (__pyx_v_pi - __pyx_v_ss);

        /* "tables/indexesextension.pyx":444
 * 
 *                 pr = pi - ss
 *                 ipr = ipi - ts             # <<<<<<<<<<<<<<
 *             else:
 *                 sptr[0] = pl
 */
        __pyx_v_ipr = (__pyx_v_ipi - __pyx_v_ts);

        /* "tables/indexesextension.pyx":434
 * 
 *             # push largest partition on stack and proceed with the other
 *             if (pi - pl) < (pr - pi):             # <<<<<<<<<<<<<<
 *                 sptr[0] = pi + ss
 *                 sptr[1] = pr
 */
        goto __pyx_L17;
      }

      /* "tables/indexesextension.pyx":446
 *                 ipr = ipi - ts
 *             else:
 *                 sptr[0] = pl             # <<<<<<<<<<<<<<
 *                 sptr[1] = pi - ss
 *                 sptr += 2
 */
      /*else*/ {
        (__pyx_v_sptr[0]) = __pyx_v_pl;

        /* "tables/indexesextension.pyx":447
 *             else:
 *                 sptr[0] = pl
 *                 sptr[1] = pi - ss             # <<<<<<<<<<<<<<
 *                 sptr += 2
 * 
 */
        (__pyx_v_sptr[1]) = (__pyx_v_pi - __pyx_v_ss);

        /* "tables/indexesextension.pyx":448
 *                 sptr[0] = pl
 *                 sptr[1] = pi - ss
 *                 sptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 isptr[0] = ipl
 */
        __pyx_v_sptr = (__pyx_v_sptr + 2);

        /* "tables/indexesextension.pyx":450
 *                 sptr += 2
 * 
 *                 isptr[0] = ipl             # <<<<<<<<<<<<<<
 *                 isptr[1] = ipi - ts
 *                 isptr += 2
 */
        (__pyx_v_isptr[0]) = __pyx_v_ipl;

        /* "tables/indexesextension.pyx":451
 * 
 *                 isptr[0] = ipl
 *                 isptr[1] = ipi - ts             # <<<<<<<<<<<<<<
 *                 isptr += 2
 * 
 */
        (__pyx_v_isptr[1]) = (__pyx_v_ipi - __pyx_v_ts);

        /* "tables/indexesextension.pyx":452
 *                 isptr[0] = ipl
 *                 isptr[1] = ipi - ts
 *                 isptr += 2             # <<<<<<<<<<<<<<
 * 
 *                 pl = pi + ss
 */
        __pyx_v_isptr = (__pyx_v_isptr + 2);

        /* "tables/indexesextension.pyx":454
 *                 isptr += 2
 * 
 *                 pl = pi + ss             # <<<<<<<<<<<<<<
 *                 ipl = ipi + ts
 * 
 */
        __pyx_v_pl = (__pyx_v_pi + __pyx_v_ss);

        /* "tables/indexesextension.pyx":455
 * 
 *                 pl = pi + ss
 *                 ipl = ipi + ts             # <<<<<<<<<<<<<<
 * 
 *         pi = pl + ss
 */
        __pyx_v_ipl = (__pyx_v_ipi + __pyx_v_ts);
      }
      __pyx_L17:;
    }

    /* "tables/indexesextension.pyx":457
 *                 ipl = ipi + ts
 * 
 *         pi = pl + ss             # <<<<<<<<<<<<<<
 *         ipi = ipl + ts
 * 
 */
    __pyx_v_pi = (__pyx_v_pl + __pyx_v_ss);

    /* "tables/indexesextension.pyx":458
 * 
 *         pi = pl + ss
 *         ipi = ipl + ts             # <<<<<<<<<<<<<<
 * 
 *         while pi <= pr:
 */
    __pyx_v_ipi = (__pyx_v_ipl + __pyx_v_ts);

    /* "tables/indexesextension.pyx":460
 *         ipi = ipl + ts
 * 
 *         while pi <= pr:             # <<<<<<<<<<<<<<
 *             memcpy(vp, pi, ss)
 *             memcpy(ivp, ipi, ts)
 */
    while (1) {
      __pyx_t_1 = ((__pyx_v_pi <= __pyx_v_pr) != 0);
      if (!__pyx_t_1) break;

      /* "tables/indexesextension.pyx":461
 * 
 *         while pi <= pr:
 *             memcpy(vp, pi, ss)             # <<<<<<<<<<<<<<
 *             memcpy(ivp, ipi, ts)
 * 
 */
      (void)(memcpy(__pyx_v_vp, __pyx_v_pi, __pyx_v_ss));

      /* "tables/indexesextension.pyx":462
 *         while pi <= pr:
 *             memcpy(vp, pi, ss)
 *             memcpy(ivp, ipi, ts)             # <<<<<<<<<<<<<<
 * 
 *             pj = pi
 */
      (void)(memcpy(__pyx_v_ivp, __pyx_v_ipi, __pyx_v_ts));

      /* "tables/indexesextension.pyx":464
 *             memcpy(ivp, ipi, ts)
 * 
 *             pj = pi             # <<<<<<<<<<<<<<
 *             pt = pi - ss
 * 
 */
      __pyx_v_pj = __pyx_v_pi;

      /* "tables/indexesextension.pyx":465
 * 
 *             pj = pi
 *             pt = pi - ss             # <<<<<<<<<<<<<<
 * 
 *             ipj = ipi
 */
      __pyx_v_pt = (__pyx_v_pi - __pyx_v_ss);

      /* "tables/indexesextension.pyx":467
 *             pt = pi - ss
 * 
 *             ipj = ipi             # <<<<<<<<<<<<<<
 *             ipt = ipi - ts
 * 
 */
      __pyx_v_ipj = __pyx_v_ipi;

      /* "tables/indexesextension.pyx":468
 * 
 *             ipj = ipi
 *             ipt = ipi - ts             # <<<<<<<<<<<<<<
 * 
 *             while pj > pl and strncmp(vp, pt, ss) < 0:
 */
      __pyx_v_ipt = (__pyx_v_ipi - __pyx_v_ts);

      /* "tables/indexesextension.pyx":470
 *             ipt = ipi - ts
 * 
 *             while pj > pl and strncmp(vp, pt, ss) < 0:             # <<<<<<<<<<<<<<
 *                 memcpy(pj, pt, ss)
 *                 pj -= ss
 */
      while (1) {
        __pyx_t_2 = ((__pyx_v_pj > __pyx_v_pl) != 0);
        if (__pyx_t_2) {
        } else {
          __pyx_t_1 = __pyx_t_2;
          goto __pyx_L22_bool_binop_done;
        }
        __pyx_t_2 = ((strncmp(__pyx_v_vp, __pyx_v_pt, __pyx_v_ss) < 0) != 0);
        __pyx_t_1 = __pyx_t_2;
        __pyx_L22_bool_binop_done:;
        if (!__pyx_t_1) break;

        /* "tables/indexesextension.pyx":471
 * 
 *             while pj > pl and strncmp(vp, pt, ss) < 0:
 *                 memcpy(pj, pt, ss)             # <<<<<<<<<<<<<<
 *                 pj -= ss
 *                 pt -= ss
 */
        (void)(memcpy(__pyx_v_pj, __pyx_v_pt, __pyx_v_ss));

        /* "tables/indexesextension.pyx":472
 *             while pj > pl and strncmp(vp, pt, ss) < 0:
 *                 memcpy(pj, pt, ss)
 *                 pj -= ss             # <<<<<<<<<<<<<<
 *                 pt -= ss
 * 
 */
        __pyx_v_pj = (__pyx_v_pj - __pyx_v_ss);

        /* "tables/indexesextension.pyx":473
 *                 memcpy(pj, pt, ss)
 *                 pj -= ss
 *                 pt -= ss             # <<<<<<<<<<<<<<
 * 
 *                 memcpy(ipj, ipt, ts)
 */
        __pyx_v_pt = (__pyx_v_pt - __pyx_v_ss);

        /* "tables/indexesextension.pyx":475
 *                 pt -= ss
 * 
 *                 memcpy(ipj, ipt, ts)             # <<<<<<<<<<<<<<
 *                 ipj -= ts
 *                 ipt -= ts
 */
        (void)(memcpy(__pyx_v_ipj, __pyx_v_ipt, __pyx_v_ts));

        /* "tables/indexesextension.pyx":476
 * 
 *                 memcpy(ipj, ipt, ts)
 *                 ipj -= ts             # <<<<<<<<<<<<<<
 *                 ipt -= ts
 * 
 */
        __pyx_v_ipj = (__pyx_v_ipj - __pyx_v_ts);

        /* "tables/indexesextension.pyx":477
 *                 memcpy(ipj, ipt, ts)
 *                 ipj -= ts
 *                 ipt -= ts             # <<<<<<<<<<<<<<
 * 
 *             memcpy(pj, vp, ss)
 */
        __pyx_v_ipt = (__pyx_v_ipt - __pyx_v_ts);
      }

      /* "tables/indexesextension.pyx":479
 *                 ipt -= ts
 * 
 *             memcpy(pj, vp, ss)             # <<<<<<<<<<<<<<
 *             memcpy(ipj, ivp, ts)
 * 
 */
      (void)(memcpy(__pyx_v_pj, __pyx_v_vp, __pyx_v_ss));

      /* "tables/indexesextension.pyx":480
 * 
 *             memcpy(pj, vp, ss)
 *             memcpy(ipj, ivp, ts)             # <<<<<<<<<<<<<<
 * 
 *             pi += ss
 */
      (void)(memcpy(__pyx_v_ipj, __pyx_v_ivp, __pyx_v_ts));

      /* "tables/indexesextension.pyx":482
 *             memcpy(ipj, ivp, ts)
 * 
 *             pi += ss             # <<<<<<<<<<<<<<
 *             ipi += ts
 * 
 */
      __pyx_v_pi = (__pyx_v_pi + __pyx_v_ss);

      /* "tables/indexesextension.pyx":483
 * 
 *             pi += ss
 *             ipi += ts             # <<<<<<<<<<<<<<
 * 
 *         if sptr == stack:
 */
      __pyx_v_ipi = (__pyx_v_ipi + __pyx_v_ts);
    }

    /* "tables/indexesextension.pyx":485
 *             ipi += ts
 * 
 *         if sptr == stack:             # <<<<<<<<<<<<<<
 *             break
 * 
 */
    __pyx_t_1 = ((__pyx_v_sptr == __pyx_v_stack) != 0);
    if (__pyx_t_1) {

      /* "tables/indexesextension.pyx":486
 * 
 *         if sptr == stack:
 *             break             # <<<<<<<<<<<<<<
 * 
 *         sptr -= 2
 */
      goto __pyx_L4_break;

      /* "tables/indexesextension.pyx":485
 *             ipi += ts
 * 
 *         if sptr == stack:             # <<<<<<<<<<<<<<
 *             break
 * 
 */
    }

    /* "tables/indexesextension.pyx":488
 *             break
 * 
 *         sptr -= 2             # <<<<<<<<<<<<<<
 *         pl = sptr[0]
 *         pr = sptr[1]
 */
    __pyx_v_sptr = (__pyx_v_sptr - 2);

    /* "tables/indexesextension.pyx":489
 * 
 *         sptr -= 2
 *         pl = sptr[0]             # <<<<<<<<<<<<<<
 *         pr = sptr[1]
 * 
 */
    __pyx_v_pl = (__pyx_v_sptr[0]);

    /* "tables/indexesextension.pyx":490
 *         sptr -= 2
 *         pl = sptr[0]
 *         pr = sptr[1]             # <<<<<<<<<<<<<<
 * 
 *         isptr -= 2
 */
    __pyx_v_pr = (__pyx_v_sptr[1]);

    /* "tables/indexesextension.pyx":492
 *         pr = sptr[1]
 * 
 *         isptr -= 2             # <<<<<<<<<<<<<<
 *         ipl = isptr[0]
 *         ipr = isptr[1]
 */
    __pyx_v_isptr = (__pyx_v_isptr - 2);

    /* "tables/indexesextension.pyx":493
 * 
 *         isptr -= 2
 *         ipl = isptr[0]             # <<<<<<<<<<<<<<
 *         ipr = isptr[1]
 * 
 */
    __pyx_v_ipl = (__pyx_v_isptr[0]);

    /* "tables/indexesextension.pyx":494
 *         isptr -= 2
 *         ipl = isptr[0]
 *         ipr = isptr[1]             # <<<<<<<<<<<<<<
 * 
 *     free(vp)
 */
    __pyx_v_ipr = (__pyx_v_isptr[1]);
  }
  __pyx_L4_break:;

  /* "tables/indexesextension.pyx":496
 *         ipr = isptr[1]
 * 
 *     free(vp)             # <<<<<<<<<<<<<<
 *     free(ivp)
 * 
 */
  free(__pyx_v_vp);

  /* "tables/indexesextension.pyx":497
 * 
 *     free(vp)
 *     free(ivp)             # <<<<<<<<<<<<<<
 * 
 * #---------------------------------------------------------------------------
 */
  free(__pyx_v_ivp);

  /* "tables/indexesextension.pyx":355
 * 
 * @cython.cdivision(True)
 * cdef void _keysort_string(char* start1, size_t ss, char* start2, size_t ts, size_t n) nogil:             # <<<<<<<<<<<<<<
 *     cdef char *pl = start1
 *     cdef char *pr = start1 + (n - 1) * ss
 */

  /* function exit code */
}

/* "tables/indexesextension.pyx":507
 * # because I forsee that this should be a very common case.
 * # 2004-05-20
 * def _bisect_left(a, x, int hi):             # <<<<<<<<<<<<<<
 *   """Return the index where to insert item x in list a, assuming a is sorted.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_16indexesextension_3_bisect_left(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6tables_16indexesextension_2_bisect_left[] = "Return the index where to insert item x in list a, assuming a is sorted.\n\n  The return value i is such that all e in a[:i] have e < x, and all e in\n  a[i:] have e >= x.  So if x already appears in the list, i points just\n  before the leftmost x already there.\n\n  ";
static PyMethodDef __pyx_mdef_6tables_16indexesextension_3_bisect_left = {"_bisect_left", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6tables_16indexesextension_3_bisect_left, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6tables_16indexesextension_2_bisect_left};
static PyObject *__pyx_pw_6tables_16indexesextension_3_bisect_left(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_a = 0;
  PyObject *__pyx_v_x = 0;
  int __pyx_v_hi;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_bisect_left (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_a,&__pyx_n_s_x,&__pyx_n_s_hi,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_a)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_bisect_left", 1, 3, 3, 1); __PYX_ERR(0, 507, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_hi)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_bisect_left", 1, 3, 3, 2); __PYX_ERR(0, 507, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_bisect_left") < 0)) __PYX_ERR(0, 507, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_a = values[0];
    __pyx_v_x = values[1];
    __pyx_v_hi = __Pyx_PyInt_As_int(values[2]); if (unlikely((__pyx_v_hi == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 507, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_bisect_left", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 507, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("tables.indexesextension._bisect_left", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_16indexesextension_2_bisect_left(__pyx_self, __pyx_v_a, __pyx_v_x, __pyx_v_hi);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_16indexesextension_2_bisect_left(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_a, PyObject *__pyx_v_x, int __pyx_v_hi) {
  int __pyx_v_lo;
  int __pyx_v_mid;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_bisect_left", 0);

  /* "tables/indexesextension.pyx":518
 *   cdef int lo, mid
 * 
 *   lo = 0             # <<<<<<<<<<<<<<
 *   if x <= a[0]: return 0
 *   if a[-1] < x: return hi
 */
  __pyx_v_lo = 0;

  /* "tables/indexesextension.pyx":519
 * 
 *   lo = 0
 *   if x <= a[0]: return 0             # <<<<<<<<<<<<<<
 *   if a[-1] < x: return hi
 *   while lo < hi:
 */
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_a, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 519, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_RichCompare(__pyx_v_x, __pyx_t_1, Py_LE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 519, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 519, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_int_0);
    __pyx_r = __pyx_int_0;
    goto __pyx_L0;
  }

  /* "tables/indexesextension.pyx":520
 *   lo = 0
 *   if x <= a[0]: return 0
 *   if a[-1] < x: return hi             # <<<<<<<<<<<<<<
 *   while lo < hi:
 *       mid = (lo+hi)//2
 */
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_a, -1L, long, 1, __Pyx_PyInt_From_long, 0, 1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 520, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyObject_RichCompare(__pyx_t_2, __pyx_v_x, Py_LT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 520, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 520, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_hi); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 520, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
  }

  /* "tables/indexesextension.pyx":521
 *   if x <= a[0]: return 0
 *   if a[-1] < x: return hi
 *   while lo < hi:             # <<<<<<<<<<<<<<
 *       mid = (lo+hi)//2
 *       if a[mid] < x: lo = mid+1
 */
  while (1) {
    __pyx_t_3 = ((__pyx_v_lo < __pyx_v_hi) != 0);
    if (!__pyx_t_3) break;

    /* "tables/indexesextension.pyx":522
 *   if a[-1] < x: return hi
 *   while lo < hi:
 *       mid = (lo+hi)//2             # <<<<<<<<<<<<<<
 *       if a[mid] < x: lo = mid+1
 *       else: hi = mid
 */
    __pyx_v_mid = __Pyx_div_long((__pyx_v_lo + __pyx_v_hi), 2);

    /* "tables/indexesextension.pyx":523
 *   while lo < hi:
 *       mid = (lo+hi)//2
 *       if a[mid] < x: lo = mid+1             # <<<<<<<<<<<<<<
 *       else: hi = mid
 *   return lo
 */
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_a, __pyx_v_mid, int, 1, __Pyx_PyInt_From_int, 0, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 523, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyObject_RichCompare(__pyx_t_1, __pyx_v_x, Py_LT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 523, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 523, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_3) {
      __pyx_v_lo = (__pyx_v_mid + 1);
      goto __pyx_L7;
    }

    /* "tables/indexesextension.pyx":524
 *       mid = (lo+hi)//2
 *       if a[mid] < x: lo = mid+1
 *       else: hi = mid             # <<<<<<<<<<<<<<
 *   return lo
 * 
 */
    /*else*/ {
      __pyx_v_hi = __pyx_v_mid;
    }
    __pyx_L7:;
  }

  /* "tables/indexesextension.pyx":525
 *       if a[mid] < x: lo = mid+1
 *       else: hi = mid
 *   return lo             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_lo); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 525, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "tables/indexesextension.pyx":507
 * # because I forsee that this should be a very common case.
 * # 2004-05-20
 * def _bisect_left(a, x, int hi):             # <<<<<<<<<<<<<<
 *   """Return the index where to insert item x in list a, assuming a is sorted.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("tables.indexesextension._bisect_left", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/indexesextension.pyx":528
 * 
 * 
 * def _bisect_right(a, x, int hi):             # <<<<<<<<<<<<<<
 *   """Return the index where to insert item x in list a, assuming a is sorted.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_16indexesextension_5_bisect_right(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6tables_16indexesextension_4_bisect_right[] = "Return the index where to insert item x in list a, assuming a is sorted.\n\n  The return value i is such that all e in a[:i] have e <= x, and all e in\n  a[i:] have e > x.  So if x already appears in the list, i points just\n  beyond the rightmost x already there.\n\n  ";
static PyMethodDef __pyx_mdef_6tables_16indexesextension_5_bisect_right = {"_bisect_right", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6tables_16indexesextension_5_bisect_right, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6tables_16indexesextension_4_bisect_right};
static PyObject *__pyx_pw_6tables_16indexesextension_5_bisect_right(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_a = 0;
  PyObject *__pyx_v_x = 0;
  int __pyx_v_hi;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_bisect_right (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_a,&__pyx_n_s_x,&__pyx_n_s_hi,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_a)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_bisect_right", 1, 3, 3, 1); __PYX_ERR(0, 528, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_hi)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_bisect_right", 1, 3, 3, 2); __PYX_ERR(0, 528, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_bisect_right") < 0)) __PYX_ERR(0, 528, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_a = values[0];
    __pyx_v_x = values[1];
    __pyx_v_hi = __Pyx_PyInt_As_int(values[2]); if (unlikely((__pyx_v_hi == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 528, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_bisect_right", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 528, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("tables.indexesextension._bisect_right", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_16indexesextension_4_bisect_right(__pyx_self, __pyx_v_a, __pyx_v_x, __pyx_v_hi);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_16indexesextension_4_bisect_right(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_a, PyObject *__pyx_v_x, int __pyx_v_hi) {
  int __pyx_v_lo;
  int __pyx_v_mid;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_bisect_right", 0);

  /* "tables/indexesextension.pyx":539
 *   cdef int lo, mid
 * 
 *   lo = 0             # <<<<<<<<<<<<<<
 *   if x < a[0]: return 0
 *   if a[-1] <= x: return hi
 */
  __pyx_v_lo = 0;

  /* "tables/indexesextension.pyx":540
 * 
 *   lo = 0
 *   if x < a[0]: return 0             # <<<<<<<<<<<<<<
 *   if a[-1] <= x: return hi
 *   while lo < hi:
 */
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_a, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 540, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_RichCompare(__pyx_v_x, __pyx_t_1, Py_LT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 540, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 540, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_int_0);
    __pyx_r = __pyx_int_0;
    goto __pyx_L0;
  }

  /* "tables/indexesextension.pyx":541
 *   lo = 0
 *   if x < a[0]: return 0
 *   if a[-1] <= x: return hi             # <<<<<<<<<<<<<<
 *   while lo < hi:
 *     mid = (lo+hi)//2
 */
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_a, -1L, long, 1, __Pyx_PyInt_From_long, 0, 1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 541, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyObject_RichCompare(__pyx_t_2, __pyx_v_x, Py_LE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 541, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 541, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_hi); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 541, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
  }

  /* "tables/indexesextension.pyx":542
 *   if x < a[0]: return 0
 *   if a[-1] <= x: return hi
 *   while lo < hi:             # <<<<<<<<<<<<<<
 *     mid = (lo+hi)//2
 *     if x < a[mid]: hi = mid
 */
  while (1) {
    __pyx_t_3 = ((__pyx_v_lo < __pyx_v_hi) != 0);
    if (!__pyx_t_3) break;

    /* "tables/indexesextension.pyx":543
 *   if a[-1] <= x: return hi
 *   while lo < hi:
 *     mid = (lo+hi)//2             # <<<<<<<<<<<<<<
 *     if x < a[mid]: hi = mid
 *     else: lo = mid+1
 */
    __pyx_v_mid = __Pyx_div_long((__pyx_v_lo + __pyx_v_hi), 2);

    /* "tables/indexesextension.pyx":544
 *   while lo < hi:
 *     mid = (lo+hi)//2
 *     if x < a[mid]: hi = mid             # <<<<<<<<<<<<<<
 *     else: lo = mid+1
 *   return lo
 */
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_a, __pyx_v_mid, int, 1, __Pyx_PyInt_From_int, 0, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 544, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyObject_RichCompare(__pyx_v_x, __pyx_t_1, Py_LT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 544, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 544, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_3) {
      __pyx_v_hi = __pyx_v_mid;
      goto __pyx_L7;
    }

    /* "tables/indexesextension.pyx":545
 *     mid = (lo+hi)//2
 *     if x < a[mid]: hi = mid
 *     else: lo = mid+1             # <<<<<<<<<<<<<<
 *   return lo
 * 
 */
    /*else*/ {
      __pyx_v_lo = (__pyx_v_mid + 1);
    }
    __pyx_L7:;
  }

  /* "tables/indexesextension.pyx":546
 *     if x < a[mid]: hi = mid
 *     else: lo = mid+1
 *   return lo             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_lo); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 546, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "tables/indexesextension.pyx":528
 * 
 * 
 * def _bisect_right(a, x, int hi):             # <<<<<<<<<<<<<<
 *   """Return the index where to insert item x in list a, assuming a is sorted.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("tables.indexesextension._bisect_right", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_16indexesextension_5Index_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_6tables_16indexesextension_5Index_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6tables_16indexesextension_5Index___reduce_cython__(((struct __pyx_obj_6tables_16indexesextension_Index *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_16indexesextension_5Index___reduce_cython__(struct __pyx_obj_6tables_16indexesextension_Index *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = ()             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __Pyx_INCREF(__pyx_empty_tuple);
  __pyx_v_state = __pyx_empty_tuple;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = ()
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_1 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v__dict = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":7
 *     state = ()
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_2 = (__pyx_v__dict != Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v__dict);
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = False
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = ()
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = False             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_Index, (type(self), 0xd41d8cd, None), state
 */
  /*else*/ {
    __pyx_v_use_setstate = 0;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = False
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Index, (type(self), 0xd41d8cd, None), state
 *     else:
 */
  __pyx_t_3 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":13
 *         use_setstate = False
 *     if use_setstate:
 *         return __pyx_unpickle_Index, (type(self), 0xd41d8cd, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_Index, (type(self), 0xd41d8cd, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_Index); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_222419149);
    __Pyx_GIVEREF(__pyx_int_222419149);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_222419149);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_1, 2, Py_None);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state);
    __pyx_t_4 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = False
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Index, (type(self), 0xd41d8cd, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_Index, (type(self), 0xd41d8cd, None), state
 *     else:
 *         return __pyx_unpickle_Index, (type(self), 0xd41d8cd, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Index__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_Index); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_222419149);
    __Pyx_GIVEREF(__pyx_int_222419149);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_222419149);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
    __pyx_t_5 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("tables.indexesextension.Index.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_Index, (type(self), 0xd41d8cd, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Index__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_16indexesextension_5Index_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_6tables_16indexesextension_5Index_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6tables_16indexesextension_5Index_2__setstate_cython__(((struct __pyx_obj_6tables_16indexesextension_Index *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_16indexesextension_5Index_2__setstate_cython__(struct __pyx_obj_6tables_16indexesextension_Index *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_Index, (type(self), 0xd41d8cd, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Index__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6tables_16indexesextension___pyx_unpickle_Index__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_Index, (type(self), 0xd41d8cd, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Index__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("tables.indexesextension.Index.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/indexesextension.pyx":564
 *   cdef hid_t mem_space_id
 * 
 *   cdef initread(self, int nbounds):             # <<<<<<<<<<<<<<
 *     # "Actions to accelerate the reads afterwards."
 * 
 */

static PyObject *__pyx_f_6tables_16indexesextension_10CacheArray_initread(struct __pyx_obj_6tables_16indexesextension_CacheArray *__pyx_v_self, int __pyx_v_nbounds) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("initread", 0);

  /* "tables/indexesextension.pyx":569
 *     # Precompute the mem_space_id
 *     if (H5ARRAYOinit_readSlice(self.dataset_id, &self.mem_space_id,
 *                                nbounds) < 0):             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems initializing the bounds array data.")
 *     return
 */
  __pyx_t_1 = ((H5ARRAYOinit_readSlice(__pyx_v_self->__pyx_base.__pyx_base.dataset_id, (&__pyx_v_self->mem_space_id), __pyx_v_nbounds) < 0) != 0);

  /* "tables/indexesextension.pyx":568
 * 
 *     # Precompute the mem_space_id
 *     if (H5ARRAYOinit_readSlice(self.dataset_id, &self.mem_space_id,             # <<<<<<<<<<<<<<
 *                                nbounds) < 0):
 *       raise HDF5ExtError("Problems initializing the bounds array data.")
 */
  if (unlikely(__pyx_t_1)) {

    /* "tables/indexesextension.pyx":570
 *     if (H5ARRAYOinit_readSlice(self.dataset_id, &self.mem_space_id,
 *                                nbounds) < 0):
 *       raise HDF5ExtError("Problems initializing the bounds array data.")             # <<<<<<<<<<<<<<
 *     return
 * 
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_HDF5ExtError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 570, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_kp_s_Problems_initializing_the_bounds) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_kp_s_Problems_initializing_the_bounds);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 570, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 570, __pyx_L1_error)

    /* "tables/indexesextension.pyx":568
 * 
 *     # Precompute the mem_space_id
 *     if (H5ARRAYOinit_readSlice(self.dataset_id, &self.mem_space_id,             # <<<<<<<<<<<<<<
 *                                nbounds) < 0):
 *       raise HDF5ExtError("Problems initializing the bounds array data.")
 */
  }

  /* "tables/indexesextension.pyx":571
 *                                nbounds) < 0):
 *       raise HDF5ExtError("Problems initializing the bounds array data.")
 *     return             # <<<<<<<<<<<<<<
 * 
 *   cdef read_slice(self, hsize_t nrow, hsize_t start, hsize_t stop, void *rbuf):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "tables/indexesextension.pyx":564
 *   cdef hid_t mem_space_id
 * 
 *   cdef initread(self, int nbounds):             # <<<<<<<<<<<<<<
 *     # "Actions to accelerate the reads afterwards."
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("tables.indexesextension.CacheArray.initread", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/indexesextension.pyx":573
 *     return
 * 
 *   cdef read_slice(self, hsize_t nrow, hsize_t start, hsize_t stop, void *rbuf):             # <<<<<<<<<<<<<<
 *     # "Read an slice of bounds."
 * 
 */

static PyObject *__pyx_f_6tables_16indexesextension_10CacheArray_read_slice(struct __pyx_obj_6tables_16indexesextension_CacheArray *__pyx_v_self, hsize_t __pyx_v_nrow, hsize_t __pyx_v_start, hsize_t __pyx_v_stop, void *__pyx_v_rbuf) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("read_slice", 0);

  /* "tables/indexesextension.pyx":578
 *     if (H5ARRAYOread_readBoundsSlice(
 *       self.dataset_id, self.mem_space_id, self.type_id,
 *       nrow, start, stop, rbuf) < 0):             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems reading the bounds array data.")
 *     return
 */
  __pyx_t_1 = ((H5ARRAYOread_readBoundsSlice(__pyx_v_self->__pyx_base.__pyx_base.dataset_id, __pyx_v_self->mem_space_id, __pyx_v_self->__pyx_base.__pyx_base.type_id, __pyx_v_nrow, __pyx_v_start, __pyx_v_stop, __pyx_v_rbuf) < 0) != 0);

  /* "tables/indexesextension.pyx":576
 *     # "Read an slice of bounds."
 * 
 *     if (H5ARRAYOread_readBoundsSlice(             # <<<<<<<<<<<<<<
 *       self.dataset_id, self.mem_space_id, self.type_id,
 *       nrow, start, stop, rbuf) < 0):
 */
  if (unlikely(__pyx_t_1)) {

    /* "tables/indexesextension.pyx":579
 *       self.dataset_id, self.mem_space_id, self.type_id,
 *       nrow, start, stop, rbuf) < 0):
 *       raise HDF5ExtError("Problems reading the bounds array data.")             # <<<<<<<<<<<<<<
 *     return
 * 
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_HDF5ExtError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 579, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_kp_s_Problems_reading_the_bounds_arra) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_kp_s_Problems_reading_the_bounds_arra);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 579, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 579, __pyx_L1_error)

    /* "tables/indexesextension.pyx":576
 *     # "Read an slice of bounds."
 * 
 *     if (H5ARRAYOread_readBoundsSlice(             # <<<<<<<<<<<<<<
 *       self.dataset_id, self.mem_space_id, self.type_id,
 *       nrow, start, stop, rbuf) < 0):
 */
  }

  /* "tables/indexesextension.pyx":580
 *       nrow, start, stop, rbuf) < 0):
 *       raise HDF5ExtError("Problems reading the bounds array data.")
 *     return             # <<<<<<<<<<<<<<
 * 
 *   def _g_close(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "tables/indexesextension.pyx":573
 *     return
 * 
 *   cdef read_slice(self, hsize_t nrow, hsize_t start, hsize_t stop, void *rbuf):             # <<<<<<<<<<<<<<
 *     # "Read an slice of bounds."
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("tables.indexesextension.CacheArray.read_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/indexesextension.pyx":582
 *     return
 * 
 *   def _g_close(self):             # <<<<<<<<<<<<<<
 *     super()._g_close()
 *     # Release specific resources of this class
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_16indexesextension_10CacheArray_1_g_close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_6tables_16indexesextension_10CacheArray_1_g_close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_close (wrapper)", 0);
  __pyx_r = __pyx_pf_6tables_16indexesextension_10CacheArray__g_close(((struct __pyx_obj_6tables_16indexesextension_CacheArray *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_16indexesextension_10CacheArray__g_close(struct __pyx_obj_6tables_16indexesextension_CacheArray *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_g_close", 0);

  /* "tables/indexesextension.pyx":583
 * 
 *   def _g_close(self):
 *     super()._g_close()             # <<<<<<<<<<<<<<
 *     # Release specific resources of this class
 *     if self.mem_space_id > 0:
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 583, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6tables_16indexesextension_CacheArray));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6tables_16indexesextension_CacheArray));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6tables_16indexesextension_CacheArray));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 583, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_g_close); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 583, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 583, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "tables/indexesextension.pyx":585
 *     super()._g_close()
 *     # Release specific resources of this class
 *     if self.mem_space_id > 0:             # <<<<<<<<<<<<<<
 *       H5Sclose(self.mem_space_id)
 * 
 */
  __pyx_t_4 = ((__pyx_v_self->mem_space_id > 0) != 0);
  if (__pyx_t_4) {

    /* "tables/indexesextension.pyx":586
 *     # Release specific resources of this class
 *     if self.mem_space_id > 0:
 *       H5Sclose(self.mem_space_id)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    (void)(H5Sclose(__pyx_v_self->mem_space_id));

    /* "tables/indexesextension.pyx":585
 *     super()._g_close()
 *     # Release specific resources of this class
 *     if self.mem_space_id > 0:             # <<<<<<<<<<<<<<
 *       H5Sclose(self.mem_space_id)
 * 
 */
  }

  /* "tables/indexesextension.pyx":582
 *     return
 * 
 *   def _g_close(self):             # <<<<<<<<<<<<<<
 *     super()._g_close()
 *     # Release specific resources of this class
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("tables.indexesextension.CacheArray._g_close", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_16indexesextension_10CacheArray_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_6tables_16indexesextension_10CacheArray_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6tables_16indexesextension_10CacheArray_2__reduce_cython__(((struct __pyx_obj_6tables_16indexesextension_CacheArray *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_16indexesextension_10CacheArray_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_16indexesextension_CacheArray *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("tables.indexesextension.CacheArray.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_16indexesextension_10CacheArray_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_6tables_16indexesextension_10CacheArray_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6tables_16indexesextension_10CacheArray_4__setstate_cython__(((struct __pyx_obj_6tables_16indexesextension_CacheArray *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_16indexesextension_10CacheArray_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_16indexesextension_CacheArray *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("tables.indexesextension.CacheArray.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/indexesextension.pyx":603
 *   cdef ndarray bufferbc, bufferlb
 * 
 *   def _read_index_slice(self, hsize_t irow, hsize_t start, hsize_t stop,             # <<<<<<<<<<<<<<
 *                       ndarray idx):
 *     cdef herr_t ret
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_1_read_index_slice(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_1_read_index_slice(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  hsize_t __pyx_v_irow;
  hsize_t __pyx_v_start;
  hsize_t __pyx_v_stop;
  PyArrayObject *__pyx_v_idx = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_read_index_slice (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_irow,&__pyx_n_s_start,&__pyx_n_s_stop,&__pyx_n_s_idx,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_irow)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_start)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_read_index_slice", 1, 4, 4, 1); __PYX_ERR(0, 603, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_stop)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_read_index_slice", 1, 4, 4, 2); __PYX_ERR(0, 603, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_idx)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_read_index_slice", 1, 4, 4, 3); __PYX_ERR(0, 603, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_read_index_slice") < 0)) __PYX_ERR(0, 603, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_irow = __Pyx_PyInt_As_hsize_t(values[0]); if (unlikely((__pyx_v_irow == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 603, __pyx_L3_error)
    __pyx_v_start = __Pyx_PyInt_As_hsize_t(values[1]); if (unlikely((__pyx_v_start == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 603, __pyx_L3_error)
    __pyx_v_stop = __Pyx_PyInt_As_hsize_t(values[2]); if (unlikely((__pyx_v_stop == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 603, __pyx_L3_error)
    __pyx_v_idx = ((PyArrayObject *)values[3]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_read_index_slice", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 603, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("tables.indexesextension.IndexArray._read_index_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_idx), __pyx_ptype_5numpy_ndarray, 1, "idx", 0))) __PYX_ERR(0, 604, __pyx_L1_error)
  __pyx_r = __pyx_pf_6tables_16indexesextension_10IndexArray__read_index_slice(((struct __pyx_obj_6tables_16indexesextension_IndexArray *)__pyx_v_self), __pyx_v_irow, __pyx_v_start, __pyx_v_stop, __pyx_v_idx);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray__read_index_slice(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, hsize_t __pyx_v_irow, hsize_t __pyx_v_start, hsize_t __pyx_v_stop, PyArrayObject *__pyx_v_idx) {
  herr_t __pyx_v_ret;
  void *__pyx_v_buf;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_read_index_slice", 0);

  /* "tables/indexesextension.pyx":606
 *                       ndarray idx):
 *     cdef herr_t ret
 *     cdef void *buf = PyArray_DATA(idx)             # <<<<<<<<<<<<<<
 * 
 *     # Do the physical read
 */
  __pyx_v_buf = PyArray_DATA(__pyx_v_idx);

  /* "tables/indexesextension.pyx":609
 * 
 *     # Do the physical read
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5ARRAYOread_readSlice(self.dataset_id, self.type_id,
 *                                      irow, start, stop, buf)
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "tables/indexesextension.pyx":610
 *     # Do the physical read
 *     with nogil:
 *         ret = H5ARRAYOread_readSlice(self.dataset_id, self.type_id,             # <<<<<<<<<<<<<<
 *                                      irow, start, stop, buf)
 * 
 */
        __pyx_v_ret = H5ARRAYOread_readSlice(__pyx_v_self->__pyx_base.__pyx_base.dataset_id, __pyx_v_self->__pyx_base.__pyx_base.type_id, __pyx_v_irow, __pyx_v_start, __pyx_v_stop, __pyx_v_buf);
      }

      /* "tables/indexesextension.pyx":609
 * 
 *     # Do the physical read
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5ARRAYOread_readSlice(self.dataset_id, self.type_id,
 *                                      irow, start, stop, buf)
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "tables/indexesextension.pyx":613
 *                                      irow, start, stop, buf)
 * 
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems reading the index indices.")
 * 
 */
  __pyx_t_1 = ((__pyx_v_ret < 0) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "tables/indexesextension.pyx":614
 * 
 *     if ret < 0:
 *       raise HDF5ExtError("Problems reading the index indices.")             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_HDF5ExtError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 614, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_kp_s_Problems_reading_the_index_indic) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_kp_s_Problems_reading_the_index_indic);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 614, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 614, __pyx_L1_error)

    /* "tables/indexesextension.pyx":613
 *                                      irow, start, stop, buf)
 * 
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems reading the index indices.")
 * 
 */
  }

  /* "tables/indexesextension.pyx":603
 *   cdef ndarray bufferbc, bufferlb
 * 
 *   def _read_index_slice(self, hsize_t irow, hsize_t start, hsize_t stop,             # <<<<<<<<<<<<<<
 *                       ndarray idx):
 *     cdef herr_t ret
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("tables.indexesextension.IndexArray._read_index_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/indexesextension.pyx":617
 * 
 * 
 *   def _init_sorted_slice(self, index):             # <<<<<<<<<<<<<<
 *     """Initialize the structures for doing a binary search."""
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_3_init_sorted_slice(PyObject *__pyx_v_self, PyObject *__pyx_v_index); /*proto*/
static char __pyx_doc_6tables_16indexesextension_10IndexArray_2_init_sorted_slice[] = "Initialize the structures for doing a binary search.";
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_3_init_sorted_slice(PyObject *__pyx_v_self, PyObject *__pyx_v_index) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_init_sorted_slice (wrapper)", 0);
  __pyx_r = __pyx_pf_6tables_16indexesextension_10IndexArray_2_init_sorted_slice(((struct __pyx_obj_6tables_16indexesextension_IndexArray *)__pyx_v_self), ((PyObject *)__pyx_v_index));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_2_init_sorted_slice(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, PyObject *__pyx_v_index) {
  int __pyx_v_rank;
  hsize_t __pyx_v_count[2];
  PyArrayObject *__pyx_v_starts = 0;
  PyArrayObject *__pyx_v_lengths = 0;
  PyArrayObject *__pyx_v_rvcache = 0;
  PyObject *__pyx_v_maxslots = 0;
  PyObject *__pyx_v_rowsize = 0;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_params = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  hsize_t __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_init_sorted_slice", 0);

  /* "tables/indexesextension.pyx":627
 *     cdef object maxslots, rowsize
 * 
 *     dtype = self.atom.dtype             # <<<<<<<<<<<<<<
 *     # Create the buffer for reading sorted data chunks if not created yet
 *     if <object>self.bufferlb is None:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_atom); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 627, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 627, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_dtype = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "tables/indexesextension.pyx":629
 *     dtype = self.atom.dtype
 *     # Create the buffer for reading sorted data chunks if not created yet
 *     if <object>self.bufferlb is None:             # <<<<<<<<<<<<<<
 *       # Internal buffers
 *       self.bufferlb = numpy.empty(dtype=dtype, shape=self.chunksize)
 */
  __pyx_t_3 = (((PyObject *)__pyx_v_self->bufferlb) == Py_None);
  __pyx_t_4 = (__pyx_t_3 != 0);
  if (__pyx_t_4) {

    /* "tables/indexesextension.pyx":631
 *     if <object>self.bufferlb is None:
 *       # Internal buffers
 *       self.bufferlb = numpy.empty(dtype=dtype, shape=self.chunksize)             # <<<<<<<<<<<<<<
 *       # Get the pointers to the different buffer data areas
 *       self.rbuflb = PyArray_DATA(self.bufferlb)
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_numpy); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 631, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 631, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 631, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_v_dtype) < 0) __PYX_ERR(0, 631, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_chunksize); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 631, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_shape, __pyx_t_5) < 0) __PYX_ERR(0, 631, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 631, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (!(likely(((__pyx_t_5) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_5, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 631, __pyx_L1_error)
    __Pyx_GIVEREF(__pyx_t_5);
    __Pyx_GOTREF(__pyx_v_self->bufferlb);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->bufferlb));
    __pyx_v_self->bufferlb = ((PyArrayObject *)__pyx_t_5);
    __pyx_t_5 = 0;

    /* "tables/indexesextension.pyx":633
 *       self.bufferlb = numpy.empty(dtype=dtype, shape=self.chunksize)
 *       # Get the pointers to the different buffer data areas
 *       self.rbuflb = PyArray_DATA(self.bufferlb)             # <<<<<<<<<<<<<<
 *       # Init structures for accelerating sorted array reads
 *       rank = 2
 */
    __pyx_t_5 = ((PyObject *)__pyx_v_self->bufferlb);
    __Pyx_INCREF(__pyx_t_5);
    __pyx_v_self->rbuflb = PyArray_DATA(((PyArrayObject *)__pyx_t_5));
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "tables/indexesextension.pyx":635
 *       self.rbuflb = PyArray_DATA(self.bufferlb)
 *       # Init structures for accelerating sorted array reads
 *       rank = 2             # <<<<<<<<<<<<<<
 *       count[0] = 1
 *       count[1] = self.chunksize
 */
    __pyx_v_rank = 2;

    /* "tables/indexesextension.pyx":636
 *       # Init structures for accelerating sorted array reads
 *       rank = 2
 *       count[0] = 1             # <<<<<<<<<<<<<<
 *       count[1] = self.chunksize
 *       self.mem_space_id = H5Screate_simple(rank, count, NULL)
 */
    (__pyx_v_count[0]) = 1;

    /* "tables/indexesextension.pyx":637
 *       rank = 2
 *       count[0] = 1
 *       count[1] = self.chunksize             # <<<<<<<<<<<<<<
 *       self.mem_space_id = H5Screate_simple(rank, count, NULL)
 *       # Cache some counters in local extension variables
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_chunksize); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 637, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyInt_As_hsize_t(__pyx_t_5); if (unlikely((__pyx_t_6 == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 637, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    (__pyx_v_count[1]) = __pyx_t_6;

    /* "tables/indexesextension.pyx":638
 *       count[0] = 1
 *       count[1] = self.chunksize
 *       self.mem_space_id = H5Screate_simple(rank, count, NULL)             # <<<<<<<<<<<<<<
 *       # Cache some counters in local extension variables
 *       self.l_chunksize = self.chunksize
 */
    __pyx_v_self->mem_space_id = H5Screate_simple(__pyx_v_rank, __pyx_v_count, NULL);

    /* "tables/indexesextension.pyx":640
 *       self.mem_space_id = H5Screate_simple(rank, count, NULL)
 *       # Cache some counters in local extension variables
 *       self.l_chunksize = self.chunksize             # <<<<<<<<<<<<<<
 *       self.l_slicesize = self.slicesize
 * 
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_chunksize); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 640, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_7 = __Pyx_PyInt_As_int(__pyx_t_5); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 640, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_self->l_chunksize = __pyx_t_7;

    /* "tables/indexesextension.pyx":641
 *       # Cache some counters in local extension variables
 *       self.l_chunksize = self.chunksize
 *       self.l_slicesize = self.slicesize             # <<<<<<<<<<<<<<
 * 
 *     # Get the addresses of buffer data
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_slicesize); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 641, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_7 = __Pyx_PyInt_As_int(__pyx_t_5); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 641, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_self->l_slicesize = __pyx_t_7;

    /* "tables/indexesextension.pyx":629
 *     dtype = self.atom.dtype
 *     # Create the buffer for reading sorted data chunks if not created yet
 *     if <object>self.bufferlb is None:             # <<<<<<<<<<<<<<
 *       # Internal buffers
 *       self.bufferlb = numpy.empty(dtype=dtype, shape=self.chunksize)
 */
  }

  /* "tables/indexesextension.pyx":644
 * 
 *     # Get the addresses of buffer data
 *     starts = index.starts             # <<<<<<<<<<<<<<
 *     lengths = index.lengths
 *     self.rbufst = PyArray_DATA(starts)
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_starts); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 644, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (!(likely(((__pyx_t_5) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_5, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 644, __pyx_L1_error)
  __pyx_v_starts = ((PyArrayObject *)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "tables/indexesextension.pyx":645
 *     # Get the addresses of buffer data
 *     starts = index.starts
 *     lengths = index.lengths             # <<<<<<<<<<<<<<
 *     self.rbufst = PyArray_DATA(starts)
 *     self.rbufln = PyArray_DATA(lengths)
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_lengths); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 645, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (!(likely(((__pyx_t_5) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_5, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 645, __pyx_L1_error)
  __pyx_v_lengths = ((PyArrayObject *)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "tables/indexesextension.pyx":646
 *     starts = index.starts
 *     lengths = index.lengths
 *     self.rbufst = PyArray_DATA(starts)             # <<<<<<<<<<<<<<
 *     self.rbufln = PyArray_DATA(lengths)
 *     # The 1st cache is loaded completely in memory and needs to be reloaded
 */
  __pyx_v_self->rbufst = PyArray_DATA(__pyx_v_starts);

  /* "tables/indexesextension.pyx":647
 *     lengths = index.lengths
 *     self.rbufst = PyArray_DATA(starts)
 *     self.rbufln = PyArray_DATA(lengths)             # <<<<<<<<<<<<<<
 *     # The 1st cache is loaded completely in memory and needs to be reloaded
 *     rvcache = index.ranges[:]
 */
  __pyx_v_self->rbufln = PyArray_DATA(__pyx_v_lengths);

  /* "tables/indexesextension.pyx":649
 *     self.rbufln = PyArray_DATA(lengths)
 *     # The 1st cache is loaded completely in memory and needs to be reloaded
 *     rvcache = index.ranges[:]             # <<<<<<<<<<<<<<
 *     self.rbufrv = PyArray_DATA(rvcache)
 *     index.rvcache = <object>rvcache
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_ranges); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 649, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = __Pyx_PyObject_GetSlice(__pyx_t_5, 0, 0, NULL, NULL, &__pyx_slice__4, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 649, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 649, __pyx_L1_error)
  __pyx_v_rvcache = ((PyArrayObject *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "tables/indexesextension.pyx":650
 *     # The 1st cache is loaded completely in memory and needs to be reloaded
 *     rvcache = index.ranges[:]
 *     self.rbufrv = PyArray_DATA(rvcache)             # <<<<<<<<<<<<<<
 *     index.rvcache = <object>rvcache
 *     # Init the bounds array for reading
 */
  __pyx_v_self->rbufrv = PyArray_DATA(__pyx_v_rvcache);

  /* "tables/indexesextension.pyx":651
 *     rvcache = index.ranges[:]
 *     self.rbufrv = PyArray_DATA(rvcache)
 *     index.rvcache = <object>rvcache             # <<<<<<<<<<<<<<
 *     # Init the bounds array for reading
 *     self.nbounds = index.bounds.shape[1]
 */
  __pyx_t_2 = ((PyObject *)__pyx_v_rvcache);
  __Pyx_INCREF(__pyx_t_2);
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_index, __pyx_n_s_rvcache, __pyx_t_2) < 0) __PYX_ERR(0, 651, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/indexesextension.pyx":653
 *     index.rvcache = <object>rvcache
 *     # Init the bounds array for reading
 *     self.nbounds = index.bounds.shape[1]             # <<<<<<<<<<<<<<
 *     self.bounds_ext = <CacheArray>index.bounds
 *     self.bounds_ext.initread(self.nbounds)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_bounds); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 653, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_shape); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 653, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_t_5, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 653, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_7 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 653, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_self->nbounds = __pyx_t_7;

  /* "tables/indexesextension.pyx":654
 *     # Init the bounds array for reading
 *     self.nbounds = index.bounds.shape[1]
 *     self.bounds_ext = <CacheArray>index.bounds             # <<<<<<<<<<<<<<
 *     self.bounds_ext.initread(self.nbounds)
 *     if str(dtype) in self._v_parent.opt_search_types:
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_bounds); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 654, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __pyx_t_2;
  __Pyx_INCREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GIVEREF(__pyx_t_5);
  __Pyx_GOTREF(__pyx_v_self->bounds_ext);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->bounds_ext));
  __pyx_v_self->bounds_ext = ((struct __pyx_obj_6tables_16indexesextension_CacheArray *)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "tables/indexesextension.pyx":655
 *     self.nbounds = index.bounds.shape[1]
 *     self.bounds_ext = <CacheArray>index.bounds
 *     self.bounds_ext.initread(self.nbounds)             # <<<<<<<<<<<<<<
 *     if str(dtype) in self._v_parent.opt_search_types:
 *       # The next caches should be defined only for optimized search types.
 */
  __pyx_t_5 = ((struct __pyx_vtabstruct_6tables_16indexesextension_CacheArray *)__pyx_v_self->bounds_ext->__pyx_base.__pyx_base.__pyx_vtab)->initread(__pyx_v_self->bounds_ext, __pyx_v_self->nbounds); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 655, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "tables/indexesextension.pyx":656
 *     self.bounds_ext = <CacheArray>index.bounds
 *     self.bounds_ext.initread(self.nbounds)
 *     if str(dtype) in self._v_parent.opt_search_types:             # <<<<<<<<<<<<<<
 *       # The next caches should be defined only for optimized search types.
 *       # The 2nd level cache will replace the already existing ObjectCache and
 */
  __pyx_t_5 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_v_dtype); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 656, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_v_parent); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 656, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_opt_search_types); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 656, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = (__Pyx_PySequence_ContainsTF(__pyx_t_5, __pyx_t_1, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 656, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (__pyx_t_4 != 0);
  if (__pyx_t_3) {

    /* "tables/indexesextension.pyx":661
 *       # already bound to the boundscache attribute. This way, the cache will
 *       # not be duplicated (I know, this smells badly, but anyway).
 *       params = self._v_file.params             # <<<<<<<<<<<<<<
 *       rowsize = (self.bounds_ext._v_chunkshape[1] * dtype.itemsize)
 *       maxslots = params['BOUNDS_MAX_SIZE'] // rowsize
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_v_file); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 661, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_params); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 661, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_params = __pyx_t_5;
    __pyx_t_5 = 0;

    /* "tables/indexesextension.pyx":662
 *       # not be duplicated (I know, this smells badly, but anyway).
 *       params = self._v_file.params
 *       rowsize = (self.bounds_ext._v_chunkshape[1] * dtype.itemsize)             # <<<<<<<<<<<<<<
 *       maxslots = params['BOUNDS_MAX_SIZE'] // rowsize
 *       self.boundscache = <NumCache>NumCache(
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->bounds_ext), __pyx_n_s_v_chunkshape); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 662, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_t_5, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 662, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 662, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = PyNumber_Multiply(__pyx_t_1, __pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 662, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_rowsize = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "tables/indexesextension.pyx":663
 *       params = self._v_file.params
 *       rowsize = (self.bounds_ext._v_chunkshape[1] * dtype.itemsize)
 *       maxslots = params['BOUNDS_MAX_SIZE'] // rowsize             # <<<<<<<<<<<<<<
 *       self.boundscache = <NumCache>NumCache(
 *         (maxslots, self.nbounds), dtype, 'non-opt types bounds')
 */
    __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_v_params, __pyx_n_s_BOUNDS_MAX_SIZE); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 663, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = PyNumber_FloorDivide(__pyx_t_2, __pyx_v_rowsize); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 663, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_maxslots = __pyx_t_5;
    __pyx_t_5 = 0;

    /* "tables/indexesextension.pyx":665
 *       maxslots = params['BOUNDS_MAX_SIZE'] // rowsize
 *       self.boundscache = <NumCache>NumCache(
 *         (maxslots, self.nbounds), dtype, 'non-opt types bounds')             # <<<<<<<<<<<<<<
 *       self.bufferbc = numpy.empty(dtype=dtype, shape=self.nbounds)
 *       # Get the pointer for the internal buffer for 2nd level cache
 */
    __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_self->nbounds); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 665, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 665, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v_maxslots);
    __Pyx_GIVEREF(__pyx_v_maxslots);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_maxslots);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_5);
    __pyx_t_5 = 0;

    /* "tables/indexesextension.pyx":664
 *       rowsize = (self.bounds_ext._v_chunkshape[1] * dtype.itemsize)
 *       maxslots = params['BOUNDS_MAX_SIZE'] // rowsize
 *       self.boundscache = <NumCache>NumCache(             # <<<<<<<<<<<<<<
 *         (maxslots, self.nbounds), dtype, 'non-opt types bounds')
 *       self.bufferbc = numpy.empty(dtype=dtype, shape=self.nbounds)
 */
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 664, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2);
    __Pyx_INCREF(__pyx_v_dtype);
    __Pyx_GIVEREF(__pyx_v_dtype);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_v_dtype);
    __Pyx_INCREF(__pyx_kp_s_non_opt_types_bounds);
    __Pyx_GIVEREF(__pyx_kp_s_non_opt_types_bounds);
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_kp_s_non_opt_types_bounds);
    __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6tables_17lrucacheextension_NumCache), __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 664, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __pyx_t_2;
    __Pyx_INCREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GIVEREF(__pyx_t_5);
    __Pyx_GOTREF(__pyx_v_self->boundscache);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->boundscache));
    __pyx_v_self->boundscache = ((struct __pyx_obj_6tables_17lrucacheextension_NumCache *)__pyx_t_5);
    __pyx_t_5 = 0;

    /* "tables/indexesextension.pyx":666
 *       self.boundscache = <NumCache>NumCache(
 *         (maxslots, self.nbounds), dtype, 'non-opt types bounds')
 *       self.bufferbc = numpy.empty(dtype=dtype, shape=self.nbounds)             # <<<<<<<<<<<<<<
 *       # Get the pointer for the internal buffer for 2nd level cache
 *       self.rbufbc = PyArray_DATA(self.bufferbc)
 */
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_numpy); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 666, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_empty); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 666, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 666, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_dtype, __pyx_v_dtype) < 0) __PYX_ERR(0, 666, __pyx_L1_error)
    __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->nbounds); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 666, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_shape, __pyx_t_1) < 0) __PYX_ERR(0, 666, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 666, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 666, __pyx_L1_error)
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->bufferbc);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->bufferbc));
    __pyx_v_self->bufferbc = ((PyArrayObject *)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "tables/indexesextension.pyx":668
 *       self.bufferbc = numpy.empty(dtype=dtype, shape=self.nbounds)
 *       # Get the pointer for the internal buffer for 2nd level cache
 *       self.rbufbc = PyArray_DATA(self.bufferbc)             # <<<<<<<<<<<<<<
 *       # Another NumCache for the sorted values
 *       rowsize = (self.chunksize*dtype.itemsize)
 */
    __pyx_t_1 = ((PyObject *)__pyx_v_self->bufferbc);
    __Pyx_INCREF(__pyx_t_1);
    __pyx_v_self->rbufbc = PyArray_DATA(((PyArrayObject *)__pyx_t_1));
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "tables/indexesextension.pyx":670
 *       self.rbufbc = PyArray_DATA(self.bufferbc)
 *       # Another NumCache for the sorted values
 *       rowsize = (self.chunksize*dtype.itemsize)             # <<<<<<<<<<<<<<
 *       maxslots = params['SORTED_MAX_SIZE'] // (self.chunksize*dtype.itemsize)
 *       self.sortedcache = <NumCache>NumCache(
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_chunksize); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 670, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 670, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = PyNumber_Multiply(__pyx_t_1, __pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 670, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF_SET(__pyx_v_rowsize, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "tables/indexesextension.pyx":671
 *       # Another NumCache for the sorted values
 *       rowsize = (self.chunksize*dtype.itemsize)
 *       maxslots = params['SORTED_MAX_SIZE'] // (self.chunksize*dtype.itemsize)             # <<<<<<<<<<<<<<
 *       self.sortedcache = <NumCache>NumCache(
 *         (maxslots, self.chunksize), dtype, 'sorted')
 */
    __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_v_params, __pyx_n_s_SORTED_MAX_SIZE); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 671, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_chunksize); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 671, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 671, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_8 = PyNumber_Multiply(__pyx_t_5, __pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 671, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyNumber_FloorDivide(__pyx_t_2, __pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 671, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF_SET(__pyx_v_maxslots, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "tables/indexesextension.pyx":673
 *       maxslots = params['SORTED_MAX_SIZE'] // (self.chunksize*dtype.itemsize)
 *       self.sortedcache = <NumCache>NumCache(
 *         (maxslots, self.chunksize), dtype, 'sorted')             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_chunksize); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 673, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_8 = PyTuple_New(2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 673, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_INCREF(__pyx_v_maxslots);
    __Pyx_GIVEREF(__pyx_v_maxslots);
    PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_v_maxslots);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "tables/indexesextension.pyx":672
 *       rowsize = (self.chunksize*dtype.itemsize)
 *       maxslots = params['SORTED_MAX_SIZE'] // (self.chunksize*dtype.itemsize)
 *       self.sortedcache = <NumCache>NumCache(             # <<<<<<<<<<<<<<
 *         (maxslots, self.chunksize), dtype, 'sorted')
 * 
 */
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 672, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_8);
    __Pyx_INCREF(__pyx_v_dtype);
    __Pyx_GIVEREF(__pyx_v_dtype);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_dtype);
    __Pyx_INCREF(__pyx_n_s_sorted);
    __Pyx_GIVEREF(__pyx_n_s_sorted);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_n_s_sorted);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6tables_17lrucacheextension_NumCache), __pyx_t_1, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 672, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __pyx_t_8;
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->sortedcache);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->sortedcache));
    __pyx_v_self->sortedcache = ((struct __pyx_obj_6tables_17lrucacheextension_NumCache *)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "tables/indexesextension.pyx":656
 *     self.bounds_ext = <CacheArray>index.bounds
 *     self.bounds_ext.initread(self.nbounds)
 *     if str(dtype) in self._v_parent.opt_search_types:             # <<<<<<<<<<<<<<
 *       # The next caches should be defined only for optimized search types.
 *       # The 2nd level cache will replace the already existing ObjectCache and
 */
  }

  /* "tables/indexesextension.pyx":617
 * 
 * 
 *   def _init_sorted_slice(self, index):             # <<<<<<<<<<<<<<
 *     """Initialize the structures for doing a binary search."""
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("tables.indexesextension.IndexArray._init_sorted_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_starts);
  __Pyx_XDECREF((PyObject *)__pyx_v_lengths);
  __Pyx_XDECREF((PyObject *)__pyx_v_rvcache);
  __Pyx_XDECREF(__pyx_v_maxslots);
  __Pyx_XDECREF(__pyx_v_rowsize);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_params);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/indexesextension.pyx":677
 * 
 * 
 *   cdef void *_g_read_sorted_slice(self, hsize_t irow, hsize_t start,             # <<<<<<<<<<<<<<
 *                                 hsize_t stop):
 *     """Read the sorted part of an index."""
 */

static void *__pyx_f_6tables_16indexesextension_10IndexArray__g_read_sorted_slice(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, hsize_t __pyx_v_irow, hsize_t __pyx_v_start, hsize_t __pyx_v_stop) {
  herr_t __pyx_v_ret;
  void *__pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_g_read_sorted_slice", 0);

  /* "tables/indexesextension.pyx":681
 *     """Read the sorted part of an index."""
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5ARRAYOread_readSortedSlice(
 *           self.dataset_id, self.mem_space_id, self.type_id,
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "tables/indexesextension.pyx":682
 * 
 *     with nogil:
 *         ret = H5ARRAYOread_readSortedSlice(             # <<<<<<<<<<<<<<
 *           self.dataset_id, self.mem_space_id, self.type_id,
 *           irow, start, stop, self.rbuflb)
 */
        __pyx_v_ret = H5ARRAYOread_readSortedSlice(__pyx_v_self->__pyx_base.__pyx_base.dataset_id, __pyx_v_self->mem_space_id, __pyx_v_self->__pyx_base.__pyx_base.type_id, __pyx_v_irow, __pyx_v_start, __pyx_v_stop, __pyx_v_self->rbuflb);
      }

      /* "tables/indexesextension.pyx":681
 *     """Read the sorted part of an index."""
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5ARRAYOread_readSortedSlice(
 *           self.dataset_id, self.mem_space_id, self.type_id,
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "tables/indexesextension.pyx":686
 *           irow, start, stop, self.rbuflb)
 * 
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems reading the array data.")
 * 
 */
  __pyx_t_1 = ((__pyx_v_ret < 0) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "tables/indexesextension.pyx":687
 * 
 *     if ret < 0:
 *       raise HDF5ExtError("Problems reading the array data.")             # <<<<<<<<<<<<<<
 * 
 *     return self.rbuflb
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_HDF5ExtError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 687, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_kp_s_Problems_reading_the_array_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_kp_s_Problems_reading_the_array_data);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 687, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 687, __pyx_L1_error)

    /* "tables/indexesextension.pyx":686
 *           irow, start, stop, self.rbuflb)
 * 
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems reading the array data.")
 * 
 */
  }

  /* "tables/indexesextension.pyx":689
 *       raise HDF5ExtError("Problems reading the array data.")
 * 
 *     return self.rbuflb             # <<<<<<<<<<<<<<
 * 
 *   # can't time machine since this function is cdef'd
 */
  __pyx_r = __pyx_v_self->rbuflb;
  goto __pyx_L0;

  /* "tables/indexesextension.pyx":677
 * 
 * 
 *   cdef void *_g_read_sorted_slice(self, hsize_t irow, hsize_t start,             # <<<<<<<<<<<<<<
 *                                 hsize_t stop):
 *     """Read the sorted part of an index."""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("tables.indexesextension.IndexArray._g_read_sorted_slice", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/indexesextension.pyx":695
 * 
 *   # This is callable from python
 *   def _read_sorted_slice(self, hsize_t irow, hsize_t start, hsize_t stop):             # <<<<<<<<<<<<<<
 *     """Read the sorted part of an index."""
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_5_read_sorted_slice(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6tables_16indexesextension_10IndexArray_4_read_sorted_slice[] = "Read the sorted part of an index.";
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_5_read_sorted_slice(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  hsize_t __pyx_v_irow;
  hsize_t __pyx_v_start;
  hsize_t __pyx_v_stop;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_read_sorted_slice (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_irow,&__pyx_n_s_start,&__pyx_n_s_stop,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_irow)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_start)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_read_sorted_slice", 1, 3, 3, 1); __PYX_ERR(0, 695, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_stop)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_read_sorted_slice", 1, 3, 3, 2); __PYX_ERR(0, 695, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_read_sorted_slice") < 0)) __PYX_ERR(0, 695, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_irow = __Pyx_PyInt_As_hsize_t(values[0]); if (unlikely((__pyx_v_irow == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 695, __pyx_L3_error)
    __pyx_v_start = __Pyx_PyInt_As_hsize_t(values[1]); if (unlikely((__pyx_v_start == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 695, __pyx_L3_error)
    __pyx_v_stop = __Pyx_PyInt_As_hsize_t(values[2]); if (unlikely((__pyx_v_stop == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 695, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_read_sorted_slice", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 695, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("tables.indexesextension.IndexArray._read_sorted_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_16indexesextension_10IndexArray_4_read_sorted_slice(((struct __pyx_obj_6tables_16indexesextension_IndexArray *)__pyx_v_self), __pyx_v_irow, __pyx_v_start, __pyx_v_stop);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_4_read_sorted_slice(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, hsize_t __pyx_v_irow, hsize_t __pyx_v_start, hsize_t __pyx_v_stop) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_read_sorted_slice", 0);

  /* "tables/indexesextension.pyx":698
 *     """Read the sorted part of an index."""
 * 
 *     self._g_read_sorted_slice(irow, start, stop)             # <<<<<<<<<<<<<<
 *     return self.bufferlb
 * 
 */
  (void)(((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->_g_read_sorted_slice(__pyx_v_self, __pyx_v_irow, __pyx_v_start, __pyx_v_stop));

  /* "tables/indexesextension.pyx":699
 * 
 *     self._g_read_sorted_slice(irow, start, stop)
 *     return self.bufferlb             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->bufferlb));
  __pyx_r = ((PyObject *)__pyx_v_self->bufferlb);
  goto __pyx_L0;

  /* "tables/indexesextension.pyx":695
 * 
 *   # This is callable from python
 *   def _read_sorted_slice(self, hsize_t irow, hsize_t start, hsize_t stop):             # <<<<<<<<<<<<<<
 *     """Read the sorted part of an index."""
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/indexesextension.pyx":702
 * 
 * 
 *   cdef void *get_lru_bounds(self, int nrow, int nbounds):             # <<<<<<<<<<<<<<
 *     """Get the bounds from the cache, or read them."""
 * 
 */

static void *__pyx_f_6tables_16indexesextension_10IndexArray_get_lru_bounds(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, int __pyx_v_nrow, int __pyx_v_nbounds) {
  void *__pyx_v_vpointer;
  long __pyx_v_nslot;
  void *__pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  void *__pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_lru_bounds", 0);

  /* "tables/indexesextension.pyx":708
 *     cdef long nslot
 * 
 *     nslot = self.boundscache.getslot_(nrow)             # <<<<<<<<<<<<<<
 *     if nslot >= 0:
 *       vpointer = self.boundscache.getitem1_(nslot)
 */
  __pyx_v_nslot = ((struct __pyx_vtabstruct_6tables_17lrucacheextension_NumCache *)__pyx_v_self->boundscache->__pyx_base.__pyx_vtab)->getslot_(__pyx_v_self->boundscache, __pyx_v_nrow);

  /* "tables/indexesextension.pyx":709
 * 
 *     nslot = self.boundscache.getslot_(nrow)
 *     if nslot >= 0:             # <<<<<<<<<<<<<<
 *       vpointer = self.boundscache.getitem1_(nslot)
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_nslot >= 0) != 0);
  if (__pyx_t_1) {

    /* "tables/indexesextension.pyx":710
 *     nslot = self.boundscache.getslot_(nrow)
 *     if nslot >= 0:
 *       vpointer = self.boundscache.getitem1_(nslot)             # <<<<<<<<<<<<<<
 *     else:
 *       # Bounds row is not in cache. Read it and put it in the LRU cache.
 */
    __pyx_v_vpointer = ((struct __pyx_vtabstruct_6tables_17lrucacheextension_NumCache *)__pyx_v_self->boundscache->__pyx_base.__pyx_vtab)->getitem1_(__pyx_v_self->boundscache, __pyx_v_nslot);

    /* "tables/indexesextension.pyx":709
 * 
 *     nslot = self.boundscache.getslot_(nrow)
 *     if nslot >= 0:             # <<<<<<<<<<<<<<
 *       vpointer = self.boundscache.getitem1_(nslot)
 *     else:
 */
    goto __pyx_L3;
  }

  /* "tables/indexesextension.pyx":713
 *     else:
 *       # Bounds row is not in cache. Read it and put it in the LRU cache.
 *       self.bounds_ext.read_slice(nrow, 0, nbounds, self.rbufbc)             # <<<<<<<<<<<<<<
 *       self.boundscache.setitem_(nrow, self.rbufbc, 0)
 *       vpointer = self.rbufbc
 */
  /*else*/ {
    __pyx_t_2 = ((struct __pyx_vtabstruct_6tables_16indexesextension_CacheArray *)__pyx_v_self->bounds_ext->__pyx_base.__pyx_base.__pyx_vtab)->read_slice(__pyx_v_self->bounds_ext, __pyx_v_nrow, 0, __pyx_v_nbounds, __pyx_v_self->rbufbc); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 713, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "tables/indexesextension.pyx":714
 *       # Bounds row is not in cache. Read it and put it in the LRU cache.
 *       self.bounds_ext.read_slice(nrow, 0, nbounds, self.rbufbc)
 *       self.boundscache.setitem_(nrow, self.rbufbc, 0)             # <<<<<<<<<<<<<<
 *       vpointer = self.rbufbc
 *     return vpointer
 */
    (void)(((struct __pyx_vtabstruct_6tables_17lrucacheextension_NumCache *)__pyx_v_self->boundscache->__pyx_base.__pyx_vtab)->setitem_(__pyx_v_self->boundscache, __pyx_v_nrow, __pyx_v_self->rbufbc, 0));

    /* "tables/indexesextension.pyx":715
 *       self.bounds_ext.read_slice(nrow, 0, nbounds, self.rbufbc)
 *       self.boundscache.setitem_(nrow, self.rbufbc, 0)
 *       vpointer = self.rbufbc             # <<<<<<<<<<<<<<
 *     return vpointer
 * 
 */
    __pyx_t_3 = __pyx_v_self->rbufbc;
    __pyx_v_vpointer = __pyx_t_3;
  }
  __pyx_L3:;

  /* "tables/indexesextension.pyx":716
 *       self.boundscache.setitem_(nrow, self.rbufbc, 0)
 *       vpointer = self.rbufbc
 *     return vpointer             # <<<<<<<<<<<<<<
 * 
 *   # can't time machine since get_lru_bounds() function is cdef'd
 */
  __pyx_r = __pyx_v_vpointer;
  goto __pyx_L0;

  /* "tables/indexesextension.pyx":702
 * 
 * 
 *   cdef void *get_lru_bounds(self, int nrow, int nbounds):             # <<<<<<<<<<<<<<
 *     """Get the bounds from the cache, or read them."""
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_WriteUnraisable("tables.indexesextension.IndexArray.get_lru_bounds", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/indexesextension.pyx":720
 *   # can't time machine since get_lru_bounds() function is cdef'd
 * 
 *   cdef void *get_lru_sorted(self, int nrow, int ncs, int nchunk, int cs):             # <<<<<<<<<<<<<<
 *     """Get the sorted row from the cache or read it."""
 * 
 */

static void *__pyx_f_6tables_16indexesextension_10IndexArray_get_lru_sorted(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, int __pyx_v_nrow, int __pyx_v_ncs, int __pyx_v_nchunk, int __pyx_v_cs) {
  void *__pyx_v_vpointer;
  npy_int64 __pyx_v_nckey;
  long __pyx_v_nslot;
  hsize_t __pyx_v_start;
  hsize_t __pyx_v_stop;
  void *__pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("get_lru_sorted", 0);

  /* "tables/indexesextension.pyx":729
 * 
 *     # Compute the number of chunk read and use it as the key for the cache.
 *     nckey = nrow*ncs+nchunk             # <<<<<<<<<<<<<<
 *     nslot = self.sortedcache.getslot_(nckey)
 *     if nslot >= 0:
 */
  __pyx_v_nckey = ((__pyx_v_nrow * __pyx_v_ncs) + __pyx_v_nchunk);

  /* "tables/indexesextension.pyx":730
 *     # Compute the number of chunk read and use it as the key for the cache.
 *     nckey = nrow*ncs+nchunk
 *     nslot = self.sortedcache.getslot_(nckey)             # <<<<<<<<<<<<<<
 *     if nslot >= 0:
 *       vpointer = self.sortedcache.getitem1_(nslot)
 */
  __pyx_v_nslot = ((struct __pyx_vtabstruct_6tables_17lrucacheextension_NumCache *)__pyx_v_self->sortedcache->__pyx_base.__pyx_vtab)->getslot_(__pyx_v_self->sortedcache, __pyx_v_nckey);

  /* "tables/indexesextension.pyx":731
 *     nckey = nrow*ncs+nchunk
 *     nslot = self.sortedcache.getslot_(nckey)
 *     if nslot >= 0:             # <<<<<<<<<<<<<<
 *       vpointer = self.sortedcache.getitem1_(nslot)
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_nslot >= 0) != 0);
  if (__pyx_t_1) {

    /* "tables/indexesextension.pyx":732
 *     nslot = self.sortedcache.getslot_(nckey)
 *     if nslot >= 0:
 *       vpointer = self.sortedcache.getitem1_(nslot)             # <<<<<<<<<<<<<<
 *     else:
 *       # The sorted chunk is not in cache. Read it and put it in the LRU cache.
 */
    __pyx_v_vpointer = ((struct __pyx_vtabstruct_6tables_17lrucacheextension_NumCache *)__pyx_v_self->sortedcache->__pyx_base.__pyx_vtab)->getitem1_(__pyx_v_self->sortedcache, __pyx_v_nslot);

    /* "tables/indexesextension.pyx":731
 *     nckey = nrow*ncs+nchunk
 *     nslot = self.sortedcache.getslot_(nckey)
 *     if nslot >= 0:             # <<<<<<<<<<<<<<
 *       vpointer = self.sortedcache.getitem1_(nslot)
 *     else:
 */
    goto __pyx_L3;
  }

  /* "tables/indexesextension.pyx":735
 *     else:
 *       # The sorted chunk is not in cache. Read it and put it in the LRU cache.
 *       start = cs*nchunk             # <<<<<<<<<<<<<<
 *       stop = cs*(nchunk+1)
 *       vpointer = self._g_read_sorted_slice(nrow, start, stop)
 */
  /*else*/ {
    __pyx_v_start = (__pyx_v_cs * __pyx_v_nchunk);

    /* "tables/indexesextension.pyx":736
 *       # The sorted chunk is not in cache. Read it and put it in the LRU cache.
 *       start = cs*nchunk
 *       stop = cs*(nchunk+1)             # <<<<<<<<<<<<<<
 *       vpointer = self._g_read_sorted_slice(nrow, start, stop)
 *       self.sortedcache.setitem_(nckey, vpointer, 0)
 */
    __pyx_v_stop = (__pyx_v_cs * (__pyx_v_nchunk + 1));

    /* "tables/indexesextension.pyx":737
 *       start = cs*nchunk
 *       stop = cs*(nchunk+1)
 *       vpointer = self._g_read_sorted_slice(nrow, start, stop)             # <<<<<<<<<<<<<<
 *       self.sortedcache.setitem_(nckey, vpointer, 0)
 *     return vpointer
 */
    __pyx_v_vpointer = ((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->_g_read_sorted_slice(__pyx_v_self, __pyx_v_nrow, __pyx_v_start, __pyx_v_stop);

    /* "tables/indexesextension.pyx":738
 *       stop = cs*(nchunk+1)
 *       vpointer = self._g_read_sorted_slice(nrow, start, stop)
 *       self.sortedcache.setitem_(nckey, vpointer, 0)             # <<<<<<<<<<<<<<
 *     return vpointer
 * 
 */
    (void)(((struct __pyx_vtabstruct_6tables_17lrucacheextension_NumCache *)__pyx_v_self->sortedcache->__pyx_base.__pyx_vtab)->setitem_(__pyx_v_self->sortedcache, __pyx_v_nckey, __pyx_v_vpointer, 0));
  }
  __pyx_L3:;

  /* "tables/indexesextension.pyx":739
 *       vpointer = self._g_read_sorted_slice(nrow, start, stop)
 *       self.sortedcache.setitem_(nckey, vpointer, 0)
 *     return vpointer             # <<<<<<<<<<<<<<
 * 
 *   # can't time machine since get_lru_sorted() function is cdef'd
 */
  __pyx_r = __pyx_v_vpointer;
  goto __pyx_L0;

  /* "tables/indexesextension.pyx":720
 *   # can't time machine since get_lru_bounds() function is cdef'd
 * 
 *   cdef void *get_lru_sorted(self, int nrow, int ncs, int nchunk, int cs):             # <<<<<<<<<<<<<<
 *     """Get the sorted row from the cache or read it."""
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/indexesextension.pyx":744
 * 
 *   # Optimized version for int8
 *   def _search_bin_na_b(self, long item1, long item2):             # <<<<<<<<<<<<<<
 *     cdef int cs, ss, ncs, nrow, nrows, nbounds, rvrow
 *     cdef int start, stop, tlength, length, bread, nchunk, nchunk2
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_7_search_bin_na_b(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_7_search_bin_na_b(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  long __pyx_v_item1;
  long __pyx_v_item2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_search_bin_na_b (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_item1,&__pyx_n_s_item2,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_item1)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_item2)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_search_bin_na_b", 1, 2, 2, 1); __PYX_ERR(0, 744, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_search_bin_na_b") < 0)) __PYX_ERR(0, 744, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_item1 = __Pyx_PyInt_As_long(values[0]); if (unlikely((__pyx_v_item1 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 744, __pyx_L3_error)
    __pyx_v_item2 = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v_item2 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 744, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_search_bin_na_b", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 744, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("tables.indexesextension.IndexArray._search_bin_na_b", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_16indexesextension_10IndexArray_6_search_bin_na_b(((struct __pyx_obj_6tables_16indexesextension_IndexArray *)__pyx_v_self), __pyx_v_item1, __pyx_v_item2);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_6_search_bin_na_b(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, long __pyx_v_item1, long __pyx_v_item2) {
  int __pyx_v_cs;
  int __pyx_v_ss;
  int __pyx_v_ncs;
  int __pyx_v_nrow;
  int __pyx_v_nrows;
  int __pyx_v_nbounds;
  int __pyx_v_rvrow;
  int __pyx_v_start;
  int __pyx_v_stop;
  int __pyx_v_tlength;
  int __pyx_v_length;
  int __pyx_v_bread;
  int __pyx_v_nchunk;
  int __pyx_v_nchunk2;
  int *__pyx_v_rbufst;
  int *__pyx_v_rbufln;
  npy_int8 *__pyx_v_rbufrv;
  npy_int8 *__pyx_v_rbufbc;
  npy_int8 *__pyx_v_rbuflb;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_search_bin_na_b", 0);

  /* "tables/indexesextension.pyx":752
 *     # Variables with specific type
 *     cdef npy_int8 *rbufrv
 *     cdef npy_int8 *rbufbc = NULL             # <<<<<<<<<<<<<<
 *     cdef npy_int8 *rbuflb = NULL
 * 
 */
  __pyx_v_rbufbc = NULL;

  /* "tables/indexesextension.pyx":753
 *     cdef npy_int8 *rbufrv
 *     cdef npy_int8 *rbufbc = NULL
 *     cdef npy_int8 *rbuflb = NULL             # <<<<<<<<<<<<<<
 * 
 *     cs = self.l_chunksize
 */
  __pyx_v_rbuflb = NULL;

  /* "tables/indexesextension.pyx":755
 *     cdef npy_int8 *rbuflb = NULL
 * 
 *     cs = self.l_chunksize             # <<<<<<<<<<<<<<
 *     ss = self.l_slicesize
 *     ncs = ss // cs
 */
  __pyx_t_1 = __pyx_v_self->l_chunksize;
  __pyx_v_cs = __pyx_t_1;

  /* "tables/indexesextension.pyx":756
 * 
 *     cs = self.l_chunksize
 *     ss = self.l_slicesize             # <<<<<<<<<<<<<<
 *     ncs = ss // cs
 *     nbounds = self.nbounds
 */
  __pyx_t_1 = __pyx_v_self->l_slicesize;
  __pyx_v_ss = __pyx_t_1;

  /* "tables/indexesextension.pyx":757
 *     cs = self.l_chunksize
 *     ss = self.l_slicesize
 *     ncs = ss // cs             # <<<<<<<<<<<<<<
 *     nbounds = self.nbounds
 *     nrows = self.nrows
 */
  if (unlikely(__pyx_v_cs == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
    __PYX_ERR(0, 757, __pyx_L1_error)
  }
  else if (sizeof(int) == sizeof(long) && (!(((int)-1) > 0)) && unlikely(__pyx_v_cs == (int)-1)  && unlikely(UNARY_NEG_WOULD_OVERFLOW(__pyx_v_ss))) {
    PyErr_SetString(PyExc_OverflowError, "value too large to perform division");
    __PYX_ERR(0, 757, __pyx_L1_error)
  }
  __pyx_v_ncs = __Pyx_div_int(__pyx_v_ss, __pyx_v_cs);

  /* "tables/indexesextension.pyx":758
 *     ss = self.l_slicesize
 *     ncs = ss // cs
 *     nbounds = self.nbounds             # <<<<<<<<<<<<<<
 *     nrows = self.nrows
 *     rbufst = <int *>self.rbufst
 */
  __pyx_t_1 = __pyx_v_self->nbounds;
  __pyx_v_nbounds = __pyx_t_1;

  /* "tables/indexesextension.pyx":759
 *     ncs = ss // cs
 *     nbounds = self.nbounds
 *     nrows = self.nrows             # <<<<<<<<<<<<<<
 *     rbufst = <int *>self.rbufst
 *     rbufln = <int *>self.rbufln
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nrows); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 759, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 759, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_nrows = __pyx_t_1;

  /* "tables/indexesextension.pyx":760
 *     nbounds = self.nbounds
 *     nrows = self.nrows
 *     rbufst = <int *>self.rbufst             # <<<<<<<<<<<<<<
 *     rbufln = <int *>self.rbufln
 *     rbufrv = <npy_int8 *>self.rbufrv
 */
  __pyx_v_rbufst = ((int *)__pyx_v_self->rbufst);

  /* "tables/indexesextension.pyx":761
 *     nrows = self.nrows
 *     rbufst = <int *>self.rbufst
 *     rbufln = <int *>self.rbufln             # <<<<<<<<<<<<<<
 *     rbufrv = <npy_int8 *>self.rbufrv
 *     tlength = 0
 */
  __pyx_v_rbufln = ((int *)__pyx_v_self->rbufln);

  /* "tables/indexesextension.pyx":762
 *     rbufst = <int *>self.rbufst
 *     rbufln = <int *>self.rbufln
 *     rbufrv = <npy_int8 *>self.rbufrv             # <<<<<<<<<<<<<<
 *     tlength = 0
 *     for nrow from 0 <= nrow < nrows:
 */
  __pyx_v_rbufrv = ((npy_int8 *)__pyx_v_self->rbufrv);

  /* "tables/indexesextension.pyx":763
 *     rbufln = <int *>self.rbufln
 *     rbufrv = <npy_int8 *>self.rbufrv
 *     tlength = 0             # <<<<<<<<<<<<<<
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2
 */
  __pyx_v_tlength = 0;

  /* "tables/indexesextension.pyx":764
 *     rbufrv = <npy_int8 *>self.rbufrv
 *     tlength = 0
 *     for nrow from 0 <= nrow < nrows:             # <<<<<<<<<<<<<<
 *       rvrow = nrow*2
 *       bread = 0
 */
  __pyx_t_1 = __pyx_v_nrows;
  for (__pyx_v_nrow = 0; __pyx_v_nrow < __pyx_t_1; __pyx_v_nrow++) {

    /* "tables/indexesextension.pyx":765
 *     tlength = 0
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2             # <<<<<<<<<<<<<<
 *       bread = 0
 *       nchunk = -1
 */
    __pyx_v_rvrow = (__pyx_v_nrow * 2);

    /* "tables/indexesextension.pyx":766
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2
 *       bread = 0             # <<<<<<<<<<<<<<
 *       nchunk = -1
 * 
 */
    __pyx_v_bread = 0;

    /* "tables/indexesextension.pyx":767
 *       rvrow = nrow*2
 *       bread = 0
 *       nchunk = -1             # <<<<<<<<<<<<<<
 * 
 *       # Look if item1 is in this row
 */
    __pyx_v_nchunk = -1;

    /* "tables/indexesextension.pyx":770
 * 
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 */
    __pyx_t_3 = ((__pyx_v_item1 > (__pyx_v_rbufrv[__pyx_v_rvrow])) != 0);
    if (__pyx_t_3) {

      /* "tables/indexesextension.pyx":771
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:
 *         if item1 <= rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_int8 *>self.get_lru_bounds(nrow, nbounds)
 */
      __pyx_t_3 = ((__pyx_v_item1 <= (__pyx_v_rbufrv[(__pyx_v_rvrow + 1)])) != 0);
      if (__pyx_t_3) {

        /* "tables/indexesextension.pyx":773
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_int8 *>self.get_lru_bounds(nrow, nbounds)             # <<<<<<<<<<<<<<
 *           bread = 1
 *           nchunk = bisect_left_b(rbufbc, item1, nbounds, 0)
 */
        __pyx_v_rbufbc = ((npy_int8 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_bounds(__pyx_v_self, __pyx_v_nrow, __pyx_v_nbounds));

        /* "tables/indexesextension.pyx":774
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_int8 *>self.get_lru_bounds(nrow, nbounds)
 *           bread = 1             # <<<<<<<<<<<<<<
 *           nchunk = bisect_left_b(rbufbc, item1, nbounds, 0)
 *           # Get the sorted row from the LRU cache or read it.
 */
        __pyx_v_bread = 1;

        /* "tables/indexesextension.pyx":775
 *           rbufbc = <npy_int8 *>self.get_lru_bounds(nrow, nbounds)
 *           bread = 1
 *           nchunk = bisect_left_b(rbufbc, item1, nbounds, 0)             # <<<<<<<<<<<<<<
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_int8 *>self.get_lru_sorted(nrow, ncs, nchunk, cs)
 */
        __pyx_v_nchunk = bisect_left_b(__pyx_v_rbufbc, __pyx_v_item1, __pyx_v_nbounds, 0);

        /* "tables/indexesextension.pyx":777
 *           nchunk = bisect_left_b(rbufbc, item1, nbounds, 0)
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_int8 *>self.get_lru_sorted(nrow, ncs, nchunk, cs)             # <<<<<<<<<<<<<<
 *           start = bisect_left_b(rbuflb, item1, cs, 0) + cs*nchunk
 *         else:
 */
        __pyx_v_rbuflb = ((npy_int8 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_sorted(__pyx_v_self, __pyx_v_nrow, __pyx_v_ncs, __pyx_v_nchunk, __pyx_v_cs));

        /* "tables/indexesextension.pyx":778
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_int8 *>self.get_lru_sorted(nrow, ncs, nchunk, cs)
 *           start = bisect_left_b(rbuflb, item1, cs, 0) + cs*nchunk             # <<<<<<<<<<<<<<
 *         else:
 *           start = ss
 */
        __pyx_v_start = (bisect_left_b(__pyx_v_rbuflb, __pyx_v_item1, __pyx_v_cs, 0) + (__pyx_v_cs * __pyx_v_nchunk));

        /* "tables/indexesextension.pyx":771
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:
 *         if item1 <= rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_int8 *>self.get_lru_bounds(nrow, nbounds)
 */
        goto __pyx_L6;
      }

      /* "tables/indexesextension.pyx":780
 *           start = bisect_left_b(rbuflb, item1, cs, 0) + cs*nchunk
 *         else:
 *           start = ss             # <<<<<<<<<<<<<<
 *       else:
 *         start = 0
 */
      /*else*/ {
        __pyx_v_start = __pyx_v_ss;
      }
      __pyx_L6:;

      /* "tables/indexesextension.pyx":770
 * 
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 */
      goto __pyx_L5;
    }

    /* "tables/indexesextension.pyx":782
 *           start = ss
 *       else:
 *         start = 0             # <<<<<<<<<<<<<<
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 */
    /*else*/ {
      __pyx_v_start = 0;
    }
    __pyx_L5:;

    /* "tables/indexesextension.pyx":784
 *         start = 0
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:
 */
    __pyx_t_3 = ((__pyx_v_item2 >= (__pyx_v_rbufrv[__pyx_v_rvrow])) != 0);
    if (__pyx_t_3) {

      /* "tables/indexesextension.pyx":785
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 */
      __pyx_t_3 = ((__pyx_v_item2 < (__pyx_v_rbufrv[(__pyx_v_rvrow + 1)])) != 0);
      if (__pyx_t_3) {

        /* "tables/indexesextension.pyx":786
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:             # <<<<<<<<<<<<<<
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_int8 *>self.get_lru_bounds(nrow, nbounds)
 */
        __pyx_t_3 = ((!(__pyx_v_bread != 0)) != 0);
        if (__pyx_t_3) {

          /* "tables/indexesextension.pyx":788
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_int8 *>self.get_lru_bounds(nrow, nbounds)             # <<<<<<<<<<<<<<
 *           nchunk2 = bisect_right_b(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:
 */
          __pyx_v_rbufbc = ((npy_int8 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_bounds(__pyx_v_self, __pyx_v_nrow, __pyx_v_nbounds));

          /* "tables/indexesextension.pyx":786
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:             # <<<<<<<<<<<<<<
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_int8 *>self.get_lru_bounds(nrow, nbounds)
 */
        }

        /* "tables/indexesextension.pyx":789
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_int8 *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_b(rbufbc, item2, nbounds, 0)             # <<<<<<<<<<<<<<
 *           if nchunk2 <> nchunk:
 *             # Get the sorted row from the LRU cache or read it.
 */
        __pyx_v_nchunk2 = bisect_right_b(__pyx_v_rbufbc, __pyx_v_item2, __pyx_v_nbounds, 0);

        /* "tables/indexesextension.pyx":790
 *             rbufbc = <npy_int8 *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_b(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:             # <<<<<<<<<<<<<<
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_int8 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 */
        __pyx_t_3 = ((__pyx_v_nchunk2 != __pyx_v_nchunk) != 0);
        if (__pyx_t_3) {

          /* "tables/indexesextension.pyx":792
 *           if nchunk2 <> nchunk:
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_int8 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)             # <<<<<<<<<<<<<<
 *           stop = bisect_right_b(rbuflb, item2, cs, 0) + cs*nchunk2
 *         else:
 */
          __pyx_v_rbuflb = ((npy_int8 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_sorted(__pyx_v_self, __pyx_v_nrow, __pyx_v_ncs, __pyx_v_nchunk2, __pyx_v_cs));

          /* "tables/indexesextension.pyx":790
 *             rbufbc = <npy_int8 *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_b(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:             # <<<<<<<<<<<<<<
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_int8 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 */
        }

        /* "tables/indexesextension.pyx":793
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_int8 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 *           stop = bisect_right_b(rbuflb, item2, cs, 0) + cs*nchunk2             # <<<<<<<<<<<<<<
 *         else:
 *           stop = ss
 */
        __pyx_v_stop = (bisect_right_b(__pyx_v_rbuflb, __pyx_v_item2, __pyx_v_cs, 0) + (__pyx_v_cs * __pyx_v_nchunk2));

        /* "tables/indexesextension.pyx":785
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 */
        goto __pyx_L8;
      }

      /* "tables/indexesextension.pyx":795
 *           stop = bisect_right_b(rbuflb, item2, cs, 0) + cs*nchunk2
 *         else:
 *           stop = ss             # <<<<<<<<<<<<<<
 *       else:
 *         stop = 0
 */
      /*else*/ {
        __pyx_v_stop = __pyx_v_ss;
      }
      __pyx_L8:;

      /* "tables/indexesextension.pyx":784
 *         start = 0
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:
 */
      goto __pyx_L7;
    }

    /* "tables/indexesextension.pyx":797
 *           stop = ss
 *       else:
 *         stop = 0             # <<<<<<<<<<<<<<
 *       length = stop - start
 *       tlength = tlength + length
 */
    /*else*/ {
      __pyx_v_stop = 0;
    }
    __pyx_L7:;

    /* "tables/indexesextension.pyx":798
 *       else:
 *         stop = 0
 *       length = stop - start             # <<<<<<<<<<<<<<
 *       tlength = tlength + length
 *       rbufst[nrow] = start
 */
    __pyx_v_length = (__pyx_v_stop - __pyx_v_start);

    /* "tables/indexesextension.pyx":799
 *         stop = 0
 *       length = stop - start
 *       tlength = tlength + length             # <<<<<<<<<<<<<<
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length
 */
    __pyx_v_tlength = (__pyx_v_tlength + __pyx_v_length);

    /* "tables/indexesextension.pyx":800
 *       length = stop - start
 *       tlength = tlength + length
 *       rbufst[nrow] = start             # <<<<<<<<<<<<<<
 *       rbufln[nrow] = length
 *     return tlength
 */
    (__pyx_v_rbufst[__pyx_v_nrow]) = __pyx_v_start;

    /* "tables/indexesextension.pyx":801
 *       tlength = tlength + length
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length             # <<<<<<<<<<<<<<
 *     return tlength
 * 
 */
    (__pyx_v_rbufln[__pyx_v_nrow]) = __pyx_v_length;
  }

  /* "tables/indexesextension.pyx":802
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length
 *     return tlength             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_tlength); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 802, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "tables/indexesextension.pyx":744
 * 
 *   # Optimized version for int8
 *   def _search_bin_na_b(self, long item1, long item2):             # <<<<<<<<<<<<<<
 *     cdef int cs, ss, ncs, nrow, nrows, nbounds, rvrow
 *     cdef int start, stop, tlength, length, bread, nchunk, nchunk2
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("tables.indexesextension.IndexArray._search_bin_na_b", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/indexesextension.pyx":806
 * 
 *   # Optimized version for uint8
 *   def _search_bin_na_ub(self, long item1, long item2):             # <<<<<<<<<<<<<<
 *     cdef int cs, ss, ncs, nrow, nrows, nbounds, rvrow
 *     cdef int start, stop, tlength, length, bread, nchunk, nchunk2
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_9_search_bin_na_ub(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_9_search_bin_na_ub(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  long __pyx_v_item1;
  long __pyx_v_item2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_search_bin_na_ub (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_item1,&__pyx_n_s_item2,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_item1)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_item2)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_search_bin_na_ub", 1, 2, 2, 1); __PYX_ERR(0, 806, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_search_bin_na_ub") < 0)) __PYX_ERR(0, 806, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_item1 = __Pyx_PyInt_As_long(values[0]); if (unlikely((__pyx_v_item1 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 806, __pyx_L3_error)
    __pyx_v_item2 = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v_item2 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 806, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_search_bin_na_ub", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 806, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("tables.indexesextension.IndexArray._search_bin_na_ub", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_16indexesextension_10IndexArray_8_search_bin_na_ub(((struct __pyx_obj_6tables_16indexesextension_IndexArray *)__pyx_v_self), __pyx_v_item1, __pyx_v_item2);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_8_search_bin_na_ub(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, long __pyx_v_item1, long __pyx_v_item2) {
  int __pyx_v_cs;
  int __pyx_v_ss;
  int __pyx_v_ncs;
  int __pyx_v_nrow;
  int __pyx_v_nrows;
  int __pyx_v_nbounds;
  int __pyx_v_rvrow;
  int __pyx_v_start;
  int __pyx_v_stop;
  int __pyx_v_tlength;
  int __pyx_v_length;
  int __pyx_v_bread;
  int __pyx_v_nchunk;
  int __pyx_v_nchunk2;
  int *__pyx_v_rbufst;
  int *__pyx_v_rbufln;
  npy_uint8 *__pyx_v_rbufrv;
  npy_uint8 *__pyx_v_rbufbc;
  npy_uint8 *__pyx_v_rbuflb;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_search_bin_na_ub", 0);

  /* "tables/indexesextension.pyx":814
 *     # Variables with specific type
 *     cdef npy_uint8 *rbufrv
 *     cdef npy_uint8 *rbufbc = NULL             # <<<<<<<<<<<<<<
 *     cdef npy_uint8 *rbuflb = NULL
 * 
 */
  __pyx_v_rbufbc = NULL;

  /* "tables/indexesextension.pyx":815
 *     cdef npy_uint8 *rbufrv
 *     cdef npy_uint8 *rbufbc = NULL
 *     cdef npy_uint8 *rbuflb = NULL             # <<<<<<<<<<<<<<
 * 
 *     cs = self.l_chunksize
 */
  __pyx_v_rbuflb = NULL;

  /* "tables/indexesextension.pyx":817
 *     cdef npy_uint8 *rbuflb = NULL
 * 
 *     cs = self.l_chunksize             # <<<<<<<<<<<<<<
 *     ss = self.l_slicesize
 *     ncs = ss // cs
 */
  __pyx_t_1 = __pyx_v_self->l_chunksize;
  __pyx_v_cs = __pyx_t_1;

  /* "tables/indexesextension.pyx":818
 * 
 *     cs = self.l_chunksize
 *     ss = self.l_slicesize             # <<<<<<<<<<<<<<
 *     ncs = ss // cs
 *     nbounds = self.nbounds
 */
  __pyx_t_1 = __pyx_v_self->l_slicesize;
  __pyx_v_ss = __pyx_t_1;

  /* "tables/indexesextension.pyx":819
 *     cs = self.l_chunksize
 *     ss = self.l_slicesize
 *     ncs = ss // cs             # <<<<<<<<<<<<<<
 *     nbounds = self.nbounds
 *     nrows = self.nrows
 */
  if (unlikely(__pyx_v_cs == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
    __PYX_ERR(0, 819, __pyx_L1_error)
  }
  else if (sizeof(int) == sizeof(long) && (!(((int)-1) > 0)) && unlikely(__pyx_v_cs == (int)-1)  && unlikely(UNARY_NEG_WOULD_OVERFLOW(__pyx_v_ss))) {
    PyErr_SetString(PyExc_OverflowError, "value too large to perform division");
    __PYX_ERR(0, 819, __pyx_L1_error)
  }
  __pyx_v_ncs = __Pyx_div_int(__pyx_v_ss, __pyx_v_cs);

  /* "tables/indexesextension.pyx":820
 *     ss = self.l_slicesize
 *     ncs = ss // cs
 *     nbounds = self.nbounds             # <<<<<<<<<<<<<<
 *     nrows = self.nrows
 *     rbufst = <int *>self.rbufst
 */
  __pyx_t_1 = __pyx_v_self->nbounds;
  __pyx_v_nbounds = __pyx_t_1;

  /* "tables/indexesextension.pyx":821
 *     ncs = ss // cs
 *     nbounds = self.nbounds
 *     nrows = self.nrows             # <<<<<<<<<<<<<<
 *     rbufst = <int *>self.rbufst
 *     rbufln = <int *>self.rbufln
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nrows); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 821, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 821, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_nrows = __pyx_t_1;

  /* "tables/indexesextension.pyx":822
 *     nbounds = self.nbounds
 *     nrows = self.nrows
 *     rbufst = <int *>self.rbufst             # <<<<<<<<<<<<<<
 *     rbufln = <int *>self.rbufln
 *     rbufrv = <npy_uint8 *>self.rbufrv
 */
  __pyx_v_rbufst = ((int *)__pyx_v_self->rbufst);

  /* "tables/indexesextension.pyx":823
 *     nrows = self.nrows
 *     rbufst = <int *>self.rbufst
 *     rbufln = <int *>self.rbufln             # <<<<<<<<<<<<<<
 *     rbufrv = <npy_uint8 *>self.rbufrv
 *     tlength = 0
 */
  __pyx_v_rbufln = ((int *)__pyx_v_self->rbufln);

  /* "tables/indexesextension.pyx":824
 *     rbufst = <int *>self.rbufst
 *     rbufln = <int *>self.rbufln
 *     rbufrv = <npy_uint8 *>self.rbufrv             # <<<<<<<<<<<<<<
 *     tlength = 0
 *     for nrow from 0 <= nrow < nrows:
 */
  __pyx_v_rbufrv = ((npy_uint8 *)__pyx_v_self->rbufrv);

  /* "tables/indexesextension.pyx":825
 *     rbufln = <int *>self.rbufln
 *     rbufrv = <npy_uint8 *>self.rbufrv
 *     tlength = 0             # <<<<<<<<<<<<<<
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2
 */
  __pyx_v_tlength = 0;

  /* "tables/indexesextension.pyx":826
 *     rbufrv = <npy_uint8 *>self.rbufrv
 *     tlength = 0
 *     for nrow from 0 <= nrow < nrows:             # <<<<<<<<<<<<<<
 *       rvrow = nrow*2
 *       bread = 0
 */
  __pyx_t_1 = __pyx_v_nrows;
  for (__pyx_v_nrow = 0; __pyx_v_nrow < __pyx_t_1; __pyx_v_nrow++) {

    /* "tables/indexesextension.pyx":827
 *     tlength = 0
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2             # <<<<<<<<<<<<<<
 *       bread = 0
 *       nchunk = -1
 */
    __pyx_v_rvrow = (__pyx_v_nrow * 2);

    /* "tables/indexesextension.pyx":828
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2
 *       bread = 0             # <<<<<<<<<<<<<<
 *       nchunk = -1
 * 
 */
    __pyx_v_bread = 0;

    /* "tables/indexesextension.pyx":829
 *       rvrow = nrow*2
 *       bread = 0
 *       nchunk = -1             # <<<<<<<<<<<<<<
 * 
 *       # Look if item1 is in this row
 */
    __pyx_v_nchunk = -1;

    /* "tables/indexesextension.pyx":832
 * 
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 */
    __pyx_t_3 = ((__pyx_v_item1 > (__pyx_v_rbufrv[__pyx_v_rvrow])) != 0);
    if (__pyx_t_3) {

      /* "tables/indexesextension.pyx":833
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:
 *         if item1 <= rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_uint8 *>self.get_lru_bounds(nrow, nbounds)
 */
      __pyx_t_3 = ((__pyx_v_item1 <= (__pyx_v_rbufrv[(__pyx_v_rvrow + 1)])) != 0);
      if (__pyx_t_3) {

        /* "tables/indexesextension.pyx":835
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_uint8 *>self.get_lru_bounds(nrow, nbounds)             # <<<<<<<<<<<<<<
 *           bread = 1
 *           nchunk = bisect_left_ub(rbufbc, item1, nbounds, 0)
 */
        __pyx_v_rbufbc = ((npy_uint8 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_bounds(__pyx_v_self, __pyx_v_nrow, __pyx_v_nbounds));

        /* "tables/indexesextension.pyx":836
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_uint8 *>self.get_lru_bounds(nrow, nbounds)
 *           bread = 1             # <<<<<<<<<<<<<<
 *           nchunk = bisect_left_ub(rbufbc, item1, nbounds, 0)
 *           # Get the sorted row from the LRU cache or read it.
 */
        __pyx_v_bread = 1;

        /* "tables/indexesextension.pyx":837
 *           rbufbc = <npy_uint8 *>self.get_lru_bounds(nrow, nbounds)
 *           bread = 1
 *           nchunk = bisect_left_ub(rbufbc, item1, nbounds, 0)             # <<<<<<<<<<<<<<
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_uint8 *>self.get_lru_sorted(nrow, ncs, nchunk, cs)
 */
        __pyx_v_nchunk = bisect_left_ub(__pyx_v_rbufbc, __pyx_v_item1, __pyx_v_nbounds, 0);

        /* "tables/indexesextension.pyx":839
 *           nchunk = bisect_left_ub(rbufbc, item1, nbounds, 0)
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_uint8 *>self.get_lru_sorted(nrow, ncs, nchunk, cs)             # <<<<<<<<<<<<<<
 *           start = bisect_left_ub(rbuflb, item1, cs, 0) + cs*nchunk
 *         else:
 */
        __pyx_v_rbuflb = ((npy_uint8 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_sorted(__pyx_v_self, __pyx_v_nrow, __pyx_v_ncs, __pyx_v_nchunk, __pyx_v_cs));

        /* "tables/indexesextension.pyx":840
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_uint8 *>self.get_lru_sorted(nrow, ncs, nchunk, cs)
 *           start = bisect_left_ub(rbuflb, item1, cs, 0) + cs*nchunk             # <<<<<<<<<<<<<<
 *         else:
 *           start = ss
 */
        __pyx_v_start = (bisect_left_ub(__pyx_v_rbuflb, __pyx_v_item1, __pyx_v_cs, 0) + (__pyx_v_cs * __pyx_v_nchunk));

        /* "tables/indexesextension.pyx":833
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:
 *         if item1 <= rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_uint8 *>self.get_lru_bounds(nrow, nbounds)
 */
        goto __pyx_L6;
      }

      /* "tables/indexesextension.pyx":842
 *           start = bisect_left_ub(rbuflb, item1, cs, 0) + cs*nchunk
 *         else:
 *           start = ss             # <<<<<<<<<<<<<<
 *       else:
 *         start = 0
 */
      /*else*/ {
        __pyx_v_start = __pyx_v_ss;
      }
      __pyx_L6:;

      /* "tables/indexesextension.pyx":832
 * 
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 */
      goto __pyx_L5;
    }

    /* "tables/indexesextension.pyx":844
 *           start = ss
 *       else:
 *         start = 0             # <<<<<<<<<<<<<<
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 */
    /*else*/ {
      __pyx_v_start = 0;
    }
    __pyx_L5:;

    /* "tables/indexesextension.pyx":846
 *         start = 0
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:
 */
    __pyx_t_3 = ((__pyx_v_item2 >= (__pyx_v_rbufrv[__pyx_v_rvrow])) != 0);
    if (__pyx_t_3) {

      /* "tables/indexesextension.pyx":847
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 */
      __pyx_t_3 = ((__pyx_v_item2 < (__pyx_v_rbufrv[(__pyx_v_rvrow + 1)])) != 0);
      if (__pyx_t_3) {

        /* "tables/indexesextension.pyx":848
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:             # <<<<<<<<<<<<<<
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_uint8 *>self.get_lru_bounds(nrow, nbounds)
 */
        __pyx_t_3 = ((!(__pyx_v_bread != 0)) != 0);
        if (__pyx_t_3) {

          /* "tables/indexesextension.pyx":850
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_uint8 *>self.get_lru_bounds(nrow, nbounds)             # <<<<<<<<<<<<<<
 *           nchunk2 = bisect_right_ub(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:
 */
          __pyx_v_rbufbc = ((npy_uint8 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_bounds(__pyx_v_self, __pyx_v_nrow, __pyx_v_nbounds));

          /* "tables/indexesextension.pyx":848
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:             # <<<<<<<<<<<<<<
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_uint8 *>self.get_lru_bounds(nrow, nbounds)
 */
        }

        /* "tables/indexesextension.pyx":851
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_uint8 *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_ub(rbufbc, item2, nbounds, 0)             # <<<<<<<<<<<<<<
 *           if nchunk2 <> nchunk:
 *             # Get the sorted row from the LRU cache or read it.
 */
        __pyx_v_nchunk2 = bisect_right_ub(__pyx_v_rbufbc, __pyx_v_item2, __pyx_v_nbounds, 0);

        /* "tables/indexesextension.pyx":852
 *             rbufbc = <npy_uint8 *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_ub(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:             # <<<<<<<<<<<<<<
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_uint8 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 */
        __pyx_t_3 = ((__pyx_v_nchunk2 != __pyx_v_nchunk) != 0);
        if (__pyx_t_3) {

          /* "tables/indexesextension.pyx":854
 *           if nchunk2 <> nchunk:
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_uint8 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)             # <<<<<<<<<<<<<<
 *           stop = bisect_right_ub(rbuflb, item2, cs, 0) + cs*nchunk2
 *         else:
 */
          __pyx_v_rbuflb = ((npy_uint8 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_sorted(__pyx_v_self, __pyx_v_nrow, __pyx_v_ncs, __pyx_v_nchunk2, __pyx_v_cs));

          /* "tables/indexesextension.pyx":852
 *             rbufbc = <npy_uint8 *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_ub(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:             # <<<<<<<<<<<<<<
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_uint8 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 */
        }

        /* "tables/indexesextension.pyx":855
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_uint8 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 *           stop = bisect_right_ub(rbuflb, item2, cs, 0) + cs*nchunk2             # <<<<<<<<<<<<<<
 *         else:
 *           stop = ss
 */
        __pyx_v_stop = (bisect_right_ub(__pyx_v_rbuflb, __pyx_v_item2, __pyx_v_cs, 0) + (__pyx_v_cs * __pyx_v_nchunk2));

        /* "tables/indexesextension.pyx":847
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 */
        goto __pyx_L8;
      }

      /* "tables/indexesextension.pyx":857
 *           stop = bisect_right_ub(rbuflb, item2, cs, 0) + cs*nchunk2
 *         else:
 *           stop = ss             # <<<<<<<<<<<<<<
 *       else:
 *         stop = 0
 */
      /*else*/ {
        __pyx_v_stop = __pyx_v_ss;
      }
      __pyx_L8:;

      /* "tables/indexesextension.pyx":846
 *         start = 0
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:
 */
      goto __pyx_L7;
    }

    /* "tables/indexesextension.pyx":859
 *           stop = ss
 *       else:
 *         stop = 0             # <<<<<<<<<<<<<<
 *       length = stop - start
 *       tlength = tlength + length
 */
    /*else*/ {
      __pyx_v_stop = 0;
    }
    __pyx_L7:;

    /* "tables/indexesextension.pyx":860
 *       else:
 *         stop = 0
 *       length = stop - start             # <<<<<<<<<<<<<<
 *       tlength = tlength + length
 *       rbufst[nrow] = start
 */
    __pyx_v_length = (__pyx_v_stop - __pyx_v_start);

    /* "tables/indexesextension.pyx":861
 *         stop = 0
 *       length = stop - start
 *       tlength = tlength + length             # <<<<<<<<<<<<<<
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length
 */
    __pyx_v_tlength = (__pyx_v_tlength + __pyx_v_length);

    /* "tables/indexesextension.pyx":862
 *       length = stop - start
 *       tlength = tlength + length
 *       rbufst[nrow] = start             # <<<<<<<<<<<<<<
 *       rbufln[nrow] = length
 *     return tlength
 */
    (__pyx_v_rbufst[__pyx_v_nrow]) = __pyx_v_start;

    /* "tables/indexesextension.pyx":863
 *       tlength = tlength + length
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length             # <<<<<<<<<<<<<<
 *     return tlength
 * 
 */
    (__pyx_v_rbufln[__pyx_v_nrow]) = __pyx_v_length;
  }

  /* "tables/indexesextension.pyx":864
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length
 *     return tlength             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_tlength); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 864, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "tables/indexesextension.pyx":806
 * 
 *   # Optimized version for uint8
 *   def _search_bin_na_ub(self, long item1, long item2):             # <<<<<<<<<<<<<<
 *     cdef int cs, ss, ncs, nrow, nrows, nbounds, rvrow
 *     cdef int start, stop, tlength, length, bread, nchunk, nchunk2
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("tables.indexesextension.IndexArray._search_bin_na_ub", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/indexesextension.pyx":868
 * 
 *   # Optimized version for int16
 *   def _search_bin_na_s(self, long item1, long item2):             # <<<<<<<<<<<<<<
 *     cdef int cs, ss, ncs, nrow, nrows, nbounds, rvrow
 *     cdef int start, stop, tlength, length, bread, nchunk, nchunk2
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_11_search_bin_na_s(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_11_search_bin_na_s(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  long __pyx_v_item1;
  long __pyx_v_item2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_search_bin_na_s (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_item1,&__pyx_n_s_item2,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_item1)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_item2)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_search_bin_na_s", 1, 2, 2, 1); __PYX_ERR(0, 868, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_search_bin_na_s") < 0)) __PYX_ERR(0, 868, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_item1 = __Pyx_PyInt_As_long(values[0]); if (unlikely((__pyx_v_item1 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 868, __pyx_L3_error)
    __pyx_v_item2 = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v_item2 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 868, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_search_bin_na_s", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 868, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("tables.indexesextension.IndexArray._search_bin_na_s", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_16indexesextension_10IndexArray_10_search_bin_na_s(((struct __pyx_obj_6tables_16indexesextension_IndexArray *)__pyx_v_self), __pyx_v_item1, __pyx_v_item2);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_10_search_bin_na_s(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, long __pyx_v_item1, long __pyx_v_item2) {
  int __pyx_v_cs;
  int __pyx_v_ss;
  int __pyx_v_ncs;
  int __pyx_v_nrow;
  int __pyx_v_nrows;
  int __pyx_v_nbounds;
  int __pyx_v_rvrow;
  int __pyx_v_start;
  int __pyx_v_stop;
  int __pyx_v_tlength;
  int __pyx_v_length;
  int __pyx_v_bread;
  int __pyx_v_nchunk;
  int __pyx_v_nchunk2;
  int *__pyx_v_rbufst;
  int *__pyx_v_rbufln;
  npy_int16 *__pyx_v_rbufrv;
  npy_int16 *__pyx_v_rbufbc;
  npy_int16 *__pyx_v_rbuflb;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_search_bin_na_s", 0);

  /* "tables/indexesextension.pyx":876
 *     # Variables with specific type
 *     cdef npy_int16 *rbufrv
 *     cdef npy_int16 *rbufbc = NULL             # <<<<<<<<<<<<<<
 *     cdef npy_int16 *rbuflb = NULL
 * 
 */
  __pyx_v_rbufbc = NULL;

  /* "tables/indexesextension.pyx":877
 *     cdef npy_int16 *rbufrv
 *     cdef npy_int16 *rbufbc = NULL
 *     cdef npy_int16 *rbuflb = NULL             # <<<<<<<<<<<<<<
 * 
 *     cs = self.l_chunksize
 */
  __pyx_v_rbuflb = NULL;

  /* "tables/indexesextension.pyx":879
 *     cdef npy_int16 *rbuflb = NULL
 * 
 *     cs = self.l_chunksize             # <<<<<<<<<<<<<<
 *     ss = self.l_slicesize
 *     ncs = ss // cs
 */
  __pyx_t_1 = __pyx_v_self->l_chunksize;
  __pyx_v_cs = __pyx_t_1;

  /* "tables/indexesextension.pyx":880
 * 
 *     cs = self.l_chunksize
 *     ss = self.l_slicesize             # <<<<<<<<<<<<<<
 *     ncs = ss // cs
 *     nbounds = self.nbounds
 */
  __pyx_t_1 = __pyx_v_self->l_slicesize;
  __pyx_v_ss = __pyx_t_1;

  /* "tables/indexesextension.pyx":881
 *     cs = self.l_chunksize
 *     ss = self.l_slicesize
 *     ncs = ss // cs             # <<<<<<<<<<<<<<
 *     nbounds = self.nbounds
 *     nrows = self.nrows
 */
  if (unlikely(__pyx_v_cs == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
    __PYX_ERR(0, 881, __pyx_L1_error)
  }
  else if (sizeof(int) == sizeof(long) && (!(((int)-1) > 0)) && unlikely(__pyx_v_cs == (int)-1)  && unlikely(UNARY_NEG_WOULD_OVERFLOW(__pyx_v_ss))) {
    PyErr_SetString(PyExc_OverflowError, "value too large to perform division");
    __PYX_ERR(0, 881, __pyx_L1_error)
  }
  __pyx_v_ncs = __Pyx_div_int(__pyx_v_ss, __pyx_v_cs);

  /* "tables/indexesextension.pyx":882
 *     ss = self.l_slicesize
 *     ncs = ss // cs
 *     nbounds = self.nbounds             # <<<<<<<<<<<<<<
 *     nrows = self.nrows
 *     rbufst = <int *>self.rbufst
 */
  __pyx_t_1 = __pyx_v_self->nbounds;
  __pyx_v_nbounds = __pyx_t_1;

  /* "tables/indexesextension.pyx":883
 *     ncs = ss // cs
 *     nbounds = self.nbounds
 *     nrows = self.nrows             # <<<<<<<<<<<<<<
 *     rbufst = <int *>self.rbufst
 *     rbufln = <int *>self.rbufln
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nrows); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 883, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 883, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_nrows = __pyx_t_1;

  /* "tables/indexesextension.pyx":884
 *     nbounds = self.nbounds
 *     nrows = self.nrows
 *     rbufst = <int *>self.rbufst             # <<<<<<<<<<<<<<
 *     rbufln = <int *>self.rbufln
 *     rbufrv = <npy_int16 *>self.rbufrv
 */
  __pyx_v_rbufst = ((int *)__pyx_v_self->rbufst);

  /* "tables/indexesextension.pyx":885
 *     nrows = self.nrows
 *     rbufst = <int *>self.rbufst
 *     rbufln = <int *>self.rbufln             # <<<<<<<<<<<<<<
 *     rbufrv = <npy_int16 *>self.rbufrv
 *     tlength = 0
 */
  __pyx_v_rbufln = ((int *)__pyx_v_self->rbufln);

  /* "tables/indexesextension.pyx":886
 *     rbufst = <int *>self.rbufst
 *     rbufln = <int *>self.rbufln
 *     rbufrv = <npy_int16 *>self.rbufrv             # <<<<<<<<<<<<<<
 *     tlength = 0
 *     for nrow from 0 <= nrow < nrows:
 */
  __pyx_v_rbufrv = ((npy_int16 *)__pyx_v_self->rbufrv);

  /* "tables/indexesextension.pyx":887
 *     rbufln = <int *>self.rbufln
 *     rbufrv = <npy_int16 *>self.rbufrv
 *     tlength = 0             # <<<<<<<<<<<<<<
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2
 */
  __pyx_v_tlength = 0;

  /* "tables/indexesextension.pyx":888
 *     rbufrv = <npy_int16 *>self.rbufrv
 *     tlength = 0
 *     for nrow from 0 <= nrow < nrows:             # <<<<<<<<<<<<<<
 *       rvrow = nrow*2
 *       bread = 0
 */
  __pyx_t_1 = __pyx_v_nrows;
  for (__pyx_v_nrow = 0; __pyx_v_nrow < __pyx_t_1; __pyx_v_nrow++) {

    /* "tables/indexesextension.pyx":889
 *     tlength = 0
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2             # <<<<<<<<<<<<<<
 *       bread = 0
 *       nchunk = -1
 */
    __pyx_v_rvrow = (__pyx_v_nrow * 2);

    /* "tables/indexesextension.pyx":890
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2
 *       bread = 0             # <<<<<<<<<<<<<<
 *       nchunk = -1
 *       # Look if item1 is in this row
 */
    __pyx_v_bread = 0;

    /* "tables/indexesextension.pyx":891
 *       rvrow = nrow*2
 *       bread = 0
 *       nchunk = -1             # <<<<<<<<<<<<<<
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:
 */
    __pyx_v_nchunk = -1;

    /* "tables/indexesextension.pyx":893
 *       nchunk = -1
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 */
    __pyx_t_3 = ((__pyx_v_item1 > (__pyx_v_rbufrv[__pyx_v_rvrow])) != 0);
    if (__pyx_t_3) {

      /* "tables/indexesextension.pyx":894
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:
 *         if item1 <= rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_int16 *>self.get_lru_bounds(nrow, nbounds)
 */
      __pyx_t_3 = ((__pyx_v_item1 <= (__pyx_v_rbufrv[(__pyx_v_rvrow + 1)])) != 0);
      if (__pyx_t_3) {

        /* "tables/indexesextension.pyx":896
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_int16 *>self.get_lru_bounds(nrow, nbounds)             # <<<<<<<<<<<<<<
 *           bread = 1
 *           nchunk = bisect_left_s(rbufbc, item1, nbounds, 0)
 */
        __pyx_v_rbufbc = ((npy_int16 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_bounds(__pyx_v_self, __pyx_v_nrow, __pyx_v_nbounds));

        /* "tables/indexesextension.pyx":897
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_int16 *>self.get_lru_bounds(nrow, nbounds)
 *           bread = 1             # <<<<<<<<<<<<<<
 *           nchunk = bisect_left_s(rbufbc, item1, nbounds, 0)
 *           # Get the sorted row from the LRU cache or read it.
 */
        __pyx_v_bread = 1;

        /* "tables/indexesextension.pyx":898
 *           rbufbc = <npy_int16 *>self.get_lru_bounds(nrow, nbounds)
 *           bread = 1
 *           nchunk = bisect_left_s(rbufbc, item1, nbounds, 0)             # <<<<<<<<<<<<<<
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_int16 *>self.get_lru_sorted(nrow, ncs, nchunk, cs)
 */
        __pyx_v_nchunk = bisect_left_s(__pyx_v_rbufbc, __pyx_v_item1, __pyx_v_nbounds, 0);

        /* "tables/indexesextension.pyx":900
 *           nchunk = bisect_left_s(rbufbc, item1, nbounds, 0)
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_int16 *>self.get_lru_sorted(nrow, ncs, nchunk, cs)             # <<<<<<<<<<<<<<
 *           start = bisect_left_s(rbuflb, item1, cs, 0) + cs*nchunk
 *         else:
 */
        __pyx_v_rbuflb = ((npy_int16 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_sorted(__pyx_v_self, __pyx_v_nrow, __pyx_v_ncs, __pyx_v_nchunk, __pyx_v_cs));

        /* "tables/indexesextension.pyx":901
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_int16 *>self.get_lru_sorted(nrow, ncs, nchunk, cs)
 *           start = bisect_left_s(rbuflb, item1, cs, 0) + cs*nchunk             # <<<<<<<<<<<<<<
 *         else:
 *           start = ss
 */
        __pyx_v_start = (bisect_left_s(__pyx_v_rbuflb, __pyx_v_item1, __pyx_v_cs, 0) + (__pyx_v_cs * __pyx_v_nchunk));

        /* "tables/indexesextension.pyx":894
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:
 *         if item1 <= rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_int16 *>self.get_lru_bounds(nrow, nbounds)
 */
        goto __pyx_L6;
      }

      /* "tables/indexesextension.pyx":903
 *           start = bisect_left_s(rbuflb, item1, cs, 0) + cs*nchunk
 *         else:
 *           start = ss             # <<<<<<<<<<<<<<
 *       else:
 *         start = 0
 */
      /*else*/ {
        __pyx_v_start = __pyx_v_ss;
      }
      __pyx_L6:;

      /* "tables/indexesextension.pyx":893
 *       nchunk = -1
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 */
      goto __pyx_L5;
    }

    /* "tables/indexesextension.pyx":905
 *           start = ss
 *       else:
 *         start = 0             # <<<<<<<<<<<<<<
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 */
    /*else*/ {
      __pyx_v_start = 0;
    }
    __pyx_L5:;

    /* "tables/indexesextension.pyx":907
 *         start = 0
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:
 */
    __pyx_t_3 = ((__pyx_v_item2 >= (__pyx_v_rbufrv[__pyx_v_rvrow])) != 0);
    if (__pyx_t_3) {

      /* "tables/indexesextension.pyx":908
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 */
      __pyx_t_3 = ((__pyx_v_item2 < (__pyx_v_rbufrv[(__pyx_v_rvrow + 1)])) != 0);
      if (__pyx_t_3) {

        /* "tables/indexesextension.pyx":909
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:             # <<<<<<<<<<<<<<
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_int16 *>self.get_lru_bounds(nrow, nbounds)
 */
        __pyx_t_3 = ((!(__pyx_v_bread != 0)) != 0);
        if (__pyx_t_3) {

          /* "tables/indexesextension.pyx":911
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_int16 *>self.get_lru_bounds(nrow, nbounds)             # <<<<<<<<<<<<<<
 *           nchunk2 = bisect_right_s(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:
 */
          __pyx_v_rbufbc = ((npy_int16 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_bounds(__pyx_v_self, __pyx_v_nrow, __pyx_v_nbounds));

          /* "tables/indexesextension.pyx":909
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:             # <<<<<<<<<<<<<<
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_int16 *>self.get_lru_bounds(nrow, nbounds)
 */
        }

        /* "tables/indexesextension.pyx":912
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_int16 *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_s(rbufbc, item2, nbounds, 0)             # <<<<<<<<<<<<<<
 *           if nchunk2 <> nchunk:
 *             # Get the sorted row from the LRU cache or read it.
 */
        __pyx_v_nchunk2 = bisect_right_s(__pyx_v_rbufbc, __pyx_v_item2, __pyx_v_nbounds, 0);

        /* "tables/indexesextension.pyx":913
 *             rbufbc = <npy_int16 *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_s(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:             # <<<<<<<<<<<<<<
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_int16 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 */
        __pyx_t_3 = ((__pyx_v_nchunk2 != __pyx_v_nchunk) != 0);
        if (__pyx_t_3) {

          /* "tables/indexesextension.pyx":915
 *           if nchunk2 <> nchunk:
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_int16 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)             # <<<<<<<<<<<<<<
 *           stop = bisect_right_s(rbuflb, item2, cs, 0) + cs*nchunk2
 *         else:
 */
          __pyx_v_rbuflb = ((npy_int16 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_sorted(__pyx_v_self, __pyx_v_nrow, __pyx_v_ncs, __pyx_v_nchunk2, __pyx_v_cs));

          /* "tables/indexesextension.pyx":913
 *             rbufbc = <npy_int16 *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_s(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:             # <<<<<<<<<<<<<<
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_int16 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 */
        }

        /* "tables/indexesextension.pyx":916
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_int16 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 *           stop = bisect_right_s(rbuflb, item2, cs, 0) + cs*nchunk2             # <<<<<<<<<<<<<<
 *         else:
 *           stop = ss
 */
        __pyx_v_stop = (bisect_right_s(__pyx_v_rbuflb, __pyx_v_item2, __pyx_v_cs, 0) + (__pyx_v_cs * __pyx_v_nchunk2));

        /* "tables/indexesextension.pyx":908
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 */
        goto __pyx_L8;
      }

      /* "tables/indexesextension.pyx":918
 *           stop = bisect_right_s(rbuflb, item2, cs, 0) + cs*nchunk2
 *         else:
 *           stop = ss             # <<<<<<<<<<<<<<
 *       else:
 *         stop = 0
 */
      /*else*/ {
        __pyx_v_stop = __pyx_v_ss;
      }
      __pyx_L8:;

      /* "tables/indexesextension.pyx":907
 *         start = 0
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:
 */
      goto __pyx_L7;
    }

    /* "tables/indexesextension.pyx":920
 *           stop = ss
 *       else:
 *         stop = 0             # <<<<<<<<<<<<<<
 *       length = stop - start
 *       tlength = tlength + length
 */
    /*else*/ {
      __pyx_v_stop = 0;
    }
    __pyx_L7:;

    /* "tables/indexesextension.pyx":921
 *       else:
 *         stop = 0
 *       length = stop - start             # <<<<<<<<<<<<<<
 *       tlength = tlength + length
 *       rbufst[nrow] = start
 */
    __pyx_v_length = (__pyx_v_stop - __pyx_v_start);

    /* "tables/indexesextension.pyx":922
 *         stop = 0
 *       length = stop - start
 *       tlength = tlength + length             # <<<<<<<<<<<<<<
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length
 */
    __pyx_v_tlength = (__pyx_v_tlength + __pyx_v_length);

    /* "tables/indexesextension.pyx":923
 *       length = stop - start
 *       tlength = tlength + length
 *       rbufst[nrow] = start             # <<<<<<<<<<<<<<
 *       rbufln[nrow] = length
 *     return tlength
 */
    (__pyx_v_rbufst[__pyx_v_nrow]) = __pyx_v_start;

    /* "tables/indexesextension.pyx":924
 *       tlength = tlength + length
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length             # <<<<<<<<<<<<<<
 *     return tlength
 * 
 */
    (__pyx_v_rbufln[__pyx_v_nrow]) = __pyx_v_length;
  }

  /* "tables/indexesextension.pyx":925
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length
 *     return tlength             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_tlength); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 925, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "tables/indexesextension.pyx":868
 * 
 *   # Optimized version for int16
 *   def _search_bin_na_s(self, long item1, long item2):             # <<<<<<<<<<<<<<
 *     cdef int cs, ss, ncs, nrow, nrows, nbounds, rvrow
 *     cdef int start, stop, tlength, length, bread, nchunk, nchunk2
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("tables.indexesextension.IndexArray._search_bin_na_s", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/indexesextension.pyx":929
 * 
 *   # Optimized version for uint16
 *   def _search_bin_na_us(self, long item1, long item2):             # <<<<<<<<<<<<<<
 *     cdef int cs, ss, ncs, nrow, nrows, nbounds, rvrow
 *     cdef int start, stop, tlength, length, bread, nchunk, nchunk2
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_13_search_bin_na_us(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_13_search_bin_na_us(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  long __pyx_v_item1;
  long __pyx_v_item2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_search_bin_na_us (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_item1,&__pyx_n_s_item2,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_item1)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_item2)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_search_bin_na_us", 1, 2, 2, 1); __PYX_ERR(0, 929, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_search_bin_na_us") < 0)) __PYX_ERR(0, 929, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_item1 = __Pyx_PyInt_As_long(values[0]); if (unlikely((__pyx_v_item1 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 929, __pyx_L3_error)
    __pyx_v_item2 = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v_item2 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 929, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_search_bin_na_us", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 929, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("tables.indexesextension.IndexArray._search_bin_na_us", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_16indexesextension_10IndexArray_12_search_bin_na_us(((struct __pyx_obj_6tables_16indexesextension_IndexArray *)__pyx_v_self), __pyx_v_item1, __pyx_v_item2);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_12_search_bin_na_us(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, long __pyx_v_item1, long __pyx_v_item2) {
  int __pyx_v_cs;
  int __pyx_v_ss;
  int __pyx_v_ncs;
  int __pyx_v_nrow;
  int __pyx_v_nrows;
  int __pyx_v_nbounds;
  int __pyx_v_rvrow;
  int __pyx_v_start;
  int __pyx_v_stop;
  int __pyx_v_tlength;
  int __pyx_v_length;
  int __pyx_v_bread;
  int __pyx_v_nchunk;
  int __pyx_v_nchunk2;
  int *__pyx_v_rbufst;
  int *__pyx_v_rbufln;
  npy_uint16 *__pyx_v_rbufrv;
  npy_uint16 *__pyx_v_rbufbc;
  npy_uint16 *__pyx_v_rbuflb;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_search_bin_na_us", 0);

  /* "tables/indexesextension.pyx":937
 *     # Variables with specific type
 *     cdef npy_uint16 *rbufrv
 *     cdef npy_uint16 *rbufbc = NULL             # <<<<<<<<<<<<<<
 *     cdef npy_uint16 *rbuflb = NULL
 * 
 */
  __pyx_v_rbufbc = NULL;

  /* "tables/indexesextension.pyx":938
 *     cdef npy_uint16 *rbufrv
 *     cdef npy_uint16 *rbufbc = NULL
 *     cdef npy_uint16 *rbuflb = NULL             # <<<<<<<<<<<<<<
 * 
 *     cs = self.l_chunksize
 */
  __pyx_v_rbuflb = NULL;

  /* "tables/indexesextension.pyx":940
 *     cdef npy_uint16 *rbuflb = NULL
 * 
 *     cs = self.l_chunksize             # <<<<<<<<<<<<<<
 *     ss = self.l_slicesize
 *     ncs = ss // cs
 */
  __pyx_t_1 = __pyx_v_self->l_chunksize;
  __pyx_v_cs = __pyx_t_1;

  /* "tables/indexesextension.pyx":941
 * 
 *     cs = self.l_chunksize
 *     ss = self.l_slicesize             # <<<<<<<<<<<<<<
 *     ncs = ss // cs
 *     nbounds = self.nbounds
 */
  __pyx_t_1 = __pyx_v_self->l_slicesize;
  __pyx_v_ss = __pyx_t_1;

  /* "tables/indexesextension.pyx":942
 *     cs = self.l_chunksize
 *     ss = self.l_slicesize
 *     ncs = ss // cs             # <<<<<<<<<<<<<<
 *     nbounds = self.nbounds
 *     nrows = self.nrows
 */
  if (unlikely(__pyx_v_cs == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
    __PYX_ERR(0, 942, __pyx_L1_error)
  }
  else if (sizeof(int) == sizeof(long) && (!(((int)-1) > 0)) && unlikely(__pyx_v_cs == (int)-1)  && unlikely(UNARY_NEG_WOULD_OVERFLOW(__pyx_v_ss))) {
    PyErr_SetString(PyExc_OverflowError, "value too large to perform division");
    __PYX_ERR(0, 942, __pyx_L1_error)
  }
  __pyx_v_ncs = __Pyx_div_int(__pyx_v_ss, __pyx_v_cs);

  /* "tables/indexesextension.pyx":943
 *     ss = self.l_slicesize
 *     ncs = ss // cs
 *     nbounds = self.nbounds             # <<<<<<<<<<<<<<
 *     nrows = self.nrows
 *     rbufst = <int *>self.rbufst
 */
  __pyx_t_1 = __pyx_v_self->nbounds;
  __pyx_v_nbounds = __pyx_t_1;

  /* "tables/indexesextension.pyx":944
 *     ncs = ss // cs
 *     nbounds = self.nbounds
 *     nrows = self.nrows             # <<<<<<<<<<<<<<
 *     rbufst = <int *>self.rbufst
 *     rbufln = <int *>self.rbufln
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nrows); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 944, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 944, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_nrows = __pyx_t_1;

  /* "tables/indexesextension.pyx":945
 *     nbounds = self.nbounds
 *     nrows = self.nrows
 *     rbufst = <int *>self.rbufst             # <<<<<<<<<<<<<<
 *     rbufln = <int *>self.rbufln
 *     rbufrv = <npy_uint16 *>self.rbufrv
 */
  __pyx_v_rbufst = ((int *)__pyx_v_self->rbufst);

  /* "tables/indexesextension.pyx":946
 *     nrows = self.nrows
 *     rbufst = <int *>self.rbufst
 *     rbufln = <int *>self.rbufln             # <<<<<<<<<<<<<<
 *     rbufrv = <npy_uint16 *>self.rbufrv
 *     tlength = 0
 */
  __pyx_v_rbufln = ((int *)__pyx_v_self->rbufln);

  /* "tables/indexesextension.pyx":947
 *     rbufst = <int *>self.rbufst
 *     rbufln = <int *>self.rbufln
 *     rbufrv = <npy_uint16 *>self.rbufrv             # <<<<<<<<<<<<<<
 *     tlength = 0
 *     for nrow from 0 <= nrow < nrows:
 */
  __pyx_v_rbufrv = ((npy_uint16 *)__pyx_v_self->rbufrv);

  /* "tables/indexesextension.pyx":948
 *     rbufln = <int *>self.rbufln
 *     rbufrv = <npy_uint16 *>self.rbufrv
 *     tlength = 0             # <<<<<<<<<<<<<<
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2
 */
  __pyx_v_tlength = 0;

  /* "tables/indexesextension.pyx":949
 *     rbufrv = <npy_uint16 *>self.rbufrv
 *     tlength = 0
 *     for nrow from 0 <= nrow < nrows:             # <<<<<<<<<<<<<<
 *       rvrow = nrow*2
 *       bread = 0
 */
  __pyx_t_1 = __pyx_v_nrows;
  for (__pyx_v_nrow = 0; __pyx_v_nrow < __pyx_t_1; __pyx_v_nrow++) {

    /* "tables/indexesextension.pyx":950
 *     tlength = 0
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2             # <<<<<<<<<<<<<<
 *       bread = 0
 *       nchunk = -1
 */
    __pyx_v_rvrow = (__pyx_v_nrow * 2);

    /* "tables/indexesextension.pyx":951
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2
 *       bread = 0             # <<<<<<<<<<<<<<
 *       nchunk = -1
 *       # Look if item1 is in this row
 */
    __pyx_v_bread = 0;

    /* "tables/indexesextension.pyx":952
 *       rvrow = nrow*2
 *       bread = 0
 *       nchunk = -1             # <<<<<<<<<<<<<<
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:
 */
    __pyx_v_nchunk = -1;

    /* "tables/indexesextension.pyx":954
 *       nchunk = -1
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 */
    __pyx_t_3 = ((__pyx_v_item1 > (__pyx_v_rbufrv[__pyx_v_rvrow])) != 0);
    if (__pyx_t_3) {

      /* "tables/indexesextension.pyx":955
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:
 *         if item1 <= rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_uint16 *>self.get_lru_bounds(nrow, nbounds)
 */
      __pyx_t_3 = ((__pyx_v_item1 <= (__pyx_v_rbufrv[(__pyx_v_rvrow + 1)])) != 0);
      if (__pyx_t_3) {

        /* "tables/indexesextension.pyx":957
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_uint16 *>self.get_lru_bounds(nrow, nbounds)             # <<<<<<<<<<<<<<
 *           bread = 1
 *           nchunk = bisect_left_us(rbufbc, item1, nbounds, 0)
 */
        __pyx_v_rbufbc = ((npy_uint16 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_bounds(__pyx_v_self, __pyx_v_nrow, __pyx_v_nbounds));

        /* "tables/indexesextension.pyx":958
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_uint16 *>self.get_lru_bounds(nrow, nbounds)
 *           bread = 1             # <<<<<<<<<<<<<<
 *           nchunk = bisect_left_us(rbufbc, item1, nbounds, 0)
 *           # Get the sorted row from the LRU cache or read it.
 */
        __pyx_v_bread = 1;

        /* "tables/indexesextension.pyx":959
 *           rbufbc = <npy_uint16 *>self.get_lru_bounds(nrow, nbounds)
 *           bread = 1
 *           nchunk = bisect_left_us(rbufbc, item1, nbounds, 0)             # <<<<<<<<<<<<<<
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_uint16 *>self.get_lru_sorted(nrow, ncs, nchunk, cs)
 */
        __pyx_v_nchunk = bisect_left_us(__pyx_v_rbufbc, __pyx_v_item1, __pyx_v_nbounds, 0);

        /* "tables/indexesextension.pyx":961
 *           nchunk = bisect_left_us(rbufbc, item1, nbounds, 0)
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_uint16 *>self.get_lru_sorted(nrow, ncs, nchunk, cs)             # <<<<<<<<<<<<<<
 *           start = bisect_left_us(rbuflb, item1, cs, 0) + cs*nchunk
 *         else:
 */
        __pyx_v_rbuflb = ((npy_uint16 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_sorted(__pyx_v_self, __pyx_v_nrow, __pyx_v_ncs, __pyx_v_nchunk, __pyx_v_cs));

        /* "tables/indexesextension.pyx":962
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_uint16 *>self.get_lru_sorted(nrow, ncs, nchunk, cs)
 *           start = bisect_left_us(rbuflb, item1, cs, 0) + cs*nchunk             # <<<<<<<<<<<<<<
 *         else:
 *           start = ss
 */
        __pyx_v_start = (bisect_left_us(__pyx_v_rbuflb, __pyx_v_item1, __pyx_v_cs, 0) + (__pyx_v_cs * __pyx_v_nchunk));

        /* "tables/indexesextension.pyx":955
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:
 *         if item1 <= rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_uint16 *>self.get_lru_bounds(nrow, nbounds)
 */
        goto __pyx_L6;
      }

      /* "tables/indexesextension.pyx":964
 *           start = bisect_left_us(rbuflb, item1, cs, 0) + cs*nchunk
 *         else:
 *           start = ss             # <<<<<<<<<<<<<<
 *       else:
 *         start = 0
 */
      /*else*/ {
        __pyx_v_start = __pyx_v_ss;
      }
      __pyx_L6:;

      /* "tables/indexesextension.pyx":954
 *       nchunk = -1
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 */
      goto __pyx_L5;
    }

    /* "tables/indexesextension.pyx":966
 *           start = ss
 *       else:
 *         start = 0             # <<<<<<<<<<<<<<
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 */
    /*else*/ {
      __pyx_v_start = 0;
    }
    __pyx_L5:;

    /* "tables/indexesextension.pyx":968
 *         start = 0
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:
 */
    __pyx_t_3 = ((__pyx_v_item2 >= (__pyx_v_rbufrv[__pyx_v_rvrow])) != 0);
    if (__pyx_t_3) {

      /* "tables/indexesextension.pyx":969
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 */
      __pyx_t_3 = ((__pyx_v_item2 < (__pyx_v_rbufrv[(__pyx_v_rvrow + 1)])) != 0);
      if (__pyx_t_3) {

        /* "tables/indexesextension.pyx":970
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:             # <<<<<<<<<<<<<<
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_uint16 *>self.get_lru_bounds(nrow, nbounds)
 */
        __pyx_t_3 = ((!(__pyx_v_bread != 0)) != 0);
        if (__pyx_t_3) {

          /* "tables/indexesextension.pyx":972
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_uint16 *>self.get_lru_bounds(nrow, nbounds)             # <<<<<<<<<<<<<<
 *           nchunk2 = bisect_right_us(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:
 */
          __pyx_v_rbufbc = ((npy_uint16 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_bounds(__pyx_v_self, __pyx_v_nrow, __pyx_v_nbounds));

          /* "tables/indexesextension.pyx":970
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:             # <<<<<<<<<<<<<<
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_uint16 *>self.get_lru_bounds(nrow, nbounds)
 */
        }

        /* "tables/indexesextension.pyx":973
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_uint16 *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_us(rbufbc, item2, nbounds, 0)             # <<<<<<<<<<<<<<
 *           if nchunk2 <> nchunk:
 *             # Get the sorted row from the LRU cache or read it.
 */
        __pyx_v_nchunk2 = bisect_right_us(__pyx_v_rbufbc, __pyx_v_item2, __pyx_v_nbounds, 0);

        /* "tables/indexesextension.pyx":974
 *             rbufbc = <npy_uint16 *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_us(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:             # <<<<<<<<<<<<<<
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_uint16 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 */
        __pyx_t_3 = ((__pyx_v_nchunk2 != __pyx_v_nchunk) != 0);
        if (__pyx_t_3) {

          /* "tables/indexesextension.pyx":976
 *           if nchunk2 <> nchunk:
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_uint16 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)             # <<<<<<<<<<<<<<
 *           stop = bisect_right_us(rbuflb, item2, cs, 0) + cs*nchunk2
 *         else:
 */
          __pyx_v_rbuflb = ((npy_uint16 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_sorted(__pyx_v_self, __pyx_v_nrow, __pyx_v_ncs, __pyx_v_nchunk2, __pyx_v_cs));

          /* "tables/indexesextension.pyx":974
 *             rbufbc = <npy_uint16 *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_us(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:             # <<<<<<<<<<<<<<
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_uint16 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 */
        }

        /* "tables/indexesextension.pyx":977
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_uint16 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 *           stop = bisect_right_us(rbuflb, item2, cs, 0) + cs*nchunk2             # <<<<<<<<<<<<<<
 *         else:
 *           stop = ss
 */
        __pyx_v_stop = (bisect_right_us(__pyx_v_rbuflb, __pyx_v_item2, __pyx_v_cs, 0) + (__pyx_v_cs * __pyx_v_nchunk2));

        /* "tables/indexesextension.pyx":969
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 */
        goto __pyx_L8;
      }

      /* "tables/indexesextension.pyx":979
 *           stop = bisect_right_us(rbuflb, item2, cs, 0) + cs*nchunk2
 *         else:
 *           stop = ss             # <<<<<<<<<<<<<<
 *       else:
 *         stop = 0
 */
      /*else*/ {
        __pyx_v_stop = __pyx_v_ss;
      }
      __pyx_L8:;

      /* "tables/indexesextension.pyx":968
 *         start = 0
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:
 */
      goto __pyx_L7;
    }

    /* "tables/indexesextension.pyx":981
 *           stop = ss
 *       else:
 *         stop = 0             # <<<<<<<<<<<<<<
 *       length = stop - start
 *       tlength = tlength + length
 */
    /*else*/ {
      __pyx_v_stop = 0;
    }
    __pyx_L7:;

    /* "tables/indexesextension.pyx":982
 *       else:
 *         stop = 0
 *       length = stop - start             # <<<<<<<<<<<<<<
 *       tlength = tlength + length
 *       rbufst[nrow] = start
 */
    __pyx_v_length = (__pyx_v_stop - __pyx_v_start);

    /* "tables/indexesextension.pyx":983
 *         stop = 0
 *       length = stop - start
 *       tlength = tlength + length             # <<<<<<<<<<<<<<
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length
 */
    __pyx_v_tlength = (__pyx_v_tlength + __pyx_v_length);

    /* "tables/indexesextension.pyx":984
 *       length = stop - start
 *       tlength = tlength + length
 *       rbufst[nrow] = start             # <<<<<<<<<<<<<<
 *       rbufln[nrow] = length
 *     return tlength
 */
    (__pyx_v_rbufst[__pyx_v_nrow]) = __pyx_v_start;

    /* "tables/indexesextension.pyx":985
 *       tlength = tlength + length
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length             # <<<<<<<<<<<<<<
 *     return tlength
 * 
 */
    (__pyx_v_rbufln[__pyx_v_nrow]) = __pyx_v_length;
  }

  /* "tables/indexesextension.pyx":986
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length
 *     return tlength             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_tlength); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 986, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "tables/indexesextension.pyx":929
 * 
 *   # Optimized version for uint16
 *   def _search_bin_na_us(self, long item1, long item2):             # <<<<<<<<<<<<<<
 *     cdef int cs, ss, ncs, nrow, nrows, nbounds, rvrow
 *     cdef int start, stop, tlength, length, bread, nchunk, nchunk2
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("tables.indexesextension.IndexArray._search_bin_na_us", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/indexesextension.pyx":990
 * 
 *   # Optimized version for int32
 *   def _search_bin_na_i(self, long item1, long item2):             # <<<<<<<<<<<<<<
 *     cdef int cs, ss, ncs, nrow, nrows, nbounds, rvrow
 *     cdef int start, stop, tlength, length, bread, nchunk, nchunk2
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_15_search_bin_na_i(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_15_search_bin_na_i(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  long __pyx_v_item1;
  long __pyx_v_item2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_search_bin_na_i (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_item1,&__pyx_n_s_item2,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_item1)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_item2)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_search_bin_na_i", 1, 2, 2, 1); __PYX_ERR(0, 990, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_search_bin_na_i") < 0)) __PYX_ERR(0, 990, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_item1 = __Pyx_PyInt_As_long(values[0]); if (unlikely((__pyx_v_item1 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 990, __pyx_L3_error)
    __pyx_v_item2 = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v_item2 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 990, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_search_bin_na_i", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 990, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("tables.indexesextension.IndexArray._search_bin_na_i", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_16indexesextension_10IndexArray_14_search_bin_na_i(((struct __pyx_obj_6tables_16indexesextension_IndexArray *)__pyx_v_self), __pyx_v_item1, __pyx_v_item2);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_14_search_bin_na_i(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, long __pyx_v_item1, long __pyx_v_item2) {
  int __pyx_v_cs;
  int __pyx_v_ss;
  int __pyx_v_ncs;
  int __pyx_v_nrow;
  int __pyx_v_nrows;
  int __pyx_v_nbounds;
  int __pyx_v_rvrow;
  int __pyx_v_start;
  int __pyx_v_stop;
  int __pyx_v_tlength;
  int __pyx_v_length;
  int __pyx_v_bread;
  int __pyx_v_nchunk;
  int __pyx_v_nchunk2;
  int *__pyx_v_rbufst;
  int *__pyx_v_rbufln;
  npy_int32 *__pyx_v_rbufrv;
  npy_int32 *__pyx_v_rbufbc;
  npy_int32 *__pyx_v_rbuflb;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_search_bin_na_i", 0);

  /* "tables/indexesextension.pyx":998
 *     # Variables with specific type
 *     cdef npy_int32 *rbufrv
 *     cdef npy_int32 *rbufbc = NULL             # <<<<<<<<<<<<<<
 *     cdef npy_int32 *rbuflb = NULL
 * 
 */
  __pyx_v_rbufbc = NULL;

  /* "tables/indexesextension.pyx":999
 *     cdef npy_int32 *rbufrv
 *     cdef npy_int32 *rbufbc = NULL
 *     cdef npy_int32 *rbuflb = NULL             # <<<<<<<<<<<<<<
 * 
 *     cs = self.l_chunksize
 */
  __pyx_v_rbuflb = NULL;

  /* "tables/indexesextension.pyx":1001
 *     cdef npy_int32 *rbuflb = NULL
 * 
 *     cs = self.l_chunksize             # <<<<<<<<<<<<<<
 *     ss = self.l_slicesize
 *     ncs = ss // cs
 */
  __pyx_t_1 = __pyx_v_self->l_chunksize;
  __pyx_v_cs = __pyx_t_1;

  /* "tables/indexesextension.pyx":1002
 * 
 *     cs = self.l_chunksize
 *     ss = self.l_slicesize             # <<<<<<<<<<<<<<
 *     ncs = ss // cs
 *     nbounds = self.nbounds
 */
  __pyx_t_1 = __pyx_v_self->l_slicesize;
  __pyx_v_ss = __pyx_t_1;

  /* "tables/indexesextension.pyx":1003
 *     cs = self.l_chunksize
 *     ss = self.l_slicesize
 *     ncs = ss // cs             # <<<<<<<<<<<<<<
 *     nbounds = self.nbounds
 *     nrows = self.nrows
 */
  if (unlikely(__pyx_v_cs == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
    __PYX_ERR(0, 1003, __pyx_L1_error)
  }
  else if (sizeof(int) == sizeof(long) && (!(((int)-1) > 0)) && unlikely(__pyx_v_cs == (int)-1)  && unlikely(UNARY_NEG_WOULD_OVERFLOW(__pyx_v_ss))) {
    PyErr_SetString(PyExc_OverflowError, "value too large to perform division");
    __PYX_ERR(0, 1003, __pyx_L1_error)
  }
  __pyx_v_ncs = __Pyx_div_int(__pyx_v_ss, __pyx_v_cs);

  /* "tables/indexesextension.pyx":1004
 *     ss = self.l_slicesize
 *     ncs = ss // cs
 *     nbounds = self.nbounds             # <<<<<<<<<<<<<<
 *     nrows = self.nrows
 *     rbufst = <int *>self.rbufst
 */
  __pyx_t_1 = __pyx_v_self->nbounds;
  __pyx_v_nbounds = __pyx_t_1;

  /* "tables/indexesextension.pyx":1005
 *     ncs = ss // cs
 *     nbounds = self.nbounds
 *     nrows = self.nrows             # <<<<<<<<<<<<<<
 *     rbufst = <int *>self.rbufst
 *     rbufln = <int *>self.rbufln
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nrows); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1005, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1005, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_nrows = __pyx_t_1;

  /* "tables/indexesextension.pyx":1006
 *     nbounds = self.nbounds
 *     nrows = self.nrows
 *     rbufst = <int *>self.rbufst             # <<<<<<<<<<<<<<
 *     rbufln = <int *>self.rbufln
 *     rbufrv = <npy_int32 *>self.rbufrv
 */
  __pyx_v_rbufst = ((int *)__pyx_v_self->rbufst);

  /* "tables/indexesextension.pyx":1007
 *     nrows = self.nrows
 *     rbufst = <int *>self.rbufst
 *     rbufln = <int *>self.rbufln             # <<<<<<<<<<<<<<
 *     rbufrv = <npy_int32 *>self.rbufrv
 *     tlength = 0
 */
  __pyx_v_rbufln = ((int *)__pyx_v_self->rbufln);

  /* "tables/indexesextension.pyx":1008
 *     rbufst = <int *>self.rbufst
 *     rbufln = <int *>self.rbufln
 *     rbufrv = <npy_int32 *>self.rbufrv             # <<<<<<<<<<<<<<
 *     tlength = 0
 *     for nrow from 0 <= nrow < nrows:
 */
  __pyx_v_rbufrv = ((npy_int32 *)__pyx_v_self->rbufrv);

  /* "tables/indexesextension.pyx":1009
 *     rbufln = <int *>self.rbufln
 *     rbufrv = <npy_int32 *>self.rbufrv
 *     tlength = 0             # <<<<<<<<<<<<<<
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2
 */
  __pyx_v_tlength = 0;

  /* "tables/indexesextension.pyx":1010
 *     rbufrv = <npy_int32 *>self.rbufrv
 *     tlength = 0
 *     for nrow from 0 <= nrow < nrows:             # <<<<<<<<<<<<<<
 *       rvrow = nrow*2
 *       bread = 0
 */
  __pyx_t_1 = __pyx_v_nrows;
  for (__pyx_v_nrow = 0; __pyx_v_nrow < __pyx_t_1; __pyx_v_nrow++) {

    /* "tables/indexesextension.pyx":1011
 *     tlength = 0
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2             # <<<<<<<<<<<<<<
 *       bread = 0
 *       nchunk = -1
 */
    __pyx_v_rvrow = (__pyx_v_nrow * 2);

    /* "tables/indexesextension.pyx":1012
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2
 *       bread = 0             # <<<<<<<<<<<<<<
 *       nchunk = -1
 *       # Look if item1 is in this row
 */
    __pyx_v_bread = 0;

    /* "tables/indexesextension.pyx":1013
 *       rvrow = nrow*2
 *       bread = 0
 *       nchunk = -1             # <<<<<<<<<<<<<<
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:
 */
    __pyx_v_nchunk = -1;

    /* "tables/indexesextension.pyx":1015
 *       nchunk = -1
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 */
    __pyx_t_3 = ((__pyx_v_item1 > (__pyx_v_rbufrv[__pyx_v_rvrow])) != 0);
    if (__pyx_t_3) {

      /* "tables/indexesextension.pyx":1016
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:
 *         if item1 <= rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_int32 *>self.get_lru_bounds(nrow, nbounds)
 */
      __pyx_t_3 = ((__pyx_v_item1 <= (__pyx_v_rbufrv[(__pyx_v_rvrow + 1)])) != 0);
      if (__pyx_t_3) {

        /* "tables/indexesextension.pyx":1018
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_int32 *>self.get_lru_bounds(nrow, nbounds)             # <<<<<<<<<<<<<<
 *           bread = 1
 *           nchunk = bisect_left_i(rbufbc, item1, nbounds, 0)
 */
        __pyx_v_rbufbc = ((npy_int32 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_bounds(__pyx_v_self, __pyx_v_nrow, __pyx_v_nbounds));

        /* "tables/indexesextension.pyx":1019
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_int32 *>self.get_lru_bounds(nrow, nbounds)
 *           bread = 1             # <<<<<<<<<<<<<<
 *           nchunk = bisect_left_i(rbufbc, item1, nbounds, 0)
 *           # Get the sorted row from the LRU cache or read it.
 */
        __pyx_v_bread = 1;

        /* "tables/indexesextension.pyx":1020
 *           rbufbc = <npy_int32 *>self.get_lru_bounds(nrow, nbounds)
 *           bread = 1
 *           nchunk = bisect_left_i(rbufbc, item1, nbounds, 0)             # <<<<<<<<<<<<<<
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_int32 *>self.get_lru_sorted(nrow, ncs, nchunk, cs)
 */
        __pyx_v_nchunk = bisect_left_i(__pyx_v_rbufbc, __pyx_v_item1, __pyx_v_nbounds, 0);

        /* "tables/indexesextension.pyx":1022
 *           nchunk = bisect_left_i(rbufbc, item1, nbounds, 0)
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_int32 *>self.get_lru_sorted(nrow, ncs, nchunk, cs)             # <<<<<<<<<<<<<<
 *           start = bisect_left_i(rbuflb, item1, cs, 0) + cs*nchunk
 *         else:
 */
        __pyx_v_rbuflb = ((npy_int32 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_sorted(__pyx_v_self, __pyx_v_nrow, __pyx_v_ncs, __pyx_v_nchunk, __pyx_v_cs));

        /* "tables/indexesextension.pyx":1023
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_int32 *>self.get_lru_sorted(nrow, ncs, nchunk, cs)
 *           start = bisect_left_i(rbuflb, item1, cs, 0) + cs*nchunk             # <<<<<<<<<<<<<<
 *         else:
 *           start = ss
 */
        __pyx_v_start = (bisect_left_i(__pyx_v_rbuflb, __pyx_v_item1, __pyx_v_cs, 0) + (__pyx_v_cs * __pyx_v_nchunk));

        /* "tables/indexesextension.pyx":1016
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:
 *         if item1 <= rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_int32 *>self.get_lru_bounds(nrow, nbounds)
 */
        goto __pyx_L6;
      }

      /* "tables/indexesextension.pyx":1025
 *           start = bisect_left_i(rbuflb, item1, cs, 0) + cs*nchunk
 *         else:
 *           start = ss             # <<<<<<<<<<<<<<
 *       else:
 *         start = 0
 */
      /*else*/ {
        __pyx_v_start = __pyx_v_ss;
      }
      __pyx_L6:;

      /* "tables/indexesextension.pyx":1015
 *       nchunk = -1
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 */
      goto __pyx_L5;
    }

    /* "tables/indexesextension.pyx":1027
 *           start = ss
 *       else:
 *         start = 0             # <<<<<<<<<<<<<<
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 */
    /*else*/ {
      __pyx_v_start = 0;
    }
    __pyx_L5:;

    /* "tables/indexesextension.pyx":1029
 *         start = 0
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:
 */
    __pyx_t_3 = ((__pyx_v_item2 >= (__pyx_v_rbufrv[__pyx_v_rvrow])) != 0);
    if (__pyx_t_3) {

      /* "tables/indexesextension.pyx":1030
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 */
      __pyx_t_3 = ((__pyx_v_item2 < (__pyx_v_rbufrv[(__pyx_v_rvrow + 1)])) != 0);
      if (__pyx_t_3) {

        /* "tables/indexesextension.pyx":1031
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:             # <<<<<<<<<<<<<<
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_int32 *>self.get_lru_bounds(nrow, nbounds)
 */
        __pyx_t_3 = ((!(__pyx_v_bread != 0)) != 0);
        if (__pyx_t_3) {

          /* "tables/indexesextension.pyx":1033
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_int32 *>self.get_lru_bounds(nrow, nbounds)             # <<<<<<<<<<<<<<
 *           nchunk2 = bisect_right_i(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:
 */
          __pyx_v_rbufbc = ((npy_int32 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_bounds(__pyx_v_self, __pyx_v_nrow, __pyx_v_nbounds));

          /* "tables/indexesextension.pyx":1031
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:             # <<<<<<<<<<<<<<
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_int32 *>self.get_lru_bounds(nrow, nbounds)
 */
        }

        /* "tables/indexesextension.pyx":1034
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_int32 *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_i(rbufbc, item2, nbounds, 0)             # <<<<<<<<<<<<<<
 *           if nchunk2 <> nchunk:
 *             # Get the sorted row from the LRU cache or read it.
 */
        __pyx_v_nchunk2 = bisect_right_i(__pyx_v_rbufbc, __pyx_v_item2, __pyx_v_nbounds, 0);

        /* "tables/indexesextension.pyx":1035
 *             rbufbc = <npy_int32 *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_i(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:             # <<<<<<<<<<<<<<
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_int32 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 */
        __pyx_t_3 = ((__pyx_v_nchunk2 != __pyx_v_nchunk) != 0);
        if (__pyx_t_3) {

          /* "tables/indexesextension.pyx":1037
 *           if nchunk2 <> nchunk:
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_int32 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)             # <<<<<<<<<<<<<<
 *           stop = bisect_right_i(rbuflb, item2, cs, 0) + cs*nchunk2
 *         else:
 */
          __pyx_v_rbuflb = ((npy_int32 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_sorted(__pyx_v_self, __pyx_v_nrow, __pyx_v_ncs, __pyx_v_nchunk2, __pyx_v_cs));

          /* "tables/indexesextension.pyx":1035
 *             rbufbc = <npy_int32 *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_i(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:             # <<<<<<<<<<<<<<
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_int32 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 */
        }

        /* "tables/indexesextension.pyx":1038
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_int32 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 *           stop = bisect_right_i(rbuflb, item2, cs, 0) + cs*nchunk2             # <<<<<<<<<<<<<<
 *         else:
 *           stop = ss
 */
        __pyx_v_stop = (bisect_right_i(__pyx_v_rbuflb, __pyx_v_item2, __pyx_v_cs, 0) + (__pyx_v_cs * __pyx_v_nchunk2));

        /* "tables/indexesextension.pyx":1030
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 */
        goto __pyx_L8;
      }

      /* "tables/indexesextension.pyx":1040
 *           stop = bisect_right_i(rbuflb, item2, cs, 0) + cs*nchunk2
 *         else:
 *           stop = ss             # <<<<<<<<<<<<<<
 *       else:
 *         stop = 0
 */
      /*else*/ {
        __pyx_v_stop = __pyx_v_ss;
      }
      __pyx_L8:;

      /* "tables/indexesextension.pyx":1029
 *         start = 0
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:
 */
      goto __pyx_L7;
    }

    /* "tables/indexesextension.pyx":1042
 *           stop = ss
 *       else:
 *         stop = 0             # <<<<<<<<<<<<<<
 *       length = stop - start
 *       tlength = tlength + length
 */
    /*else*/ {
      __pyx_v_stop = 0;
    }
    __pyx_L7:;

    /* "tables/indexesextension.pyx":1043
 *       else:
 *         stop = 0
 *       length = stop - start             # <<<<<<<<<<<<<<
 *       tlength = tlength + length
 *       rbufst[nrow] = start
 */
    __pyx_v_length = (__pyx_v_stop - __pyx_v_start);

    /* "tables/indexesextension.pyx":1044
 *         stop = 0
 *       length = stop - start
 *       tlength = tlength + length             # <<<<<<<<<<<<<<
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length
 */
    __pyx_v_tlength = (__pyx_v_tlength + __pyx_v_length);

    /* "tables/indexesextension.pyx":1045
 *       length = stop - start
 *       tlength = tlength + length
 *       rbufst[nrow] = start             # <<<<<<<<<<<<<<
 *       rbufln[nrow] = length
 *     return tlength
 */
    (__pyx_v_rbufst[__pyx_v_nrow]) = __pyx_v_start;

    /* "tables/indexesextension.pyx":1046
 *       tlength = tlength + length
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length             # <<<<<<<<<<<<<<
 *     return tlength
 * 
 */
    (__pyx_v_rbufln[__pyx_v_nrow]) = __pyx_v_length;
  }

  /* "tables/indexesextension.pyx":1047
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length
 *     return tlength             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_tlength); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1047, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "tables/indexesextension.pyx":990
 * 
 *   # Optimized version for int32
 *   def _search_bin_na_i(self, long item1, long item2):             # <<<<<<<<<<<<<<
 *     cdef int cs, ss, ncs, nrow, nrows, nbounds, rvrow
 *     cdef int start, stop, tlength, length, bread, nchunk, nchunk2
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("tables.indexesextension.IndexArray._search_bin_na_i", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/indexesextension.pyx":1051
 * 
 *   # Optimized version for uint32
 *   def _search_bin_na_ui(self, npy_uint32 item1, npy_uint32 item2):             # <<<<<<<<<<<<<<
 *     cdef int cs, ss, ncs, nrow, nrows, nbounds, rvrow
 *     cdef int start, stop, tlength, length, bread, nchunk, nchunk2
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_17_search_bin_na_ui(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_17_search_bin_na_ui(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  npy_uint32 __pyx_v_item1;
  npy_uint32 __pyx_v_item2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_search_bin_na_ui (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_item1,&__pyx_n_s_item2,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_item1)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_item2)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_search_bin_na_ui", 1, 2, 2, 1); __PYX_ERR(0, 1051, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_search_bin_na_ui") < 0)) __PYX_ERR(0, 1051, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_item1 = __Pyx_PyInt_As_npy_uint32(values[0]); if (unlikely((__pyx_v_item1 == ((npy_uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1051, __pyx_L3_error)
    __pyx_v_item2 = __Pyx_PyInt_As_npy_uint32(values[1]); if (unlikely((__pyx_v_item2 == ((npy_uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1051, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_search_bin_na_ui", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1051, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("tables.indexesextension.IndexArray._search_bin_na_ui", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_16indexesextension_10IndexArray_16_search_bin_na_ui(((struct __pyx_obj_6tables_16indexesextension_IndexArray *)__pyx_v_self), __pyx_v_item1, __pyx_v_item2);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_16_search_bin_na_ui(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, npy_uint32 __pyx_v_item1, npy_uint32 __pyx_v_item2) {
  int __pyx_v_cs;
  int __pyx_v_ss;
  int __pyx_v_ncs;
  int __pyx_v_nrow;
  int __pyx_v_nrows;
  int __pyx_v_nbounds;
  int __pyx_v_rvrow;
  int __pyx_v_start;
  int __pyx_v_stop;
  int __pyx_v_tlength;
  int __pyx_v_length;
  int __pyx_v_bread;
  int __pyx_v_nchunk;
  int __pyx_v_nchunk2;
  int *__pyx_v_rbufst;
  int *__pyx_v_rbufln;
  npy_uint32 *__pyx_v_rbufrv;
  npy_uint32 *__pyx_v_rbufbc;
  npy_uint32 *__pyx_v_rbuflb;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_search_bin_na_ui", 0);

  /* "tables/indexesextension.pyx":1059
 *     # Variables with specific type
 *     cdef npy_uint32 *rbufrv
 *     cdef npy_uint32 *rbufbc = NULL             # <<<<<<<<<<<<<<
 *     cdef npy_uint32 *rbuflb = NULL
 * 
 */
  __pyx_v_rbufbc = NULL;

  /* "tables/indexesextension.pyx":1060
 *     cdef npy_uint32 *rbufrv
 *     cdef npy_uint32 *rbufbc = NULL
 *     cdef npy_uint32 *rbuflb = NULL             # <<<<<<<<<<<<<<
 * 
 *     cs = self.l_chunksize
 */
  __pyx_v_rbuflb = NULL;

  /* "tables/indexesextension.pyx":1062
 *     cdef npy_uint32 *rbuflb = NULL
 * 
 *     cs = self.l_chunksize             # <<<<<<<<<<<<<<
 *     ss = self.l_slicesize
 *     ncs = ss // cs
 */
  __pyx_t_1 = __pyx_v_self->l_chunksize;
  __pyx_v_cs = __pyx_t_1;

  /* "tables/indexesextension.pyx":1063
 * 
 *     cs = self.l_chunksize
 *     ss = self.l_slicesize             # <<<<<<<<<<<<<<
 *     ncs = ss // cs
 *     nbounds = self.nbounds
 */
  __pyx_t_1 = __pyx_v_self->l_slicesize;
  __pyx_v_ss = __pyx_t_1;

  /* "tables/indexesextension.pyx":1064
 *     cs = self.l_chunksize
 *     ss = self.l_slicesize
 *     ncs = ss // cs             # <<<<<<<<<<<<<<
 *     nbounds = self.nbounds
 *     nrows = self.nrows
 */
  if (unlikely(__pyx_v_cs == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
    __PYX_ERR(0, 1064, __pyx_L1_error)
  }
  else if (sizeof(int) == sizeof(long) && (!(((int)-1) > 0)) && unlikely(__pyx_v_cs == (int)-1)  && unlikely(UNARY_NEG_WOULD_OVERFLOW(__pyx_v_ss))) {
    PyErr_SetString(PyExc_OverflowError, "value too large to perform division");
    __PYX_ERR(0, 1064, __pyx_L1_error)
  }
  __pyx_v_ncs = __Pyx_div_int(__pyx_v_ss, __pyx_v_cs);

  /* "tables/indexesextension.pyx":1065
 *     ss = self.l_slicesize
 *     ncs = ss // cs
 *     nbounds = self.nbounds             # <<<<<<<<<<<<<<
 *     nrows = self.nrows
 *     rbufst = <int *>self.rbufst
 */
  __pyx_t_1 = __pyx_v_self->nbounds;
  __pyx_v_nbounds = __pyx_t_1;

  /* "tables/indexesextension.pyx":1066
 *     ncs = ss // cs
 *     nbounds = self.nbounds
 *     nrows = self.nrows             # <<<<<<<<<<<<<<
 *     rbufst = <int *>self.rbufst
 *     rbufln = <int *>self.rbufln
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nrows); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1066, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1066, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_nrows = __pyx_t_1;

  /* "tables/indexesextension.pyx":1067
 *     nbounds = self.nbounds
 *     nrows = self.nrows
 *     rbufst = <int *>self.rbufst             # <<<<<<<<<<<<<<
 *     rbufln = <int *>self.rbufln
 *     rbufrv = <npy_uint32 *>self.rbufrv
 */
  __pyx_v_rbufst = ((int *)__pyx_v_self->rbufst);

  /* "tables/indexesextension.pyx":1068
 *     nrows = self.nrows
 *     rbufst = <int *>self.rbufst
 *     rbufln = <int *>self.rbufln             # <<<<<<<<<<<<<<
 *     rbufrv = <npy_uint32 *>self.rbufrv
 *     tlength = 0
 */
  __pyx_v_rbufln = ((int *)__pyx_v_self->rbufln);

  /* "tables/indexesextension.pyx":1069
 *     rbufst = <int *>self.rbufst
 *     rbufln = <int *>self.rbufln
 *     rbufrv = <npy_uint32 *>self.rbufrv             # <<<<<<<<<<<<<<
 *     tlength = 0
 *     for nrow from 0 <= nrow < nrows:
 */
  __pyx_v_rbufrv = ((npy_uint32 *)__pyx_v_self->rbufrv);

  /* "tables/indexesextension.pyx":1070
 *     rbufln = <int *>self.rbufln
 *     rbufrv = <npy_uint32 *>self.rbufrv
 *     tlength = 0             # <<<<<<<<<<<<<<
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2
 */
  __pyx_v_tlength = 0;

  /* "tables/indexesextension.pyx":1071
 *     rbufrv = <npy_uint32 *>self.rbufrv
 *     tlength = 0
 *     for nrow from 0 <= nrow < nrows:             # <<<<<<<<<<<<<<
 *       rvrow = nrow*2
 *       bread = 0
 */
  __pyx_t_1 = __pyx_v_nrows;
  for (__pyx_v_nrow = 0; __pyx_v_nrow < __pyx_t_1; __pyx_v_nrow++) {

    /* "tables/indexesextension.pyx":1072
 *     tlength = 0
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2             # <<<<<<<<<<<<<<
 *       bread = 0
 *       nchunk = -1
 */
    __pyx_v_rvrow = (__pyx_v_nrow * 2);

    /* "tables/indexesextension.pyx":1073
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2
 *       bread = 0             # <<<<<<<<<<<<<<
 *       nchunk = -1
 *       # Look if item1 is in this row
 */
    __pyx_v_bread = 0;

    /* "tables/indexesextension.pyx":1074
 *       rvrow = nrow*2
 *       bread = 0
 *       nchunk = -1             # <<<<<<<<<<<<<<
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:
 */
    __pyx_v_nchunk = -1;

    /* "tables/indexesextension.pyx":1076
 *       nchunk = -1
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 */
    __pyx_t_3 = ((__pyx_v_item1 > (__pyx_v_rbufrv[__pyx_v_rvrow])) != 0);
    if (__pyx_t_3) {

      /* "tables/indexesextension.pyx":1077
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:
 *         if item1 <= rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_uint32 *>self.get_lru_bounds(nrow, nbounds)
 */
      __pyx_t_3 = ((__pyx_v_item1 <= (__pyx_v_rbufrv[(__pyx_v_rvrow + 1)])) != 0);
      if (__pyx_t_3) {

        /* "tables/indexesextension.pyx":1079
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_uint32 *>self.get_lru_bounds(nrow, nbounds)             # <<<<<<<<<<<<<<
 *           bread = 1
 *           nchunk = bisect_left_ui(rbufbc, item1, nbounds, 0)
 */
        __pyx_v_rbufbc = ((npy_uint32 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_bounds(__pyx_v_self, __pyx_v_nrow, __pyx_v_nbounds));

        /* "tables/indexesextension.pyx":1080
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_uint32 *>self.get_lru_bounds(nrow, nbounds)
 *           bread = 1             # <<<<<<<<<<<<<<
 *           nchunk = bisect_left_ui(rbufbc, item1, nbounds, 0)
 *           # Get the sorted row from the LRU cache or read it.
 */
        __pyx_v_bread = 1;

        /* "tables/indexesextension.pyx":1081
 *           rbufbc = <npy_uint32 *>self.get_lru_bounds(nrow, nbounds)
 *           bread = 1
 *           nchunk = bisect_left_ui(rbufbc, item1, nbounds, 0)             # <<<<<<<<<<<<<<
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_uint32 *>self.get_lru_sorted(nrow, ncs, nchunk, cs)
 */
        __pyx_v_nchunk = bisect_left_ui(__pyx_v_rbufbc, __pyx_v_item1, __pyx_v_nbounds, 0);

        /* "tables/indexesextension.pyx":1083
 *           nchunk = bisect_left_ui(rbufbc, item1, nbounds, 0)
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_uint32 *>self.get_lru_sorted(nrow, ncs, nchunk, cs)             # <<<<<<<<<<<<<<
 *           start = bisect_left_ui(rbuflb, item1, cs, 0) + cs*nchunk
 *         else:
 */
        __pyx_v_rbuflb = ((npy_uint32 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_sorted(__pyx_v_self, __pyx_v_nrow, __pyx_v_ncs, __pyx_v_nchunk, __pyx_v_cs));

        /* "tables/indexesextension.pyx":1084
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_uint32 *>self.get_lru_sorted(nrow, ncs, nchunk, cs)
 *           start = bisect_left_ui(rbuflb, item1, cs, 0) + cs*nchunk             # <<<<<<<<<<<<<<
 *         else:
 *           start = ss
 */
        __pyx_v_start = (bisect_left_ui(__pyx_v_rbuflb, __pyx_v_item1, __pyx_v_cs, 0) + (__pyx_v_cs * __pyx_v_nchunk));

        /* "tables/indexesextension.pyx":1077
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:
 *         if item1 <= rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_uint32 *>self.get_lru_bounds(nrow, nbounds)
 */
        goto __pyx_L6;
      }

      /* "tables/indexesextension.pyx":1086
 *           start = bisect_left_ui(rbuflb, item1, cs, 0) + cs*nchunk
 *         else:
 *           start = ss             # <<<<<<<<<<<<<<
 *       else:
 *         start = 0
 */
      /*else*/ {
        __pyx_v_start = __pyx_v_ss;
      }
      __pyx_L6:;

      /* "tables/indexesextension.pyx":1076
 *       nchunk = -1
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 */
      goto __pyx_L5;
    }

    /* "tables/indexesextension.pyx":1088
 *           start = ss
 *       else:
 *         start = 0             # <<<<<<<<<<<<<<
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 */
    /*else*/ {
      __pyx_v_start = 0;
    }
    __pyx_L5:;

    /* "tables/indexesextension.pyx":1090
 *         start = 0
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:
 */
    __pyx_t_3 = ((__pyx_v_item2 >= (__pyx_v_rbufrv[__pyx_v_rvrow])) != 0);
    if (__pyx_t_3) {

      /* "tables/indexesextension.pyx":1091
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 */
      __pyx_t_3 = ((__pyx_v_item2 < (__pyx_v_rbufrv[(__pyx_v_rvrow + 1)])) != 0);
      if (__pyx_t_3) {

        /* "tables/indexesextension.pyx":1092
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:             # <<<<<<<<<<<<<<
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_uint32 *>self.get_lru_bounds(nrow, nbounds)
 */
        __pyx_t_3 = ((!(__pyx_v_bread != 0)) != 0);
        if (__pyx_t_3) {

          /* "tables/indexesextension.pyx":1094
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_uint32 *>self.get_lru_bounds(nrow, nbounds)             # <<<<<<<<<<<<<<
 *           nchunk2 = bisect_right_ui(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:
 */
          __pyx_v_rbufbc = ((npy_uint32 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_bounds(__pyx_v_self, __pyx_v_nrow, __pyx_v_nbounds));

          /* "tables/indexesextension.pyx":1092
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:             # <<<<<<<<<<<<<<
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_uint32 *>self.get_lru_bounds(nrow, nbounds)
 */
        }

        /* "tables/indexesextension.pyx":1095
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_uint32 *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_ui(rbufbc, item2, nbounds, 0)             # <<<<<<<<<<<<<<
 *           if nchunk2 <> nchunk:
 *             # Get the sorted row from the LRU cache or read it.
 */
        __pyx_v_nchunk2 = bisect_right_ui(__pyx_v_rbufbc, __pyx_v_item2, __pyx_v_nbounds, 0);

        /* "tables/indexesextension.pyx":1096
 *             rbufbc = <npy_uint32 *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_ui(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:             # <<<<<<<<<<<<<<
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_uint32 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 */
        __pyx_t_3 = ((__pyx_v_nchunk2 != __pyx_v_nchunk) != 0);
        if (__pyx_t_3) {

          /* "tables/indexesextension.pyx":1098
 *           if nchunk2 <> nchunk:
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_uint32 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)             # <<<<<<<<<<<<<<
 *           stop = bisect_right_ui(rbuflb, item2, cs, 0) + cs*nchunk2
 *         else:
 */
          __pyx_v_rbuflb = ((npy_uint32 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_sorted(__pyx_v_self, __pyx_v_nrow, __pyx_v_ncs, __pyx_v_nchunk2, __pyx_v_cs));

          /* "tables/indexesextension.pyx":1096
 *             rbufbc = <npy_uint32 *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_ui(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:             # <<<<<<<<<<<<<<
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_uint32 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 */
        }

        /* "tables/indexesextension.pyx":1099
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_uint32 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 *           stop = bisect_right_ui(rbuflb, item2, cs, 0) + cs*nchunk2             # <<<<<<<<<<<<<<
 *         else:
 *           stop = ss
 */
        __pyx_v_stop = (bisect_right_ui(__pyx_v_rbuflb, __pyx_v_item2, __pyx_v_cs, 0) + (__pyx_v_cs * __pyx_v_nchunk2));

        /* "tables/indexesextension.pyx":1091
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 */
        goto __pyx_L8;
      }

      /* "tables/indexesextension.pyx":1101
 *           stop = bisect_right_ui(rbuflb, item2, cs, 0) + cs*nchunk2
 *         else:
 *           stop = ss             # <<<<<<<<<<<<<<
 *       else:
 *         stop = 0
 */
      /*else*/ {
        __pyx_v_stop = __pyx_v_ss;
      }
      __pyx_L8:;

      /* "tables/indexesextension.pyx":1090
 *         start = 0
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:
 */
      goto __pyx_L7;
    }

    /* "tables/indexesextension.pyx":1103
 *           stop = ss
 *       else:
 *         stop = 0             # <<<<<<<<<<<<<<
 *       length = stop - start
 *       tlength = tlength + length
 */
    /*else*/ {
      __pyx_v_stop = 0;
    }
    __pyx_L7:;

    /* "tables/indexesextension.pyx":1104
 *       else:
 *         stop = 0
 *       length = stop - start             # <<<<<<<<<<<<<<
 *       tlength = tlength + length
 *       rbufst[nrow] = start
 */
    __pyx_v_length = (__pyx_v_stop - __pyx_v_start);

    /* "tables/indexesextension.pyx":1105
 *         stop = 0
 *       length = stop - start
 *       tlength = tlength + length             # <<<<<<<<<<<<<<
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length
 */
    __pyx_v_tlength = (__pyx_v_tlength + __pyx_v_length);

    /* "tables/indexesextension.pyx":1106
 *       length = stop - start
 *       tlength = tlength + length
 *       rbufst[nrow] = start             # <<<<<<<<<<<<<<
 *       rbufln[nrow] = length
 *     return tlength
 */
    (__pyx_v_rbufst[__pyx_v_nrow]) = __pyx_v_start;

    /* "tables/indexesextension.pyx":1107
 *       tlength = tlength + length
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length             # <<<<<<<<<<<<<<
 *     return tlength
 * 
 */
    (__pyx_v_rbufln[__pyx_v_nrow]) = __pyx_v_length;
  }

  /* "tables/indexesextension.pyx":1108
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length
 *     return tlength             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_tlength); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1108, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "tables/indexesextension.pyx":1051
 * 
 *   # Optimized version for uint32
 *   def _search_bin_na_ui(self, npy_uint32 item1, npy_uint32 item2):             # <<<<<<<<<<<<<<
 *     cdef int cs, ss, ncs, nrow, nrows, nbounds, rvrow
 *     cdef int start, stop, tlength, length, bread, nchunk, nchunk2
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("tables.indexesextension.IndexArray._search_bin_na_ui", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/indexesextension.pyx":1112
 * 
 *   # Optimized version for int64
 *   def _search_bin_na_ll(self, npy_int64 item1, npy_int64 item2):             # <<<<<<<<<<<<<<
 *     cdef int cs, ss, ncs, nrow, nrows, nbounds, rvrow
 *     cdef int start, stop, tlength, length, bread, nchunk, nchunk2
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_19_search_bin_na_ll(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_19_search_bin_na_ll(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  npy_int64 __pyx_v_item1;
  npy_int64 __pyx_v_item2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_search_bin_na_ll (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_item1,&__pyx_n_s_item2,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_item1)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_item2)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_search_bin_na_ll", 1, 2, 2, 1); __PYX_ERR(0, 1112, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_search_bin_na_ll") < 0)) __PYX_ERR(0, 1112, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_item1 = __Pyx_PyInt_As_npy_int64(values[0]); if (unlikely((__pyx_v_item1 == ((npy_int64)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1112, __pyx_L3_error)
    __pyx_v_item2 = __Pyx_PyInt_As_npy_int64(values[1]); if (unlikely((__pyx_v_item2 == ((npy_int64)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1112, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_search_bin_na_ll", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1112, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("tables.indexesextension.IndexArray._search_bin_na_ll", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_16indexesextension_10IndexArray_18_search_bin_na_ll(((struct __pyx_obj_6tables_16indexesextension_IndexArray *)__pyx_v_self), __pyx_v_item1, __pyx_v_item2);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_18_search_bin_na_ll(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, npy_int64 __pyx_v_item1, npy_int64 __pyx_v_item2) {
  int __pyx_v_cs;
  int __pyx_v_ss;
  int __pyx_v_ncs;
  int __pyx_v_nrow;
  int __pyx_v_nrows;
  int __pyx_v_nbounds;
  int __pyx_v_rvrow;
  int __pyx_v_start;
  int __pyx_v_stop;
  int __pyx_v_tlength;
  int __pyx_v_length;
  int __pyx_v_bread;
  int __pyx_v_nchunk;
  int __pyx_v_nchunk2;
  int *__pyx_v_rbufst;
  int *__pyx_v_rbufln;
  npy_int64 *__pyx_v_rbufrv;
  npy_int64 *__pyx_v_rbufbc;
  npy_int64 *__pyx_v_rbuflb;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_search_bin_na_ll", 0);

  /* "tables/indexesextension.pyx":1120
 *     # Variables with specific type
 *     cdef npy_int64 *rbufrv
 *     cdef npy_int64 *rbufbc = NULL             # <<<<<<<<<<<<<<
 *     cdef npy_int64 *rbuflb = NULL
 * 
 */
  __pyx_v_rbufbc = NULL;

  /* "tables/indexesextension.pyx":1121
 *     cdef npy_int64 *rbufrv
 *     cdef npy_int64 *rbufbc = NULL
 *     cdef npy_int64 *rbuflb = NULL             # <<<<<<<<<<<<<<
 * 
 *     cs = self.l_chunksize
 */
  __pyx_v_rbuflb = NULL;

  /* "tables/indexesextension.pyx":1123
 *     cdef npy_int64 *rbuflb = NULL
 * 
 *     cs = self.l_chunksize             # <<<<<<<<<<<<<<
 *     ss = self.l_slicesize
 *     ncs = ss // cs
 */
  __pyx_t_1 = __pyx_v_self->l_chunksize;
  __pyx_v_cs = __pyx_t_1;

  /* "tables/indexesextension.pyx":1124
 * 
 *     cs = self.l_chunksize
 *     ss = self.l_slicesize             # <<<<<<<<<<<<<<
 *     ncs = ss // cs
 *     nbounds = self.nbounds
 */
  __pyx_t_1 = __pyx_v_self->l_slicesize;
  __pyx_v_ss = __pyx_t_1;

  /* "tables/indexesextension.pyx":1125
 *     cs = self.l_chunksize
 *     ss = self.l_slicesize
 *     ncs = ss // cs             # <<<<<<<<<<<<<<
 *     nbounds = self.nbounds
 *     nrows = self.nrows
 */
  if (unlikely(__pyx_v_cs == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
    __PYX_ERR(0, 1125, __pyx_L1_error)
  }
  else if (sizeof(int) == sizeof(long) && (!(((int)-1) > 0)) && unlikely(__pyx_v_cs == (int)-1)  && unlikely(UNARY_NEG_WOULD_OVERFLOW(__pyx_v_ss))) {
    PyErr_SetString(PyExc_OverflowError, "value too large to perform division");
    __PYX_ERR(0, 1125, __pyx_L1_error)
  }
  __pyx_v_ncs = __Pyx_div_int(__pyx_v_ss, __pyx_v_cs);

  /* "tables/indexesextension.pyx":1126
 *     ss = self.l_slicesize
 *     ncs = ss // cs
 *     nbounds = self.nbounds             # <<<<<<<<<<<<<<
 *     nrows = self.nrows
 *     rbufst = <int *>self.rbufst
 */
  __pyx_t_1 = __pyx_v_self->nbounds;
  __pyx_v_nbounds = __pyx_t_1;

  /* "tables/indexesextension.pyx":1127
 *     ncs = ss // cs
 *     nbounds = self.nbounds
 *     nrows = self.nrows             # <<<<<<<<<<<<<<
 *     rbufst = <int *>self.rbufst
 *     rbufln = <int *>self.rbufln
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nrows); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1127, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1127, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_nrows = __pyx_t_1;

  /* "tables/indexesextension.pyx":1128
 *     nbounds = self.nbounds
 *     nrows = self.nrows
 *     rbufst = <int *>self.rbufst             # <<<<<<<<<<<<<<
 *     rbufln = <int *>self.rbufln
 *     rbufrv = <npy_int64 *>self.rbufrv
 */
  __pyx_v_rbufst = ((int *)__pyx_v_self->rbufst);

  /* "tables/indexesextension.pyx":1129
 *     nrows = self.nrows
 *     rbufst = <int *>self.rbufst
 *     rbufln = <int *>self.rbufln             # <<<<<<<<<<<<<<
 *     rbufrv = <npy_int64 *>self.rbufrv
 *     tlength = 0
 */
  __pyx_v_rbufln = ((int *)__pyx_v_self->rbufln);

  /* "tables/indexesextension.pyx":1130
 *     rbufst = <int *>self.rbufst
 *     rbufln = <int *>self.rbufln
 *     rbufrv = <npy_int64 *>self.rbufrv             # <<<<<<<<<<<<<<
 *     tlength = 0
 *     for nrow from 0 <= nrow < nrows:
 */
  __pyx_v_rbufrv = ((npy_int64 *)__pyx_v_self->rbufrv);

  /* "tables/indexesextension.pyx":1131
 *     rbufln = <int *>self.rbufln
 *     rbufrv = <npy_int64 *>self.rbufrv
 *     tlength = 0             # <<<<<<<<<<<<<<
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2
 */
  __pyx_v_tlength = 0;

  /* "tables/indexesextension.pyx":1132
 *     rbufrv = <npy_int64 *>self.rbufrv
 *     tlength = 0
 *     for nrow from 0 <= nrow < nrows:             # <<<<<<<<<<<<<<
 *       rvrow = nrow*2
 *       bread = 0
 */
  __pyx_t_1 = __pyx_v_nrows;
  for (__pyx_v_nrow = 0; __pyx_v_nrow < __pyx_t_1; __pyx_v_nrow++) {

    /* "tables/indexesextension.pyx":1133
 *     tlength = 0
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2             # <<<<<<<<<<<<<<
 *       bread = 0
 *       nchunk = -1
 */
    __pyx_v_rvrow = (__pyx_v_nrow * 2);

    /* "tables/indexesextension.pyx":1134
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2
 *       bread = 0             # <<<<<<<<<<<<<<
 *       nchunk = -1
 *       # Look if item1 is in this row
 */
    __pyx_v_bread = 0;

    /* "tables/indexesextension.pyx":1135
 *       rvrow = nrow*2
 *       bread = 0
 *       nchunk = -1             # <<<<<<<<<<<<<<
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:
 */
    __pyx_v_nchunk = -1;

    /* "tables/indexesextension.pyx":1137
 *       nchunk = -1
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 */
    __pyx_t_3 = ((__pyx_v_item1 > (__pyx_v_rbufrv[__pyx_v_rvrow])) != 0);
    if (__pyx_t_3) {

      /* "tables/indexesextension.pyx":1138
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:
 *         if item1 <= rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_int64 *>self.get_lru_bounds(nrow, nbounds)
 */
      __pyx_t_3 = ((__pyx_v_item1 <= (__pyx_v_rbufrv[(__pyx_v_rvrow + 1)])) != 0);
      if (__pyx_t_3) {

        /* "tables/indexesextension.pyx":1140
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_int64 *>self.get_lru_bounds(nrow, nbounds)             # <<<<<<<<<<<<<<
 *           bread = 1
 *           nchunk = bisect_left_ll(rbufbc, item1, nbounds, 0)
 */
        __pyx_v_rbufbc = ((npy_int64 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_bounds(__pyx_v_self, __pyx_v_nrow, __pyx_v_nbounds));

        /* "tables/indexesextension.pyx":1141
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_int64 *>self.get_lru_bounds(nrow, nbounds)
 *           bread = 1             # <<<<<<<<<<<<<<
 *           nchunk = bisect_left_ll(rbufbc, item1, nbounds, 0)
 *           # Get the sorted row from the LRU cache or read it.
 */
        __pyx_v_bread = 1;

        /* "tables/indexesextension.pyx":1142
 *           rbufbc = <npy_int64 *>self.get_lru_bounds(nrow, nbounds)
 *           bread = 1
 *           nchunk = bisect_left_ll(rbufbc, item1, nbounds, 0)             # <<<<<<<<<<<<<<
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_int64 *>self.get_lru_sorted(nrow, ncs, nchunk, cs)
 */
        __pyx_v_nchunk = bisect_left_ll(__pyx_v_rbufbc, __pyx_v_item1, __pyx_v_nbounds, 0);

        /* "tables/indexesextension.pyx":1144
 *           nchunk = bisect_left_ll(rbufbc, item1, nbounds, 0)
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_int64 *>self.get_lru_sorted(nrow, ncs, nchunk, cs)             # <<<<<<<<<<<<<<
 *           start = bisect_left_ll(rbuflb, item1, cs, 0) + cs*nchunk
 *         else:
 */
        __pyx_v_rbuflb = ((npy_int64 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_sorted(__pyx_v_self, __pyx_v_nrow, __pyx_v_ncs, __pyx_v_nchunk, __pyx_v_cs));

        /* "tables/indexesextension.pyx":1145
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_int64 *>self.get_lru_sorted(nrow, ncs, nchunk, cs)
 *           start = bisect_left_ll(rbuflb, item1, cs, 0) + cs*nchunk             # <<<<<<<<<<<<<<
 *         else:
 *           start = ss
 */
        __pyx_v_start = (bisect_left_ll(__pyx_v_rbuflb, __pyx_v_item1, __pyx_v_cs, 0) + (__pyx_v_cs * __pyx_v_nchunk));

        /* "tables/indexesextension.pyx":1138
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:
 *         if item1 <= rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_int64 *>self.get_lru_bounds(nrow, nbounds)
 */
        goto __pyx_L6;
      }

      /* "tables/indexesextension.pyx":1147
 *           start = bisect_left_ll(rbuflb, item1, cs, 0) + cs*nchunk
 *         else:
 *           start = ss             # <<<<<<<<<<<<<<
 *       else:
 *         start = 0
 */
      /*else*/ {
        __pyx_v_start = __pyx_v_ss;
      }
      __pyx_L6:;

      /* "tables/indexesextension.pyx":1137
 *       nchunk = -1
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 */
      goto __pyx_L5;
    }

    /* "tables/indexesextension.pyx":1149
 *           start = ss
 *       else:
 *         start = 0             # <<<<<<<<<<<<<<
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 */
    /*else*/ {
      __pyx_v_start = 0;
    }
    __pyx_L5:;

    /* "tables/indexesextension.pyx":1151
 *         start = 0
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:
 */
    __pyx_t_3 = ((__pyx_v_item2 >= (__pyx_v_rbufrv[__pyx_v_rvrow])) != 0);
    if (__pyx_t_3) {

      /* "tables/indexesextension.pyx":1152
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 */
      __pyx_t_3 = ((__pyx_v_item2 < (__pyx_v_rbufrv[(__pyx_v_rvrow + 1)])) != 0);
      if (__pyx_t_3) {

        /* "tables/indexesextension.pyx":1153
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:             # <<<<<<<<<<<<<<
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_int64 *>self.get_lru_bounds(nrow, nbounds)
 */
        __pyx_t_3 = ((!(__pyx_v_bread != 0)) != 0);
        if (__pyx_t_3) {

          /* "tables/indexesextension.pyx":1155
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_int64 *>self.get_lru_bounds(nrow, nbounds)             # <<<<<<<<<<<<<<
 *           nchunk2 = bisect_right_ll(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:
 */
          __pyx_v_rbufbc = ((npy_int64 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_bounds(__pyx_v_self, __pyx_v_nrow, __pyx_v_nbounds));

          /* "tables/indexesextension.pyx":1153
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:             # <<<<<<<<<<<<<<
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_int64 *>self.get_lru_bounds(nrow, nbounds)
 */
        }

        /* "tables/indexesextension.pyx":1156
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_int64 *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_ll(rbufbc, item2, nbounds, 0)             # <<<<<<<<<<<<<<
 *           if nchunk2 <> nchunk:
 *             # Get the sorted row from the LRU cache or read it.
 */
        __pyx_v_nchunk2 = bisect_right_ll(__pyx_v_rbufbc, __pyx_v_item2, __pyx_v_nbounds, 0);

        /* "tables/indexesextension.pyx":1157
 *             rbufbc = <npy_int64 *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_ll(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:             # <<<<<<<<<<<<<<
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_int64 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 */
        __pyx_t_3 = ((__pyx_v_nchunk2 != __pyx_v_nchunk) != 0);
        if (__pyx_t_3) {

          /* "tables/indexesextension.pyx":1159
 *           if nchunk2 <> nchunk:
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_int64 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)             # <<<<<<<<<<<<<<
 *           stop = bisect_right_ll(rbuflb, item2, cs, 0) + cs*nchunk2
 *         else:
 */
          __pyx_v_rbuflb = ((npy_int64 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_sorted(__pyx_v_self, __pyx_v_nrow, __pyx_v_ncs, __pyx_v_nchunk2, __pyx_v_cs));

          /* "tables/indexesextension.pyx":1157
 *             rbufbc = <npy_int64 *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_ll(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:             # <<<<<<<<<<<<<<
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_int64 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 */
        }

        /* "tables/indexesextension.pyx":1160
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_int64 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 *           stop = bisect_right_ll(rbuflb, item2, cs, 0) + cs*nchunk2             # <<<<<<<<<<<<<<
 *         else:
 *           stop = ss
 */
        __pyx_v_stop = (bisect_right_ll(__pyx_v_rbuflb, __pyx_v_item2, __pyx_v_cs, 0) + (__pyx_v_cs * __pyx_v_nchunk2));

        /* "tables/indexesextension.pyx":1152
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 */
        goto __pyx_L8;
      }

      /* "tables/indexesextension.pyx":1162
 *           stop = bisect_right_ll(rbuflb, item2, cs, 0) + cs*nchunk2
 *         else:
 *           stop = ss             # <<<<<<<<<<<<<<
 *       else:
 *         stop = 0
 */
      /*else*/ {
        __pyx_v_stop = __pyx_v_ss;
      }
      __pyx_L8:;

      /* "tables/indexesextension.pyx":1151
 *         start = 0
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:
 */
      goto __pyx_L7;
    }

    /* "tables/indexesextension.pyx":1164
 *           stop = ss
 *       else:
 *         stop = 0             # <<<<<<<<<<<<<<
 *       length = stop - start
 *       tlength = tlength + length
 */
    /*else*/ {
      __pyx_v_stop = 0;
    }
    __pyx_L7:;

    /* "tables/indexesextension.pyx":1165
 *       else:
 *         stop = 0
 *       length = stop - start             # <<<<<<<<<<<<<<
 *       tlength = tlength + length
 *       rbufst[nrow] = start
 */
    __pyx_v_length = (__pyx_v_stop - __pyx_v_start);

    /* "tables/indexesextension.pyx":1166
 *         stop = 0
 *       length = stop - start
 *       tlength = tlength + length             # <<<<<<<<<<<<<<
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length
 */
    __pyx_v_tlength = (__pyx_v_tlength + __pyx_v_length);

    /* "tables/indexesextension.pyx":1167
 *       length = stop - start
 *       tlength = tlength + length
 *       rbufst[nrow] = start             # <<<<<<<<<<<<<<
 *       rbufln[nrow] = length
 *     return tlength
 */
    (__pyx_v_rbufst[__pyx_v_nrow]) = __pyx_v_start;

    /* "tables/indexesextension.pyx":1168
 *       tlength = tlength + length
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length             # <<<<<<<<<<<<<<
 *     return tlength
 * 
 */
    (__pyx_v_rbufln[__pyx_v_nrow]) = __pyx_v_length;
  }

  /* "tables/indexesextension.pyx":1169
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length
 *     return tlength             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_tlength); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1169, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "tables/indexesextension.pyx":1112
 * 
 *   # Optimized version for int64
 *   def _search_bin_na_ll(self, npy_int64 item1, npy_int64 item2):             # <<<<<<<<<<<<<<
 *     cdef int cs, ss, ncs, nrow, nrows, nbounds, rvrow
 *     cdef int start, stop, tlength, length, bread, nchunk, nchunk2
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("tables.indexesextension.IndexArray._search_bin_na_ll", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/indexesextension.pyx":1173
 * 
 *   # Optimized version for uint64
 *   def _search_bin_na_ull(self, npy_uint64 item1, npy_uint64 item2):             # <<<<<<<<<<<<<<
 *     cdef int cs, ss, ncs, nrow, nrows, nbounds, rvrow
 *     cdef int start, stop, tlength, length, bread, nchunk, nchunk2
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_21_search_bin_na_ull(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_21_search_bin_na_ull(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  npy_uint64 __pyx_v_item1;
  npy_uint64 __pyx_v_item2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_search_bin_na_ull (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_item1,&__pyx_n_s_item2,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_item1)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_item2)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_search_bin_na_ull", 1, 2, 2, 1); __PYX_ERR(0, 1173, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_search_bin_na_ull") < 0)) __PYX_ERR(0, 1173, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_item1 = __Pyx_PyInt_As_npy_uint64(values[0]); if (unlikely((__pyx_v_item1 == ((npy_uint64)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1173, __pyx_L3_error)
    __pyx_v_item2 = __Pyx_PyInt_As_npy_uint64(values[1]); if (unlikely((__pyx_v_item2 == ((npy_uint64)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1173, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_search_bin_na_ull", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1173, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("tables.indexesextension.IndexArray._search_bin_na_ull", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_16indexesextension_10IndexArray_20_search_bin_na_ull(((struct __pyx_obj_6tables_16indexesextension_IndexArray *)__pyx_v_self), __pyx_v_item1, __pyx_v_item2);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_20_search_bin_na_ull(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, npy_uint64 __pyx_v_item1, npy_uint64 __pyx_v_item2) {
  int __pyx_v_cs;
  int __pyx_v_ss;
  int __pyx_v_ncs;
  int __pyx_v_nrow;
  int __pyx_v_nrows;
  int __pyx_v_nbounds;
  int __pyx_v_rvrow;
  int __pyx_v_start;
  int __pyx_v_stop;
  int __pyx_v_tlength;
  int __pyx_v_length;
  int __pyx_v_bread;
  int __pyx_v_nchunk;
  int __pyx_v_nchunk2;
  int *__pyx_v_rbufst;
  int *__pyx_v_rbufln;
  npy_uint64 *__pyx_v_rbufrv;
  npy_uint64 *__pyx_v_rbufbc;
  npy_uint64 *__pyx_v_rbuflb;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_search_bin_na_ull", 0);

  /* "tables/indexesextension.pyx":1181
 *     # Variables with specific type
 *     cdef npy_uint64 *rbufrv
 *     cdef npy_uint64 *rbufbc = NULL             # <<<<<<<<<<<<<<
 *     cdef npy_uint64 *rbuflb = NULL
 * 
 */
  __pyx_v_rbufbc = NULL;

  /* "tables/indexesextension.pyx":1182
 *     cdef npy_uint64 *rbufrv
 *     cdef npy_uint64 *rbufbc = NULL
 *     cdef npy_uint64 *rbuflb = NULL             # <<<<<<<<<<<<<<
 * 
 *     cs = self.l_chunksize
 */
  __pyx_v_rbuflb = NULL;

  /* "tables/indexesextension.pyx":1184
 *     cdef npy_uint64 *rbuflb = NULL
 * 
 *     cs = self.l_chunksize             # <<<<<<<<<<<<<<
 *     ss = self.l_slicesize
 *     ncs = ss // cs
 */
  __pyx_t_1 = __pyx_v_self->l_chunksize;
  __pyx_v_cs = __pyx_t_1;

  /* "tables/indexesextension.pyx":1185
 * 
 *     cs = self.l_chunksize
 *     ss = self.l_slicesize             # <<<<<<<<<<<<<<
 *     ncs = ss // cs
 *     nbounds = self.nbounds
 */
  __pyx_t_1 = __pyx_v_self->l_slicesize;
  __pyx_v_ss = __pyx_t_1;

  /* "tables/indexesextension.pyx":1186
 *     cs = self.l_chunksize
 *     ss = self.l_slicesize
 *     ncs = ss // cs             # <<<<<<<<<<<<<<
 *     nbounds = self.nbounds
 *     nrows = self.nrows
 */
  if (unlikely(__pyx_v_cs == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
    __PYX_ERR(0, 1186, __pyx_L1_error)
  }
  else if (sizeof(int) == sizeof(long) && (!(((int)-1) > 0)) && unlikely(__pyx_v_cs == (int)-1)  && unlikely(UNARY_NEG_WOULD_OVERFLOW(__pyx_v_ss))) {
    PyErr_SetString(PyExc_OverflowError, "value too large to perform division");
    __PYX_ERR(0, 1186, __pyx_L1_error)
  }
  __pyx_v_ncs = __Pyx_div_int(__pyx_v_ss, __pyx_v_cs);

  /* "tables/indexesextension.pyx":1187
 *     ss = self.l_slicesize
 *     ncs = ss // cs
 *     nbounds = self.nbounds             # <<<<<<<<<<<<<<
 *     nrows = self.nrows
 *     rbufst = <int *>self.rbufst
 */
  __pyx_t_1 = __pyx_v_self->nbounds;
  __pyx_v_nbounds = __pyx_t_1;

  /* "tables/indexesextension.pyx":1188
 *     ncs = ss // cs
 *     nbounds = self.nbounds
 *     nrows = self.nrows             # <<<<<<<<<<<<<<
 *     rbufst = <int *>self.rbufst
 *     rbufln = <int *>self.rbufln
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nrows); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1188, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1188, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_nrows = __pyx_t_1;

  /* "tables/indexesextension.pyx":1189
 *     nbounds = self.nbounds
 *     nrows = self.nrows
 *     rbufst = <int *>self.rbufst             # <<<<<<<<<<<<<<
 *     rbufln = <int *>self.rbufln
 *     rbufrv = <npy_uint64 *>self.rbufrv
 */
  __pyx_v_rbufst = ((int *)__pyx_v_self->rbufst);

  /* "tables/indexesextension.pyx":1190
 *     nrows = self.nrows
 *     rbufst = <int *>self.rbufst
 *     rbufln = <int *>self.rbufln             # <<<<<<<<<<<<<<
 *     rbufrv = <npy_uint64 *>self.rbufrv
 *     tlength = 0
 */
  __pyx_v_rbufln = ((int *)__pyx_v_self->rbufln);

  /* "tables/indexesextension.pyx":1191
 *     rbufst = <int *>self.rbufst
 *     rbufln = <int *>self.rbufln
 *     rbufrv = <npy_uint64 *>self.rbufrv             # <<<<<<<<<<<<<<
 *     tlength = 0
 *     for nrow from 0 <= nrow < nrows:
 */
  __pyx_v_rbufrv = ((npy_uint64 *)__pyx_v_self->rbufrv);

  /* "tables/indexesextension.pyx":1192
 *     rbufln = <int *>self.rbufln
 *     rbufrv = <npy_uint64 *>self.rbufrv
 *     tlength = 0             # <<<<<<<<<<<<<<
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2
 */
  __pyx_v_tlength = 0;

  /* "tables/indexesextension.pyx":1193
 *     rbufrv = <npy_uint64 *>self.rbufrv
 *     tlength = 0
 *     for nrow from 0 <= nrow < nrows:             # <<<<<<<<<<<<<<
 *       rvrow = nrow*2
 *       bread = 0
 */
  __pyx_t_1 = __pyx_v_nrows;
  for (__pyx_v_nrow = 0; __pyx_v_nrow < __pyx_t_1; __pyx_v_nrow++) {

    /* "tables/indexesextension.pyx":1194
 *     tlength = 0
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2             # <<<<<<<<<<<<<<
 *       bread = 0
 *       nchunk = -1
 */
    __pyx_v_rvrow = (__pyx_v_nrow * 2);

    /* "tables/indexesextension.pyx":1195
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2
 *       bread = 0             # <<<<<<<<<<<<<<
 *       nchunk = -1
 *       # Look if item1 is in this row
 */
    __pyx_v_bread = 0;

    /* "tables/indexesextension.pyx":1196
 *       rvrow = nrow*2
 *       bread = 0
 *       nchunk = -1             # <<<<<<<<<<<<<<
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:
 */
    __pyx_v_nchunk = -1;

    /* "tables/indexesextension.pyx":1198
 *       nchunk = -1
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 */
    __pyx_t_3 = ((__pyx_v_item1 > (__pyx_v_rbufrv[__pyx_v_rvrow])) != 0);
    if (__pyx_t_3) {

      /* "tables/indexesextension.pyx":1199
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:
 *         if item1 <= rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_uint64 *>self.get_lru_bounds(nrow, nbounds)
 */
      __pyx_t_3 = ((__pyx_v_item1 <= (__pyx_v_rbufrv[(__pyx_v_rvrow + 1)])) != 0);
      if (__pyx_t_3) {

        /* "tables/indexesextension.pyx":1201
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_uint64 *>self.get_lru_bounds(nrow, nbounds)             # <<<<<<<<<<<<<<
 *           bread = 1
 *           nchunk = bisect_left_ull(rbufbc, item1, nbounds, 0)
 */
        __pyx_v_rbufbc = ((npy_uint64 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_bounds(__pyx_v_self, __pyx_v_nrow, __pyx_v_nbounds));

        /* "tables/indexesextension.pyx":1202
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_uint64 *>self.get_lru_bounds(nrow, nbounds)
 *           bread = 1             # <<<<<<<<<<<<<<
 *           nchunk = bisect_left_ull(rbufbc, item1, nbounds, 0)
 *           # Get the sorted row from the LRU cache or read it.
 */
        __pyx_v_bread = 1;

        /* "tables/indexesextension.pyx":1203
 *           rbufbc = <npy_uint64 *>self.get_lru_bounds(nrow, nbounds)
 *           bread = 1
 *           nchunk = bisect_left_ull(rbufbc, item1, nbounds, 0)             # <<<<<<<<<<<<<<
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_uint64 *>self.get_lru_sorted(nrow, ncs, nchunk, cs)
 */
        __pyx_v_nchunk = bisect_left_ull(__pyx_v_rbufbc, __pyx_v_item1, __pyx_v_nbounds, 0);

        /* "tables/indexesextension.pyx":1205
 *           nchunk = bisect_left_ull(rbufbc, item1, nbounds, 0)
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_uint64 *>self.get_lru_sorted(nrow, ncs, nchunk, cs)             # <<<<<<<<<<<<<<
 *           start = bisect_left_ull(rbuflb, item1, cs, 0) + cs*nchunk
 *         else:
 */
        __pyx_v_rbuflb = ((npy_uint64 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_sorted(__pyx_v_self, __pyx_v_nrow, __pyx_v_ncs, __pyx_v_nchunk, __pyx_v_cs));

        /* "tables/indexesextension.pyx":1206
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_uint64 *>self.get_lru_sorted(nrow, ncs, nchunk, cs)
 *           start = bisect_left_ull(rbuflb, item1, cs, 0) + cs*nchunk             # <<<<<<<<<<<<<<
 *         else:
 *           start = ss
 */
        __pyx_v_start = (bisect_left_ull(__pyx_v_rbuflb, __pyx_v_item1, __pyx_v_cs, 0) + (__pyx_v_cs * __pyx_v_nchunk));

        /* "tables/indexesextension.pyx":1199
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:
 *         if item1 <= rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_uint64 *>self.get_lru_bounds(nrow, nbounds)
 */
        goto __pyx_L6;
      }

      /* "tables/indexesextension.pyx":1208
 *           start = bisect_left_ull(rbuflb, item1, cs, 0) + cs*nchunk
 *         else:
 *           start = ss             # <<<<<<<<<<<<<<
 *       else:
 *         start = 0
 */
      /*else*/ {
        __pyx_v_start = __pyx_v_ss;
      }
      __pyx_L6:;

      /* "tables/indexesextension.pyx":1198
 *       nchunk = -1
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 */
      goto __pyx_L5;
    }

    /* "tables/indexesextension.pyx":1210
 *           start = ss
 *       else:
 *         start = 0             # <<<<<<<<<<<<<<
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 */
    /*else*/ {
      __pyx_v_start = 0;
    }
    __pyx_L5:;

    /* "tables/indexesextension.pyx":1212
 *         start = 0
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:
 */
    __pyx_t_3 = ((__pyx_v_item2 >= (__pyx_v_rbufrv[__pyx_v_rvrow])) != 0);
    if (__pyx_t_3) {

      /* "tables/indexesextension.pyx":1213
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 */
      __pyx_t_3 = ((__pyx_v_item2 < (__pyx_v_rbufrv[(__pyx_v_rvrow + 1)])) != 0);
      if (__pyx_t_3) {

        /* "tables/indexesextension.pyx":1214
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:             # <<<<<<<<<<<<<<
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_uint64 *>self.get_lru_bounds(nrow, nbounds)
 */
        __pyx_t_3 = ((!(__pyx_v_bread != 0)) != 0);
        if (__pyx_t_3) {

          /* "tables/indexesextension.pyx":1216
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_uint64 *>self.get_lru_bounds(nrow, nbounds)             # <<<<<<<<<<<<<<
 *           nchunk2 = bisect_right_ull(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:
 */
          __pyx_v_rbufbc = ((npy_uint64 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_bounds(__pyx_v_self, __pyx_v_nrow, __pyx_v_nbounds));

          /* "tables/indexesextension.pyx":1214
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:             # <<<<<<<<<<<<<<
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_uint64 *>self.get_lru_bounds(nrow, nbounds)
 */
        }

        /* "tables/indexesextension.pyx":1217
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_uint64 *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_ull(rbufbc, item2, nbounds, 0)             # <<<<<<<<<<<<<<
 *           if nchunk2 <> nchunk:
 *             # Get the sorted row from the LRU cache or read it.
 */
        __pyx_v_nchunk2 = bisect_right_ull(__pyx_v_rbufbc, __pyx_v_item2, __pyx_v_nbounds, 0);

        /* "tables/indexesextension.pyx":1218
 *             rbufbc = <npy_uint64 *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_ull(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:             # <<<<<<<<<<<<<<
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_uint64 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 */
        __pyx_t_3 = ((__pyx_v_nchunk2 != __pyx_v_nchunk) != 0);
        if (__pyx_t_3) {

          /* "tables/indexesextension.pyx":1220
 *           if nchunk2 <> nchunk:
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_uint64 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)             # <<<<<<<<<<<<<<
 *           stop = bisect_right_ull(rbuflb, item2, cs, 0) + cs*nchunk2
 *         else:
 */
          __pyx_v_rbuflb = ((npy_uint64 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_sorted(__pyx_v_self, __pyx_v_nrow, __pyx_v_ncs, __pyx_v_nchunk2, __pyx_v_cs));

          /* "tables/indexesextension.pyx":1218
 *             rbufbc = <npy_uint64 *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_ull(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:             # <<<<<<<<<<<<<<
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_uint64 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 */
        }

        /* "tables/indexesextension.pyx":1221
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_uint64 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 *           stop = bisect_right_ull(rbuflb, item2, cs, 0) + cs*nchunk2             # <<<<<<<<<<<<<<
 *         else:
 *           stop = ss
 */
        __pyx_v_stop = (bisect_right_ull(__pyx_v_rbuflb, __pyx_v_item2, __pyx_v_cs, 0) + (__pyx_v_cs * __pyx_v_nchunk2));

        /* "tables/indexesextension.pyx":1213
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 */
        goto __pyx_L8;
      }

      /* "tables/indexesextension.pyx":1223
 *           stop = bisect_right_ull(rbuflb, item2, cs, 0) + cs*nchunk2
 *         else:
 *           stop = ss             # <<<<<<<<<<<<<<
 *       else:
 *         stop = 0
 */
      /*else*/ {
        __pyx_v_stop = __pyx_v_ss;
      }
      __pyx_L8:;

      /* "tables/indexesextension.pyx":1212
 *         start = 0
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:
 */
      goto __pyx_L7;
    }

    /* "tables/indexesextension.pyx":1225
 *           stop = ss
 *       else:
 *         stop = 0             # <<<<<<<<<<<<<<
 *       length = stop - start
 *       tlength = tlength + length
 */
    /*else*/ {
      __pyx_v_stop = 0;
    }
    __pyx_L7:;

    /* "tables/indexesextension.pyx":1226
 *       else:
 *         stop = 0
 *       length = stop - start             # <<<<<<<<<<<<<<
 *       tlength = tlength + length
 *       rbufst[nrow] = start
 */
    __pyx_v_length = (__pyx_v_stop - __pyx_v_start);

    /* "tables/indexesextension.pyx":1227
 *         stop = 0
 *       length = stop - start
 *       tlength = tlength + length             # <<<<<<<<<<<<<<
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length
 */
    __pyx_v_tlength = (__pyx_v_tlength + __pyx_v_length);

    /* "tables/indexesextension.pyx":1228
 *       length = stop - start
 *       tlength = tlength + length
 *       rbufst[nrow] = start             # <<<<<<<<<<<<<<
 *       rbufln[nrow] = length
 *     return tlength
 */
    (__pyx_v_rbufst[__pyx_v_nrow]) = __pyx_v_start;

    /* "tables/indexesextension.pyx":1229
 *       tlength = tlength + length
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length             # <<<<<<<<<<<<<<
 *     return tlength
 * 
 */
    (__pyx_v_rbufln[__pyx_v_nrow]) = __pyx_v_length;
  }

  /* "tables/indexesextension.pyx":1230
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length
 *     return tlength             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_tlength); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1230, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "tables/indexesextension.pyx":1173
 * 
 *   # Optimized version for uint64
 *   def _search_bin_na_ull(self, npy_uint64 item1, npy_uint64 item2):             # <<<<<<<<<<<<<<
 *     cdef int cs, ss, ncs, nrow, nrows, nbounds, rvrow
 *     cdef int start, stop, tlength, length, bread, nchunk, nchunk2
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("tables.indexesextension.IndexArray._search_bin_na_ull", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/indexesextension.pyx":1234
 * 
 *   # Optimized version for float16
 *   def _search_bin_na_e(self, npy_float64 item1, npy_float64 item2):             # <<<<<<<<<<<<<<
 *     cdef int cs, ss, ncs, nrow, nrows, nrow2, nbounds, rvrow
 *     cdef int start, stop, tlength, length, bread, nchunk, nchunk2
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_23_search_bin_na_e(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_23_search_bin_na_e(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  npy_float64 __pyx_v_item1;
  npy_float64 __pyx_v_item2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_search_bin_na_e (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_item1,&__pyx_n_s_item2,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_item1)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_item2)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_search_bin_na_e", 1, 2, 2, 1); __PYX_ERR(0, 1234, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_search_bin_na_e") < 0)) __PYX_ERR(0, 1234, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_item1 = __pyx_PyFloat_AsDouble(values[0]); if (unlikely((__pyx_v_item1 == ((npy_float64)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1234, __pyx_L3_error)
    __pyx_v_item2 = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_item2 == ((npy_float64)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1234, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_search_bin_na_e", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1234, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("tables.indexesextension.IndexArray._search_bin_na_e", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_16indexesextension_10IndexArray_22_search_bin_na_e(((struct __pyx_obj_6tables_16indexesextension_IndexArray *)__pyx_v_self), __pyx_v_item1, __pyx_v_item2);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_22_search_bin_na_e(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, npy_float64 __pyx_v_item1, npy_float64 __pyx_v_item2) {
  int __pyx_v_cs;
  int __pyx_v_ss;
  int __pyx_v_ncs;
  int __pyx_v_nrow;
  int __pyx_v_nrows;
  int __pyx_v_nbounds;
  int __pyx_v_rvrow;
  int __pyx_v_start;
  int __pyx_v_stop;
  int __pyx_v_tlength;
  int __pyx_v_length;
  int __pyx_v_bread;
  int __pyx_v_nchunk;
  int __pyx_v_nchunk2;
  int *__pyx_v_rbufst;
  int *__pyx_v_rbufln;
  __pyx_t_6tables_16indexesextension_npy_float16 *__pyx_v_rbufrv;
  __pyx_t_6tables_16indexesextension_npy_float16 *__pyx_v_rbufbc;
  __pyx_t_6tables_16indexesextension_npy_float16 *__pyx_v_rbuflb;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_search_bin_na_e", 0);

  /* "tables/indexesextension.pyx":1242
 *     # Variables with specific type
 *     cdef npy_float16 *rbufrv
 *     cdef npy_float16 *rbufbc = NULL             # <<<<<<<<<<<<<<
 *     cdef npy_float16 *rbuflb = NULL
 * 
 */
  __pyx_v_rbufbc = NULL;

  /* "tables/indexesextension.pyx":1243
 *     cdef npy_float16 *rbufrv
 *     cdef npy_float16 *rbufbc = NULL
 *     cdef npy_float16 *rbuflb = NULL             # <<<<<<<<<<<<<<
 * 
 *     cs = self.l_chunksize
 */
  __pyx_v_rbuflb = NULL;

  /* "tables/indexesextension.pyx":1245
 *     cdef npy_float16 *rbuflb = NULL
 * 
 *     cs = self.l_chunksize             # <<<<<<<<<<<<<<
 *     ss = self.l_slicesize
 *     ncs = ss // cs
 */
  __pyx_t_1 = __pyx_v_self->l_chunksize;
  __pyx_v_cs = __pyx_t_1;

  /* "tables/indexesextension.pyx":1246
 * 
 *     cs = self.l_chunksize
 *     ss = self.l_slicesize             # <<<<<<<<<<<<<<
 *     ncs = ss // cs
 *     nbounds = self.nbounds
 */
  __pyx_t_1 = __pyx_v_self->l_slicesize;
  __pyx_v_ss = __pyx_t_1;

  /* "tables/indexesextension.pyx":1247
 *     cs = self.l_chunksize
 *     ss = self.l_slicesize
 *     ncs = ss // cs             # <<<<<<<<<<<<<<
 *     nbounds = self.nbounds
 *     nrows = self.nrows
 */
  if (unlikely(__pyx_v_cs == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
    __PYX_ERR(0, 1247, __pyx_L1_error)
  }
  else if (sizeof(int) == sizeof(long) && (!(((int)-1) > 0)) && unlikely(__pyx_v_cs == (int)-1)  && unlikely(UNARY_NEG_WOULD_OVERFLOW(__pyx_v_ss))) {
    PyErr_SetString(PyExc_OverflowError, "value too large to perform division");
    __PYX_ERR(0, 1247, __pyx_L1_error)
  }
  __pyx_v_ncs = __Pyx_div_int(__pyx_v_ss, __pyx_v_cs);

  /* "tables/indexesextension.pyx":1248
 *     ss = self.l_slicesize
 *     ncs = ss // cs
 *     nbounds = self.nbounds             # <<<<<<<<<<<<<<
 *     nrows = self.nrows
 *     tlength = 0
 */
  __pyx_t_1 = __pyx_v_self->nbounds;
  __pyx_v_nbounds = __pyx_t_1;

  /* "tables/indexesextension.pyx":1249
 *     ncs = ss // cs
 *     nbounds = self.nbounds
 *     nrows = self.nrows             # <<<<<<<<<<<<<<
 *     tlength = 0
 *     rbufst = <int *>self.rbufst
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nrows); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1249, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_nrows = __pyx_t_1;

  /* "tables/indexesextension.pyx":1250
 *     nbounds = self.nbounds
 *     nrows = self.nrows
 *     tlength = 0             # <<<<<<<<<<<<<<
 *     rbufst = <int *>self.rbufst
 *     rbufln = <int *>self.rbufln
 */
  __pyx_v_tlength = 0;

  /* "tables/indexesextension.pyx":1251
 *     nrows = self.nrows
 *     tlength = 0
 *     rbufst = <int *>self.rbufst             # <<<<<<<<<<<<<<
 *     rbufln = <int *>self.rbufln
 *     # Limits not in cache, do a lookup
 */
  __pyx_v_rbufst = ((int *)__pyx_v_self->rbufst);

  /* "tables/indexesextension.pyx":1252
 *     tlength = 0
 *     rbufst = <int *>self.rbufst
 *     rbufln = <int *>self.rbufln             # <<<<<<<<<<<<<<
 *     # Limits not in cache, do a lookup
 *     rbufrv = <npy_float16 *>self.rbufrv
 */
  __pyx_v_rbufln = ((int *)__pyx_v_self->rbufln);

  /* "tables/indexesextension.pyx":1254
 *     rbufln = <int *>self.rbufln
 *     # Limits not in cache, do a lookup
 *     rbufrv = <npy_float16 *>self.rbufrv             # <<<<<<<<<<<<<<
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2
 */
  __pyx_v_rbufrv = ((__pyx_t_6tables_16indexesextension_npy_float16 *)__pyx_v_self->rbufrv);

  /* "tables/indexesextension.pyx":1255
 *     # Limits not in cache, do a lookup
 *     rbufrv = <npy_float16 *>self.rbufrv
 *     for nrow from 0 <= nrow < nrows:             # <<<<<<<<<<<<<<
 *       rvrow = nrow*2
 *       bread = 0
 */
  __pyx_t_1 = __pyx_v_nrows;
  for (__pyx_v_nrow = 0; __pyx_v_nrow < __pyx_t_1; __pyx_v_nrow++) {

    /* "tables/indexesextension.pyx":1256
 *     rbufrv = <npy_float16 *>self.rbufrv
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2             # <<<<<<<<<<<<<<
 *       bread = 0
 *       nchunk = -1
 */
    __pyx_v_rvrow = (__pyx_v_nrow * 2);

    /* "tables/indexesextension.pyx":1257
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2
 *       bread = 0             # <<<<<<<<<<<<<<
 *       nchunk = -1
 * 
 */
    __pyx_v_bread = 0;

    /* "tables/indexesextension.pyx":1258
 *       rvrow = nrow*2
 *       bread = 0
 *       nchunk = -1             # <<<<<<<<<<<<<<
 * 
 *       # Look if item1 is in this row
 */
    __pyx_v_nchunk = -1;

    /* "tables/indexesextension.pyx":1261
 * 
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 */
    __pyx_t_3 = ((__pyx_v_item1 > (__pyx_v_rbufrv[__pyx_v_rvrow])) != 0);
    if (__pyx_t_3) {

      /* "tables/indexesextension.pyx":1262
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:
 *         if item1 <= rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_float16 *>self.get_lru_bounds(nrow, nbounds)
 */
      __pyx_t_3 = ((__pyx_v_item1 <= (__pyx_v_rbufrv[(__pyx_v_rvrow + 1)])) != 0);
      if (__pyx_t_3) {

        /* "tables/indexesextension.pyx":1264
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_float16 *>self.get_lru_bounds(nrow, nbounds)             # <<<<<<<<<<<<<<
 *           bread = 1
 *           nchunk = bisect_left_e(rbufbc, item1, nbounds, 0)
 */
        __pyx_v_rbufbc = ((__pyx_t_6tables_16indexesextension_npy_float16 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_bounds(__pyx_v_self, __pyx_v_nrow, __pyx_v_nbounds));

        /* "tables/indexesextension.pyx":1265
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_float16 *>self.get_lru_bounds(nrow, nbounds)
 *           bread = 1             # <<<<<<<<<<<<<<
 *           nchunk = bisect_left_e(rbufbc, item1, nbounds, 0)
 *           # Get the sorted row from the LRU cache or read it.
 */
        __pyx_v_bread = 1;

        /* "tables/indexesextension.pyx":1266
 *           rbufbc = <npy_float16 *>self.get_lru_bounds(nrow, nbounds)
 *           bread = 1
 *           nchunk = bisect_left_e(rbufbc, item1, nbounds, 0)             # <<<<<<<<<<<<<<
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_float16 *>self.get_lru_sorted(nrow, ncs, nchunk, cs)
 */
        __pyx_v_nchunk = bisect_left_e(__pyx_v_rbufbc, __pyx_v_item1, __pyx_v_nbounds, 0);

        /* "tables/indexesextension.pyx":1268
 *           nchunk = bisect_left_e(rbufbc, item1, nbounds, 0)
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_float16 *>self.get_lru_sorted(nrow, ncs, nchunk, cs)             # <<<<<<<<<<<<<<
 *           start = bisect_left_e(rbuflb, item1, cs, 0) + cs*nchunk
 *         else:
 */
        __pyx_v_rbuflb = ((__pyx_t_6tables_16indexesextension_npy_float16 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_sorted(__pyx_v_self, __pyx_v_nrow, __pyx_v_ncs, __pyx_v_nchunk, __pyx_v_cs));

        /* "tables/indexesextension.pyx":1269
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_float16 *>self.get_lru_sorted(nrow, ncs, nchunk, cs)
 *           start = bisect_left_e(rbuflb, item1, cs, 0) + cs*nchunk             # <<<<<<<<<<<<<<
 *         else:
 *           start = ss
 */
        __pyx_v_start = (bisect_left_e(__pyx_v_rbuflb, __pyx_v_item1, __pyx_v_cs, 0) + (__pyx_v_cs * __pyx_v_nchunk));

        /* "tables/indexesextension.pyx":1262
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:
 *         if item1 <= rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_float16 *>self.get_lru_bounds(nrow, nbounds)
 */
        goto __pyx_L6;
      }

      /* "tables/indexesextension.pyx":1271
 *           start = bisect_left_e(rbuflb, item1, cs, 0) + cs*nchunk
 *         else:
 *           start = ss             # <<<<<<<<<<<<<<
 *       else:
 *         start = 0
 */
      /*else*/ {
        __pyx_v_start = __pyx_v_ss;
      }
      __pyx_L6:;

      /* "tables/indexesextension.pyx":1261
 * 
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 */
      goto __pyx_L5;
    }

    /* "tables/indexesextension.pyx":1273
 *           start = ss
 *       else:
 *         start = 0             # <<<<<<<<<<<<<<
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 */
    /*else*/ {
      __pyx_v_start = 0;
    }
    __pyx_L5:;

    /* "tables/indexesextension.pyx":1275
 *         start = 0
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:
 */
    __pyx_t_3 = ((__pyx_v_item2 >= (__pyx_v_rbufrv[__pyx_v_rvrow])) != 0);
    if (__pyx_t_3) {

      /* "tables/indexesextension.pyx":1276
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 */
      __pyx_t_3 = ((__pyx_v_item2 < (__pyx_v_rbufrv[(__pyx_v_rvrow + 1)])) != 0);
      if (__pyx_t_3) {

        /* "tables/indexesextension.pyx":1277
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:             # <<<<<<<<<<<<<<
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_float16 *>self.get_lru_bounds(nrow, nbounds)
 */
        __pyx_t_3 = ((!(__pyx_v_bread != 0)) != 0);
        if (__pyx_t_3) {

          /* "tables/indexesextension.pyx":1279
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_float16 *>self.get_lru_bounds(nrow, nbounds)             # <<<<<<<<<<<<<<
 *           nchunk2 = bisect_right_e(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:
 */
          __pyx_v_rbufbc = ((__pyx_t_6tables_16indexesextension_npy_float16 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_bounds(__pyx_v_self, __pyx_v_nrow, __pyx_v_nbounds));

          /* "tables/indexesextension.pyx":1277
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:             # <<<<<<<<<<<<<<
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_float16 *>self.get_lru_bounds(nrow, nbounds)
 */
        }

        /* "tables/indexesextension.pyx":1280
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_float16 *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_e(rbufbc, item2, nbounds, 0)             # <<<<<<<<<<<<<<
 *           if nchunk2 <> nchunk:
 *             # Get the sorted row from the LRU cache or read it.
 */
        __pyx_v_nchunk2 = bisect_right_e(__pyx_v_rbufbc, __pyx_v_item2, __pyx_v_nbounds, 0);

        /* "tables/indexesextension.pyx":1281
 *             rbufbc = <npy_float16 *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_e(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:             # <<<<<<<<<<<<<<
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_float16 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 */
        __pyx_t_3 = ((__pyx_v_nchunk2 != __pyx_v_nchunk) != 0);
        if (__pyx_t_3) {

          /* "tables/indexesextension.pyx":1283
 *           if nchunk2 <> nchunk:
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_float16 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)             # <<<<<<<<<<<<<<
 *           stop = bisect_right_e(rbuflb, item2, cs, 0) + cs*nchunk2
 *         else:
 */
          __pyx_v_rbuflb = ((__pyx_t_6tables_16indexesextension_npy_float16 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_sorted(__pyx_v_self, __pyx_v_nrow, __pyx_v_ncs, __pyx_v_nchunk2, __pyx_v_cs));

          /* "tables/indexesextension.pyx":1281
 *             rbufbc = <npy_float16 *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_e(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:             # <<<<<<<<<<<<<<
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_float16 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 */
        }

        /* "tables/indexesextension.pyx":1284
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_float16 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 *           stop = bisect_right_e(rbuflb, item2, cs, 0) + cs*nchunk2             # <<<<<<<<<<<<<<
 *         else:
 *           stop = ss
 */
        __pyx_v_stop = (bisect_right_e(__pyx_v_rbuflb, __pyx_v_item2, __pyx_v_cs, 0) + (__pyx_v_cs * __pyx_v_nchunk2));

        /* "tables/indexesextension.pyx":1276
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 */
        goto __pyx_L8;
      }

      /* "tables/indexesextension.pyx":1286
 *           stop = bisect_right_e(rbuflb, item2, cs, 0) + cs*nchunk2
 *         else:
 *           stop = ss             # <<<<<<<<<<<<<<
 *       else:
 *         stop = 0
 */
      /*else*/ {
        __pyx_v_stop = __pyx_v_ss;
      }
      __pyx_L8:;

      /* "tables/indexesextension.pyx":1275
 *         start = 0
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:
 */
      goto __pyx_L7;
    }

    /* "tables/indexesextension.pyx":1288
 *           stop = ss
 *       else:
 *         stop = 0             # <<<<<<<<<<<<<<
 *       length = stop - start
 *       tlength = tlength + length
 */
    /*else*/ {
      __pyx_v_stop = 0;
    }
    __pyx_L7:;

    /* "tables/indexesextension.pyx":1289
 *       else:
 *         stop = 0
 *       length = stop - start             # <<<<<<<<<<<<<<
 *       tlength = tlength + length
 *       rbufst[nrow] = start
 */
    __pyx_v_length = (__pyx_v_stop - __pyx_v_start);

    /* "tables/indexesextension.pyx":1290
 *         stop = 0
 *       length = stop - start
 *       tlength = tlength + length             # <<<<<<<<<<<<<<
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length
 */
    __pyx_v_tlength = (__pyx_v_tlength + __pyx_v_length);

    /* "tables/indexesextension.pyx":1291
 *       length = stop - start
 *       tlength = tlength + length
 *       rbufst[nrow] = start             # <<<<<<<<<<<<<<
 *       rbufln[nrow] = length
 *     return tlength
 */
    (__pyx_v_rbufst[__pyx_v_nrow]) = __pyx_v_start;

    /* "tables/indexesextension.pyx":1292
 *       tlength = tlength + length
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length             # <<<<<<<<<<<<<<
 *     return tlength
 * 
 */
    (__pyx_v_rbufln[__pyx_v_nrow]) = __pyx_v_length;
  }

  /* "tables/indexesextension.pyx":1293
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length
 *     return tlength             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_tlength); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1293, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "tables/indexesextension.pyx":1234
 * 
 *   # Optimized version for float16
 *   def _search_bin_na_e(self, npy_float64 item1, npy_float64 item2):             # <<<<<<<<<<<<<<
 *     cdef int cs, ss, ncs, nrow, nrows, nrow2, nbounds, rvrow
 *     cdef int start, stop, tlength, length, bread, nchunk, nchunk2
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("tables.indexesextension.IndexArray._search_bin_na_e", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/indexesextension.pyx":1297
 * 
 *   # Optimized version for float32
 *   def _search_bin_na_f(self, npy_float64 item1, npy_float64 item2):             # <<<<<<<<<<<<<<
 *     cdef int cs, ss, ncs, nrow, nrows, nrow2, nbounds, rvrow
 *     cdef int start, stop, tlength, length, bread, nchunk, nchunk2
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_25_search_bin_na_f(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_25_search_bin_na_f(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  npy_float64 __pyx_v_item1;
  npy_float64 __pyx_v_item2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_search_bin_na_f (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_item1,&__pyx_n_s_item2,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_item1)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_item2)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_search_bin_na_f", 1, 2, 2, 1); __PYX_ERR(0, 1297, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_search_bin_na_f") < 0)) __PYX_ERR(0, 1297, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_item1 = __pyx_PyFloat_AsDouble(values[0]); if (unlikely((__pyx_v_item1 == ((npy_float64)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1297, __pyx_L3_error)
    __pyx_v_item2 = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_item2 == ((npy_float64)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1297, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_search_bin_na_f", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1297, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("tables.indexesextension.IndexArray._search_bin_na_f", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_16indexesextension_10IndexArray_24_search_bin_na_f(((struct __pyx_obj_6tables_16indexesextension_IndexArray *)__pyx_v_self), __pyx_v_item1, __pyx_v_item2);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_24_search_bin_na_f(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, npy_float64 __pyx_v_item1, npy_float64 __pyx_v_item2) {
  int __pyx_v_cs;
  int __pyx_v_ss;
  int __pyx_v_ncs;
  int __pyx_v_nrow;
  int __pyx_v_nrows;
  int __pyx_v_nbounds;
  int __pyx_v_rvrow;
  int __pyx_v_start;
  int __pyx_v_stop;
  int __pyx_v_tlength;
  int __pyx_v_length;
  int __pyx_v_bread;
  int __pyx_v_nchunk;
  int __pyx_v_nchunk2;
  int *__pyx_v_rbufst;
  int *__pyx_v_rbufln;
  npy_float32 *__pyx_v_rbufrv;
  npy_float32 *__pyx_v_rbufbc;
  npy_float32 *__pyx_v_rbuflb;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_search_bin_na_f", 0);

  /* "tables/indexesextension.pyx":1304
 *     # Variables with specific type
 *     cdef npy_float32 *rbufrv
 *     cdef npy_float32 *rbufbc = NULL             # <<<<<<<<<<<<<<
 *     cdef npy_float32 *rbuflb = NULL
 * 
 */
  __pyx_v_rbufbc = NULL;

  /* "tables/indexesextension.pyx":1305
 *     cdef npy_float32 *rbufrv
 *     cdef npy_float32 *rbufbc = NULL
 *     cdef npy_float32 *rbuflb = NULL             # <<<<<<<<<<<<<<
 * 
 *     cs = self.l_chunksize
 */
  __pyx_v_rbuflb = NULL;

  /* "tables/indexesextension.pyx":1307
 *     cdef npy_float32 *rbuflb = NULL
 * 
 *     cs = self.l_chunksize             # <<<<<<<<<<<<<<
 *     ss = self.l_slicesize
 *     ncs = ss // cs
 */
  __pyx_t_1 = __pyx_v_self->l_chunksize;
  __pyx_v_cs = __pyx_t_1;

  /* "tables/indexesextension.pyx":1308
 * 
 *     cs = self.l_chunksize
 *     ss = self.l_slicesize             # <<<<<<<<<<<<<<
 *     ncs = ss // cs
 *     nbounds = self.nbounds
 */
  __pyx_t_1 = __pyx_v_self->l_slicesize;
  __pyx_v_ss = __pyx_t_1;

  /* "tables/indexesextension.pyx":1309
 *     cs = self.l_chunksize
 *     ss = self.l_slicesize
 *     ncs = ss // cs             # <<<<<<<<<<<<<<
 *     nbounds = self.nbounds
 *     nrows = self.nrows
 */
  if (unlikely(__pyx_v_cs == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
    __PYX_ERR(0, 1309, __pyx_L1_error)
  }
  else if (sizeof(int) == sizeof(long) && (!(((int)-1) > 0)) && unlikely(__pyx_v_cs == (int)-1)  && unlikely(UNARY_NEG_WOULD_OVERFLOW(__pyx_v_ss))) {
    PyErr_SetString(PyExc_OverflowError, "value too large to perform division");
    __PYX_ERR(0, 1309, __pyx_L1_error)
  }
  __pyx_v_ncs = __Pyx_div_int(__pyx_v_ss, __pyx_v_cs);

  /* "tables/indexesextension.pyx":1310
 *     ss = self.l_slicesize
 *     ncs = ss // cs
 *     nbounds = self.nbounds             # <<<<<<<<<<<<<<
 *     nrows = self.nrows
 *     tlength = 0
 */
  __pyx_t_1 = __pyx_v_self->nbounds;
  __pyx_v_nbounds = __pyx_t_1;

  /* "tables/indexesextension.pyx":1311
 *     ncs = ss // cs
 *     nbounds = self.nbounds
 *     nrows = self.nrows             # <<<<<<<<<<<<<<
 *     tlength = 0
 *     rbufst = <int *>self.rbufst
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nrows); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1311, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1311, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_nrows = __pyx_t_1;

  /* "tables/indexesextension.pyx":1312
 *     nbounds = self.nbounds
 *     nrows = self.nrows
 *     tlength = 0             # <<<<<<<<<<<<<<
 *     rbufst = <int *>self.rbufst
 *     rbufln = <int *>self.rbufln
 */
  __pyx_v_tlength = 0;

  /* "tables/indexesextension.pyx":1313
 *     nrows = self.nrows
 *     tlength = 0
 *     rbufst = <int *>self.rbufst             # <<<<<<<<<<<<<<
 *     rbufln = <int *>self.rbufln
 * 
 */
  __pyx_v_rbufst = ((int *)__pyx_v_self->rbufst);

  /* "tables/indexesextension.pyx":1314
 *     tlength = 0
 *     rbufst = <int *>self.rbufst
 *     rbufln = <int *>self.rbufln             # <<<<<<<<<<<<<<
 * 
 *     # Limits not in cache, do a lookup
 */
  __pyx_v_rbufln = ((int *)__pyx_v_self->rbufln);

  /* "tables/indexesextension.pyx":1317
 * 
 *     # Limits not in cache, do a lookup
 *     rbufrv = <npy_float32 *>self.rbufrv             # <<<<<<<<<<<<<<
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2
 */
  __pyx_v_rbufrv = ((npy_float32 *)__pyx_v_self->rbufrv);

  /* "tables/indexesextension.pyx":1318
 *     # Limits not in cache, do a lookup
 *     rbufrv = <npy_float32 *>self.rbufrv
 *     for nrow from 0 <= nrow < nrows:             # <<<<<<<<<<<<<<
 *       rvrow = nrow*2
 *       bread = 0
 */
  __pyx_t_1 = __pyx_v_nrows;
  for (__pyx_v_nrow = 0; __pyx_v_nrow < __pyx_t_1; __pyx_v_nrow++) {

    /* "tables/indexesextension.pyx":1319
 *     rbufrv = <npy_float32 *>self.rbufrv
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2             # <<<<<<<<<<<<<<
 *       bread = 0
 *       nchunk = -1
 */
    __pyx_v_rvrow = (__pyx_v_nrow * 2);

    /* "tables/indexesextension.pyx":1320
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2
 *       bread = 0             # <<<<<<<<<<<<<<
 *       nchunk = -1
 *       # Look if item1 is in this row
 */
    __pyx_v_bread = 0;

    /* "tables/indexesextension.pyx":1321
 *       rvrow = nrow*2
 *       bread = 0
 *       nchunk = -1             # <<<<<<<<<<<<<<
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:
 */
    __pyx_v_nchunk = -1;

    /* "tables/indexesextension.pyx":1323
 *       nchunk = -1
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 */
    __pyx_t_3 = ((__pyx_v_item1 > (__pyx_v_rbufrv[__pyx_v_rvrow])) != 0);
    if (__pyx_t_3) {

      /* "tables/indexesextension.pyx":1324
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:
 *         if item1 <= rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_float32 *>self.get_lru_bounds(nrow, nbounds)
 */
      __pyx_t_3 = ((__pyx_v_item1 <= (__pyx_v_rbufrv[(__pyx_v_rvrow + 1)])) != 0);
      if (__pyx_t_3) {

        /* "tables/indexesextension.pyx":1326
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_float32 *>self.get_lru_bounds(nrow, nbounds)             # <<<<<<<<<<<<<<
 *           bread = 1
 *           nchunk = bisect_left_f(rbufbc, item1, nbounds, 0)
 */
        __pyx_v_rbufbc = ((npy_float32 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_bounds(__pyx_v_self, __pyx_v_nrow, __pyx_v_nbounds));

        /* "tables/indexesextension.pyx":1327
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_float32 *>self.get_lru_bounds(nrow, nbounds)
 *           bread = 1             # <<<<<<<<<<<<<<
 *           nchunk = bisect_left_f(rbufbc, item1, nbounds, 0)
 *           # Get the sorted row from the LRU cache or read it.
 */
        __pyx_v_bread = 1;

        /* "tables/indexesextension.pyx":1328
 *           rbufbc = <npy_float32 *>self.get_lru_bounds(nrow, nbounds)
 *           bread = 1
 *           nchunk = bisect_left_f(rbufbc, item1, nbounds, 0)             # <<<<<<<<<<<<<<
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_float32 *>self.get_lru_sorted(nrow, ncs, nchunk, cs)
 */
        __pyx_v_nchunk = bisect_left_f(__pyx_v_rbufbc, __pyx_v_item1, __pyx_v_nbounds, 0);

        /* "tables/indexesextension.pyx":1330
 *           nchunk = bisect_left_f(rbufbc, item1, nbounds, 0)
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_float32 *>self.get_lru_sorted(nrow, ncs, nchunk, cs)             # <<<<<<<<<<<<<<
 *           start = bisect_left_f(rbuflb, item1, cs, 0) + cs*nchunk
 *         else:
 */
        __pyx_v_rbuflb = ((npy_float32 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_sorted(__pyx_v_self, __pyx_v_nrow, __pyx_v_ncs, __pyx_v_nchunk, __pyx_v_cs));

        /* "tables/indexesextension.pyx":1331
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_float32 *>self.get_lru_sorted(nrow, ncs, nchunk, cs)
 *           start = bisect_left_f(rbuflb, item1, cs, 0) + cs*nchunk             # <<<<<<<<<<<<<<
 *         else:
 *           start = ss
 */
        __pyx_v_start = (bisect_left_f(__pyx_v_rbuflb, __pyx_v_item1, __pyx_v_cs, 0) + (__pyx_v_cs * __pyx_v_nchunk));

        /* "tables/indexesextension.pyx":1324
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:
 *         if item1 <= rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_float32 *>self.get_lru_bounds(nrow, nbounds)
 */
        goto __pyx_L6;
      }

      /* "tables/indexesextension.pyx":1333
 *           start = bisect_left_f(rbuflb, item1, cs, 0) + cs*nchunk
 *         else:
 *           start = ss             # <<<<<<<<<<<<<<
 *       else:
 *         start = 0
 */
      /*else*/ {
        __pyx_v_start = __pyx_v_ss;
      }
      __pyx_L6:;

      /* "tables/indexesextension.pyx":1323
 *       nchunk = -1
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 */
      goto __pyx_L5;
    }

    /* "tables/indexesextension.pyx":1335
 *           start = ss
 *       else:
 *         start = 0             # <<<<<<<<<<<<<<
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 */
    /*else*/ {
      __pyx_v_start = 0;
    }
    __pyx_L5:;

    /* "tables/indexesextension.pyx":1337
 *         start = 0
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:
 */
    __pyx_t_3 = ((__pyx_v_item2 >= (__pyx_v_rbufrv[__pyx_v_rvrow])) != 0);
    if (__pyx_t_3) {

      /* "tables/indexesextension.pyx":1338
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 */
      __pyx_t_3 = ((__pyx_v_item2 < (__pyx_v_rbufrv[(__pyx_v_rvrow + 1)])) != 0);
      if (__pyx_t_3) {

        /* "tables/indexesextension.pyx":1339
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:             # <<<<<<<<<<<<<<
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_float32 *>self.get_lru_bounds(nrow, nbounds)
 */
        __pyx_t_3 = ((!(__pyx_v_bread != 0)) != 0);
        if (__pyx_t_3) {

          /* "tables/indexesextension.pyx":1341
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_float32 *>self.get_lru_bounds(nrow, nbounds)             # <<<<<<<<<<<<<<
 *           nchunk2 = bisect_right_f(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:
 */
          __pyx_v_rbufbc = ((npy_float32 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_bounds(__pyx_v_self, __pyx_v_nrow, __pyx_v_nbounds));

          /* "tables/indexesextension.pyx":1339
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:             # <<<<<<<<<<<<<<
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_float32 *>self.get_lru_bounds(nrow, nbounds)
 */
        }

        /* "tables/indexesextension.pyx":1342
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_float32 *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_f(rbufbc, item2, nbounds, 0)             # <<<<<<<<<<<<<<
 *           if nchunk2 <> nchunk:
 *             # Get the sorted row from the LRU cache or read it.
 */
        __pyx_v_nchunk2 = bisect_right_f(__pyx_v_rbufbc, __pyx_v_item2, __pyx_v_nbounds, 0);

        /* "tables/indexesextension.pyx":1343
 *             rbufbc = <npy_float32 *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_f(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:             # <<<<<<<<<<<<<<
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_float32 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 */
        __pyx_t_3 = ((__pyx_v_nchunk2 != __pyx_v_nchunk) != 0);
        if (__pyx_t_3) {

          /* "tables/indexesextension.pyx":1345
 *           if nchunk2 <> nchunk:
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_float32 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)             # <<<<<<<<<<<<<<
 *           stop = bisect_right_f(rbuflb, item2, cs, 0) + cs*nchunk2
 *         else:
 */
          __pyx_v_rbuflb = ((npy_float32 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_sorted(__pyx_v_self, __pyx_v_nrow, __pyx_v_ncs, __pyx_v_nchunk2, __pyx_v_cs));

          /* "tables/indexesextension.pyx":1343
 *             rbufbc = <npy_float32 *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_f(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:             # <<<<<<<<<<<<<<
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_float32 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 */
        }

        /* "tables/indexesextension.pyx":1346
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_float32 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 *           stop = bisect_right_f(rbuflb, item2, cs, 0) + cs*nchunk2             # <<<<<<<<<<<<<<
 *         else:
 *           stop = ss
 */
        __pyx_v_stop = (bisect_right_f(__pyx_v_rbuflb, __pyx_v_item2, __pyx_v_cs, 0) + (__pyx_v_cs * __pyx_v_nchunk2));

        /* "tables/indexesextension.pyx":1338
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 */
        goto __pyx_L8;
      }

      /* "tables/indexesextension.pyx":1348
 *           stop = bisect_right_f(rbuflb, item2, cs, 0) + cs*nchunk2
 *         else:
 *           stop = ss             # <<<<<<<<<<<<<<
 *       else:
 *         stop = 0
 */
      /*else*/ {
        __pyx_v_stop = __pyx_v_ss;
      }
      __pyx_L8:;

      /* "tables/indexesextension.pyx":1337
 *         start = 0
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:
 */
      goto __pyx_L7;
    }

    /* "tables/indexesextension.pyx":1350
 *           stop = ss
 *       else:
 *         stop = 0             # <<<<<<<<<<<<<<
 *       length = stop - start
 *       tlength = tlength + length
 */
    /*else*/ {
      __pyx_v_stop = 0;
    }
    __pyx_L7:;

    /* "tables/indexesextension.pyx":1351
 *       else:
 *         stop = 0
 *       length = stop - start             # <<<<<<<<<<<<<<
 *       tlength = tlength + length
 *       rbufst[nrow] = start
 */
    __pyx_v_length = (__pyx_v_stop - __pyx_v_start);

    /* "tables/indexesextension.pyx":1352
 *         stop = 0
 *       length = stop - start
 *       tlength = tlength + length             # <<<<<<<<<<<<<<
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length
 */
    __pyx_v_tlength = (__pyx_v_tlength + __pyx_v_length);

    /* "tables/indexesextension.pyx":1353
 *       length = stop - start
 *       tlength = tlength + length
 *       rbufst[nrow] = start             # <<<<<<<<<<<<<<
 *       rbufln[nrow] = length
 *     return tlength
 */
    (__pyx_v_rbufst[__pyx_v_nrow]) = __pyx_v_start;

    /* "tables/indexesextension.pyx":1354
 *       tlength = tlength + length
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length             # <<<<<<<<<<<<<<
 *     return tlength
 * 
 */
    (__pyx_v_rbufln[__pyx_v_nrow]) = __pyx_v_length;
  }

  /* "tables/indexesextension.pyx":1355
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length
 *     return tlength             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_tlength); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1355, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "tables/indexesextension.pyx":1297
 * 
 *   # Optimized version for float32
 *   def _search_bin_na_f(self, npy_float64 item1, npy_float64 item2):             # <<<<<<<<<<<<<<
 *     cdef int cs, ss, ncs, nrow, nrows, nrow2, nbounds, rvrow
 *     cdef int start, stop, tlength, length, bread, nchunk, nchunk2
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("tables.indexesextension.IndexArray._search_bin_na_f", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/indexesextension.pyx":1359
 * 
 *   # Optimized version for float64
 *   def _search_bin_na_d(self, npy_float64 item1, npy_float64 item2):             # <<<<<<<<<<<<<<
 *     cdef int cs, ss, ncs, nrow, nrows, nrow2, nbounds, rvrow
 *     cdef int start, stop, tlength, length, bread, nchunk, nchunk2
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_27_search_bin_na_d(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_27_search_bin_na_d(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  npy_float64 __pyx_v_item1;
  npy_float64 __pyx_v_item2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_search_bin_na_d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_item1,&__pyx_n_s_item2,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_item1)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_item2)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_search_bin_na_d", 1, 2, 2, 1); __PYX_ERR(0, 1359, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_search_bin_na_d") < 0)) __PYX_ERR(0, 1359, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_item1 = __pyx_PyFloat_AsDouble(values[0]); if (unlikely((__pyx_v_item1 == ((npy_float64)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1359, __pyx_L3_error)
    __pyx_v_item2 = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_item2 == ((npy_float64)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1359, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_search_bin_na_d", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1359, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("tables.indexesextension.IndexArray._search_bin_na_d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_16indexesextension_10IndexArray_26_search_bin_na_d(((struct __pyx_obj_6tables_16indexesextension_IndexArray *)__pyx_v_self), __pyx_v_item1, __pyx_v_item2);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_26_search_bin_na_d(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, npy_float64 __pyx_v_item1, npy_float64 __pyx_v_item2) {
  int __pyx_v_cs;
  int __pyx_v_ss;
  int __pyx_v_ncs;
  int __pyx_v_nrow;
  int __pyx_v_nrows;
  int __pyx_v_nbounds;
  int __pyx_v_rvrow;
  int __pyx_v_start;
  int __pyx_v_stop;
  int __pyx_v_tlength;
  int __pyx_v_length;
  int __pyx_v_bread;
  int __pyx_v_nchunk;
  int __pyx_v_nchunk2;
  int *__pyx_v_rbufst;
  int *__pyx_v_rbufln;
  npy_float64 *__pyx_v_rbufrv;
  npy_float64 *__pyx_v_rbufbc;
  npy_float64 *__pyx_v_rbuflb;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_search_bin_na_d", 0);

  /* "tables/indexesextension.pyx":1367
 *     # Variables with specific type
 *     cdef npy_float64 *rbufrv
 *     cdef npy_float64 *rbufbc = NULL             # <<<<<<<<<<<<<<
 *     cdef npy_float64 *rbuflb = NULL
 * 
 */
  __pyx_v_rbufbc = NULL;

  /* "tables/indexesextension.pyx":1368
 *     cdef npy_float64 *rbufrv
 *     cdef npy_float64 *rbufbc = NULL
 *     cdef npy_float64 *rbuflb = NULL             # <<<<<<<<<<<<<<
 * 
 *     cs = self.l_chunksize
 */
  __pyx_v_rbuflb = NULL;

  /* "tables/indexesextension.pyx":1370
 *     cdef npy_float64 *rbuflb = NULL
 * 
 *     cs = self.l_chunksize             # <<<<<<<<<<<<<<
 *     ss = self.l_slicesize
 *     ncs = ss // cs
 */
  __pyx_t_1 = __pyx_v_self->l_chunksize;
  __pyx_v_cs = __pyx_t_1;

  /* "tables/indexesextension.pyx":1371
 * 
 *     cs = self.l_chunksize
 *     ss = self.l_slicesize             # <<<<<<<<<<<<<<
 *     ncs = ss // cs
 *     nbounds = self.nbounds
 */
  __pyx_t_1 = __pyx_v_self->l_slicesize;
  __pyx_v_ss = __pyx_t_1;

  /* "tables/indexesextension.pyx":1372
 *     cs = self.l_chunksize
 *     ss = self.l_slicesize
 *     ncs = ss // cs             # <<<<<<<<<<<<<<
 *     nbounds = self.nbounds
 *     nrows = self.nrows
 */
  if (unlikely(__pyx_v_cs == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
    __PYX_ERR(0, 1372, __pyx_L1_error)
  }
  else if (sizeof(int) == sizeof(long) && (!(((int)-1) > 0)) && unlikely(__pyx_v_cs == (int)-1)  && unlikely(UNARY_NEG_WOULD_OVERFLOW(__pyx_v_ss))) {
    PyErr_SetString(PyExc_OverflowError, "value too large to perform division");
    __PYX_ERR(0, 1372, __pyx_L1_error)
  }
  __pyx_v_ncs = __Pyx_div_int(__pyx_v_ss, __pyx_v_cs);

  /* "tables/indexesextension.pyx":1373
 *     ss = self.l_slicesize
 *     ncs = ss // cs
 *     nbounds = self.nbounds             # <<<<<<<<<<<<<<
 *     nrows = self.nrows
 *     tlength = 0
 */
  __pyx_t_1 = __pyx_v_self->nbounds;
  __pyx_v_nbounds = __pyx_t_1;

  /* "tables/indexesextension.pyx":1374
 *     ncs = ss // cs
 *     nbounds = self.nbounds
 *     nrows = self.nrows             # <<<<<<<<<<<<<<
 *     tlength = 0
 *     rbufst = <int *>self.rbufst
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nrows); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1374, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1374, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_nrows = __pyx_t_1;

  /* "tables/indexesextension.pyx":1375
 *     nbounds = self.nbounds
 *     nrows = self.nrows
 *     tlength = 0             # <<<<<<<<<<<<<<
 *     rbufst = <int *>self.rbufst
 *     rbufln = <int *>self.rbufln
 */
  __pyx_v_tlength = 0;

  /* "tables/indexesextension.pyx":1376
 *     nrows = self.nrows
 *     tlength = 0
 *     rbufst = <int *>self.rbufst             # <<<<<<<<<<<<<<
 *     rbufln = <int *>self.rbufln
 * 
 */
  __pyx_v_rbufst = ((int *)__pyx_v_self->rbufst);

  /* "tables/indexesextension.pyx":1377
 *     tlength = 0
 *     rbufst = <int *>self.rbufst
 *     rbufln = <int *>self.rbufln             # <<<<<<<<<<<<<<
 * 
 *     # Limits not in cache, do a lookup
 */
  __pyx_v_rbufln = ((int *)__pyx_v_self->rbufln);

  /* "tables/indexesextension.pyx":1380
 * 
 *     # Limits not in cache, do a lookup
 *     rbufrv = <npy_float64 *>self.rbufrv             # <<<<<<<<<<<<<<
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2
 */
  __pyx_v_rbufrv = ((npy_float64 *)__pyx_v_self->rbufrv);

  /* "tables/indexesextension.pyx":1381
 *     # Limits not in cache, do a lookup
 *     rbufrv = <npy_float64 *>self.rbufrv
 *     for nrow from 0 <= nrow < nrows:             # <<<<<<<<<<<<<<
 *       rvrow = nrow*2
 *       bread = 0
 */
  __pyx_t_1 = __pyx_v_nrows;
  for (__pyx_v_nrow = 0; __pyx_v_nrow < __pyx_t_1; __pyx_v_nrow++) {

    /* "tables/indexesextension.pyx":1382
 *     rbufrv = <npy_float64 *>self.rbufrv
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2             # <<<<<<<<<<<<<<
 *       bread = 0
 *       nchunk = -1
 */
    __pyx_v_rvrow = (__pyx_v_nrow * 2);

    /* "tables/indexesextension.pyx":1383
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2
 *       bread = 0             # <<<<<<<<<<<<<<
 *       nchunk = -1
 * 
 */
    __pyx_v_bread = 0;

    /* "tables/indexesextension.pyx":1384
 *       rvrow = nrow*2
 *       bread = 0
 *       nchunk = -1             # <<<<<<<<<<<<<<
 * 
 *       # Look if item1 is in this row
 */
    __pyx_v_nchunk = -1;

    /* "tables/indexesextension.pyx":1387
 * 
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 */
    __pyx_t_3 = ((__pyx_v_item1 > (__pyx_v_rbufrv[__pyx_v_rvrow])) != 0);
    if (__pyx_t_3) {

      /* "tables/indexesextension.pyx":1388
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:
 *         if item1 <= rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_float64 *>self.get_lru_bounds(nrow, nbounds)
 */
      __pyx_t_3 = ((__pyx_v_item1 <= (__pyx_v_rbufrv[(__pyx_v_rvrow + 1)])) != 0);
      if (__pyx_t_3) {

        /* "tables/indexesextension.pyx":1390
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_float64 *>self.get_lru_bounds(nrow, nbounds)             # <<<<<<<<<<<<<<
 *           bread = 1
 *           nchunk = bisect_left_d(rbufbc, item1, nbounds, 0)
 */
        __pyx_v_rbufbc = ((npy_float64 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_bounds(__pyx_v_self, __pyx_v_nrow, __pyx_v_nbounds));

        /* "tables/indexesextension.pyx":1391
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_float64 *>self.get_lru_bounds(nrow, nbounds)
 *           bread = 1             # <<<<<<<<<<<<<<
 *           nchunk = bisect_left_d(rbufbc, item1, nbounds, 0)
 *           # Get the sorted row from the LRU cache or read it.
 */
        __pyx_v_bread = 1;

        /* "tables/indexesextension.pyx":1392
 *           rbufbc = <npy_float64 *>self.get_lru_bounds(nrow, nbounds)
 *           bread = 1
 *           nchunk = bisect_left_d(rbufbc, item1, nbounds, 0)             # <<<<<<<<<<<<<<
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_float64 *>self.get_lru_sorted(nrow, ncs, nchunk, cs)
 */
        __pyx_v_nchunk = bisect_left_d(__pyx_v_rbufbc, __pyx_v_item1, __pyx_v_nbounds, 0);

        /* "tables/indexesextension.pyx":1394
 *           nchunk = bisect_left_d(rbufbc, item1, nbounds, 0)
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_float64 *>self.get_lru_sorted(nrow, ncs, nchunk, cs)             # <<<<<<<<<<<<<<
 *           start = bisect_left_d(rbuflb, item1, cs, 0) + cs*nchunk
 *         else:
 */
        __pyx_v_rbuflb = ((npy_float64 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_sorted(__pyx_v_self, __pyx_v_nrow, __pyx_v_ncs, __pyx_v_nchunk, __pyx_v_cs));

        /* "tables/indexesextension.pyx":1395
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_float64 *>self.get_lru_sorted(nrow, ncs, nchunk, cs)
 *           start = bisect_left_d(rbuflb, item1, cs, 0) + cs*nchunk             # <<<<<<<<<<<<<<
 *         else:
 *           start = ss
 */
        __pyx_v_start = (bisect_left_d(__pyx_v_rbuflb, __pyx_v_item1, __pyx_v_cs, 0) + (__pyx_v_cs * __pyx_v_nchunk));

        /* "tables/indexesextension.pyx":1388
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:
 *         if item1 <= rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_float64 *>self.get_lru_bounds(nrow, nbounds)
 */
        goto __pyx_L6;
      }

      /* "tables/indexesextension.pyx":1397
 *           start = bisect_left_d(rbuflb, item1, cs, 0) + cs*nchunk
 *         else:
 *           start = ss             # <<<<<<<<<<<<<<
 *       else:
 *         start = 0
 */
      /*else*/ {
        __pyx_v_start = __pyx_v_ss;
      }
      __pyx_L6:;

      /* "tables/indexesextension.pyx":1387
 * 
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 */
      goto __pyx_L5;
    }

    /* "tables/indexesextension.pyx":1399
 *           start = ss
 *       else:
 *         start = 0             # <<<<<<<<<<<<<<
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 */
    /*else*/ {
      __pyx_v_start = 0;
    }
    __pyx_L5:;

    /* "tables/indexesextension.pyx":1401
 *         start = 0
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:
 */
    __pyx_t_3 = ((__pyx_v_item2 >= (__pyx_v_rbufrv[__pyx_v_rvrow])) != 0);
    if (__pyx_t_3) {

      /* "tables/indexesextension.pyx":1402
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 */
      __pyx_t_3 = ((__pyx_v_item2 < (__pyx_v_rbufrv[(__pyx_v_rvrow + 1)])) != 0);
      if (__pyx_t_3) {

        /* "tables/indexesextension.pyx":1403
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:             # <<<<<<<<<<<<<<
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_float64 *>self.get_lru_bounds(nrow, nbounds)
 */
        __pyx_t_3 = ((!(__pyx_v_bread != 0)) != 0);
        if (__pyx_t_3) {

          /* "tables/indexesextension.pyx":1405
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_float64 *>self.get_lru_bounds(nrow, nbounds)             # <<<<<<<<<<<<<<
 *           nchunk2 = bisect_right_d(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:
 */
          __pyx_v_rbufbc = ((npy_float64 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_bounds(__pyx_v_self, __pyx_v_nrow, __pyx_v_nbounds));

          /* "tables/indexesextension.pyx":1403
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:             # <<<<<<<<<<<<<<
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_float64 *>self.get_lru_bounds(nrow, nbounds)
 */
        }

        /* "tables/indexesextension.pyx":1406
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_float64 *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_d(rbufbc, item2, nbounds, 0)             # <<<<<<<<<<<<<<
 *           if nchunk2 <> nchunk:
 *             # Get the sorted row from the LRU cache or read it.
 */
        __pyx_v_nchunk2 = bisect_right_d(__pyx_v_rbufbc, __pyx_v_item2, __pyx_v_nbounds, 0);

        /* "tables/indexesextension.pyx":1407
 *             rbufbc = <npy_float64 *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_d(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:             # <<<<<<<<<<<<<<
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_float64 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 */
        __pyx_t_3 = ((__pyx_v_nchunk2 != __pyx_v_nchunk) != 0);
        if (__pyx_t_3) {

          /* "tables/indexesextension.pyx":1409
 *           if nchunk2 <> nchunk:
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_float64 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)             # <<<<<<<<<<<<<<
 *           stop = bisect_right_d(rbuflb, item2, cs, 0) + cs*nchunk2
 *         else:
 */
          __pyx_v_rbuflb = ((npy_float64 *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_sorted(__pyx_v_self, __pyx_v_nrow, __pyx_v_ncs, __pyx_v_nchunk2, __pyx_v_cs));

          /* "tables/indexesextension.pyx":1407
 *             rbufbc = <npy_float64 *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_d(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:             # <<<<<<<<<<<<<<
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_float64 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 */
        }

        /* "tables/indexesextension.pyx":1410
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_float64 *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 *           stop = bisect_right_d(rbuflb, item2, cs, 0) + cs*nchunk2             # <<<<<<<<<<<<<<
 *         else:
 *           stop = ss
 */
        __pyx_v_stop = (bisect_right_d(__pyx_v_rbuflb, __pyx_v_item2, __pyx_v_cs, 0) + (__pyx_v_cs * __pyx_v_nchunk2));

        /* "tables/indexesextension.pyx":1402
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 */
        goto __pyx_L8;
      }

      /* "tables/indexesextension.pyx":1412
 *           stop = bisect_right_d(rbuflb, item2, cs, 0) + cs*nchunk2
 *         else:
 *           stop = ss             # <<<<<<<<<<<<<<
 *       else:
 *         stop = 0
 */
      /*else*/ {
        __pyx_v_stop = __pyx_v_ss;
      }
      __pyx_L8:;

      /* "tables/indexesextension.pyx":1401
 *         start = 0
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:
 */
      goto __pyx_L7;
    }

    /* "tables/indexesextension.pyx":1414
 *           stop = ss
 *       else:
 *         stop = 0             # <<<<<<<<<<<<<<
 *       length = stop - start
 *       tlength = tlength + length
 */
    /*else*/ {
      __pyx_v_stop = 0;
    }
    __pyx_L7:;

    /* "tables/indexesextension.pyx":1415
 *       else:
 *         stop = 0
 *       length = stop - start             # <<<<<<<<<<<<<<
 *       tlength = tlength + length
 *       rbufst[nrow] = start
 */
    __pyx_v_length = (__pyx_v_stop - __pyx_v_start);

    /* "tables/indexesextension.pyx":1416
 *         stop = 0
 *       length = stop - start
 *       tlength = tlength + length             # <<<<<<<<<<<<<<
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length
 */
    __pyx_v_tlength = (__pyx_v_tlength + __pyx_v_length);

    /* "tables/indexesextension.pyx":1417
 *       length = stop - start
 *       tlength = tlength + length
 *       rbufst[nrow] = start             # <<<<<<<<<<<<<<
 *       rbufln[nrow] = length
 *     return tlength
 */
    (__pyx_v_rbufst[__pyx_v_nrow]) = __pyx_v_start;

    /* "tables/indexesextension.pyx":1418
 *       tlength = tlength + length
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length             # <<<<<<<<<<<<<<
 *     return tlength
 * 
 */
    (__pyx_v_rbufln[__pyx_v_nrow]) = __pyx_v_length;
  }

  /* "tables/indexesextension.pyx":1419
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length
 *     return tlength             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_tlength); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1419, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "tables/indexesextension.pyx":1359
 * 
 *   # Optimized version for float64
 *   def _search_bin_na_d(self, npy_float64 item1, npy_float64 item2):             # <<<<<<<<<<<<<<
 *     cdef int cs, ss, ncs, nrow, nrows, nrow2, nbounds, rvrow
 *     cdef int start, stop, tlength, length, bread, nchunk, nchunk2
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("tables.indexesextension.IndexArray._search_bin_na_d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/indexesextension.pyx":1423
 * 
 *   # Optimized version for npy_longdouble/float96/float128
 *   def _search_bin_na_g(self, npy_longdouble item1, npy_longdouble item2):             # <<<<<<<<<<<<<<
 *     cdef int cs, ss, ncs, nrow, nrows, nrow2, nbounds, rvrow
 *     cdef int start, stop, tlength, length, bread, nchunk, nchunk2
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_29_search_bin_na_g(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_29_search_bin_na_g(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  npy_longdouble __pyx_v_item1;
  npy_longdouble __pyx_v_item2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_search_bin_na_g (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_item1,&__pyx_n_s_item2,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_item1)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_item2)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_search_bin_na_g", 1, 2, 2, 1); __PYX_ERR(0, 1423, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_search_bin_na_g") < 0)) __PYX_ERR(0, 1423, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_item1 = __pyx_PyFloat_AsDouble(values[0]); if (unlikely((__pyx_v_item1 == ((npy_longdouble)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1423, __pyx_L3_error)
    __pyx_v_item2 = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_item2 == ((npy_longdouble)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1423, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_search_bin_na_g", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1423, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("tables.indexesextension.IndexArray._search_bin_na_g", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_16indexesextension_10IndexArray_28_search_bin_na_g(((struct __pyx_obj_6tables_16indexesextension_IndexArray *)__pyx_v_self), __pyx_v_item1, __pyx_v_item2);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_28_search_bin_na_g(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, npy_longdouble __pyx_v_item1, npy_longdouble __pyx_v_item2) {
  int __pyx_v_cs;
  int __pyx_v_ss;
  int __pyx_v_ncs;
  int __pyx_v_nrow;
  int __pyx_v_nrows;
  int __pyx_v_nbounds;
  int __pyx_v_rvrow;
  int __pyx_v_start;
  int __pyx_v_stop;
  int __pyx_v_tlength;
  int __pyx_v_length;
  int __pyx_v_bread;
  int __pyx_v_nchunk;
  int __pyx_v_nchunk2;
  int *__pyx_v_rbufst;
  int *__pyx_v_rbufln;
  npy_longdouble *__pyx_v_rbufrv;
  npy_longdouble *__pyx_v_rbufbc;
  npy_longdouble *__pyx_v_rbuflb;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_search_bin_na_g", 0);

  /* "tables/indexesextension.pyx":1431
 *     # Variables with specific type
 *     cdef npy_longdouble *rbufrv
 *     cdef npy_longdouble *rbufbc = NULL             # <<<<<<<<<<<<<<
 *     cdef npy_longdouble *rbuflb = NULL
 * 
 */
  __pyx_v_rbufbc = NULL;

  /* "tables/indexesextension.pyx":1432
 *     cdef npy_longdouble *rbufrv
 *     cdef npy_longdouble *rbufbc = NULL
 *     cdef npy_longdouble *rbuflb = NULL             # <<<<<<<<<<<<<<
 * 
 *     cs = self.l_chunksize
 */
  __pyx_v_rbuflb = NULL;

  /* "tables/indexesextension.pyx":1434
 *     cdef npy_longdouble *rbuflb = NULL
 * 
 *     cs = self.l_chunksize             # <<<<<<<<<<<<<<
 *     ss = self.l_slicesize
 *     ncs = ss // cs
 */
  __pyx_t_1 = __pyx_v_self->l_chunksize;
  __pyx_v_cs = __pyx_t_1;

  /* "tables/indexesextension.pyx":1435
 * 
 *     cs = self.l_chunksize
 *     ss = self.l_slicesize             # <<<<<<<<<<<<<<
 *     ncs = ss // cs
 *     nbounds = self.nbounds
 */
  __pyx_t_1 = __pyx_v_self->l_slicesize;
  __pyx_v_ss = __pyx_t_1;

  /* "tables/indexesextension.pyx":1436
 *     cs = self.l_chunksize
 *     ss = self.l_slicesize
 *     ncs = ss // cs             # <<<<<<<<<<<<<<
 *     nbounds = self.nbounds
 *     nrows = self.nrows
 */
  if (unlikely(__pyx_v_cs == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
    __PYX_ERR(0, 1436, __pyx_L1_error)
  }
  else if (sizeof(int) == sizeof(long) && (!(((int)-1) > 0)) && unlikely(__pyx_v_cs == (int)-1)  && unlikely(UNARY_NEG_WOULD_OVERFLOW(__pyx_v_ss))) {
    PyErr_SetString(PyExc_OverflowError, "value too large to perform division");
    __PYX_ERR(0, 1436, __pyx_L1_error)
  }
  __pyx_v_ncs = __Pyx_div_int(__pyx_v_ss, __pyx_v_cs);

  /* "tables/indexesextension.pyx":1437
 *     ss = self.l_slicesize
 *     ncs = ss // cs
 *     nbounds = self.nbounds             # <<<<<<<<<<<<<<
 *     nrows = self.nrows
 *     tlength = 0
 */
  __pyx_t_1 = __pyx_v_self->nbounds;
  __pyx_v_nbounds = __pyx_t_1;

  /* "tables/indexesextension.pyx":1438
 *     ncs = ss // cs
 *     nbounds = self.nbounds
 *     nrows = self.nrows             # <<<<<<<<<<<<<<
 *     tlength = 0
 *     rbufst = <int *>self.rbufst
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nrows); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1438, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1438, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_nrows = __pyx_t_1;

  /* "tables/indexesextension.pyx":1439
 *     nbounds = self.nbounds
 *     nrows = self.nrows
 *     tlength = 0             # <<<<<<<<<<<<<<
 *     rbufst = <int *>self.rbufst
 *     rbufln = <int *>self.rbufln
 */
  __pyx_v_tlength = 0;

  /* "tables/indexesextension.pyx":1440
 *     nrows = self.nrows
 *     tlength = 0
 *     rbufst = <int *>self.rbufst             # <<<<<<<<<<<<<<
 *     rbufln = <int *>self.rbufln
 * 
 */
  __pyx_v_rbufst = ((int *)__pyx_v_self->rbufst);

  /* "tables/indexesextension.pyx":1441
 *     tlength = 0
 *     rbufst = <int *>self.rbufst
 *     rbufln = <int *>self.rbufln             # <<<<<<<<<<<<<<
 * 
 *     # Limits not in cache, do a lookup
 */
  __pyx_v_rbufln = ((int *)__pyx_v_self->rbufln);

  /* "tables/indexesextension.pyx":1444
 * 
 *     # Limits not in cache, do a lookup
 *     rbufrv = <npy_longdouble *>self.rbufrv             # <<<<<<<<<<<<<<
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2
 */
  __pyx_v_rbufrv = ((npy_longdouble *)__pyx_v_self->rbufrv);

  /* "tables/indexesextension.pyx":1445
 *     # Limits not in cache, do a lookup
 *     rbufrv = <npy_longdouble *>self.rbufrv
 *     for nrow from 0 <= nrow < nrows:             # <<<<<<<<<<<<<<
 *       rvrow = nrow*2
 *       bread = 0
 */
  __pyx_t_1 = __pyx_v_nrows;
  for (__pyx_v_nrow = 0; __pyx_v_nrow < __pyx_t_1; __pyx_v_nrow++) {

    /* "tables/indexesextension.pyx":1446
 *     rbufrv = <npy_longdouble *>self.rbufrv
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2             # <<<<<<<<<<<<<<
 *       bread = 0
 *       nchunk = -1
 */
    __pyx_v_rvrow = (__pyx_v_nrow * 2);

    /* "tables/indexesextension.pyx":1447
 *     for nrow from 0 <= nrow < nrows:
 *       rvrow = nrow*2
 *       bread = 0             # <<<<<<<<<<<<<<
 *       nchunk = -1
 * 
 */
    __pyx_v_bread = 0;

    /* "tables/indexesextension.pyx":1448
 *       rvrow = nrow*2
 *       bread = 0
 *       nchunk = -1             # <<<<<<<<<<<<<<
 * 
 *       # Look if item1 is in this row
 */
    __pyx_v_nchunk = -1;

    /* "tables/indexesextension.pyx":1451
 * 
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 */
    __pyx_t_3 = ((__pyx_v_item1 > (__pyx_v_rbufrv[__pyx_v_rvrow])) != 0);
    if (__pyx_t_3) {

      /* "tables/indexesextension.pyx":1452
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:
 *         if item1 <= rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_longdouble *>self.get_lru_bounds(nrow, nbounds)
 */
      __pyx_t_3 = ((__pyx_v_item1 <= (__pyx_v_rbufrv[(__pyx_v_rvrow + 1)])) != 0);
      if (__pyx_t_3) {

        /* "tables/indexesextension.pyx":1454
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_longdouble *>self.get_lru_bounds(nrow, nbounds)             # <<<<<<<<<<<<<<
 *           bread = 1
 *           nchunk = bisect_left_g(rbufbc, item1, nbounds, 0)
 */
        __pyx_v_rbufbc = ((npy_longdouble *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_bounds(__pyx_v_self, __pyx_v_nrow, __pyx_v_nbounds));

        /* "tables/indexesextension.pyx":1455
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_longdouble *>self.get_lru_bounds(nrow, nbounds)
 *           bread = 1             # <<<<<<<<<<<<<<
 *           nchunk = bisect_left_g(rbufbc, item1, nbounds, 0)
 *           # Get the sorted row from the LRU cache or read it.
 */
        __pyx_v_bread = 1;

        /* "tables/indexesextension.pyx":1456
 *           rbufbc = <npy_longdouble *>self.get_lru_bounds(nrow, nbounds)
 *           bread = 1
 *           nchunk = bisect_left_g(rbufbc, item1, nbounds, 0)             # <<<<<<<<<<<<<<
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_longdouble *>self.get_lru_sorted(nrow, ncs, nchunk, cs)
 */
        __pyx_v_nchunk = bisect_left_g(__pyx_v_rbufbc, __pyx_v_item1, __pyx_v_nbounds, 0);

        /* "tables/indexesextension.pyx":1458
 *           nchunk = bisect_left_g(rbufbc, item1, nbounds, 0)
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_longdouble *>self.get_lru_sorted(nrow, ncs, nchunk, cs)             # <<<<<<<<<<<<<<
 *           start = bisect_left_g(rbuflb, item1, cs, 0) + cs*nchunk
 *         else:
 */
        __pyx_v_rbuflb = ((npy_longdouble *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_sorted(__pyx_v_self, __pyx_v_nrow, __pyx_v_ncs, __pyx_v_nchunk, __pyx_v_cs));

        /* "tables/indexesextension.pyx":1459
 *           # Get the sorted row from the LRU cache or read it.
 *           rbuflb = <npy_longdouble *>self.get_lru_sorted(nrow, ncs, nchunk, cs)
 *           start = bisect_left_g(rbuflb, item1, cs, 0) + cs*nchunk             # <<<<<<<<<<<<<<
 *         else:
 *           start = ss
 */
        __pyx_v_start = (bisect_left_g(__pyx_v_rbuflb, __pyx_v_item1, __pyx_v_cs, 0) + (__pyx_v_cs * __pyx_v_nchunk));

        /* "tables/indexesextension.pyx":1452
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:
 *         if item1 <= rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           # Get the bounds row from the LRU cache or read them.
 *           rbufbc = <npy_longdouble *>self.get_lru_bounds(nrow, nbounds)
 */
        goto __pyx_L6;
      }

      /* "tables/indexesextension.pyx":1461
 *           start = bisect_left_g(rbuflb, item1, cs, 0) + cs*nchunk
 *         else:
 *           start = ss             # <<<<<<<<<<<<<<
 *       else:
 *         start = 0
 */
      /*else*/ {
        __pyx_v_start = __pyx_v_ss;
      }
      __pyx_L6:;

      /* "tables/indexesextension.pyx":1451
 * 
 *       # Look if item1 is in this row
 *       if item1 > rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item1 <= rbufrv[rvrow+1]:
 *           # Get the bounds row from the LRU cache or read them.
 */
      goto __pyx_L5;
    }

    /* "tables/indexesextension.pyx":1463
 *           start = ss
 *       else:
 *         start = 0             # <<<<<<<<<<<<<<
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 */
    /*else*/ {
      __pyx_v_start = 0;
    }
    __pyx_L5:;

    /* "tables/indexesextension.pyx":1465
 *         start = 0
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:
 */
    __pyx_t_3 = ((__pyx_v_item2 >= (__pyx_v_rbufrv[__pyx_v_rvrow])) != 0);
    if (__pyx_t_3) {

      /* "tables/indexesextension.pyx":1466
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 */
      __pyx_t_3 = ((__pyx_v_item2 < (__pyx_v_rbufrv[(__pyx_v_rvrow + 1)])) != 0);
      if (__pyx_t_3) {

        /* "tables/indexesextension.pyx":1467
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:             # <<<<<<<<<<<<<<
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_longdouble *>self.get_lru_bounds(nrow, nbounds)
 */
        __pyx_t_3 = ((!(__pyx_v_bread != 0)) != 0);
        if (__pyx_t_3) {

          /* "tables/indexesextension.pyx":1469
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_longdouble *>self.get_lru_bounds(nrow, nbounds)             # <<<<<<<<<<<<<<
 *           nchunk2 = bisect_right_g(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:
 */
          __pyx_v_rbufbc = ((npy_longdouble *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_bounds(__pyx_v_self, __pyx_v_nrow, __pyx_v_nbounds));

          /* "tables/indexesextension.pyx":1467
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:             # <<<<<<<<<<<<<<
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_longdouble *>self.get_lru_bounds(nrow, nbounds)
 */
        }

        /* "tables/indexesextension.pyx":1470
 *             # Get the bounds row from the LRU cache or read them.
 *             rbufbc = <npy_longdouble *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_g(rbufbc, item2, nbounds, 0)             # <<<<<<<<<<<<<<
 *           if nchunk2 <> nchunk:
 *             # Get the sorted row from the LRU cache or read it.
 */
        __pyx_v_nchunk2 = bisect_right_g(__pyx_v_rbufbc, __pyx_v_item2, __pyx_v_nbounds, 0);

        /* "tables/indexesextension.pyx":1471
 *             rbufbc = <npy_longdouble *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_g(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:             # <<<<<<<<<<<<<<
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_longdouble *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 */
        __pyx_t_3 = ((__pyx_v_nchunk2 != __pyx_v_nchunk) != 0);
        if (__pyx_t_3) {

          /* "tables/indexesextension.pyx":1473
 *           if nchunk2 <> nchunk:
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_longdouble *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)             # <<<<<<<<<<<<<<
 *           stop = bisect_right_g(rbuflb, item2, cs, 0) + cs*nchunk2
 *         else:
 */
          __pyx_v_rbuflb = ((npy_longdouble *)((struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->get_lru_sorted(__pyx_v_self, __pyx_v_nrow, __pyx_v_ncs, __pyx_v_nchunk2, __pyx_v_cs));

          /* "tables/indexesextension.pyx":1471
 *             rbufbc = <npy_longdouble *>self.get_lru_bounds(nrow, nbounds)
 *           nchunk2 = bisect_right_g(rbufbc, item2, nbounds, 0)
 *           if nchunk2 <> nchunk:             # <<<<<<<<<<<<<<
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_longdouble *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 */
        }

        /* "tables/indexesextension.pyx":1474
 *             # Get the sorted row from the LRU cache or read it.
 *             rbuflb = <npy_longdouble *>self.get_lru_sorted(nrow, ncs, nchunk2, cs)
 *           stop = bisect_right_g(rbuflb, item2, cs, 0) + cs*nchunk2             # <<<<<<<<<<<<<<
 *         else:
 *           stop = ss
 */
        __pyx_v_stop = (bisect_right_g(__pyx_v_rbuflb, __pyx_v_item2, __pyx_v_cs, 0) + (__pyx_v_cs * __pyx_v_nchunk2));

        /* "tables/indexesextension.pyx":1466
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:
 *         if item2 < rbufrv[rvrow+1]:             # <<<<<<<<<<<<<<
 *           if not bread:
 *             # Get the bounds row from the LRU cache or read them.
 */
        goto __pyx_L8;
      }

      /* "tables/indexesextension.pyx":1476
 *           stop = bisect_right_g(rbuflb, item2, cs, 0) + cs*nchunk2
 *         else:
 *           stop = ss             # <<<<<<<<<<<<<<
 *       else:
 *         stop = 0
 */
      /*else*/ {
        __pyx_v_stop = __pyx_v_ss;
      }
      __pyx_L8:;

      /* "tables/indexesextension.pyx":1465
 *         start = 0
 *       # Now, for item2
 *       if item2 >= rbufrv[rvrow]:             # <<<<<<<<<<<<<<
 *         if item2 < rbufrv[rvrow+1]:
 *           if not bread:
 */
      goto __pyx_L7;
    }

    /* "tables/indexesextension.pyx":1478
 *           stop = ss
 *       else:
 *         stop = 0             # <<<<<<<<<<<<<<
 *       length = stop - start
 *       tlength = tlength + length
 */
    /*else*/ {
      __pyx_v_stop = 0;
    }
    __pyx_L7:;

    /* "tables/indexesextension.pyx":1479
 *       else:
 *         stop = 0
 *       length = stop - start             # <<<<<<<<<<<<<<
 *       tlength = tlength + length
 *       rbufst[nrow] = start
 */
    __pyx_v_length = (__pyx_v_stop - __pyx_v_start);

    /* "tables/indexesextension.pyx":1480
 *         stop = 0
 *       length = stop - start
 *       tlength = tlength + length             # <<<<<<<<<<<<<<
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length
 */
    __pyx_v_tlength = (__pyx_v_tlength + __pyx_v_length);

    /* "tables/indexesextension.pyx":1481
 *       length = stop - start
 *       tlength = tlength + length
 *       rbufst[nrow] = start             # <<<<<<<<<<<<<<
 *       rbufln[nrow] = length
 *     return tlength
 */
    (__pyx_v_rbufst[__pyx_v_nrow]) = __pyx_v_start;

    /* "tables/indexesextension.pyx":1482
 *       tlength = tlength + length
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length             # <<<<<<<<<<<<<<
 *     return tlength
 * 
 */
    (__pyx_v_rbufln[__pyx_v_nrow]) = __pyx_v_length;
  }

  /* "tables/indexesextension.pyx":1483
 *       rbufst[nrow] = start
 *       rbufln[nrow] = length
 *     return tlength             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_tlength); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1483, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "tables/indexesextension.pyx":1423
 * 
 *   # Optimized version for npy_longdouble/float96/float128
 *   def _search_bin_na_g(self, npy_longdouble item1, npy_longdouble item2):             # <<<<<<<<<<<<<<
 *     cdef int cs, ss, ncs, nrow, nrows, nrow2, nbounds, rvrow
 *     cdef int start, stop, tlength, length, bread, nchunk, nchunk2
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("tables.indexesextension.IndexArray._search_bin_na_g", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/indexesextension.pyx":1486
 * 
 * 
 *   def _g_close(self):             # <<<<<<<<<<<<<<
 *     super()._g_close()
 *     # Release specific resources of this class
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_31_g_close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_31_g_close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_close (wrapper)", 0);
  __pyx_r = __pyx_pf_6tables_16indexesextension_10IndexArray_30_g_close(((struct __pyx_obj_6tables_16indexesextension_IndexArray *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_30_g_close(struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_g_close", 0);

  /* "tables/indexesextension.pyx":1487
 * 
 *   def _g_close(self):
 *     super()._g_close()             # <<<<<<<<<<<<<<
 *     # Release specific resources of this class
 *     if self.mem_space_id > 0:
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1487, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6tables_16indexesextension_IndexArray));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6tables_16indexesextension_IndexArray));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6tables_16indexesextension_IndexArray));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1487, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_g_close); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1487, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1487, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "tables/indexesextension.pyx":1489
 *     super()._g_close()
 *     # Release specific resources of this class
 *     if self.mem_space_id > 0:             # <<<<<<<<<<<<<<
 *       H5Sclose(self.mem_space_id)
 * 
 */
  __pyx_t_4 = ((__pyx_v_self->mem_space_id > 0) != 0);
  if (__pyx_t_4) {

    /* "tables/indexesextension.pyx":1490
 *     # Release specific resources of this class
 *     if self.mem_space_id > 0:
 *       H5Sclose(self.mem_space_id)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    (void)(H5Sclose(__pyx_v_self->mem_space_id));

    /* "tables/indexesextension.pyx":1489
 *     super()._g_close()
 *     # Release specific resources of this class
 *     if self.mem_space_id > 0:             # <<<<<<<<<<<<<<
 *       H5Sclose(self.mem_space_id)
 * 
 */
  }

  /* "tables/indexesextension.pyx":1486
 * 
 * 
 *   def _g_close(self):             # <<<<<<<<<<<<<<
 *     super()._g_close()
 *     # Release specific resources of this class
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("tables.indexesextension.IndexArray._g_close", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims,self.rbufbc,self.rbuflb,self.rbufln,self.rbufrv,self.rbufst cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_33__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_33__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6tables_16indexesextension_10IndexArray_32__reduce_cython__(((struct __pyx_obj_6tables_16indexesextension_IndexArray *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_32__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims,self.rbufbc,self.rbuflb,self.rbufln,self.rbufrv,self.rbufst cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims,self.rbufbc,self.rbuflb,self.rbufln,self.rbufrv,self.rbufst cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims,self.rbufbc,self.rbuflb,self.rbufln,self.rbufrv,self.rbufst cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("tables.indexesextension.IndexArray.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims,self.rbufbc,self.rbuflb,self.rbufln,self.rbufrv,self.rbufst cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims,self.rbufbc,self.rbuflb,self.rbufln,self.rbufrv,self.rbufst cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_35__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_6tables_16indexesextension_10IndexArray_35__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6tables_16indexesextension_10IndexArray_34__setstate_cython__(((struct __pyx_obj_6tables_16indexesextension_IndexArray *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_16indexesextension_10IndexArray_34__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims,self.rbufbc,self.rbuflb,self.rbufln,self.rbufrv,self.rbufst cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims,self.rbufbc,self.rbuflb,self.rbufln,self.rbufrv,self.rbufst cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims,self.rbufbc,self.rbuflb,self.rbufln,self.rbufrv,self.rbufst cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims,self.rbufbc,self.rbuflb,self.rbufln,self.rbufrv,self.rbufst cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("tables.indexesextension.IndexArray.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/indexesextension.pyx":1498
 *   """
 * 
 *   def _read_index_slice(self, hsize_t start, hsize_t stop, ndarray idx):             # <<<<<<<<<<<<<<
 *     """Read the reverse index part of an LR index."""
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_16indexesextension_12LastRowArray_1_read_index_slice(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6tables_16indexesextension_12LastRowArray__read_index_slice[] = "Read the reverse index part of an LR index.";
static PyObject *__pyx_pw_6tables_16indexesextension_12LastRowArray_1_read_index_slice(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  hsize_t __pyx_v_start;
  hsize_t __pyx_v_stop;
  PyArrayObject *__pyx_v_idx = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_read_index_slice (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_start,&__pyx_n_s_stop,&__pyx_n_s_idx,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_start)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_stop)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_read_index_slice", 1, 3, 3, 1); __PYX_ERR(0, 1498, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_idx)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_read_index_slice", 1, 3, 3, 2); __PYX_ERR(0, 1498, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_read_index_slice") < 0)) __PYX_ERR(0, 1498, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_start = __Pyx_PyInt_As_hsize_t(values[0]); if (unlikely((__pyx_v_start == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1498, __pyx_L3_error)
    __pyx_v_stop = __Pyx_PyInt_As_hsize_t(values[1]); if (unlikely((__pyx_v_stop == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1498, __pyx_L3_error)
    __pyx_v_idx = ((PyArrayObject *)values[2]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_read_index_slice", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1498, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("tables.indexesextension.LastRowArray._read_index_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_idx), __pyx_ptype_5numpy_ndarray, 1, "idx", 0))) __PYX_ERR(0, 1498, __pyx_L1_error)
  __pyx_r = __pyx_pf_6tables_16indexesextension_12LastRowArray__read_index_slice(((struct __pyx_obj_6tables_16indexesextension_LastRowArray *)__pyx_v_self), __pyx_v_start, __pyx_v_stop, __pyx_v_idx);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_16indexesextension_12LastRowArray__read_index_slice(struct __pyx_obj_6tables_16indexesextension_LastRowArray *__pyx_v_self, hsize_t __pyx_v_start, hsize_t __pyx_v_stop, PyArrayObject *__pyx_v_idx) {
  void *__pyx_v_buf;
  herr_t __pyx_v_ret;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_read_index_slice", 0);

  /* "tables/indexesextension.pyx":1501
 *     """Read the reverse index part of an LR index."""
 * 
 *     cdef void *buf = PyArray_DATA(idx)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         ret = H5ARRAYOreadSliceLR(self.dataset_id, self.type_id,
 */
  __pyx_v_buf = PyArray_DATA(__pyx_v_idx);

  /* "tables/indexesextension.pyx":1502
 * 
 *     cdef void *buf = PyArray_DATA(idx)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5ARRAYOreadSliceLR(self.dataset_id, self.type_id,
 *                                   start, stop, buf)
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "tables/indexesextension.pyx":1503
 *     cdef void *buf = PyArray_DATA(idx)
 *     with nogil:
 *         ret = H5ARRAYOreadSliceLR(self.dataset_id, self.type_id,             # <<<<<<<<<<<<<<
 *                                   start, stop, buf)
 * 
 */
        __pyx_v_ret = H5ARRAYOreadSliceLR(__pyx_v_self->__pyx_base.__pyx_base.dataset_id, __pyx_v_self->__pyx_base.__pyx_base.type_id, __pyx_v_start, __pyx_v_stop, __pyx_v_buf);
      }

      /* "tables/indexesextension.pyx":1502
 * 
 *     cdef void *buf = PyArray_DATA(idx)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5ARRAYOreadSliceLR(self.dataset_id, self.type_id,
 *                                   start, stop, buf)
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "tables/indexesextension.pyx":1506
 *                                   start, stop, buf)
 * 
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems reading the index data in Last Row.")
 * 
 */
  __pyx_t_1 = ((__pyx_v_ret < 0) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "tables/indexesextension.pyx":1507
 * 
 *     if ret < 0:
 *       raise HDF5ExtError("Problems reading the index data in Last Row.")             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_HDF5ExtError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1507, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_kp_s_Problems_reading_the_index_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_kp_s_Problems_reading_the_index_data);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1507, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1507, __pyx_L1_error)

    /* "tables/indexesextension.pyx":1506
 *                                   start, stop, buf)
 * 
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems reading the index data in Last Row.")
 * 
 */
  }

  /* "tables/indexesextension.pyx":1498
 *   """
 * 
 *   def _read_index_slice(self, hsize_t start, hsize_t stop, ndarray idx):             # <<<<<<<<<<<<<<
 *     """Read the reverse index part of an LR index."""
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("tables.indexesextension.LastRowArray._read_index_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/indexesextension.pyx":1510
 * 
 * 
 *   def _read_sorted_slice(self, IndexArray sorted, hsize_t start, hsize_t stop):             # <<<<<<<<<<<<<<
 *     """Read the sorted part of an LR index."""
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_16indexesextension_12LastRowArray_3_read_sorted_slice(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6tables_16indexesextension_12LastRowArray_2_read_sorted_slice[] = "Read the sorted part of an LR index.";
static PyObject *__pyx_pw_6tables_16indexesextension_12LastRowArray_3_read_sorted_slice(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_sorted = 0;
  hsize_t __pyx_v_start;
  hsize_t __pyx_v_stop;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_read_sorted_slice (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_sorted,&__pyx_n_s_start,&__pyx_n_s_stop,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_sorted)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_start)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_read_sorted_slice", 1, 3, 3, 1); __PYX_ERR(0, 1510, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_stop)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_read_sorted_slice", 1, 3, 3, 2); __PYX_ERR(0, 1510, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_read_sorted_slice") < 0)) __PYX_ERR(0, 1510, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_sorted = ((struct __pyx_obj_6tables_16indexesextension_IndexArray *)values[0]);
    __pyx_v_start = __Pyx_PyInt_As_hsize_t(values[1]); if (unlikely((__pyx_v_start == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1510, __pyx_L3_error)
    __pyx_v_stop = __Pyx_PyInt_As_hsize_t(values[2]); if (unlikely((__pyx_v_stop == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1510, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_read_sorted_slice", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1510, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("tables.indexesextension.LastRowArray._read_sorted_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sorted), __pyx_ptype_6tables_16indexesextension_IndexArray, 1, "sorted", 0))) __PYX_ERR(0, 1510, __pyx_L1_error)
  __pyx_r = __pyx_pf_6tables_16indexesextension_12LastRowArray_2_read_sorted_slice(((struct __pyx_obj_6tables_16indexesextension_LastRowArray *)__pyx_v_self), __pyx_v_sorted, __pyx_v_start, __pyx_v_stop);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_16indexesextension_12LastRowArray_2_read_sorted_slice(struct __pyx_obj_6tables_16indexesextension_LastRowArray *__pyx_v_self, struct __pyx_obj_6tables_16indexesextension_IndexArray *__pyx_v_sorted, hsize_t __pyx_v_start, hsize_t __pyx_v_stop) {
  void *__pyx_v_rbuflb;
  herr_t __pyx_v_ret;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  void *__pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_read_sorted_slice", 0);

  /* "tables/indexesextension.pyx":1515
 *     cdef void  *rbuflb
 * 
 *     rbuflb = sorted.rbuflb  # direct access to rbuflb: very fast.             # <<<<<<<<<<<<<<
 *     with nogil:
 *         ret = H5ARRAYOreadSliceLR(self.dataset_id, self.type_id,
 */
  __pyx_t_1 = __pyx_v_sorted->rbuflb;
  __pyx_v_rbuflb = __pyx_t_1;

  /* "tables/indexesextension.pyx":1516
 * 
 *     rbuflb = sorted.rbuflb  # direct access to rbuflb: very fast.
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5ARRAYOreadSliceLR(self.dataset_id, self.type_id,
 *                                   start, stop, rbuflb)
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "tables/indexesextension.pyx":1517
 *     rbuflb = sorted.rbuflb  # direct access to rbuflb: very fast.
 *     with nogil:
 *         ret = H5ARRAYOreadSliceLR(self.dataset_id, self.type_id,             # <<<<<<<<<<<<<<
 *                                   start, stop, rbuflb)
 * 
 */
        __pyx_v_ret = H5ARRAYOreadSliceLR(__pyx_v_self->__pyx_base.__pyx_base.dataset_id, __pyx_v_self->__pyx_base.__pyx_base.type_id, __pyx_v_start, __pyx_v_stop, __pyx_v_rbuflb);
      }

      /* "tables/indexesextension.pyx":1516
 * 
 *     rbuflb = sorted.rbuflb  # direct access to rbuflb: very fast.
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5ARRAYOreadSliceLR(self.dataset_id, self.type_id,
 *                                   start, stop, rbuflb)
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "tables/indexesextension.pyx":1520
 *                                   start, stop, rbuflb)
 * 
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems reading the index data.")
 *     return sorted.bufferlb[:stop-start]
 */
  __pyx_t_2 = ((__pyx_v_ret < 0) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "tables/indexesextension.pyx":1521
 * 
 *     if ret < 0:
 *       raise HDF5ExtError("Problems reading the index data.")             # <<<<<<<<<<<<<<
 *     return sorted.bufferlb[:stop-start]
 * 
 */
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_HDF5ExtError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1521, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_kp_s_Problems_reading_the_index_data_2) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_kp_s_Problems_reading_the_index_data_2);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1521, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 1521, __pyx_L1_error)

    /* "tables/indexesextension.pyx":1520
 *                                   start, stop, rbuflb)
 * 
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems reading the index data.")
 *     return sorted.bufferlb[:stop-start]
 */
  }

  /* "tables/indexesextension.pyx":1522
 *     if ret < 0:
 *       raise HDF5ExtError("Problems reading the index data.")
 *     return sorted.bufferlb[:stop-start]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = __Pyx_PyObject_GetSlice(((PyObject *)__pyx_v_sorted->bufferlb), 0, (__pyx_v_stop - __pyx_v_start), NULL, NULL, NULL, 0, 1, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1522, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "tables/indexesextension.pyx":1510
 * 
 * 
 *   def _read_sorted_slice(self, IndexArray sorted, hsize_t start, hsize_t stop):             # <<<<<<<<<<<<<<
 *     """Read the sorted part of an LR index."""
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("tables.indexesextension.LastRowArray._read_sorted_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_16indexesextension_12LastRowArray_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_6tables_16indexesextension_12LastRowArray_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6tables_16indexesextension_12LastRowArray_4__reduce_cython__(((struct __pyx_obj_6tables_16indexesextension_LastRowArray *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_16indexesextension_12LastRowArray_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_16indexesextension_LastRowArray *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("tables.indexesextension.LastRowArray.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_16indexesextension_12LastRowArray_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_6tables_16indexesextension_12LastRowArray_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6tables_16indexesextension_12LastRowArray_6__setstate_cython__(((struct __pyx_obj_6tables_16indexesextension_LastRowArray *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_16indexesextension_12LastRowArray_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_16indexesextension_LastRowArray *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("tables.indexesextension.LastRowArray.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_Index(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_16indexesextension_7__pyx_unpickle_Index(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_6tables_16indexesextension_7__pyx_unpickle_Index = {"__pyx_unpickle_Index", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6tables_16indexesextension_7__pyx_unpickle_Index, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_16indexesextension_7__pyx_unpickle_Index(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_Index (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Index", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Index", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_Index") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Index", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("tables.indexesextension.__pyx_unpickle_Index", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_16indexesextension_6__pyx_unpickle_Index(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_16indexesextension_6__pyx_unpickle_Index(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_Index", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xd41d8cd:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xd41d8cd) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xd41d8cd:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)
 *     __pyx_result = Index.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0xd41d8cd:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = Index.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xd4, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xd41d8cd:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)
 *     __pyx_result = Index.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Index__set_state(<Index> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6tables_16indexesextension_Index), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)
 *     __pyx_result = Index.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Index__set_state(<Index> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = Index.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Index__set_state(<Index> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_Index__set_state(Index __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6tables_16indexesextension___pyx_unpickle_Index__set_state(((struct __pyx_obj_6tables_16indexesextension_Index *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)
 *     __pyx_result = Index.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Index__set_state(<Index> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Index__set_state(<Index> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_Index__set_state(Index __pyx_result, tuple __pyx_state):
 *     if len(__pyx_state) > 0 and hasattr(__pyx_result, '__dict__'):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_Index(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("tables.indexesextension.__pyx_unpickle_Index", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_Index__set_state(<Index> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Index__set_state(Index __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 0 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[0])
 */

static PyObject *__pyx_f_6tables_16indexesextension___pyx_unpickle_Index__set_state(struct __pyx_obj_6tables_16indexesextension_Index *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_Index__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_Index__set_state(Index __pyx_result, tuple __pyx_state):
 *     if len(__pyx_state) > 0 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[0])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_2 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(1, 12, __pyx_L1_error)
  __pyx_t_3 = ((__pyx_t_2 > 0) != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 12, __pyx_L1_error)
  __pyx_t_4 = (__pyx_t_3 != 0);
  __pyx_t_1 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_Index__set_state(Index __pyx_result, tuple __pyx_state):
 *     if len(__pyx_state) > 0 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[0])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 13, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_5 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_Index__set_state(Index __pyx_result, tuple __pyx_state):
 *     if len(__pyx_state) > 0 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[0])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_Index__set_state(<Index> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Index__set_state(Index __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 0 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[0])
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("tables.indexesextension.__pyx_unpickle_Index__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":735
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew1(PyObject *__pyx_v_a) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew1", 0);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":736
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 *     return PyArray_MultiIterNew(1, <void*>a)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(1, ((void *)__pyx_v_a)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 736, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":735
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":738
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew2(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew2", 0);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":739
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(2, ((void *)__pyx_v_a), ((void *)__pyx_v_b)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 739, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":738
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew2", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":741
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew3(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew3", 0);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":742
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(3, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 742, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":741
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew3", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":744
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew4(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew4", 0);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":745
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(4, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 745, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":744
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew4", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":747
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew5(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d, PyObject *__pyx_v_e) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew5", 0);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":748
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)             # <<<<<<<<<<<<<<
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(5, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d), ((void *)__pyx_v_e)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 748, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":747
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew5", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":750
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyDataType_SHAPE(PyArray_Descr *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("PyDataType_SHAPE", 0);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":751
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */
  __pyx_t_1 = (PyDataType_HASSUBARRAY(__pyx_v_d) != 0);
  if (__pyx_t_1) {

    /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":752
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape             # <<<<<<<<<<<<<<
 *     else:
 *         return ()
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject*)__pyx_v_d->subarray->shape));
    __pyx_r = ((PyObject*)__pyx_v_d->subarray->shape);
    goto __pyx_L0;

    /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":751
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */
  }

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":754
 *         return <tuple>d.subarray.shape
 *     else:
 *         return ()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_empty_tuple);
    __pyx_r = __pyx_empty_tuple;
    goto __pyx_L0;
  }

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":750
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":929
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

static CYTHON_INLINE void __pyx_f_5numpy_set_array_base(PyArrayObject *__pyx_v_arr, PyObject *__pyx_v_base) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_array_base", 0);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":930
 * 
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!             # <<<<<<<<<<<<<<
 *     PyArray_SetBaseObject(arr, base)
 * 
 */
  Py_INCREF(__pyx_v_base);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":931
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object get_array_base(ndarray arr):
 */
  (void)(PyArray_SetBaseObject(__pyx_v_arr, __pyx_v_base));

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":929
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":933
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_get_array_base(PyArrayObject *__pyx_v_arr) {
  PyObject *__pyx_v_base;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("get_array_base", 0);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":934
 * 
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)             # <<<<<<<<<<<<<<
 *     if base is NULL:
 *         return None
 */
  __pyx_v_base = PyArray_BASE(__pyx_v_arr);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":935
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */
  __pyx_t_1 = ((__pyx_v_base == NULL) != 0);
  if (__pyx_t_1) {

    /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":936
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 *         return None             # <<<<<<<<<<<<<<
 *     return <object>base
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":935
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */
  }

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":937
 *     if base is NULL:
 *         return None
 *     return <object>base             # <<<<<<<<<<<<<<
 * 
 * # Versions of the import_* functions which are more suitable for
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_base));
  __pyx_r = ((PyObject *)__pyx_v_base);
  goto __pyx_L0;

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":933
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":941
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         __pyx_import_array()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_array(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_array", 0);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":942
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":943
 * cdef inline int import_array() except -1:
 *     try:
 *         __pyx_import_array()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")
 */
      __pyx_t_4 = _import_array(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 943, __pyx_L3_error)

      /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":942
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":944
 *     try:
 *         __pyx_import_array()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(2, 944, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":945
 *         __pyx_import_array()
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__9, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 945, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(2, 945, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":942
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":941
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         __pyx_import_array()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":947
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_umath(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_umath", 0);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":948
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":949
 * cdef inline int import_umath() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")
 */
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 949, __pyx_L3_error)

      /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":948
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":950
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(2, 950, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":951
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__10, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 951, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(2, 951, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":948
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":947
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":953
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_ufunc(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_ufunc", 0);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":954
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":955
 * cdef inline int import_ufunc() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")
 */
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 955, __pyx_L3_error)

      /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":954
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":956
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(2, 956, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":957
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef extern from *:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__10, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 957, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(2, 957, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":954
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":953
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":967
 * 
 * 
 * cdef inline bint is_timedelta64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.timedelta64)`
 */

static CYTHON_INLINE int __pyx_f_5numpy_is_timedelta64_object(PyObject *__pyx_v_obj) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_timedelta64_object", 0);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":979
 *     bool
 *     """
 *     return PyObject_TypeCheck(obj, &PyTimedeltaArrType_Type)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = PyObject_TypeCheck(__pyx_v_obj, (&PyTimedeltaArrType_Type));
  goto __pyx_L0;

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":967
 * 
 * 
 * cdef inline bint is_timedelta64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.timedelta64)`
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":982
 * 
 * 
 * cdef inline bint is_datetime64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.datetime64)`
 */

static CYTHON_INLINE int __pyx_f_5numpy_is_datetime64_object(PyObject *__pyx_v_obj) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_datetime64_object", 0);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":994
 *     bool
 *     """
 *     return PyObject_TypeCheck(obj, &PyDatetimeArrType_Type)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = PyObject_TypeCheck(__pyx_v_obj, (&PyDatetimeArrType_Type));
  goto __pyx_L0;

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":982
 * 
 * 
 * cdef inline bint is_datetime64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.datetime64)`
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":997
 * 
 * 
 * cdef inline npy_datetime get_datetime64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy datetime64 object
 */

static CYTHON_INLINE npy_datetime __pyx_f_5numpy_get_datetime64_value(PyObject *__pyx_v_obj) {
  npy_datetime __pyx_r;

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":1004
 *     also needed.  That can be found using `get_datetime64_unit`.
 *     """
 *     return (<PyDatetimeScalarObject*>obj).obval             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((PyDatetimeScalarObject *)__pyx_v_obj)->obval;
  goto __pyx_L0;

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":997
 * 
 * 
 * cdef inline npy_datetime get_datetime64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy datetime64 object
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":1007
 * 
 * 
 * cdef inline npy_timedelta get_timedelta64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy timedelta64 object
 */

static CYTHON_INLINE npy_timedelta __pyx_f_5numpy_get_timedelta64_value(PyObject *__pyx_v_obj) {
  npy_timedelta __pyx_r;

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":1011
 *     returns the int64 value underlying scalar numpy timedelta64 object
 *     """
 *     return (<PyTimedeltaScalarObject*>obj).obval             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((PyTimedeltaScalarObject *)__pyx_v_obj)->obval;
  goto __pyx_L0;

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":1007
 * 
 * 
 * cdef inline npy_timedelta get_timedelta64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy timedelta64 object
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":1014
 * 
 * 
 * cdef inline NPY_DATETIMEUNIT get_datetime64_unit(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the unit part of the dtype for a numpy datetime64 object.
 */

static CYTHON_INLINE NPY_DATETIMEUNIT __pyx_f_5numpy_get_datetime64_unit(PyObject *__pyx_v_obj) {
  NPY_DATETIMEUNIT __pyx_r;

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":1018
 *     returns the unit part of the dtype for a numpy datetime64 object.
 *     """
 *     return <NPY_DATETIMEUNIT>(<PyDatetimeScalarObject*>obj).obmeta.base             # <<<<<<<<<<<<<<
 */
  __pyx_r = ((NPY_DATETIMEUNIT)((PyDatetimeScalarObject *)__pyx_v_obj)->obmeta.base);
  goto __pyx_L0;

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":1014
 * 
 * 
 * cdef inline NPY_DATETIMEUNIT get_datetime64_unit(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the unit part of the dtype for a numpy datetime64 object.
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static PyObject *__pyx_tp_new_6tables_16indexesextension_Index(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  return o;
}

static void __pyx_tp_dealloc_6tables_16indexesextension_Index(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  (*Py_TYPE(o)->tp_free)(o);
}

static PyMethodDef __pyx_methods_6tables_16indexesextension_Index[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6tables_16indexesextension_5Index_1__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6tables_16indexesextension_5Index_3__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6tables_16indexesextension_Index = {
  PyVarObject_HEAD_INIT(0, 0)
  "tables.indexesextension.Index", /*tp_name*/
  sizeof(struct __pyx_obj_6tables_16indexesextension_Index), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6tables_16indexesextension_Index, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6tables_16indexesextension_Index, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6tables_16indexesextension_Index, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM+0 >= 0x06000000
  0, /*tp_pypy_flags*/
  #endif
};
static struct __pyx_vtabstruct_6tables_16indexesextension_CacheArray __pyx_vtable_6tables_16indexesextension_CacheArray;

static PyObject *__pyx_tp_new_6tables_16indexesextension_CacheArray(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6tables_16indexesextension_CacheArray *p;
  PyObject *o = __pyx_ptype_6tables_13hdf5extension_Array->tp_new(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6tables_16indexesextension_CacheArray *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6tables_13hdf5extension_Leaf*)__pyx_vtabptr_6tables_16indexesextension_CacheArray;
  return o;
}

static void __pyx_tp_dealloc_6tables_16indexesextension_CacheArray(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  PyObject_GC_Track(o);
  if (likely(__pyx_ptype_6tables_13hdf5extension_Array)) __pyx_ptype_6tables_13hdf5extension_Array->tp_dealloc(o); else __Pyx_call_next_tp_dealloc(o, __pyx_tp_dealloc_6tables_16indexesextension_CacheArray);
}

static int __pyx_tp_traverse_6tables_16indexesextension_CacheArray(PyObject *o, visitproc v, void *a) {
  int e;
  e = ((likely(__pyx_ptype_6tables_13hdf5extension_Array)) ? ((__pyx_ptype_6tables_13hdf5extension_Array->tp_traverse) ? __pyx_ptype_6tables_13hdf5extension_Array->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_6tables_16indexesextension_CacheArray)); if (e) return e;
  return 0;
}

static int __pyx_tp_clear_6tables_16indexesextension_CacheArray(PyObject *o) {
  if (likely(__pyx_ptype_6tables_13hdf5extension_Array)) { if (__pyx_ptype_6tables_13hdf5extension_Array->tp_clear) __pyx_ptype_6tables_13hdf5extension_Array->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_6tables_16indexesextension_CacheArray);
  return 0;
}

static PyMethodDef __pyx_methods_6tables_16indexesextension_CacheArray[] = {
  {"_g_close", (PyCFunction)__pyx_pw_6tables_16indexesextension_10CacheArray_1_g_close, METH_NOARGS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6tables_16indexesextension_10CacheArray_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6tables_16indexesextension_10CacheArray_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6tables_16indexesextension_CacheArray = {
  PyVarObject_HEAD_INIT(0, 0)
  "tables.indexesextension.CacheArray", /*tp_name*/
  sizeof(struct __pyx_obj_6tables_16indexesextension_CacheArray), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6tables_16indexesextension_CacheArray, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "Container for keeping index caches of 1st and 2nd level.", /*tp_doc*/
  __pyx_tp_traverse_6tables_16indexesextension_CacheArray, /*tp_traverse*/
  __pyx_tp_clear_6tables_16indexesextension_CacheArray, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6tables_16indexesextension_CacheArray, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6tables_16indexesextension_CacheArray, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM+0 >= 0x06000000
  0, /*tp_pypy_flags*/
  #endif
};
static struct __pyx_vtabstruct_6tables_16indexesextension_IndexArray __pyx_vtable_6tables_16indexesextension_IndexArray;

static PyObject *__pyx_tp_new_6tables_16indexesextension_IndexArray(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6tables_16indexesextension_IndexArray *p;
  PyObject *o = __pyx_ptype_6tables_13hdf5extension_Array->tp_new(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6tables_16indexesextension_IndexArray *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6tables_13hdf5extension_Leaf*)__pyx_vtabptr_6tables_16indexesextension_IndexArray;
  p->bounds_ext = ((struct __pyx_obj_6tables_16indexesextension_CacheArray *)Py_None); Py_INCREF(Py_None);
  p->boundscache = ((struct __pyx_obj_6tables_17lrucacheextension_NumCache *)Py_None); Py_INCREF(Py_None);
  p->sortedcache = ((struct __pyx_obj_6tables_17lrucacheextension_NumCache *)Py_None); Py_INCREF(Py_None);
  p->bufferbc = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  p->bufferlb = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6tables_16indexesextension_IndexArray(PyObject *o) {
  struct __pyx_obj_6tables_16indexesextension_IndexArray *p = (struct __pyx_obj_6tables_16indexesextension_IndexArray *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->bounds_ext);
  Py_CLEAR(p->boundscache);
  Py_CLEAR(p->sortedcache);
  Py_CLEAR(p->bufferbc);
  Py_CLEAR(p->bufferlb);
  PyObject_GC_Track(o);
  if (likely(__pyx_ptype_6tables_13hdf5extension_Array)) __pyx_ptype_6tables_13hdf5extension_Array->tp_dealloc(o); else __Pyx_call_next_tp_dealloc(o, __pyx_tp_dealloc_6tables_16indexesextension_IndexArray);
}

static int __pyx_tp_traverse_6tables_16indexesextension_IndexArray(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6tables_16indexesextension_IndexArray *p = (struct __pyx_obj_6tables_16indexesextension_IndexArray *)o;
  e = ((likely(__pyx_ptype_6tables_13hdf5extension_Array)) ? ((__pyx_ptype_6tables_13hdf5extension_Array->tp_traverse) ? __pyx_ptype_6tables_13hdf5extension_Array->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_6tables_16indexesextension_IndexArray)); if (e) return e;
  if (p->bounds_ext) {
    e = (*v)(((PyObject *)p->bounds_ext), a); if (e) return e;
  }
  if (p->boundscache) {
    e = (*v)(((PyObject *)p->boundscache), a); if (e) return e;
  }
  if (p->sortedcache) {
    e = (*v)(((PyObject *)p->sortedcache), a); if (e) return e;
  }
  if (p->bufferbc) {
    e = (*v)(((PyObject *)p->bufferbc), a); if (e) return e;
  }
  if (p->bufferlb) {
    e = (*v)(((PyObject *)p->bufferlb), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6tables_16indexesextension_IndexArray(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6tables_16indexesextension_IndexArray *p = (struct __pyx_obj_6tables_16indexesextension_IndexArray *)o;
  if (likely(__pyx_ptype_6tables_13hdf5extension_Array)) { if (__pyx_ptype_6tables_13hdf5extension_Array->tp_clear) __pyx_ptype_6tables_13hdf5extension_Array->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_6tables_16indexesextension_IndexArray);
  tmp = ((PyObject*)p->bounds_ext);
  p->bounds_ext = ((struct __pyx_obj_6tables_16indexesextension_CacheArray *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->boundscache);
  p->boundscache = ((struct __pyx_obj_6tables_17lrucacheextension_NumCache *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->sortedcache);
  p->sortedcache = ((struct __pyx_obj_6tables_17lrucacheextension_NumCache *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->bufferbc);
  p->bufferbc = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->bufferlb);
  p->bufferlb = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_6tables_16indexesextension_IndexArray[] = {
  {"_read_index_slice", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6tables_16indexesextension_10IndexArray_1_read_index_slice, METH_VARARGS|METH_KEYWORDS, 0},
  {"_init_sorted_slice", (PyCFunction)__pyx_pw_6tables_16indexesextension_10IndexArray_3_init_sorted_slice, METH_O, __pyx_doc_6tables_16indexesextension_10IndexArray_2_init_sorted_slice},
  {"_read_sorted_slice", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6tables_16indexesextension_10IndexArray_5_read_sorted_slice, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6tables_16indexesextension_10IndexArray_4_read_sorted_slice},
  {"_search_bin_na_b", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6tables_16indexesextension_10IndexArray_7_search_bin_na_b, METH_VARARGS|METH_KEYWORDS, 0},
  {"_search_bin_na_ub", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6tables_16indexesextension_10IndexArray_9_search_bin_na_ub, METH_VARARGS|METH_KEYWORDS, 0},
  {"_search_bin_na_s", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6tables_16indexesextension_10IndexArray_11_search_bin_na_s, METH_VARARGS|METH_KEYWORDS, 0},
  {"_search_bin_na_us", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6tables_16indexesextension_10IndexArray_13_search_bin_na_us, METH_VARARGS|METH_KEYWORDS, 0},
  {"_search_bin_na_i", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6tables_16indexesextension_10IndexArray_15_search_bin_na_i, METH_VARARGS|METH_KEYWORDS, 0},
  {"_search_bin_na_ui", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6tables_16indexesextension_10IndexArray_17_search_bin_na_ui, METH_VARARGS|METH_KEYWORDS, 0},
  {"_search_bin_na_ll", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6tables_16indexesextension_10IndexArray_19_search_bin_na_ll, METH_VARARGS|METH_KEYWORDS, 0},
  {"_search_bin_na_ull", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6tables_16indexesextension_10IndexArray_21_search_bin_na_ull, METH_VARARGS|METH_KEYWORDS, 0},
  {"_search_bin_na_e", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6tables_16indexesextension_10IndexArray_23_search_bin_na_e, METH_VARARGS|METH_KEYWORDS, 0},
  {"_search_bin_na_f", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6tables_16indexesextension_10IndexArray_25_search_bin_na_f, METH_VARARGS|METH_KEYWORDS, 0},
  {"_search_bin_na_d", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6tables_16indexesextension_10IndexArray_27_search_bin_na_d, METH_VARARGS|METH_KEYWORDS, 0},
  {"_search_bin_na_g", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6tables_16indexesextension_10IndexArray_29_search_bin_na_g, METH_VARARGS|METH_KEYWORDS, 0},
  {"_g_close", (PyCFunction)__pyx_pw_6tables_16indexesextension_10IndexArray_31_g_close, METH_NOARGS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6tables_16indexesextension_10IndexArray_33__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6tables_16indexesextension_10IndexArray_35__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6tables_16indexesextension_IndexArray = {
  PyVarObject_HEAD_INIT(0, 0)
  "tables.indexesextension.IndexArray", /*tp_name*/
  sizeof(struct __pyx_obj_6tables_16indexesextension_IndexArray), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6tables_16indexesextension_IndexArray, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "Container for keeping sorted and indices values.", /*tp_doc*/
  __pyx_tp_traverse_6tables_16indexesextension_IndexArray, /*tp_traverse*/
  __pyx_tp_clear_6tables_16indexesextension_IndexArray, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6tables_16indexesextension_IndexArray, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6tables_16indexesextension_IndexArray, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM+0 >= 0x06000000
  0, /*tp_pypy_flags*/
  #endif
};
static struct __pyx_vtabstruct_6tables_16indexesextension_LastRowArray __pyx_vtable_6tables_16indexesextension_LastRowArray;

static PyObject *__pyx_tp_new_6tables_16indexesextension_LastRowArray(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6tables_16indexesextension_LastRowArray *p;
  PyObject *o = __pyx_ptype_6tables_13hdf5extension_Array->tp_new(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6tables_16indexesextension_LastRowArray *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6tables_13hdf5extension_Leaf*)__pyx_vtabptr_6tables_16indexesextension_LastRowArray;
  return o;
}

static void __pyx_tp_dealloc_6tables_16indexesextension_LastRowArray(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  PyObject_GC_Track(o);
  if (likely(__pyx_ptype_6tables_13hdf5extension_Array)) __pyx_ptype_6tables_13hdf5extension_Array->tp_dealloc(o); else __Pyx_call_next_tp_dealloc(o, __pyx_tp_dealloc_6tables_16indexesextension_LastRowArray);
}

static int __pyx_tp_traverse_6tables_16indexesextension_LastRowArray(PyObject *o, visitproc v, void *a) {
  int e;
  e = ((likely(__pyx_ptype_6tables_13hdf5extension_Array)) ? ((__pyx_ptype_6tables_13hdf5extension_Array->tp_traverse) ? __pyx_ptype_6tables_13hdf5extension_Array->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_6tables_16indexesextension_LastRowArray)); if (e) return e;
  return 0;
}

static int __pyx_tp_clear_6tables_16indexesextension_LastRowArray(PyObject *o) {
  if (likely(__pyx_ptype_6tables_13hdf5extension_Array)) { if (__pyx_ptype_6tables_13hdf5extension_Array->tp_clear) __pyx_ptype_6tables_13hdf5extension_Array->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_6tables_16indexesextension_LastRowArray);
  return 0;
}

static PyMethodDef __pyx_methods_6tables_16indexesextension_LastRowArray[] = {
  {"_read_index_slice", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6tables_16indexesextension_12LastRowArray_1_read_index_slice, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6tables_16indexesextension_12LastRowArray__read_index_slice},
  {"_read_sorted_slice", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6tables_16indexesextension_12LastRowArray_3_read_sorted_slice, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6tables_16indexesextension_12LastRowArray_2_read_sorted_slice},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6tables_16indexesextension_12LastRowArray_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6tables_16indexesextension_12LastRowArray_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6tables_16indexesextension_LastRowArray = {
  PyVarObject_HEAD_INIT(0, 0)
  "tables.indexesextension.LastRowArray", /*tp_name*/
  sizeof(struct __pyx_obj_6tables_16indexesextension_LastRowArray), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6tables_16indexesextension_LastRowArray, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n  Container for keeping sorted and indices values of last rows of an index.\n  ", /*tp_doc*/
  __pyx_tp_traverse_6tables_16indexesextension_LastRowArray, /*tp_traverse*/
  __pyx_tp_clear_6tables_16indexesextension_LastRowArray, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6tables_16indexesextension_LastRowArray, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6tables_16indexesextension_LastRowArray, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM+0 >= 0x06000000
  0, /*tp_pypy_flags*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_indexesextension(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_indexesextension},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "indexesextension",
    __pyx_k_cython_interface_for_keeping_ind, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_s_BOUNDS_MAX_SIZE, __pyx_k_BOUNDS_MAX_SIZE, sizeof(__pyx_k_BOUNDS_MAX_SIZE), 0, 0, 1, 1},
  {&__pyx_n_s_CacheArray, __pyx_k_CacheArray, sizeof(__pyx_k_CacheArray), 0, 0, 1, 1},
  {&__pyx_n_s_HDF5ExtError, __pyx_k_HDF5ExtError, sizeof(__pyx_k_HDF5ExtError), 0, 0, 1, 1},
  {&__pyx_n_s_ImportError, __pyx_k_ImportError, sizeof(__pyx_k_ImportError), 0, 0, 1, 1},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0xd4, __pyx_k_Incompatible_checksums_s_vs_0xd4, sizeof(__pyx_k_Incompatible_checksums_s_vs_0xd4), 0, 0, 1, 0},
  {&__pyx_n_s_Index, __pyx_k_Index, sizeof(__pyx_k_Index), 0, 0, 1, 1},
  {&__pyx_n_s_IndexArray, __pyx_k_IndexArray, sizeof(__pyx_k_IndexArray), 0, 0, 1, 1},
  {&__pyx_n_s_LastRowArray, __pyx_k_LastRowArray, sizeof(__pyx_k_LastRowArray), 0, 0, 1, 1},
  {&__pyx_n_s_PickleError, __pyx_k_PickleError, sizeof(__pyx_k_PickleError), 0, 0, 1, 1},
  {&__pyx_kp_s_Problems_initializing_the_bounds, __pyx_k_Problems_initializing_the_bounds, sizeof(__pyx_k_Problems_initializing_the_bounds), 0, 0, 1, 0},
  {&__pyx_kp_s_Problems_reading_the_array_data, __pyx_k_Problems_reading_the_array_data, sizeof(__pyx_k_Problems_reading_the_array_data), 0, 0, 1, 0},
  {&__pyx_kp_s_Problems_reading_the_bounds_arra, __pyx_k_Problems_reading_the_bounds_arra, sizeof(__pyx_k_Problems_reading_the_bounds_arra), 0, 0, 1, 0},
  {&__pyx_kp_s_Problems_reading_the_index_data, __pyx_k_Problems_reading_the_index_data, sizeof(__pyx_k_Problems_reading_the_index_data), 0, 0, 1, 0},
  {&__pyx_kp_s_Problems_reading_the_index_data_2, __pyx_k_Problems_reading_the_index_data_2, sizeof(__pyx_k_Problems_reading_the_index_data_2), 0, 0, 1, 0},
  {&__pyx_kp_s_Problems_reading_the_index_indic, __pyx_k_Problems_reading_the_index_indic, sizeof(__pyx_k_Problems_reading_the_index_indic), 0, 0, 1, 0},
  {&__pyx_n_s_SORTED_MAX_SIZE, __pyx_k_SORTED_MAX_SIZE, sizeof(__pyx_k_SORTED_MAX_SIZE), 0, 0, 1, 1},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_kp_s_Unknown_array_datatype, __pyx_k_Unknown_array_datatype, sizeof(__pyx_k_Unknown_array_datatype), 0, 0, 1, 0},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_n_s_a, __pyx_k_a, sizeof(__pyx_k_a), 0, 0, 1, 1},
  {&__pyx_n_s_array1, __pyx_k_array1, sizeof(__pyx_k_array1), 0, 0, 1, 1},
  {&__pyx_n_s_array2, __pyx_k_array2, sizeof(__pyx_k_array2), 0, 0, 1, 1},
  {&__pyx_n_s_atom, __pyx_k_atom, sizeof(__pyx_k_atom), 0, 0, 1, 1},
  {&__pyx_n_s_bisect_left, __pyx_k_bisect_left, sizeof(__pyx_k_bisect_left), 0, 0, 1, 1},
  {&__pyx_n_s_bisect_right, __pyx_k_bisect_right, sizeof(__pyx_k_bisect_right), 0, 0, 1, 1},
  {&__pyx_n_s_bounds, __pyx_k_bounds, sizeof(__pyx_k_bounds), 0, 0, 1, 1},
  {&__pyx_n_s_chunksize, __pyx_k_chunksize, sizeof(__pyx_k_chunksize), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_dict, __pyx_k_dict, sizeof(__pyx_k_dict), 0, 0, 1, 1},
  {&__pyx_n_s_dtype, __pyx_k_dtype, sizeof(__pyx_k_dtype), 0, 0, 1, 1},
  {&__pyx_n_s_elsize1, __pyx_k_elsize1, sizeof(__pyx_k_elsize1), 0, 0, 1, 1},
  {&__pyx_n_s_elsize2, __pyx_k_elsize2, sizeof(__pyx_k_elsize2), 0, 0, 1, 1},
  {&__pyx_n_s_empty, __pyx_k_empty, sizeof(__pyx_k_empty), 0, 0, 1, 1},
  {&__pyx_n_s_exceptions, __pyx_k_exceptions, sizeof(__pyx_k_exceptions), 0, 0, 1, 1},
  {&__pyx_n_s_g_close, __pyx_k_g_close, sizeof(__pyx_k_g_close), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_hi, __pyx_k_hi, sizeof(__pyx_k_hi), 0, 0, 1, 1},
  {&__pyx_n_s_idx, __pyx_k_idx, sizeof(__pyx_k_idx), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_irow, __pyx_k_irow, sizeof(__pyx_k_irow), 0, 0, 1, 1},
  {&__pyx_n_s_item1, __pyx_k_item1, sizeof(__pyx_k_item1), 0, 0, 1, 1},
  {&__pyx_n_s_item2, __pyx_k_item2, sizeof(__pyx_k_item2), 0, 0, 1, 1},
  {&__pyx_n_s_itemsize, __pyx_k_itemsize, sizeof(__pyx_k_itemsize), 0, 0, 1, 1},
  {&__pyx_n_s_keysort, __pyx_k_keysort, sizeof(__pyx_k_keysort), 0, 0, 1, 1},
  {&__pyx_n_s_lengths, __pyx_k_lengths, sizeof(__pyx_k_lengths), 0, 0, 1, 1},
  {&__pyx_n_s_lo, __pyx_k_lo, sizeof(__pyx_k_lo), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_mid, __pyx_k_mid, sizeof(__pyx_k_mid), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_new, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
  {&__pyx_kp_s_non_opt_types_bounds, __pyx_k_non_opt_types_bounds, sizeof(__pyx_k_non_opt_types_bounds), 0, 0, 1, 0},
  {&__pyx_n_s_nrows, __pyx_k_nrows, sizeof(__pyx_k_nrows), 0, 0, 1, 1},
  {&__pyx_n_s_numpy, __pyx_k_numpy, sizeof(__pyx_k_numpy), 0, 0, 1, 1},
  {&__pyx_kp_s_numpy_core_multiarray_failed_to, __pyx_k_numpy_core_multiarray_failed_to, sizeof(__pyx_k_numpy_core_multiarray_failed_to), 0, 0, 1, 0},
  {&__pyx_kp_s_numpy_core_umath_failed_to_impor, __pyx_k_numpy_core_umath_failed_to_impor, sizeof(__pyx_k_numpy_core_umath_failed_to_impor), 0, 0, 1, 0},
  {&__pyx_n_s_opt_search_types, __pyx_k_opt_search_types, sizeof(__pyx_k_opt_search_types), 0, 0, 1, 1},
  {&__pyx_n_s_params, __pyx_k_params, sizeof(__pyx_k_params), 0, 0, 1, 1},
  {&__pyx_n_s_pickle, __pyx_k_pickle, sizeof(__pyx_k_pickle), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_PickleError, __pyx_k_pyx_PickleError, sizeof(__pyx_k_pyx_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_checksum, __pyx_k_pyx_checksum, sizeof(__pyx_k_pyx_checksum), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_result, __pyx_k_pyx_result, sizeof(__pyx_k_pyx_result), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_state, __pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_type, __pyx_k_pyx_type, sizeof(__pyx_k_pyx_type), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_Index, __pyx_k_pyx_unpickle_Index, sizeof(__pyx_k_pyx_unpickle_Index), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_ranges, __pyx_k_ranges, sizeof(__pyx_k_ranges), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_rvcache, __pyx_k_rvcache, sizeof(__pyx_k_rvcache), 0, 0, 1, 1},
  {&__pyx_kp_s_self_dims_self_dims_chunk_self_m, __pyx_k_self_dims_self_dims_chunk_self_m, sizeof(__pyx_k_self_dims_self_dims_chunk_self_m), 0, 0, 1, 0},
  {&__pyx_kp_s_self_dims_self_dims_chunk_self_m_2, __pyx_k_self_dims_self_dims_chunk_self_m_2, sizeof(__pyx_k_self_dims_self_dims_chunk_self_m_2), 0, 0, 1, 0},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_shape, __pyx_k_shape, sizeof(__pyx_k_shape), 0, 0, 1, 1},
  {&__pyx_n_s_size, __pyx_k_size, sizeof(__pyx_k_size), 0, 0, 1, 1},
  {&__pyx_n_s_slicesize, __pyx_k_slicesize, sizeof(__pyx_k_slicesize), 0, 0, 1, 1},
  {&__pyx_n_s_sorted, __pyx_k_sorted, sizeof(__pyx_k_sorted), 0, 0, 1, 1},
  {&__pyx_n_s_start, __pyx_k_start, sizeof(__pyx_k_start), 0, 0, 1, 1},
  {&__pyx_n_s_starts, __pyx_k_starts, sizeof(__pyx_k_starts), 0, 0, 1, 1},
  {&__pyx_n_s_stop, __pyx_k_stop, sizeof(__pyx_k_stop), 0, 0, 1, 1},
  {&__pyx_kp_s_stringsource, __pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 0, 1, 0},
  {&__pyx_n_s_super, __pyx_k_super, sizeof(__pyx_k_super), 0, 0, 1, 1},
  {&__pyx_n_s_tables_indexesextension, __pyx_k_tables_indexesextension, sizeof(__pyx_k_tables_indexesextension), 0, 0, 1, 1},
  {&__pyx_kp_s_tables_indexesextension_pyx, __pyx_k_tables_indexesextension_pyx, sizeof(__pyx_k_tables_indexesextension_pyx), 0, 0, 1, 0},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_type_num, __pyx_k_type_num, sizeof(__pyx_k_type_num), 0, 0, 1, 1},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {&__pyx_n_s_v_chunkshape, __pyx_k_v_chunkshape, sizeof(__pyx_k_v_chunkshape), 0, 0, 1, 1},
  {&__pyx_n_s_v_file, __pyx_k_v_file, sizeof(__pyx_k_v_file), 0, 0, 1, 1},
  {&__pyx_n_s_v_parent, __pyx_k_v_parent, sizeof(__pyx_k_v_parent), 0, 0, 1, 1},
  {&__pyx_n_s_x, __pyx_k_x, sizeof(__pyx_k_x), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(0, 189, __pyx_L1_error)
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 200, __pyx_L1_error)
  __pyx_builtin_super = __Pyx_GetBuiltinName(__pyx_n_s_super); if (!__pyx_builtin_super) __PYX_ERR(0, 583, __pyx_L1_error)
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(1, 2, __pyx_L1_error)
  __pyx_builtin_ImportError = __Pyx_GetBuiltinName(__pyx_n_s_ImportError); if (!__pyx_builtin_ImportError) __PYX_ERR(2, 945, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "tables/indexesextension.pyx":189
 *         _keysort_string(PyArray_BYTES(array1), elsize1, PyArray_BYTES(array2), elsize2, size)
 *     else:
 *         raise ValueError("Unknown array datatype")             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_s_Unknown_array_datatype); if (unlikely(!__pyx_tuple_)) __PYX_ERR(0, 189, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_s_self_dims_self_dims_chunk_self_m); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "(tree fragment)":4
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_s_self_dims_self_dims_chunk_self_m); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "tables/indexesextension.pyx":649
 *     self.rbufln = PyArray_DATA(lengths)
 *     # The 1st cache is loaded completely in memory and needs to be reloaded
 *     rvcache = index.ranges[:]             # <<<<<<<<<<<<<<
 *     self.rbufrv = PyArray_DATA(rvcache)
 *     index.rvcache = <object>rvcache
 */
  __pyx_slice__4 = PySlice_New(Py_None, Py_None, Py_None); if (unlikely(!__pyx_slice__4)) __PYX_ERR(0, 649, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__4);
  __Pyx_GIVEREF(__pyx_slice__4);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims,self.rbufbc,self.rbuflb,self.rbufln,self.rbufrv,self.rbufst cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims,self.rbufbc,self.rbuflb,self.rbufln,self.rbufrv,self.rbufst cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_kp_s_self_dims_self_dims_chunk_self_m_2); if (unlikely(!__pyx_tuple__5)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "(tree fragment)":4
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims,self.rbufbc,self.rbuflb,self.rbufln,self.rbufrv,self.rbufst cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims,self.rbufbc,self.rbuflb,self.rbufln,self.rbufrv,self.rbufst cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_kp_s_self_dims_self_dims_chunk_self_m_2); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__7 = PyTuple_Pack(1, __pyx_kp_s_self_dims_self_dims_chunk_self_m); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);

  /* "(tree fragment)":4
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__8 = PyTuple_Pack(1, __pyx_kp_s_self_dims_self_dims_chunk_self_m); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":945
 *         __pyx_import_array()
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */
  __pyx_tuple__9 = PyTuple_Pack(1, __pyx_kp_s_numpy_core_multiarray_failed_to); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(2, 945, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":951
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */
  __pyx_tuple__10 = PyTuple_Pack(1, __pyx_kp_s_numpy_core_umath_failed_to_impor); if (unlikely(!__pyx_tuple__10)) __PYX_ERR(2, 951, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);

  /* "tables/indexesextension.pyx":144
 * DEF SMALL_QUICKSORT = 15
 * 
 * def keysort(ndarray array1, ndarray array2):             # <<<<<<<<<<<<<<
 *     """Sort array1 in-place. array2 is also sorted following the array1 order.
 * 
 */
  __pyx_tuple__11 = PyTuple_Pack(6, __pyx_n_s_array1, __pyx_n_s_array2, __pyx_n_s_size, __pyx_n_s_elsize1, __pyx_n_s_elsize2, __pyx_n_s_type_num); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(0, 144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);
  __pyx_codeobj__12 = (PyObject*)__Pyx_PyCode_New(2, 0, 6, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__11, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_tables_indexesextension_pyx, __pyx_n_s_keysort, 144, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__12)) __PYX_ERR(0, 144, __pyx_L1_error)

  /* "tables/indexesextension.pyx":507
 * # because I forsee that this should be a very common case.
 * # 2004-05-20
 * def _bisect_left(a, x, int hi):             # <<<<<<<<<<<<<<
 *   """Return the index where to insert item x in list a, assuming a is sorted.
 * 
 */
  __pyx_tuple__13 = PyTuple_Pack(5, __pyx_n_s_a, __pyx_n_s_x, __pyx_n_s_hi, __pyx_n_s_lo, __pyx_n_s_mid); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(0, 507, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);
  __pyx_codeobj__14 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__13, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_tables_indexesextension_pyx, __pyx_n_s_bisect_left, 507, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__14)) __PYX_ERR(0, 507, __pyx_L1_error)

  /* "tables/indexesextension.pyx":528
 * 
 * 
 * def _bisect_right(a, x, int hi):             # <<<<<<<<<<<<<<
 *   """Return the index where to insert item x in list a, assuming a is sorted.
 * 
 */
  __pyx_tuple__15 = PyTuple_Pack(5, __pyx_n_s_a, __pyx_n_s_x, __pyx_n_s_hi, __pyx_n_s_lo, __pyx_n_s_mid); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(0, 528, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);
  __pyx_codeobj__16 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__15, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_tables_indexesextension_pyx, __pyx_n_s_bisect_right, 528, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__16)) __PYX_ERR(0, 528, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __pyx_unpickle_Index(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_tuple__17 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);
  __pyx_codeobj__18 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__17, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_Index, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__18)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_222419149 = PyInt_FromLong(222419149L); if (unlikely(!__pyx_int_222419149)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  if (PyType_Ready(&__pyx_type_6tables_16indexesextension_Index) < 0) __PYX_ERR(0, 555, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6tables_16indexesextension_Index.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6tables_16indexesextension_Index.tp_dictoffset && __pyx_type_6tables_16indexesextension_Index.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6tables_16indexesextension_Index.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_Index, (PyObject *)&__pyx_type_6tables_16indexesextension_Index) < 0) __PYX_ERR(0, 555, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6tables_16indexesextension_Index) < 0) __PYX_ERR(0, 555, __pyx_L1_error)
  __pyx_ptype_6tables_16indexesextension_Index = &__pyx_type_6tables_16indexesextension_Index;
  __pyx_t_1 = PyImport_ImportModule("tables.hdf5extension"); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_6tables_13hdf5extension_Array = __Pyx_ImportType(__pyx_t_1, "tables.hdf5extension", "Array", sizeof(struct __pyx_obj_6tables_13hdf5extension_Array), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6tables_13hdf5extension_Array) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_vtabptr_6tables_13hdf5extension_Array = (struct __pyx_vtabstruct_6tables_13hdf5extension_Array*)__Pyx_GetVtable(__pyx_ptype_6tables_13hdf5extension_Array->tp_dict); if (unlikely(!__pyx_vtabptr_6tables_13hdf5extension_Array)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_vtabptr_6tables_16indexesextension_CacheArray = &__pyx_vtable_6tables_16indexesextension_CacheArray;
  __pyx_vtable_6tables_16indexesextension_CacheArray.__pyx_base = *__pyx_vtabptr_6tables_13hdf5extension_Array;
  __pyx_vtable_6tables_16indexesextension_CacheArray.initread = (PyObject *(*)(struct __pyx_obj_6tables_16indexesextension_CacheArray *, int))__pyx_f_6tables_16indexesextension_10CacheArray_initread;
  __pyx_vtable_6tables_16indexesextension_CacheArray.read_slice = (PyObject *(*)(struct __pyx_obj_6tables_16indexesextension_CacheArray *, hsize_t, hsize_t, hsize_t, void *))__pyx_f_6tables_16indexesextension_10CacheArray_read_slice;
  __pyx_type_6tables_16indexesextension_CacheArray.tp_base = __pyx_ptype_6tables_13hdf5extension_Array;
  if (PyType_Ready(&__pyx_type_6tables_16indexesextension_CacheArray) < 0) __PYX_ERR(0, 559, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6tables_16indexesextension_CacheArray.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6tables_16indexesextension_CacheArray.tp_dictoffset && __pyx_type_6tables_16indexesextension_CacheArray.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6tables_16indexesextension_CacheArray.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6tables_16indexesextension_CacheArray.tp_dict, __pyx_vtabptr_6tables_16indexesextension_CacheArray) < 0) __PYX_ERR(0, 559, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_CacheArray, (PyObject *)&__pyx_type_6tables_16indexesextension_CacheArray) < 0) __PYX_ERR(0, 559, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6tables_16indexesextension_CacheArray) < 0) __PYX_ERR(0, 559, __pyx_L1_error)
  __pyx_ptype_6tables_16indexesextension_CacheArray = &__pyx_type_6tables_16indexesextension_CacheArray;
  __pyx_vtabptr_6tables_16indexesextension_IndexArray = &__pyx_vtable_6tables_16indexesextension_IndexArray;
  __pyx_vtable_6tables_16indexesextension_IndexArray.__pyx_base = *__pyx_vtabptr_6tables_13hdf5extension_Array;
  __pyx_vtable_6tables_16indexesextension_IndexArray._g_read_sorted_slice = (void *(*)(struct __pyx_obj_6tables_16indexesextension_IndexArray *, hsize_t, hsize_t, hsize_t))__pyx_f_6tables_16indexesextension_10IndexArray__g_read_sorted_slice;
  __pyx_vtable_6tables_16indexesextension_IndexArray.get_lru_bounds = (void *(*)(struct __pyx_obj_6tables_16indexesextension_IndexArray *, int, int))__pyx_f_6tables_16indexesextension_10IndexArray_get_lru_bounds;
  __pyx_vtable_6tables_16indexesextension_IndexArray.get_lru_sorted = (void *(*)(struct __pyx_obj_6tables_16indexesextension_IndexArray *, int, int, int, int))__pyx_f_6tables_16indexesextension_10IndexArray_get_lru_sorted;
  __pyx_type_6tables_16indexesextension_IndexArray.tp_base = __pyx_ptype_6tables_13hdf5extension_Array;
  if (PyType_Ready(&__pyx_type_6tables_16indexesextension_IndexArray) < 0) __PYX_ERR(0, 589, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6tables_16indexesextension_IndexArray.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6tables_16indexesextension_IndexArray.tp_dictoffset && __pyx_type_6tables_16indexesextension_IndexArray.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6tables_16indexesextension_IndexArray.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6tables_16indexesextension_IndexArray.tp_dict, __pyx_vtabptr_6tables_16indexesextension_IndexArray) < 0) __PYX_ERR(0, 589, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_IndexArray, (PyObject *)&__pyx_type_6tables_16indexesextension_IndexArray) < 0) __PYX_ERR(0, 589, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6tables_16indexesextension_IndexArray) < 0) __PYX_ERR(0, 589, __pyx_L1_error)
  __pyx_ptype_6tables_16indexesextension_IndexArray = &__pyx_type_6tables_16indexesextension_IndexArray;
  __pyx_vtabptr_6tables_16indexesextension_LastRowArray = &__pyx_vtable_6tables_16indexesextension_LastRowArray;
  __pyx_vtable_6tables_16indexesextension_LastRowArray.__pyx_base = *__pyx_vtabptr_6tables_13hdf5extension_Array;
  __pyx_type_6tables_16indexesextension_LastRowArray.tp_base = __pyx_ptype_6tables_13hdf5extension_Array;
  if (PyType_Ready(&__pyx_type_6tables_16indexesextension_LastRowArray) < 0) __PYX_ERR(0, 1493, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6tables_16indexesextension_LastRowArray.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6tables_16indexesextension_LastRowArray.tp_dictoffset && __pyx_type_6tables_16indexesextension_LastRowArray.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6tables_16indexesextension_LastRowArray.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6tables_16indexesextension_LastRowArray.tp_dict, __pyx_vtabptr_6tables_16indexesextension_LastRowArray) < 0) __PYX_ERR(0, 1493, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_LastRowArray, (PyObject *)&__pyx_type_6tables_16indexesextension_LastRowArray) < 0) __PYX_ERR(0, 1493, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6tables_16indexesextension_LastRowArray) < 0) __PYX_ERR(0, 1493, __pyx_L1_error)
  __pyx_ptype_6tables_16indexesextension_LastRowArray = &__pyx_type_6tables_16indexesextension_LastRowArray;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type", 
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_4type_type) __PYX_ERR(3, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("numpy"); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_5numpy_dtype = __Pyx_ImportType(__pyx_t_1, "numpy", "dtype", sizeof(PyArray_Descr), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_dtype) __PYX_ERR(2, 200, __pyx_L1_error)
  __pyx_ptype_5numpy_flatiter = __Pyx_ImportType(__pyx_t_1, "numpy", "flatiter", sizeof(PyArrayIterObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_flatiter) __PYX_ERR(2, 223, __pyx_L1_error)
  __pyx_ptype_5numpy_broadcast = __Pyx_ImportType(__pyx_t_1, "numpy", "broadcast", sizeof(PyArrayMultiIterObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_broadcast) __PYX_ERR(2, 227, __pyx_L1_error)
  __pyx_ptype_5numpy_ndarray = __Pyx_ImportType(__pyx_t_1, "numpy", "ndarray", sizeof(PyArrayObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_ndarray) __PYX_ERR(2, 239, __pyx_L1_error)
  __pyx_ptype_5numpy_generic = __Pyx_ImportType(__pyx_t_1, "numpy", "generic", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_generic) __PYX_ERR(2, 771, __pyx_L1_error)
  __pyx_ptype_5numpy_number = __Pyx_ImportType(__pyx_t_1, "numpy", "number", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_number) __PYX_ERR(2, 773, __pyx_L1_error)
  __pyx_ptype_5numpy_integer = __Pyx_ImportType(__pyx_t_1, "numpy", "integer", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_integer) __PYX_ERR(2, 775, __pyx_L1_error)
  __pyx_ptype_5numpy_signedinteger = __Pyx_ImportType(__pyx_t_1, "numpy", "signedinteger", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_signedinteger) __PYX_ERR(2, 777, __pyx_L1_error)
  __pyx_ptype_5numpy_unsignedinteger = __Pyx_ImportType(__pyx_t_1, "numpy", "unsignedinteger", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_unsignedinteger) __PYX_ERR(2, 779, __pyx_L1_error)
  __pyx_ptype_5numpy_inexact = __Pyx_ImportType(__pyx_t_1, "numpy", "inexact", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_inexact) __PYX_ERR(2, 781, __pyx_L1_error)
  __pyx_ptype_5numpy_floating = __Pyx_ImportType(__pyx_t_1, "numpy", "floating", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_floating) __PYX_ERR(2, 783, __pyx_L1_error)
  __pyx_ptype_5numpy_complexfloating = __Pyx_ImportType(__pyx_t_1, "numpy", "complexfloating", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_complexfloating) __PYX_ERR(2, 785, __pyx_L1_error)
  __pyx_ptype_5numpy_flexible = __Pyx_ImportType(__pyx_t_1, "numpy", "flexible", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_flexible) __PYX_ERR(2, 787, __pyx_L1_error)
  __pyx_ptype_5numpy_character = __Pyx_ImportType(__pyx_t_1, "numpy", "character", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_character) __PYX_ERR(2, 789, __pyx_L1_error)
  __pyx_ptype_5numpy_ufunc = __Pyx_ImportType(__pyx_t_1, "numpy", "ufunc", sizeof(PyUFuncObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_ufunc) __PYX_ERR(2, 827, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("tables.hdf5extension"); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_6tables_13hdf5extension_Node = __Pyx_ImportType(__pyx_t_1, "tables.hdf5extension", "Node", sizeof(struct __pyx_obj_6tables_13hdf5extension_Node), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6tables_13hdf5extension_Node) __PYX_ERR(4, 16, __pyx_L1_error)
  __pyx_ptype_6tables_13hdf5extension_Leaf = __Pyx_ImportType(__pyx_t_1, "tables.hdf5extension", "Leaf", sizeof(struct __pyx_obj_6tables_13hdf5extension_Leaf), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6tables_13hdf5extension_Leaf) __PYX_ERR(4, 20, __pyx_L1_error)
  __pyx_vtabptr_6tables_13hdf5extension_Leaf = (struct __pyx_vtabstruct_6tables_13hdf5extension_Leaf*)__Pyx_GetVtable(__pyx_ptype_6tables_13hdf5extension_Leaf->tp_dict); if (unlikely(!__pyx_vtabptr_6tables_13hdf5extension_Leaf)) __PYX_ERR(4, 20, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("tables.lrucacheextension"); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_6tables_17lrucacheextension_NodeCache = __Pyx_ImportType(__pyx_t_1, "tables.lrucacheextension", "NodeCache", sizeof(struct __pyx_obj_6tables_17lrucacheextension_NodeCache), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6tables_17lrucacheextension_NodeCache) __PYX_ERR(5, 15, __pyx_L1_error)
  __pyx_vtabptr_6tables_17lrucacheextension_NodeCache = (struct __pyx_vtabstruct_6tables_17lrucacheextension_NodeCache*)__Pyx_GetVtable(__pyx_ptype_6tables_17lrucacheextension_NodeCache->tp_dict); if (unlikely(!__pyx_vtabptr_6tables_17lrucacheextension_NodeCache)) __PYX_ERR(5, 15, __pyx_L1_error)
  __pyx_ptype_6tables_17lrucacheextension_BaseCache = __Pyx_ImportType(__pyx_t_1, "tables.lrucacheextension", "BaseCache", sizeof(struct __pyx_obj_6tables_17lrucacheextension_BaseCache), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6tables_17lrucacheextension_BaseCache) __PYX_ERR(5, 25, __pyx_L1_error)
  __pyx_vtabptr_6tables_17lrucacheextension_BaseCache = (struct __pyx_vtabstruct_6tables_17lrucacheextension_BaseCache*)__Pyx_GetVtable(__pyx_ptype_6tables_17lrucacheextension_BaseCache->tp_dict); if (unlikely(!__pyx_vtabptr_6tables_17lrucacheextension_BaseCache)) __PYX_ERR(5, 25, __pyx_L1_error)
  __pyx_ptype_6tables_17lrucacheextension_ObjectNode = __Pyx_ImportType(__pyx_t_1, "tables.lrucacheextension", "ObjectNode", sizeof(struct __pyx_obj_6tables_17lrucacheextension_ObjectNode), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6tables_17lrucacheextension_ObjectNode) __PYX_ERR(5, 42, __pyx_L1_error)
  __pyx_ptype_6tables_17lrucacheextension_ObjectCache = __Pyx_ImportType(__pyx_t_1, "tables.lrucacheextension", "ObjectCache", sizeof(struct __pyx_obj_6tables_17lrucacheextension_ObjectCache), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6tables_17lrucacheextension_ObjectCache) __PYX_ERR(5, 48, __pyx_L1_error)
  __pyx_vtabptr_6tables_17lrucacheextension_ObjectCache = (struct __pyx_vtabstruct_6tables_17lrucacheextension_ObjectCache*)__Pyx_GetVtable(__pyx_ptype_6tables_17lrucacheextension_ObjectCache->tp_dict); if (unlikely(!__pyx_vtabptr_6tables_17lrucacheextension_ObjectCache)) __PYX_ERR(5, 48, __pyx_L1_error)
  __pyx_ptype_6tables_17lrucacheextension_NumCache = __Pyx_ImportType(__pyx_t_1, "tables.lrucacheextension", "NumCache", sizeof(struct __pyx_obj_6tables_17lrucacheextension_NumCache), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6tables_17lrucacheextension_NumCache) __PYX_ERR(5, 63, __pyx_L1_error)
  __pyx_vtabptr_6tables_17lrucacheextension_NumCache = (struct __pyx_vtabstruct_6tables_17lrucacheextension_NumCache*)__Pyx_GetVtable(__pyx_ptype_6tables_17lrucacheextension_NumCache->tp_dict); if (unlikely(!__pyx_vtabptr_6tables_17lrucacheextension_NumCache)) __PYX_ERR(5, 63, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initindexesextension(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initindexesextension(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_indexesextension(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_indexesextension(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_indexesextension(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'indexesextension' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_indexesextension(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(WITH_THREAD) && PY_VERSION_HEX < 0x030700F0 && defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  PyEval_InitThreads();
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("indexesextension", __pyx_methods, __pyx_k_cython_interface_for_keeping_ind, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_b);
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_cython_runtime);
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_tables__indexesextension) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "tables.indexesextension")) {
      if (unlikely(PyDict_SetItemString(modules, "tables.indexesextension", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  if (unlikely(__Pyx_modinit_type_init_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_type_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_variable_import_code();
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "tables/indexesextension.pyx":28
 * 
 * import cython
 * import numpy             # <<<<<<<<<<<<<<
 * cimport numpy as cnp
 * 
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_numpy, __pyx_t_1) < 0) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "tables/indexesextension.pyx":31
 * cimport numpy as cnp
 * 
 * from .exceptions import HDF5ExtError             # <<<<<<<<<<<<<<
 * from .hdf5extension cimport Array
 * 
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_HDF5ExtError);
  __Pyx_GIVEREF(__pyx_n_s_HDF5ExtError);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_HDF5ExtError);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_exceptions, __pyx_t_1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_HDF5ExtError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_HDF5ExtError, __pyx_t_1) < 0) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/indexesextension.pyx":108
 * # The numpy API requires this function to be called before
 * # using any numpy facilities in an extension module.
 * import_array()             # <<<<<<<<<<<<<<
 * 
 * #---------------------------------------------------------------------------
 */
  __pyx_t_3 = __pyx_f_5numpy_import_array(); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 108, __pyx_L1_error)

  /* "tables/indexesextension.pyx":144
 * DEF SMALL_QUICKSORT = 15
 * 
 * def keysort(ndarray array1, ndarray array2):             # <<<<<<<<<<<<<<
 *     """Sort array1 in-place. array2 is also sorted following the array1 order.
 * 
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6tables_16indexesextension_1keysort, NULL, __pyx_n_s_tables_indexesextension); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_keysort, __pyx_t_2) < 0) __PYX_ERR(0, 144, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/indexesextension.pyx":507
 * # because I forsee that this should be a very common case.
 * # 2004-05-20
 * def _bisect_left(a, x, int hi):             # <<<<<<<<<<<<<<
 *   """Return the index where to insert item x in list a, assuming a is sorted.
 * 
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6tables_16indexesextension_3_bisect_left, NULL, __pyx_n_s_tables_indexesextension); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 507, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_bisect_left, __pyx_t_2) < 0) __PYX_ERR(0, 507, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/indexesextension.pyx":528
 * 
 * 
 * def _bisect_right(a, x, int hi):             # <<<<<<<<<<<<<<
 *   """Return the index where to insert item x in list a, assuming a is sorted.
 * 
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6tables_16indexesextension_5_bisect_right, NULL, __pyx_n_s_tables_indexesextension); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 528, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_bisect_right, __pyx_t_2) < 0) __PYX_ERR(0, 528, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_Index(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6tables_16indexesextension_7__pyx_unpickle_Index, NULL, __pyx_n_s_tables_indexesextension); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_Index, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/indexesextension.pyx":1
 * ########################################################################             # <<<<<<<<<<<<<<
 * #
 * # License: BSD
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":1014
 * 
 * 
 * cdef inline NPY_DATETIMEUNIT get_datetime64_unit(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the unit part of the dtype for a numpy datetime64 object.
 */

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init tables.indexesextension", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init tables.indexesextension");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* None */
static CYTHON_INLINE long __Pyx_div_long(long a, long b) {
    long q = a / b;
    long r = a - q*b;
    q -= ((r != 0) & ((r ^ b) < 0));
    return q;
}

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* GetAttr */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* GetAttr3 */
static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r = __Pyx_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
}

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall2Args */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *result = NULL;
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyFunction_FastCall(function, args, 2);
    }
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyCFunction_FastCall(function, args, 2);
    }
    #endif
    args = PyTuple_New(2);
    if (unlikely(!args)) goto done;
    Py_INCREF(arg1);
    PyTuple_SET_ITEM(args, 0, arg1);
    Py_INCREF(arg2);
    PyTuple_SET_ITEM(args, 1, arg2);
    Py_INCREF(function);
    result = __Pyx_PyObject_Call(function, args, NULL);
    Py_DECREF(args);
    Py_DECREF(function);
done:
    return result;
}

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (__Pyx_PyFastCFunction_Check(func)) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_CyFunction_Check(func)))
#else
    if (likely(PyCFunction_Check(func)))
#endif
    {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* SliceObject */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(PyObject* obj,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp;
#if PY_MAJOR_VERSION < 3
    PySequenceMethods* ms = Py_TYPE(obj)->tp_as_sequence;
    if (likely(ms && ms->sq_slice)) {
        if (!has_cstart) {
            if (_py_start && (*_py_start != Py_None)) {
                cstart = __Pyx_PyIndex_AsSsize_t(*_py_start);
                if ((cstart == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstart = 0;
        }
        if (!has_cstop) {
            if (_py_stop && (*_py_stop != Py_None)) {
                cstop = __Pyx_PyIndex_AsSsize_t(*_py_stop);
                if ((cstop == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstop = PY_SSIZE_T_MAX;
        }
        if (wraparound && unlikely((cstart < 0) | (cstop < 0)) && likely(ms->sq_length)) {
            Py_ssize_t l = ms->sq_length(obj);
            if (likely(l >= 0)) {
                if (cstop < 0) {
                    cstop += l;
                    if (cstop < 0) cstop = 0;
                }
                if (cstart < 0) {
                    cstart += l;
                    if (cstart < 0) cstart = 0;
                }
            } else {
                if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                    goto bad;
                PyErr_Clear();
            }
        }
        return ms->sq_slice(obj, cstart, cstop);
    }
#endif
    mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_subscript))
#endif
    {
        PyObject* result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyInt_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyInt_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_subscript(obj, py_slice);
#else
        result = PyObject_GetItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    PyErr_Format(PyExc_TypeError,
        "'%.200s' object is unsliceable", Py_TYPE(obj)->tp_name);
bad:
    return NULL;
}

/* PyObjectSetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_setattr))
        return tp->tp_setattr(obj, PyString_AS_STRING(attr_name), value);
#endif
    return PyObject_SetAttr(obj, attr_name, value);
}
#endif

/* DictGetItem */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            if (unlikely(PyTuple_Check(key))) {
                PyObject* args = PyTuple_Pack(1, key);
                if (likely(args)) {
                    PyErr_SetObject(PyExc_KeyError, args);
                    Py_DECREF(args);
                }
            } else {
                PyErr_SetObject(PyExc_KeyError, key);
            }
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#endif

/* WriteUnraisableException */
static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* None */
static CYTHON_INLINE int __Pyx_div_int(int a, int b) {
    int q = a / b;
    int r = a - q*b;
    q -= ((r != 0) & ((r ^ b) < 0));
    return q;
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if ((1) && (strchr(__Pyx_MODULE_NAME, '.'))) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* HasAttr */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (unlikely(!r)) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_type == NULL || exc_info->exc_type == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* CallNextTpDealloc */
static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_dealloc != current_tp_dealloc)
        type = type->tp_base;
    while (type && type->tp_dealloc == current_tp_dealloc)
        type = type->tp_base;
    if (type)
        type->tp_dealloc(obj);
}

/* CallNextTpTraverse */
static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_traverse != current_tp_traverse)
        type = type->tp_base;
    while (type && type->tp_traverse == current_tp_traverse)
        type = type->tp_base;
    if (type && type->tp_traverse)
        return type->tp_traverse(obj, v, a);
    return 0;
}

/* CallNextTpClear */
static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_clear) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_clear != current_tp_clear)
        type = type->tp_base;
    while (type && type->tp_clear == current_tp_clear)
        type = type->tp_base;
    if (type && type->tp_clear)
        type->tp_clear(obj);
}

/* PyObject_GenericGetAttrNoDict */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, attr_name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(attr_name));
#endif
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* PyObjectGetAttrStrNoError */
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS && PY_VERSION_HEX >= 0x030700B1
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
}

/* SetupReduce */
static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_reduce_cython);
            if (likely(reduce_cython)) {
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(PyObject *module, const char *module_name, const char *class_name,
    size_t size, enum __Pyx_ImportType_CheckSize check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if ((size_t)basicsize < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error && (size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* GetVTable */
static void* __Pyx_GetVtable(PyObject *dict) {
    void* ptr;
    PyObject *ob = PyObject_GetItem(dict, __pyx_n_s_pyx_vtable);
    if (!ob)
        goto bad;
#if PY_VERSION_HEX >= 0x02070000
    ptr = PyCapsule_GetPointer(ob, 0);
#else
    ptr = PyCObject_AsVoidPtr(ob);
#endif
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* SetVTable */
static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_NCP_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        (void) PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_srcfile = NULL;
    py_srcfile = PyString_FromString(filename);
    if (!py_srcfile) goto bad;
    #endif
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        if (!py_funcname) goto bad;
        #endif
    }
    #if PY_MAJOR_VERSION < 3
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    #else
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    #endif
    Py_XDECREF(py_funcname);  // XDECREF since it's only set on Py3 if cline
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_srcfile);
    #endif
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* Declarations */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return ::std::complex< float >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return x + y*(__pyx_t_float_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      __pyx_t_float_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabsf(b.real) >= fabsf(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                float r = b.imag / b.real;
                float s = (float)(1.0) / (b.real + b.imag * r);
                return __pyx_t_float_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            float r = b.real / b.imag;
            float s = (float)(1.0) / (b.imag + b.real * r);
            return __pyx_t_float_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            float denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_float_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrtf(z.real*z.real + z.imag*z.imag);
          #else
            return hypotf(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
            __pyx_t_float_complex z;
            float r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    float denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_float(a, a);
                    case 3:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, a);
                    case 4:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if (b.imag == 0) {
                    z.real = powf(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2f(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_float(a);
                theta = atan2f(a.imag, a.real);
            }
            lnr = logf(r);
            z_r = expf(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cosf(z_theta);
            z.imag = z_r * sinf(z_theta);
            return z;
        }
    #endif
#endif

/* Declarations */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return ::std::complex< double >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return x + y*(__pyx_t_double_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      __pyx_t_double_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabs(b.real) >= fabs(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                double r = b.imag / b.real;
                double s = (double)(1.0) / (b.real + b.imag * r);
                return __pyx_t_double_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            double r = b.real / b.imag;
            double s = (double)(1.0) / (b.imag + b.real * r);
            return __pyx_t_double_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            double denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_double_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrt(z.real*z.real + z.imag*z.imag);
          #else
            return hypot(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
            __pyx_t_double_complex z;
            double r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    double denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_double(a, a);
                    case 3:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, a);
                    case 4:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if (b.imag == 0) {
                    z.real = pow(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_double(a);
                theta = atan2(a.imag, a.real);
            }
            lnr = log(r);
            z_r = exp(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cos(z_theta);
            z.imag = z_r * sin(z_theta);
            return z;
        }
    #endif
#endif

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
static CYTHON_INLINE hsize_t __Pyx_PyInt_As_hsize_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const hsize_t neg_one = (hsize_t) -1, const_zero = (hsize_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(hsize_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(hsize_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (hsize_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (hsize_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(hsize_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(hsize_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hsize_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hsize_t) >= 2 * PyLong_SHIFT) {
                            return (hsize_t) (((((hsize_t)digits[1]) << PyLong_SHIFT) | (hsize_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(hsize_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hsize_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hsize_t) >= 3 * PyLong_SHIFT) {
                            return (hsize_t) (((((((hsize_t)digits[2]) << PyLong_SHIFT) | (hsize_t)digits[1]) << PyLong_SHIFT) | (hsize_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(hsize_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hsize_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hsize_t) >= 4 * PyLong_SHIFT) {
                            return (hsize_t) (((((((((hsize_t)digits[3]) << PyLong_SHIFT) | (hsize_t)digits[2]) << PyLong_SHIFT) | (hsize_t)digits[1]) << PyLong_SHIFT) | (hsize_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (hsize_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(hsize_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(hsize_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(hsize_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(hsize_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (hsize_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(hsize_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(hsize_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(hsize_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hsize_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hsize_t) - 1 > 2 * PyLong_SHIFT) {
                            return (hsize_t) (((hsize_t)-1)*(((((hsize_t)digits[1]) << PyLong_SHIFT) | (hsize_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(hsize_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hsize_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hsize_t) - 1 > 2 * PyLong_SHIFT) {
                            return (hsize_t) ((((((hsize_t)digits[1]) << PyLong_SHIFT) | (hsize_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(hsize_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hsize_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hsize_t) - 1 > 3 * PyLong_SHIFT) {
                            return (hsize_t) (((hsize_t)-1)*(((((((hsize_t)digits[2]) << PyLong_SHIFT) | (hsize_t)digits[1]) << PyLong_SHIFT) | (hsize_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(hsize_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hsize_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hsize_t) - 1 > 3 * PyLong_SHIFT) {
                            return (hsize_t) ((((((((hsize_t)digits[2]) << PyLong_SHIFT) | (hsize_t)digits[1]) << PyLong_SHIFT) | (hsize_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(hsize_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hsize_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hsize_t) - 1 > 4 * PyLong_SHIFT) {
                            return (hsize_t) (((hsize_t)-1)*(((((((((hsize_t)digits[3]) << PyLong_SHIFT) | (hsize_t)digits[2]) << PyLong_SHIFT) | (hsize_t)digits[1]) << PyLong_SHIFT) | (hsize_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(hsize_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hsize_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hsize_t) - 1 > 4 * PyLong_SHIFT) {
                            return (hsize_t) ((((((((((hsize_t)digits[3]) << PyLong_SHIFT) | (hsize_t)digits[2]) << PyLong_SHIFT) | (hsize_t)digits[1]) << PyLong_SHIFT) | (hsize_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(hsize_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(hsize_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(hsize_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(hsize_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            hsize_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (hsize_t) -1;
        }
    } else {
        hsize_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (hsize_t) -1;
        val = __Pyx_PyInt_As_hsize_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to hsize_t");
    return (hsize_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to hsize_t");
    return (hsize_t) -1;
}

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
static CYTHON_INLINE npy_uint32 __Pyx_PyInt_As_npy_uint32(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const npy_uint32 neg_one = (npy_uint32) -1, const_zero = (npy_uint32) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(npy_uint32) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(npy_uint32, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (npy_uint32) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (npy_uint32) 0;
                case  1: __PYX_VERIFY_RETURN_INT(npy_uint32, digit, digits[0])
                case 2:
                    if (8 * sizeof(npy_uint32) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_uint32, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_uint32) >= 2 * PyLong_SHIFT) {
                            return (npy_uint32) (((((npy_uint32)digits[1]) << PyLong_SHIFT) | (npy_uint32)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(npy_uint32) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_uint32, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_uint32) >= 3 * PyLong_SHIFT) {
                            return (npy_uint32) (((((((npy_uint32)digits[2]) << PyLong_SHIFT) | (npy_uint32)digits[1]) << PyLong_SHIFT) | (npy_uint32)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(npy_uint32) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_uint32, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_uint32) >= 4 * PyLong_SHIFT) {
                            return (npy_uint32) (((((((((npy_uint32)digits[3]) << PyLong_SHIFT) | (npy_uint32)digits[2]) << PyLong_SHIFT) | (npy_uint32)digits[1]) << PyLong_SHIFT) | (npy_uint32)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (npy_uint32) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(npy_uint32) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(npy_uint32, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(npy_uint32) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(npy_uint32, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (npy_uint32) 0;
                case -1: __PYX_VERIFY_RETURN_INT(npy_uint32, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(npy_uint32,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(npy_uint32) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_uint32, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_uint32) - 1 > 2 * PyLong_SHIFT) {
                            return (npy_uint32) (((npy_uint32)-1)*(((((npy_uint32)digits[1]) << PyLong_SHIFT) | (npy_uint32)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(npy_uint32) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_uint32, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_uint32) - 1 > 2 * PyLong_SHIFT) {
                            return (npy_uint32) ((((((npy_uint32)digits[1]) << PyLong_SHIFT) | (npy_uint32)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(npy_uint32) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_uint32, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_uint32) - 1 > 3 * PyLong_SHIFT) {
                            return (npy_uint32) (((npy_uint32)-1)*(((((((npy_uint32)digits[2]) << PyLong_SHIFT) | (npy_uint32)digits[1]) << PyLong_SHIFT) | (npy_uint32)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(npy_uint32) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_uint32, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_uint32) - 1 > 3 * PyLong_SHIFT) {
                            return (npy_uint32) ((((((((npy_uint32)digits[2]) << PyLong_SHIFT) | (npy_uint32)digits[1]) << PyLong_SHIFT) | (npy_uint32)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(npy_uint32) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_uint32, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_uint32) - 1 > 4 * PyLong_SHIFT) {
                            return (npy_uint32) (((npy_uint32)-1)*(((((((((npy_uint32)digits[3]) << PyLong_SHIFT) | (npy_uint32)digits[2]) << PyLong_SHIFT) | (npy_uint32)digits[1]) << PyLong_SHIFT) | (npy_uint32)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(npy_uint32) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_uint32, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_uint32) - 1 > 4 * PyLong_SHIFT) {
                            return (npy_uint32) ((((((((((npy_uint32)digits[3]) << PyLong_SHIFT) | (npy_uint32)digits[2]) << PyLong_SHIFT) | (npy_uint32)digits[1]) << PyLong_SHIFT) | (npy_uint32)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(npy_uint32) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(npy_uint32, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(npy_uint32) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(npy_uint32, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            npy_uint32 val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (npy_uint32) -1;
        }
    } else {
        npy_uint32 val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (npy_uint32) -1;
        val = __Pyx_PyInt_As_npy_uint32(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to npy_uint32");
    return (npy_uint32) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to npy_uint32");
    return (npy_uint32) -1;
}

/* CIntFromPy */
static CYTHON_INLINE npy_int64 __Pyx_PyInt_As_npy_int64(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const npy_int64 neg_one = (npy_int64) -1, const_zero = (npy_int64) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(npy_int64) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(npy_int64, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (npy_int64) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (npy_int64) 0;
                case  1: __PYX_VERIFY_RETURN_INT(npy_int64, digit, digits[0])
                case 2:
                    if (8 * sizeof(npy_int64) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_int64, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_int64) >= 2 * PyLong_SHIFT) {
                            return (npy_int64) (((((npy_int64)digits[1]) << PyLong_SHIFT) | (npy_int64)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(npy_int64) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_int64, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_int64) >= 3 * PyLong_SHIFT) {
                            return (npy_int64) (((((((npy_int64)digits[2]) << PyLong_SHIFT) | (npy_int64)digits[1]) << PyLong_SHIFT) | (npy_int64)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(npy_int64) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_int64, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_int64) >= 4 * PyLong_SHIFT) {
                            return (npy_int64) (((((((((npy_int64)digits[3]) << PyLong_SHIFT) | (npy_int64)digits[2]) << PyLong_SHIFT) | (npy_int64)digits[1]) << PyLong_SHIFT) | (npy_int64)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (npy_int64) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(npy_int64) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(npy_int64, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(npy_int64) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(npy_int64, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (npy_int64) 0;
                case -1: __PYX_VERIFY_RETURN_INT(npy_int64, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(npy_int64,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(npy_int64) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_int64, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_int64) - 1 > 2 * PyLong_SHIFT) {
                            return (npy_int64) (((npy_int64)-1)*(((((npy_int64)digits[1]) << PyLong_SHIFT) | (npy_int64)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(npy_int64) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_int64, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_int64) - 1 > 2 * PyLong_SHIFT) {
                            return (npy_int64) ((((((npy_int64)digits[1]) << PyLong_SHIFT) | (npy_int64)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(npy_int64) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_int64, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_int64) - 1 > 3 * PyLong_SHIFT) {
                            return (npy_int64) (((npy_int64)-1)*(((((((npy_int64)digits[2]) << PyLong_SHIFT) | (npy_int64)digits[1]) << PyLong_SHIFT) | (npy_int64)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(npy_int64) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_int64, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_int64) - 1 > 3 * PyLong_SHIFT) {
                            return (npy_int64) ((((((((npy_int64)digits[2]) << PyLong_SHIFT) | (npy_int64)digits[1]) << PyLong_SHIFT) | (npy_int64)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(npy_int64) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_int64, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_int64) - 1 > 4 * PyLong_SHIFT) {
                            return (npy_int64) (((npy_int64)-1)*(((((((((npy_int64)digits[3]) << PyLong_SHIFT) | (npy_int64)digits[2]) << PyLong_SHIFT) | (npy_int64)digits[1]) << PyLong_SHIFT) | (npy_int64)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(npy_int64) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_int64, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_int64) - 1 > 4 * PyLong_SHIFT) {
                            return (npy_int64) ((((((((((npy_int64)digits[3]) << PyLong_SHIFT) | (npy_int64)digits[2]) << PyLong_SHIFT) | (npy_int64)digits[1]) << PyLong_SHIFT) | (npy_int64)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(npy_int64) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(npy_int64, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(npy_int64) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(npy_int64, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            npy_int64 val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (npy_int64) -1;
        }
    } else {
        npy_int64 val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (npy_int64) -1;
        val = __Pyx_PyInt_As_npy_int64(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to npy_int64");
    return (npy_int64) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to npy_int64");
    return (npy_int64) -1;
}

/* CIntFromPy */
static CYTHON_INLINE npy_uint64 __Pyx_PyInt_As_npy_uint64(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const npy_uint64 neg_one = (npy_uint64) -1, const_zero = (npy_uint64) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(npy_uint64) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(npy_uint64, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (npy_uint64) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (npy_uint64) 0;
                case  1: __PYX_VERIFY_RETURN_INT(npy_uint64, digit, digits[0])
                case 2:
                    if (8 * sizeof(npy_uint64) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_uint64, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_uint64) >= 2 * PyLong_SHIFT) {
                            return (npy_uint64) (((((npy_uint64)digits[1]) << PyLong_SHIFT) | (npy_uint64)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(npy_uint64) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_uint64, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_uint64) >= 3 * PyLong_SHIFT) {
                            return (npy_uint64) (((((((npy_uint64)digits[2]) << PyLong_SHIFT) | (npy_uint64)digits[1]) << PyLong_SHIFT) | (npy_uint64)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(npy_uint64) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_uint64, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_uint64) >= 4 * PyLong_SHIFT) {
                            return (npy_uint64) (((((((((npy_uint64)digits[3]) << PyLong_SHIFT) | (npy_uint64)digits[2]) << PyLong_SHIFT) | (npy_uint64)digits[1]) << PyLong_SHIFT) | (npy_uint64)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (npy_uint64) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(npy_uint64) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(npy_uint64, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(npy_uint64) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(npy_uint64, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (npy_uint64) 0;
                case -1: __PYX_VERIFY_RETURN_INT(npy_uint64, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(npy_uint64,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(npy_uint64) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_uint64, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_uint64) - 1 > 2 * PyLong_SHIFT) {
                            return (npy_uint64) (((npy_uint64)-1)*(((((npy_uint64)digits[1]) << PyLong_SHIFT) | (npy_uint64)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(npy_uint64) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_uint64, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_uint64) - 1 > 2 * PyLong_SHIFT) {
                            return (npy_uint64) ((((((npy_uint64)digits[1]) << PyLong_SHIFT) | (npy_uint64)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(npy_uint64) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_uint64, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_uint64) - 1 > 3 * PyLong_SHIFT) {
                            return (npy_uint64) (((npy_uint64)-1)*(((((((npy_uint64)digits[2]) << PyLong_SHIFT) | (npy_uint64)digits[1]) << PyLong_SHIFT) | (npy_uint64)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(npy_uint64) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_uint64, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_uint64) - 1 > 3 * PyLong_SHIFT) {
                            return (npy_uint64) ((((((((npy_uint64)digits[2]) << PyLong_SHIFT) | (npy_uint64)digits[1]) << PyLong_SHIFT) | (npy_uint64)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(npy_uint64) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_uint64, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_uint64) - 1 > 4 * PyLong_SHIFT) {
                            return (npy_uint64) (((npy_uint64)-1)*(((((((((npy_uint64)digits[3]) << PyLong_SHIFT) | (npy_uint64)digits[2]) << PyLong_SHIFT) | (npy_uint64)digits[1]) << PyLong_SHIFT) | (npy_uint64)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(npy_uint64) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_uint64, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_uint64) - 1 > 4 * PyLong_SHIFT) {
                            return (npy_uint64) ((((((((((npy_uint64)digits[3]) << PyLong_SHIFT) | (npy_uint64)digits[2]) << PyLong_SHIFT) | (npy_uint64)digits[1]) << PyLong_SHIFT) | (npy_uint64)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(npy_uint64) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(npy_uint64, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(npy_uint64) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(npy_uint64, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            npy_uint64 val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (npy_uint64) -1;
        }
    } else {
        npy_uint64 val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (npy_uint64) -1;
        val = __Pyx_PyInt_As_npy_uint64(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to npy_uint64");
    return (npy_uint64) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to npy_uint64");
    return (npy_uint64) -1;
}

/* CIntFromPy */
static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const size_t neg_one = (size_t) -1, const_zero = (size_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(size_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(size_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (size_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(size_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 2 * PyLong_SHIFT) {
                            return (size_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 3 * PyLong_SHIFT) {
                            return (size_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 4 * PyLong_SHIFT) {
                            return (size_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (size_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(size_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(size_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(size_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(size_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) ((((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) ((((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) ((((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(size_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            size_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (size_t) -1;
        }
    } else {
        size_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (size_t) -1;
        val = __Pyx_PyInt_As_size_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to size_t");
    return (size_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to size_t");
    return (size_t) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* InitStrings */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
#if PY_MAJOR_VERSION < 3
  } else if (likely(PyInt_CheckExact(o))) {
    return PyInt_AS_LONG(o);
#endif
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyInt_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
