/* Generated by Cython 0.29.26 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": [],
        "name": "tables.tableextension",
        "sources": [
            "tables/tableextension.pyx"
        ]
    },
    "module_name": "tables.tableextension"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_26"
#define CYTHON_HEX_VERSION 0x001D1AF0
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0 || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL (PY_VERSION_HEX < 0x030B00A1)
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS (PY_VERSION_HEX >= 0x030600B1)
  #endif
  #ifndef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_MAJOR_VERSION < 3
    #include "longintrepr.h"
  #endif
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_DefaultClassType PyType_Type
#if PY_VERSION_HEX >= 0x030B00A1
    static CYTHON_INLINE PyCodeObject* __Pyx_PyCode_New(int a, int k, int l, int s, int f,
                                                    PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                                    PyObject *fv, PyObject *cell, PyObject* fn,
                                                    PyObject *name, int fline, PyObject *lnos) {
        PyObject *kwds=NULL, *argcount=NULL, *posonlyargcount=NULL, *kwonlyargcount=NULL;
        PyObject *nlocals=NULL, *stacksize=NULL, *flags=NULL, *replace=NULL, *call_result=NULL, *empty=NULL;
        const char *fn_cstr=NULL;
        const char *name_cstr=NULL;
        PyCodeObject* co=NULL;
        PyObject *type, *value, *traceback;
        PyErr_Fetch(&type, &value, &traceback);
        if (!(kwds=PyDict_New())) goto end;
        if (!(argcount=PyLong_FromLong(a))) goto end;
        if (PyDict_SetItemString(kwds, "co_argcount", argcount) != 0) goto end;
        if (!(posonlyargcount=PyLong_FromLong(0))) goto end;
        if (PyDict_SetItemString(kwds, "co_posonlyargcount", posonlyargcount) != 0) goto end;
        if (!(kwonlyargcount=PyLong_FromLong(k))) goto end;
        if (PyDict_SetItemString(kwds, "co_kwonlyargcount", kwonlyargcount) != 0) goto end;
        if (!(nlocals=PyLong_FromLong(l))) goto end;
        if (PyDict_SetItemString(kwds, "co_nlocals", nlocals) != 0) goto end;
        if (!(stacksize=PyLong_FromLong(s))) goto end;
        if (PyDict_SetItemString(kwds, "co_stacksize", stacksize) != 0) goto end;
        if (!(flags=PyLong_FromLong(f))) goto end;
        if (PyDict_SetItemString(kwds, "co_flags", flags) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_code", code) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_consts", c) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_names", n) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_varnames", v) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_freevars", fv) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_cellvars", cell) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_linetable", lnos) != 0) goto end;
        if (!(fn_cstr=PyUnicode_AsUTF8AndSize(fn, NULL))) goto end;
        if (!(name_cstr=PyUnicode_AsUTF8AndSize(name, NULL))) goto end;
        if (!(co = PyCode_NewEmpty(fn_cstr, name_cstr, fline))) goto end;
        if (!(replace = PyObject_GetAttrString((PyObject*)co, "replace"))) goto cleanup_code_too;
        if (!(empty = PyTuple_New(0))) goto cleanup_code_too; // unfortunately __pyx_empty_tuple isn't available here
        if (!(call_result = PyObject_Call(replace, empty, kwds))) goto cleanup_code_too;
        Py_XDECREF((PyObject*)co);
        co = (PyCodeObject*)call_result;
        call_result = NULL;
        if (0) {
            cleanup_code_too:
            Py_XDECREF((PyObject*)co);
            co = NULL;
        }
        end:
        Py_XDECREF(kwds);
        Py_XDECREF(argcount);
        Py_XDECREF(posonlyargcount);
        Py_XDECREF(kwonlyargcount);
        Py_XDECREF(nlocals);
        Py_XDECREF(stacksize);
        Py_XDECREF(replace);
        Py_XDECREF(call_result);
        Py_XDECREF(empty);
        if (type) {
            PyErr_Restore(type, value, traceback);
        }
        return co;
    }
#else
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #if defined(PyUnicode_IS_READY)
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #else
  #define __Pyx_PyUnicode_READY(op)       (0)
  #endif
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #if defined(PyUnicode_IS_READY) && defined(PyUnicode_GET_SIZE)
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
  #endif
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsHash_t
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? ((void)(klass), PyMethod_New(func, self)) : __Pyx_NewRef(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__; (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__tables__tableextension
#define __PYX_HAVE_API__tables__tableextension
/* Early includes */
#include "time.h"
#include <string.h>
#include <stdio.h>
#include "numpy/arrayobject.h"
#include "numpy/ndarrayobject.h"
#include "numpy/ndarraytypes.h"
#include "numpy/arrayscalars.h"
#include "numpy/ufuncobject.h"

    /* NumPy API declarations from "numpy/__init__.pxd" */
    
#include "hdf5.h"
#include "H5ATTR.h"
#include "H5ARRAY.h"
#include "utils.h"
#include "typeconv.h"
#include "blosc_filter.h"
#include "pythread.h"
#include <stdlib.h>
#include "H5TB-opt.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;

/* Header.proto */
#if !defined(CYTHON_CCOMPLEX)
  #if defined(__cplusplus)
    #define CYTHON_CCOMPLEX 1
  #elif defined(_Complex_I)
    #define CYTHON_CCOMPLEX 1
  #else
    #define CYTHON_CCOMPLEX 0
  #endif
#endif
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #include <complex>
  #else
    #include <complex.h>
  #endif
#endif
#if CYTHON_CCOMPLEX && !defined(__cplusplus) && defined(__sun__) && defined(__GNUC__)
  #undef _Complex_I
  #define _Complex_I 1.0fj
#endif


static const char *__pyx_f[] = {
  "tables/tableextension.pyx",
  "stringsource",
  "__init__.pxd",
  "type.pxd",
  "bool.pxd",
  "complex.pxd",
  "tables/hdf5extension.pxd",
  "tables/lrucacheextension.pxd",
};
/* NoFastGil.proto */
#define __Pyx_PyGILState_Ensure PyGILState_Ensure
#define __Pyx_PyGILState_Release PyGILState_Release
#define __Pyx_FastGIL_Remember()
#define __Pyx_FastGIL_Forget()
#define __Pyx_FastGilFuncInit()

/* ForceInitThreads.proto */
#ifndef __PYX_FORCE_INIT_THREADS
  #define __PYX_FORCE_INIT_THREADS 0
#endif


/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":690
 * # in Cython to enable them only on the right systems.
 * 
 * ctypedef npy_int8       int8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 */
typedef npy_int8 __pyx_t_5numpy_int8_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":691
 * 
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t
 */
typedef npy_int16 __pyx_t_5numpy_int16_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":692
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int64      int64_t
 * #ctypedef npy_int96      int96_t
 */
typedef npy_int32 __pyx_t_5numpy_int32_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":693
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_int96      int96_t
 * #ctypedef npy_int128     int128_t
 */
typedef npy_int64 __pyx_t_5numpy_int64_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":697
 * #ctypedef npy_int128     int128_t
 * 
 * ctypedef npy_uint8      uint8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 */
typedef npy_uint8 __pyx_t_5numpy_uint8_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":698
 * 
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t
 */
typedef npy_uint16 __pyx_t_5numpy_uint16_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":699
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint64     uint64_t
 * #ctypedef npy_uint96     uint96_t
 */
typedef npy_uint32 __pyx_t_5numpy_uint32_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":700
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_uint96     uint96_t
 * #ctypedef npy_uint128    uint128_t
 */
typedef npy_uint64 __pyx_t_5numpy_uint64_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":704
 * #ctypedef npy_uint128    uint128_t
 * 
 * ctypedef npy_float32    float32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_float64    float64_t
 * #ctypedef npy_float80    float80_t
 */
typedef npy_float32 __pyx_t_5numpy_float32_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":705
 * 
 * ctypedef npy_float32    float32_t
 * ctypedef npy_float64    float64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_float80    float80_t
 * #ctypedef npy_float128   float128_t
 */
typedef npy_float64 __pyx_t_5numpy_float64_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":714
 * # The int types are mapped a bit surprising --
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t
 */
typedef npy_long __pyx_t_5numpy_int_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":715
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   longlong_t
 * 
 */
typedef npy_longlong __pyx_t_5numpy_long_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":716
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_ulong      uint_t
 */
typedef npy_longlong __pyx_t_5numpy_longlong_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":718
 * ctypedef npy_longlong   longlong_t
 * 
 * ctypedef npy_ulong      uint_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t
 */
typedef npy_ulong __pyx_t_5numpy_uint_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":719
 * 
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 */
typedef npy_ulonglong __pyx_t_5numpy_ulong_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":720
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_intp       intp_t
 */
typedef npy_ulonglong __pyx_t_5numpy_ulonglong_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":722
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 * ctypedef npy_intp       intp_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uintp      uintp_t
 * 
 */
typedef npy_intp __pyx_t_5numpy_intp_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":723
 * 
 * ctypedef npy_intp       intp_t
 * ctypedef npy_uintp      uintp_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_double     float_t
 */
typedef npy_uintp __pyx_t_5numpy_uintp_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":725
 * ctypedef npy_uintp      uintp_t
 * 
 * ctypedef npy_double     float_t             # <<<<<<<<<<<<<<
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t
 */
typedef npy_double __pyx_t_5numpy_float_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":726
 * 
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longdouble longdouble_t
 * 
 */
typedef npy_double __pyx_t_5numpy_double_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":727
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cfloat      cfloat_t
 */
typedef npy_longdouble __pyx_t_5numpy_longdouble_t;
/* Declarations.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< float > __pyx_t_float_complex;
  #else
    typedef float _Complex __pyx_t_float_complex;
  #endif
#else
    typedef struct { float real, imag; } __pyx_t_float_complex;
#endif
static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float, float);

/* Declarations.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< double > __pyx_t_double_complex;
  #else
    typedef double _Complex __pyx_t_double_complex;
  #endif
#else
    typedef struct { double real, imag; } __pyx_t_double_complex;
#endif
static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double, double);


/*--- Type declarations ---*/
struct __pyx_obj_6tables_13hdf5extension_Node;
struct __pyx_obj_6tables_13hdf5extension_Leaf;
struct __pyx_obj_6tables_13hdf5extension_Array;
struct __pyx_obj_6tables_17lrucacheextension_NodeCache;
struct __pyx_obj_6tables_17lrucacheextension_BaseCache;
struct __pyx_obj_6tables_17lrucacheextension_ObjectNode;
struct __pyx_obj_6tables_17lrucacheextension_ObjectCache;
struct __pyx_obj_6tables_17lrucacheextension_NumCache;
struct __pyx_obj_6tables_14tableextension_Table;
struct __pyx_obj_6tables_14tableextension_Row;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":729
 * ctypedef npy_longdouble longdouble_t
 * 
 * ctypedef npy_cfloat      cfloat_t             # <<<<<<<<<<<<<<
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t
 */
typedef npy_cfloat __pyx_t_5numpy_cfloat_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":730
 * 
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t             # <<<<<<<<<<<<<<
 * ctypedef npy_clongdouble clongdouble_t
 * 
 */
typedef npy_cdouble __pyx_t_5numpy_cdouble_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":731
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cdouble     complex_t
 */
typedef npy_clongdouble __pyx_t_5numpy_clongdouble_t;

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":733
 * ctypedef npy_clongdouble clongdouble_t
 * 
 * ctypedef npy_cdouble     complex_t             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 */
typedef npy_cdouble __pyx_t_5numpy_complex_t;

/* "hdf5extension.pxd":16
 * 
 * # Declaration of instance variables for shared classes
 * cdef class Node:             # <<<<<<<<<<<<<<
 *   cdef object name
 *   cdef hid_t  parent_id
 */
struct __pyx_obj_6tables_13hdf5extension_Node {
  PyObject_HEAD
  PyObject *name;
  hid_t parent_id;
};


/* "hdf5extension.pxd":20
 *   cdef hid_t  parent_id
 * 
 * cdef class Leaf(Node):             # <<<<<<<<<<<<<<
 *   cdef hid_t   dataset_id
 *   cdef hid_t   type_id
 */
struct __pyx_obj_6tables_13hdf5extension_Leaf {
  struct __pyx_obj_6tables_13hdf5extension_Node __pyx_base;
  struct __pyx_vtabstruct_6tables_13hdf5extension_Leaf *__pyx_vtab;
  hid_t dataset_id;
  hid_t type_id;
  hid_t base_type_id;
  hid_t disk_type_id;
  hsize_t *dims;
};


/* "hdf5extension.pxd":29
 *   cdef _convert_time64(self, ndarray nparr, int sense)
 * 
 * cdef class Array(Leaf):             # <<<<<<<<<<<<<<
 *   cdef int      rank
 *   cdef hsize_t *maxdims
 */
struct __pyx_obj_6tables_13hdf5extension_Array {
  struct __pyx_obj_6tables_13hdf5extension_Leaf __pyx_base;
  int rank;
  hsize_t *maxdims;
  hsize_t *dims_chunk;
};


/* "lrucacheextension.pxd":15
 * # Declaration of instance variables for shared classes
 * # The NodeCache class is useful for caching general objects (like Nodes).
 * cdef class NodeCache:             # <<<<<<<<<<<<<<
 *   cdef readonly long nslots
 *   cdef long nextslot
 */
struct __pyx_obj_6tables_17lrucacheextension_NodeCache {
  PyObject_HEAD
  struct __pyx_vtabstruct_6tables_17lrucacheextension_NodeCache *__pyx_vtab;
  long nslots;
  long nextslot;
  PyObject *nodes;
  PyObject *paths;
};


/* "lrucacheextension.pxd":25
 * 
 * # Base class for other caches
 * cdef class BaseCache:             # <<<<<<<<<<<<<<
 *   cdef int iscachedisabled, incsetcount
 *   cdef long setcount, getcount, containscount
 */
struct __pyx_obj_6tables_17lrucacheextension_BaseCache {
  PyObject_HEAD
  struct __pyx_vtabstruct_6tables_17lrucacheextension_BaseCache *__pyx_vtab;
  int iscachedisabled;
  int incsetcount;
  long setcount;
  long getcount;
  long containscount;
  long disablecyclecount;
  long disableeverycycles;
  long enablecyclecount;
  long enableeverycycles;
  double nprobes;
  double hitratio;
  long seqn_;
  long nextslot;
  long nslots;
  long *ratimes;
  double lowesthr;
  PyArrayObject *atimes;
  PyObject *name;
};


/* "lrucacheextension.pxd":42
 * 
 * #  Helper class for ObjectCache
 * cdef class ObjectNode:             # <<<<<<<<<<<<<<
 *   cdef object key, obj
 *   cdef long nslot
 */
struct __pyx_obj_6tables_17lrucacheextension_ObjectNode {
  PyObject_HEAD
  PyObject *key;
  PyObject *obj;
  long nslot;
};


/* "lrucacheextension.pxd":48
 * 
 * # The ObjectCache class is useful for general python objects
 * cdef class ObjectCache(BaseCache):             # <<<<<<<<<<<<<<
 *   cdef long maxcachesize, cachesize, maxobjsize
 *   cdef long *rsizes
 */
struct __pyx_obj_6tables_17lrucacheextension_ObjectCache {
  struct __pyx_obj_6tables_17lrucacheextension_BaseCache __pyx_base;
  long maxcachesize;
  long cachesize;
  long maxobjsize;
  long *rsizes;
  PyArrayObject *sizes;
  PyObject *__pyx___list;
  PyObject *__pyx___dict;
  struct __pyx_obj_6tables_17lrucacheextension_ObjectNode *mrunode;
};


/* "lrucacheextension.pxd":63
 * 
 * # The NumCache class is useful for caching numerical data in an efficient way
 * cdef class NumCache(BaseCache):             # <<<<<<<<<<<<<<
 *   cdef long itemsize, slotsize
 *   cdef ndarray cacheobj, keys
 */
struct __pyx_obj_6tables_17lrucacheextension_NumCache {
  struct __pyx_obj_6tables_17lrucacheextension_BaseCache __pyx_base;
  long itemsize;
  long slotsize;
  PyArrayObject *cacheobj;
  PyArrayObject *keys;
  void *rcache;
  PY_LONG_LONG *rkeys;
  PyObject *__pyx___dict;
};


/* "tables/tableextension.pyx":145
 * # Public classes
 * 
 * cdef class Table(Leaf):             # <<<<<<<<<<<<<<
 *   # instance variables
 *   cdef void     *wbuf
 */
struct __pyx_obj_6tables_14tableextension_Table {
  struct __pyx_obj_6tables_13hdf5extension_Leaf __pyx_base;
  void *wbuf;
};


/* "tables/tableextension.pyx":686
 *     return nrecords
 * 
 * cdef class Row:             # <<<<<<<<<<<<<<
 *   """Table row iterator and field accessor.
 * 
 */
struct __pyx_obj_6tables_14tableextension_Row {
  PyObject_HEAD
  struct __pyx_vtabstruct_6tables_14tableextension_Row *__pyx_vtab;
  npy_intp _stride;
  long _row;
  long _unsaved_nrows;
  long _mod_nrows;
  hsize_t start;
  hsize_t absstep;
  PY_LONG_LONG stop;
  PY_LONG_LONG step;
  PY_LONG_LONG nextelement;
  PY_LONG_LONG _nrow;
  PY_LONG_LONG stopb;
  hsize_t nrowsinbuf;
  hsize_t nrows;
  hsize_t nrowsread;
  hsize_t chunksize;
  hsize_t nchunksinbuf;
  hsize_t totalchunks;
  hsize_t startb;
  hsize_t lenbuf;
  PY_LONG_LONG indexchunk;
  int bufcounter;
  int counter;
  int exist_enum_cols;
  int _riterator;
  int _rowsize;
  int _write_to_seqcache;
  int wherecond;
  int indexed;
  int ro_filemode;
  int chunked;
  int _bufferinfo_done;
  int sss_on;
  int iterseq_max_elements;
  PyArrayObject *bufcoords;
  PyArrayObject *indexvalid;
  PyArrayObject *indexvalues;
  PyArrayObject *chunkmap;
  hsize_t *bufcoords_data;
  hsize_t *index_values_data;
  char *chunkmap_data;
  char *index_valid_data;
  PyObject *dtype;
  PyObject *iobuf;
  PyObject *iobufcpy;
  PyObject *wrec;
  PyObject *wreccpy;
  PyObject *wfields;
  PyObject *rfields;
  PyObject *coords;
  PyObject *condfunc;
  PyObject *condargs;
  PyObject *condkwargs;
  PyObject *mod_elements;
  PyObject *colenums;
  PyObject *rfieldscache;
  PyObject *wfieldscache;
  PyObject *iterseq;
  PyObject *_table_file;
  PyObject *_table_path;
  PyObject *modified_fields;
  PyObject *seqcache_key;
};



/* "hdf5extension.pxd":20
 *   cdef hid_t  parent_id
 * 
 * cdef class Leaf(Node):             # <<<<<<<<<<<<<<
 *   cdef hid_t   dataset_id
 *   cdef hid_t   type_id
 */

struct __pyx_vtabstruct_6tables_13hdf5extension_Leaf {
  PyObject *(*_get_type_ids)(struct __pyx_obj_6tables_13hdf5extension_Leaf *);
  PyObject *(*_convert_time64)(struct __pyx_obj_6tables_13hdf5extension_Leaf *, PyArrayObject *, int);
};
static struct __pyx_vtabstruct_6tables_13hdf5extension_Leaf *__pyx_vtabptr_6tables_13hdf5extension_Leaf;


/* "hdf5extension.pxd":29
 *   cdef _convert_time64(self, ndarray nparr, int sense)
 * 
 * cdef class Array(Leaf):             # <<<<<<<<<<<<<<
 *   cdef int      rank
 *   cdef hsize_t *maxdims
 */

struct __pyx_vtabstruct_6tables_13hdf5extension_Array {
  struct __pyx_vtabstruct_6tables_13hdf5extension_Leaf __pyx_base;
};
static struct __pyx_vtabstruct_6tables_13hdf5extension_Array *__pyx_vtabptr_6tables_13hdf5extension_Array;


/* "lrucacheextension.pxd":15
 * # Declaration of instance variables for shared classes
 * # The NodeCache class is useful for caching general objects (like Nodes).
 * cdef class NodeCache:             # <<<<<<<<<<<<<<
 *   cdef readonly long nslots
 *   cdef long nextslot
 */

struct __pyx_vtabstruct_6tables_17lrucacheextension_NodeCache {
  PyObject *(*setitem)(struct __pyx_obj_6tables_17lrucacheextension_NodeCache *, PyObject *, PyObject *);
  long (*getslot)(struct __pyx_obj_6tables_17lrucacheextension_NodeCache *, PyObject *);
  PyObject *(*cpop)(struct __pyx_obj_6tables_17lrucacheextension_NodeCache *, PyObject *);
};
static struct __pyx_vtabstruct_6tables_17lrucacheextension_NodeCache *__pyx_vtabptr_6tables_17lrucacheextension_NodeCache;


/* "lrucacheextension.pxd":25
 * 
 * # Base class for other caches
 * cdef class BaseCache:             # <<<<<<<<<<<<<<
 *   cdef int iscachedisabled, incsetcount
 *   cdef long setcount, getcount, containscount
 */

struct __pyx_vtabstruct_6tables_17lrucacheextension_BaseCache {
  int (*checkhitratio)(struct __pyx_obj_6tables_17lrucacheextension_BaseCache *);
  int (*couldenablecache_)(struct __pyx_obj_6tables_17lrucacheextension_BaseCache *);
  long (*incseqn)(struct __pyx_obj_6tables_17lrucacheextension_BaseCache *);
};
static struct __pyx_vtabstruct_6tables_17lrucacheextension_BaseCache *__pyx_vtabptr_6tables_17lrucacheextension_BaseCache;


/* "lrucacheextension.pxd":48
 * 
 * # The ObjectCache class is useful for general python objects
 * cdef class ObjectCache(BaseCache):             # <<<<<<<<<<<<<<
 *   cdef long maxcachesize, cachesize, maxobjsize
 *   cdef long *rsizes
 */

struct __pyx_vtabstruct_6tables_17lrucacheextension_ObjectCache {
  struct __pyx_vtabstruct_6tables_17lrucacheextension_BaseCache __pyx_base;
  PyObject *(*removeslot_)(struct __pyx_obj_6tables_17lrucacheextension_ObjectCache *, long);
  PyObject *(*clearcache_)(struct __pyx_obj_6tables_17lrucacheextension_ObjectCache *);
  PyObject *(*updateslot_)(struct __pyx_obj_6tables_17lrucacheextension_ObjectCache *, long, long, PyObject *, PyObject *);
  long (*setitem_)(struct __pyx_obj_6tables_17lrucacheextension_ObjectCache *, PyObject *, PyObject *, long);
  long (*getslot_)(struct __pyx_obj_6tables_17lrucacheextension_ObjectCache *, PyObject *);
  PyObject *(*getitem_)(struct __pyx_obj_6tables_17lrucacheextension_ObjectCache *, long);
};
static struct __pyx_vtabstruct_6tables_17lrucacheextension_ObjectCache *__pyx_vtabptr_6tables_17lrucacheextension_ObjectCache;


/* "lrucacheextension.pxd":63
 * 
 * # The NumCache class is useful for caching numerical data in an efficient way
 * cdef class NumCache(BaseCache):             # <<<<<<<<<<<<<<
 *   cdef long itemsize, slotsize
 *   cdef ndarray cacheobj, keys
 */

struct __pyx_vtabstruct_6tables_17lrucacheextension_NumCache {
  struct __pyx_vtabstruct_6tables_17lrucacheextension_BaseCache __pyx_base;
  void *(*getaddrslot_)(struct __pyx_obj_6tables_17lrucacheextension_NumCache *, long);
  long (*setitem_)(struct __pyx_obj_6tables_17lrucacheextension_NumCache *, PY_LONG_LONG, void *, long);
  long (*setitem1_)(struct __pyx_obj_6tables_17lrucacheextension_NumCache *, PY_LONG_LONG);
  long (*getslot_)(struct __pyx_obj_6tables_17lrucacheextension_NumCache *, PY_LONG_LONG);
  PyObject *(*getitem_)(struct __pyx_obj_6tables_17lrucacheextension_NumCache *, long, void *, long);
  void *(*getitem1_)(struct __pyx_obj_6tables_17lrucacheextension_NumCache *, long);
};
static struct __pyx_vtabstruct_6tables_17lrucacheextension_NumCache *__pyx_vtabptr_6tables_17lrucacheextension_NumCache;


/* "tables/tableextension.pyx":145
 * # Public classes
 * 
 * cdef class Table(Leaf):             # <<<<<<<<<<<<<<
 *   # instance variables
 *   cdef void     *wbuf
 */

struct __pyx_vtabstruct_6tables_14tableextension_Table {
  struct __pyx_vtabstruct_6tables_13hdf5extension_Leaf __pyx_base;
  PyObject *(*get_nested_type)(struct __pyx_obj_6tables_14tableextension_Table *, hid_t, hid_t, PyObject *, PyObject *);
  PyObject *(*_convert_time64_)(struct __pyx_obj_6tables_14tableextension_Table *, PyArrayObject *, hsize_t, int);
  PyObject *(*_convert_types)(struct __pyx_obj_6tables_14tableextension_Table *, PyArrayObject *, hsize_t, int, int __pyx_skip_dispatch);
  hsize_t (*_read_chunk)(struct __pyx_obj_6tables_14tableextension_Table *, hsize_t, PyArrayObject *, long);
};
static struct __pyx_vtabstruct_6tables_14tableextension_Table *__pyx_vtabptr_6tables_14tableextension_Table;


/* "tables/tableextension.pyx":686
 *     return nrecords
 * 
 * cdef class Row:             # <<<<<<<<<<<<<<
 *   """Table row iterator and field accessor.
 * 
 */

struct __pyx_vtabstruct_6tables_14tableextension_Row {
  PyObject *(*_new_buffer)(struct __pyx_obj_6tables_14tableextension_Row *, PyObject *);
  PyObject *(*_init_loop)(struct __pyx_obj_6tables_14tableextension_Row *, hsize_t, PY_LONG_LONG, PY_LONG_LONG, PyObject *, PyObject *);
  PyObject *(*__pyx___next__indexed)(struct __pyx_obj_6tables_14tableextension_Row *);
  PyObject *(*__pyx___next__coords)(struct __pyx_obj_6tables_14tableextension_Row *);
  PyObject *(*__pyx___next__inkernel)(struct __pyx_obj_6tables_14tableextension_Row *);
  PyObject *(*__pyx___next__general)(struct __pyx_obj_6tables_14tableextension_Row *);
  PyObject *(*_finish_riterator)(struct __pyx_obj_6tables_14tableextension_Row *);
};
static struct __pyx_vtabstruct_6tables_14tableextension_Row *__pyx_vtabptr_6tables_14tableextension_Row;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
#define __Pyx_GetModuleGlobalNameUncached(var, name)  {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
#if CYTHON_FAST_PYCALL
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif // CYTHON_FAST_PYCALL
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* StrEquals.proto */
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyString_Equals __Pyx_PyUnicode_Equals
#else
#define __Pyx_PyString_Equals __Pyx_PyBytes_Equals
#endif

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* PyObjectCall2Args.proto */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* DictGetItem.proto */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key);
#define __Pyx_PyObject_Dict_GetItem(obj, name)\
    (likely(PyDict_CheckExact(obj)) ?\
     __Pyx_PyDict_GetItem(obj, name) : PyObject_GetItem(obj, name))
#else
#define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#define __Pyx_PyObject_Dict_GetItem(obj, name)  PyObject_GetItem(obj, name)
#endif

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o, n, NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value);
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_AddObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceAdd(op1, op2) : PyNumber_Add(op1, op2))
#endif

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* PyObjectGetMethod.proto */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method);

/* PyObjectCallMethod1.proto */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);

/* append.proto */
static CYTHON_INLINE int __Pyx_PyObject_Append(PyObject* L, PyObject* x);

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* abs_longlong.proto */
static CYTHON_INLINE PY_LONG_LONG __Pyx_abs_longlong(PY_LONG_LONG x) {
#if defined (__cplusplus) && __cplusplus >= 201103L
    return std::abs(x);
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    return llabs(x);
#elif defined (_MSC_VER)
    return _abs64(x);
#elif defined (__GNUC__)
    return __builtin_llabs(x);
#else
    if (sizeof(PY_LONG_LONG) <= sizeof(Py_ssize_t))
        return __Pyx_sst_abs(x);
    return (x<0) ? -x : x;
#endif
}

/* SliceObject.proto */
#define __Pyx_PyObject_DelSlice(obj, cstart, cstop, py_start, py_stop, py_slice, has_cstart, has_cstop, wraparound)\
    __Pyx_PyObject_SetSlice(obj, (PyObject*)NULL, cstart, cstop, py_start, py_stop, py_slice, has_cstart, has_cstop, wraparound)
static CYTHON_INLINE int __Pyx_PyObject_SetSlice(
        PyObject* obj, PyObject* value, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* SliceObject.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(
        PyObject* obj, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* None.proto */
static CYTHON_INLINE PY_LONG_LONG __Pyx_mod_PY_LONG_LONG(PY_LONG_LONG, PY_LONG_LONG);

/* SetItemInt.proto */
#define __Pyx_SetItemInt(o, i, v, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_SetItemInt_Fast(o, (Py_ssize_t)i, v, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list assignment index out of range"), -1) :\
               __Pyx_SetItemInt_Generic(o, to_py_func(i), v)))
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v);
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v,
                                               int is_list, int wraparound, int boundscheck);

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* CallNextTpDealloc.proto */
static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc);

/* CallNextTpTraverse.proto */
static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse);

/* CallNextTpClear.proto */
static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_dealloc);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto
#define __PYX_HAVE_RT_ImportType_proto
enum __Pyx_ImportType_CheckSize {
   __Pyx_ImportType_CheckSize_Error = 0,
   __Pyx_ImportType_CheckSize_Warn = 1,
   __Pyx_ImportType_CheckSize_Ignore = 2
};
static PyTypeObject *__Pyx_ImportType(PyObject* module, const char *module_name, const char *class_name, size_t size, enum __Pyx_ImportType_CheckSize check_size);
#endif

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyObject *dict);

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* GCCDiagnostics.proto */
#if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* RealImag.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #define __Pyx_CREAL(z) ((z).real())
    #define __Pyx_CIMAG(z) ((z).imag())
  #else
    #define __Pyx_CREAL(z) (__real__(z))
    #define __Pyx_CIMAG(z) (__imag__(z))
  #endif
#else
    #define __Pyx_CREAL(z) ((z).real)
    #define __Pyx_CIMAG(z) ((z).imag)
#endif
#if defined(__cplusplus) && CYTHON_CCOMPLEX\
        && (defined(_WIN32) || defined(__clang__) || (defined(__GNUC__) && (__GNUC__ >= 5 || __GNUC__ == 4 && __GNUC_MINOR__ >= 4 )) || __cplusplus >= 201103)
    #define __Pyx_SET_CREAL(z,x) ((z).real(x))
    #define __Pyx_SET_CIMAG(z,y) ((z).imag(y))
#else
    #define __Pyx_SET_CREAL(z,x) __Pyx_CREAL(z) = (x)
    #define __Pyx_SET_CIMAG(z,y) __Pyx_CIMAG(z) = (y)
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq_float(a, b)   ((a)==(b))
    #define __Pyx_c_sum_float(a, b)  ((a)+(b))
    #define __Pyx_c_diff_float(a, b) ((a)-(b))
    #define __Pyx_c_prod_float(a, b) ((a)*(b))
    #define __Pyx_c_quot_float(a, b) ((a)/(b))
    #define __Pyx_c_neg_float(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_float(z) ((z)==(float)0)
    #define __Pyx_c_conj_float(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (::std::abs(z))
        #define __Pyx_c_pow_float(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_float(z) ((z)==0)
    #define __Pyx_c_conj_float(z)    (conjf(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (cabsf(z))
        #define __Pyx_c_pow_float(a, b)  (cpowf(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex);
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex);
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex, __pyx_t_float_complex);
    #endif
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq_double(a, b)   ((a)==(b))
    #define __Pyx_c_sum_double(a, b)  ((a)+(b))
    #define __Pyx_c_diff_double(a, b) ((a)-(b))
    #define __Pyx_c_prod_double(a, b) ((a)*(b))
    #define __Pyx_c_quot_double(a, b) ((a)/(b))
    #define __Pyx_c_neg_double(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_double(z) ((z)==(double)0)
    #define __Pyx_c_conj_double(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (::std::abs(z))
        #define __Pyx_c_pow_double(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_double(z) ((z)==0)
    #define __Pyx_c_conj_double(z)    (conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (cabs(z))
        #define __Pyx_c_pow_double(a, b)  (cpow(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex);
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex);
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex, __pyx_t_double_complex);
    #endif
#endif

/* CIntFromPy.proto */
static CYTHON_INLINE hsize_t __Pyx_PyInt_As_hsize_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE hid_t __Pyx_PyInt_As_hid_t(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE hbool_t __Pyx_PyInt_As_hbool_t(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_hid_t(hid_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_hsize_t(hsize_t value);

/* CIntFromPy.proto */
static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__H5T_order_t(enum H5T_order_t value);

/* CIntFromPy.proto */
static CYTHON_INLINE Py_intptr_t __Pyx_PyInt_As_Py_intptr_t(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_PY_LONG_LONG(PY_LONG_LONG value);

/* CIntFromPy.proto */
static CYTHON_INLINE PY_LONG_LONG __Pyx_PyInt_As_PY_LONG_LONG(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* FunctionImport.proto */
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static PyObject *__pyx_f_6tables_14tableextension_5Table_get_nested_type(struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self, hid_t __pyx_v_type_id, hid_t __pyx_v_native_type_id, PyObject *__pyx_v_colpath, PyObject *__pyx_v_field_byteorders); /* proto*/
static PyObject *__pyx_f_6tables_14tableextension_5Table__convert_time64_(CYTHON_UNUSED struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self, PyArrayObject *__pyx_v_nparr, hsize_t __pyx_v_nrecords, int __pyx_v_sense); /* proto*/
static PyObject *__pyx_f_6tables_14tableextension_5Table__convert_types(struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self, PyArrayObject *__pyx_v_recarr, hsize_t __pyx_v_nrecords, int __pyx_v_sense, int __pyx_skip_dispatch); /* proto*/
static hsize_t __pyx_f_6tables_14tableextension_5Table__read_chunk(struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self, hsize_t __pyx_v_nchunk, PyArrayObject *__pyx_v_iobuf, long __pyx_v_cstart); /* proto*/
static PyObject *__pyx_f_6tables_14tableextension_3Row__new_buffer(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self, PyObject *__pyx_v_table); /* proto*/
static PyObject *__pyx_f_6tables_14tableextension_3Row__init_loop(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self, hsize_t __pyx_v_start, PY_LONG_LONG __pyx_v_stop, PY_LONG_LONG __pyx_v_step, PyObject *__pyx_v_coords, PyObject *__pyx_v_chunkmap); /* proto*/
static PyObject *__pyx_f_6tables_14tableextension_3Row___next__indexed(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_6tables_14tableextension_3Row___next__coords(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_6tables_14tableextension_3Row___next__inkernel(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_6tables_14tableextension_3Row___next__general(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_6tables_14tableextension_3Row__finish_riterator(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self); /* proto*/

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from 'cpython.buffer' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'cpython.version' */

/* Module declarations from 'cpython.exc' */

/* Module declarations from 'cpython.module' */

/* Module declarations from 'cpython.mem' */

/* Module declarations from 'cpython.tuple' */

/* Module declarations from 'cpython.list' */

/* Module declarations from 'cpython.sequence' */

/* Module declarations from 'cpython.mapping' */

/* Module declarations from 'cpython.iterator' */

/* Module declarations from 'cpython.number' */

/* Module declarations from 'cpython.int' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.bool' */
static PyTypeObject *__pyx_ptype_7cpython_4bool_bool = 0;

/* Module declarations from 'cpython.long' */

/* Module declarations from 'cpython.float' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.complex' */
static PyTypeObject *__pyx_ptype_7cpython_7complex_complex = 0;

/* Module declarations from 'cpython.string' */

/* Module declarations from 'cpython.unicode' */

/* Module declarations from 'cpython.dict' */

/* Module declarations from 'cpython.instance' */

/* Module declarations from 'cpython.function' */

/* Module declarations from 'cpython.method' */

/* Module declarations from 'cpython.weakref' */

/* Module declarations from 'cpython.getargs' */

/* Module declarations from 'cpython.pythread' */

/* Module declarations from 'cpython.pystate' */

/* Module declarations from 'cpython.cobject' */

/* Module declarations from 'cpython.oldbuffer' */

/* Module declarations from 'cpython.set' */

/* Module declarations from 'cpython.bytes' */

/* Module declarations from 'cpython.pycapsule' */

/* Module declarations from 'cpython' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'numpy' */

/* Module declarations from 'numpy' */
static PyTypeObject *__pyx_ptype_5numpy_dtype = 0;
static PyTypeObject *__pyx_ptype_5numpy_flatiter = 0;
static PyTypeObject *__pyx_ptype_5numpy_broadcast = 0;
static PyTypeObject *__pyx_ptype_5numpy_ndarray = 0;
static PyTypeObject *__pyx_ptype_5numpy_generic = 0;
static PyTypeObject *__pyx_ptype_5numpy_number = 0;
static PyTypeObject *__pyx_ptype_5numpy_integer = 0;
static PyTypeObject *__pyx_ptype_5numpy_signedinteger = 0;
static PyTypeObject *__pyx_ptype_5numpy_unsignedinteger = 0;
static PyTypeObject *__pyx_ptype_5numpy_inexact = 0;
static PyTypeObject *__pyx_ptype_5numpy_floating = 0;
static PyTypeObject *__pyx_ptype_5numpy_complexfloating = 0;
static PyTypeObject *__pyx_ptype_5numpy_flexible = 0;
static PyTypeObject *__pyx_ptype_5numpy_character = 0;
static PyTypeObject *__pyx_ptype_5numpy_ufunc = 0;
static CYTHON_INLINE int __pyx_f_5numpy_import_array(void); /*proto*/

/* Module declarations from 'tables.definitions' */

/* Module declarations from 'tables.utilsextension' */
static hid_t (*__pyx_f_6tables_14utilsextension_get_native_type)(hid_t); /*proto*/
static PyObject *(*__pyx_f_6tables_14utilsextension_cstr_to_pystr)(char const *); /*proto*/

/* Module declarations from 'tables.hdf5extension' */
static PyTypeObject *__pyx_ptype_6tables_13hdf5extension_Node = 0;
static PyTypeObject *__pyx_ptype_6tables_13hdf5extension_Leaf = 0;
static PyTypeObject *__pyx_ptype_6tables_13hdf5extension_Array = 0;

/* Module declarations from 'libc.stdlib' */

/* Module declarations from 'tables.lrucacheextension' */
static PyTypeObject *__pyx_ptype_6tables_17lrucacheextension_NodeCache = 0;
static PyTypeObject *__pyx_ptype_6tables_17lrucacheextension_BaseCache = 0;
static PyTypeObject *__pyx_ptype_6tables_17lrucacheextension_ObjectNode = 0;
static PyTypeObject *__pyx_ptype_6tables_17lrucacheextension_ObjectCache = 0;
static PyTypeObject *__pyx_ptype_6tables_17lrucacheextension_NumCache = 0;

/* Module declarations from 'tables.tableextension' */
static PyTypeObject *__pyx_ptype_6tables_14tableextension_Table = 0;
static PyTypeObject *__pyx_ptype_6tables_14tableextension_Row = 0;
static PyObject *__pyx_f_6tables_14tableextension_get_nested_field_cache(PyObject *, PyObject *, PyObject *); /*proto*/
static PyObject *__pyx_f_6tables_14tableextension_join_path(PyObject *, PyObject *); /*proto*/
#define __Pyx_MODULE_NAME "tables.tableextension"
extern int __pyx_module_is_main_tables__tableextension;
int __pyx_module_is_main_tables__tableextension = 0;

/* Implementation of 'tables.tableextension' */
static PyObject *__pyx_builtin_KeyError;
static PyObject *__pyx_builtin_enumerate;
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_StopIteration;
static PyObject *__pyx_builtin_IOError;
static PyObject *__pyx_builtin_NotImplementedError;
static PyObject *__pyx_builtin_ImportError;
static const char __pyx_k_[] = "";
static const char __pyx_k_r[] = "r";
static const char __pyx_k__2[] = "/";
static const char __pyx_k_eq[] = "__eq__";
static const char __pyx_k_Col[] = "Col";
static const char __pyx_k_Row[] = "Row";
static const char __pyx_k_add[] = "add";
static const char __pyx_k_big[] = "big";
static const char __pyx_k_int[] = "int";
static const char __pyx_k_pos[] = "pos";
static const char __pyx_k_sys[] = "sys";
static const char __pyx_k_args[] = "args";
static const char __pyx_k_copy[] = "copy";
static const char __pyx_k_enum[] = "enum";
static const char __pyx_k_flat[] = "flat";
static const char __pyx_k_h5tb[] = "h5tb";
static const char __pyx_k_item[] = "item";
static const char __pyx_k_kind[] = "kind";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_mode[] = "mode";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_size[] = "size";
static const char __pyx_k_step[] = "step";
static const char __pyx_k_stop[] = "stop";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_time[] = "time";
static const char __pyx_k_uint[] = "uint";
static const char __pyx_k_CLASS[] = "CLASS";
static const char __pyx_k_NROWS[] = "NROWS";
static const char __pyx_k_TITLE[] = "TITLE";
static const char __pyx_k_Table[] = "Table";
static const char __pyx_k_array[] = "array";
static const char __pyx_k_class[] = "__class__";
static const char __pyx_k_dtype[] = "dtype";
static const char __pyx_k_empty[] = "empty";
static const char __pyx_k_field[] = "field";
static const char __pyx_k_float[] = "float";
static const char __pyx_k_int64[] = "int64";
static const char __pyx_k_mixed[] = "mixed";
static const char __pyx_k_names[] = "names";
static const char __pyx_k_nrows[] = "nrows";
static const char __pyx_k_numpy[] = "numpy";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_sense[] = "sense";
static const char __pyx_k_shape[] = "shape";
static const char __pyx_k_start[] = "start";
static const char __pyx_k_table[] = "table";
static const char __pyx_k_title[] = "title";
static const char __pyx_k_utf_8[] = "utf-8";
static const char __pyx_k_utils[] = "utils";
static const char __pyx_k_v_pos[] = "_v_pos";
static const char __pyx_k_zeros[] = "zeros";
static const char __pyx_k_append[] = "append";
static const char __pyx_k_arange[] = "arange";
static const char __pyx_k_coords[] = "coords";
static const char __pyx_k_encode[] = "encode";
static const char __pyx_k_extend[] = "extend";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_little[] = "little";
static const char __pyx_k_offset[] = "_offset";
static const char __pyx_k_params[] = "params";
static const char __pyx_k_recarr[] = "recarr";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_result[] = "result";
static const char __pyx_k_time32[] = "time32";
static const char __pyx_k_time64[] = "time64";
static const char __pyx_k_uint64[] = "uint64";
static const char __pyx_k_v_file[] = "_v_file";
static const char __pyx_k_IOError[] = "IOError";
static const char __pyx_k_VERSION[] = "VERSION";
static const char __pyx_k_alltrue[] = "alltrue";
static const char __pyx_k_asarray[] = "asarray";
static const char __pyx_k_chunked[] = "_chunked";
static const char __pyx_k_complex[] = "complex";
static const char __pyx_k_complib[] = "complib";
static const char __pyx_k_filters[] = "filters";
static const char __pyx_k_rowsize[] = "rowsize";
static const char __pyx_k_v_dtype[] = "_v_dtype";
static const char __pyx_k_v_names[] = "_v_names";
static const char __pyx_k_KeyError[] = "KeyError";
static const char __pyx_k_SizeType[] = "SizeType";
static const char __pyx_k_byteswap[] = "byteswap";
static const char __pyx_k_chunkmap[] = "chunkmap";
static const char __pyx_k_colenums[] = "_colenums";
static const char __pyx_k_coltypes[] = "coltypes";
static const char __pyx_k_get_node[] = "_get_node";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_itemsize[] = "itemsize";
static const char __pyx_k_nrecords[] = "nrecords";
static const char __pyx_k_seqcache[] = "_seqcache";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_sometrue[] = "sometrue";
static const char __pyx_k_v_leaves[] = "_v_leaves";
static const char __pyx_k_v_offset[] = "_v_offset";
static const char __pyx_k_v_parent[] = "_v_parent";
static const char __pyx_k_v_wdflts[] = "_v_wdflts";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_byteorder[] = "byteorder";
static const char __pyx_k_c_classid[] = "_c_classid";
static const char __pyx_k_coldescrs[] = "coldescrs";
static const char __pyx_k_complevel[] = "complevel";
static const char __pyx_k_enumerate[] = "enumerate";
static const char __pyx_k_from_atom[] = "from_atom";
static const char __pyx_k_obversion[] = "obversion";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_use_index[] = "_use_index";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_check_open[] = "_check_open";
static const char __pyx_k_chunkcache[] = "_chunkcache";
static const char __pyx_k_chunkshape[] = "chunkshape";
static const char __pyx_k_conditions[] = "conditions";
static const char __pyx_k_dirtycache[] = "_dirtycache";
static const char __pyx_k_exceptions[] = "exceptions";
static const char __pyx_k_fletcher32[] = "fletcher32";
static const char __pyx_k_irrelevant[] = "irrelevant";
static const char __pyx_k_nrowsinbuf[] = "nrowsinbuf";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_v_itemsize[] = "_v_itemsize";
static const char __pyx_k_v_pathname[] = "_v_pathname";
static const char __pyx_k_v_recarray[] = "_v_recarray";
static const char __pyx_k_H5T_STD_I64[] = "H5T_STD_I64";
static const char __pyx_k_ImportError[] = "ImportError";
static const char __pyx_k_byteorder_2[] = "_byteorder";
static const char __pyx_k_description[] = "description";
static const char __pyx_k_remove_rows[] = "_remove_rows";
static const char __pyx_k_HDF5ExtError[] = "HDF5ExtError";
static const char __pyx_k_colpathnames[] = "colpathnames";
static const char __pyx_k_read_records[] = "_read_records";
static const char __pyx_k_seqcache_key[] = "_seqcache_key";
static const char __pyx_k_StopIteration[] = "StopIteration";
static const char __pyx_k_convert_types[] = "_convert_types";
static const char __pyx_k_get_container[] = "_get_container";
static const char __pyx_k_read_elements[] = "_read_elements";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_call_on_recarr[] = "call_on_recarr";
static const char __pyx_k_flush_mod_rows[] = "_flush_mod_rows";
static const char __pyx_k_pttype_to_hdf5[] = "pttype_to_hdf5";
static const char __pyx_k_time64colnames[] = "_time64colnames";
static const char __pyx_k_utilsextension[] = "utilsextension";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_update_elements[] = "_update_elements";
static const char __pyx_k_where_condition[] = "_where_condition";
static const char __pyx_k_fetch_all_fields[] = "fetch_all_fields";
static const char __pyx_k_get_nested_field[] = "get_nested_field";
static const char __pyx_k_pt_special_kinds[] = "pt_special_kinds";
static const char __pyx_k_want_track_times[] = "_want_track_times";
static const char __pyx_k_PYTABLES_SYS_ATTRS[] = "PYTABLES_SYS_ATTRS";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_create_nested_type[] = "create_nested_type";
static const char __pyx_k_platform_byteorder[] = "platform_byteorder";
static const char __pyx_k_save_buffered_rows[] = "_save_buffered_rows";
static const char __pyx_k_shuffle_bitshuffle[] = "shuffle_bitshuffle";
static const char __pyx_k_table_s_column_s_s[] = "table ``%s``, column ``%s``: %%s";
static const char __pyx_k_NotImplementedError[] = "NotImplementedError";
static const char __pyx_k_atom_from_hdf5_type[] = "atom_from_hdf5_type";
static const char __pyx_k_flush_buffered_rows[] = "_flush_buffered_rows";
static const char __pyx_k_hdf5_to_np_ext_type[] = "hdf5_to_np_ext_type";
static const char __pyx_k_ITERSEQ_MAX_ELEMENTS[] = "ITERSEQ_MAX_ELEMENTS";
static const char __pyx_k_hdf5_class_to_string[] = "hdf5_class_to_string";
static const char __pyx_k_mark_columns_as_dirty[] = "_mark_columns_as_dirty";
static const char __pyx_k_step_size_may_not_be_0[] = "step size may not be 0.";
static const char __pyx_k_Problems_reading_records[] = "Problems reading records.";
static const char __pyx_k_Problems_deleting_records[] = "Problems deleting records.";
static const char __pyx_k_npext_prefixes_to_ptkinds[] = "npext_prefixes_to_ptkinds";
static const char __pyx_k_s_row_s_pointing_to_row_d[] = "%s.row (%s), pointing to row #%d";
static const char __pyx_k_Non_existing_node_s_under_s[] = "Non-existing node ``%s`` under ``%s``";
static const char __pyx_k_Problems_creating_the_table[] = "Problems creating the table";
static const char __pyx_k_invalid_type_s_for_column_s[] = "invalid type (%s) for column ``%s``";
static const char __pyx_k_Node_s_is_not_a_Table_object[] = "Node ``%s`` is not a Table object";
static const char __pyx_k_Problems_updating_the_records[] = "Problems updating the records.";
static const char __pyx_k_Problems_appending_the_records[] = "Problems appending the records.";
static const char __pyx_k_Problems_reading_chunk_records[] = "Problems reading chunk records.";
static const char __pyx_k_Problems_getting_desciption_for[] = "Problems getting desciption for table %s";
static const char __pyx_k_You_cannot_append_rows_to_a_non[] = "You cannot append rows to a non-chunked table.";
static const char __pyx_k_numpy_core_multiarray_failed_to[] = "numpy.core.multiarray failed to import";
static const char __pyx_k_Attempt_to_write_over_a_file_ope[] = "Attempt to write over a file opened in read-only mode";
static const char __pyx_k_Can_t_set_attribute_s_in_table_s[] = "Can't set attribute '%s' in table:\n %s.";
static const char __pyx_k_Here_is_where_Table_and_Row_exte[] = "Here is where Table and Row extension types live.\n\nClasses (type extensions):\n\n    Table\n    Row\n\nFunctions:\n\nMisc variables:\n\n";
static const char __pyx_k_Problems_setting_the_NROWS_attri[] = "Problems setting the NROWS attribute.";
static const char __pyx_k_Warning_Row_iterator_has_not_bee[] = "Warning: Row iterator has not been initialized for table:\n  %s\n You will normally want to use this method in iterator contexts.";
static const char __pyx_k_You_are_only_allowed_to_update_r[] = "You are only allowed to update rows through the Row.update() method if you are in the middle of a table iterator.";
static const char __pyx_k_You_cannot_append_rows_when_in_m[] = "You cannot append rows when in middle of a table iterator. If what you want is to update records, use Row.update() instead.";
static const char __pyx_k_attempt_to_write_over_a_file_ope[] = "attempt to write over a file opened in read-only mode";
static const char __pyx_k_no_default___reduce___due_to_non[] = "no default __reduce__ due to non-trivial __cinit__";
static const char __pyx_k_numpy_core_umath_failed_to_impor[] = "numpy.core.umath failed to import";
static const char __pyx_k_self_dims_self_wbuf_cannot_be_co[] = "self.dims,self.wbuf cannot be converted to a Python object for pickling";
static const char __pyx_k_Warning_Row_iterator_has_not_bee_2[] = "Warning: Row iterator has not been initialized for table:\n  %s\n You will normally want to use this object in iterator contexts.";
static PyObject *__pyx_kp_s_;
static PyObject *__pyx_kp_s_Attempt_to_write_over_a_file_ope;
static PyObject *__pyx_n_s_CLASS;
static PyObject *__pyx_kp_s_Can_t_set_attribute_s_in_table_s;
static PyObject *__pyx_n_s_Col;
static PyObject *__pyx_n_s_H5T_STD_I64;
static PyObject *__pyx_n_s_HDF5ExtError;
static PyObject *__pyx_n_s_IOError;
static PyObject *__pyx_n_s_ITERSEQ_MAX_ELEMENTS;
static PyObject *__pyx_n_s_ImportError;
static PyObject *__pyx_n_s_KeyError;
static PyObject *__pyx_n_s_NROWS;
static PyObject *__pyx_kp_s_Node_s_is_not_a_Table_object;
static PyObject *__pyx_kp_s_Non_existing_node_s_under_s;
static PyObject *__pyx_n_s_NotImplementedError;
static PyObject *__pyx_n_s_PYTABLES_SYS_ATTRS;
static PyObject *__pyx_kp_s_Problems_appending_the_records;
static PyObject *__pyx_kp_s_Problems_creating_the_table;
static PyObject *__pyx_kp_s_Problems_deleting_records;
static PyObject *__pyx_kp_s_Problems_getting_desciption_for;
static PyObject *__pyx_kp_s_Problems_reading_chunk_records;
static PyObject *__pyx_kp_s_Problems_reading_records;
static PyObject *__pyx_kp_s_Problems_setting_the_NROWS_attri;
static PyObject *__pyx_kp_s_Problems_updating_the_records;
static PyObject *__pyx_n_s_Row;
static PyObject *__pyx_n_s_SizeType;
static PyObject *__pyx_n_s_StopIteration;
static PyObject *__pyx_n_s_TITLE;
static PyObject *__pyx_n_s_Table;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_n_s_VERSION;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_kp_s_Warning_Row_iterator_has_not_bee;
static PyObject *__pyx_kp_s_Warning_Row_iterator_has_not_bee_2;
static PyObject *__pyx_kp_s_You_are_only_allowed_to_update_r;
static PyObject *__pyx_kp_s_You_cannot_append_rows_to_a_non;
static PyObject *__pyx_kp_s_You_cannot_append_rows_when_in_m;
static PyObject *__pyx_kp_s__2;
static PyObject *__pyx_n_s_add;
static PyObject *__pyx_n_s_alltrue;
static PyObject *__pyx_n_s_append;
static PyObject *__pyx_n_s_arange;
static PyObject *__pyx_n_s_args;
static PyObject *__pyx_n_s_array;
static PyObject *__pyx_n_s_asarray;
static PyObject *__pyx_n_s_atom_from_hdf5_type;
static PyObject *__pyx_kp_s_attempt_to_write_over_a_file_ope;
static PyObject *__pyx_n_s_big;
static PyObject *__pyx_n_s_byteorder;
static PyObject *__pyx_n_s_byteorder_2;
static PyObject *__pyx_n_s_byteswap;
static PyObject *__pyx_n_s_c_classid;
static PyObject *__pyx_n_s_call_on_recarr;
static PyObject *__pyx_n_s_check_open;
static PyObject *__pyx_n_s_chunkcache;
static PyObject *__pyx_n_s_chunked;
static PyObject *__pyx_n_s_chunkmap;
static PyObject *__pyx_n_s_chunkshape;
static PyObject *__pyx_n_s_class;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_coldescrs;
static PyObject *__pyx_n_s_colenums;
static PyObject *__pyx_n_s_colpathnames;
static PyObject *__pyx_n_s_coltypes;
static PyObject *__pyx_n_s_complevel;
static PyObject *__pyx_n_s_complex;
static PyObject *__pyx_n_s_complib;
static PyObject *__pyx_n_s_conditions;
static PyObject *__pyx_n_s_convert_types;
static PyObject *__pyx_n_s_coords;
static PyObject *__pyx_n_s_copy;
static PyObject *__pyx_n_s_create_nested_type;
static PyObject *__pyx_n_s_description;
static PyObject *__pyx_n_s_dirtycache;
static PyObject *__pyx_n_s_dtype;
static PyObject *__pyx_n_s_empty;
static PyObject *__pyx_n_s_encode;
static PyObject *__pyx_n_s_enum;
static PyObject *__pyx_n_s_enumerate;
static PyObject *__pyx_n_s_eq;
static PyObject *__pyx_n_s_exceptions;
static PyObject *__pyx_n_s_extend;
static PyObject *__pyx_n_s_fetch_all_fields;
static PyObject *__pyx_n_s_field;
static PyObject *__pyx_n_s_filters;
static PyObject *__pyx_n_s_flat;
static PyObject *__pyx_n_s_fletcher32;
static PyObject *__pyx_n_s_float;
static PyObject *__pyx_n_s_flush_buffered_rows;
static PyObject *__pyx_n_s_flush_mod_rows;
static PyObject *__pyx_n_s_from_atom;
static PyObject *__pyx_n_s_get_container;
static PyObject *__pyx_n_s_get_nested_field;
static PyObject *__pyx_n_s_get_node;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_h5tb;
static PyObject *__pyx_n_s_hdf5_class_to_string;
static PyObject *__pyx_n_s_hdf5_to_np_ext_type;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_int;
static PyObject *__pyx_n_s_int64;
static PyObject *__pyx_kp_s_invalid_type_s_for_column_s;
static PyObject *__pyx_n_s_irrelevant;
static PyObject *__pyx_n_s_item;
static PyObject *__pyx_n_s_itemsize;
static PyObject *__pyx_n_s_kind;
static PyObject *__pyx_n_s_little;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_mark_columns_as_dirty;
static PyObject *__pyx_n_s_mixed;
static PyObject *__pyx_n_s_mode;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_names;
static PyObject *__pyx_kp_s_no_default___reduce___due_to_non;
static PyObject *__pyx_n_s_npext_prefixes_to_ptkinds;
static PyObject *__pyx_n_s_nrecords;
static PyObject *__pyx_n_s_nrows;
static PyObject *__pyx_n_s_nrowsinbuf;
static PyObject *__pyx_n_s_numpy;
static PyObject *__pyx_kp_s_numpy_core_multiarray_failed_to;
static PyObject *__pyx_kp_s_numpy_core_umath_failed_to_impor;
static PyObject *__pyx_n_s_obversion;
static PyObject *__pyx_n_s_offset;
static PyObject *__pyx_n_s_params;
static PyObject *__pyx_n_s_platform_byteorder;
static PyObject *__pyx_n_s_pos;
static PyObject *__pyx_n_s_pt_special_kinds;
static PyObject *__pyx_n_s_pttype_to_hdf5;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_r;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_read_elements;
static PyObject *__pyx_n_s_read_records;
static PyObject *__pyx_n_s_recarr;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_remove_rows;
static PyObject *__pyx_n_s_result;
static PyObject *__pyx_n_s_rowsize;
static PyObject *__pyx_kp_s_s_row_s_pointing_to_row_d;
static PyObject *__pyx_n_s_save_buffered_rows;
static PyObject *__pyx_kp_s_self_dims_self_wbuf_cannot_be_co;
static PyObject *__pyx_n_s_sense;
static PyObject *__pyx_n_s_seqcache;
static PyObject *__pyx_n_s_seqcache_key;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_shape;
static PyObject *__pyx_n_s_shuffle_bitshuffle;
static PyObject *__pyx_n_s_size;
static PyObject *__pyx_n_s_sometrue;
static PyObject *__pyx_n_s_start;
static PyObject *__pyx_n_s_step;
static PyObject *__pyx_kp_s_step_size_may_not_be_0;
static PyObject *__pyx_n_s_stop;
static PyObject *__pyx_n_s_sys;
static PyObject *__pyx_n_s_table;
static PyObject *__pyx_kp_s_table_s_column_s_s;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_time;
static PyObject *__pyx_n_s_time32;
static PyObject *__pyx_n_s_time64;
static PyObject *__pyx_n_s_time64colnames;
static PyObject *__pyx_n_s_title;
static PyObject *__pyx_n_s_uint;
static PyObject *__pyx_n_s_uint64;
static PyObject *__pyx_n_s_update_elements;
static PyObject *__pyx_n_s_use_index;
static PyObject *__pyx_kp_s_utf_8;
static PyObject *__pyx_n_s_utils;
static PyObject *__pyx_n_s_utilsextension;
static PyObject *__pyx_n_s_v_dtype;
static PyObject *__pyx_n_s_v_file;
static PyObject *__pyx_n_s_v_itemsize;
static PyObject *__pyx_n_s_v_leaves;
static PyObject *__pyx_n_s_v_names;
static PyObject *__pyx_n_s_v_offset;
static PyObject *__pyx_n_s_v_parent;
static PyObject *__pyx_n_s_v_pathname;
static PyObject *__pyx_n_s_v_pos;
static PyObject *__pyx_n_s_v_recarray;
static PyObject *__pyx_n_s_v_wdflts;
static PyObject *__pyx_n_s_want_track_times;
static PyObject *__pyx_n_s_where_condition;
static PyObject *__pyx_n_s_zeros;
static PyObject *__pyx_pf_6tables_14tableextension_5Table__create_table(struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self, PyObject *__pyx_v_title, PyObject *__pyx_v_complib, PyObject *__pyx_v_obversion); /* proto */
static PyObject *__pyx_pf_6tables_14tableextension_5Table_2_get_info(struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_14tableextension_5Table_4_convert_types(struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self, PyArrayObject *__pyx_v_recarr, hsize_t __pyx_v_nrecords, int __pyx_v_sense); /* proto */
static PyObject *__pyx_pf_6tables_14tableextension_5Table_6_open_append(struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self, PyArrayObject *__pyx_v_recarr); /* proto */
static PyObject *__pyx_pf_6tables_14tableextension_5Table_8_append_records(struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self, hsize_t __pyx_v_nrecords); /* proto */
static PyObject *__pyx_pf_6tables_14tableextension_5Table_10_close_append(struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_14tableextension_5Table_12_update_records(struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self, hsize_t __pyx_v_start, hsize_t __pyx_v_stop, hsize_t __pyx_v_step, PyArrayObject *__pyx_v_recarr); /* proto */
static PyObject *__pyx_pf_6tables_14tableextension_5Table_14_update_elements(struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self, hsize_t __pyx_v_nrecords, PyArrayObject *__pyx_v_coords, PyArrayObject *__pyx_v_recarr); /* proto */
static PyObject *__pyx_pf_6tables_14tableextension_5Table_16_read_records(struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self, hsize_t __pyx_v_start, hsize_t __pyx_v_nrecords, PyArrayObject *__pyx_v_recarr); /* proto */
static PyObject *__pyx_pf_6tables_14tableextension_5Table_18_read_elements(struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self, PyArrayObject *__pyx_v_coords, PyArrayObject *__pyx_v_recarr); /* proto */
static PyObject *__pyx_pf_6tables_14tableextension_5Table_20_remove_rows(struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self, hsize_t __pyx_v_start, hsize_t __pyx_v_stop, long __pyx_v_step); /* proto */
static PyObject *__pyx_pf_6tables_14tableextension_5Table_22__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_14tableextension_5Table_24__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6tables_14tableextension_3Row_4nrow___get__(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_14tableextension_3Row_5table___get__(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self); /* proto */
static int __pyx_pf_6tables_14tableextension_3Row___cinit__(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self, PyObject *__pyx_v_table); /* proto */
static PyObject *__pyx_pf_6tables_14tableextension_3Row_2_iter(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self, PyObject *__pyx_v_start, PyObject *__pyx_v_stop, PyObject *__pyx_v_step, PyObject *__pyx_v_coords, PyObject *__pyx_v_chunkmap); /* proto */
static PyObject *__pyx_pf_6tables_14tableextension_3Row_4__iter__(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_14tableextension_3Row_6__next__(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_14tableextension_3Row_8_fill_col(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self, PyObject *__pyx_v_result, PyObject *__pyx_v_start, PyObject *__pyx_v_stop, PyObject *__pyx_v_step, PyObject *__pyx_v_field); /* proto */
static PyObject *__pyx_pf_6tables_14tableextension_3Row_10append(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_14tableextension_3Row_12_flush_buffered_rows(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_14tableextension_3Row_14_get_unsaved_nrows(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_14tableextension_3Row_16update(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_14tableextension_3Row_18_flush_mod_rows(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self); /* proto */
static int __pyx_pf_6tables_14tableextension_3Row_20__contains__(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self, PyObject *__pyx_v_item); /* proto */
static PyObject *__pyx_pf_6tables_14tableextension_3Row_22__getitem__(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self, PyObject *__pyx_v_key); /* proto */
static int __pyx_pf_6tables_14tableextension_3Row_24__setitem__(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6tables_14tableextension_3Row_26fetch_all_fields(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_14tableextension_3Row_28__str__(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_14tableextension_3Row_30__repr__(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_14tableextension_3Row_32__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_14tableextension_3Row_34__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_6tables_14tableextension_Table(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6tables_14tableextension_Row(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_slice__7;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__14;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__16;
/* Late includes */

/* "tables/tableextension.pyx":114
 * 
 * # Private functions
 * cdef get_nested_field_cache(recarray, fieldname, fieldcache):             # <<<<<<<<<<<<<<
 *   """Get the maybe nested field named `fieldname` from the `recarray`.
 * 
 */

static PyObject *__pyx_f_6tables_14tableextension_get_nested_field_cache(PyObject *__pyx_v_recarray, PyObject *__pyx_v_fieldname, PyObject *__pyx_v_fieldcache) {
  PyObject *__pyx_v_field = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_nested_field_cache", 0);

  /* "tables/tableextension.pyx":123
 *   """
 * 
 *   try:             # <<<<<<<<<<<<<<
 *     field = fieldcache[fieldname]
 *   except KeyError:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "tables/tableextension.pyx":124
 * 
 *   try:
 *     field = fieldcache[fieldname]             # <<<<<<<<<<<<<<
 *   except KeyError:
 *     # Check whether fieldname is an integer and if so, get the field
 */
      __pyx_t_4 = __Pyx_PyObject_GetItem(__pyx_v_fieldcache, __pyx_v_fieldname); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 124, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_v_field = __pyx_t_4;
      __pyx_t_4 = 0;

      /* "tables/tableextension.pyx":123
 *   """
 * 
 *   try:             # <<<<<<<<<<<<<<
 *     field = fieldcache[fieldname]
 *   except KeyError:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "tables/tableextension.pyx":125
 *   try:
 *     field = fieldcache[fieldname]
 *   except KeyError:             # <<<<<<<<<<<<<<
 *     # Check whether fieldname is an integer and if so, get the field
 *     # straight from the recarray dictionary (it can't be anywhere else)
 */
    __pyx_t_5 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_KeyError);
    if (__pyx_t_5) {
      __Pyx_AddTraceback("tables.tableextension.get_nested_field_cache", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(0, 125, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "tables/tableextension.pyx":128
 *     # Check whether fieldname is an integer and if so, get the field
 *     # straight from the recarray dictionary (it can't be anywhere else)
 *     if isinstance(fieldname, int):             # <<<<<<<<<<<<<<
 *       field = recarray[fieldname]
 *     else:
 */
      __pyx_t_8 = PyInt_Check(__pyx_v_fieldname); 
      __pyx_t_9 = (__pyx_t_8 != 0);
      if (__pyx_t_9) {

        /* "tables/tableextension.pyx":129
 *     # straight from the recarray dictionary (it can't be anywhere else)
 *     if isinstance(fieldname, int):
 *       field = recarray[fieldname]             # <<<<<<<<<<<<<<
 *     else:
 *       field = get_nested_field(recarray, fieldname)
 */
        __pyx_t_10 = __Pyx_PyObject_GetItem(__pyx_v_recarray, __pyx_v_fieldname); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 129, __pyx_L5_except_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_XDECREF_SET(__pyx_v_field, __pyx_t_10);
        __pyx_t_10 = 0;

        /* "tables/tableextension.pyx":128
 *     # Check whether fieldname is an integer and if so, get the field
 *     # straight from the recarray dictionary (it can't be anywhere else)
 *     if isinstance(fieldname, int):             # <<<<<<<<<<<<<<
 *       field = recarray[fieldname]
 *     else:
 */
        goto __pyx_L11;
      }

      /* "tables/tableextension.pyx":131
 *       field = recarray[fieldname]
 *     else:
 *       field = get_nested_field(recarray, fieldname)             # <<<<<<<<<<<<<<
 *     fieldcache[fieldname] = field
 *   return field
 */
      /*else*/ {
        __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_n_s_get_nested_field); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 131, __pyx_L5_except_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_12 = NULL;
        __pyx_t_5 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_11))) {
          __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_11);
          if (likely(__pyx_t_12)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_11);
            __Pyx_INCREF(__pyx_t_12);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_11, function);
            __pyx_t_5 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_11)) {
          PyObject *__pyx_temp[3] = {__pyx_t_12, __pyx_v_recarray, __pyx_v_fieldname};
          __pyx_t_10 = __Pyx_PyFunction_FastCall(__pyx_t_11, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 131, __pyx_L5_except_error)
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_GOTREF(__pyx_t_10);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_11)) {
          PyObject *__pyx_temp[3] = {__pyx_t_12, __pyx_v_recarray, __pyx_v_fieldname};
          __pyx_t_10 = __Pyx_PyCFunction_FastCall(__pyx_t_11, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 131, __pyx_L5_except_error)
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_GOTREF(__pyx_t_10);
        } else
        #endif
        {
          __pyx_t_13 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 131, __pyx_L5_except_error)
          __Pyx_GOTREF(__pyx_t_13);
          if (__pyx_t_12) {
            __Pyx_GIVEREF(__pyx_t_12); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_12); __pyx_t_12 = NULL;
          }
          __Pyx_INCREF(__pyx_v_recarray);
          __Pyx_GIVEREF(__pyx_v_recarray);
          PyTuple_SET_ITEM(__pyx_t_13, 0+__pyx_t_5, __pyx_v_recarray);
          __Pyx_INCREF(__pyx_v_fieldname);
          __Pyx_GIVEREF(__pyx_v_fieldname);
          PyTuple_SET_ITEM(__pyx_t_13, 1+__pyx_t_5, __pyx_v_fieldname);
          __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_11, __pyx_t_13, NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 131, __pyx_L5_except_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        }
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_XDECREF_SET(__pyx_v_field, __pyx_t_10);
        __pyx_t_10 = 0;
      }
      __pyx_L11:;

      /* "tables/tableextension.pyx":132
 *     else:
 *       field = get_nested_field(recarray, fieldname)
 *     fieldcache[fieldname] = field             # <<<<<<<<<<<<<<
 *   return field
 * 
 */
      if (unlikely(PyObject_SetItem(__pyx_v_fieldcache, __pyx_v_fieldname, __pyx_v_field) < 0)) __PYX_ERR(0, 132, __pyx_L5_except_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L4_exception_handled;
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "tables/tableextension.pyx":123
 *   """
 * 
 *   try:             # <<<<<<<<<<<<<<
 *     field = fieldcache[fieldname]
 *   except KeyError:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L4_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    __pyx_L8_try_end:;
  }

  /* "tables/tableextension.pyx":133
 *       field = get_nested_field(recarray, fieldname)
 *     fieldcache[fieldname] = field
 *   return field             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_field);
  __pyx_r = __pyx_v_field;
  goto __pyx_L0;

  /* "tables/tableextension.pyx":114
 * 
 * # Private functions
 * cdef get_nested_field_cache(recarray, fieldname, fieldcache):             # <<<<<<<<<<<<<<
 *   """Get the maybe nested field named `fieldname` from the `recarray`.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("tables.tableextension.get_nested_field_cache", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_field);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":136
 * 
 * 
 * cdef join_path(object parent, object name):             # <<<<<<<<<<<<<<
 *   if parent == "":
 *     return name
 */

static PyObject *__pyx_f_6tables_14tableextension_join_path(PyObject *__pyx_v_parent, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("join_path", 0);

  /* "tables/tableextension.pyx":137
 * 
 * cdef join_path(object parent, object name):
 *   if parent == "":             # <<<<<<<<<<<<<<
 *     return name
 *   else:
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_parent, __pyx_kp_s_, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 137, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "tables/tableextension.pyx":138
 * cdef join_path(object parent, object name):
 *   if parent == "":
 *     return name             # <<<<<<<<<<<<<<
 *   else:
 *     return parent + '/' + name
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_name);
    __pyx_r = __pyx_v_name;
    goto __pyx_L0;

    /* "tables/tableextension.pyx":137
 * 
 * cdef join_path(object parent, object name):
 *   if parent == "":             # <<<<<<<<<<<<<<
 *     return name
 *   else:
 */
  }

  /* "tables/tableextension.pyx":140
 *     return name
 *   else:
 *     return parent + '/' + name             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = PyNumber_Add(__pyx_v_parent, __pyx_kp_s__2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 140, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyNumber_Add(__pyx_t_2, __pyx_v_name); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 140, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "tables/tableextension.pyx":136
 * 
 * 
 * cdef join_path(object parent, object name):             # <<<<<<<<<<<<<<
 *   if parent == "":
 *     return name
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("tables.tableextension.join_path", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":149
 *   cdef void     *wbuf
 * 
 *   def _create_table(self, title, complib, obversion):             # <<<<<<<<<<<<<<
 *     cdef int     offset
 *     cdef int     ret
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_14tableextension_5Table_1_create_table(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_6tables_14tableextension_5Table_1_create_table(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_title = 0;
  PyObject *__pyx_v_complib = 0;
  PyObject *__pyx_v_obversion = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_create_table (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_title,&__pyx_n_s_complib,&__pyx_n_s_obversion,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_title)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_complib)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_create_table", 1, 3, 3, 1); __PYX_ERR(0, 149, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_obversion)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_create_table", 1, 3, 3, 2); __PYX_ERR(0, 149, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_create_table") < 0)) __PYX_ERR(0, 149, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_title = values[0];
    __pyx_v_complib = values[1];
    __pyx_v_obversion = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_create_table", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 149, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("tables.tableextension.Table._create_table", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_14tableextension_5Table__create_table(((struct __pyx_obj_6tables_14tableextension_Table *)__pyx_v_self), __pyx_v_title, __pyx_v_complib, __pyx_v_obversion);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_14tableextension_5Table__create_table(struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self, PyObject *__pyx_v_title, PyObject *__pyx_v_complib, PyObject *__pyx_v_obversion) {
  int __pyx_v_ret;
  void *__pyx_v_data;
  hsize_t __pyx_v_nrows;
  PyObject *__pyx_v_class_ = 0;
  PyArrayObject *__pyx_v_wdflts = 0;
  void *__pyx_v_fill_data;
  PyArrayObject *__pyx_v_recarr = 0;
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_encoded_title = 0;
  PyObject *__pyx_v_encoded_complib = 0;
  PyObject *__pyx_v_encoded_obversion = 0;
  char *__pyx_v_ctitle;
  char *__pyx_v_cobversion;
  PyObject *__pyx_v_encoded_name = 0;
  char __pyx_v_fieldname[0x80];
  int __pyx_v_i;
  enum H5T_cset_t __pyx_v_cset;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  char *__pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  hid_t __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  char *__pyx_t_12;
  hsize_t __pyx_t_13;
  hsize_t __pyx_t_14;
  char *__pyx_t_15;
  int __pyx_t_16;
  int __pyx_t_17;
  hbool_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  PyObject *(*__pyx_t_20)(PyObject *);
  Py_ssize_t __pyx_t_21;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_create_table", 0);

  /* "tables/tableextension.pyx":162
 *     cdef object  name
 *     cdef bytes encoded_title, encoded_complib, encoded_obversion
 *     cdef char *ctitle = NULL             # <<<<<<<<<<<<<<
 *     cdef char *cobversion = NULL
 *     cdef bytes encoded_name
 */
  __pyx_v_ctitle = NULL;

  /* "tables/tableextension.pyx":163
 *     cdef bytes encoded_title, encoded_complib, encoded_obversion
 *     cdef char *ctitle = NULL
 *     cdef char *cobversion = NULL             # <<<<<<<<<<<<<<
 *     cdef bytes encoded_name
 *     cdef char fieldname[128]
 */
  __pyx_v_cobversion = NULL;

  /* "tables/tableextension.pyx":167
 *     cdef char fieldname[128]
 *     cdef int i
 *     cdef H5T_cset_t cset = H5T_CSET_ASCII             # <<<<<<<<<<<<<<
 * 
 *     encoded_title = title.encode('utf-8')
 */
  __pyx_v_cset = H5T_CSET_ASCII;

  /* "tables/tableextension.pyx":169
 *     cdef H5T_cset_t cset = H5T_CSET_ASCII
 * 
 *     encoded_title = title.encode('utf-8')             # <<<<<<<<<<<<<<
 *     encoded_complib = complib.encode('utf-8')
 *     encoded_obversion = obversion.encode('utf-8')
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_title, __pyx_n_s_encode); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 169, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_kp_s_utf_8) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_s_utf_8);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 169, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 169, __pyx_L1_error)
  __pyx_v_encoded_title = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":170
 * 
 *     encoded_title = title.encode('utf-8')
 *     encoded_complib = complib.encode('utf-8')             # <<<<<<<<<<<<<<
 *     encoded_obversion = obversion.encode('utf-8')
 *     encoded_name = self.name.encode('utf-8')
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_complib, __pyx_n_s_encode); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 170, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_kp_s_utf_8) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_s_utf_8);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 170, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 170, __pyx_L1_error)
  __pyx_v_encoded_complib = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":171
 *     encoded_title = title.encode('utf-8')
 *     encoded_complib = complib.encode('utf-8')
 *     encoded_obversion = obversion.encode('utf-8')             # <<<<<<<<<<<<<<
 *     encoded_name = self.name.encode('utf-8')
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_obversion, __pyx_n_s_encode); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_kp_s_utf_8) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_s_utf_8);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 171, __pyx_L1_error)
  __pyx_v_encoded_obversion = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":172
 *     encoded_complib = complib.encode('utf-8')
 *     encoded_obversion = obversion.encode('utf-8')
 *     encoded_name = self.name.encode('utf-8')             # <<<<<<<<<<<<<<
 * 
 *     # Get the C pointer
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->__pyx_base.__pyx_base.name, __pyx_n_s_encode); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_kp_s_utf_8) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_s_utf_8);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 172, __pyx_L1_error)
  __pyx_v_encoded_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":175
 * 
 *     # Get the C pointer
 *     ctitle = encoded_title             # <<<<<<<<<<<<<<
 *     cobversion = encoded_obversion
 * 
 */
  if (unlikely(__pyx_v_encoded_title == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 175, __pyx_L1_error)
  }
  __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_encoded_title); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(0, 175, __pyx_L1_error)
  __pyx_v_ctitle = __pyx_t_4;

  /* "tables/tableextension.pyx":176
 *     # Get the C pointer
 *     ctitle = encoded_title
 *     cobversion = encoded_obversion             # <<<<<<<<<<<<<<
 * 
 *     # Compute the complete compound datatype based on the table description
 */
  if (unlikely(__pyx_v_encoded_obversion == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 176, __pyx_L1_error)
  }
  __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_encoded_obversion); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(0, 176, __pyx_L1_error)
  __pyx_v_cobversion = __pyx_t_4;

  /* "tables/tableextension.pyx":179
 * 
 *     # Compute the complete compound datatype based on the table description
 *     self.disk_type_id = create_nested_type(self.description, self.byteorder)             # <<<<<<<<<<<<<<
 *     #self.type_id = H5Tcopy(self.disk_type_id)
 *     # A H5Tcopy only is not enough, as we want the in-memory type to be
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_create_nested_type); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 179, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_description); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 179, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_byteorder); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 179, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_3, __pyx_t_5};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 179, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_3, __pyx_t_5};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 179, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  {
    __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 179, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (__pyx_t_6) {
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_5);
    __pyx_t_3 = 0;
    __pyx_t_5 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 179, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_9 = __Pyx_PyInt_As_hid_t(__pyx_t_1); if (unlikely((__pyx_t_9 == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 179, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->__pyx_base.disk_type_id = __pyx_t_9;

  /* "tables/tableextension.pyx":183
 *     # A H5Tcopy only is not enough, as we want the in-memory type to be
 *     # in the byteorder of the machine (sys.byteorder).
 *     self.type_id = create_nested_type(self.description, sys.byteorder)             # <<<<<<<<<<<<<<
 * 
 *     # The fill values area
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_create_nested_type); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 183, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_description); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 183, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_sys); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 183, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_byteorder); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 183, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_8, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 183, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_8, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 183, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_6 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 183, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_7, __pyx_t_8);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_7, __pyx_t_3);
    __pyx_t_8 = 0;
    __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 183, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_9 = __Pyx_PyInt_As_hid_t(__pyx_t_1); if (unlikely((__pyx_t_9 == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 183, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->__pyx_base.type_id = __pyx_t_9;

  /* "tables/tableextension.pyx":186
 * 
 *     # The fill values area
 *     wdflts = self._v_wdflts             # <<<<<<<<<<<<<<
 *     if wdflts is None:
 *       fill_data = NULL
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_v_wdflts); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 186, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 186, __pyx_L1_error)
  __pyx_v_wdflts = ((PyArrayObject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":187
 *     # The fill values area
 *     wdflts = self._v_wdflts
 *     if wdflts is None:             # <<<<<<<<<<<<<<
 *       fill_data = NULL
 *     else:
 */
  __pyx_t_10 = (((PyObject *)__pyx_v_wdflts) == Py_None);
  __pyx_t_11 = (__pyx_t_10 != 0);
  if (__pyx_t_11) {

    /* "tables/tableextension.pyx":188
 *     wdflts = self._v_wdflts
 *     if wdflts is None:
 *       fill_data = NULL             # <<<<<<<<<<<<<<
 *     else:
 *       fill_data = PyArray_DATA(wdflts)
 */
    __pyx_v_fill_data = NULL;

    /* "tables/tableextension.pyx":187
 *     # The fill values area
 *     wdflts = self._v_wdflts
 *     if wdflts is None:             # <<<<<<<<<<<<<<
 *       fill_data = NULL
 *     else:
 */
    goto __pyx_L3;
  }

  /* "tables/tableextension.pyx":190
 *       fill_data = NULL
 *     else:
 *       fill_data = PyArray_DATA(wdflts)             # <<<<<<<<<<<<<<
 * 
 *     # test if there is data to be saved initially
 */
  /*else*/ {
    __pyx_v_fill_data = PyArray_DATA(__pyx_v_wdflts);
  }
  __pyx_L3:;

  /* "tables/tableextension.pyx":193
 * 
 *     # test if there is data to be saved initially
 *     if self._v_recarray is not None:             # <<<<<<<<<<<<<<
 *       recarr = self._v_recarray
 *       data = PyArray_DATA(recarr)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_v_recarray); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_11 = (__pyx_t_1 != Py_None);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_10 = (__pyx_t_11 != 0);
  if (__pyx_t_10) {

    /* "tables/tableextension.pyx":194
 *     # test if there is data to be saved initially
 *     if self._v_recarray is not None:
 *       recarr = self._v_recarray             # <<<<<<<<<<<<<<
 *       data = PyArray_DATA(recarr)
 *     else:
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_v_recarray); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 194, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 194, __pyx_L1_error)
    __pyx_v_recarr = ((PyArrayObject *)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "tables/tableextension.pyx":195
 *     if self._v_recarray is not None:
 *       recarr = self._v_recarray
 *       data = PyArray_DATA(recarr)             # <<<<<<<<<<<<<<
 *     else:
 *       data = NULL
 */
    __pyx_v_data = PyArray_DATA(__pyx_v_recarr);

    /* "tables/tableextension.pyx":193
 * 
 *     # test if there is data to be saved initially
 *     if self._v_recarray is not None:             # <<<<<<<<<<<<<<
 *       recarr = self._v_recarray
 *       data = PyArray_DATA(recarr)
 */
    goto __pyx_L4;
  }

  /* "tables/tableextension.pyx":197
 *       data = PyArray_DATA(recarr)
 *     else:
 *       data = NULL             # <<<<<<<<<<<<<<
 * 
 *     class_ = self._c_classid.encode('utf-8')
 */
  /*else*/ {
    __pyx_v_data = NULL;
  }
  __pyx_L4:;

  /* "tables/tableextension.pyx":199
 *       data = NULL
 * 
 *     class_ = self._c_classid.encode('utf-8')             # <<<<<<<<<<<<<<
 *     self.dataset_id = H5TBOmake_table(ctitle, self.parent_id, encoded_name,
 *                                       cobversion, class_, self.disk_type_id,
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_c_classid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_encode); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_2, __pyx_kp_s_utf_8) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_kp_s_utf_8);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 199, __pyx_L1_error)
  __pyx_v_class_ = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":200
 * 
 *     class_ = self._c_classid.encode('utf-8')
 *     self.dataset_id = H5TBOmake_table(ctitle, self.parent_id, encoded_name,             # <<<<<<<<<<<<<<
 *                                       cobversion, class_, self.disk_type_id,
 *                                       self.nrows, self.chunkshape[0],
 */
  if (unlikely(__pyx_v_encoded_name == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 200, __pyx_L1_error)
  }
  __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_encoded_name); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(0, 200, __pyx_L1_error)

  /* "tables/tableextension.pyx":201
 *     class_ = self._c_classid.encode('utf-8')
 *     self.dataset_id = H5TBOmake_table(ctitle, self.parent_id, encoded_name,
 *                                       cobversion, class_, self.disk_type_id,             # <<<<<<<<<<<<<<
 *                                       self.nrows, self.chunkshape[0],
 *                                       fill_data,
 */
  if (unlikely(__pyx_v_class_ == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 201, __pyx_L1_error)
  }
  __pyx_t_12 = __Pyx_PyBytes_AsWritableString(__pyx_v_class_); if (unlikely((!__pyx_t_12) && PyErr_Occurred())) __PYX_ERR(0, 201, __pyx_L1_error)

  /* "tables/tableextension.pyx":202
 *     self.dataset_id = H5TBOmake_table(ctitle, self.parent_id, encoded_name,
 *                                       cobversion, class_, self.disk_type_id,
 *                                       self.nrows, self.chunkshape[0],             # <<<<<<<<<<<<<<
 *                                       fill_data,
 *                                       self.filters.complevel, encoded_complib,
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nrows); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 202, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_13 = __Pyx_PyInt_As_hsize_t(__pyx_t_1); if (unlikely((__pyx_t_13 == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 202, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_chunkshape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 202, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = __Pyx_GetItemInt(__pyx_t_1, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 202, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_14 = __Pyx_PyInt_As_hsize_t(__pyx_t_6); if (unlikely((__pyx_t_14 == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 202, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "tables/tableextension.pyx":204
 *                                       self.nrows, self.chunkshape[0],
 *                                       fill_data,
 *                                       self.filters.complevel, encoded_complib,             # <<<<<<<<<<<<<<
 *                                       self.filters.shuffle_bitshuffle,
 *                                       self.filters.fletcher32,
 */
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_filters); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_complevel); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_7 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(__pyx_v_encoded_complib == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 204, __pyx_L1_error)
  }
  __pyx_t_15 = __Pyx_PyBytes_AsWritableString(__pyx_v_encoded_complib); if (unlikely((!__pyx_t_15) && PyErr_Occurred())) __PYX_ERR(0, 204, __pyx_L1_error)

  /* "tables/tableextension.pyx":205
 *                                       fill_data,
 *                                       self.filters.complevel, encoded_complib,
 *                                       self.filters.shuffle_bitshuffle,             # <<<<<<<<<<<<<<
 *                                       self.filters.fletcher32,
 *                                       self._want_track_times, data)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_filters); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 205, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_shuffle_bitshuffle); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 205, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_16 = __Pyx_PyInt_As_int(__pyx_t_6); if (unlikely((__pyx_t_16 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 205, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "tables/tableextension.pyx":206
 *                                       self.filters.complevel, encoded_complib,
 *                                       self.filters.shuffle_bitshuffle,
 *                                       self.filters.fletcher32,             # <<<<<<<<<<<<<<
 *                                       self._want_track_times, data)
 *     if self.dataset_id < 0:
 */
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_filters); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_fletcher32); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_17 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_17 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 206, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":207
 *                                       self.filters.shuffle_bitshuffle,
 *                                       self.filters.fletcher32,
 *                                       self._want_track_times, data)             # <<<<<<<<<<<<<<
 *     if self.dataset_id < 0:
 *       raise HDF5ExtError("Problems creating the table")
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_want_track_times); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 207, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_18 = __Pyx_PyInt_As_hbool_t(__pyx_t_1); if (unlikely((__pyx_t_18 == ((hbool_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 207, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":200
 * 
 *     class_ = self._c_classid.encode('utf-8')
 *     self.dataset_id = H5TBOmake_table(ctitle, self.parent_id, encoded_name,             # <<<<<<<<<<<<<<
 *                                       cobversion, class_, self.disk_type_id,
 *                                       self.nrows, self.chunkshape[0],
 */
  __pyx_v_self->__pyx_base.dataset_id = H5TBOmake_table(__pyx_v_ctitle, __pyx_v_self->__pyx_base.__pyx_base.parent_id, __pyx_t_4, __pyx_v_cobversion, __pyx_t_12, __pyx_v_self->__pyx_base.disk_type_id, __pyx_t_13, __pyx_t_14, __pyx_v_fill_data, __pyx_t_7, __pyx_t_15, __pyx_t_16, __pyx_t_17, __pyx_t_18, __pyx_v_data);

  /* "tables/tableextension.pyx":208
 *                                       self.filters.fletcher32,
 *                                       self._want_track_times, data)
 *     if self.dataset_id < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems creating the table")
 * 
 */
  __pyx_t_10 = ((__pyx_v_self->__pyx_base.dataset_id < 0) != 0);
  if (unlikely(__pyx_t_10)) {

    /* "tables/tableextension.pyx":209
 *                                       self._want_track_times, data)
 *     if self.dataset_id < 0:
 *       raise HDF5ExtError("Problems creating the table")             # <<<<<<<<<<<<<<
 * 
 *     if self._v_file.params['PYTABLES_SYS_ATTRS']:
 */
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_HDF5ExtError); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 209, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
      }
    }
    __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_2, __pyx_kp_s_Problems_creating_the_table) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_kp_s_Problems_creating_the_table);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 209, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 209, __pyx_L1_error)

    /* "tables/tableextension.pyx":208
 *                                       self.filters.fletcher32,
 *                                       self._want_track_times, data)
 *     if self.dataset_id < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems creating the table")
 * 
 */
  }

  /* "tables/tableextension.pyx":211
 *       raise HDF5ExtError("Problems creating the table")
 * 
 *     if self._v_file.params['PYTABLES_SYS_ATTRS']:             # <<<<<<<<<<<<<<
 *       cset = H5T_CSET_UTF8
 *       # Set the conforming table attributes
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_v_file); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_params); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_6, __pyx_n_s_PYTABLES_SYS_ATTRS); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_10) {

    /* "tables/tableextension.pyx":212
 * 
 *     if self._v_file.params['PYTABLES_SYS_ATTRS']:
 *       cset = H5T_CSET_UTF8             # <<<<<<<<<<<<<<
 *       # Set the conforming table attributes
 *       # Attach the CLASS attribute
 */
    __pyx_v_cset = H5T_CSET_UTF8;

    /* "tables/tableextension.pyx":215
 *       # Set the conforming table attributes
 *       # Attach the CLASS attribute
 *       ret = H5ATTRset_attribute_string(self.dataset_id, "CLASS", class_,             # <<<<<<<<<<<<<<
 *                                        len(class_), cset)
 *       if ret < 0:
 */
    if (unlikely(__pyx_v_class_ == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(0, 215, __pyx_L1_error)
    }
    __pyx_t_15 = __Pyx_PyBytes_AsWritableString(__pyx_v_class_); if (unlikely((!__pyx_t_15) && PyErr_Occurred())) __PYX_ERR(0, 215, __pyx_L1_error)

    /* "tables/tableextension.pyx":216
 *       # Attach the CLASS attribute
 *       ret = H5ATTRset_attribute_string(self.dataset_id, "CLASS", class_,
 *                                        len(class_), cset)             # <<<<<<<<<<<<<<
 *       if ret < 0:
 *         raise HDF5ExtError("Can't set attribute '%s' in table:\n %s." %
 */
    if (unlikely(__pyx_v_class_ == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 216, __pyx_L1_error)
    }
    __pyx_t_19 = PyBytes_GET_SIZE(__pyx_v_class_); if (unlikely(__pyx_t_19 == ((Py_ssize_t)-1))) __PYX_ERR(0, 216, __pyx_L1_error)

    /* "tables/tableextension.pyx":215
 *       # Set the conforming table attributes
 *       # Attach the CLASS attribute
 *       ret = H5ATTRset_attribute_string(self.dataset_id, "CLASS", class_,             # <<<<<<<<<<<<<<
 *                                        len(class_), cset)
 *       if ret < 0:
 */
    __pyx_v_ret = H5ATTRset_attribute_string(__pyx_v_self->__pyx_base.dataset_id, ((char *)"CLASS"), __pyx_t_15, __pyx_t_19, __pyx_v_cset);

    /* "tables/tableextension.pyx":217
 *       ret = H5ATTRset_attribute_string(self.dataset_id, "CLASS", class_,
 *                                        len(class_), cset)
 *       if ret < 0:             # <<<<<<<<<<<<<<
 *         raise HDF5ExtError("Can't set attribute '%s' in table:\n %s." %
 *                            ("CLASS", self.name))
 */
    __pyx_t_10 = ((__pyx_v_ret < 0) != 0);
    if (unlikely(__pyx_t_10)) {

      /* "tables/tableextension.pyx":218
 *                                        len(class_), cset)
 *       if ret < 0:
 *         raise HDF5ExtError("Can't set attribute '%s' in table:\n %s." %             # <<<<<<<<<<<<<<
 *                            ("CLASS", self.name))
 *       # Attach the VERSION attribute
 */
      __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_HDF5ExtError); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 218, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);

      /* "tables/tableextension.pyx":219
 *       if ret < 0:
 *         raise HDF5ExtError("Can't set attribute '%s' in table:\n %s." %
 *                            ("CLASS", self.name))             # <<<<<<<<<<<<<<
 *       # Attach the VERSION attribute
 *       ret = H5ATTRset_attribute_string(self.dataset_id, "VERSION", cobversion,
 */
      __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 219, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_INCREF(__pyx_n_s_CLASS);
      __Pyx_GIVEREF(__pyx_n_s_CLASS);
      PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_CLASS);
      __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.name);
      __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.name);
      PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_self->__pyx_base.__pyx_base.name);

      /* "tables/tableextension.pyx":218
 *                                        len(class_), cset)
 *       if ret < 0:
 *         raise HDF5ExtError("Can't set attribute '%s' in table:\n %s." %             # <<<<<<<<<<<<<<
 *                            ("CLASS", self.name))
 *       # Attach the VERSION attribute
 */
      __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_Can_t_set_attribute_s_in_table_s, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 218, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
        }
      }
      __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_3);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 218, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_Raise(__pyx_t_1, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __PYX_ERR(0, 218, __pyx_L1_error)

      /* "tables/tableextension.pyx":217
 *       ret = H5ATTRset_attribute_string(self.dataset_id, "CLASS", class_,
 *                                        len(class_), cset)
 *       if ret < 0:             # <<<<<<<<<<<<<<
 *         raise HDF5ExtError("Can't set attribute '%s' in table:\n %s." %
 *                            ("CLASS", self.name))
 */
    }

    /* "tables/tableextension.pyx":222
 *       # Attach the VERSION attribute
 *       ret = H5ATTRset_attribute_string(self.dataset_id, "VERSION", cobversion,
 *                                        len(encoded_obversion), cset)             # <<<<<<<<<<<<<<
 *       if ret < 0:
 *         raise HDF5ExtError("Can't set attribute '%s' in table:\n %s." %
 */
    if (unlikely(__pyx_v_encoded_obversion == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 222, __pyx_L1_error)
    }
    __pyx_t_19 = PyBytes_GET_SIZE(__pyx_v_encoded_obversion); if (unlikely(__pyx_t_19 == ((Py_ssize_t)-1))) __PYX_ERR(0, 222, __pyx_L1_error)

    /* "tables/tableextension.pyx":221
 *                            ("CLASS", self.name))
 *       # Attach the VERSION attribute
 *       ret = H5ATTRset_attribute_string(self.dataset_id, "VERSION", cobversion,             # <<<<<<<<<<<<<<
 *                                        len(encoded_obversion), cset)
 *       if ret < 0:
 */
    __pyx_v_ret = H5ATTRset_attribute_string(__pyx_v_self->__pyx_base.dataset_id, ((char *)"VERSION"), __pyx_v_cobversion, __pyx_t_19, __pyx_v_cset);

    /* "tables/tableextension.pyx":223
 *       ret = H5ATTRset_attribute_string(self.dataset_id, "VERSION", cobversion,
 *                                        len(encoded_obversion), cset)
 *       if ret < 0:             # <<<<<<<<<<<<<<
 *         raise HDF5ExtError("Can't set attribute '%s' in table:\n %s." %
 *                            ("VERSION", self.name))
 */
    __pyx_t_10 = ((__pyx_v_ret < 0) != 0);
    if (unlikely(__pyx_t_10)) {

      /* "tables/tableextension.pyx":224
 *                                        len(encoded_obversion), cset)
 *       if ret < 0:
 *         raise HDF5ExtError("Can't set attribute '%s' in table:\n %s." %             # <<<<<<<<<<<<<<
 *                            ("VERSION", self.name))
 *       # Attach the TITLE attribute
 */
      __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_HDF5ExtError); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 224, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);

      /* "tables/tableextension.pyx":225
 *       if ret < 0:
 *         raise HDF5ExtError("Can't set attribute '%s' in table:\n %s." %
 *                            ("VERSION", self.name))             # <<<<<<<<<<<<<<
 *       # Attach the TITLE attribute
 *       ret = H5ATTRset_attribute_string(self.dataset_id, "TITLE", ctitle,
 */
      __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 225, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_n_s_VERSION);
      __Pyx_GIVEREF(__pyx_n_s_VERSION);
      PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_n_s_VERSION);
      __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.name);
      __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.name);
      PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_self->__pyx_base.__pyx_base.name);

      /* "tables/tableextension.pyx":224
 *                                        len(encoded_obversion), cset)
 *       if ret < 0:
 *         raise HDF5ExtError("Can't set attribute '%s' in table:\n %s." %             # <<<<<<<<<<<<<<
 *                            ("VERSION", self.name))
 *       # Attach the TITLE attribute
 */
      __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_Can_t_set_attribute_s_in_table_s, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 224, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_3)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
        }
      }
      __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_3, __pyx_t_2) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_2);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 224, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_Raise(__pyx_t_1, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __PYX_ERR(0, 224, __pyx_L1_error)

      /* "tables/tableextension.pyx":223
 *       ret = H5ATTRset_attribute_string(self.dataset_id, "VERSION", cobversion,
 *                                        len(encoded_obversion), cset)
 *       if ret < 0:             # <<<<<<<<<<<<<<
 *         raise HDF5ExtError("Can't set attribute '%s' in table:\n %s." %
 *                            ("VERSION", self.name))
 */
    }

    /* "tables/tableextension.pyx":228
 *       # Attach the TITLE attribute
 *       ret = H5ATTRset_attribute_string(self.dataset_id, "TITLE", ctitle,
 *                                        len(encoded_title), cset)             # <<<<<<<<<<<<<<
 *       if ret < 0:
 *         raise HDF5ExtError("Can't set attribute '%s' in table:\n %s." %
 */
    if (unlikely(__pyx_v_encoded_title == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 228, __pyx_L1_error)
    }
    __pyx_t_19 = PyBytes_GET_SIZE(__pyx_v_encoded_title); if (unlikely(__pyx_t_19 == ((Py_ssize_t)-1))) __PYX_ERR(0, 228, __pyx_L1_error)

    /* "tables/tableextension.pyx":227
 *                            ("VERSION", self.name))
 *       # Attach the TITLE attribute
 *       ret = H5ATTRset_attribute_string(self.dataset_id, "TITLE", ctitle,             # <<<<<<<<<<<<<<
 *                                        len(encoded_title), cset)
 *       if ret < 0:
 */
    __pyx_v_ret = H5ATTRset_attribute_string(__pyx_v_self->__pyx_base.dataset_id, ((char *)"TITLE"), __pyx_v_ctitle, __pyx_t_19, __pyx_v_cset);

    /* "tables/tableextension.pyx":229
 *       ret = H5ATTRset_attribute_string(self.dataset_id, "TITLE", ctitle,
 *                                        len(encoded_title), cset)
 *       if ret < 0:             # <<<<<<<<<<<<<<
 *         raise HDF5ExtError("Can't set attribute '%s' in table:\n %s." %
 *                            ("TITLE", self.name))
 */
    __pyx_t_10 = ((__pyx_v_ret < 0) != 0);
    if (unlikely(__pyx_t_10)) {

      /* "tables/tableextension.pyx":230
 *                                        len(encoded_title), cset)
 *       if ret < 0:
 *         raise HDF5ExtError("Can't set attribute '%s' in table:\n %s." %             # <<<<<<<<<<<<<<
 *                            ("TITLE", self.name))
 *       # Attach the NROWS attribute
 */
      __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_HDF5ExtError); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 230, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);

      /* "tables/tableextension.pyx":231
 *       if ret < 0:
 *         raise HDF5ExtError("Can't set attribute '%s' in table:\n %s." %
 *                            ("TITLE", self.name))             # <<<<<<<<<<<<<<
 *       # Attach the NROWS attribute
 *       nrows = self.nrows
 */
      __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 231, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_INCREF(__pyx_n_s_TITLE);
      __Pyx_GIVEREF(__pyx_n_s_TITLE);
      PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_TITLE);
      __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.name);
      __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.name);
      PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_self->__pyx_base.__pyx_base.name);

      /* "tables/tableextension.pyx":230
 *                                        len(encoded_title), cset)
 *       if ret < 0:
 *         raise HDF5ExtError("Can't set attribute '%s' in table:\n %s." %             # <<<<<<<<<<<<<<
 *                            ("TITLE", self.name))
 *       # Attach the NROWS attribute
 */
      __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_Can_t_set_attribute_s_in_table_s, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 230, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
        }
      }
      __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_3);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 230, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_Raise(__pyx_t_1, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __PYX_ERR(0, 230, __pyx_L1_error)

      /* "tables/tableextension.pyx":229
 *       ret = H5ATTRset_attribute_string(self.dataset_id, "TITLE", ctitle,
 *                                        len(encoded_title), cset)
 *       if ret < 0:             # <<<<<<<<<<<<<<
 *         raise HDF5ExtError("Can't set attribute '%s' in table:\n %s." %
 *                            ("TITLE", self.name))
 */
    }

    /* "tables/tableextension.pyx":233
 *                            ("TITLE", self.name))
 *       # Attach the NROWS attribute
 *       nrows = self.nrows             # <<<<<<<<<<<<<<
 *       ret = H5ATTRset_attribute(self.dataset_id, "NROWS", H5T_STD_I64,
 *                                 0, NULL, <char *>&nrows)
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nrows); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 233, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_14 = __Pyx_PyInt_As_hsize_t(__pyx_t_1); if (unlikely((__pyx_t_14 == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 233, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_nrows = __pyx_t_14;

    /* "tables/tableextension.pyx":234
 *       # Attach the NROWS attribute
 *       nrows = self.nrows
 *       ret = H5ATTRset_attribute(self.dataset_id, "NROWS", H5T_STD_I64,             # <<<<<<<<<<<<<<
 *                                 0, NULL, <char *>&nrows)
 *       if ret < 0:
 */
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_H5T_STD_I64); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 234, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_9 = __Pyx_PyInt_As_hid_t(__pyx_t_1); if (unlikely((__pyx_t_9 == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 234, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "tables/tableextension.pyx":235
 *       nrows = self.nrows
 *       ret = H5ATTRset_attribute(self.dataset_id, "NROWS", H5T_STD_I64,
 *                                 0, NULL, <char *>&nrows)             # <<<<<<<<<<<<<<
 *       if ret < 0:
 *         raise HDF5ExtError("Can't set attribute '%s' in table:\n %s." %
 */
    __pyx_v_ret = H5ATTRset_attribute(__pyx_v_self->__pyx_base.dataset_id, ((char *)"NROWS"), __pyx_t_9, 0, NULL, ((char *)(&__pyx_v_nrows)));

    /* "tables/tableextension.pyx":236
 *       ret = H5ATTRset_attribute(self.dataset_id, "NROWS", H5T_STD_I64,
 *                                 0, NULL, <char *>&nrows)
 *       if ret < 0:             # <<<<<<<<<<<<<<
 *         raise HDF5ExtError("Can't set attribute '%s' in table:\n %s." %
 *                            ("NROWS", self.name))
 */
    __pyx_t_10 = ((__pyx_v_ret < 0) != 0);
    if (unlikely(__pyx_t_10)) {

      /* "tables/tableextension.pyx":237
 *                                 0, NULL, <char *>&nrows)
 *       if ret < 0:
 *         raise HDF5ExtError("Can't set attribute '%s' in table:\n %s." %             # <<<<<<<<<<<<<<
 *                            ("NROWS", self.name))
 * 
 */
      __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_HDF5ExtError); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 237, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);

      /* "tables/tableextension.pyx":238
 *       if ret < 0:
 *         raise HDF5ExtError("Can't set attribute '%s' in table:\n %s." %
 *                            ("NROWS", self.name))             # <<<<<<<<<<<<<<
 * 
 *       # Attach the FIELD_N_NAME attributes
 */
      __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 238, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_n_s_NROWS);
      __Pyx_GIVEREF(__pyx_n_s_NROWS);
      PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_n_s_NROWS);
      __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.name);
      __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.name);
      PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_self->__pyx_base.__pyx_base.name);

      /* "tables/tableextension.pyx":237
 *                                 0, NULL, <char *>&nrows)
 *       if ret < 0:
 *         raise HDF5ExtError("Can't set attribute '%s' in table:\n %s." %             # <<<<<<<<<<<<<<
 *                            ("NROWS", self.name))
 * 
 */
      __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_Can_t_set_attribute_s_in_table_s, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 237, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_3)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
        }
      }
      __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_3, __pyx_t_2) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_2);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 237, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_Raise(__pyx_t_1, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __PYX_ERR(0, 237, __pyx_L1_error)

      /* "tables/tableextension.pyx":236
 *       ret = H5ATTRset_attribute(self.dataset_id, "NROWS", H5T_STD_I64,
 *                                 0, NULL, <char *>&nrows)
 *       if ret < 0:             # <<<<<<<<<<<<<<
 *         raise HDF5ExtError("Can't set attribute '%s' in table:\n %s." %
 *                            ("NROWS", self.name))
 */
    }

    /* "tables/tableextension.pyx":242
 *       # Attach the FIELD_N_NAME attributes
 *       # We write only the first level names
 *       for i, name in enumerate(self.description._v_names):             # <<<<<<<<<<<<<<
 *         snprintf(fieldname, 128, "FIELD_%d_NAME", i)
 *         encoded_name = name.encode('utf-8')
 */
    __pyx_t_17 = 0;
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_description); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 242, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_v_names); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 242, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (likely(PyList_CheckExact(__pyx_t_6)) || PyTuple_CheckExact(__pyx_t_6)) {
      __pyx_t_1 = __pyx_t_6; __Pyx_INCREF(__pyx_t_1); __pyx_t_19 = 0;
      __pyx_t_20 = NULL;
    } else {
      __pyx_t_19 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 242, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_20 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 242, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    for (;;) {
      if (likely(!__pyx_t_20)) {
        if (likely(PyList_CheckExact(__pyx_t_1))) {
          if (__pyx_t_19 >= PyList_GET_SIZE(__pyx_t_1)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_6 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_19); __Pyx_INCREF(__pyx_t_6); __pyx_t_19++; if (unlikely(0 < 0)) __PYX_ERR(0, 242, __pyx_L1_error)
          #else
          __pyx_t_6 = PySequence_ITEM(__pyx_t_1, __pyx_t_19); __pyx_t_19++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 242, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          #endif
        } else {
          if (__pyx_t_19 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_6 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_19); __Pyx_INCREF(__pyx_t_6); __pyx_t_19++; if (unlikely(0 < 0)) __PYX_ERR(0, 242, __pyx_L1_error)
          #else
          __pyx_t_6 = PySequence_ITEM(__pyx_t_1, __pyx_t_19); __pyx_t_19++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 242, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          #endif
        }
      } else {
        __pyx_t_6 = __pyx_t_20(__pyx_t_1);
        if (unlikely(!__pyx_t_6)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 242, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_6);
      }
      __Pyx_XDECREF_SET(__pyx_v_name, __pyx_t_6);
      __pyx_t_6 = 0;
      __pyx_v_i = __pyx_t_17;
      __pyx_t_17 = (__pyx_t_17 + 1);

      /* "tables/tableextension.pyx":243
 *       # We write only the first level names
 *       for i, name in enumerate(self.description._v_names):
 *         snprintf(fieldname, 128, "FIELD_%d_NAME", i)             # <<<<<<<<<<<<<<
 *         encoded_name = name.encode('utf-8')
 *         ret = H5ATTRset_attribute_string(self.dataset_id, fieldname,
 */
      (void)(snprintf(__pyx_v_fieldname, 0x80, ((char const *)"FIELD_%d_NAME"), __pyx_v_i));

      /* "tables/tableextension.pyx":244
 *       for i, name in enumerate(self.description._v_names):
 *         snprintf(fieldname, 128, "FIELD_%d_NAME", i)
 *         encoded_name = name.encode('utf-8')             # <<<<<<<<<<<<<<
 *         ret = H5ATTRset_attribute_string(self.dataset_id, fieldname,
 *                                          encoded_name, len(encoded_name),
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_name, __pyx_n_s_encode); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 244, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_3)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_6 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_kp_s_utf_8) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_s_utf_8);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 244, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (!(likely(PyBytes_CheckExact(__pyx_t_6))||((__pyx_t_6) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_6)->tp_name), 0))) __PYX_ERR(0, 244, __pyx_L1_error)
      __Pyx_DECREF_SET(__pyx_v_encoded_name, ((PyObject*)__pyx_t_6));
      __pyx_t_6 = 0;

      /* "tables/tableextension.pyx":246
 *         encoded_name = name.encode('utf-8')
 *         ret = H5ATTRset_attribute_string(self.dataset_id, fieldname,
 *                                          encoded_name, len(encoded_name),             # <<<<<<<<<<<<<<
 *                                          cset)
 *       if ret < 0:
 */
      if (unlikely(__pyx_v_encoded_name == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
        __PYX_ERR(0, 246, __pyx_L1_error)
      }
      __pyx_t_15 = __Pyx_PyBytes_AsWritableString(__pyx_v_encoded_name); if (unlikely((!__pyx_t_15) && PyErr_Occurred())) __PYX_ERR(0, 246, __pyx_L1_error)
      if (unlikely(__pyx_v_encoded_name == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
        __PYX_ERR(0, 246, __pyx_L1_error)
      }
      __pyx_t_21 = PyBytes_GET_SIZE(__pyx_v_encoded_name); if (unlikely(__pyx_t_21 == ((Py_ssize_t)-1))) __PYX_ERR(0, 246, __pyx_L1_error)

      /* "tables/tableextension.pyx":245
 *         snprintf(fieldname, 128, "FIELD_%d_NAME", i)
 *         encoded_name = name.encode('utf-8')
 *         ret = H5ATTRset_attribute_string(self.dataset_id, fieldname,             # <<<<<<<<<<<<<<
 *                                          encoded_name, len(encoded_name),
 *                                          cset)
 */
      __pyx_v_ret = H5ATTRset_attribute_string(__pyx_v_self->__pyx_base.dataset_id, __pyx_v_fieldname, __pyx_t_15, __pyx_t_21, __pyx_v_cset);

      /* "tables/tableextension.pyx":242
 *       # Attach the FIELD_N_NAME attributes
 *       # We write only the first level names
 *       for i, name in enumerate(self.description._v_names):             # <<<<<<<<<<<<<<
 *         snprintf(fieldname, 128, "FIELD_%d_NAME", i)
 *         encoded_name = name.encode('utf-8')
 */
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "tables/tableextension.pyx":248
 *                                          encoded_name, len(encoded_name),
 *                                          cset)
 *       if ret < 0:             # <<<<<<<<<<<<<<
 *         raise HDF5ExtError("Can't set attribute '%s' in table:\n %s." %
 *                            (fieldname, self.name))
 */
    __pyx_t_10 = ((__pyx_v_ret < 0) != 0);
    if (unlikely(__pyx_t_10)) {

      /* "tables/tableextension.pyx":249
 *                                          cset)
 *       if ret < 0:
 *         raise HDF5ExtError("Can't set attribute '%s' in table:\n %s." %             # <<<<<<<<<<<<<<
 *                            (fieldname, self.name))
 * 
 */
      __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_HDF5ExtError); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 249, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);

      /* "tables/tableextension.pyx":250
 *       if ret < 0:
 *         raise HDF5ExtError("Can't set attribute '%s' in table:\n %s." %
 *                            (fieldname, self.name))             # <<<<<<<<<<<<<<
 * 
 *     # If created in PyTables, the table is always chunked
 */
      __pyx_t_2 = __Pyx_PyObject_FromString(__pyx_v_fieldname); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 250, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 250, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_2);
      PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
      __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.name);
      __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.name);
      PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_self->__pyx_base.__pyx_base.name);
      __pyx_t_2 = 0;

      /* "tables/tableextension.pyx":249
 *                                          cset)
 *       if ret < 0:
 *         raise HDF5ExtError("Can't set attribute '%s' in table:\n %s." %             # <<<<<<<<<<<<<<
 *                            (fieldname, self.name))
 * 
 */
      __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_Can_t_set_attribute_s_in_table_s, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 249, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_3)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
        }
      }
      __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_3, __pyx_t_2) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_2);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 249, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_Raise(__pyx_t_1, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __PYX_ERR(0, 249, __pyx_L1_error)

      /* "tables/tableextension.pyx":248
 *                                          encoded_name, len(encoded_name),
 *                                          cset)
 *       if ret < 0:             # <<<<<<<<<<<<<<
 *         raise HDF5ExtError("Can't set attribute '%s' in table:\n %s." %
 *                            (fieldname, self.name))
 */
    }

    /* "tables/tableextension.pyx":211
 *       raise HDF5ExtError("Problems creating the table")
 * 
 *     if self._v_file.params['PYTABLES_SYS_ATTRS']:             # <<<<<<<<<<<<<<
 *       cset = H5T_CSET_UTF8
 *       # Set the conforming table attributes
 */
  }

  /* "tables/tableextension.pyx":253
 * 
 *     # If created in PyTables, the table is always chunked
 *     self._chunked = True  # Accessible from python             # <<<<<<<<<<<<<<
 * 
 *     # Finally, return the object identifier.
 */
  if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_chunked, Py_True) < 0) __PYX_ERR(0, 253, __pyx_L1_error)

  /* "tables/tableextension.pyx":256
 * 
 *     # Finally, return the object identifier.
 *     return self.dataset_id             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_hid_t(__pyx_v_self->__pyx_base.dataset_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 256, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "tables/tableextension.pyx":149
 *   cdef void     *wbuf
 * 
 *   def _create_table(self, title, complib, obversion):             # <<<<<<<<<<<<<<
 *     cdef int     offset
 *     cdef int     ret
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("tables.tableextension.Table._create_table", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_class_);
  __Pyx_XDECREF((PyObject *)__pyx_v_wdflts);
  __Pyx_XDECREF((PyObject *)__pyx_v_recarr);
  __Pyx_XDECREF(__pyx_v_name);
  __Pyx_XDECREF(__pyx_v_encoded_title);
  __Pyx_XDECREF(__pyx_v_encoded_complib);
  __Pyx_XDECREF(__pyx_v_encoded_obversion);
  __Pyx_XDECREF(__pyx_v_encoded_name);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":259
 * 
 * 
 *   cdef get_nested_type(self, hid_t type_id, hid_t native_type_id,             # <<<<<<<<<<<<<<
 *                        object colpath, object field_byteorders):
 *     """Open a nested type and return a nested dictionary as description."""
 */

static PyObject *__pyx_f_6tables_14tableextension_5Table_get_nested_type(struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self, hid_t __pyx_v_type_id, hid_t __pyx_v_native_type_id, PyObject *__pyx_v_colpath, PyObject *__pyx_v_field_byteorders) {
  hid_t __pyx_v_member_type_id;
  hid_t __pyx_v_native_member_type_id;
  hid_t __pyx_v_member_offset;
  hsize_t __pyx_v_nfields;
  hsize_t __pyx_v_i;
  size_t __pyx_v_itemsize;
  char *__pyx_v_c_colname;
  enum H5T_class_t __pyx_v_class_id;
  char __pyx_v_c_byteorder2[11];
  PyObject *__pyx_v_desc = 0;
  PyObject *__pyx_v_colobj = 0;
  PyObject *__pyx_v_colpath2 = 0;
  PyObject *__pyx_v_byteorder = 0;
  PyObject *__pyx_v_colname = 0;
  PyObject *__pyx_v_byteorder2 = 0;
  PyObject *__pyx_v_offset = NULL;
  PyObject *__pyx_v_position_order = NULL;
  PyObject *__pyx_v_pos = NULL;
  PyObject *__pyx_v_atom = NULL;
  PyObject *__pyx_v_te = NULL;
  PyObject *__pyx_v_x = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  hsize_t __pyx_t_2;
  hsize_t __pyx_t_3;
  hsize_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  int __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *(*__pyx_t_14)(PyObject *);
  size_t __pyx_t_15;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  int __pyx_t_19;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  PyObject *__pyx_t_22 = NULL;
  int __pyx_t_23;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_nested_type", 0);
  __Pyx_INCREF(__pyx_v_field_byteorders);

  /* "tables/tableextension.pyx":275
 *     cdef str     colname, byteorder2
 * 
 *     offset = 0             # <<<<<<<<<<<<<<
 *     desc = {}
 *     # Get the number of members
 */
  __Pyx_INCREF(__pyx_int_0);
  __pyx_v_offset = __pyx_int_0;

  /* "tables/tableextension.pyx":276
 * 
 *     offset = 0
 *     desc = {}             # <<<<<<<<<<<<<<
 *     # Get the number of members
 *     nfields = H5Tget_nmembers(type_id)
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 276, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_desc = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":278
 *     desc = {}
 *     # Get the number of members
 *     nfields = H5Tget_nmembers(type_id)             # <<<<<<<<<<<<<<
 * 
 *     # Iterate through fields to get the correct order that elements may appear in
 */
  __pyx_v_nfields = H5Tget_nmembers(__pyx_v_type_id);

  /* "tables/tableextension.pyx":282
 *     # Iterate through fields to get the correct order that elements may appear in
 *     # The object type can be stored not in order, so order based on the offset in the data
 *     position_order = []             # <<<<<<<<<<<<<<
 *     for i in range(nfields):
 *       member_offset = H5Tget_member_offset(type_id, i)
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 282, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_position_order = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":283
 *     # The object type can be stored not in order, so order based on the offset in the data
 *     position_order = []
 *     for i in range(nfields):             # <<<<<<<<<<<<<<
 *       member_offset = H5Tget_member_offset(type_id, i)
 *       position_order.append((member_offset, i))
 */
  __pyx_t_2 = __pyx_v_nfields;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "tables/tableextension.pyx":284
 *     position_order = []
 *     for i in range(nfields):
 *       member_offset = H5Tget_member_offset(type_id, i)             # <<<<<<<<<<<<<<
 *       position_order.append((member_offset, i))
 * 
 */
    __pyx_v_member_offset = H5Tget_member_offset(__pyx_v_type_id, __pyx_v_i);

    /* "tables/tableextension.pyx":285
 *     for i in range(nfields):
 *       member_offset = H5Tget_member_offset(type_id, i)
 *       position_order.append((member_offset, i))             # <<<<<<<<<<<<<<
 * 
 *     position_order.sort()
 */
    __pyx_t_1 = __Pyx_PyInt_From_hid_t(__pyx_v_member_offset); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 285, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_PyInt_From_hsize_t(__pyx_v_i); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 285, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 285, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_5);
    __pyx_t_1 = 0;
    __pyx_t_5 = 0;
    __pyx_t_7 = __Pyx_PyList_Append(__pyx_v_position_order, __pyx_t_6); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 285, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }

  /* "tables/tableextension.pyx":287
 *       position_order.append((member_offset, i))
 * 
 *     position_order.sort()             # <<<<<<<<<<<<<<
 * 
 *     # Iterate thru the members
 */
  __pyx_t_7 = PyList_Sort(__pyx_v_position_order); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 287, __pyx_L1_error)

  /* "tables/tableextension.pyx":290
 * 
 *     # Iterate thru the members
 *     for pos, i in enumerate([x[1] for x in position_order]):             # <<<<<<<<<<<<<<
 *       # Get the member name
 *       c_colname = H5Tget_member_name(type_id, i)
 */
  __Pyx_INCREF(__pyx_int_0);
  __pyx_t_6 = __pyx_int_0;
  __pyx_t_5 = PyList_New(0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 290, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __pyx_v_position_order; __Pyx_INCREF(__pyx_t_1); __pyx_t_8 = 0;
  for (;;) {
    if (__pyx_t_8 >= PyList_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_9 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_8); __Pyx_INCREF(__pyx_t_9); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 290, __pyx_L1_error)
    #else
    __pyx_t_9 = PySequence_ITEM(__pyx_t_1, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 290, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_x, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_GetItemInt(__pyx_v_x, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 290, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_5, (PyObject*)__pyx_t_9))) __PYX_ERR(0, 290, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __pyx_t_5; __Pyx_INCREF(__pyx_t_1); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  for (;;) {
    if (__pyx_t_8 >= PyList_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_5 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_8); __Pyx_INCREF(__pyx_t_5); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 290, __pyx_L1_error)
    #else
    __pyx_t_5 = PySequence_ITEM(__pyx_t_1, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 290, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    #endif
    __pyx_t_2 = __Pyx_PyInt_As_hsize_t(__pyx_t_5); if (unlikely((__pyx_t_2 == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 290, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_i = __pyx_t_2;
    __Pyx_INCREF(__pyx_t_6);
    __Pyx_XDECREF_SET(__pyx_v_pos, __pyx_t_6);
    __pyx_t_5 = __Pyx_PyInt_AddObjC(__pyx_t_6, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 290, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6);
    __pyx_t_6 = __pyx_t_5;
    __pyx_t_5 = 0;

    /* "tables/tableextension.pyx":292
 *     for pos, i in enumerate([x[1] for x in position_order]):
 *       # Get the member name
 *       c_colname = H5Tget_member_name(type_id, i)             # <<<<<<<<<<<<<<
 *       colname = cstr_to_pystr(c_colname)
 * 
 */
    __pyx_v_c_colname = H5Tget_member_name(__pyx_v_type_id, __pyx_v_i);

    /* "tables/tableextension.pyx":293
 *       # Get the member name
 *       c_colname = H5Tget_member_name(type_id, i)
 *       colname = cstr_to_pystr(c_colname)             # <<<<<<<<<<<<<<
 * 
 *       # Get the member type
 */
    __pyx_t_5 = __pyx_f_6tables_14utilsextension_cstr_to_pystr(__pyx_v_c_colname); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 293, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_XDECREF_SET(__pyx_v_colname, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "tables/tableextension.pyx":296
 * 
 *       # Get the member type
 *       member_type_id = H5Tget_member_type(type_id, i)             # <<<<<<<<<<<<<<
 *       # Get the member offset
 *       member_offset = H5Tget_member_offset(type_id, i)
 */
    __pyx_v_member_type_id = H5Tget_member_type(__pyx_v_type_id, __pyx_v_i);

    /* "tables/tableextension.pyx":298
 *       member_type_id = H5Tget_member_type(type_id, i)
 *       # Get the member offset
 *       member_offset = H5Tget_member_offset(type_id, i)             # <<<<<<<<<<<<<<
 *       # Get the HDF5 class
 *       class_id = H5Tget_class(member_type_id)
 */
    __pyx_v_member_offset = H5Tget_member_offset(__pyx_v_type_id, __pyx_v_i);

    /* "tables/tableextension.pyx":300
 *       member_offset = H5Tget_member_offset(type_id, i)
 *       # Get the HDF5 class
 *       class_id = H5Tget_class(member_type_id)             # <<<<<<<<<<<<<<
 *       if class_id == H5T_COMPOUND and not is_complex(member_type_id):
 *         colpath2 = join_path(colpath, colname)
 */
    __pyx_v_class_id = H5Tget_class(__pyx_v_member_type_id);

    /* "tables/tableextension.pyx":301
 *       # Get the HDF5 class
 *       class_id = H5Tget_class(member_type_id)
 *       if class_id == H5T_COMPOUND and not is_complex(member_type_id):             # <<<<<<<<<<<<<<
 *         colpath2 = join_path(colpath, colname)
 *         # Create the native data in-memory
 */
    __pyx_t_11 = ((__pyx_v_class_id == H5T_COMPOUND) != 0);
    if (__pyx_t_11) {
    } else {
      __pyx_t_10 = __pyx_t_11;
      goto __pyx_L10_bool_binop_done;
    }
    __pyx_t_11 = ((!(is_complex(__pyx_v_member_type_id) != 0)) != 0);
    __pyx_t_10 = __pyx_t_11;
    __pyx_L10_bool_binop_done:;
    if (__pyx_t_10) {

      /* "tables/tableextension.pyx":302
 *       class_id = H5Tget_class(member_type_id)
 *       if class_id == H5T_COMPOUND and not is_complex(member_type_id):
 *         colpath2 = join_path(colpath, colname)             # <<<<<<<<<<<<<<
 *         # Create the native data in-memory
 *         itemsize = H5Tget_size(member_type_id)
 */
      __pyx_t_5 = __pyx_f_6tables_14tableextension_join_path(__pyx_v_colpath, __pyx_v_colname); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 302, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_XDECREF_SET(__pyx_v_colpath2, __pyx_t_5);
      __pyx_t_5 = 0;

      /* "tables/tableextension.pyx":304
 *         colpath2 = join_path(colpath, colname)
 *         # Create the native data in-memory
 *         itemsize = H5Tget_size(member_type_id)             # <<<<<<<<<<<<<<
 *         native_member_type_id = H5Tcreate(H5T_COMPOUND, itemsize)
 *         desc[colname], itemsize = self.get_nested_type(
 */
      __pyx_v_itemsize = H5Tget_size(__pyx_v_member_type_id);

      /* "tables/tableextension.pyx":305
 *         # Create the native data in-memory
 *         itemsize = H5Tget_size(member_type_id)
 *         native_member_type_id = H5Tcreate(H5T_COMPOUND, itemsize)             # <<<<<<<<<<<<<<
 *         desc[colname], itemsize = self.get_nested_type(
 *           member_type_id, native_member_type_id, colpath2, field_byteorders)
 */
      __pyx_v_native_member_type_id = H5Tcreate(H5T_COMPOUND, __pyx_v_itemsize);

      /* "tables/tableextension.pyx":306
 *         itemsize = H5Tget_size(member_type_id)
 *         native_member_type_id = H5Tcreate(H5T_COMPOUND, itemsize)
 *         desc[colname], itemsize = self.get_nested_type(             # <<<<<<<<<<<<<<
 *           member_type_id, native_member_type_id, colpath2, field_byteorders)
 *         desc[colname]["_v_pos"] = pos
 */
      __pyx_t_5 = ((struct __pyx_vtabstruct_6tables_14tableextension_Table *)__pyx_v_self->__pyx_base.__pyx_vtab)->get_nested_type(__pyx_v_self, __pyx_v_member_type_id, __pyx_v_native_member_type_id, __pyx_v_colpath2, __pyx_v_field_byteorders); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 306, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if ((likely(PyTuple_CheckExact(__pyx_t_5))) || (PyList_CheckExact(__pyx_t_5))) {
        PyObject* sequence = __pyx_t_5;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 306, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_9 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_12 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_9 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_12 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_12);
        #else
        __pyx_t_9 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 306, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __pyx_t_12 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 306, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        #endif
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_13 = PyObject_GetIter(__pyx_t_5); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 306, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_14 = Py_TYPE(__pyx_t_13)->tp_iternext;
        index = 0; __pyx_t_9 = __pyx_t_14(__pyx_t_13); if (unlikely(!__pyx_t_9)) goto __pyx_L12_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_9);
        index = 1; __pyx_t_12 = __pyx_t_14(__pyx_t_13); if (unlikely(!__pyx_t_12)) goto __pyx_L12_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_12);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_14(__pyx_t_13), 2) < 0) __PYX_ERR(0, 306, __pyx_L1_error)
        __pyx_t_14 = NULL;
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        goto __pyx_L13_unpacking_done;
        __pyx_L12_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        __pyx_t_14 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 306, __pyx_L1_error)
        __pyx_L13_unpacking_done:;
      }
      __pyx_t_15 = __Pyx_PyInt_As_size_t(__pyx_t_12); if (unlikely((__pyx_t_15 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 306, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      if (unlikely(PyObject_SetItem(__pyx_v_desc, __pyx_v_colname, __pyx_t_9) < 0)) __PYX_ERR(0, 306, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_v_itemsize = __pyx_t_15;

      /* "tables/tableextension.pyx":308
 *         desc[colname], itemsize = self.get_nested_type(
 *           member_type_id, native_member_type_id, colpath2, field_byteorders)
 *         desc[colname]["_v_pos"] = pos             # <<<<<<<<<<<<<<
 *         desc[colname]["_v_offset"] = member_offset
 *       else:
 */
      __pyx_t_5 = __Pyx_PyObject_Dict_GetItem(__pyx_v_desc, __pyx_v_colname); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 308, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (unlikely(PyObject_SetItem(__pyx_t_5, __pyx_n_s_v_pos, __pyx_v_pos) < 0)) __PYX_ERR(0, 308, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "tables/tableextension.pyx":309
 *           member_type_id, native_member_type_id, colpath2, field_byteorders)
 *         desc[colname]["_v_pos"] = pos
 *         desc[colname]["_v_offset"] = member_offset             # <<<<<<<<<<<<<<
 *       else:
 *         # Get the member format and the corresponding Col object
 */
      __pyx_t_5 = __Pyx_PyInt_From_hid_t(__pyx_v_member_offset); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 309, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_12 = __Pyx_PyObject_Dict_GetItem(__pyx_v_desc, __pyx_v_colname); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 309, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      if (unlikely(PyObject_SetItem(__pyx_t_12, __pyx_n_s_v_offset, __pyx_t_5) < 0)) __PYX_ERR(0, 309, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "tables/tableextension.pyx":301
 *       # Get the HDF5 class
 *       class_id = H5Tget_class(member_type_id)
 *       if class_id == H5T_COMPOUND and not is_complex(member_type_id):             # <<<<<<<<<<<<<<
 *         colpath2 = join_path(colpath, colname)
 *         # Create the native data in-memory
 */
      goto __pyx_L9;
    }

    /* "tables/tableextension.pyx":312
 *       else:
 *         # Get the member format and the corresponding Col object
 *         try:             # <<<<<<<<<<<<<<
 *           native_member_type_id = get_native_type(member_type_id)
 *           atom = atom_from_hdf5_type(native_member_type_id)
 */
    /*else*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_16, &__pyx_t_17, &__pyx_t_18);
        __Pyx_XGOTREF(__pyx_t_16);
        __Pyx_XGOTREF(__pyx_t_17);
        __Pyx_XGOTREF(__pyx_t_18);
        /*try:*/ {

          /* "tables/tableextension.pyx":313
 *         # Get the member format and the corresponding Col object
 *         try:
 *           native_member_type_id = get_native_type(member_type_id)             # <<<<<<<<<<<<<<
 *           atom = atom_from_hdf5_type(native_member_type_id)
 *           colobj = Col.from_atom(atom, pos=pos, _offset=member_offset)
 */
          __pyx_v_native_member_type_id = __pyx_f_6tables_14utilsextension_get_native_type(__pyx_v_member_type_id);

          /* "tables/tableextension.pyx":314
 *         try:
 *           native_member_type_id = get_native_type(member_type_id)
 *           atom = atom_from_hdf5_type(native_member_type_id)             # <<<<<<<<<<<<<<
 *           colobj = Col.from_atom(atom, pos=pos, _offset=member_offset)
 *           itemsize = H5Tget_size(native_member_type_id)
 */
          __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_n_s_atom_from_hdf5_type); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 314, __pyx_L14_error)
          __Pyx_GOTREF(__pyx_t_12);
          __pyx_t_9 = __Pyx_PyInt_From_hid_t(__pyx_v_native_member_type_id); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 314, __pyx_L14_error)
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_13 = NULL;
          if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_12))) {
            __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_12);
            if (likely(__pyx_t_13)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
              __Pyx_INCREF(__pyx_t_13);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_12, function);
            }
          }
          __pyx_t_5 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_12, __pyx_t_13, __pyx_t_9) : __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_t_9);
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 314, __pyx_L14_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_XDECREF_SET(__pyx_v_atom, __pyx_t_5);
          __pyx_t_5 = 0;

          /* "tables/tableextension.pyx":315
 *           native_member_type_id = get_native_type(member_type_id)
 *           atom = atom_from_hdf5_type(native_member_type_id)
 *           colobj = Col.from_atom(atom, pos=pos, _offset=member_offset)             # <<<<<<<<<<<<<<
 *           itemsize = H5Tget_size(native_member_type_id)
 *         except TypeError, te:
 */
          __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_Col); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 315, __pyx_L14_error)
          __Pyx_GOTREF(__pyx_t_5);
          __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_from_atom); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 315, __pyx_L14_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 315, __pyx_L14_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_INCREF(__pyx_v_atom);
          __Pyx_GIVEREF(__pyx_v_atom);
          PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_atom);
          __pyx_t_9 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 315, __pyx_L14_error)
          __Pyx_GOTREF(__pyx_t_9);
          if (PyDict_SetItem(__pyx_t_9, __pyx_n_s_pos, __pyx_v_pos) < 0) __PYX_ERR(0, 315, __pyx_L14_error)
          __pyx_t_13 = __Pyx_PyInt_From_hid_t(__pyx_v_member_offset); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 315, __pyx_L14_error)
          __Pyx_GOTREF(__pyx_t_13);
          if (PyDict_SetItem(__pyx_t_9, __pyx_n_s_offset, __pyx_t_13) < 0) __PYX_ERR(0, 315, __pyx_L14_error)
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __pyx_t_13 = __Pyx_PyObject_Call(__pyx_t_12, __pyx_t_5, __pyx_t_9); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 315, __pyx_L14_error)
          __Pyx_GOTREF(__pyx_t_13);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          __Pyx_XDECREF_SET(__pyx_v_colobj, __pyx_t_13);
          __pyx_t_13 = 0;

          /* "tables/tableextension.pyx":316
 *           atom = atom_from_hdf5_type(native_member_type_id)
 *           colobj = Col.from_atom(atom, pos=pos, _offset=member_offset)
 *           itemsize = H5Tget_size(native_member_type_id)             # <<<<<<<<<<<<<<
 *         except TypeError, te:
 *           # Re-raise TypeError again with more info
 */
          __pyx_v_itemsize = H5Tget_size(__pyx_v_native_member_type_id);

          /* "tables/tableextension.pyx":312
 *       else:
 *         # Get the member format and the corresponding Col object
 *         try:             # <<<<<<<<<<<<<<
 *           native_member_type_id = get_native_type(member_type_id)
 *           atom = atom_from_hdf5_type(native_member_type_id)
 */
        }
        __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
        __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
        __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L21_try_end;
        __pyx_L14_error:;
        __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

        /* "tables/tableextension.pyx":317
 *           colobj = Col.from_atom(atom, pos=pos, _offset=member_offset)
 *           itemsize = H5Tget_size(native_member_type_id)
 *         except TypeError, te:             # <<<<<<<<<<<<<<
 *           # Re-raise TypeError again with more info
 *           raise TypeError(
 */
        __pyx_t_19 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_TypeError);
        if (__pyx_t_19) {
          __Pyx_AddTraceback("tables.tableextension.Table.get_nested_type", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_13, &__pyx_t_9, &__pyx_t_5) < 0) __PYX_ERR(0, 317, __pyx_L16_except_error)
          __Pyx_GOTREF(__pyx_t_13);
          __Pyx_GOTREF(__pyx_t_9);
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_9);
          __pyx_v_te = __pyx_t_9;

          /* "tables/tableextension.pyx":320
 *           # Re-raise TypeError again with more info
 *           raise TypeError(
 *             ("table ``%s``, column ``%s``: %%s" % (self.name, colname))             # <<<<<<<<<<<<<<
 *             % te.args[0])
 *         desc[colname] = colobj
 */
          __pyx_t_12 = PyTuple_New(2); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 320, __pyx_L16_except_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.name);
          __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.name);
          PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_v_self->__pyx_base.__pyx_base.name);
          __Pyx_INCREF(__pyx_v_colname);
          __Pyx_GIVEREF(__pyx_v_colname);
          PyTuple_SET_ITEM(__pyx_t_12, 1, __pyx_v_colname);
          __pyx_t_20 = __Pyx_PyString_Format(__pyx_kp_s_table_s_column_s_s, __pyx_t_12); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 320, __pyx_L16_except_error)
          __Pyx_GOTREF(__pyx_t_20);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

          /* "tables/tableextension.pyx":321
 *           raise TypeError(
 *             ("table ``%s``, column ``%s``: %%s" % (self.name, colname))
 *             % te.args[0])             # <<<<<<<<<<<<<<
 *         desc[colname] = colobj
 *         # For time kinds, save the byteorder of the column
 */
          __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_v_te, __pyx_n_s_args); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 321, __pyx_L16_except_error)
          __Pyx_GOTREF(__pyx_t_12);
          __pyx_t_21 = __Pyx_GetItemInt(__pyx_t_12, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_21)) __PYX_ERR(0, 321, __pyx_L16_except_error)
          __Pyx_GOTREF(__pyx_t_21);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          __pyx_t_12 = PyNumber_Remainder(__pyx_t_20, __pyx_t_21); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 321, __pyx_L16_except_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
          __Pyx_DECREF(__pyx_t_21); __pyx_t_21 = 0;

          /* "tables/tableextension.pyx":319
 *         except TypeError, te:
 *           # Re-raise TypeError again with more info
 *           raise TypeError(             # <<<<<<<<<<<<<<
 *             ("table ``%s``, column ``%s``: %%s" % (self.name, colname))
 *             % te.args[0])
 */
          __pyx_t_21 = __Pyx_PyObject_CallOneArg(__pyx_builtin_TypeError, __pyx_t_12); if (unlikely(!__pyx_t_21)) __PYX_ERR(0, 319, __pyx_L16_except_error)
          __Pyx_GOTREF(__pyx_t_21);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_Raise(__pyx_t_21, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_21); __pyx_t_21 = 0;
          __PYX_ERR(0, 319, __pyx_L16_except_error)
        }
        goto __pyx_L16_except_error;
        __pyx_L16_except_error:;

        /* "tables/tableextension.pyx":312
 *       else:
 *         # Get the member format and the corresponding Col object
 *         try:             # <<<<<<<<<<<<<<
 *           native_member_type_id = get_native_type(member_type_id)
 *           atom = atom_from_hdf5_type(native_member_type_id)
 */
        __Pyx_XGIVEREF(__pyx_t_16);
        __Pyx_XGIVEREF(__pyx_t_17);
        __Pyx_XGIVEREF(__pyx_t_18);
        __Pyx_ExceptionReset(__pyx_t_16, __pyx_t_17, __pyx_t_18);
        goto __pyx_L1_error;
        __pyx_L21_try_end:;
      }

      /* "tables/tableextension.pyx":322
 *             ("table ``%s``, column ``%s``: %%s" % (self.name, colname))
 *             % te.args[0])
 *         desc[colname] = colobj             # <<<<<<<<<<<<<<
 *         # For time kinds, save the byteorder of the column
 *         # (useful for conversion of time datatypes later on)
 */
      if (unlikely(PyObject_SetItem(__pyx_v_desc, __pyx_v_colname, __pyx_v_colobj) < 0)) __PYX_ERR(0, 322, __pyx_L1_error)

      /* "tables/tableextension.pyx":325
 *         # For time kinds, save the byteorder of the column
 *         # (useful for conversion of time datatypes later on)
 *         if colobj.kind == "time":             # <<<<<<<<<<<<<<
 *           colobj._byteorder = H5Tget_order(member_type_id)
 *           if colobj._byteorder == H5T_ORDER_LE:
 */
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_colobj, __pyx_n_s_kind); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 325, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_10 = (__Pyx_PyString_Equals(__pyx_t_5, __pyx_n_s_time, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 325, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (__pyx_t_10) {

        /* "tables/tableextension.pyx":326
 *         # (useful for conversion of time datatypes later on)
 *         if colobj.kind == "time":
 *           colobj._byteorder = H5Tget_order(member_type_id)             # <<<<<<<<<<<<<<
 *           if colobj._byteorder == H5T_ORDER_LE:
 *             field_byteorders.append("little")
 */
        __pyx_t_5 = __Pyx_PyInt_From_enum__H5T_order_t(H5Tget_order(__pyx_v_member_type_id)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 326, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        if (__Pyx_PyObject_SetAttrStr(__pyx_v_colobj, __pyx_n_s_byteorder_2, __pyx_t_5) < 0) __PYX_ERR(0, 326, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

        /* "tables/tableextension.pyx":327
 *         if colobj.kind == "time":
 *           colobj._byteorder = H5Tget_order(member_type_id)
 *           if colobj._byteorder == H5T_ORDER_LE:             # <<<<<<<<<<<<<<
 *             field_byteorders.append("little")
 *           else:
 */
        __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_colobj, __pyx_n_s_byteorder_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 327, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_9 = __Pyx_PyInt_From_enum__H5T_order_t(H5T_ORDER_LE); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 327, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __pyx_t_13 = PyObject_RichCompare(__pyx_t_5, __pyx_t_9, Py_EQ); __Pyx_XGOTREF(__pyx_t_13); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 327, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_13); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 327, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        if (__pyx_t_10) {

          /* "tables/tableextension.pyx":328
 *           colobj._byteorder = H5Tget_order(member_type_id)
 *           if colobj._byteorder == H5T_ORDER_LE:
 *             field_byteorders.append("little")             # <<<<<<<<<<<<<<
 *           else:
 *             field_byteorders.append("big")
 */
          __pyx_t_7 = __Pyx_PyObject_Append(__pyx_v_field_byteorders, __pyx_n_s_little); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 328, __pyx_L1_error)

          /* "tables/tableextension.pyx":327
 *         if colobj.kind == "time":
 *           colobj._byteorder = H5Tget_order(member_type_id)
 *           if colobj._byteorder == H5T_ORDER_LE:             # <<<<<<<<<<<<<<
 *             field_byteorders.append("little")
 *           else:
 */
          goto __pyx_L25;
        }

        /* "tables/tableextension.pyx":330
 *             field_byteorders.append("little")
 *           else:
 *             field_byteorders.append("big")             # <<<<<<<<<<<<<<
 *         elif colobj.kind in ['int', 'uint', 'float', 'complex', 'enum']:
 *           # Keep track of the byteorder for this column
 */
        /*else*/ {
          __pyx_t_7 = __Pyx_PyObject_Append(__pyx_v_field_byteorders, __pyx_n_s_big); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 330, __pyx_L1_error)
        }
        __pyx_L25:;

        /* "tables/tableextension.pyx":325
 *         # For time kinds, save the byteorder of the column
 *         # (useful for conversion of time datatypes later on)
 *         if colobj.kind == "time":             # <<<<<<<<<<<<<<
 *           colobj._byteorder = H5Tget_order(member_type_id)
 *           if colobj._byteorder == H5T_ORDER_LE:
 */
        goto __pyx_L24;
      }

      /* "tables/tableextension.pyx":331
 *           else:
 *             field_byteorders.append("big")
 *         elif colobj.kind in ['int', 'uint', 'float', 'complex', 'enum']:             # <<<<<<<<<<<<<<
 *           # Keep track of the byteorder for this column
 *           get_order(member_type_id, c_byteorder2)
 */
      __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_v_colobj, __pyx_n_s_kind); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 331, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __pyx_t_11 = (__Pyx_PyString_Equals(__pyx_t_13, __pyx_n_s_int, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 331, __pyx_L1_error)
      if (!__pyx_t_11) {
      } else {
        __pyx_t_10 = __pyx_t_11;
        goto __pyx_L26_bool_binop_done;
      }
      __pyx_t_11 = (__Pyx_PyString_Equals(__pyx_t_13, __pyx_n_s_uint, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 331, __pyx_L1_error)
      if (!__pyx_t_11) {
      } else {
        __pyx_t_10 = __pyx_t_11;
        goto __pyx_L26_bool_binop_done;
      }
      __pyx_t_11 = (__Pyx_PyString_Equals(__pyx_t_13, __pyx_n_s_float, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 331, __pyx_L1_error)
      if (!__pyx_t_11) {
      } else {
        __pyx_t_10 = __pyx_t_11;
        goto __pyx_L26_bool_binop_done;
      }
      __pyx_t_11 = (__Pyx_PyString_Equals(__pyx_t_13, __pyx_n_s_complex, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 331, __pyx_L1_error)
      if (!__pyx_t_11) {
      } else {
        __pyx_t_10 = __pyx_t_11;
        goto __pyx_L26_bool_binop_done;
      }
      __pyx_t_11 = (__Pyx_PyString_Equals(__pyx_t_13, __pyx_n_s_enum, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 331, __pyx_L1_error)
      __pyx_t_10 = __pyx_t_11;
      __pyx_L26_bool_binop_done:;
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __pyx_t_11 = (__pyx_t_10 != 0);
      if (__pyx_t_11) {

        /* "tables/tableextension.pyx":333
 *         elif colobj.kind in ['int', 'uint', 'float', 'complex', 'enum']:
 *           # Keep track of the byteorder for this column
 *           get_order(member_type_id, c_byteorder2)             # <<<<<<<<<<<<<<
 *           byteorder2 = cstr_to_pystr(c_byteorder2)
 *           if byteorder2 in ["little", "big"]:
 */
        (void)(get_order(__pyx_v_member_type_id, __pyx_v_c_byteorder2));

        /* "tables/tableextension.pyx":334
 *           # Keep track of the byteorder for this column
 *           get_order(member_type_id, c_byteorder2)
 *           byteorder2 = cstr_to_pystr(c_byteorder2)             # <<<<<<<<<<<<<<
 *           if byteorder2 in ["little", "big"]:
 *             field_byteorders.append(byteorder2)
 */
        __pyx_t_13 = __pyx_f_6tables_14utilsextension_cstr_to_pystr(__pyx_v_c_byteorder2); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 334, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __Pyx_XDECREF_SET(__pyx_v_byteorder2, ((PyObject*)__pyx_t_13));
        __pyx_t_13 = 0;

        /* "tables/tableextension.pyx":335
 *           get_order(member_type_id, c_byteorder2)
 *           byteorder2 = cstr_to_pystr(c_byteorder2)
 *           if byteorder2 in ["little", "big"]:             # <<<<<<<<<<<<<<
 *             field_byteorders.append(byteorder2)
 * 
 */
        __Pyx_INCREF(__pyx_v_byteorder2);
        __pyx_t_22 = __pyx_v_byteorder2;
        __pyx_t_10 = (__Pyx_PyString_Equals(__pyx_t_22, __pyx_n_s_little, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 335, __pyx_L1_error)
        __pyx_t_23 = (__pyx_t_10 != 0);
        if (!__pyx_t_23) {
        } else {
          __pyx_t_11 = __pyx_t_23;
          goto __pyx_L32_bool_binop_done;
        }
        __pyx_t_23 = (__Pyx_PyString_Equals(__pyx_t_22, __pyx_n_s_big, Py_EQ)); if (unlikely(__pyx_t_23 < 0)) __PYX_ERR(0, 335, __pyx_L1_error)
        __pyx_t_10 = (__pyx_t_23 != 0);
        __pyx_t_11 = __pyx_t_10;
        __pyx_L32_bool_binop_done:;
        __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
        __pyx_t_10 = (__pyx_t_11 != 0);
        if (__pyx_t_10) {

          /* "tables/tableextension.pyx":336
 *           byteorder2 = cstr_to_pystr(c_byteorder2)
 *           if byteorder2 in ["little", "big"]:
 *             field_byteorders.append(byteorder2)             # <<<<<<<<<<<<<<
 * 
 *       # Insert the native member
 */
          __pyx_t_7 = __Pyx_PyObject_Append(__pyx_v_field_byteorders, __pyx_v_byteorder2); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 336, __pyx_L1_error)

          /* "tables/tableextension.pyx":335
 *           get_order(member_type_id, c_byteorder2)
 *           byteorder2 = cstr_to_pystr(c_byteorder2)
 *           if byteorder2 in ["little", "big"]:             # <<<<<<<<<<<<<<
 *             field_byteorders.append(byteorder2)
 * 
 */
        }

        /* "tables/tableextension.pyx":331
 *           else:
 *             field_byteorders.append("big")
 *         elif colobj.kind in ['int', 'uint', 'float', 'complex', 'enum']:             # <<<<<<<<<<<<<<
 *           # Keep track of the byteorder for this column
 *           get_order(member_type_id, c_byteorder2)
 */
      }
      __pyx_L24:;
    }
    __pyx_L9:;

    /* "tables/tableextension.pyx":339
 * 
 *       # Insert the native member
 *       H5Tinsert(native_type_id, c_colname, member_offset, native_member_type_id)             # <<<<<<<<<<<<<<
 *       # Update the offset
 *       offset = offset + itemsize
 */
    (void)(H5Tinsert(__pyx_v_native_type_id, __pyx_v_c_colname, __pyx_v_member_offset, __pyx_v_native_member_type_id));

    /* "tables/tableextension.pyx":341
 *       H5Tinsert(native_type_id, c_colname, member_offset, native_member_type_id)
 *       # Update the offset
 *       offset = offset + itemsize             # <<<<<<<<<<<<<<
 *       # Release resources
 *       H5Tclose(native_member_type_id)
 */
    __pyx_t_13 = __Pyx_PyInt_FromSize_t(__pyx_v_itemsize); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 341, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    __pyx_t_9 = PyNumber_Add(__pyx_v_offset, __pyx_t_13); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 341, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
    __Pyx_DECREF_SET(__pyx_v_offset, __pyx_t_9);
    __pyx_t_9 = 0;

    /* "tables/tableextension.pyx":343
 *       offset = offset + itemsize
 *       # Release resources
 *       H5Tclose(native_member_type_id)             # <<<<<<<<<<<<<<
 *       H5Tclose(member_type_id)
 *       pt_H5free_memory(c_colname)
 */
    (void)(H5Tclose(__pyx_v_native_member_type_id));

    /* "tables/tableextension.pyx":344
 *       # Release resources
 *       H5Tclose(native_member_type_id)
 *       H5Tclose(member_type_id)             # <<<<<<<<<<<<<<
 *       pt_H5free_memory(c_colname)
 * 
 */
    (void)(H5Tclose(__pyx_v_member_type_id));

    /* "tables/tableextension.pyx":345
 *       H5Tclose(native_member_type_id)
 *       H5Tclose(member_type_id)
 *       pt_H5free_memory(c_colname)             # <<<<<<<<<<<<<<
 * 
 *     # set the byteorder and other things (just in top level)
 */
    (void)(pt_H5free_memory(__pyx_v_c_colname));

    /* "tables/tableextension.pyx":290
 * 
 *     # Iterate thru the members
 *     for pos, i in enumerate([x[1] for x in position_order]):             # <<<<<<<<<<<<<<
 *       # Get the member name
 *       c_colname = H5Tget_member_name(type_id, i)
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "tables/tableextension.pyx":348
 * 
 *     # set the byteorder and other things (just in top level)
 *     if colpath == "":             # <<<<<<<<<<<<<<
 *       # Compute a byteorder for the entire table
 *       if len(field_byteorders) > 0:
 */
  __pyx_t_10 = (__Pyx_PyString_Equals(__pyx_v_colpath, __pyx_kp_s_, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 348, __pyx_L1_error)
  if (__pyx_t_10) {

    /* "tables/tableextension.pyx":350
 *     if colpath == "":
 *       # Compute a byteorder for the entire table
 *       if len(field_byteorders) > 0:             # <<<<<<<<<<<<<<
 *         field_byteorders = numpy.array(field_byteorders)
 *         # Cython doesn't interpret well the extended comparison
 */
    __pyx_t_8 = PyObject_Length(__pyx_v_field_byteorders); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(0, 350, __pyx_L1_error)
    __pyx_t_10 = ((__pyx_t_8 > 0) != 0);
    if (__pyx_t_10) {

      /* "tables/tableextension.pyx":351
 *       # Compute a byteorder for the entire table
 *       if len(field_byteorders) > 0:
 *         field_byteorders = numpy.array(field_byteorders)             # <<<<<<<<<<<<<<
 *         # Cython doesn't interpret well the extended comparison
 *         # operators so this: field_byteorders == "little" doesn't work
 */
      __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_numpy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 351, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_array); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 351, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_9))) {
        __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_9);
        if (likely(__pyx_t_1)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
          __Pyx_INCREF(__pyx_t_1);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_9, function);
        }
      }
      __pyx_t_6 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_1, __pyx_v_field_byteorders) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_v_field_byteorders);
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 351, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF_SET(__pyx_v_field_byteorders, __pyx_t_6);
      __pyx_t_6 = 0;

      /* "tables/tableextension.pyx":355
 *         # operators so this: field_byteorders == "little" doesn't work
 *         # as expected
 *         if numpy.alltrue(field_byteorders.__eq__("little")):             # <<<<<<<<<<<<<<
 *           byteorder = "little"
 *         elif numpy.alltrue(field_byteorders.__eq__("big")):
 */
      __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_n_s_numpy); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 355, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_alltrue); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 355, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_v_field_byteorders, __pyx_n_s_eq); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 355, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __pyx_t_5 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_13))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_13);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_13, function);
        }
      }
      __pyx_t_9 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_13, __pyx_t_5, __pyx_n_s_little) : __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_n_s_little);
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 355, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __pyx_t_13 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_1);
        if (likely(__pyx_t_13)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
          __Pyx_INCREF(__pyx_t_13);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_1, function);
        }
      }
      __pyx_t_6 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_13, __pyx_t_9) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_9);
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 355, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 355, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (__pyx_t_10) {

        /* "tables/tableextension.pyx":356
 *         # as expected
 *         if numpy.alltrue(field_byteorders.__eq__("little")):
 *           byteorder = "little"             # <<<<<<<<<<<<<<
 *         elif numpy.alltrue(field_byteorders.__eq__("big")):
 *           byteorder = "big"
 */
        __Pyx_INCREF(__pyx_n_s_little);
        __pyx_v_byteorder = __pyx_n_s_little;

        /* "tables/tableextension.pyx":355
 *         # operators so this: field_byteorders == "little" doesn't work
 *         # as expected
 *         if numpy.alltrue(field_byteorders.__eq__("little")):             # <<<<<<<<<<<<<<
 *           byteorder = "little"
 *         elif numpy.alltrue(field_byteorders.__eq__("big")):
 */
        goto __pyx_L36;
      }

      /* "tables/tableextension.pyx":357
 *         if numpy.alltrue(field_byteorders.__eq__("little")):
 *           byteorder = "little"
 *         elif numpy.alltrue(field_byteorders.__eq__("big")):             # <<<<<<<<<<<<<<
 *           byteorder = "big"
 *         else:  # Yes! someone has done it!
 */
      __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_numpy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 357, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_alltrue); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 357, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_v_field_byteorders, __pyx_n_s_eq); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 357, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __pyx_t_5 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_13))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_13);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_13, function);
        }
      }
      __pyx_t_1 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_13, __pyx_t_5, __pyx_n_s_big) : __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_n_s_big);
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 357, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __pyx_t_13 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_9))) {
        __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_9);
        if (likely(__pyx_t_13)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
          __Pyx_INCREF(__pyx_t_13);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_9, function);
        }
      }
      __pyx_t_6 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_13, __pyx_t_1) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_1);
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 357, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 357, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (__pyx_t_10) {

        /* "tables/tableextension.pyx":358
 *           byteorder = "little"
 *         elif numpy.alltrue(field_byteorders.__eq__("big")):
 *           byteorder = "big"             # <<<<<<<<<<<<<<
 *         else:  # Yes! someone has done it!
 *           byteorder = "mixed"
 */
        __Pyx_INCREF(__pyx_n_s_big);
        __pyx_v_byteorder = __pyx_n_s_big;

        /* "tables/tableextension.pyx":357
 *         if numpy.alltrue(field_byteorders.__eq__("little")):
 *           byteorder = "little"
 *         elif numpy.alltrue(field_byteorders.__eq__("big")):             # <<<<<<<<<<<<<<
 *           byteorder = "big"
 *         else:  # Yes! someone has done it!
 */
        goto __pyx_L36;
      }

      /* "tables/tableextension.pyx":360
 *           byteorder = "big"
 *         else:  # Yes! someone has done it!
 *           byteorder = "mixed"             # <<<<<<<<<<<<<<
 *       else:
 *         byteorder = "irrelevant"
 */
      /*else*/ {
        __Pyx_INCREF(__pyx_n_s_mixed);
        __pyx_v_byteorder = __pyx_n_s_mixed;
      }
      __pyx_L36:;

      /* "tables/tableextension.pyx":350
 *     if colpath == "":
 *       # Compute a byteorder for the entire table
 *       if len(field_byteorders) > 0:             # <<<<<<<<<<<<<<
 *         field_byteorders = numpy.array(field_byteorders)
 *         # Cython doesn't interpret well the extended comparison
 */
      goto __pyx_L35;
    }

    /* "tables/tableextension.pyx":362
 *           byteorder = "mixed"
 *       else:
 *         byteorder = "irrelevant"             # <<<<<<<<<<<<<<
 *       self.byteorder = byteorder
 * 
 */
    /*else*/ {
      __Pyx_INCREF(__pyx_n_s_irrelevant);
      __pyx_v_byteorder = __pyx_n_s_irrelevant;
    }
    __pyx_L35:;

    /* "tables/tableextension.pyx":363
 *       else:
 *         byteorder = "irrelevant"
 *       self.byteorder = byteorder             # <<<<<<<<<<<<<<
 * 
 *     return desc, offset
 */
    if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_byteorder, __pyx_v_byteorder) < 0) __PYX_ERR(0, 363, __pyx_L1_error)

    /* "tables/tableextension.pyx":348
 * 
 *     # set the byteorder and other things (just in top level)
 *     if colpath == "":             # <<<<<<<<<<<<<<
 *       # Compute a byteorder for the entire table
 *       if len(field_byteorders) > 0:
 */
  }

  /* "tables/tableextension.pyx":365
 *       self.byteorder = byteorder
 * 
 *     return desc, offset             # <<<<<<<<<<<<<<
 * 
 *   def _get_info(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 365, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_v_desc);
  __Pyx_GIVEREF(__pyx_v_desc);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_desc);
  __Pyx_INCREF(__pyx_v_offset);
  __Pyx_GIVEREF(__pyx_v_offset);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_v_offset);
  __pyx_r = __pyx_t_6;
  __pyx_t_6 = 0;
  goto __pyx_L0;

  /* "tables/tableextension.pyx":259
 * 
 * 
 *   cdef get_nested_type(self, hid_t type_id, hid_t native_type_id,             # <<<<<<<<<<<<<<
 *                        object colpath, object field_byteorders):
 *     """Open a nested type and return a nested dictionary as description."""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_20);
  __Pyx_XDECREF(__pyx_t_21);
  __Pyx_XDECREF(__pyx_t_22);
  __Pyx_AddTraceback("tables.tableextension.Table.get_nested_type", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_desc);
  __Pyx_XDECREF(__pyx_v_colobj);
  __Pyx_XDECREF(__pyx_v_colpath2);
  __Pyx_XDECREF(__pyx_v_byteorder);
  __Pyx_XDECREF(__pyx_v_colname);
  __Pyx_XDECREF(__pyx_v_byteorder2);
  __Pyx_XDECREF(__pyx_v_offset);
  __Pyx_XDECREF(__pyx_v_position_order);
  __Pyx_XDECREF(__pyx_v_pos);
  __Pyx_XDECREF(__pyx_v_atom);
  __Pyx_XDECREF(__pyx_v_te);
  __Pyx_XDECREF(__pyx_v_x);
  __Pyx_XDECREF(__pyx_v_field_byteorders);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":367
 *     return desc, offset
 * 
 *   def _get_info(self):             # <<<<<<<<<<<<<<
 *     """Get info from a table on disk."""
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_14tableextension_5Table_3_get_info(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6tables_14tableextension_5Table_2_get_info[] = "Get info from a table on disk.";
static PyObject *__pyx_pw_6tables_14tableextension_5Table_3_get_info(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_info (wrapper)", 0);
  __pyx_r = __pyx_pf_6tables_14tableextension_5Table_2_get_info(((struct __pyx_obj_6tables_14tableextension_Table *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_14tableextension_5Table_2_get_info(struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self) {
  hid_t __pyx_v_space_id;
  hid_t __pyx_v_plist;
  size_t __pyx_v_type_size;
  hsize_t __pyx_v_dims[1];
  hsize_t __pyx_v_chunksize[1];
  enum H5D_layout_t __pyx_v_layout;
  PyObject *__pyx_v_encoded_name = 0;
  PyObject *__pyx_v_desc = NULL;
  PyObject *__pyx_v_offset = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  char *__pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *(*__pyx_t_7)(PyObject *);
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_info", 0);

  /* "tables/tableextension.pyx":377
 *     cdef bytes encoded_name
 * 
 *     encoded_name = self.name.encode('utf-8')             # <<<<<<<<<<<<<<
 * 
 *     # Open the dataset
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->__pyx_base.__pyx_base.name, __pyx_n_s_encode); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 377, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_kp_s_utf_8) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_s_utf_8);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 377, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 377, __pyx_L1_error)
  __pyx_v_encoded_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":380
 * 
 *     # Open the dataset
 *     self.dataset_id = H5Dopen(self.parent_id, encoded_name, H5P_DEFAULT)             # <<<<<<<<<<<<<<
 *     if self.dataset_id < 0:
 *       raise HDF5ExtError("Non-existing node ``%s`` under ``%s``" %
 */
  if (unlikely(__pyx_v_encoded_name == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 380, __pyx_L1_error)
  }
  __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_encoded_name); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(0, 380, __pyx_L1_error)
  __pyx_v_self->__pyx_base.dataset_id = H5Dopen(__pyx_v_self->__pyx_base.__pyx_base.parent_id, __pyx_t_4, H5P_DEFAULT);

  /* "tables/tableextension.pyx":381
 *     # Open the dataset
 *     self.dataset_id = H5Dopen(self.parent_id, encoded_name, H5P_DEFAULT)
 *     if self.dataset_id < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Non-existing node ``%s`` under ``%s``" %
 *                          (self.name, self._v_parent._v_pathname))
 */
  __pyx_t_5 = ((__pyx_v_self->__pyx_base.dataset_id < 0) != 0);
  if (unlikely(__pyx_t_5)) {

    /* "tables/tableextension.pyx":382
 *     self.dataset_id = H5Dopen(self.parent_id, encoded_name, H5P_DEFAULT)
 *     if self.dataset_id < 0:
 *       raise HDF5ExtError("Non-existing node ``%s`` under ``%s``" %             # <<<<<<<<<<<<<<
 *                          (self.name, self._v_parent._v_pathname))
 * 
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_HDF5ExtError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 382, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);

    /* "tables/tableextension.pyx":383
 *     if self.dataset_id < 0:
 *       raise HDF5ExtError("Non-existing node ``%s`` under ``%s``" %
 *                          (self.name, self._v_parent._v_pathname))             # <<<<<<<<<<<<<<
 * 
 *     # Get the datatype on disk
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_v_parent); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 383, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_v_pathname); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 383, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 383, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.name);
    __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.name);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_self->__pyx_base.__pyx_base.name);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_6);
    __pyx_t_6 = 0;

    /* "tables/tableextension.pyx":382
 *     self.dataset_id = H5Dopen(self.parent_id, encoded_name, H5P_DEFAULT)
 *     if self.dataset_id < 0:
 *       raise HDF5ExtError("Non-existing node ``%s`` under ``%s``" %             # <<<<<<<<<<<<<<
 *                          (self.name, self._v_parent._v_pathname))
 * 
 */
    __pyx_t_6 = __Pyx_PyString_Format(__pyx_kp_s_Non_existing_node_s_under_s, __pyx_t_3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 382, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 382, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 382, __pyx_L1_error)

    /* "tables/tableextension.pyx":381
 *     # Open the dataset
 *     self.dataset_id = H5Dopen(self.parent_id, encoded_name, H5P_DEFAULT)
 *     if self.dataset_id < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Non-existing node ``%s`` under ``%s``" %
 *                          (self.name, self._v_parent._v_pathname))
 */
  }

  /* "tables/tableextension.pyx":386
 * 
 *     # Get the datatype on disk
 *     self.disk_type_id = H5Dget_type(self.dataset_id)             # <<<<<<<<<<<<<<
 *     if H5Tget_class(self.disk_type_id) != H5T_COMPOUND:
 *         raise ValueError("Node ``%s`` is not a Table object" %
 */
  __pyx_v_self->__pyx_base.disk_type_id = H5Dget_type(__pyx_v_self->__pyx_base.dataset_id);

  /* "tables/tableextension.pyx":387
 *     # Get the datatype on disk
 *     self.disk_type_id = H5Dget_type(self.dataset_id)
 *     if H5Tget_class(self.disk_type_id) != H5T_COMPOUND:             # <<<<<<<<<<<<<<
 *         raise ValueError("Node ``%s`` is not a Table object" %
 *                          (self._v_parent._v_leaves[self.name]._v_pathname))
 */
  __pyx_t_5 = ((H5Tget_class(__pyx_v_self->__pyx_base.disk_type_id) != H5T_COMPOUND) != 0);
  if (unlikely(__pyx_t_5)) {

    /* "tables/tableextension.pyx":389
 *     if H5Tget_class(self.disk_type_id) != H5T_COMPOUND:
 *         raise ValueError("Node ``%s`` is not a Table object" %
 *                          (self._v_parent._v_leaves[self.name]._v_pathname))             # <<<<<<<<<<<<<<
 *     # Get the number of rows
 *     space_id = H5Dget_space(self.dataset_id)
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_v_parent); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 389, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_v_leaves); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 389, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_t_2, __pyx_v_self->__pyx_base.__pyx_base.name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 389, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_v_pathname); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 389, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "tables/tableextension.pyx":388
 *     self.disk_type_id = H5Dget_type(self.dataset_id)
 *     if H5Tget_class(self.disk_type_id) != H5T_COMPOUND:
 *         raise ValueError("Node ``%s`` is not a Table object" %             # <<<<<<<<<<<<<<
 *                          (self._v_parent._v_leaves[self.name]._v_pathname))
 *     # Get the number of rows
 */
    __pyx_t_1 = __Pyx_PyString_FormatSafe(__pyx_kp_s_Node_s_is_not_a_Table_object, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 388, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 388, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 388, __pyx_L1_error)

    /* "tables/tableextension.pyx":387
 *     # Get the datatype on disk
 *     self.disk_type_id = H5Dget_type(self.dataset_id)
 *     if H5Tget_class(self.disk_type_id) != H5T_COMPOUND:             # <<<<<<<<<<<<<<
 *         raise ValueError("Node ``%s`` is not a Table object" %
 *                          (self._v_parent._v_leaves[self.name]._v_pathname))
 */
  }

  /* "tables/tableextension.pyx":391
 *                          (self._v_parent._v_leaves[self.name]._v_pathname))
 *     # Get the number of rows
 *     space_id = H5Dget_space(self.dataset_id)             # <<<<<<<<<<<<<<
 *     H5Sget_simple_extent_dims(space_id, dims, NULL)
 *     self.nrows = SizeType(dims[0])
 */
  __pyx_v_space_id = H5Dget_space(__pyx_v_self->__pyx_base.dataset_id);

  /* "tables/tableextension.pyx":392
 *     # Get the number of rows
 *     space_id = H5Dget_space(self.dataset_id)
 *     H5Sget_simple_extent_dims(space_id, dims, NULL)             # <<<<<<<<<<<<<<
 *     self.nrows = SizeType(dims[0])
 *     # Free resources
 */
  (void)(H5Sget_simple_extent_dims(__pyx_v_space_id, __pyx_v_dims, NULL));

  /* "tables/tableextension.pyx":393
 *     space_id = H5Dget_space(self.dataset_id)
 *     H5Sget_simple_extent_dims(space_id, dims, NULL)
 *     self.nrows = SizeType(dims[0])             # <<<<<<<<<<<<<<
 *     # Free resources
 *     H5Sclose(space_id)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SizeType); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 393, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = __Pyx_PyInt_From_hsize_t((__pyx_v_dims[0])); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 393, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_2 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_3, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_6);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 393, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nrows, __pyx_t_2) < 0) __PYX_ERR(0, 393, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/tableextension.pyx":395
 *     self.nrows = SizeType(dims[0])
 *     # Free resources
 *     H5Sclose(space_id)             # <<<<<<<<<<<<<<
 * 
 *     # Get the layout of the datatype
 */
  (void)(H5Sclose(__pyx_v_space_id));

  /* "tables/tableextension.pyx":398
 * 
 *     # Get the layout of the datatype
 *     plist = H5Dget_create_plist(self.dataset_id)             # <<<<<<<<<<<<<<
 *     layout = H5Pget_layout(plist)
 *     if layout == H5D_CHUNKED:
 */
  __pyx_v_plist = H5Dget_create_plist(__pyx_v_self->__pyx_base.dataset_id);

  /* "tables/tableextension.pyx":399
 *     # Get the layout of the datatype
 *     plist = H5Dget_create_plist(self.dataset_id)
 *     layout = H5Pget_layout(plist)             # <<<<<<<<<<<<<<
 *     if layout == H5D_CHUNKED:
 *       self._chunked = 1
 */
  __pyx_v_layout = H5Pget_layout(__pyx_v_plist);

  /* "tables/tableextension.pyx":400
 *     plist = H5Dget_create_plist(self.dataset_id)
 *     layout = H5Pget_layout(plist)
 *     if layout == H5D_CHUNKED:             # <<<<<<<<<<<<<<
 *       self._chunked = 1
 *       # Get the chunksize
 */
  __pyx_t_5 = ((__pyx_v_layout == H5D_CHUNKED) != 0);
  if (__pyx_t_5) {

    /* "tables/tableextension.pyx":401
 *     layout = H5Pget_layout(plist)
 *     if layout == H5D_CHUNKED:
 *       self._chunked = 1             # <<<<<<<<<<<<<<
 *       # Get the chunksize
 *       H5Pget_chunk(plist, 1, chunksize)
 */
    if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_chunked, __pyx_int_1) < 0) __PYX_ERR(0, 401, __pyx_L1_error)

    /* "tables/tableextension.pyx":403
 *       self._chunked = 1
 *       # Get the chunksize
 *       H5Pget_chunk(plist, 1, chunksize)             # <<<<<<<<<<<<<<
 *     else:
 *       self._chunked = 0
 */
    (void)(H5Pget_chunk(__pyx_v_plist, 1, __pyx_v_chunksize));

    /* "tables/tableextension.pyx":400
 *     plist = H5Dget_create_plist(self.dataset_id)
 *     layout = H5Pget_layout(plist)
 *     if layout == H5D_CHUNKED:             # <<<<<<<<<<<<<<
 *       self._chunked = 1
 *       # Get the chunksize
 */
    goto __pyx_L5;
  }

  /* "tables/tableextension.pyx":405
 *       H5Pget_chunk(plist, 1, chunksize)
 *     else:
 *       self._chunked = 0             # <<<<<<<<<<<<<<
 *       chunksize[0] = 0
 *     H5Pclose(plist)
 */
  /*else*/ {
    if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_chunked, __pyx_int_0) < 0) __PYX_ERR(0, 405, __pyx_L1_error)

    /* "tables/tableextension.pyx":406
 *     else:
 *       self._chunked = 0
 *       chunksize[0] = 0             # <<<<<<<<<<<<<<
 *     H5Pclose(plist)
 * 
 */
    (__pyx_v_chunksize[0]) = 0;
  }
  __pyx_L5:;

  /* "tables/tableextension.pyx":407
 *       self._chunked = 0
 *       chunksize[0] = 0
 *     H5Pclose(plist)             # <<<<<<<<<<<<<<
 * 
 *     # Get the type size
 */
  (void)(H5Pclose(__pyx_v_plist));

  /* "tables/tableextension.pyx":410
 * 
 *     # Get the type size
 *     type_size = H5Tget_size(self.disk_type_id)             # <<<<<<<<<<<<<<
 *     # Create the native data in-memory
 *     self.type_id = H5Tcreate(H5T_COMPOUND, type_size)
 */
  __pyx_v_type_size = H5Tget_size(__pyx_v_self->__pyx_base.disk_type_id);

  /* "tables/tableextension.pyx":412
 *     type_size = H5Tget_size(self.disk_type_id)
 *     # Create the native data in-memory
 *     self.type_id = H5Tcreate(H5T_COMPOUND, type_size)             # <<<<<<<<<<<<<<
 *     # Fill-up the (nested) native type and description
 *     desc, offset = self.get_nested_type(self.disk_type_id, self.type_id, "", [])
 */
  __pyx_v_self->__pyx_base.type_id = H5Tcreate(H5T_COMPOUND, __pyx_v_type_size);

  /* "tables/tableextension.pyx":414
 *     self.type_id = H5Tcreate(H5T_COMPOUND, type_size)
 *     # Fill-up the (nested) native type and description
 *     desc, offset = self.get_nested_type(self.disk_type_id, self.type_id, "", [])             # <<<<<<<<<<<<<<
 * 
 *     if desc == {}:
 */
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = ((struct __pyx_vtabstruct_6tables_14tableextension_Table *)__pyx_v_self->__pyx_base.__pyx_vtab)->get_nested_type(__pyx_v_self, __pyx_v_self->__pyx_base.disk_type_id, __pyx_v_self->__pyx_base.type_id, __pyx_kp_s_, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
    PyObject* sequence = __pyx_t_1;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 414, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_6 = PyTuple_GET_ITEM(sequence, 1); 
    } else {
      __pyx_t_2 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_6 = PyList_GET_ITEM(sequence, 1); 
    }
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_6);
    #else
    __pyx_t_2 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 414, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_6 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 414, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_3 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 414, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = Py_TYPE(__pyx_t_3)->tp_iternext;
    index = 0; __pyx_t_2 = __pyx_t_7(__pyx_t_3); if (unlikely(!__pyx_t_2)) goto __pyx_L6_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_2);
    index = 1; __pyx_t_6 = __pyx_t_7(__pyx_t_3); if (unlikely(!__pyx_t_6)) goto __pyx_L6_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_6);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_7(__pyx_t_3), 2) < 0) __PYX_ERR(0, 414, __pyx_L1_error)
    __pyx_t_7 = NULL;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L7_unpacking_done;
    __pyx_L6_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_7 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 414, __pyx_L1_error)
    __pyx_L7_unpacking_done:;
  }
  __pyx_v_desc = __pyx_t_2;
  __pyx_t_2 = 0;
  __pyx_v_offset = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "tables/tableextension.pyx":416
 *     desc, offset = self.get_nested_type(self.disk_type_id, self.type_id, "", [])
 * 
 *     if desc == {}:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems getting desciption for table %s", self.name)
 * 
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 416, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = PyObject_RichCompare(__pyx_v_desc, __pyx_t_1, Py_EQ); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 416, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 416, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (unlikely(__pyx_t_5)) {

    /* "tables/tableextension.pyx":417
 * 
 *     if desc == {}:
 *       raise HDF5ExtError("Problems getting desciption for table %s", self.name)             # <<<<<<<<<<<<<<
 * 
 *     if offset < type_size:
 */
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_HDF5ExtError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 417, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = NULL;
    __pyx_t_8 = 0;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
        __pyx_t_8 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_kp_s_Problems_getting_desciption_for, __pyx_v_self->__pyx_base.__pyx_base.name};
      __pyx_t_6 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 417, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_6);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_kp_s_Problems_getting_desciption_for, __pyx_v_self->__pyx_base.__pyx_base.name};
      __pyx_t_6 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 417, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_6);
    } else
    #endif
    {
      __pyx_t_3 = PyTuple_New(2+__pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 417, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      if (__pyx_t_2) {
        __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2); __pyx_t_2 = NULL;
      }
      __Pyx_INCREF(__pyx_kp_s_Problems_getting_desciption_for);
      __Pyx_GIVEREF(__pyx_kp_s_Problems_getting_desciption_for);
      PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_8, __pyx_kp_s_Problems_getting_desciption_for);
      __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.name);
      __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.name);
      PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_8, __pyx_v_self->__pyx_base.__pyx_base.name);
      __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_3, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 417, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __PYX_ERR(0, 417, __pyx_L1_error)

    /* "tables/tableextension.pyx":416
 *     desc, offset = self.get_nested_type(self.disk_type_id, self.type_id, "", [])
 * 
 *     if desc == {}:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems getting desciption for table %s", self.name)
 * 
 */
  }

  /* "tables/tableextension.pyx":419
 *       raise HDF5ExtError("Problems getting desciption for table %s", self.name)
 * 
 *     if offset < type_size:             # <<<<<<<<<<<<<<
 *       # Trailing padding, set the itemsize to the correct type_size (see #765)
 *       desc['_v_itemsize'] = type_size
 */
  __pyx_t_6 = __Pyx_PyInt_FromSize_t(__pyx_v_type_size); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 419, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_offset, __pyx_t_6, Py_LT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 419, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 419, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_5) {

    /* "tables/tableextension.pyx":421
 *     if offset < type_size:
 *       # Trailing padding, set the itemsize to the correct type_size (see #765)
 *       desc['_v_itemsize'] = type_size             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_type_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 421, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (unlikely(PyObject_SetItem(__pyx_v_desc, __pyx_n_s_v_itemsize, __pyx_t_1) < 0)) __PYX_ERR(0, 421, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "tables/tableextension.pyx":419
 *       raise HDF5ExtError("Problems getting desciption for table %s", self.name)
 * 
 *     if offset < type_size:             # <<<<<<<<<<<<<<
 *       # Trailing padding, set the itemsize to the correct type_size (see #765)
 *       desc['_v_itemsize'] = type_size
 */
  }

  /* "tables/tableextension.pyx":425
 * 
 *     # Return the object ID and the description
 *     return (self.dataset_id, desc, SizeType(chunksize[0]))             # <<<<<<<<<<<<<<
 * 
 *   cdef _convert_time64_(self, ndarray nparr, hsize_t nrecords, int sense):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_hid_t(__pyx_v_self->__pyx_base.dataset_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 425, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_SizeType); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 425, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyInt_From_hsize_t((__pyx_v_chunksize[0])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 425, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_9 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_9)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_6 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_9, __pyx_t_2) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2);
  __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 425, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 425, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_desc);
  __Pyx_GIVEREF(__pyx_v_desc);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_desc);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_6);
  __pyx_t_1 = 0;
  __pyx_t_6 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "tables/tableextension.pyx":367
 *     return desc, offset
 * 
 *   def _get_info(self):             # <<<<<<<<<<<<<<
 *     """Get info from a table on disk."""
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("tables.tableextension.Table._get_info", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_encoded_name);
  __Pyx_XDECREF(__pyx_v_desc);
  __Pyx_XDECREF(__pyx_v_offset);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":427
 *     return (self.dataset_id, desc, SizeType(chunksize[0]))
 * 
 *   cdef _convert_time64_(self, ndarray nparr, hsize_t nrecords, int sense):             # <<<<<<<<<<<<<<
 *     """Converts a NumPy of Time64 elements between NumPy and HDF5 formats.
 * 
 */

static PyObject *__pyx_f_6tables_14tableextension_5Table__convert_time64_(CYTHON_UNUSED struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self, PyArrayObject *__pyx_v_nparr, hsize_t __pyx_v_nrecords, int __pyx_v_sense) {
  void *__pyx_v_t64buf;
  long __pyx_v_byteoffset;
  npy_intp __pyx_v_bytestride;
  npy_intp __pyx_v_nelements;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  npy_intp __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_convert_time64_", 0);

  /* "tables/tableextension.pyx":440
 *     cdef npy_intp bytestride, nelements
 * 
 *     byteoffset = 0   # NumPy objects doesn't have an offset             # <<<<<<<<<<<<<<
 *     bytestride = PyArray_STRIDE(nparr, 0)  # supports multi-dimensional recarray
 *     # Compute the number of elements in the multidimensional cell
 */
  __pyx_v_byteoffset = 0;

  /* "tables/tableextension.pyx":441
 * 
 *     byteoffset = 0   # NumPy objects doesn't have an offset
 *     bytestride = PyArray_STRIDE(nparr, 0)  # supports multi-dimensional recarray             # <<<<<<<<<<<<<<
 *     # Compute the number of elements in the multidimensional cell
 *     nelements = nparr.size // len(nparr)
 */
  __pyx_v_bytestride = PyArray_STRIDE(__pyx_v_nparr, 0);

  /* "tables/tableextension.pyx":443
 *     bytestride = PyArray_STRIDE(nparr, 0)  # supports multi-dimensional recarray
 *     # Compute the number of elements in the multidimensional cell
 *     nelements = nparr.size // len(nparr)             # <<<<<<<<<<<<<<
 *     t64buf = PyArray_DATA(nparr)
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_nparr), __pyx_n_s_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 443, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_Length(((PyObject *)__pyx_v_nparr)); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 443, __pyx_L1_error)
  __pyx_t_3 = PyInt_FromSsize_t(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 443, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyNumber_FloorDivide(__pyx_t_1, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 443, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = __Pyx_PyInt_As_Py_intptr_t(__pyx_t_4); if (unlikely((__pyx_t_5 == ((npy_intp)-1)) && PyErr_Occurred())) __PYX_ERR(0, 443, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_nelements = __pyx_t_5;

  /* "tables/tableextension.pyx":444
 *     # Compute the number of elements in the multidimensional cell
 *     nelements = nparr.size // len(nparr)
 *     t64buf = PyArray_DATA(nparr)             # <<<<<<<<<<<<<<
 * 
 *     conv_float64_timeval32(
 */
  __pyx_v_t64buf = PyArray_DATA(__pyx_v_nparr);

  /* "tables/tableextension.pyx":446
 *     t64buf = PyArray_DATA(nparr)
 * 
 *     conv_float64_timeval32(             # <<<<<<<<<<<<<<
 *       t64buf, byteoffset, bytestride, nrecords, nelements, sense)
 * 
 */
  conv_float64_timeval32(__pyx_v_t64buf, __pyx_v_byteoffset, __pyx_v_bytestride, __pyx_v_nrecords, __pyx_v_nelements, __pyx_v_sense);

  /* "tables/tableextension.pyx":427
 *     return (self.dataset_id, desc, SizeType(chunksize[0]))
 * 
 *   cdef _convert_time64_(self, ndarray nparr, hsize_t nrecords, int sense):             # <<<<<<<<<<<<<<
 *     """Converts a NumPy of Time64 elements between NumPy and HDF5 formats.
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("tables.tableextension.Table._convert_time64_", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":449
 *       t64buf, byteoffset, bytestride, nrecords, nelements, sense)
 * 
 *   cpdef _convert_types(self, ndarray recarr, hsize_t nrecords, int sense):             # <<<<<<<<<<<<<<
 *     """Converts columns in 'recarr' between NumPy and HDF5 formats.
 * 
 */

static PyObject *__pyx_pw_6tables_14tableextension_5Table_5_convert_types(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_6tables_14tableextension_5Table__convert_types(struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self, PyArrayObject *__pyx_v_recarr, hsize_t __pyx_v_nrecords, int __pyx_v_sense, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_colpathname = NULL;
  PyObject *__pyx_v_colobj = NULL;
  PyObject *__pyx_v_column = NULL;
  PyObject *__pyx_v_t64cname = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  PyObject *(*__pyx_t_11)(PyObject *);
  int __pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_convert_types", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_convert_types); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 449, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6tables_14tableextension_5Table_5_convert_types)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = __Pyx_PyInt_From_hsize_t(__pyx_v_nrecords); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 449, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_sense); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 449, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_5 = __pyx_t_1; __pyx_t_6 = NULL;
        __pyx_t_7 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_6)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_6);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
            __pyx_t_7 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[4] = {__pyx_t_6, ((PyObject *)__pyx_v_recarr), __pyx_t_3, __pyx_t_4};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 449, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[4] = {__pyx_t_6, ((PyObject *)__pyx_v_recarr), __pyx_t_3, __pyx_t_4};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 449, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else
        #endif
        {
          __pyx_t_8 = PyTuple_New(3+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 449, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          if (__pyx_t_6) {
            __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
          }
          __Pyx_INCREF(((PyObject *)__pyx_v_recarr));
          __Pyx_GIVEREF(((PyObject *)__pyx_v_recarr));
          PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, ((PyObject *)__pyx_v_recarr));
          __Pyx_GIVEREF(__pyx_t_3);
          PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_3);
          __Pyx_GIVEREF(__pyx_t_4);
          PyTuple_SET_ITEM(__pyx_t_8, 2+__pyx_t_7, __pyx_t_4);
          __pyx_t_3 = 0;
          __pyx_t_4 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 449, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "tables/tableextension.pyx":460
 *     # For reading, first swap the byteorder by hand
 *     # (this is not currently supported by HDF5)
 *     if sense == 1:             # <<<<<<<<<<<<<<
 *       for colpathname in self.colpathnames:
 *         if self.coltypes[colpathname] in ["time32", "time64"]:
 */
  __pyx_t_9 = ((__pyx_v_sense == 1) != 0);
  if (__pyx_t_9) {

    /* "tables/tableextension.pyx":461
 *     # (this is not currently supported by HDF5)
 *     if sense == 1:
 *       for colpathname in self.colpathnames:             # <<<<<<<<<<<<<<
 *         if self.coltypes[colpathname] in ["time32", "time64"]:
 *           colobj = self.coldescrs[colpathname]
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_colpathnames); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 461, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_2 = __pyx_t_1; __Pyx_INCREF(__pyx_t_2); __pyx_t_10 = 0;
      __pyx_t_11 = NULL;
    } else {
      __pyx_t_10 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 461, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_11 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 461, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_11)) {
        if (likely(PyList_CheckExact(__pyx_t_2))) {
          if (__pyx_t_10 >= PyList_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_10); __Pyx_INCREF(__pyx_t_1); __pyx_t_10++; if (unlikely(0 < 0)) __PYX_ERR(0, 461, __pyx_L1_error)
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_10); __pyx_t_10++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 461, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_10 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_10); __Pyx_INCREF(__pyx_t_1); __pyx_t_10++; if (unlikely(0 < 0)) __PYX_ERR(0, 461, __pyx_L1_error)
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_10); __pyx_t_10++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 461, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_11(__pyx_t_2);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 461, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_XDECREF_SET(__pyx_v_colpathname, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "tables/tableextension.pyx":462
 *     if sense == 1:
 *       for colpathname in self.colpathnames:
 *         if self.coltypes[colpathname] in ["time32", "time64"]:             # <<<<<<<<<<<<<<
 *           colobj = self.coldescrs[colpathname]
 *           if hasattr(colobj, "_byteorder"):
 */
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_coltypes); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 462, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_5 = __Pyx_PyObject_GetItem(__pyx_t_1, __pyx_v_colpathname); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 462, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_12 = (__Pyx_PyString_Equals(__pyx_t_5, __pyx_n_s_time32, Py_EQ)); if (unlikely(__pyx_t_12 < 0)) __PYX_ERR(0, 462, __pyx_L1_error)
      if (!__pyx_t_12) {
      } else {
        __pyx_t_9 = __pyx_t_12;
        goto __pyx_L7_bool_binop_done;
      }
      __pyx_t_12 = (__Pyx_PyString_Equals(__pyx_t_5, __pyx_n_s_time64, Py_EQ)); if (unlikely(__pyx_t_12 < 0)) __PYX_ERR(0, 462, __pyx_L1_error)
      __pyx_t_9 = __pyx_t_12;
      __pyx_L7_bool_binop_done:;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_12 = (__pyx_t_9 != 0);
      if (__pyx_t_12) {

        /* "tables/tableextension.pyx":463
 *       for colpathname in self.colpathnames:
 *         if self.coltypes[colpathname] in ["time32", "time64"]:
 *           colobj = self.coldescrs[colpathname]             # <<<<<<<<<<<<<<
 *           if hasattr(colobj, "_byteorder"):
 *             if colobj._byteorder != platform_byteorder:
 */
        __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_coldescrs); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 463, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_t_5, __pyx_v_colpathname); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 463, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF_SET(__pyx_v_colobj, __pyx_t_1);
        __pyx_t_1 = 0;

        /* "tables/tableextension.pyx":464
 *         if self.coltypes[colpathname] in ["time32", "time64"]:
 *           colobj = self.coldescrs[colpathname]
 *           if hasattr(colobj, "_byteorder"):             # <<<<<<<<<<<<<<
 *             if colobj._byteorder != platform_byteorder:
 *               column = get_nested_field(recarr, colpathname)
 */
        __pyx_t_12 = __Pyx_HasAttr(__pyx_v_colobj, __pyx_n_s_byteorder_2); if (unlikely(__pyx_t_12 == ((int)-1))) __PYX_ERR(0, 464, __pyx_L1_error)
        __pyx_t_9 = (__pyx_t_12 != 0);
        if (__pyx_t_9) {

          /* "tables/tableextension.pyx":465
 *           colobj = self.coldescrs[colpathname]
 *           if hasattr(colobj, "_byteorder"):
 *             if colobj._byteorder != platform_byteorder:             # <<<<<<<<<<<<<<
 *               column = get_nested_field(recarr, colpathname)
 *               # Do an *inplace* byteswapping
 */
          __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_colobj, __pyx_n_s_byteorder_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 465, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_platform_byteorder); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 465, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __pyx_t_8 = PyObject_RichCompare(__pyx_t_1, __pyx_t_5, Py_NE); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 465, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 465, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          if (__pyx_t_9) {

            /* "tables/tableextension.pyx":466
 *           if hasattr(colobj, "_byteorder"):
 *             if colobj._byteorder != platform_byteorder:
 *               column = get_nested_field(recarr, colpathname)             # <<<<<<<<<<<<<<
 *               # Do an *inplace* byteswapping
 *               column.byteswap(True)
 */
            __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_get_nested_field); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 466, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_5);
            __pyx_t_1 = NULL;
            __pyx_t_7 = 0;
            if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
              __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_5);
              if (likely(__pyx_t_1)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
                __Pyx_INCREF(__pyx_t_1);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_5, function);
                __pyx_t_7 = 1;
              }
            }
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(__pyx_t_5)) {
              PyObject *__pyx_temp[3] = {__pyx_t_1, ((PyObject *)__pyx_v_recarr), __pyx_v_colpathname};
              __pyx_t_8 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 466, __pyx_L1_error)
              __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
              __Pyx_GOTREF(__pyx_t_8);
            } else
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
              PyObject *__pyx_temp[3] = {__pyx_t_1, ((PyObject *)__pyx_v_recarr), __pyx_v_colpathname};
              __pyx_t_8 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 466, __pyx_L1_error)
              __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
              __Pyx_GOTREF(__pyx_t_8);
            } else
            #endif
            {
              __pyx_t_4 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 466, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_4);
              if (__pyx_t_1) {
                __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1); __pyx_t_1 = NULL;
              }
              __Pyx_INCREF(((PyObject *)__pyx_v_recarr));
              __Pyx_GIVEREF(((PyObject *)__pyx_v_recarr));
              PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_7, ((PyObject *)__pyx_v_recarr));
              __Pyx_INCREF(__pyx_v_colpathname);
              __Pyx_GIVEREF(__pyx_v_colpathname);
              PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_7, __pyx_v_colpathname);
              __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_4, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 466, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_8);
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            }
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
            __Pyx_XDECREF_SET(__pyx_v_column, __pyx_t_8);
            __pyx_t_8 = 0;

            /* "tables/tableextension.pyx":468
 *               column = get_nested_field(recarr, colpathname)
 *               # Do an *inplace* byteswapping
 *               column.byteswap(True)             # <<<<<<<<<<<<<<
 * 
 *     # This should be generalised to support other type conversions.
 */
            __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_column, __pyx_n_s_byteswap); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 468, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_5);
            __pyx_t_4 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
              __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
              if (likely(__pyx_t_4)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
                __Pyx_INCREF(__pyx_t_4);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_5, function);
              }
            }
            __pyx_t_8 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_4, Py_True) : __Pyx_PyObject_CallOneArg(__pyx_t_5, Py_True);
            __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
            if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 468, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_8);
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
            __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

            /* "tables/tableextension.pyx":465
 *           colobj = self.coldescrs[colpathname]
 *           if hasattr(colobj, "_byteorder"):
 *             if colobj._byteorder != platform_byteorder:             # <<<<<<<<<<<<<<
 *               column = get_nested_field(recarr, colpathname)
 *               # Do an *inplace* byteswapping
 */
          }

          /* "tables/tableextension.pyx":464
 *         if self.coltypes[colpathname] in ["time32", "time64"]:
 *           colobj = self.coldescrs[colpathname]
 *           if hasattr(colobj, "_byteorder"):             # <<<<<<<<<<<<<<
 *             if colobj._byteorder != platform_byteorder:
 *               column = get_nested_field(recarr, colpathname)
 */
        }

        /* "tables/tableextension.pyx":462
 *     if sense == 1:
 *       for colpathname in self.colpathnames:
 *         if self.coltypes[colpathname] in ["time32", "time64"]:             # <<<<<<<<<<<<<<
 *           colobj = self.coldescrs[colpathname]
 *           if hasattr(colobj, "_byteorder"):
 */
      }

      /* "tables/tableextension.pyx":461
 *     # (this is not currently supported by HDF5)
 *     if sense == 1:
 *       for colpathname in self.colpathnames:             # <<<<<<<<<<<<<<
 *         if self.coltypes[colpathname] in ["time32", "time64"]:
 *           colobj = self.coldescrs[colpathname]
 */
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "tables/tableextension.pyx":460
 *     # For reading, first swap the byteorder by hand
 *     # (this is not currently supported by HDF5)
 *     if sense == 1:             # <<<<<<<<<<<<<<
 *       for colpathname in self.colpathnames:
 *         if self.coltypes[colpathname] in ["time32", "time64"]:
 */
  }

  /* "tables/tableextension.pyx":471
 * 
 *     # This should be generalised to support other type conversions.
 *     for t64cname in self._time64colnames:             # <<<<<<<<<<<<<<
 *       column = get_nested_field(recarr, t64cname)
 *       self._convert_time64_(column, nrecords, sense)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_time64colnames); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 471, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
    __pyx_t_8 = __pyx_t_2; __Pyx_INCREF(__pyx_t_8); __pyx_t_10 = 0;
    __pyx_t_11 = NULL;
  } else {
    __pyx_t_10 = -1; __pyx_t_8 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 471, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_11 = Py_TYPE(__pyx_t_8)->tp_iternext; if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 471, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  for (;;) {
    if (likely(!__pyx_t_11)) {
      if (likely(PyList_CheckExact(__pyx_t_8))) {
        if (__pyx_t_10 >= PyList_GET_SIZE(__pyx_t_8)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyList_GET_ITEM(__pyx_t_8, __pyx_t_10); __Pyx_INCREF(__pyx_t_2); __pyx_t_10++; if (unlikely(0 < 0)) __PYX_ERR(0, 471, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_8, __pyx_t_10); __pyx_t_10++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 471, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        if (__pyx_t_10 >= PyTuple_GET_SIZE(__pyx_t_8)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_8, __pyx_t_10); __Pyx_INCREF(__pyx_t_2); __pyx_t_10++; if (unlikely(0 < 0)) __PYX_ERR(0, 471, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_8, __pyx_t_10); __pyx_t_10++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 471, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      }
    } else {
      __pyx_t_2 = __pyx_t_11(__pyx_t_8);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 471, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_XDECREF_SET(__pyx_v_t64cname, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "tables/tableextension.pyx":472
 *     # This should be generalised to support other type conversions.
 *     for t64cname in self._time64colnames:
 *       column = get_nested_field(recarr, t64cname)             # <<<<<<<<<<<<<<
 *       self._convert_time64_(column, nrecords, sense)
 * 
 */
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_get_nested_field); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 472, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = NULL;
    __pyx_t_7 = 0;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_7 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, ((PyObject *)__pyx_v_recarr), __pyx_v_t64cname};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 472, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, ((PyObject *)__pyx_v_recarr), __pyx_v_t64cname};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 472, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else
    #endif
    {
      __pyx_t_1 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 472, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (__pyx_t_4) {
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_4); __pyx_t_4 = NULL;
      }
      __Pyx_INCREF(((PyObject *)__pyx_v_recarr));
      __Pyx_GIVEREF(((PyObject *)__pyx_v_recarr));
      PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_7, ((PyObject *)__pyx_v_recarr));
      __Pyx_INCREF(__pyx_v_t64cname);
      __Pyx_GIVEREF(__pyx_v_t64cname);
      PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_7, __pyx_v_t64cname);
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 472, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF_SET(__pyx_v_column, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "tables/tableextension.pyx":473
 *     for t64cname in self._time64colnames:
 *       column = get_nested_field(recarr, t64cname)
 *       self._convert_time64_(column, nrecords, sense)             # <<<<<<<<<<<<<<
 * 
 *   def _open_append(self, ndarray recarr):
 */
    if (!(likely(((__pyx_v_column) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_column, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 473, __pyx_L1_error)
    __pyx_t_2 = ((struct __pyx_vtabstruct_6tables_14tableextension_Table *)__pyx_v_self->__pyx_base.__pyx_vtab)->_convert_time64_(__pyx_v_self, ((PyArrayObject *)__pyx_v_column), __pyx_v_nrecords, __pyx_v_sense); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 473, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "tables/tableextension.pyx":471
 * 
 *     # This should be generalised to support other type conversions.
 *     for t64cname in self._time64colnames:             # <<<<<<<<<<<<<<
 *       column = get_nested_field(recarr, t64cname)
 *       self._convert_time64_(column, nrecords, sense)
 */
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "tables/tableextension.pyx":449
 *       t64buf, byteoffset, bytestride, nrecords, nelements, sense)
 * 
 *   cpdef _convert_types(self, ndarray recarr, hsize_t nrecords, int sense):             # <<<<<<<<<<<<<<
 *     """Converts columns in 'recarr' between NumPy and HDF5 formats.
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("tables.tableextension.Table._convert_types", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_colpathname);
  __Pyx_XDECREF(__pyx_v_colobj);
  __Pyx_XDECREF(__pyx_v_column);
  __Pyx_XDECREF(__pyx_v_t64cname);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6tables_14tableextension_5Table_5_convert_types(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6tables_14tableextension_5Table_4_convert_types[] = "Converts columns in 'recarr' between NumPy and HDF5 formats.\n\n    NumPy to HDF5 conversion is performed when 'sense' is 0.  Otherwise, HDF5\n    to NumPy conversion is performed.  The conversion is done in place,\n    i.e. 'recarr' is modified.\n\n    ";
static PyObject *__pyx_pw_6tables_14tableextension_5Table_5_convert_types(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyArrayObject *__pyx_v_recarr = 0;
  hsize_t __pyx_v_nrecords;
  int __pyx_v_sense;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_convert_types (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_recarr,&__pyx_n_s_nrecords,&__pyx_n_s_sense,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_recarr)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_nrecords)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_convert_types", 1, 3, 3, 1); __PYX_ERR(0, 449, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_sense)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_convert_types", 1, 3, 3, 2); __PYX_ERR(0, 449, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_convert_types") < 0)) __PYX_ERR(0, 449, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_recarr = ((PyArrayObject *)values[0]);
    __pyx_v_nrecords = __Pyx_PyInt_As_hsize_t(values[1]); if (unlikely((__pyx_v_nrecords == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 449, __pyx_L3_error)
    __pyx_v_sense = __Pyx_PyInt_As_int(values[2]); if (unlikely((__pyx_v_sense == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 449, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_convert_types", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 449, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("tables.tableextension.Table._convert_types", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_recarr), __pyx_ptype_5numpy_ndarray, 1, "recarr", 0))) __PYX_ERR(0, 449, __pyx_L1_error)
  __pyx_r = __pyx_pf_6tables_14tableextension_5Table_4_convert_types(((struct __pyx_obj_6tables_14tableextension_Table *)__pyx_v_self), __pyx_v_recarr, __pyx_v_nrecords, __pyx_v_sense);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_14tableextension_5Table_4_convert_types(struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self, PyArrayObject *__pyx_v_recarr, hsize_t __pyx_v_nrecords, int __pyx_v_sense) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_convert_types", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6tables_14tableextension_5Table__convert_types(__pyx_v_self, __pyx_v_recarr, __pyx_v_nrecords, __pyx_v_sense, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 449, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("tables.tableextension.Table._convert_types", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":475
 *       self._convert_time64_(column, nrecords, sense)
 * 
 *   def _open_append(self, ndarray recarr):             # <<<<<<<<<<<<<<
 *     self._v_recarray = <object>recarr
 *     # Get the pointer to the buffer data area
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_14tableextension_5Table_7_open_append(PyObject *__pyx_v_self, PyObject *__pyx_v_recarr); /*proto*/
static PyObject *__pyx_pw_6tables_14tableextension_5Table_7_open_append(PyObject *__pyx_v_self, PyObject *__pyx_v_recarr) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_open_append (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_recarr), __pyx_ptype_5numpy_ndarray, 1, "recarr", 0))) __PYX_ERR(0, 475, __pyx_L1_error)
  __pyx_r = __pyx_pf_6tables_14tableextension_5Table_6_open_append(((struct __pyx_obj_6tables_14tableextension_Table *)__pyx_v_self), ((PyArrayObject *)__pyx_v_recarr));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_14tableextension_5Table_6_open_append(struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self, PyArrayObject *__pyx_v_recarr) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_open_append", 0);

  /* "tables/tableextension.pyx":476
 * 
 *   def _open_append(self, ndarray recarr):
 *     self._v_recarray = <object>recarr             # <<<<<<<<<<<<<<
 *     # Get the pointer to the buffer data area
 *     self.wbuf = PyArray_DATA(recarr)
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_recarr);
  __Pyx_INCREF(__pyx_t_1);
  if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_v_recarray, __pyx_t_1) < 0) __PYX_ERR(0, 476, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":478
 *     self._v_recarray = <object>recarr
 *     # Get the pointer to the buffer data area
 *     self.wbuf = PyArray_DATA(recarr)             # <<<<<<<<<<<<<<
 * 
 *   def _append_records(self, hsize_t nrecords):
 */
  __pyx_v_self->wbuf = PyArray_DATA(__pyx_v_recarr);

  /* "tables/tableextension.pyx":475
 *       self._convert_time64_(column, nrecords, sense)
 * 
 *   def _open_append(self, ndarray recarr):             # <<<<<<<<<<<<<<
 *     self._v_recarray = <object>recarr
 *     # Get the pointer to the buffer data area
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("tables.tableextension.Table._open_append", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":480
 *     self.wbuf = PyArray_DATA(recarr)
 * 
 *   def _append_records(self, hsize_t nrecords):             # <<<<<<<<<<<<<<
 *     cdef int ret
 *     cdef hsize_t nrows
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_14tableextension_5Table_9_append_records(PyObject *__pyx_v_self, PyObject *__pyx_arg_nrecords); /*proto*/
static PyObject *__pyx_pw_6tables_14tableextension_5Table_9_append_records(PyObject *__pyx_v_self, PyObject *__pyx_arg_nrecords) {
  hsize_t __pyx_v_nrecords;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_append_records (wrapper)", 0);
  assert(__pyx_arg_nrecords); {
    __pyx_v_nrecords = __Pyx_PyInt_As_hsize_t(__pyx_arg_nrecords); if (unlikely((__pyx_v_nrecords == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 480, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("tables.tableextension.Table._append_records", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_14tableextension_5Table_8_append_records(((struct __pyx_obj_6tables_14tableextension_Table *)__pyx_v_self), ((hsize_t)__pyx_v_nrecords));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_14tableextension_5Table_8_append_records(struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self, hsize_t __pyx_v_nrecords) {
  int __pyx_v_ret;
  hsize_t __pyx_v_nrows;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  hsize_t __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_append_records", 0);

  /* "tables/tableextension.pyx":485
 * 
 *     # Convert some NumPy types to HDF5 before storing.
 *     self._convert_types(self._v_recarray, nrecords, 0)             # <<<<<<<<<<<<<<
 * 
 *     nrows = self.nrows
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_v_recarray); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 485, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 485, __pyx_L1_error)
  __pyx_t_2 = ((struct __pyx_vtabstruct_6tables_14tableextension_Table *)__pyx_v_self->__pyx_base.__pyx_vtab)->_convert_types(__pyx_v_self, ((PyArrayObject *)__pyx_t_1), __pyx_v_nrecords, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 485, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/tableextension.pyx":487
 *     self._convert_types(self._v_recarray, nrecords, 0)
 * 
 *     nrows = self.nrows             # <<<<<<<<<<<<<<
 *     # release GIL (allow other threads to use the Python interpreter)
 *     with nogil:
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nrows); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 487, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_As_hsize_t(__pyx_t_2); if (unlikely((__pyx_t_3 == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 487, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_nrows = __pyx_t_3;

  /* "tables/tableextension.pyx":489
 *     nrows = self.nrows
 *     # release GIL (allow other threads to use the Python interpreter)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         # Append the records:
 *         ret = H5TBOappend_records(self.dataset_id, self.type_id,
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "tables/tableextension.pyx":491
 *     with nogil:
 *         # Append the records:
 *         ret = H5TBOappend_records(self.dataset_id, self.type_id,             # <<<<<<<<<<<<<<
 *                                   nrecords, nrows, self.wbuf)
 * 
 */
        __pyx_v_ret = H5TBOappend_records(__pyx_v_self->__pyx_base.dataset_id, __pyx_v_self->__pyx_base.type_id, __pyx_v_nrecords, __pyx_v_nrows, __pyx_v_self->wbuf);
      }

      /* "tables/tableextension.pyx":489
 *     nrows = self.nrows
 *     # release GIL (allow other threads to use the Python interpreter)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         # Append the records:
 *         ret = H5TBOappend_records(self.dataset_id, self.type_id,
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "tables/tableextension.pyx":494
 *                                   nrecords, nrows, self.wbuf)
 * 
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems appending the records.")
 * 
 */
  __pyx_t_4 = ((__pyx_v_ret < 0) != 0);
  if (unlikely(__pyx_t_4)) {

    /* "tables/tableextension.pyx":495
 * 
 *     if ret < 0:
 *       raise HDF5ExtError("Problems appending the records.")             # <<<<<<<<<<<<<<
 * 
 *     self.nrows = self.nrows + nrecords
 */
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_HDF5ExtError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 495, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_5, __pyx_kp_s_Problems_appending_the_records) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_kp_s_Problems_appending_the_records);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 495, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 495, __pyx_L1_error)

    /* "tables/tableextension.pyx":494
 *                                   nrecords, nrows, self.wbuf)
 * 
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems appending the records.")
 * 
 */
  }

  /* "tables/tableextension.pyx":497
 *       raise HDF5ExtError("Problems appending the records.")
 * 
 *     self.nrows = self.nrows + nrecords             # <<<<<<<<<<<<<<
 * 
 *   def _close_append(self):
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nrows); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_From_hsize_t(__pyx_v_nrecords); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyNumber_Add(__pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nrows, __pyx_t_5) < 0) __PYX_ERR(0, 497, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "tables/tableextension.pyx":480
 *     self.wbuf = PyArray_DATA(recarr)
 * 
 *   def _append_records(self, hsize_t nrecords):             # <<<<<<<<<<<<<<
 *     cdef int ret
 *     cdef hsize_t nrows
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("tables.tableextension.Table._append_records", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":499
 *     self.nrows = self.nrows + nrecords
 * 
 *   def _close_append(self):             # <<<<<<<<<<<<<<
 *     cdef hsize_t nrows
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_14tableextension_5Table_11_close_append(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_6tables_14tableextension_5Table_11_close_append(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_close_append (wrapper)", 0);
  __pyx_r = __pyx_pf_6tables_14tableextension_5Table_10_close_append(((struct __pyx_obj_6tables_14tableextension_Table *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_14tableextension_5Table_10_close_append(struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self) {
  hsize_t __pyx_v_nrows;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  hsize_t __pyx_t_4;
  hid_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_close_append", 0);

  /* "tables/tableextension.pyx":502
 *     cdef hsize_t nrows
 * 
 *     if self._v_file.params['PYTABLES_SYS_ATTRS']:             # <<<<<<<<<<<<<<
 *       # Update the NROWS attribute
 *       nrows = self.nrows
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_v_file); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 502, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_params); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 502, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_s_PYTABLES_SYS_ATTRS); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 502, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 502, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {

    /* "tables/tableextension.pyx":504
 *     if self._v_file.params['PYTABLES_SYS_ATTRS']:
 *       # Update the NROWS attribute
 *       nrows = self.nrows             # <<<<<<<<<<<<<<
 *       if (H5ATTRset_attribute(self.dataset_id, "NROWS", H5T_STD_I64,
 *                               0, NULL, <char *>&nrows) < 0):
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nrows); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 504, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyInt_As_hsize_t(__pyx_t_1); if (unlikely((__pyx_t_4 == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 504, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_nrows = __pyx_t_4;

    /* "tables/tableextension.pyx":505
 *       # Update the NROWS attribute
 *       nrows = self.nrows
 *       if (H5ATTRset_attribute(self.dataset_id, "NROWS", H5T_STD_I64,             # <<<<<<<<<<<<<<
 *                               0, NULL, <char *>&nrows) < 0):
 *         raise HDF5ExtError("Problems setting the NROWS attribute.")
 */
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_H5T_STD_I64); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 505, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_PyInt_As_hid_t(__pyx_t_1); if (unlikely((__pyx_t_5 == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 505, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "tables/tableextension.pyx":506
 *       nrows = self.nrows
 *       if (H5ATTRset_attribute(self.dataset_id, "NROWS", H5T_STD_I64,
 *                               0, NULL, <char *>&nrows) < 0):             # <<<<<<<<<<<<<<
 *         raise HDF5ExtError("Problems setting the NROWS attribute.")
 * 
 */
    __pyx_t_3 = ((H5ATTRset_attribute(__pyx_v_self->__pyx_base.dataset_id, ((char *)"NROWS"), __pyx_t_5, 0, NULL, ((char *)(&__pyx_v_nrows))) < 0) != 0);

    /* "tables/tableextension.pyx":505
 *       # Update the NROWS attribute
 *       nrows = self.nrows
 *       if (H5ATTRset_attribute(self.dataset_id, "NROWS", H5T_STD_I64,             # <<<<<<<<<<<<<<
 *                               0, NULL, <char *>&nrows) < 0):
 *         raise HDF5ExtError("Problems setting the NROWS attribute.")
 */
    if (unlikely(__pyx_t_3)) {

      /* "tables/tableextension.pyx":507
 *       if (H5ATTRset_attribute(self.dataset_id, "NROWS", H5T_STD_I64,
 *                               0, NULL, <char *>&nrows) < 0):
 *         raise HDF5ExtError("Problems setting the NROWS attribute.")             # <<<<<<<<<<<<<<
 * 
 *     # Set the caches to dirty (in fact, and for the append case,
 */
      __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_HDF5ExtError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 507, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_6 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_1 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_6, __pyx_kp_s_Problems_setting_the_NROWS_attri) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_s_Problems_setting_the_NROWS_attri);
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 507, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_Raise(__pyx_t_1, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __PYX_ERR(0, 507, __pyx_L1_error)

      /* "tables/tableextension.pyx":505
 *       # Update the NROWS attribute
 *       nrows = self.nrows
 *       if (H5ATTRset_attribute(self.dataset_id, "NROWS", H5T_STD_I64,             # <<<<<<<<<<<<<<
 *                               0, NULL, <char *>&nrows) < 0):
 *         raise HDF5ExtError("Problems setting the NROWS attribute.")
 */
    }

    /* "tables/tableextension.pyx":502
 *     cdef hsize_t nrows
 * 
 *     if self._v_file.params['PYTABLES_SYS_ATTRS']:             # <<<<<<<<<<<<<<
 *       # Update the NROWS attribute
 *       nrows = self.nrows
 */
  }

  /* "tables/tableextension.pyx":511
 *     # Set the caches to dirty (in fact, and for the append case,
 *     # it should be only the caches based on limits, but anyway)
 *     self._dirtycache = True             # <<<<<<<<<<<<<<
 *     # Delete the reference to recarray as we doesn't need it anymore
 *     self._v_recarray = None
 */
  if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_dirtycache, Py_True) < 0) __PYX_ERR(0, 511, __pyx_L1_error)

  /* "tables/tableextension.pyx":513
 *     self._dirtycache = True
 *     # Delete the reference to recarray as we doesn't need it anymore
 *     self._v_recarray = None             # <<<<<<<<<<<<<<
 * 
 *   def _update_records(self, hsize_t start, hsize_t stop,
 */
  if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_v_recarray, Py_None) < 0) __PYX_ERR(0, 513, __pyx_L1_error)

  /* "tables/tableextension.pyx":499
 *     self.nrows = self.nrows + nrecords
 * 
 *   def _close_append(self):             # <<<<<<<<<<<<<<
 *     cdef hsize_t nrows
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("tables.tableextension.Table._close_append", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":515
 *     self._v_recarray = None
 * 
 *   def _update_records(self, hsize_t start, hsize_t stop,             # <<<<<<<<<<<<<<
 *                       hsize_t step, ndarray recarr):
 *     cdef herr_t ret
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_14tableextension_5Table_13_update_records(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_6tables_14tableextension_5Table_13_update_records(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  hsize_t __pyx_v_start;
  hsize_t __pyx_v_stop;
  hsize_t __pyx_v_step;
  PyArrayObject *__pyx_v_recarr = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_update_records (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_start,&__pyx_n_s_stop,&__pyx_n_s_step,&__pyx_n_s_recarr,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_start)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_stop)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_update_records", 1, 4, 4, 1); __PYX_ERR(0, 515, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_step)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_update_records", 1, 4, 4, 2); __PYX_ERR(0, 515, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_recarr)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_update_records", 1, 4, 4, 3); __PYX_ERR(0, 515, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_update_records") < 0)) __PYX_ERR(0, 515, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_start = __Pyx_PyInt_As_hsize_t(values[0]); if (unlikely((__pyx_v_start == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 515, __pyx_L3_error)
    __pyx_v_stop = __Pyx_PyInt_As_hsize_t(values[1]); if (unlikely((__pyx_v_stop == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 515, __pyx_L3_error)
    __pyx_v_step = __Pyx_PyInt_As_hsize_t(values[2]); if (unlikely((__pyx_v_step == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 516, __pyx_L3_error)
    __pyx_v_recarr = ((PyArrayObject *)values[3]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_update_records", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 515, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("tables.tableextension.Table._update_records", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_recarr), __pyx_ptype_5numpy_ndarray, 1, "recarr", 0))) __PYX_ERR(0, 516, __pyx_L1_error)
  __pyx_r = __pyx_pf_6tables_14tableextension_5Table_12_update_records(((struct __pyx_obj_6tables_14tableextension_Table *)__pyx_v_self), __pyx_v_start, __pyx_v_stop, __pyx_v_step, __pyx_v_recarr);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_14tableextension_5Table_12_update_records(struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self, hsize_t __pyx_v_start, hsize_t __pyx_v_stop, hsize_t __pyx_v_step, PyArrayObject *__pyx_v_recarr) {
  herr_t __pyx_v_ret;
  void *__pyx_v_rbuf;
  hsize_t __pyx_v_nrecords;
  hsize_t __pyx_v_nrows;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_update_records", 0);

  /* "tables/tableextension.pyx":522
 * 
 *     # Get the pointer to the buffer data area
 *     rbuf = PyArray_DATA(recarr)             # <<<<<<<<<<<<<<
 * 
 *     # Compute the number of records to update
 */
  __pyx_v_rbuf = PyArray_DATA(__pyx_v_recarr);

  /* "tables/tableextension.pyx":525
 * 
 *     # Compute the number of records to update
 *     nrecords = len(recarr)             # <<<<<<<<<<<<<<
 *     nrows = get_len_of_range(start, stop, step)
 *     if nrecords > nrows:
 */
  __pyx_t_1 = PyObject_Length(((PyObject *)__pyx_v_recarr)); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 525, __pyx_L1_error)
  __pyx_v_nrecords = __pyx_t_1;

  /* "tables/tableextension.pyx":526
 *     # Compute the number of records to update
 *     nrecords = len(recarr)
 *     nrows = get_len_of_range(start, stop, step)             # <<<<<<<<<<<<<<
 *     if nrecords > nrows:
 *       nrecords = nrows
 */
  __pyx_v_nrows = get_len_of_range(__pyx_v_start, __pyx_v_stop, __pyx_v_step);

  /* "tables/tableextension.pyx":527
 *     nrecords = len(recarr)
 *     nrows = get_len_of_range(start, stop, step)
 *     if nrecords > nrows:             # <<<<<<<<<<<<<<
 *       nrecords = nrows
 * 
 */
  __pyx_t_2 = ((__pyx_v_nrecords > __pyx_v_nrows) != 0);
  if (__pyx_t_2) {

    /* "tables/tableextension.pyx":528
 *     nrows = get_len_of_range(start, stop, step)
 *     if nrecords > nrows:
 *       nrecords = nrows             # <<<<<<<<<<<<<<
 * 
 *     # Convert some NumPy types to HDF5 before storing.
 */
    __pyx_v_nrecords = __pyx_v_nrows;

    /* "tables/tableextension.pyx":527
 *     nrecords = len(recarr)
 *     nrows = get_len_of_range(start, stop, step)
 *     if nrecords > nrows:             # <<<<<<<<<<<<<<
 *       nrecords = nrows
 * 
 */
  }

  /* "tables/tableextension.pyx":531
 * 
 *     # Convert some NumPy types to HDF5 before storing.
 *     self._convert_types(recarr, nrecords, 0)             # <<<<<<<<<<<<<<
 *     # Update the records:
 *     with nogil:
 */
  __pyx_t_3 = ((struct __pyx_vtabstruct_6tables_14tableextension_Table *)__pyx_v_self->__pyx_base.__pyx_vtab)->_convert_types(__pyx_v_self, __pyx_v_recarr, __pyx_v_nrecords, 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 531, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "tables/tableextension.pyx":533
 *     self._convert_types(recarr, nrecords, 0)
 *     # Update the records:
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5TBOwrite_records(self.dataset_id, self.type_id,
 *                                  start, nrecords, step, rbuf )
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "tables/tableextension.pyx":534
 *     # Update the records:
 *     with nogil:
 *         ret = H5TBOwrite_records(self.dataset_id, self.type_id,             # <<<<<<<<<<<<<<
 *                                  start, nrecords, step, rbuf )
 * 
 */
        __pyx_v_ret = H5TBOwrite_records(__pyx_v_self->__pyx_base.dataset_id, __pyx_v_self->__pyx_base.type_id, __pyx_v_start, __pyx_v_nrecords, __pyx_v_step, __pyx_v_rbuf);
      }

      /* "tables/tableextension.pyx":533
 *     self._convert_types(recarr, nrecords, 0)
 *     # Update the records:
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5TBOwrite_records(self.dataset_id, self.type_id,
 *                                  start, nrecords, step, rbuf )
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "tables/tableextension.pyx":537
 *                                  start, nrecords, step, rbuf )
 * 
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems updating the records.")
 * 
 */
  __pyx_t_2 = ((__pyx_v_ret < 0) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "tables/tableextension.pyx":538
 * 
 *     if ret < 0:
 *       raise HDF5ExtError("Problems updating the records.")             # <<<<<<<<<<<<<<
 * 
 *     # Set the caches to dirty
 */
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_HDF5ExtError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 538, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_kp_s_Problems_updating_the_records) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_kp_s_Problems_updating_the_records);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 538, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 538, __pyx_L1_error)

    /* "tables/tableextension.pyx":537
 *                                  start, nrecords, step, rbuf )
 * 
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems updating the records.")
 * 
 */
  }

  /* "tables/tableextension.pyx":541
 * 
 *     # Set the caches to dirty
 *     self._dirtycache = True             # <<<<<<<<<<<<<<
 * 
 *   def _update_elements(self, hsize_t nrecords, ndarray coords,
 */
  if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_dirtycache, Py_True) < 0) __PYX_ERR(0, 541, __pyx_L1_error)

  /* "tables/tableextension.pyx":515
 *     self._v_recarray = None
 * 
 *   def _update_records(self, hsize_t start, hsize_t stop,             # <<<<<<<<<<<<<<
 *                       hsize_t step, ndarray recarr):
 *     cdef herr_t ret
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("tables.tableextension.Table._update_records", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":543
 *     self._dirtycache = True
 * 
 *   def _update_elements(self, hsize_t nrecords, ndarray coords,             # <<<<<<<<<<<<<<
 *                        ndarray recarr):
 *     cdef herr_t ret
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_14tableextension_5Table_15_update_elements(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_6tables_14tableextension_5Table_15_update_elements(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  hsize_t __pyx_v_nrecords;
  PyArrayObject *__pyx_v_coords = 0;
  PyArrayObject *__pyx_v_recarr = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_update_elements (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_nrecords,&__pyx_n_s_coords,&__pyx_n_s_recarr,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_nrecords)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_coords)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_update_elements", 1, 3, 3, 1); __PYX_ERR(0, 543, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_recarr)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_update_elements", 1, 3, 3, 2); __PYX_ERR(0, 543, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_update_elements") < 0)) __PYX_ERR(0, 543, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_nrecords = __Pyx_PyInt_As_hsize_t(values[0]); if (unlikely((__pyx_v_nrecords == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 543, __pyx_L3_error)
    __pyx_v_coords = ((PyArrayObject *)values[1]);
    __pyx_v_recarr = ((PyArrayObject *)values[2]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_update_elements", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 543, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("tables.tableextension.Table._update_elements", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_coords), __pyx_ptype_5numpy_ndarray, 1, "coords", 0))) __PYX_ERR(0, 543, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_recarr), __pyx_ptype_5numpy_ndarray, 1, "recarr", 0))) __PYX_ERR(0, 544, __pyx_L1_error)
  __pyx_r = __pyx_pf_6tables_14tableextension_5Table_14_update_elements(((struct __pyx_obj_6tables_14tableextension_Table *)__pyx_v_self), __pyx_v_nrecords, __pyx_v_coords, __pyx_v_recarr);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_14tableextension_5Table_14_update_elements(struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self, hsize_t __pyx_v_nrecords, PyArrayObject *__pyx_v_coords, PyArrayObject *__pyx_v_recarr) {
  herr_t __pyx_v_ret;
  void *__pyx_v_rbuf;
  void *__pyx_v_rcoords;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_update_elements", 0);

  /* "tables/tableextension.pyx":550
 * 
 *     # Get the chunk of the coords that correspond to a buffer
 *     rcoords = PyArray_DATA(coords)             # <<<<<<<<<<<<<<
 * 
 *     # Get the pointer to the buffer data area
 */
  __pyx_v_rcoords = PyArray_DATA(__pyx_v_coords);

  /* "tables/tableextension.pyx":553
 * 
 *     # Get the pointer to the buffer data area
 *     rbuf = PyArray_DATA(recarr)             # <<<<<<<<<<<<<<
 * 
 *     # Convert some NumPy types to HDF5 before storing.
 */
  __pyx_v_rbuf = PyArray_DATA(__pyx_v_recarr);

  /* "tables/tableextension.pyx":556
 * 
 *     # Convert some NumPy types to HDF5 before storing.
 *     self._convert_types(recarr, nrecords, 0)             # <<<<<<<<<<<<<<
 * 
 *     # Update the records:
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6tables_14tableextension_Table *)__pyx_v_self->__pyx_base.__pyx_vtab)->_convert_types(__pyx_v_self, __pyx_v_recarr, __pyx_v_nrecords, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 556, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":559
 * 
 *     # Update the records:
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5TBOwrite_elements(self.dataset_id, self.type_id,
 *                                   nrecords, rcoords, rbuf)
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "tables/tableextension.pyx":560
 *     # Update the records:
 *     with nogil:
 *         ret = H5TBOwrite_elements(self.dataset_id, self.type_id,             # <<<<<<<<<<<<<<
 *                                   nrecords, rcoords, rbuf)
 * 
 */
        __pyx_v_ret = H5TBOwrite_elements(__pyx_v_self->__pyx_base.dataset_id, __pyx_v_self->__pyx_base.type_id, __pyx_v_nrecords, __pyx_v_rcoords, __pyx_v_rbuf);
      }

      /* "tables/tableextension.pyx":559
 * 
 *     # Update the records:
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5TBOwrite_elements(self.dataset_id, self.type_id,
 *                                   nrecords, rcoords, rbuf)
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "tables/tableextension.pyx":563
 *                                   nrecords, rcoords, rbuf)
 * 
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems updating the records.")
 * 
 */
  __pyx_t_2 = ((__pyx_v_ret < 0) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "tables/tableextension.pyx":564
 * 
 *     if ret < 0:
 *       raise HDF5ExtError("Problems updating the records.")             # <<<<<<<<<<<<<<
 * 
 *     # Set the caches to dirty
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_HDF5ExtError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 564, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_kp_s_Problems_updating_the_records) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_kp_s_Problems_updating_the_records);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 564, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 564, __pyx_L1_error)

    /* "tables/tableextension.pyx":563
 *                                   nrecords, rcoords, rbuf)
 * 
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems updating the records.")
 * 
 */
  }

  /* "tables/tableextension.pyx":567
 * 
 *     # Set the caches to dirty
 *     self._dirtycache = True             # <<<<<<<<<<<<<<
 * 
 *   def _read_records(self, hsize_t start, hsize_t nrecords, ndarray recarr):
 */
  if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_dirtycache, Py_True) < 0) __PYX_ERR(0, 567, __pyx_L1_error)

  /* "tables/tableextension.pyx":543
 *     self._dirtycache = True
 * 
 *   def _update_elements(self, hsize_t nrecords, ndarray coords,             # <<<<<<<<<<<<<<
 *                        ndarray recarr):
 *     cdef herr_t ret
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("tables.tableextension.Table._update_elements", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":569
 *     self._dirtycache = True
 * 
 *   def _read_records(self, hsize_t start, hsize_t nrecords, ndarray recarr):             # <<<<<<<<<<<<<<
 *     cdef void *rbuf
 *     cdef int ret
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_14tableextension_5Table_17_read_records(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_6tables_14tableextension_5Table_17_read_records(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  hsize_t __pyx_v_start;
  hsize_t __pyx_v_nrecords;
  PyArrayObject *__pyx_v_recarr = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_read_records (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_start,&__pyx_n_s_nrecords,&__pyx_n_s_recarr,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_start)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_nrecords)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_read_records", 1, 3, 3, 1); __PYX_ERR(0, 569, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_recarr)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_read_records", 1, 3, 3, 2); __PYX_ERR(0, 569, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_read_records") < 0)) __PYX_ERR(0, 569, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_start = __Pyx_PyInt_As_hsize_t(values[0]); if (unlikely((__pyx_v_start == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 569, __pyx_L3_error)
    __pyx_v_nrecords = __Pyx_PyInt_As_hsize_t(values[1]); if (unlikely((__pyx_v_nrecords == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 569, __pyx_L3_error)
    __pyx_v_recarr = ((PyArrayObject *)values[2]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_read_records", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 569, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("tables.tableextension.Table._read_records", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_recarr), __pyx_ptype_5numpy_ndarray, 1, "recarr", 0))) __PYX_ERR(0, 569, __pyx_L1_error)
  __pyx_r = __pyx_pf_6tables_14tableextension_5Table_16_read_records(((struct __pyx_obj_6tables_14tableextension_Table *)__pyx_v_self), __pyx_v_start, __pyx_v_nrecords, __pyx_v_recarr);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_14tableextension_5Table_16_read_records(struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self, hsize_t __pyx_v_start, hsize_t __pyx_v_nrecords, PyArrayObject *__pyx_v_recarr) {
  void *__pyx_v_rbuf;
  int __pyx_v_ret;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  hsize_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_read_records", 0);

  /* "tables/tableextension.pyx":574
 * 
 *     # Correct the number of records to read, if needed
 *     if (start + nrecords) > self.nrows:             # <<<<<<<<<<<<<<
 *       nrecords = self.nrows - start
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_hsize_t((__pyx_v_start + __pyx_v_nrecords)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 574, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nrows); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 574, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyObject_RichCompare(__pyx_t_1, __pyx_t_2, Py_GT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 574, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 574, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_4) {

    /* "tables/tableextension.pyx":575
 *     # Correct the number of records to read, if needed
 *     if (start + nrecords) > self.nrows:
 *       nrecords = self.nrows - start             # <<<<<<<<<<<<<<
 * 
 *     # Get the pointer to the buffer data area
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nrows); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 575, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyInt_From_hsize_t(__pyx_v_start); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 575, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = PyNumber_Subtract(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 575, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_5 = __Pyx_PyInt_As_hsize_t(__pyx_t_1); if (unlikely((__pyx_t_5 == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 575, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_nrecords = __pyx_t_5;

    /* "tables/tableextension.pyx":574
 * 
 *     # Correct the number of records to read, if needed
 *     if (start + nrecords) > self.nrows:             # <<<<<<<<<<<<<<
 *       nrecords = self.nrows - start
 * 
 */
  }

  /* "tables/tableextension.pyx":578
 * 
 *     # Get the pointer to the buffer data area
 *     rbuf = PyArray_DATA(recarr)             # <<<<<<<<<<<<<<
 * 
 *     # Read the records from disk
 */
  __pyx_v_rbuf = PyArray_DATA(__pyx_v_recarr);

  /* "tables/tableextension.pyx":581
 * 
 *     # Read the records from disk
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5TBOread_records(self.dataset_id, self.type_id, start,
 *                                 nrecords, rbuf)
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "tables/tableextension.pyx":582
 *     # Read the records from disk
 *     with nogil:
 *         ret = H5TBOread_records(self.dataset_id, self.type_id, start,             # <<<<<<<<<<<<<<
 *                                 nrecords, rbuf)
 * 
 */
        __pyx_v_ret = H5TBOread_records(__pyx_v_self->__pyx_base.dataset_id, __pyx_v_self->__pyx_base.type_id, __pyx_v_start, __pyx_v_nrecords, __pyx_v_rbuf);
      }

      /* "tables/tableextension.pyx":581
 * 
 *     # Read the records from disk
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5TBOread_records(self.dataset_id, self.type_id, start,
 *                                 nrecords, rbuf)
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "tables/tableextension.pyx":585
 *                                 nrecords, rbuf)
 * 
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems reading records.")
 * 
 */
  __pyx_t_4 = ((__pyx_v_ret < 0) != 0);
  if (unlikely(__pyx_t_4)) {

    /* "tables/tableextension.pyx":586
 * 
 *     if ret < 0:
 *       raise HDF5ExtError("Problems reading records.")             # <<<<<<<<<<<<<<
 * 
 *     # Convert some HDF5 types to NumPy after reading.
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_HDF5ExtError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 586, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_kp_s_Problems_reading_records) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_s_Problems_reading_records);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 586, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 586, __pyx_L1_error)

    /* "tables/tableextension.pyx":585
 *                                 nrecords, rbuf)
 * 
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems reading records.")
 * 
 */
  }

  /* "tables/tableextension.pyx":589
 * 
 *     # Convert some HDF5 types to NumPy after reading.
 *     self._convert_types(recarr, nrecords, 1)             # <<<<<<<<<<<<<<
 * 
 *     return nrecords
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6tables_14tableextension_Table *)__pyx_v_self->__pyx_base.__pyx_vtab)->_convert_types(__pyx_v_self, __pyx_v_recarr, __pyx_v_nrecords, 1, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 589, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":591
 *     self._convert_types(recarr, nrecords, 1)
 * 
 *     return nrecords             # <<<<<<<<<<<<<<
 * 
 *   cdef hsize_t _read_chunk(self, hsize_t nchunk, ndarray iobuf, long cstart):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_hsize_t(__pyx_v_nrecords); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 591, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "tables/tableextension.pyx":569
 *     self._dirtycache = True
 * 
 *   def _read_records(self, hsize_t start, hsize_t nrecords, ndarray recarr):             # <<<<<<<<<<<<<<
 *     cdef void *rbuf
 *     cdef int ret
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("tables.tableextension.Table._read_records", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":593
 *     return nrecords
 * 
 *   cdef hsize_t _read_chunk(self, hsize_t nchunk, ndarray iobuf, long cstart):             # <<<<<<<<<<<<<<
 *     cdef long nslot
 *     cdef hsize_t start, nrecords, chunkshape
 */

static hsize_t __pyx_f_6tables_14tableextension_5Table__read_chunk(struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self, hsize_t __pyx_v_nchunk, PyArrayObject *__pyx_v_iobuf, long __pyx_v_cstart) {
  long __pyx_v_nslot;
  hsize_t __pyx_v_start;
  hsize_t __pyx_v_nrecords;
  hsize_t __pyx_v_chunkshape;
  int __pyx_v_ret;
  void *__pyx_v_rbuf;
  struct __pyx_obj_6tables_17lrucacheextension_NumCache *__pyx_v_chunkcache = 0;
  hsize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  long __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  hsize_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_read_chunk", 0);

  /* "tables/tableextension.pyx":600
 *     cdef NumCache chunkcache
 * 
 *     chunkcache = self._chunkcache             # <<<<<<<<<<<<<<
 *     chunkshape = chunkcache.slotsize
 *     # Correct the number of records to read, if needed
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_chunkcache); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 600, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6tables_17lrucacheextension_NumCache))))) __PYX_ERR(0, 600, __pyx_L1_error)
  __pyx_v_chunkcache = ((struct __pyx_obj_6tables_17lrucacheextension_NumCache *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":601
 * 
 *     chunkcache = self._chunkcache
 *     chunkshape = chunkcache.slotsize             # <<<<<<<<<<<<<<
 *     # Correct the number of records to read, if needed
 *     start = nchunk*chunkshape
 */
  __pyx_t_2 = __pyx_v_chunkcache->slotsize;
  __pyx_v_chunkshape = __pyx_t_2;

  /* "tables/tableextension.pyx":603
 *     chunkshape = chunkcache.slotsize
 *     # Correct the number of records to read, if needed
 *     start = nchunk*chunkshape             # <<<<<<<<<<<<<<
 *     nrecords = chunkshape
 *     if (start + nrecords) > self.nrows:
 */
  __pyx_v_start = (__pyx_v_nchunk * __pyx_v_chunkshape);

  /* "tables/tableextension.pyx":604
 *     # Correct the number of records to read, if needed
 *     start = nchunk*chunkshape
 *     nrecords = chunkshape             # <<<<<<<<<<<<<<
 *     if (start + nrecords) > self.nrows:
 *       nrecords = self.nrows - start
 */
  __pyx_v_nrecords = __pyx_v_chunkshape;

  /* "tables/tableextension.pyx":605
 *     start = nchunk*chunkshape
 *     nrecords = chunkshape
 *     if (start + nrecords) > self.nrows:             # <<<<<<<<<<<<<<
 *       nrecords = self.nrows - start
 *     rbuf = PyArray_BYTES(iobuf) + cstart * chunkcache.itemsize
 */
  __pyx_t_1 = __Pyx_PyInt_From_hsize_t((__pyx_v_start + __pyx_v_nrecords)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 605, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nrows); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 605, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyObject_RichCompare(__pyx_t_1, __pyx_t_3, Py_GT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 605, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 605, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__pyx_t_5) {

    /* "tables/tableextension.pyx":606
 *     nrecords = chunkshape
 *     if (start + nrecords) > self.nrows:
 *       nrecords = self.nrows - start             # <<<<<<<<<<<<<<
 *     rbuf = PyArray_BYTES(iobuf) + cstart * chunkcache.itemsize
 *     # Try to see if the chunk is in cache
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nrows); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 606, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __Pyx_PyInt_From_hsize_t(__pyx_v_start); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 606, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = PyNumber_Subtract(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 606, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_6 = __Pyx_PyInt_As_hsize_t(__pyx_t_1); if (unlikely((__pyx_t_6 == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 606, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_nrecords = __pyx_t_6;

    /* "tables/tableextension.pyx":605
 *     start = nchunk*chunkshape
 *     nrecords = chunkshape
 *     if (start + nrecords) > self.nrows:             # <<<<<<<<<<<<<<
 *       nrecords = self.nrows - start
 *     rbuf = PyArray_BYTES(iobuf) + cstart * chunkcache.itemsize
 */
  }

  /* "tables/tableextension.pyx":607
 *     if (start + nrecords) > self.nrows:
 *       nrecords = self.nrows - start
 *     rbuf = PyArray_BYTES(iobuf) + cstart * chunkcache.itemsize             # <<<<<<<<<<<<<<
 *     # Try to see if the chunk is in cache
 *     nslot = chunkcache.getslot_(nchunk)
 */
  __pyx_v_rbuf = (PyArray_BYTES(__pyx_v_iobuf) + (__pyx_v_cstart * __pyx_v_chunkcache->itemsize));

  /* "tables/tableextension.pyx":609
 *     rbuf = PyArray_BYTES(iobuf) + cstart * chunkcache.itemsize
 *     # Try to see if the chunk is in cache
 *     nslot = chunkcache.getslot_(nchunk)             # <<<<<<<<<<<<<<
 *     if nslot >= 0:
 *       chunkcache.getitem_(nslot, rbuf, 0)
 */
  __pyx_v_nslot = ((struct __pyx_vtabstruct_6tables_17lrucacheextension_NumCache *)__pyx_v_chunkcache->__pyx_base.__pyx_vtab)->getslot_(__pyx_v_chunkcache, __pyx_v_nchunk);

  /* "tables/tableextension.pyx":610
 *     # Try to see if the chunk is in cache
 *     nslot = chunkcache.getslot_(nchunk)
 *     if nslot >= 0:             # <<<<<<<<<<<<<<
 *       chunkcache.getitem_(nslot, rbuf, 0)
 *     else:
 */
  __pyx_t_5 = ((__pyx_v_nslot >= 0) != 0);
  if (__pyx_t_5) {

    /* "tables/tableextension.pyx":611
 *     nslot = chunkcache.getslot_(nchunk)
 *     if nslot >= 0:
 *       chunkcache.getitem_(nslot, rbuf, 0)             # <<<<<<<<<<<<<<
 *     else:
 *       # Chunk is not in cache. Read it and put it in the LRU cache.
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_6tables_17lrucacheextension_NumCache *)__pyx_v_chunkcache->__pyx_base.__pyx_vtab)->getitem_(__pyx_v_chunkcache, __pyx_v_nslot, __pyx_v_rbuf, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 611, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "tables/tableextension.pyx":610
 *     # Try to see if the chunk is in cache
 *     nslot = chunkcache.getslot_(nchunk)
 *     if nslot >= 0:             # <<<<<<<<<<<<<<
 *       chunkcache.getitem_(nslot, rbuf, 0)
 *     else:
 */
    goto __pyx_L4;
  }

  /* "tables/tableextension.pyx":614
 *     else:
 *       # Chunk is not in cache. Read it and put it in the LRU cache.
 *       with nogil:             # <<<<<<<<<<<<<<
 *           ret = H5TBOread_records(self.dataset_id, self.type_id,
 *                                   start, nrecords, rbuf)
 */
  /*else*/ {
    {
        #ifdef WITH_THREAD
        PyThreadState *_save;
        Py_UNBLOCK_THREADS
        __Pyx_FastGIL_Remember();
        #endif
        /*try:*/ {

          /* "tables/tableextension.pyx":615
 *       # Chunk is not in cache. Read it and put it in the LRU cache.
 *       with nogil:
 *           ret = H5TBOread_records(self.dataset_id, self.type_id,             # <<<<<<<<<<<<<<
 *                                   start, nrecords, rbuf)
 * 
 */
          __pyx_v_ret = H5TBOread_records(__pyx_v_self->__pyx_base.dataset_id, __pyx_v_self->__pyx_base.type_id, __pyx_v_start, __pyx_v_nrecords, __pyx_v_rbuf);
        }

        /* "tables/tableextension.pyx":614
 *     else:
 *       # Chunk is not in cache. Read it and put it in the LRU cache.
 *       with nogil:             # <<<<<<<<<<<<<<
 *           ret = H5TBOread_records(self.dataset_id, self.type_id,
 *                                   start, nrecords, rbuf)
 */
        /*finally:*/ {
          /*normal exit:*/{
            #ifdef WITH_THREAD
            __Pyx_FastGIL_Forget();
            Py_BLOCK_THREADS
            #endif
            goto __pyx_L7;
          }
          __pyx_L7:;
        }
    }

    /* "tables/tableextension.pyx":618
 *                                   start, nrecords, rbuf)
 * 
 *       if ret < 0:             # <<<<<<<<<<<<<<
 *         raise HDF5ExtError("Problems reading chunk records.")
 *       nslot = chunkcache.setitem_(nchunk, rbuf, 0)
 */
    __pyx_t_5 = ((__pyx_v_ret < 0) != 0);
    if (unlikely(__pyx_t_5)) {

      /* "tables/tableextension.pyx":619
 * 
 *       if ret < 0:
 *         raise HDF5ExtError("Problems reading chunk records.")             # <<<<<<<<<<<<<<
 *       nslot = chunkcache.setitem_(nchunk, rbuf, 0)
 *     return nrecords
 */
      __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_HDF5ExtError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 619, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_kp_s_Problems_reading_chunk_records) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_kp_s_Problems_reading_chunk_records);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 619, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_Raise(__pyx_t_1, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __PYX_ERR(0, 619, __pyx_L1_error)

      /* "tables/tableextension.pyx":618
 *                                   start, nrecords, rbuf)
 * 
 *       if ret < 0:             # <<<<<<<<<<<<<<
 *         raise HDF5ExtError("Problems reading chunk records.")
 *       nslot = chunkcache.setitem_(nchunk, rbuf, 0)
 */
    }

    /* "tables/tableextension.pyx":620
 *       if ret < 0:
 *         raise HDF5ExtError("Problems reading chunk records.")
 *       nslot = chunkcache.setitem_(nchunk, rbuf, 0)             # <<<<<<<<<<<<<<
 *     return nrecords
 * 
 */
    __pyx_v_nslot = ((struct __pyx_vtabstruct_6tables_17lrucacheextension_NumCache *)__pyx_v_chunkcache->__pyx_base.__pyx_vtab)->setitem_(__pyx_v_chunkcache, __pyx_v_nchunk, __pyx_v_rbuf, 0);
  }
  __pyx_L4:;

  /* "tables/tableextension.pyx":621
 *         raise HDF5ExtError("Problems reading chunk records.")
 *       nslot = chunkcache.setitem_(nchunk, rbuf, 0)
 *     return nrecords             # <<<<<<<<<<<<<<
 * 
 *   def _read_elements(self, ndarray coords, ndarray recarr):
 */
  __pyx_r = __pyx_v_nrecords;
  goto __pyx_L0;

  /* "tables/tableextension.pyx":593
 *     return nrecords
 * 
 *   cdef hsize_t _read_chunk(self, hsize_t nchunk, ndarray iobuf, long cstart):             # <<<<<<<<<<<<<<
 *     cdef long nslot
 *     cdef hsize_t start, nrecords, chunkshape
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("tables.tableextension.Table._read_chunk", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_chunkcache);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":623
 *     return nrecords
 * 
 *   def _read_elements(self, ndarray coords, ndarray recarr):             # <<<<<<<<<<<<<<
 *     cdef long nrecords
 *     cdef void *rbuf
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_14tableextension_5Table_19_read_elements(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_6tables_14tableextension_5Table_19_read_elements(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyArrayObject *__pyx_v_coords = 0;
  PyArrayObject *__pyx_v_recarr = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_read_elements (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_coords,&__pyx_n_s_recarr,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_coords)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_recarr)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_read_elements", 1, 2, 2, 1); __PYX_ERR(0, 623, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_read_elements") < 0)) __PYX_ERR(0, 623, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_coords = ((PyArrayObject *)values[0]);
    __pyx_v_recarr = ((PyArrayObject *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_read_elements", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 623, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("tables.tableextension.Table._read_elements", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_coords), __pyx_ptype_5numpy_ndarray, 1, "coords", 0))) __PYX_ERR(0, 623, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_recarr), __pyx_ptype_5numpy_ndarray, 1, "recarr", 0))) __PYX_ERR(0, 623, __pyx_L1_error)
  __pyx_r = __pyx_pf_6tables_14tableextension_5Table_18_read_elements(((struct __pyx_obj_6tables_14tableextension_Table *)__pyx_v_self), __pyx_v_coords, __pyx_v_recarr);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_14tableextension_5Table_18_read_elements(struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self, PyArrayObject *__pyx_v_coords, PyArrayObject *__pyx_v_recarr) {
  long __pyx_v_nrecords;
  void *__pyx_v_rbuf;
  void *__pyx_v_rbuf2;
  int __pyx_v_ret;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  long __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_read_elements", 0);

  /* "tables/tableextension.pyx":630
 * 
 *     # Get the chunk of the coords that correspond to a buffer
 *     nrecords = coords.size             # <<<<<<<<<<<<<<
 *     # Get the pointer to the buffer data area
 *     rbuf = PyArray_DATA(recarr)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_coords), __pyx_n_s_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 630, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_long(__pyx_t_1); if (unlikely((__pyx_t_2 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 630, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_nrecords = __pyx_t_2;

  /* "tables/tableextension.pyx":632
 *     nrecords = coords.size
 *     # Get the pointer to the buffer data area
 *     rbuf = PyArray_DATA(recarr)             # <<<<<<<<<<<<<<
 *     # Get the pointer to the buffer coords area
 *     rbuf2 = PyArray_DATA(coords)
 */
  __pyx_v_rbuf = PyArray_DATA(__pyx_v_recarr);

  /* "tables/tableextension.pyx":634
 *     rbuf = PyArray_DATA(recarr)
 *     # Get the pointer to the buffer coords area
 *     rbuf2 = PyArray_DATA(coords)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_v_rbuf2 = PyArray_DATA(__pyx_v_coords);

  /* "tables/tableextension.pyx":636
 *     rbuf2 = PyArray_DATA(coords)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5TBOread_elements(self.dataset_id, self.type_id,
 *                                  nrecords, rbuf2, rbuf)
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "tables/tableextension.pyx":637
 * 
 *     with nogil:
 *         ret = H5TBOread_elements(self.dataset_id, self.type_id,             # <<<<<<<<<<<<<<
 *                                  nrecords, rbuf2, rbuf)
 * 
 */
        __pyx_v_ret = H5TBOread_elements(__pyx_v_self->__pyx_base.dataset_id, __pyx_v_self->__pyx_base.type_id, __pyx_v_nrecords, __pyx_v_rbuf2, __pyx_v_rbuf);
      }

      /* "tables/tableextension.pyx":636
 *     rbuf2 = PyArray_DATA(coords)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5TBOread_elements(self.dataset_id, self.type_id,
 *                                  nrecords, rbuf2, rbuf)
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "tables/tableextension.pyx":640
 *                                  nrecords, rbuf2, rbuf)
 * 
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems reading records.")
 * 
 */
  __pyx_t_3 = ((__pyx_v_ret < 0) != 0);
  if (unlikely(__pyx_t_3)) {

    /* "tables/tableextension.pyx":641
 * 
 *     if ret < 0:
 *       raise HDF5ExtError("Problems reading records.")             # <<<<<<<<<<<<<<
 * 
 *     # Convert some HDF5 types to NumPy after reading.
 */
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_HDF5ExtError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 641, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_1 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_kp_s_Problems_reading_records) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_kp_s_Problems_reading_records);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 641, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 641, __pyx_L1_error)

    /* "tables/tableextension.pyx":640
 *                                  nrecords, rbuf2, rbuf)
 * 
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems reading records.")
 * 
 */
  }

  /* "tables/tableextension.pyx":644
 * 
 *     # Convert some HDF5 types to NumPy after reading.
 *     self._convert_types(recarr, nrecords, 1)             # <<<<<<<<<<<<<<
 * 
 *     return nrecords
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6tables_14tableextension_Table *)__pyx_v_self->__pyx_base.__pyx_vtab)->_convert_types(__pyx_v_self, __pyx_v_recarr, __pyx_v_nrecords, 1, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 644, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":646
 *     self._convert_types(recarr, nrecords, 1)
 * 
 *     return nrecords             # <<<<<<<<<<<<<<
 * 
 *   def _remove_rows(self, hsize_t start, hsize_t stop, long step):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v_nrecords); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 646, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "tables/tableextension.pyx":623
 *     return nrecords
 * 
 *   def _read_elements(self, ndarray coords, ndarray recarr):             # <<<<<<<<<<<<<<
 *     cdef long nrecords
 *     cdef void *rbuf
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("tables.tableextension.Table._read_elements", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":648
 *     return nrecords
 * 
 *   def _remove_rows(self, hsize_t start, hsize_t stop, long step):             # <<<<<<<<<<<<<<
 *     cdef size_t rowsize
 *     cdef hsize_t nrecords=0, nrecords2
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_14tableextension_5Table_21_remove_rows(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_6tables_14tableextension_5Table_21_remove_rows(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  hsize_t __pyx_v_start;
  hsize_t __pyx_v_stop;
  long __pyx_v_step;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_remove_rows (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_start,&__pyx_n_s_stop,&__pyx_n_s_step,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_start)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_stop)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_remove_rows", 1, 3, 3, 1); __PYX_ERR(0, 648, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_step)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_remove_rows", 1, 3, 3, 2); __PYX_ERR(0, 648, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_remove_rows") < 0)) __PYX_ERR(0, 648, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_start = __Pyx_PyInt_As_hsize_t(values[0]); if (unlikely((__pyx_v_start == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 648, __pyx_L3_error)
    __pyx_v_stop = __Pyx_PyInt_As_hsize_t(values[1]); if (unlikely((__pyx_v_stop == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 648, __pyx_L3_error)
    __pyx_v_step = __Pyx_PyInt_As_long(values[2]); if (unlikely((__pyx_v_step == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 648, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_remove_rows", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 648, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("tables.tableextension.Table._remove_rows", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_14tableextension_5Table_20_remove_rows(((struct __pyx_obj_6tables_14tableextension_Table *)__pyx_v_self), __pyx_v_start, __pyx_v_stop, __pyx_v_step);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_14tableextension_5Table_20_remove_rows(struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self, hsize_t __pyx_v_start, hsize_t __pyx_v_stop, long __pyx_v_step) {
  size_t __pyx_v_rowsize;
  hsize_t __pyx_v_nrecords;
  hsize_t __pyx_v_nrecords2;
  hsize_t __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  hsize_t __pyx_t_4;
  hsize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  hid_t __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  Py_ssize_t __pyx_t_13;
  PyObject *(*__pyx_t_14)(PyObject *);
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_remove_rows", 0);

  /* "tables/tableextension.pyx":650
 *   def _remove_rows(self, hsize_t start, hsize_t stop, long step):
 *     cdef size_t rowsize
 *     cdef hsize_t nrecords=0, nrecords2             # <<<<<<<<<<<<<<
 *     cdef hsize_t i
 * 
 */
  __pyx_v_nrecords = 0;

  /* "tables/tableextension.pyx":653
 *     cdef hsize_t i
 * 
 *     if step == 1:             # <<<<<<<<<<<<<<
 *       nrecords = stop - start
 *       rowsize = self.rowsize
 */
  __pyx_t_1 = ((__pyx_v_step == 1) != 0);
  if (__pyx_t_1) {

    /* "tables/tableextension.pyx":654
 * 
 *     if step == 1:
 *       nrecords = stop - start             # <<<<<<<<<<<<<<
 *       rowsize = self.rowsize
 *       # Using self.disk_type_id should be faster (i.e. less conversions)
 */
    __pyx_v_nrecords = (__pyx_v_stop - __pyx_v_start);

    /* "tables/tableextension.pyx":655
 *     if step == 1:
 *       nrecords = stop - start
 *       rowsize = self.rowsize             # <<<<<<<<<<<<<<
 *       # Using self.disk_type_id should be faster (i.e. less conversions)
 *       if (H5TBOdelete_records(self.dataset_id, self.disk_type_id,
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_rowsize); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 655, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyInt_As_size_t(__pyx_t_2); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 655, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_rowsize = __pyx_t_3;

    /* "tables/tableextension.pyx":658
 *       # Using self.disk_type_id should be faster (i.e. less conversions)
 *       if (H5TBOdelete_records(self.dataset_id, self.disk_type_id,
 *                               self.nrows, rowsize, start, nrecords,             # <<<<<<<<<<<<<<
 *                               self.nrowsinbuf) < 0):
 *         raise HDF5ExtError("Problems deleting records.")
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nrows); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 658, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyInt_As_hsize_t(__pyx_t_2); if (unlikely((__pyx_t_4 == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 658, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "tables/tableextension.pyx":659
 *       if (H5TBOdelete_records(self.dataset_id, self.disk_type_id,
 *                               self.nrows, rowsize, start, nrecords,
 *                               self.nrowsinbuf) < 0):             # <<<<<<<<<<<<<<
 *         raise HDF5ExtError("Problems deleting records.")
 * 
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nrowsinbuf); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 659, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = __Pyx_PyInt_As_hsize_t(__pyx_t_2); if (unlikely((__pyx_t_5 == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 659, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "tables/tableextension.pyx":657
 *       rowsize = self.rowsize
 *       # Using self.disk_type_id should be faster (i.e. less conversions)
 *       if (H5TBOdelete_records(self.dataset_id, self.disk_type_id,             # <<<<<<<<<<<<<<
 *                               self.nrows, rowsize, start, nrecords,
 *                               self.nrowsinbuf) < 0):
 */
    __pyx_t_1 = ((H5TBOdelete_records(__pyx_v_self->__pyx_base.dataset_id, __pyx_v_self->__pyx_base.disk_type_id, __pyx_t_4, __pyx_v_rowsize, __pyx_v_start, __pyx_v_nrecords, __pyx_t_5) < 0) != 0);
    if (unlikely(__pyx_t_1)) {

      /* "tables/tableextension.pyx":660
 *                               self.nrows, rowsize, start, nrecords,
 *                               self.nrowsinbuf) < 0):
 *         raise HDF5ExtError("Problems deleting records.")             # <<<<<<<<<<<<<<
 * 
 *       self.nrows = self.nrows - nrecords
 */
      __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_HDF5ExtError); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 660, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_7 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
        }
      }
      __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_7, __pyx_kp_s_Problems_deleting_records) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_kp_s_Problems_deleting_records);
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 660, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 660, __pyx_L1_error)

      /* "tables/tableextension.pyx":657
 *       rowsize = self.rowsize
 *       # Using self.disk_type_id should be faster (i.e. less conversions)
 *       if (H5TBOdelete_records(self.dataset_id, self.disk_type_id,             # <<<<<<<<<<<<<<
 *                               self.nrows, rowsize, start, nrecords,
 *                               self.nrowsinbuf) < 0):
 */
    }

    /* "tables/tableextension.pyx":662
 *         raise HDF5ExtError("Problems deleting records.")
 * 
 *       self.nrows = self.nrows - nrecords             # <<<<<<<<<<<<<<
 *       if self._v_file.params['PYTABLES_SYS_ATTRS']:
 *         # Attach the NROWS attribute
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nrows); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 662, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_6 = __Pyx_PyInt_From_hsize_t(__pyx_v_nrecords); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 662, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = PyNumber_Subtract(__pyx_t_2, __pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 662, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nrows, __pyx_t_7) < 0) __PYX_ERR(0, 662, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "tables/tableextension.pyx":663
 * 
 *       self.nrows = self.nrows - nrecords
 *       if self._v_file.params['PYTABLES_SYS_ATTRS']:             # <<<<<<<<<<<<<<
 *         # Attach the NROWS attribute
 *         nrecords2 = self.nrows
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_v_file); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 663, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_params); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 663, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = __Pyx_PyObject_Dict_GetItem(__pyx_t_6, __pyx_n_s_PYTABLES_SYS_ATTRS); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 663, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 663, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (__pyx_t_1) {

      /* "tables/tableextension.pyx":665
 *       if self._v_file.params['PYTABLES_SYS_ATTRS']:
 *         # Attach the NROWS attribute
 *         nrecords2 = self.nrows             # <<<<<<<<<<<<<<
 *         H5ATTRset_attribute(self.dataset_id, "NROWS", H5T_STD_I64,
 *                             0, NULL, <char *>&nrecords2)
 */
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nrows); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 665, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_5 = __Pyx_PyInt_As_hsize_t(__pyx_t_7); if (unlikely((__pyx_t_5 == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 665, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_v_nrecords2 = __pyx_t_5;

      /* "tables/tableextension.pyx":666
 *         # Attach the NROWS attribute
 *         nrecords2 = self.nrows
 *         H5ATTRset_attribute(self.dataset_id, "NROWS", H5T_STD_I64,             # <<<<<<<<<<<<<<
 *                             0, NULL, <char *>&nrecords2)
 *       # Set the caches to dirty
 */
      __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_H5T_STD_I64); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 666, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = __Pyx_PyInt_As_hid_t(__pyx_t_7); if (unlikely((__pyx_t_8 == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 666, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "tables/tableextension.pyx":667
 *         nrecords2 = self.nrows
 *         H5ATTRset_attribute(self.dataset_id, "NROWS", H5T_STD_I64,
 *                             0, NULL, <char *>&nrecords2)             # <<<<<<<<<<<<<<
 *       # Set the caches to dirty
 *       self._dirtycache = True
 */
      (void)(H5ATTRset_attribute(__pyx_v_self->__pyx_base.dataset_id, ((char *)"NROWS"), __pyx_t_8, 0, NULL, ((char *)(&__pyx_v_nrecords2))));

      /* "tables/tableextension.pyx":663
 * 
 *       self.nrows = self.nrows - nrecords
 *       if self._v_file.params['PYTABLES_SYS_ATTRS']:             # <<<<<<<<<<<<<<
 *         # Attach the NROWS attribute
 *         nrecords2 = self.nrows
 */
    }

    /* "tables/tableextension.pyx":669
 *                             0, NULL, <char *>&nrecords2)
 *       # Set the caches to dirty
 *       self._dirtycache = True             # <<<<<<<<<<<<<<
 *     elif step == -1:
 *       nrecords = self._remove_rows(stop+1, start+1, 1)
 */
    if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_dirtycache, Py_True) < 0) __PYX_ERR(0, 669, __pyx_L1_error)

    /* "tables/tableextension.pyx":653
 *     cdef hsize_t i
 * 
 *     if step == 1:             # <<<<<<<<<<<<<<
 *       nrecords = stop - start
 *       rowsize = self.rowsize
 */
    goto __pyx_L3;
  }

  /* "tables/tableextension.pyx":670
 *       # Set the caches to dirty
 *       self._dirtycache = True
 *     elif step == -1:             # <<<<<<<<<<<<<<
 *       nrecords = self._remove_rows(stop+1, start+1, 1)
 *     elif step >= 1:
 */
  __pyx_t_1 = ((__pyx_v_step == -1L) != 0);
  if (__pyx_t_1) {

    /* "tables/tableextension.pyx":671
 *       self._dirtycache = True
 *     elif step == -1:
 *       nrecords = self._remove_rows(stop+1, start+1, 1)             # <<<<<<<<<<<<<<
 *     elif step >= 1:
 *       # always want to go through the space backwards
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_remove_rows); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 671, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_2 = __Pyx_PyInt_From_hsize_t((__pyx_v_stop + 1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 671, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyInt_From_hsize_t((__pyx_v_start + 1)); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 671, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10 = NULL;
    __pyx_t_11 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
        __pyx_t_11 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_6)) {
      PyObject *__pyx_temp[4] = {__pyx_t_10, __pyx_t_2, __pyx_t_9, __pyx_int_1};
      __pyx_t_7 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_11, 3+__pyx_t_11); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 671, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
      PyObject *__pyx_temp[4] = {__pyx_t_10, __pyx_t_2, __pyx_t_9, __pyx_int_1};
      __pyx_t_7 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_11, 3+__pyx_t_11); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 671, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    } else
    #endif
    {
      __pyx_t_12 = PyTuple_New(3+__pyx_t_11); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 671, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      if (__pyx_t_10) {
        __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_10); __pyx_t_10 = NULL;
      }
      __Pyx_GIVEREF(__pyx_t_2);
      PyTuple_SET_ITEM(__pyx_t_12, 0+__pyx_t_11, __pyx_t_2);
      __Pyx_GIVEREF(__pyx_t_9);
      PyTuple_SET_ITEM(__pyx_t_12, 1+__pyx_t_11, __pyx_t_9);
      __Pyx_INCREF(__pyx_int_1);
      __Pyx_GIVEREF(__pyx_int_1);
      PyTuple_SET_ITEM(__pyx_t_12, 2+__pyx_t_11, __pyx_int_1);
      __pyx_t_2 = 0;
      __pyx_t_9 = 0;
      __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_12, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 671, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    }
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_5 = __Pyx_PyInt_As_hsize_t(__pyx_t_7); if (unlikely((__pyx_t_5 == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 671, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_v_nrecords = __pyx_t_5;

    /* "tables/tableextension.pyx":670
 *       # Set the caches to dirty
 *       self._dirtycache = True
 *     elif step == -1:             # <<<<<<<<<<<<<<
 *       nrecords = self._remove_rows(stop+1, start+1, 1)
 *     elif step >= 1:
 */
    goto __pyx_L3;
  }

  /* "tables/tableextension.pyx":672
 *     elif step == -1:
 *       nrecords = self._remove_rows(stop+1, start+1, 1)
 *     elif step >= 1:             # <<<<<<<<<<<<<<
 *       # always want to go through the space backwards
 *       for i in range(stop - step, <ssize_t>start - step, -step):
 */
  __pyx_t_1 = ((__pyx_v_step >= 1) != 0);
  if (__pyx_t_1) {

    /* "tables/tableextension.pyx":674
 *     elif step >= 1:
 *       # always want to go through the space backwards
 *       for i in range(stop - step, <ssize_t>start - step, -step):             # <<<<<<<<<<<<<<
 *         nrecords += self._remove_rows(i, i+1, 1)
 *     elif step <= -1:
 */
    __pyx_t_7 = __Pyx_PyInt_From_hsize_t((__pyx_v_stop - __pyx_v_step)); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 674, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = PyInt_FromSsize_t((((Py_ssize_t)__pyx_v_start) - __pyx_v_step)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 674, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_12 = __Pyx_PyInt_From_long((-__pyx_v_step)); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 674, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __pyx_t_9 = PyTuple_New(3); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 674, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_12);
    PyTuple_SET_ITEM(__pyx_t_9, 2, __pyx_t_12);
    __pyx_t_7 = 0;
    __pyx_t_6 = 0;
    __pyx_t_12 = 0;
    __pyx_t_12 = __Pyx_PyObject_Call(__pyx_builtin_range, __pyx_t_9, NULL); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 674, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (likely(PyList_CheckExact(__pyx_t_12)) || PyTuple_CheckExact(__pyx_t_12)) {
      __pyx_t_9 = __pyx_t_12; __Pyx_INCREF(__pyx_t_9); __pyx_t_13 = 0;
      __pyx_t_14 = NULL;
    } else {
      __pyx_t_13 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_12); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 674, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_14 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 674, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    for (;;) {
      if (likely(!__pyx_t_14)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_13 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_12 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_13); __Pyx_INCREF(__pyx_t_12); __pyx_t_13++; if (unlikely(0 < 0)) __PYX_ERR(0, 674, __pyx_L1_error)
          #else
          __pyx_t_12 = PySequence_ITEM(__pyx_t_9, __pyx_t_13); __pyx_t_13++; if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 674, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          #endif
        } else {
          if (__pyx_t_13 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_12 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_13); __Pyx_INCREF(__pyx_t_12); __pyx_t_13++; if (unlikely(0 < 0)) __PYX_ERR(0, 674, __pyx_L1_error)
          #else
          __pyx_t_12 = PySequence_ITEM(__pyx_t_9, __pyx_t_13); __pyx_t_13++; if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 674, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          #endif
        }
      } else {
        __pyx_t_12 = __pyx_t_14(__pyx_t_9);
        if (unlikely(!__pyx_t_12)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 674, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_12);
      }
      __pyx_t_5 = __Pyx_PyInt_As_hsize_t(__pyx_t_12); if (unlikely((__pyx_t_5 == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 674, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __pyx_v_i = __pyx_t_5;

      /* "tables/tableextension.pyx":675
 *       # always want to go through the space backwards
 *       for i in range(stop - step, <ssize_t>start - step, -step):
 *         nrecords += self._remove_rows(i, i+1, 1)             # <<<<<<<<<<<<<<
 *     elif step <= -1:
 *       # always want to go through the space backwards
 */
      __pyx_t_12 = __Pyx_PyInt_From_hsize_t(__pyx_v_nrecords); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 675, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_remove_rows); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 675, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_2 = __Pyx_PyInt_From_hsize_t(__pyx_v_i); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 675, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_10 = __Pyx_PyInt_From_hsize_t((__pyx_v_i + 1)); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 675, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_15 = NULL;
      __pyx_t_11 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_15)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_15);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
          __pyx_t_11 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_7)) {
        PyObject *__pyx_temp[4] = {__pyx_t_15, __pyx_t_2, __pyx_t_10, __pyx_int_1};
        __pyx_t_6 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_11, 3+__pyx_t_11); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 675, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
        PyObject *__pyx_temp[4] = {__pyx_t_15, __pyx_t_2, __pyx_t_10, __pyx_int_1};
        __pyx_t_6 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_11, 3+__pyx_t_11); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 675, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      } else
      #endif
      {
        __pyx_t_16 = PyTuple_New(3+__pyx_t_11); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 675, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_16);
        if (__pyx_t_15) {
          __Pyx_GIVEREF(__pyx_t_15); PyTuple_SET_ITEM(__pyx_t_16, 0, __pyx_t_15); __pyx_t_15 = NULL;
        }
        __Pyx_GIVEREF(__pyx_t_2);
        PyTuple_SET_ITEM(__pyx_t_16, 0+__pyx_t_11, __pyx_t_2);
        __Pyx_GIVEREF(__pyx_t_10);
        PyTuple_SET_ITEM(__pyx_t_16, 1+__pyx_t_11, __pyx_t_10);
        __Pyx_INCREF(__pyx_int_1);
        __Pyx_GIVEREF(__pyx_int_1);
        PyTuple_SET_ITEM(__pyx_t_16, 2+__pyx_t_11, __pyx_int_1);
        __pyx_t_2 = 0;
        __pyx_t_10 = 0;
        __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_16, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 675, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
      }
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = PyNumber_InPlaceAdd(__pyx_t_12, __pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 675, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_5 = __Pyx_PyInt_As_hsize_t(__pyx_t_7); if (unlikely((__pyx_t_5 == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 675, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_v_nrecords = __pyx_t_5;

      /* "tables/tableextension.pyx":674
 *     elif step >= 1:
 *       # always want to go through the space backwards
 *       for i in range(stop - step, <ssize_t>start - step, -step):             # <<<<<<<<<<<<<<
 *         nrecords += self._remove_rows(i, i+1, 1)
 *     elif step <= -1:
 */
    }
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

    /* "tables/tableextension.pyx":672
 *     elif step == -1:
 *       nrecords = self._remove_rows(stop+1, start+1, 1)
 *     elif step >= 1:             # <<<<<<<<<<<<<<
 *       # always want to go through the space backwards
 *       for i in range(stop - step, <ssize_t>start - step, -step):
 */
    goto __pyx_L3;
  }

  /* "tables/tableextension.pyx":676
 *       for i in range(stop - step, <ssize_t>start - step, -step):
 *         nrecords += self._remove_rows(i, i+1, 1)
 *     elif step <= -1:             # <<<<<<<<<<<<<<
 *       # always want to go through the space backwards
 *       for i in range(start, stop, step):
 */
  __pyx_t_1 = ((__pyx_v_step <= -1L) != 0);
  if (likely(__pyx_t_1)) {

    /* "tables/tableextension.pyx":678
 *     elif step <= -1:
 *       # always want to go through the space backwards
 *       for i in range(start, stop, step):             # <<<<<<<<<<<<<<
 *         nrecords += self._remove_rows(i, i+1, 1)
 *     else:
 */
    __pyx_t_9 = __Pyx_PyInt_From_hsize_t(__pyx_v_start); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 678, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_7 = __Pyx_PyInt_From_hsize_t(__pyx_v_stop); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 678, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = __Pyx_PyInt_From_long(__pyx_v_step); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 678, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_12 = PyTuple_New(3); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 678, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_GIVEREF(__pyx_t_9);
    PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_12, 1, __pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_12, 2, __pyx_t_6);
    __pyx_t_9 = 0;
    __pyx_t_7 = 0;
    __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_range, __pyx_t_12, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 678, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    if (likely(PyList_CheckExact(__pyx_t_6)) || PyTuple_CheckExact(__pyx_t_6)) {
      __pyx_t_12 = __pyx_t_6; __Pyx_INCREF(__pyx_t_12); __pyx_t_13 = 0;
      __pyx_t_14 = NULL;
    } else {
      __pyx_t_13 = -1; __pyx_t_12 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 678, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __pyx_t_14 = Py_TYPE(__pyx_t_12)->tp_iternext; if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 678, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    for (;;) {
      if (likely(!__pyx_t_14)) {
        if (likely(PyList_CheckExact(__pyx_t_12))) {
          if (__pyx_t_13 >= PyList_GET_SIZE(__pyx_t_12)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_6 = PyList_GET_ITEM(__pyx_t_12, __pyx_t_13); __Pyx_INCREF(__pyx_t_6); __pyx_t_13++; if (unlikely(0 < 0)) __PYX_ERR(0, 678, __pyx_L1_error)
          #else
          __pyx_t_6 = PySequence_ITEM(__pyx_t_12, __pyx_t_13); __pyx_t_13++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 678, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          #endif
        } else {
          if (__pyx_t_13 >= PyTuple_GET_SIZE(__pyx_t_12)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_6 = PyTuple_GET_ITEM(__pyx_t_12, __pyx_t_13); __Pyx_INCREF(__pyx_t_6); __pyx_t_13++; if (unlikely(0 < 0)) __PYX_ERR(0, 678, __pyx_L1_error)
          #else
          __pyx_t_6 = PySequence_ITEM(__pyx_t_12, __pyx_t_13); __pyx_t_13++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 678, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          #endif
        }
      } else {
        __pyx_t_6 = __pyx_t_14(__pyx_t_12);
        if (unlikely(!__pyx_t_6)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 678, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_6);
      }
      __pyx_t_5 = __Pyx_PyInt_As_hsize_t(__pyx_t_6); if (unlikely((__pyx_t_5 == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 678, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_v_i = __pyx_t_5;

      /* "tables/tableextension.pyx":679
 *       # always want to go through the space backwards
 *       for i in range(start, stop, step):
 *         nrecords += self._remove_rows(i, i+1, 1)             # <<<<<<<<<<<<<<
 *     else:
 *       raise ValueError("step size may not be 0.")
 */
      __pyx_t_6 = __Pyx_PyInt_From_hsize_t(__pyx_v_nrecords); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 679, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_remove_rows); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 679, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_16 = __Pyx_PyInt_From_hsize_t(__pyx_v_i); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 679, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_16);
      __pyx_t_10 = __Pyx_PyInt_From_hsize_t((__pyx_v_i + 1)); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 679, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_2 = NULL;
      __pyx_t_11 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_9);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_9, function);
          __pyx_t_11 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_9)) {
        PyObject *__pyx_temp[4] = {__pyx_t_2, __pyx_t_16, __pyx_t_10, __pyx_int_1};
        __pyx_t_7 = __Pyx_PyFunction_FastCall(__pyx_t_9, __pyx_temp+1-__pyx_t_11, 3+__pyx_t_11); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 679, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_9)) {
        PyObject *__pyx_temp[4] = {__pyx_t_2, __pyx_t_16, __pyx_t_10, __pyx_int_1};
        __pyx_t_7 = __Pyx_PyCFunction_FastCall(__pyx_t_9, __pyx_temp+1-__pyx_t_11, 3+__pyx_t_11); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 679, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      } else
      #endif
      {
        __pyx_t_15 = PyTuple_New(3+__pyx_t_11); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 679, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        if (__pyx_t_2) {
          __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_15, 0, __pyx_t_2); __pyx_t_2 = NULL;
        }
        __Pyx_GIVEREF(__pyx_t_16);
        PyTuple_SET_ITEM(__pyx_t_15, 0+__pyx_t_11, __pyx_t_16);
        __Pyx_GIVEREF(__pyx_t_10);
        PyTuple_SET_ITEM(__pyx_t_15, 1+__pyx_t_11, __pyx_t_10);
        __Pyx_INCREF(__pyx_int_1);
        __Pyx_GIVEREF(__pyx_int_1);
        PyTuple_SET_ITEM(__pyx_t_15, 2+__pyx_t_11, __pyx_int_1);
        __pyx_t_16 = 0;
        __pyx_t_10 = 0;
        __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_15, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 679, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
      }
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_9 = PyNumber_InPlaceAdd(__pyx_t_6, __pyx_t_7); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 679, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_5 = __Pyx_PyInt_As_hsize_t(__pyx_t_9); if (unlikely((__pyx_t_5 == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 679, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_v_nrecords = __pyx_t_5;

      /* "tables/tableextension.pyx":678
 *     elif step <= -1:
 *       # always want to go through the space backwards
 *       for i in range(start, stop, step):             # <<<<<<<<<<<<<<
 *         nrecords += self._remove_rows(i, i+1, 1)
 *     else:
 */
    }
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

    /* "tables/tableextension.pyx":676
 *       for i in range(stop - step, <ssize_t>start - step, -step):
 *         nrecords += self._remove_rows(i, i+1, 1)
 *     elif step <= -1:             # <<<<<<<<<<<<<<
 *       # always want to go through the space backwards
 *       for i in range(start, stop, step):
 */
    goto __pyx_L3;
  }

  /* "tables/tableextension.pyx":681
 *         nrecords += self._remove_rows(i, i+1, 1)
 *     else:
 *       raise ValueError("step size may not be 0.")             # <<<<<<<<<<<<<<
 * 
 *     # Return the number of records removed
 */
  /*else*/ {
    __pyx_t_12 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 681, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_Raise(__pyx_t_12, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __PYX_ERR(0, 681, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "tables/tableextension.pyx":684
 * 
 *     # Return the number of records removed
 *     return nrecords             # <<<<<<<<<<<<<<
 * 
 * cdef class Row:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_12 = __Pyx_PyInt_From_hsize_t(__pyx_v_nrecords); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 684, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_r = __pyx_t_12;
  __pyx_t_12 = 0;
  goto __pyx_L0;

  /* "tables/tableextension.pyx":648
 *     return nrecords
 * 
 *   def _remove_rows(self, hsize_t start, hsize_t stop, long step):             # <<<<<<<<<<<<<<
 *     cdef size_t rowsize
 *     cdef hsize_t nrecords=0, nrecords2
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_AddTraceback("tables.tableextension.Table._remove_rows", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.dims,self.wbuf cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_14tableextension_5Table_23__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_6tables_14tableextension_5Table_23__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6tables_14tableextension_5Table_22__reduce_cython__(((struct __pyx_obj_6tables_14tableextension_Table *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_14tableextension_5Table_22__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.dims,self.wbuf cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.dims,self.wbuf cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.dims,self.wbuf cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("tables.tableextension.Table.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.dims,self.wbuf cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.dims,self.wbuf cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_14tableextension_5Table_25__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_6tables_14tableextension_5Table_25__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6tables_14tableextension_5Table_24__setstate_cython__(((struct __pyx_obj_6tables_14tableextension_Table *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_14tableextension_5Table_24__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.dims,self.wbuf cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.dims,self.wbuf cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.dims,self.wbuf cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.dims,self.wbuf cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("tables.tableextension.Table.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":753
 *     """
 * 
 *     def __get__(self):             # <<<<<<<<<<<<<<
 *       return SizeType(self._nrow)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_14tableextension_3Row_4nrow_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6tables_14tableextension_3Row_4nrow_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6tables_14tableextension_3Row_4nrow___get__(((struct __pyx_obj_6tables_14tableextension_Row *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_14tableextension_3Row_4nrow___get__(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "tables/tableextension.pyx":754
 * 
 *     def __get__(self):
 *       return SizeType(self._nrow)             # <<<<<<<<<<<<<<
 * 
 *   property table:
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_SizeType); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 754, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_PY_LONG_LONG(__pyx_v_self->_nrow); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 754, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 754, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "tables/tableextension.pyx":753
 *     """
 * 
 *     def __get__(self):             # <<<<<<<<<<<<<<
 *       return SizeType(self._nrow)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("tables.tableextension.Row.nrow.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":757
 * 
 *   property table:
 *     def __get__(self):             # <<<<<<<<<<<<<<
 *         self._table_file._check_open()
 *         return self._table_file._get_node(self._table_path)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_14tableextension_3Row_5table_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6tables_14tableextension_3Row_5table_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6tables_14tableextension_3Row_5table___get__(((struct __pyx_obj_6tables_14tableextension_Row *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_14tableextension_3Row_5table___get__(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "tables/tableextension.pyx":758
 *   property table:
 *     def __get__(self):
 *         self._table_file._check_open()             # <<<<<<<<<<<<<<
 *         return self._table_file._get_node(self._table_path)
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->_table_file, __pyx_n_s_check_open); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 758, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 758, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":759
 *     def __get__(self):
 *         self._table_file._check_open()
 *         return self._table_file._get_node(self._table_path)             # <<<<<<<<<<<<<<
 * 
 *   def __cinit__(self, table):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->_table_file, __pyx_n_s_get_node); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 759, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_self->_table_path) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_self->_table_path);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 759, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "tables/tableextension.pyx":757
 * 
 *   property table:
 *     def __get__(self):             # <<<<<<<<<<<<<<
 *         self._table_file._check_open()
 *         return self._table_file._get_node(self._table_path)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("tables.tableextension.Row.table.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":761
 *         return self._table_file._get_node(self._table_path)
 * 
 *   def __cinit__(self, table):             # <<<<<<<<<<<<<<
 *     cdef int nfields, i
 *     # Location-dependent information.
 */

/* Python wrapper */
static int __pyx_pw_6tables_14tableextension_3Row_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6tables_14tableextension_3Row_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_table = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_table,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_table)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(0, 761, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_table = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 761, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("tables.tableextension.Row.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_14tableextension_3Row___cinit__(((struct __pyx_obj_6tables_14tableextension_Row *)__pyx_v_self), __pyx_v_table);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6tables_14tableextension_3Row___cinit__(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self, PyObject *__pyx_v_table) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  hsize_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "tables/tableextension.pyx":764
 *     cdef int nfields, i
 *     # Location-dependent information.
 *     self._table_file = table._v_file             # <<<<<<<<<<<<<<
 *     self._table_path = table._v_pathname
 *     self._unsaved_nrows = 0
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_table, __pyx_n_s_v_file); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 764, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_table_file);
  __Pyx_DECREF(__pyx_v_self->_table_file);
  __pyx_v_self->_table_file = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":765
 *     # Location-dependent information.
 *     self._table_file = table._v_file
 *     self._table_path = table._v_pathname             # <<<<<<<<<<<<<<
 *     self._unsaved_nrows = 0
 *     self._mod_nrows = 0
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_table, __pyx_n_s_v_pathname); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 765, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_table_path);
  __Pyx_DECREF(__pyx_v_self->_table_path);
  __pyx_v_self->_table_path = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":766
 *     self._table_file = table._v_file
 *     self._table_path = table._v_pathname
 *     self._unsaved_nrows = 0             # <<<<<<<<<<<<<<
 *     self._mod_nrows = 0
 *     self._row = 0
 */
  __pyx_v_self->_unsaved_nrows = 0;

  /* "tables/tableextension.pyx":767
 *     self._table_path = table._v_pathname
 *     self._unsaved_nrows = 0
 *     self._mod_nrows = 0             # <<<<<<<<<<<<<<
 *     self._row = 0
 *     self._nrow = 0   # Useful in mod_append read iterators
 */
  __pyx_v_self->_mod_nrows = 0;

  /* "tables/tableextension.pyx":768
 *     self._unsaved_nrows = 0
 *     self._mod_nrows = 0
 *     self._row = 0             # <<<<<<<<<<<<<<
 *     self._nrow = 0   # Useful in mod_append read iterators
 *     self._riterator = 0
 */
  __pyx_v_self->_row = 0;

  /* "tables/tableextension.pyx":769
 *     self._mod_nrows = 0
 *     self._row = 0
 *     self._nrow = 0   # Useful in mod_append read iterators             # <<<<<<<<<<<<<<
 *     self._riterator = 0
 *     self._bufferinfo_done = 0
 */
  __pyx_v_self->_nrow = 0;

  /* "tables/tableextension.pyx":770
 *     self._row = 0
 *     self._nrow = 0   # Useful in mod_append read iterators
 *     self._riterator = 0             # <<<<<<<<<<<<<<
 *     self._bufferinfo_done = 0
 *     # Some variables from table will be cached here
 */
  __pyx_v_self->_riterator = 0;

  /* "tables/tableextension.pyx":771
 *     self._nrow = 0   # Useful in mod_append read iterators
 *     self._riterator = 0
 *     self._bufferinfo_done = 0             # <<<<<<<<<<<<<<
 *     # Some variables from table will be cached here
 *     if table._v_file.mode == 'r':
 */
  __pyx_v_self->_bufferinfo_done = 0;

  /* "tables/tableextension.pyx":773
 *     self._bufferinfo_done = 0
 *     # Some variables from table will be cached here
 *     if table._v_file.mode == 'r':             # <<<<<<<<<<<<<<
 *       self.ro_filemode = 1
 *     else:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_table, __pyx_n_s_v_file); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 773, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_mode); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 773, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (__Pyx_PyString_Equals(__pyx_t_2, __pyx_n_s_r, Py_EQ)); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 773, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {

    /* "tables/tableextension.pyx":774
 *     # Some variables from table will be cached here
 *     if table._v_file.mode == 'r':
 *       self.ro_filemode = 1             # <<<<<<<<<<<<<<
 *     else:
 *       self.ro_filemode = 0
 */
    __pyx_v_self->ro_filemode = 1;

    /* "tables/tableextension.pyx":773
 *     self._bufferinfo_done = 0
 *     # Some variables from table will be cached here
 *     if table._v_file.mode == 'r':             # <<<<<<<<<<<<<<
 *       self.ro_filemode = 1
 *     else:
 */
    goto __pyx_L3;
  }

  /* "tables/tableextension.pyx":776
 *       self.ro_filemode = 1
 *     else:
 *       self.ro_filemode = 0             # <<<<<<<<<<<<<<
 *     self.chunked = table._chunked
 *     self.colenums = table._colenums
 */
  /*else*/ {
    __pyx_v_self->ro_filemode = 0;
  }
  __pyx_L3:;

  /* "tables/tableextension.pyx":777
 *     else:
 *       self.ro_filemode = 0
 *     self.chunked = table._chunked             # <<<<<<<<<<<<<<
 *     self.colenums = table._colenums
 *     self.exist_enum_cols = len(self.colenums)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_table, __pyx_n_s_chunked); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 777, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 777, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_self->chunked = __pyx_t_4;

  /* "tables/tableextension.pyx":778
 *       self.ro_filemode = 0
 *     self.chunked = table._chunked
 *     self.colenums = table._colenums             # <<<<<<<<<<<<<<
 *     self.exist_enum_cols = len(self.colenums)
 *     self.nrowsinbuf = table.nrowsinbuf
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_table, __pyx_n_s_colenums); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 778, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->colenums);
  __Pyx_DECREF(__pyx_v_self->colenums);
  __pyx_v_self->colenums = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "tables/tableextension.pyx":779
 *     self.chunked = table._chunked
 *     self.colenums = table._colenums
 *     self.exist_enum_cols = len(self.colenums)             # <<<<<<<<<<<<<<
 *     self.nrowsinbuf = table.nrowsinbuf
 *     self.chunksize = table.chunkshape[0]
 */
  __pyx_t_2 = __pyx_v_self->colenums;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_5 = PyObject_Length(__pyx_t_2); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 779, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_self->exist_enum_cols = __pyx_t_5;

  /* "tables/tableextension.pyx":780
 *     self.colenums = table._colenums
 *     self.exist_enum_cols = len(self.colenums)
 *     self.nrowsinbuf = table.nrowsinbuf             # <<<<<<<<<<<<<<
 *     self.chunksize = table.chunkshape[0]
 *     self.nchunksinbuf = self.nrowsinbuf // self.chunksize
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_table, __pyx_n_s_nrowsinbuf); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 780, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __Pyx_PyInt_As_hsize_t(__pyx_t_2); if (unlikely((__pyx_t_6 == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 780, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_self->nrowsinbuf = __pyx_t_6;

  /* "tables/tableextension.pyx":781
 *     self.exist_enum_cols = len(self.colenums)
 *     self.nrowsinbuf = table.nrowsinbuf
 *     self.chunksize = table.chunkshape[0]             # <<<<<<<<<<<<<<
 *     self.nchunksinbuf = self.nrowsinbuf // self.chunksize
 *     self.dtype = table._v_dtype
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_table, __pyx_n_s_chunkshape); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 781, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_t_2, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 781, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_6 = __Pyx_PyInt_As_hsize_t(__pyx_t_1); if (unlikely((__pyx_t_6 == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 781, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->chunksize = __pyx_t_6;

  /* "tables/tableextension.pyx":782
 *     self.nrowsinbuf = table.nrowsinbuf
 *     self.chunksize = table.chunkshape[0]
 *     self.nchunksinbuf = self.nrowsinbuf // self.chunksize             # <<<<<<<<<<<<<<
 *     self.dtype = table._v_dtype
 *     self._new_buffer(table)
 */
  if (unlikely(__pyx_v_self->chunksize == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
    __PYX_ERR(0, 782, __pyx_L1_error)
  }
  __pyx_v_self->nchunksinbuf = (__pyx_v_self->nrowsinbuf / __pyx_v_self->chunksize);

  /* "tables/tableextension.pyx":783
 *     self.chunksize = table.chunkshape[0]
 *     self.nchunksinbuf = self.nrowsinbuf // self.chunksize
 *     self.dtype = table._v_dtype             # <<<<<<<<<<<<<<
 *     self._new_buffer(table)
 *     self.mod_elements = None
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_table, __pyx_n_s_v_dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 783, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->dtype);
  __Pyx_DECREF(__pyx_v_self->dtype);
  __pyx_v_self->dtype = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":784
 *     self.nchunksinbuf = self.nrowsinbuf // self.chunksize
 *     self.dtype = table._v_dtype
 *     self._new_buffer(table)             # <<<<<<<<<<<<<<
 *     self.mod_elements = None
 *     self.rfieldscache = {}
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6tables_14tableextension_Row *)__pyx_v_self->__pyx_vtab)->_new_buffer(__pyx_v_self, __pyx_v_table); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 784, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":785
 *     self.dtype = table._v_dtype
 *     self._new_buffer(table)
 *     self.mod_elements = None             # <<<<<<<<<<<<<<
 *     self.rfieldscache = {}
 *     self.wfieldscache = {}
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->mod_elements);
  __Pyx_DECREF(__pyx_v_self->mod_elements);
  __pyx_v_self->mod_elements = Py_None;

  /* "tables/tableextension.pyx":786
 *     self._new_buffer(table)
 *     self.mod_elements = None
 *     self.rfieldscache = {}             # <<<<<<<<<<<<<<
 *     self.wfieldscache = {}
 *     self.modified_fields = set()
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 786, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->rfieldscache);
  __Pyx_DECREF(__pyx_v_self->rfieldscache);
  __pyx_v_self->rfieldscache = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":787
 *     self.mod_elements = None
 *     self.rfieldscache = {}
 *     self.wfieldscache = {}             # <<<<<<<<<<<<<<
 *     self.modified_fields = set()
 * 
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 787, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->wfieldscache);
  __Pyx_DECREF(__pyx_v_self->wfieldscache);
  __pyx_v_self->wfieldscache = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":788
 *     self.rfieldscache = {}
 *     self.wfieldscache = {}
 *     self.modified_fields = set()             # <<<<<<<<<<<<<<
 * 
 *   def _iter(self, start=0, stop=0, step=1, coords=None, chunkmap=None):
 */
  __pyx_t_1 = PySet_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 788, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->modified_fields);
  __Pyx_DECREF(__pyx_v_self->modified_fields);
  __pyx_v_self->modified_fields = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":761
 *         return self._table_file._get_node(self._table_path)
 * 
 *   def __cinit__(self, table):             # <<<<<<<<<<<<<<
 *     cdef int nfields, i
 *     # Location-dependent information.
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("tables.tableextension.Row.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":790
 *     self.modified_fields = set()
 * 
 *   def _iter(self, start=0, stop=0, step=1, coords=None, chunkmap=None):             # <<<<<<<<<<<<<<
 *     """Return an iterator for traversiong the data in table."""
 *     self._init_loop(start, stop, step, coords, chunkmap)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_14tableextension_3Row_3_iter(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6tables_14tableextension_3Row_2_iter[] = "Return an iterator for traversiong the data in table.";
static PyObject *__pyx_pw_6tables_14tableextension_3Row_3_iter(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_start = 0;
  PyObject *__pyx_v_stop = 0;
  PyObject *__pyx_v_step = 0;
  PyObject *__pyx_v_coords = 0;
  PyObject *__pyx_v_chunkmap = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_iter (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_start,&__pyx_n_s_stop,&__pyx_n_s_step,&__pyx_n_s_coords,&__pyx_n_s_chunkmap,0};
    PyObject* values[5] = {0,0,0,0,0};
    values[0] = ((PyObject *)__pyx_int_0);
    values[1] = ((PyObject *)__pyx_int_0);
    values[2] = ((PyObject *)__pyx_int_1);
    values[3] = ((PyObject *)Py_None);
    values[4] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_start);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_stop);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_step);
          if (value) { values[2] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_coords);
          if (value) { values[3] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_chunkmap);
          if (value) { values[4] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_iter") < 0)) __PYX_ERR(0, 790, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_start = values[0];
    __pyx_v_stop = values[1];
    __pyx_v_step = values[2];
    __pyx_v_coords = values[3];
    __pyx_v_chunkmap = values[4];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_iter", 0, 0, 5, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 790, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("tables.tableextension.Row._iter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_14tableextension_3Row_2_iter(((struct __pyx_obj_6tables_14tableextension_Row *)__pyx_v_self), __pyx_v_start, __pyx_v_stop, __pyx_v_step, __pyx_v_coords, __pyx_v_chunkmap);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_14tableextension_3Row_2_iter(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self, PyObject *__pyx_v_start, PyObject *__pyx_v_stop, PyObject *__pyx_v_step, PyObject *__pyx_v_coords, PyObject *__pyx_v_chunkmap) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  hsize_t __pyx_t_1;
  PY_LONG_LONG __pyx_t_2;
  PY_LONG_LONG __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_iter", 0);

  /* "tables/tableextension.pyx":792
 *   def _iter(self, start=0, stop=0, step=1, coords=None, chunkmap=None):
 *     """Return an iterator for traversiong the data in table."""
 *     self._init_loop(start, stop, step, coords, chunkmap)             # <<<<<<<<<<<<<<
 *     return iter(self)
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_As_hsize_t(__pyx_v_start); if (unlikely((__pyx_t_1 == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 792, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_PY_LONG_LONG(__pyx_v_stop); if (unlikely((__pyx_t_2 == (PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(0, 792, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_PY_LONG_LONG(__pyx_v_step); if (unlikely((__pyx_t_3 == (PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(0, 792, __pyx_L1_error)
  __pyx_t_4 = ((struct __pyx_vtabstruct_6tables_14tableextension_Row *)__pyx_v_self->__pyx_vtab)->_init_loop(__pyx_v_self, __pyx_t_1, __pyx_t_2, __pyx_t_3, __pyx_v_coords, __pyx_v_chunkmap); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 792, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "tables/tableextension.pyx":793
 *     """Return an iterator for traversiong the data in table."""
 *     self._init_loop(start, stop, step, coords, chunkmap)
 *     return iter(self)             # <<<<<<<<<<<<<<
 * 
 *   def __iter__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = PyObject_GetIter(((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 793, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "tables/tableextension.pyx":790
 *     self.modified_fields = set()
 * 
 *   def _iter(self, start=0, stop=0, step=1, coords=None, chunkmap=None):             # <<<<<<<<<<<<<<
 *     """Return an iterator for traversiong the data in table."""
 *     self._init_loop(start, stop, step, coords, chunkmap)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("tables.tableextension.Row._iter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":795
 *     return iter(self)
 * 
 *   def __iter__(self):             # <<<<<<<<<<<<<<
 *     """Iterator that traverses all the data in the Table"""
 *     return self
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_14tableextension_3Row_5__iter__(PyObject *__pyx_v_self); /*proto*/
static char __pyx_doc_6tables_14tableextension_3Row_4__iter__[] = "Iterator that traverses all the data in the Table";
#if CYTHON_COMPILING_IN_CPYTHON
struct wrapperbase __pyx_wrapperbase_6tables_14tableextension_3Row_4__iter__;
#endif
static PyObject *__pyx_pw_6tables_14tableextension_3Row_5__iter__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6tables_14tableextension_3Row_4__iter__(((struct __pyx_obj_6tables_14tableextension_Row *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_14tableextension_3Row_4__iter__(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__", 0);

  /* "tables/tableextension.pyx":797
 *   def __iter__(self):
 *     """Iterator that traverses all the data in the Table"""
 *     return self             # <<<<<<<<<<<<<<
 * 
 *   cdef _new_buffer(self, table):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "tables/tableextension.pyx":795
 *     return iter(self)
 * 
 *   def __iter__(self):             # <<<<<<<<<<<<<<
 *     """Iterator that traverses all the data in the Table"""
 *     return self
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":799
 *     return self
 * 
 *   cdef _new_buffer(self, table):             # <<<<<<<<<<<<<<
 *     """Create the recarrays for I/O buffering"""
 * 
 */

static PyObject *__pyx_f_6tables_14tableextension_3Row__new_buffer(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self, PyObject *__pyx_v_table) {
  PyObject *__pyx_v_wdflts = NULL;
  PyObject *__pyx_v_name = NULL;
  PyObject *__pyx_v_buff = NULL;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  PyObject *(*__pyx_t_7)(PyObject *);
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  hsize_t __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_new_buffer", 0);

  /* "tables/tableextension.pyx":802
 *     """Create the recarrays for I/O buffering"""
 * 
 *     wdflts = table._v_wdflts             # <<<<<<<<<<<<<<
 *     if wdflts is None:
 *       self.wrec = numpy.zeros(1, dtype=self.dtype)  # Defaults are zero
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_table, __pyx_n_s_v_wdflts); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 802, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_wdflts = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":803
 * 
 *     wdflts = table._v_wdflts
 *     if wdflts is None:             # <<<<<<<<<<<<<<
 *       self.wrec = numpy.zeros(1, dtype=self.dtype)  # Defaults are zero
 *     else:
 */
  __pyx_t_2 = (__pyx_v_wdflts == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "tables/tableextension.pyx":804
 *     wdflts = table._v_wdflts
 *     if wdflts is None:
 *       self.wrec = numpy.zeros(1, dtype=self.dtype)  # Defaults are zero             # <<<<<<<<<<<<<<
 *     else:
 *       self.wrec = table._v_wdflts.copy()
 */
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_numpy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 804, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 804, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 804, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_v_self->dtype) < 0) __PYX_ERR(0, 804, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__6, __pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 804, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GIVEREF(__pyx_t_5);
    __Pyx_GOTREF(__pyx_v_self->wrec);
    __Pyx_DECREF(__pyx_v_self->wrec);
    __pyx_v_self->wrec = __pyx_t_5;
    __pyx_t_5 = 0;

    /* "tables/tableextension.pyx":803
 * 
 *     wdflts = table._v_wdflts
 *     if wdflts is None:             # <<<<<<<<<<<<<<
 *       self.wrec = numpy.zeros(1, dtype=self.dtype)  # Defaults are zero
 *     else:
 */
    goto __pyx_L3;
  }

  /* "tables/tableextension.pyx":806
 *       self.wrec = numpy.zeros(1, dtype=self.dtype)  # Defaults are zero
 *     else:
 *       self.wrec = table._v_wdflts.copy()             # <<<<<<<<<<<<<<
 *     self.wreccpy = self.wrec.copy()  # A copy of the defaults
 *     # Build the wfields dictionary for faster access to columns
 */
  /*else*/ {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_table, __pyx_n_s_v_wdflts); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 806, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_copy); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 806, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_5 = (__pyx_t_1) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_1) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 806, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GIVEREF(__pyx_t_5);
    __Pyx_GOTREF(__pyx_v_self->wrec);
    __Pyx_DECREF(__pyx_v_self->wrec);
    __pyx_v_self->wrec = __pyx_t_5;
    __pyx_t_5 = 0;
  }
  __pyx_L3:;

  /* "tables/tableextension.pyx":807
 *     else:
 *       self.wrec = table._v_wdflts.copy()
 *     self.wreccpy = self.wrec.copy()  # A copy of the defaults             # <<<<<<<<<<<<<<
 *     # Build the wfields dictionary for faster access to columns
 *     self.wfields = {}
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->wrec, __pyx_n_s_copy); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 807, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_5 = (__pyx_t_1) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_1) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 807, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_GIVEREF(__pyx_t_5);
  __Pyx_GOTREF(__pyx_v_self->wreccpy);
  __Pyx_DECREF(__pyx_v_self->wreccpy);
  __pyx_v_self->wreccpy = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "tables/tableextension.pyx":809
 *     self.wreccpy = self.wrec.copy()  # A copy of the defaults
 *     # Build the wfields dictionary for faster access to columns
 *     self.wfields = {}             # <<<<<<<<<<<<<<
 *     for name in self.dtype.names:
 *       self.wfields[name] = self.wrec[name]
 */
  __pyx_t_5 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 809, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_5);
  __Pyx_GOTREF(__pyx_v_self->wfields);
  __Pyx_DECREF(__pyx_v_self->wfields);
  __pyx_v_self->wfields = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "tables/tableextension.pyx":810
 *     # Build the wfields dictionary for faster access to columns
 *     self.wfields = {}
 *     for name in self.dtype.names:             # <<<<<<<<<<<<<<
 *       self.wfields[name] = self.wrec[name]
 * 
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->dtype, __pyx_n_s_names); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 810, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (likely(PyList_CheckExact(__pyx_t_5)) || PyTuple_CheckExact(__pyx_t_5)) {
    __pyx_t_4 = __pyx_t_5; __Pyx_INCREF(__pyx_t_4); __pyx_t_6 = 0;
    __pyx_t_7 = NULL;
  } else {
    __pyx_t_6 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 810, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_7 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 810, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  for (;;) {
    if (likely(!__pyx_t_7)) {
      if (likely(PyList_CheckExact(__pyx_t_4))) {
        if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_5 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_6); __Pyx_INCREF(__pyx_t_5); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 810, __pyx_L1_error)
        #else
        __pyx_t_5 = PySequence_ITEM(__pyx_t_4, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 810, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        #endif
      } else {
        if (__pyx_t_6 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_6); __Pyx_INCREF(__pyx_t_5); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 810, __pyx_L1_error)
        #else
        __pyx_t_5 = PySequence_ITEM(__pyx_t_4, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 810, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        #endif
      }
    } else {
      __pyx_t_5 = __pyx_t_7(__pyx_t_4);
      if (unlikely(!__pyx_t_5)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 810, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_5);
    }
    __Pyx_XDECREF_SET(__pyx_v_name, __pyx_t_5);
    __pyx_t_5 = 0;

    /* "tables/tableextension.pyx":811
 *     self.wfields = {}
 *     for name in self.dtype.names:
 *       self.wfields[name] = self.wrec[name]             # <<<<<<<<<<<<<<
 * 
 *     # Get the read buffer for this instance (it is private, remember!)
 */
    __pyx_t_5 = __Pyx_PyObject_GetItem(__pyx_v_self->wrec, __pyx_v_name); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 811, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (unlikely(PyObject_SetItem(__pyx_v_self->wfields, __pyx_v_name, __pyx_t_5) < 0)) __PYX_ERR(0, 811, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "tables/tableextension.pyx":810
 *     # Build the wfields dictionary for faster access to columns
 *     self.wfields = {}
 *     for name in self.dtype.names:             # <<<<<<<<<<<<<<
 *       self.wfields[name] = self.wrec[name]
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "tables/tableextension.pyx":814
 * 
 *     # Get the read buffer for this instance (it is private, remember!)
 *     buff = self.iobuf = table._get_container(self.nrowsinbuf)             # <<<<<<<<<<<<<<
 *     # Build the rfields dictionary for faster access to columns
 *     # This is quite fast, as it only takes around 5 us per column
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_table, __pyx_n_s_get_container); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 814, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyInt_From_hsize_t(__pyx_v_self->nrowsinbuf); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 814, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_8 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_8)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_4 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_8, __pyx_t_1) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 814, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_INCREF(__pyx_t_4);
  __pyx_v_buff = __pyx_t_4;
  __Pyx_INCREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_v_self->iobuf);
  __Pyx_DECREF(__pyx_v_self->iobuf);
  __pyx_v_self->iobuf = __pyx_t_4;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "tables/tableextension.pyx":819
 *     # in my laptop (Pentium 4 @ 2 GHz).
 *     # F. Alted 2006-08-18
 *     self.rfields = {}             # <<<<<<<<<<<<<<
 *     for i, name in enumerate(self.dtype.names):
 *       self.rfields[i] = buff[name]
 */
  __pyx_t_4 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 819, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_v_self->rfields);
  __Pyx_DECREF(__pyx_v_self->rfields);
  __pyx_v_self->rfields = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "tables/tableextension.pyx":820
 *     # F. Alted 2006-08-18
 *     self.rfields = {}
 *     for i, name in enumerate(self.dtype.names):             # <<<<<<<<<<<<<<
 *       self.rfields[i] = buff[name]
 *       self.rfields[name] = buff[name]
 */
  __Pyx_INCREF(__pyx_int_0);
  __pyx_t_4 = __pyx_int_0;
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->dtype, __pyx_n_s_names); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 820, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (likely(PyList_CheckExact(__pyx_t_5)) || PyTuple_CheckExact(__pyx_t_5)) {
    __pyx_t_1 = __pyx_t_5; __Pyx_INCREF(__pyx_t_1); __pyx_t_6 = 0;
    __pyx_t_7 = NULL;
  } else {
    __pyx_t_6 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 820, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 820, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  for (;;) {
    if (likely(!__pyx_t_7)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_5 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_5); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 820, __pyx_L1_error)
        #else
        __pyx_t_5 = PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 820, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        #endif
      } else {
        if (__pyx_t_6 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_5); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 820, __pyx_L1_error)
        #else
        __pyx_t_5 = PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 820, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        #endif
      }
    } else {
      __pyx_t_5 = __pyx_t_7(__pyx_t_1);
      if (unlikely(!__pyx_t_5)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 820, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_5);
    }
    __Pyx_XDECREF_SET(__pyx_v_name, __pyx_t_5);
    __pyx_t_5 = 0;
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_4);
    __pyx_t_5 = __Pyx_PyInt_AddObjC(__pyx_t_4, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 820, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4);
    __pyx_t_4 = __pyx_t_5;
    __pyx_t_5 = 0;

    /* "tables/tableextension.pyx":821
 *     self.rfields = {}
 *     for i, name in enumerate(self.dtype.names):
 *       self.rfields[i] = buff[name]             # <<<<<<<<<<<<<<
 *       self.rfields[name] = buff[name]
 * 
 */
    __pyx_t_5 = __Pyx_PyObject_GetItem(__pyx_v_buff, __pyx_v_name); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 821, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (unlikely(PyObject_SetItem(__pyx_v_self->rfields, __pyx_v_i, __pyx_t_5) < 0)) __PYX_ERR(0, 821, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "tables/tableextension.pyx":822
 *     for i, name in enumerate(self.dtype.names):
 *       self.rfields[i] = buff[name]
 *       self.rfields[name] = buff[name]             # <<<<<<<<<<<<<<
 * 
 *     # Get the stride of these buffers
 */
    __pyx_t_5 = __Pyx_PyObject_GetItem(__pyx_v_buff, __pyx_v_name); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 822, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (unlikely(PyObject_SetItem(__pyx_v_self->rfields, __pyx_v_name, __pyx_t_5) < 0)) __PYX_ERR(0, 822, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "tables/tableextension.pyx":820
 *     # F. Alted 2006-08-18
 *     self.rfields = {}
 *     for i, name in enumerate(self.dtype.names):             # <<<<<<<<<<<<<<
 *       self.rfields[i] = buff[name]
 *       self.rfields[name] = buff[name]
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "tables/tableextension.pyx":825
 * 
 *     # Get the stride of these buffers
 *     self._stride = PyArray_STRIDE(buff, 0)             # <<<<<<<<<<<<<<
 *     # The rowsize
 *     self._rowsize = self.dtype.itemsize
 */
  if (!(likely(((__pyx_v_buff) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_buff, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 825, __pyx_L1_error)
  __pyx_v_self->_stride = PyArray_STRIDE(((PyArrayObject *)__pyx_v_buff), 0);

  /* "tables/tableextension.pyx":827
 *     self._stride = PyArray_STRIDE(buff, 0)
 *     # The rowsize
 *     self._rowsize = self.dtype.itemsize             # <<<<<<<<<<<<<<
 *     self.nrows = table.nrows  # This value may change
 * 
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 827, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_9 = __Pyx_PyInt_As_int(__pyx_t_4); if (unlikely((__pyx_t_9 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 827, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_self->_rowsize = __pyx_t_9;

  /* "tables/tableextension.pyx":828
 *     # The rowsize
 *     self._rowsize = self.dtype.itemsize
 *     self.nrows = table.nrows  # This value may change             # <<<<<<<<<<<<<<
 * 
 *   cdef _init_loop(self, hsize_t start, long long stop, long long step,
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_table, __pyx_n_s_nrows); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 828, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_10 = __Pyx_PyInt_As_hsize_t(__pyx_t_4); if (unlikely((__pyx_t_10 == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 828, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_self->nrows = __pyx_t_10;

  /* "tables/tableextension.pyx":799
 *     return self
 * 
 *   cdef _new_buffer(self, table):             # <<<<<<<<<<<<<<
 *     """Create the recarrays for I/O buffering"""
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("tables.tableextension.Row._new_buffer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_wdflts);
  __Pyx_XDECREF(__pyx_v_name);
  __Pyx_XDECREF(__pyx_v_buff);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":830
 *     self.nrows = table.nrows  # This value may change
 * 
 *   cdef _init_loop(self, hsize_t start, long long stop, long long step,             # <<<<<<<<<<<<<<
 *                  object coords, object chunkmap):
 *     """Initialization for the __iter__ iterator"""
 */

static PyObject *__pyx_f_6tables_14tableextension_3Row__init_loop(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self, hsize_t __pyx_v_start, PY_LONG_LONG __pyx_v_stop, PY_LONG_LONG __pyx_v_step, PyObject *__pyx_v_coords, PyObject *__pyx_v_chunkmap) {
  PyObject *__pyx_v_table = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  hsize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  PY_LONG_LONG __pyx_t_7;
  PY_LONG_LONG __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *(*__pyx_t_13)(PyObject *);
  int __pyx_t_14;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_init_loop", 0);

  /* "tables/tableextension.pyx":833
 *                  object coords, object chunkmap):
 *     """Initialization for the __iter__ iterator"""
 *     table = self.table             # <<<<<<<<<<<<<<
 *     self._riterator = 1   # We are inside a read iterator
 *     self.start = start
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_table); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 833, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_table = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":834
 *     """Initialization for the __iter__ iterator"""
 *     table = self.table
 *     self._riterator = 1   # We are inside a read iterator             # <<<<<<<<<<<<<<
 *     self.start = start
 *     self.stop = stop
 */
  __pyx_v_self->_riterator = 1;

  /* "tables/tableextension.pyx":835
 *     table = self.table
 *     self._riterator = 1   # We are inside a read iterator
 *     self.start = start             # <<<<<<<<<<<<<<
 *     self.stop = stop
 *     self.step = step
 */
  __pyx_v_self->start = __pyx_v_start;

  /* "tables/tableextension.pyx":836
 *     self._riterator = 1   # We are inside a read iterator
 *     self.start = start
 *     self.stop = stop             # <<<<<<<<<<<<<<
 *     self.step = step
 *     self.coords = coords
 */
  __pyx_v_self->stop = __pyx_v_stop;

  /* "tables/tableextension.pyx":837
 *     self.start = start
 *     self.stop = stop
 *     self.step = step             # <<<<<<<<<<<<<<
 *     self.coords = coords
 *     self.startb = 0
 */
  __pyx_v_self->step = __pyx_v_step;

  /* "tables/tableextension.pyx":838
 *     self.stop = stop
 *     self.step = step
 *     self.coords = coords             # <<<<<<<<<<<<<<
 *     self.startb = 0
 *     if step > 0:
 */
  __Pyx_INCREF(__pyx_v_coords);
  __Pyx_GIVEREF(__pyx_v_coords);
  __Pyx_GOTREF(__pyx_v_self->coords);
  __Pyx_DECREF(__pyx_v_self->coords);
  __pyx_v_self->coords = __pyx_v_coords;

  /* "tables/tableextension.pyx":839
 *     self.step = step
 *     self.coords = coords
 *     self.startb = 0             # <<<<<<<<<<<<<<
 *     if step > 0:
 *         self._row = -1  # a sentinel
 */
  __pyx_v_self->startb = 0;

  /* "tables/tableextension.pyx":840
 *     self.coords = coords
 *     self.startb = 0
 *     if step > 0:             # <<<<<<<<<<<<<<
 *         self._row = -1  # a sentinel
 *         self.nrowsread = start
 */
  __pyx_t_2 = ((__pyx_v_step > 0) != 0);
  if (__pyx_t_2) {

    /* "tables/tableextension.pyx":841
 *     self.startb = 0
 *     if step > 0:
 *         self._row = -1  # a sentinel             # <<<<<<<<<<<<<<
 *         self.nrowsread = start
 *     elif step < 0:
 */
    __pyx_v_self->_row = -1L;

    /* "tables/tableextension.pyx":842
 *     if step > 0:
 *         self._row = -1  # a sentinel
 *         self.nrowsread = start             # <<<<<<<<<<<<<<
 *     elif step < 0:
 *         self._row = 0
 */
    __pyx_v_self->nrowsread = __pyx_v_start;

    /* "tables/tableextension.pyx":840
 *     self.coords = coords
 *     self.startb = 0
 *     if step > 0:             # <<<<<<<<<<<<<<
 *         self._row = -1  # a sentinel
 *         self.nrowsread = start
 */
    goto __pyx_L3;
  }

  /* "tables/tableextension.pyx":843
 *         self._row = -1  # a sentinel
 *         self.nrowsread = start
 *     elif step < 0:             # <<<<<<<<<<<<<<
 *         self._row = 0
 *         self.nrowsread = 0
 */
  __pyx_t_2 = ((__pyx_v_step < 0) != 0);
  if (__pyx_t_2) {

    /* "tables/tableextension.pyx":844
 *         self.nrowsread = start
 *     elif step < 0:
 *         self._row = 0             # <<<<<<<<<<<<<<
 *         self.nrowsread = 0
 *         self.nextelement = start
 */
    __pyx_v_self->_row = 0;

    /* "tables/tableextension.pyx":845
 *     elif step < 0:
 *         self._row = 0
 *         self.nrowsread = 0             # <<<<<<<<<<<<<<
 *         self.nextelement = start
 *     self._nrow = start - self.step
 */
    __pyx_v_self->nrowsread = 0;

    /* "tables/tableextension.pyx":846
 *         self._row = 0
 *         self.nrowsread = 0
 *         self.nextelement = start             # <<<<<<<<<<<<<<
 *     self._nrow = start - self.step
 *     self.wherecond = 0
 */
    __pyx_v_self->nextelement = __pyx_v_start;

    /* "tables/tableextension.pyx":843
 *         self._row = -1  # a sentinel
 *         self.nrowsread = start
 *     elif step < 0:             # <<<<<<<<<<<<<<
 *         self._row = 0
 *         self.nrowsread = 0
 */
  }
  __pyx_L3:;

  /* "tables/tableextension.pyx":847
 *         self.nrowsread = 0
 *         self.nextelement = start
 *     self._nrow = start - self.step             # <<<<<<<<<<<<<<
 *     self.wherecond = 0
 *     self.indexed = 0
 */
  __pyx_v_self->_nrow = (__pyx_v_start - __pyx_v_self->step);

  /* "tables/tableextension.pyx":848
 *         self.nextelement = start
 *     self._nrow = start - self.step
 *     self.wherecond = 0             # <<<<<<<<<<<<<<
 *     self.indexed = 0
 * 
 */
  __pyx_v_self->wherecond = 0;

  /* "tables/tableextension.pyx":849
 *     self._nrow = start - self.step
 *     self.wherecond = 0
 *     self.indexed = 0             # <<<<<<<<<<<<<<
 * 
 *     self.nrows = table.nrows   # Update the row counter
 */
  __pyx_v_self->indexed = 0;

  /* "tables/tableextension.pyx":851
 *     self.indexed = 0
 * 
 *     self.nrows = table.nrows   # Update the row counter             # <<<<<<<<<<<<<<
 * 
 *     if coords is not None and 0 < step:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_table, __pyx_n_s_nrows); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 851, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_hsize_t(__pyx_t_1); if (unlikely((__pyx_t_3 == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 851, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->nrows = __pyx_t_3;

  /* "tables/tableextension.pyx":853
 *     self.nrows = table.nrows   # Update the row counter
 * 
 *     if coords is not None and 0 < step:             # <<<<<<<<<<<<<<
 *       self.nrowsread = start
 *       self.nextelement = start
 */
  __pyx_t_4 = (__pyx_v_coords != Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_2 = __pyx_t_5;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_5 = ((0 < __pyx_v_step) != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L5_bool_binop_done:;
  if (__pyx_t_2) {

    /* "tables/tableextension.pyx":854
 * 
 *     if coords is not None and 0 < step:
 *       self.nrowsread = start             # <<<<<<<<<<<<<<
 *       self.nextelement = start
 *       self.stop = min(stop, len(coords))
 */
    __pyx_v_self->nrowsread = __pyx_v_start;

    /* "tables/tableextension.pyx":855
 *     if coords is not None and 0 < step:
 *       self.nrowsread = start
 *       self.nextelement = start             # <<<<<<<<<<<<<<
 *       self.stop = min(stop, len(coords))
 *       self.absstep = abs(step)
 */
    __pyx_v_self->nextelement = __pyx_v_start;

    /* "tables/tableextension.pyx":856
 *       self.nrowsread = start
 *       self.nextelement = start
 *       self.stop = min(stop, len(coords))             # <<<<<<<<<<<<<<
 *       self.absstep = abs(step)
 *       return
 */
    __pyx_t_6 = PyObject_Length(__pyx_v_coords); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 856, __pyx_L1_error)
    __pyx_t_7 = __pyx_v_stop;
    if (((__pyx_t_6 < __pyx_t_7) != 0)) {
      __pyx_t_8 = __pyx_t_6;
    } else {
      __pyx_t_8 = __pyx_t_7;
    }
    __pyx_v_self->stop = __pyx_t_8;

    /* "tables/tableextension.pyx":857
 *       self.nextelement = start
 *       self.stop = min(stop, len(coords))
 *       self.absstep = abs(step)             # <<<<<<<<<<<<<<
 *       return
 *     elif coords is not None and 0 > step:
 */
    __pyx_t_8 = __Pyx_abs_longlong(__pyx_v_step); 
    __pyx_v_self->absstep = __pyx_t_8;

    /* "tables/tableextension.pyx":858
 *       self.stop = min(stop, len(coords))
 *       self.absstep = abs(step)
 *       return             # <<<<<<<<<<<<<<
 *     elif coords is not None and 0 > step:
 *       #self.nrowsread = 0
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "tables/tableextension.pyx":853
 *     self.nrows = table.nrows   # Update the row counter
 * 
 *     if coords is not None and 0 < step:             # <<<<<<<<<<<<<<
 *       self.nrowsread = start
 *       self.nextelement = start
 */
  }

  /* "tables/tableextension.pyx":859
 *       self.absstep = abs(step)
 *       return
 *     elif coords is not None and 0 > step:             # <<<<<<<<<<<<<<
 *       #self.nrowsread = 0
 *       #self.nextelement = start
 */
  __pyx_t_5 = (__pyx_v_coords != Py_None);
  __pyx_t_4 = (__pyx_t_5 != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_4 = ((0 > __pyx_v_step) != 0);
  __pyx_t_2 = __pyx_t_4;
  __pyx_L7_bool_binop_done:;
  if (__pyx_t_2) {

    /* "tables/tableextension.pyx":864
 *       #self.stop = min(stop, len(coords))
 *       #self.stop = max(stop, start - len(coords))
 *       self.absstep = abs(step)             # <<<<<<<<<<<<<<
 *       return
 * 
 */
    __pyx_t_8 = __Pyx_abs_longlong(__pyx_v_step); 
    __pyx_v_self->absstep = __pyx_t_8;

    /* "tables/tableextension.pyx":865
 *       #self.stop = max(stop, start - len(coords))
 *       self.absstep = abs(step)
 *       return             # <<<<<<<<<<<<<<
 * 
 *     if table._where_condition:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "tables/tableextension.pyx":859
 *       self.absstep = abs(step)
 *       return
 *     elif coords is not None and 0 > step:             # <<<<<<<<<<<<<<
 *       #self.nrowsread = 0
 *       #self.nextelement = start
 */
  }

  /* "tables/tableextension.pyx":867
 *       return
 * 
 *     if table._where_condition:             # <<<<<<<<<<<<<<
 *       self.wherecond = 1
 *       #self.condkwargs = {'ex_uses_vml': True}
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_table, __pyx_n_s_where_condition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 867, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 867, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "tables/tableextension.pyx":868
 * 
 *     if table._where_condition:
 *       self.wherecond = 1             # <<<<<<<<<<<<<<
 *       #self.condkwargs = {'ex_uses_vml': True}
 *       self.condfunc, self.condargs, self.condkwargs = table._where_condition
 */
    __pyx_v_self->wherecond = 1;

    /* "tables/tableextension.pyx":870
 *       self.wherecond = 1
 *       #self.condkwargs = {'ex_uses_vml': True}
 *       self.condfunc, self.condargs, self.condkwargs = table._where_condition             # <<<<<<<<<<<<<<
 *       table._where_condition = None
 * 
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_table, __pyx_n_s_where_condition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 870, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
      PyObject* sequence = __pyx_t_1;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 3)) {
        if (size > 3) __Pyx_RaiseTooManyValuesError(3);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 870, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_9 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_10 = PyTuple_GET_ITEM(sequence, 1); 
        __pyx_t_11 = PyTuple_GET_ITEM(sequence, 2); 
      } else {
        __pyx_t_9 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_10 = PyList_GET_ITEM(sequence, 1); 
        __pyx_t_11 = PyList_GET_ITEM(sequence, 2); 
      }
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_11);
      #else
      __pyx_t_9 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 870, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_10 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 870, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_11 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 870, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_12 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 870, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_13 = Py_TYPE(__pyx_t_12)->tp_iternext;
      index = 0; __pyx_t_9 = __pyx_t_13(__pyx_t_12); if (unlikely(!__pyx_t_9)) goto __pyx_L10_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_9);
      index = 1; __pyx_t_10 = __pyx_t_13(__pyx_t_12); if (unlikely(!__pyx_t_10)) goto __pyx_L10_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_10);
      index = 2; __pyx_t_11 = __pyx_t_13(__pyx_t_12); if (unlikely(!__pyx_t_11)) goto __pyx_L10_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_11);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_13(__pyx_t_12), 3) < 0) __PYX_ERR(0, 870, __pyx_L1_error)
      __pyx_t_13 = NULL;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      goto __pyx_L11_unpacking_done;
      __pyx_L10_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __pyx_t_13 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 870, __pyx_L1_error)
      __pyx_L11_unpacking_done:;
    }
    __Pyx_GIVEREF(__pyx_t_9);
    __Pyx_GOTREF(__pyx_v_self->condfunc);
    __Pyx_DECREF(__pyx_v_self->condfunc);
    __pyx_v_self->condfunc = __pyx_t_9;
    __pyx_t_9 = 0;
    __Pyx_GIVEREF(__pyx_t_10);
    __Pyx_GOTREF(__pyx_v_self->condargs);
    __Pyx_DECREF(__pyx_v_self->condargs);
    __pyx_v_self->condargs = __pyx_t_10;
    __pyx_t_10 = 0;
    __Pyx_GIVEREF(__pyx_t_11);
    __Pyx_GOTREF(__pyx_v_self->condkwargs);
    __Pyx_DECREF(__pyx_v_self->condkwargs);
    __pyx_v_self->condkwargs = __pyx_t_11;
    __pyx_t_11 = 0;

    /* "tables/tableextension.pyx":871
 *       #self.condkwargs = {'ex_uses_vml': True}
 *       self.condfunc, self.condargs, self.condkwargs = table._where_condition
 *       table._where_condition = None             # <<<<<<<<<<<<<<
 * 
 *     if table._use_index:
 */
    if (__Pyx_PyObject_SetAttrStr(__pyx_v_table, __pyx_n_s_where_condition, Py_None) < 0) __PYX_ERR(0, 871, __pyx_L1_error)

    /* "tables/tableextension.pyx":867
 *       return
 * 
 *     if table._where_condition:             # <<<<<<<<<<<<<<
 *       self.wherecond = 1
 *       #self.condkwargs = {'ex_uses_vml': True}
 */
  }

  /* "tables/tableextension.pyx":873
 *       table._where_condition = None
 * 
 *     if table._use_index:             # <<<<<<<<<<<<<<
 *       # Indexing code depends on this condition (see #319)
 *       assert self.nrowsinbuf % self.chunksize == 0
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_table, __pyx_n_s_use_index); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 873, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 873, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "tables/tableextension.pyx":875
 *     if table._use_index:
 *       # Indexing code depends on this condition (see #319)
 *       assert self.nrowsinbuf % self.chunksize == 0             # <<<<<<<<<<<<<<
 *       self.indexed = 1
 *       # Compute totalchunks here because self.nrows can change during the
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(!Py_OptimizeFlag)) {
      if (unlikely(__pyx_v_self->chunksize == 0)) {
        PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
        __PYX_ERR(0, 875, __pyx_L1_error)
      }
      if (unlikely(!(((__pyx_v_self->nrowsinbuf % __pyx_v_self->chunksize) == 0) != 0))) {
        PyErr_SetNone(PyExc_AssertionError);
        __PYX_ERR(0, 875, __pyx_L1_error)
      }
    }
    #endif

    /* "tables/tableextension.pyx":876
 *       # Indexing code depends on this condition (see #319)
 *       assert self.nrowsinbuf % self.chunksize == 0
 *       self.indexed = 1             # <<<<<<<<<<<<<<
 *       # Compute totalchunks here because self.nrows can change during the
 *       # life of a Row instance.
 */
    __pyx_v_self->indexed = 1;

    /* "tables/tableextension.pyx":879
 *       # Compute totalchunks here because self.nrows can change during the
 *       # life of a Row instance.
 *       self.totalchunks = self.nrows // self.chunksize             # <<<<<<<<<<<<<<
 *       if self.nrows % self.chunksize:
 *         self.totalchunks = self.totalchunks + 1
 */
    if (unlikely(__pyx_v_self->chunksize == 0)) {
      PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
      __PYX_ERR(0, 879, __pyx_L1_error)
    }
    __pyx_v_self->totalchunks = (__pyx_v_self->nrows / __pyx_v_self->chunksize);

    /* "tables/tableextension.pyx":880
 *       # life of a Row instance.
 *       self.totalchunks = self.nrows // self.chunksize
 *       if self.nrows % self.chunksize:             # <<<<<<<<<<<<<<
 *         self.totalchunks = self.totalchunks + 1
 *       self.nrowsread = 0
 */
    if (unlikely(__pyx_v_self->chunksize == 0)) {
      PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
      __PYX_ERR(0, 880, __pyx_L1_error)
    }
    __pyx_t_2 = ((__pyx_v_self->nrows % __pyx_v_self->chunksize) != 0);
    if (__pyx_t_2) {

      /* "tables/tableextension.pyx":881
 *       self.totalchunks = self.nrows // self.chunksize
 *       if self.nrows % self.chunksize:
 *         self.totalchunks = self.totalchunks + 1             # <<<<<<<<<<<<<<
 *       self.nrowsread = 0
 *       self.nextelement = 0
 */
      __pyx_v_self->totalchunks = (__pyx_v_self->totalchunks + 1);

      /* "tables/tableextension.pyx":880
 *       # life of a Row instance.
 *       self.totalchunks = self.nrows // self.chunksize
 *       if self.nrows % self.chunksize:             # <<<<<<<<<<<<<<
 *         self.totalchunks = self.totalchunks + 1
 *       self.nrowsread = 0
 */
    }

    /* "tables/tableextension.pyx":882
 *       if self.nrows % self.chunksize:
 *         self.totalchunks = self.totalchunks + 1
 *       self.nrowsread = 0             # <<<<<<<<<<<<<<
 *       self.nextelement = 0
 *       self.chunkmap = chunkmap
 */
    __pyx_v_self->nrowsread = 0;

    /* "tables/tableextension.pyx":883
 *         self.totalchunks = self.totalchunks + 1
 *       self.nrowsread = 0
 *       self.nextelement = 0             # <<<<<<<<<<<<<<
 *       self.chunkmap = chunkmap
 *       self.chunkmap_data = PyArray_BYTES(self.chunkmap)
 */
    __pyx_v_self->nextelement = 0;

    /* "tables/tableextension.pyx":884
 *       self.nrowsread = 0
 *       self.nextelement = 0
 *       self.chunkmap = chunkmap             # <<<<<<<<<<<<<<
 *       self.chunkmap_data = PyArray_BYTES(self.chunkmap)
 *       table._use_index = False
 */
    if (!(likely(((__pyx_v_chunkmap) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_chunkmap, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 884, __pyx_L1_error)
    __pyx_t_1 = __pyx_v_chunkmap;
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->chunkmap);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->chunkmap));
    __pyx_v_self->chunkmap = ((PyArrayObject *)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "tables/tableextension.pyx":885
 *       self.nextelement = 0
 *       self.chunkmap = chunkmap
 *       self.chunkmap_data = PyArray_BYTES(self.chunkmap)             # <<<<<<<<<<<<<<
 *       table._use_index = False
 *       self.lenbuf = self.nrowsinbuf
 */
    __pyx_t_1 = ((PyObject *)__pyx_v_self->chunkmap);
    __Pyx_INCREF(__pyx_t_1);
    __pyx_v_self->chunkmap_data = PyArray_BYTES(((PyArrayObject *)__pyx_t_1));
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "tables/tableextension.pyx":886
 *       self.chunkmap = chunkmap
 *       self.chunkmap_data = PyArray_BYTES(self.chunkmap)
 *       table._use_index = False             # <<<<<<<<<<<<<<
 *       self.lenbuf = self.nrowsinbuf
 *       # Check if we have limitations on start, stop, step
 */
    if (__Pyx_PyObject_SetAttrStr(__pyx_v_table, __pyx_n_s_use_index, Py_False) < 0) __PYX_ERR(0, 886, __pyx_L1_error)

    /* "tables/tableextension.pyx":887
 *       self.chunkmap_data = PyArray_BYTES(self.chunkmap)
 *       table._use_index = False
 *       self.lenbuf = self.nrowsinbuf             # <<<<<<<<<<<<<<
 *       # Check if we have limitations on start, stop, step
 *       self.sss_on = (self.start > 0 or self.stop < self.nrows or self.step > 1)
 */
    __pyx_t_3 = __pyx_v_self->nrowsinbuf;
    __pyx_v_self->lenbuf = __pyx_t_3;

    /* "tables/tableextension.pyx":889
 *       self.lenbuf = self.nrowsinbuf
 *       # Check if we have limitations on start, stop, step
 *       self.sss_on = (self.start > 0 or self.stop < self.nrows or self.step > 1)             # <<<<<<<<<<<<<<
 * 
 *     self.seqcache_key = table._seqcache_key
 */
    __pyx_t_2 = (__pyx_v_self->start > 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_14 = __pyx_t_2;
      goto __pyx_L14_bool_binop_done;
    }
    __pyx_t_2 = (__pyx_v_self->stop < __pyx_v_self->nrows);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_14 = __pyx_t_2;
      goto __pyx_L14_bool_binop_done;
    }
    __pyx_t_2 = (__pyx_v_self->step > 1);
    __pyx_t_14 = __pyx_t_2;
    __pyx_L14_bool_binop_done:;
    __pyx_v_self->sss_on = __pyx_t_14;

    /* "tables/tableextension.pyx":873
 *       table._where_condition = None
 * 
 *     if table._use_index:             # <<<<<<<<<<<<<<
 *       # Indexing code depends on this condition (see #319)
 *       assert self.nrowsinbuf % self.chunksize == 0
 */
  }

  /* "tables/tableextension.pyx":891
 *       self.sss_on = (self.start > 0 or self.stop < self.nrows or self.step > 1)
 * 
 *     self.seqcache_key = table._seqcache_key             # <<<<<<<<<<<<<<
 *     table._seqcache_key = None
 *     if self.seqcache_key is not None:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_table, __pyx_n_s_seqcache_key); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 891, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->seqcache_key);
  __Pyx_DECREF(__pyx_v_self->seqcache_key);
  __pyx_v_self->seqcache_key = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":892
 * 
 *     self.seqcache_key = table._seqcache_key
 *     table._seqcache_key = None             # <<<<<<<<<<<<<<
 *     if self.seqcache_key is not None:
 *       self._write_to_seqcache = 1
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_table, __pyx_n_s_seqcache_key, Py_None) < 0) __PYX_ERR(0, 892, __pyx_L1_error)

  /* "tables/tableextension.pyx":893
 *     self.seqcache_key = table._seqcache_key
 *     table._seqcache_key = None
 *     if self.seqcache_key is not None:             # <<<<<<<<<<<<<<
 *       self._write_to_seqcache = 1
 *       self.iterseq_max_elements = table._v_file.params['ITERSEQ_MAX_ELEMENTS']
 */
  __pyx_t_2 = (__pyx_v_self->seqcache_key != Py_None);
  __pyx_t_4 = (__pyx_t_2 != 0);
  if (__pyx_t_4) {

    /* "tables/tableextension.pyx":894
 *     table._seqcache_key = None
 *     if self.seqcache_key is not None:
 *       self._write_to_seqcache = 1             # <<<<<<<<<<<<<<
 *       self.iterseq_max_elements = table._v_file.params['ITERSEQ_MAX_ELEMENTS']
 *       self.iterseq = [] # all the row indexes, unless it would be longer than ITERSEQ_MAX_ELEMENTS
 */
    __pyx_v_self->_write_to_seqcache = 1;

    /* "tables/tableextension.pyx":895
 *     if self.seqcache_key is not None:
 *       self._write_to_seqcache = 1
 *       self.iterseq_max_elements = table._v_file.params['ITERSEQ_MAX_ELEMENTS']             # <<<<<<<<<<<<<<
 *       self.iterseq = [] # all the row indexes, unless it would be longer than ITERSEQ_MAX_ELEMENTS
 *     else:
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_table, __pyx_n_s_v_file); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 895, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_params); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 895, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_11, __pyx_n_s_ITERSEQ_MAX_ELEMENTS); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 895, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __pyx_t_14 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_14 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 895, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_self->iterseq_max_elements = __pyx_t_14;

    /* "tables/tableextension.pyx":896
 *       self._write_to_seqcache = 1
 *       self.iterseq_max_elements = table._v_file.params['ITERSEQ_MAX_ELEMENTS']
 *       self.iterseq = [] # all the row indexes, unless it would be longer than ITERSEQ_MAX_ELEMENTS             # <<<<<<<<<<<<<<
 *     else:
 *       self._write_to_seqcache = 0
 */
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 896, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->iterseq);
    __Pyx_DECREF(__pyx_v_self->iterseq);
    __pyx_v_self->iterseq = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "tables/tableextension.pyx":893
 *     self.seqcache_key = table._seqcache_key
 *     table._seqcache_key = None
 *     if self.seqcache_key is not None:             # <<<<<<<<<<<<<<
 *       self._write_to_seqcache = 1
 *       self.iterseq_max_elements = table._v_file.params['ITERSEQ_MAX_ELEMENTS']
 */
    goto __pyx_L17;
  }

  /* "tables/tableextension.pyx":898
 *       self.iterseq = [] # all the row indexes, unless it would be longer than ITERSEQ_MAX_ELEMENTS
 *     else:
 *       self._write_to_seqcache = 0             # <<<<<<<<<<<<<<
 *       self.iterseq = None
 * 
 */
  /*else*/ {
    __pyx_v_self->_write_to_seqcache = 0;

    /* "tables/tableextension.pyx":899
 *     else:
 *       self._write_to_seqcache = 0
 *       self.iterseq = None             # <<<<<<<<<<<<<<
 * 
 *   def __next__(self):
 */
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_self->iterseq);
    __Pyx_DECREF(__pyx_v_self->iterseq);
    __pyx_v_self->iterseq = Py_None;
  }
  __pyx_L17:;

  /* "tables/tableextension.pyx":830
 *     self.nrows = table.nrows  # This value may change
 * 
 *   cdef _init_loop(self, hsize_t start, long long stop, long long step,             # <<<<<<<<<<<<<<
 *                  object coords, object chunkmap):
 *     """Initialization for the __iter__ iterator"""
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("tables.tableextension.Row._init_loop", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_table);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":901
 *       self.iterseq = None
 * 
 *   def __next__(self):             # <<<<<<<<<<<<<<
 *     """next() method for __iter__() that is called on each iteration"""
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_14tableextension_3Row_7__next__(PyObject *__pyx_v_self); /*proto*/
static char __pyx_doc_6tables_14tableextension_3Row_6__next__[] = "next() method for __iter__() that is called on each iteration";
#if CYTHON_COMPILING_IN_CPYTHON
struct wrapperbase __pyx_wrapperbase_6tables_14tableextension_3Row_6__next__;
#endif
static PyObject *__pyx_pw_6tables_14tableextension_3Row_7__next__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__next__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6tables_14tableextension_3Row_6__next__(((struct __pyx_obj_6tables_14tableextension_Row *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_14tableextension_3Row_6__next__(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__next__", 0);

  /* "tables/tableextension.pyx":904
 *     """next() method for __iter__() that is called on each iteration"""
 * 
 *     if not self._riterator:             # <<<<<<<<<<<<<<
 *       # The iterator is already exhausted!
 *       raise StopIteration
 */
  __pyx_t_1 = ((!(__pyx_v_self->_riterator != 0)) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "tables/tableextension.pyx":906
 *     if not self._riterator:
 *       # The iterator is already exhausted!
 *       raise StopIteration             # <<<<<<<<<<<<<<
 *     if self.indexed:
 *       return self.__next__indexed()
 */
    __Pyx_Raise(__pyx_builtin_StopIteration, 0, 0, 0);
    __PYX_ERR(0, 906, __pyx_L1_error)

    /* "tables/tableextension.pyx":904
 *     """next() method for __iter__() that is called on each iteration"""
 * 
 *     if not self._riterator:             # <<<<<<<<<<<<<<
 *       # The iterator is already exhausted!
 *       raise StopIteration
 */
  }

  /* "tables/tableextension.pyx":907
 *       # The iterator is already exhausted!
 *       raise StopIteration
 *     if self.indexed:             # <<<<<<<<<<<<<<
 *       return self.__next__indexed()
 *     elif self.coords is not None:
 */
  __pyx_t_1 = (__pyx_v_self->indexed != 0);
  if (__pyx_t_1) {

    /* "tables/tableextension.pyx":908
 *       raise StopIteration
 *     if self.indexed:
 *       return self.__next__indexed()             # <<<<<<<<<<<<<<
 *     elif self.coords is not None:
 *       return self.__next__coords()
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = ((struct __pyx_vtabstruct_6tables_14tableextension_Row *)__pyx_v_self->__pyx_vtab)->__pyx___next__indexed(__pyx_v_self); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 908, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "tables/tableextension.pyx":907
 *       # The iterator is already exhausted!
 *       raise StopIteration
 *     if self.indexed:             # <<<<<<<<<<<<<<
 *       return self.__next__indexed()
 *     elif self.coords is not None:
 */
  }

  /* "tables/tableextension.pyx":909
 *     if self.indexed:
 *       return self.__next__indexed()
 *     elif self.coords is not None:             # <<<<<<<<<<<<<<
 *       return self.__next__coords()
 *     elif self.wherecond:
 */
  __pyx_t_1 = (__pyx_v_self->coords != Py_None);
  __pyx_t_3 = (__pyx_t_1 != 0);
  if (__pyx_t_3) {

    /* "tables/tableextension.pyx":910
 *       return self.__next__indexed()
 *     elif self.coords is not None:
 *       return self.__next__coords()             # <<<<<<<<<<<<<<
 *     elif self.wherecond:
 *       return self.__next__inkernel()
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = ((struct __pyx_vtabstruct_6tables_14tableextension_Row *)__pyx_v_self->__pyx_vtab)->__pyx___next__coords(__pyx_v_self); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 910, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "tables/tableextension.pyx":909
 *     if self.indexed:
 *       return self.__next__indexed()
 *     elif self.coords is not None:             # <<<<<<<<<<<<<<
 *       return self.__next__coords()
 *     elif self.wherecond:
 */
  }

  /* "tables/tableextension.pyx":911
 *     elif self.coords is not None:
 *       return self.__next__coords()
 *     elif self.wherecond:             # <<<<<<<<<<<<<<
 *       return self.__next__inkernel()
 *     else:
 */
  __pyx_t_3 = (__pyx_v_self->wherecond != 0);
  if (__pyx_t_3) {

    /* "tables/tableextension.pyx":912
 *       return self.__next__coords()
 *     elif self.wherecond:
 *       return self.__next__inkernel()             # <<<<<<<<<<<<<<
 *     else:
 *       return self.__next__general()
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = ((struct __pyx_vtabstruct_6tables_14tableextension_Row *)__pyx_v_self->__pyx_vtab)->__pyx___next__inkernel(__pyx_v_self); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 912, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "tables/tableextension.pyx":911
 *     elif self.coords is not None:
 *       return self.__next__coords()
 *     elif self.wherecond:             # <<<<<<<<<<<<<<
 *       return self.__next__inkernel()
 *     else:
 */
  }

  /* "tables/tableextension.pyx":914
 *       return self.__next__inkernel()
 *     else:
 *       return self.__next__general()             # <<<<<<<<<<<<<<
 * 
 *   cdef __next__indexed(self):
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = ((struct __pyx_vtabstruct_6tables_14tableextension_Row *)__pyx_v_self->__pyx_vtab)->__pyx___next__general(__pyx_v_self); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 914, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "tables/tableextension.pyx":901
 *       self.iterseq = None
 * 
 *   def __next__(self):             # <<<<<<<<<<<<<<
 *     """next() method for __iter__() that is called on each iteration"""
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("tables.tableextension.Row.__next__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":916
 *       return self.__next__general()
 * 
 *   cdef __next__indexed(self):             # <<<<<<<<<<<<<<
 *     """The version of next() for indexed columns and a chunkmap."""
 * 
 */

static PyObject *__pyx_f_6tables_14tableextension_3Row___next__indexed(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self) {
  long __pyx_v_recout;
  long __pyx_v_j;
  long __pyx_v_cs;
  hsize_t __pyx_v_nchunksread;
  PyObject *__pyx_v_tmp_range = 0;
  struct __pyx_obj_6tables_14tableextension_Table *__pyx_v_table = 0;
  PyArrayObject *__pyx_v_iobuf = 0;
  PyObject *__pyx_v_seqcache = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  hsize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PY_LONG_LONG __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__next__indexed", 0);

  /* "tables/tableextension.pyx":929
 *     cdef object seqcache
 * 
 *     assert self.nrowsinbuf >= self.chunksize             # <<<<<<<<<<<<<<
 *     while self.nextelement < self.stop:
 *       if self.nextelement >= self.nrowsread:
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_self->nrowsinbuf >= __pyx_v_self->chunksize) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 929, __pyx_L1_error)
    }
  }
  #endif

  /* "tables/tableextension.pyx":930
 * 
 *     assert self.nrowsinbuf >= self.chunksize
 *     while self.nextelement < self.stop:             # <<<<<<<<<<<<<<
 *       if self.nextelement >= self.nrowsread:
 *         # Skip until there is interesting information
 */
  while (1) {
    __pyx_t_1 = ((__pyx_v_self->nextelement < __pyx_v_self->stop) != 0);
    if (!__pyx_t_1) break;

    /* "tables/tableextension.pyx":931
 *     assert self.nrowsinbuf >= self.chunksize
 *     while self.nextelement < self.stop:
 *       if self.nextelement >= self.nrowsread:             # <<<<<<<<<<<<<<
 *         # Skip until there is interesting information
 *         while self.start > self.nrowsread + self.nrowsinbuf:
 */
    __pyx_t_1 = ((__pyx_v_self->nextelement >= __pyx_v_self->nrowsread) != 0);
    if (__pyx_t_1) {

      /* "tables/tableextension.pyx":933
 *       if self.nextelement >= self.nrowsread:
 *         # Skip until there is interesting information
 *         while self.start > self.nrowsread + self.nrowsinbuf:             # <<<<<<<<<<<<<<
 *           self.nrowsread = self.nrowsread + self.nrowsinbuf
 *           self.nextelement = self.nextelement + self.nrowsinbuf
 */
      while (1) {
        __pyx_t_1 = ((__pyx_v_self->start > (__pyx_v_self->nrowsread + __pyx_v_self->nrowsinbuf)) != 0);
        if (!__pyx_t_1) break;

        /* "tables/tableextension.pyx":934
 *         # Skip until there is interesting information
 *         while self.start > self.nrowsread + self.nrowsinbuf:
 *           self.nrowsread = self.nrowsread + self.nrowsinbuf             # <<<<<<<<<<<<<<
 *           self.nextelement = self.nextelement + self.nrowsinbuf
 * 
 */
        __pyx_v_self->nrowsread = (__pyx_v_self->nrowsread + __pyx_v_self->nrowsinbuf);

        /* "tables/tableextension.pyx":935
 *         while self.start > self.nrowsread + self.nrowsinbuf:
 *           self.nrowsread = self.nrowsread + self.nrowsinbuf
 *           self.nextelement = self.nextelement + self.nrowsinbuf             # <<<<<<<<<<<<<<
 * 
 *         table = self.table
 */
        __pyx_v_self->nextelement = (__pyx_v_self->nextelement + __pyx_v_self->nrowsinbuf);
      }

      /* "tables/tableextension.pyx":937
 *           self.nextelement = self.nextelement + self.nrowsinbuf
 * 
 *         table = self.table             # <<<<<<<<<<<<<<
 *         iobuf = self.iobuf
 *         j = 0;  recout = 0;  cs = self.chunksize
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_table); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 937, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_6tables_14tableextension_Table))))) __PYX_ERR(0, 937, __pyx_L1_error)
      __Pyx_XDECREF_SET(__pyx_v_table, ((struct __pyx_obj_6tables_14tableextension_Table *)__pyx_t_2));
      __pyx_t_2 = 0;

      /* "tables/tableextension.pyx":938
 * 
 *         table = self.table
 *         iobuf = self.iobuf             # <<<<<<<<<<<<<<
 *         j = 0;  recout = 0;  cs = self.chunksize
 *         nchunksread = self.nrowsread // cs
 */
      if (!(likely(((__pyx_v_self->iobuf) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_self->iobuf, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 938, __pyx_L1_error)
      __pyx_t_2 = __pyx_v_self->iobuf;
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_XDECREF_SET(__pyx_v_iobuf, ((PyArrayObject *)__pyx_t_2));
      __pyx_t_2 = 0;

      /* "tables/tableextension.pyx":939
 *         table = self.table
 *         iobuf = self.iobuf
 *         j = 0;  recout = 0;  cs = self.chunksize             # <<<<<<<<<<<<<<
 *         nchunksread = self.nrowsread // cs
 *         tmp_range = numpy.arange(0, cs, dtype='int64')
 */
      __pyx_v_j = 0;
      __pyx_v_recout = 0;
      __pyx_t_3 = __pyx_v_self->chunksize;
      __pyx_v_cs = __pyx_t_3;

      /* "tables/tableextension.pyx":940
 *         iobuf = self.iobuf
 *         j = 0;  recout = 0;  cs = self.chunksize
 *         nchunksread = self.nrowsread // cs             # <<<<<<<<<<<<<<
 *         tmp_range = numpy.arange(0, cs, dtype='int64')
 *         self.bufcoords = numpy.empty(self.nrowsinbuf, dtype='int64')
 */
      if (unlikely(__pyx_v_cs == 0)) {
        PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
        __PYX_ERR(0, 940, __pyx_L1_error)
      }
      __pyx_v_nchunksread = (__pyx_v_self->nrowsread / __pyx_v_cs);

      /* "tables/tableextension.pyx":941
 *         j = 0;  recout = 0;  cs = self.chunksize
 *         nchunksread = self.nrowsread // cs
 *         tmp_range = numpy.arange(0, cs, dtype='int64')             # <<<<<<<<<<<<<<
 *         self.bufcoords = numpy.empty(self.nrowsinbuf, dtype='int64')
 *         # Fetch valid chunks until the I/O buffer is full
 */
      __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_numpy); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 941, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_arange); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 941, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v_cs); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 941, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 941, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_int_0);
      __Pyx_GIVEREF(__pyx_int_0);
      PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_int_0);
      __Pyx_GIVEREF(__pyx_t_2);
      PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
      __pyx_t_2 = 0;
      __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 941, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_n_s_int64) < 0) __PYX_ERR(0, 941, __pyx_L1_error)
      __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_5, __pyx_t_2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 941, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF_SET(__pyx_v_tmp_range, __pyx_t_6);
      __pyx_t_6 = 0;

      /* "tables/tableextension.pyx":942
 *         nchunksread = self.nrowsread // cs
 *         tmp_range = numpy.arange(0, cs, dtype='int64')
 *         self.bufcoords = numpy.empty(self.nrowsinbuf, dtype='int64')             # <<<<<<<<<<<<<<
 *         # Fetch valid chunks until the I/O buffer is full
 *         while nchunksread < self.totalchunks:
 */
      __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_numpy); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 942, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_empty); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 942, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = __Pyx_PyInt_From_hsize_t(__pyx_v_self->nrowsinbuf); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 942, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 942, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_6);
      __pyx_t_6 = 0;
      __pyx_t_6 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 942, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_dtype, __pyx_n_s_int64) < 0) __PYX_ERR(0, 942, __pyx_L1_error)
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 942, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 942, __pyx_L1_error)
      __Pyx_GIVEREF(__pyx_t_4);
      __Pyx_GOTREF(__pyx_v_self->bufcoords);
      __Pyx_DECREF(((PyObject *)__pyx_v_self->bufcoords));
      __pyx_v_self->bufcoords = ((PyArrayObject *)__pyx_t_4);
      __pyx_t_4 = 0;

      /* "tables/tableextension.pyx":944
 *         self.bufcoords = numpy.empty(self.nrowsinbuf, dtype='int64')
 *         # Fetch valid chunks until the I/O buffer is full
 *         while nchunksread < self.totalchunks:             # <<<<<<<<<<<<<<
 *           if self.chunkmap_data[nchunksread]:
 *             self.bufcoords[j*cs:(j+1)*cs] = tmp_range + self.nrowsread
 */
      while (1) {
        __pyx_t_1 = ((__pyx_v_nchunksread < __pyx_v_self->totalchunks) != 0);
        if (!__pyx_t_1) break;

        /* "tables/tableextension.pyx":945
 *         # Fetch valid chunks until the I/O buffer is full
 *         while nchunksread < self.totalchunks:
 *           if self.chunkmap_data[nchunksread]:             # <<<<<<<<<<<<<<
 *             self.bufcoords[j*cs:(j+1)*cs] = tmp_range + self.nrowsread
 *             # Not optimized read
 */
        __pyx_t_1 = ((__pyx_v_self->chunkmap_data[__pyx_v_nchunksread]) != 0);
        if (__pyx_t_1) {

          /* "tables/tableextension.pyx":946
 *         while nchunksread < self.totalchunks:
 *           if self.chunkmap_data[nchunksread]:
 *             self.bufcoords[j*cs:(j+1)*cs] = tmp_range + self.nrowsread             # <<<<<<<<<<<<<<
 *             # Not optimized read
 *             #  recout = recout + table._read_records(
 */
          __pyx_t_4 = __Pyx_PyInt_From_hsize_t(__pyx_v_self->nrowsread); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 946, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_6 = PyNumber_Add(__pyx_v_tmp_range, __pyx_t_4); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 946, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (__Pyx_PyObject_SetSlice(((PyObject *)__pyx_v_self->bufcoords), __pyx_t_6, (__pyx_v_j * __pyx_v_cs), ((__pyx_v_j + 1) * __pyx_v_cs), NULL, NULL, NULL, 1, 1, 1) < 0) __PYX_ERR(0, 946, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

          /* "tables/tableextension.pyx":955
 *             # cache, but using the PyTables one has the advantage that the
 *             # user can easily change this parameter.
 *             recout = recout + table._read_chunk(nchunksread, iobuf, j*cs)             # <<<<<<<<<<<<<<
 *             j = j + 1
 *           self.nrowsread = (nchunksread+1)*cs
 */
          __pyx_v_recout = (__pyx_v_recout + ((struct __pyx_vtabstruct_6tables_14tableextension_Table *)__pyx_v_table->__pyx_base.__pyx_vtab)->_read_chunk(__pyx_v_table, __pyx_v_nchunksread, __pyx_v_iobuf, (__pyx_v_j * __pyx_v_cs)));

          /* "tables/tableextension.pyx":956
 *             # user can easily change this parameter.
 *             recout = recout + table._read_chunk(nchunksread, iobuf, j*cs)
 *             j = j + 1             # <<<<<<<<<<<<<<
 *           self.nrowsread = (nchunksread+1)*cs
 *           if self.nrowsread > self.stop:
 */
          __pyx_v_j = (__pyx_v_j + 1);

          /* "tables/tableextension.pyx":945
 *         # Fetch valid chunks until the I/O buffer is full
 *         while nchunksread < self.totalchunks:
 *           if self.chunkmap_data[nchunksread]:             # <<<<<<<<<<<<<<
 *             self.bufcoords[j*cs:(j+1)*cs] = tmp_range + self.nrowsread
 *             # Not optimized read
 */
        }

        /* "tables/tableextension.pyx":957
 *             recout = recout + table._read_chunk(nchunksread, iobuf, j*cs)
 *             j = j + 1
 *           self.nrowsread = (nchunksread+1)*cs             # <<<<<<<<<<<<<<
 *           if self.nrowsread > self.stop:
 *             self.nrowsread = self.stop
 */
        __pyx_v_self->nrowsread = ((__pyx_v_nchunksread + 1) * __pyx_v_cs);

        /* "tables/tableextension.pyx":958
 *             j = j + 1
 *           self.nrowsread = (nchunksread+1)*cs
 *           if self.nrowsread > self.stop:             # <<<<<<<<<<<<<<
 *             self.nrowsread = self.stop
 *             break
 */
        __pyx_t_1 = ((__pyx_v_self->nrowsread > __pyx_v_self->stop) != 0);
        if (__pyx_t_1) {

          /* "tables/tableextension.pyx":959
 *           self.nrowsread = (nchunksread+1)*cs
 *           if self.nrowsread > self.stop:
 *             self.nrowsread = self.stop             # <<<<<<<<<<<<<<
 *             break
 *           elif j == self.nchunksinbuf:
 */
          __pyx_t_7 = __pyx_v_self->stop;
          __pyx_v_self->nrowsread = __pyx_t_7;

          /* "tables/tableextension.pyx":960
 *           if self.nrowsread > self.stop:
 *             self.nrowsread = self.stop
 *             break             # <<<<<<<<<<<<<<
 *           elif j == self.nchunksinbuf:
 *             break
 */
          goto __pyx_L9_break;

          /* "tables/tableextension.pyx":958
 *             j = j + 1
 *           self.nrowsread = (nchunksread+1)*cs
 *           if self.nrowsread > self.stop:             # <<<<<<<<<<<<<<
 *             self.nrowsread = self.stop
 *             break
 */
        }

        /* "tables/tableextension.pyx":961
 *             self.nrowsread = self.stop
 *             break
 *           elif j == self.nchunksinbuf:             # <<<<<<<<<<<<<<
 *             break
 *           nchunksread = nchunksread + 1
 */
        __pyx_t_1 = ((__pyx_v_j == __pyx_v_self->nchunksinbuf) != 0);
        if (__pyx_t_1) {

          /* "tables/tableextension.pyx":962
 *             break
 *           elif j == self.nchunksinbuf:
 *             break             # <<<<<<<<<<<<<<
 *           nchunksread = nchunksread + 1
 * 
 */
          goto __pyx_L9_break;

          /* "tables/tableextension.pyx":961
 *             self.nrowsread = self.stop
 *             break
 *           elif j == self.nchunksinbuf:             # <<<<<<<<<<<<<<
 *             break
 *           nchunksread = nchunksread + 1
 */
        }

        /* "tables/tableextension.pyx":963
 *           elif j == self.nchunksinbuf:
 *             break
 *           nchunksread = nchunksread + 1             # <<<<<<<<<<<<<<
 * 
 *         # Evaluate the condition on this table fragment.
 */
        __pyx_v_nchunksread = (__pyx_v_nchunksread + 1);
      }
      __pyx_L9_break:;

      /* "tables/tableextension.pyx":966
 * 
 *         # Evaluate the condition on this table fragment.
 *         iobuf = iobuf[:recout]             # <<<<<<<<<<<<<<
 * 
 *         if len(iobuf) > 0:
 */
      __pyx_t_6 = __Pyx_PyObject_GetSlice(((PyObject *)__pyx_v_iobuf), 0, __pyx_v_recout, NULL, NULL, NULL, 0, 1, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 966, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 966, __pyx_L1_error)
      __Pyx_DECREF_SET(__pyx_v_iobuf, ((PyArrayObject *)__pyx_t_6));
      __pyx_t_6 = 0;

      /* "tables/tableextension.pyx":968
 *         iobuf = iobuf[:recout]
 * 
 *         if len(iobuf) > 0:             # <<<<<<<<<<<<<<
 *           self.table._convert_types(iobuf, len(iobuf), 1)
 *         self.indexvalid = call_on_recarr(
 */
      __pyx_t_8 = PyObject_Length(((PyObject *)__pyx_v_iobuf)); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(0, 968, __pyx_L1_error)
      __pyx_t_1 = ((__pyx_t_8 > 0) != 0);
      if (__pyx_t_1) {

        /* "tables/tableextension.pyx":969
 * 
 *         if len(iobuf) > 0:
 *           self.table._convert_types(iobuf, len(iobuf), 1)             # <<<<<<<<<<<<<<
 *         self.indexvalid = call_on_recarr(
 *           self.condfunc, self.condargs, iobuf, **self.condkwargs)
 */
        __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_table); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 969, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_convert_types); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 969, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_8 = PyObject_Length(((PyObject *)__pyx_v_iobuf)); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(0, 969, __pyx_L1_error)
        __pyx_t_4 = PyInt_FromSsize_t(__pyx_t_8); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 969, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_2 = NULL;
        __pyx_t_9 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_2)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_2);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
            __pyx_t_9 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[4] = {__pyx_t_2, ((PyObject *)__pyx_v_iobuf), __pyx_t_4, __pyx_int_1};
          __pyx_t_6 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_9, 3+__pyx_t_9); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 969, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[4] = {__pyx_t_2, ((PyObject *)__pyx_v_iobuf), __pyx_t_4, __pyx_int_1};
          __pyx_t_6 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_9, 3+__pyx_t_9); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 969, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else
        #endif
        {
          __pyx_t_10 = PyTuple_New(3+__pyx_t_9); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 969, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          if (__pyx_t_2) {
            __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_2); __pyx_t_2 = NULL;
          }
          __Pyx_INCREF(((PyObject *)__pyx_v_iobuf));
          __Pyx_GIVEREF(((PyObject *)__pyx_v_iobuf));
          PyTuple_SET_ITEM(__pyx_t_10, 0+__pyx_t_9, ((PyObject *)__pyx_v_iobuf));
          __Pyx_GIVEREF(__pyx_t_4);
          PyTuple_SET_ITEM(__pyx_t_10, 1+__pyx_t_9, __pyx_t_4);
          __Pyx_INCREF(__pyx_int_1);
          __Pyx_GIVEREF(__pyx_int_1);
          PyTuple_SET_ITEM(__pyx_t_10, 2+__pyx_t_9, __pyx_int_1);
          __pyx_t_4 = 0;
          __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_10, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 969, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        }
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

        /* "tables/tableextension.pyx":968
 *         iobuf = iobuf[:recout]
 * 
 *         if len(iobuf) > 0:             # <<<<<<<<<<<<<<
 *           self.table._convert_types(iobuf, len(iobuf), 1)
 *         self.indexvalid = call_on_recarr(
 */
      }

      /* "tables/tableextension.pyx":970
 *         if len(iobuf) > 0:
 *           self.table._convert_types(iobuf, len(iobuf), 1)
 *         self.indexvalid = call_on_recarr(             # <<<<<<<<<<<<<<
 *           self.condfunc, self.condargs, iobuf, **self.condkwargs)
 *         self.index_valid_data = PyArray_BYTES(self.indexvalid)
 */
      __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_call_on_recarr); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 970, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);

      /* "tables/tableextension.pyx":971
 *           self.table._convert_types(iobuf, len(iobuf), 1)
 *         self.indexvalid = call_on_recarr(
 *           self.condfunc, self.condargs, iobuf, **self.condkwargs)             # <<<<<<<<<<<<<<
 *         self.index_valid_data = PyArray_BYTES(self.indexvalid)
 *         # Get the valid coordinates
 */
      __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 970, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_v_self->condfunc);
      __Pyx_GIVEREF(__pyx_v_self->condfunc);
      PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_self->condfunc);
      __Pyx_INCREF(__pyx_v_self->condargs);
      __Pyx_GIVEREF(__pyx_v_self->condargs);
      PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_v_self->condargs);
      __Pyx_INCREF(((PyObject *)__pyx_v_iobuf));
      __Pyx_GIVEREF(((PyObject *)__pyx_v_iobuf));
      PyTuple_SET_ITEM(__pyx_t_5, 2, ((PyObject *)__pyx_v_iobuf));

      /* "tables/tableextension.pyx":970
 *         if len(iobuf) > 0:
 *           self.table._convert_types(iobuf, len(iobuf), 1)
 *         self.indexvalid = call_on_recarr(             # <<<<<<<<<<<<<<
 *           self.condfunc, self.condargs, iobuf, **self.condkwargs)
 *         self.index_valid_data = PyArray_BYTES(self.indexvalid)
 */
      if (unlikely(__pyx_v_self->condkwargs == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "argument after ** must be a mapping, not NoneType");
        __PYX_ERR(0, 971, __pyx_L1_error)
      }
      if (likely(PyDict_CheckExact(__pyx_v_self->condkwargs))) {
        __pyx_t_10 = PyDict_Copy(__pyx_v_self->condkwargs); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 971, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
      } else {
        __pyx_t_10 = PyObject_CallFunctionObjArgs((PyObject*)&PyDict_Type, __pyx_v_self->condkwargs, NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 970, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
      }
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_5, __pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 970, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 970, __pyx_L1_error)
      __Pyx_GIVEREF(__pyx_t_4);
      __Pyx_GOTREF(__pyx_v_self->indexvalid);
      __Pyx_DECREF(((PyObject *)__pyx_v_self->indexvalid));
      __pyx_v_self->indexvalid = ((PyArrayObject *)__pyx_t_4);
      __pyx_t_4 = 0;

      /* "tables/tableextension.pyx":972
 *         self.indexvalid = call_on_recarr(
 *           self.condfunc, self.condargs, iobuf, **self.condkwargs)
 *         self.index_valid_data = PyArray_BYTES(self.indexvalid)             # <<<<<<<<<<<<<<
 *         # Get the valid coordinates
 *         self.indexvalues = self.bufcoords[:recout][self.indexvalid]
 */
      __pyx_t_4 = ((PyObject *)__pyx_v_self->indexvalid);
      __Pyx_INCREF(__pyx_t_4);
      __pyx_v_self->index_valid_data = PyArray_BYTES(((PyArrayObject *)__pyx_t_4));
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "tables/tableextension.pyx":974
 *         self.index_valid_data = PyArray_BYTES(self.indexvalid)
 *         # Get the valid coordinates
 *         self.indexvalues = self.bufcoords[:recout][self.indexvalid]             # <<<<<<<<<<<<<<
 *         self.index_values_data = <hsize_t *>PyArray_DATA(self.indexvalues)
 *         self.lenbuf = self.indexvalues.size
 */
      __pyx_t_4 = __Pyx_PyObject_GetSlice(((PyObject *)__pyx_v_self->bufcoords), 0, __pyx_v_recout, NULL, NULL, NULL, 0, 1, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 974, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_10 = __Pyx_PyObject_GetItem(__pyx_t_4, ((PyObject *)__pyx_v_self->indexvalid)); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 974, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (!(likely(((__pyx_t_10) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_10, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 974, __pyx_L1_error)
      __Pyx_GIVEREF(__pyx_t_10);
      __Pyx_GOTREF(__pyx_v_self->indexvalues);
      __Pyx_DECREF(((PyObject *)__pyx_v_self->indexvalues));
      __pyx_v_self->indexvalues = ((PyArrayObject *)__pyx_t_10);
      __pyx_t_10 = 0;

      /* "tables/tableextension.pyx":975
 *         # Get the valid coordinates
 *         self.indexvalues = self.bufcoords[:recout][self.indexvalid]
 *         self.index_values_data = <hsize_t *>PyArray_DATA(self.indexvalues)             # <<<<<<<<<<<<<<
 *         self.lenbuf = self.indexvalues.size
 *         # Place the valid results at the beginning of the buffer
 */
      __pyx_t_10 = ((PyObject *)__pyx_v_self->indexvalues);
      __Pyx_INCREF(__pyx_t_10);
      __pyx_v_self->index_values_data = ((hsize_t *)PyArray_DATA(((PyArrayObject *)__pyx_t_10)));
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

      /* "tables/tableextension.pyx":976
 *         self.indexvalues = self.bufcoords[:recout][self.indexvalid]
 *         self.index_values_data = <hsize_t *>PyArray_DATA(self.indexvalues)
 *         self.lenbuf = self.indexvalues.size             # <<<<<<<<<<<<<<
 *         # Place the valid results at the beginning of the buffer
 *         iobuf[:self.lenbuf] = iobuf[self.indexvalid]
 */
      __pyx_t_10 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->indexvalues), __pyx_n_s_size); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 976, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_3 = __Pyx_PyInt_As_hsize_t(__pyx_t_10); if (unlikely((__pyx_t_3 == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 976, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_v_self->lenbuf = __pyx_t_3;

      /* "tables/tableextension.pyx":978
 *         self.lenbuf = self.indexvalues.size
 *         # Place the valid results at the beginning of the buffer
 *         iobuf[:self.lenbuf] = iobuf[self.indexvalid]             # <<<<<<<<<<<<<<
 * 
 *         # Initialize the internal buffer row counter
 */
      __pyx_t_10 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_iobuf), ((PyObject *)__pyx_v_self->indexvalid)); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 978, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      if (__Pyx_PyObject_SetSlice(((PyObject *)__pyx_v_iobuf), __pyx_t_10, 0, __pyx_v_self->lenbuf, NULL, NULL, NULL, 0, 1, 1) < 0) __PYX_ERR(0, 978, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

      /* "tables/tableextension.pyx":981
 * 
 *         # Initialize the internal buffer row counter
 *         self._row = -1             # <<<<<<<<<<<<<<
 * 
 *         if self._write_to_seqcache:
 */
      __pyx_v_self->_row = -1L;

      /* "tables/tableextension.pyx":983
 *         self._row = -1
 * 
 *         if self._write_to_seqcache:             # <<<<<<<<<<<<<<
 *           # Feed the indexvalues into the seqcache
 *           seqcache = self.iterseq
 */
      __pyx_t_1 = (__pyx_v_self->_write_to_seqcache != 0);
      if (__pyx_t_1) {

        /* "tables/tableextension.pyx":985
 *         if self._write_to_seqcache:
 *           # Feed the indexvalues into the seqcache
 *           seqcache = self.iterseq             # <<<<<<<<<<<<<<
 *           if self.lenbuf + len(seqcache) < self.iterseq_max_elements:
 *             seqcache.extend(self.indexvalues)
 */
        __pyx_t_10 = __pyx_v_self->iterseq;
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_XDECREF_SET(__pyx_v_seqcache, __pyx_t_10);
        __pyx_t_10 = 0;

        /* "tables/tableextension.pyx":986
 *           # Feed the indexvalues into the seqcache
 *           seqcache = self.iterseq
 *           if self.lenbuf + len(seqcache) < self.iterseq_max_elements:             # <<<<<<<<<<<<<<
 *             seqcache.extend(self.indexvalues)
 *           else:
 */
        __pyx_t_8 = PyObject_Length(__pyx_v_seqcache); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(0, 986, __pyx_L1_error)
        __pyx_t_1 = (((__pyx_v_self->lenbuf + __pyx_t_8) < __pyx_v_self->iterseq_max_elements) != 0);
        if (__pyx_t_1) {

          /* "tables/tableextension.pyx":987
 *           seqcache = self.iterseq
 *           if self.lenbuf + len(seqcache) < self.iterseq_max_elements:
 *             seqcache.extend(self.indexvalues)             # <<<<<<<<<<<<<<
 *           else:
 *             self.iterseq = None
 */
          __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_seqcache, __pyx_n_s_extend); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 987, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_5 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
            __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
            if (likely(__pyx_t_5)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
              __Pyx_INCREF(__pyx_t_5);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_4, function);
            }
          }
          __pyx_t_10 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, ((PyObject *)__pyx_v_self->indexvalues)) : __Pyx_PyObject_CallOneArg(__pyx_t_4, ((PyObject *)__pyx_v_self->indexvalues));
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 987, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

          /* "tables/tableextension.pyx":986
 *           # Feed the indexvalues into the seqcache
 *           seqcache = self.iterseq
 *           if self.lenbuf + len(seqcache) < self.iterseq_max_elements:             # <<<<<<<<<<<<<<
 *             seqcache.extend(self.indexvalues)
 *           else:
 */
          goto __pyx_L14;
        }

        /* "tables/tableextension.pyx":989
 *             seqcache.extend(self.indexvalues)
 *           else:
 *             self.iterseq = None             # <<<<<<<<<<<<<<
 *             self._write_to_seqcache = 0
 * 
 */
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __Pyx_GIVEREF(Py_None);
          __Pyx_GOTREF(__pyx_v_self->iterseq);
          __Pyx_DECREF(__pyx_v_self->iterseq);
          __pyx_v_self->iterseq = Py_None;

          /* "tables/tableextension.pyx":990
 *           else:
 *             self.iterseq = None
 *             self._write_to_seqcache = 0             # <<<<<<<<<<<<<<
 * 
 *       self._row = self._row + 1
 */
          __pyx_v_self->_write_to_seqcache = 0;
        }
        __pyx_L14:;

        /* "tables/tableextension.pyx":983
 *         self._row = -1
 * 
 *         if self._write_to_seqcache:             # <<<<<<<<<<<<<<
 *           # Feed the indexvalues into the seqcache
 *           seqcache = self.iterseq
 */
      }

      /* "tables/tableextension.pyx":931
 *     assert self.nrowsinbuf >= self.chunksize
 *     while self.nextelement < self.stop:
 *       if self.nextelement >= self.nrowsread:             # <<<<<<<<<<<<<<
 *         # Skip until there is interesting information
 *         while self.start > self.nrowsread + self.nrowsinbuf:
 */
    }

    /* "tables/tableextension.pyx":992
 *             self._write_to_seqcache = 0
 * 
 *       self._row = self._row + 1             # <<<<<<<<<<<<<<
 *       # Check whether we have read all the rows in buf
 *       if self._row == self.lenbuf:
 */
    __pyx_v_self->_row = (__pyx_v_self->_row + 1);

    /* "tables/tableextension.pyx":994
 *       self._row = self._row + 1
 *       # Check whether we have read all the rows in buf
 *       if self._row == self.lenbuf:             # <<<<<<<<<<<<<<
 *         self.nextelement = self.nrowsread
 *         # Make _row to point to the last valid entry in buffer
 */
    __pyx_t_1 = ((__pyx_v_self->_row == __pyx_v_self->lenbuf) != 0);
    if (__pyx_t_1) {

      /* "tables/tableextension.pyx":995
 *       # Check whether we have read all the rows in buf
 *       if self._row == self.lenbuf:
 *         self.nextelement = self.nrowsread             # <<<<<<<<<<<<<<
 *         # Make _row to point to the last valid entry in buffer
 *         # (this is useful for accessing the last row after an iterator loop)
 */
      __pyx_t_3 = __pyx_v_self->nrowsread;
      __pyx_v_self->nextelement = __pyx_t_3;

      /* "tables/tableextension.pyx":998
 *         # Make _row to point to the last valid entry in buffer
 *         # (this is useful for accessing the last row after an iterator loop)
 *         self._row = self._row - 1             # <<<<<<<<<<<<<<
 *         continue
 *       self._nrow = self.index_values_data[self._row]
 */
      __pyx_v_self->_row = (__pyx_v_self->_row - 1);

      /* "tables/tableextension.pyx":999
 *         # (this is useful for accessing the last row after an iterator loop)
 *         self._row = self._row - 1
 *         continue             # <<<<<<<<<<<<<<
 *       self._nrow = self.index_values_data[self._row]
 *       # Check additional conditions on start, stop, step params
 */
      goto __pyx_L3_continue;

      /* "tables/tableextension.pyx":994
 *       self._row = self._row + 1
 *       # Check whether we have read all the rows in buf
 *       if self._row == self.lenbuf:             # <<<<<<<<<<<<<<
 *         self.nextelement = self.nrowsread
 *         # Make _row to point to the last valid entry in buffer
 */
    }

    /* "tables/tableextension.pyx":1000
 *         self._row = self._row - 1
 *         continue
 *       self._nrow = self.index_values_data[self._row]             # <<<<<<<<<<<<<<
 *       # Check additional conditions on start, stop, step params
 *       if self.sss_on:
 */
    __pyx_v_self->_nrow = (__pyx_v_self->index_values_data[__pyx_v_self->_row]);

    /* "tables/tableextension.pyx":1002
 *       self._nrow = self.index_values_data[self._row]
 *       # Check additional conditions on start, stop, step params
 *       if self.sss_on:             # <<<<<<<<<<<<<<
 *         if (self._nrow < self.start or self._nrow >= self.stop):
 *           self.nextelement = self.nextelement + 1
 */
    __pyx_t_1 = (__pyx_v_self->sss_on != 0);
    if (__pyx_t_1) {

      /* "tables/tableextension.pyx":1003
 *       # Check additional conditions on start, stop, step params
 *       if self.sss_on:
 *         if (self._nrow < self.start or self._nrow >= self.stop):             # <<<<<<<<<<<<<<
 *           self.nextelement = self.nextelement + 1
 *           continue
 */
      __pyx_t_11 = ((__pyx_v_self->_nrow < __pyx_v_self->start) != 0);
      if (!__pyx_t_11) {
      } else {
        __pyx_t_1 = __pyx_t_11;
        goto __pyx_L18_bool_binop_done;
      }
      __pyx_t_11 = ((__pyx_v_self->_nrow >= __pyx_v_self->stop) != 0);
      __pyx_t_1 = __pyx_t_11;
      __pyx_L18_bool_binop_done:;
      if (__pyx_t_1) {

        /* "tables/tableextension.pyx":1004
 *       if self.sss_on:
 *         if (self._nrow < self.start or self._nrow >= self.stop):
 *           self.nextelement = self.nextelement + 1             # <<<<<<<<<<<<<<
 *           continue
 *         if (self.step > 1 and
 */
        __pyx_v_self->nextelement = (__pyx_v_self->nextelement + 1);

        /* "tables/tableextension.pyx":1005
 *         if (self._nrow < self.start or self._nrow >= self.stop):
 *           self.nextelement = self.nextelement + 1
 *           continue             # <<<<<<<<<<<<<<
 *         if (self.step > 1 and
 *             ((self._nrow - self.start) % self.step > 0)):
 */
        goto __pyx_L3_continue;

        /* "tables/tableextension.pyx":1003
 *       # Check additional conditions on start, stop, step params
 *       if self.sss_on:
 *         if (self._nrow < self.start or self._nrow >= self.stop):             # <<<<<<<<<<<<<<
 *           self.nextelement = self.nextelement + 1
 *           continue
 */
      }

      /* "tables/tableextension.pyx":1006
 *           self.nextelement = self.nextelement + 1
 *           continue
 *         if (self.step > 1 and             # <<<<<<<<<<<<<<
 *             ((self._nrow - self.start) % self.step > 0)):
 *           self.nextelement = self.nextelement + 1
 */
      __pyx_t_11 = ((__pyx_v_self->step > 1) != 0);
      if (__pyx_t_11) {
      } else {
        __pyx_t_1 = __pyx_t_11;
        goto __pyx_L21_bool_binop_done;
      }

      /* "tables/tableextension.pyx":1007
 *           continue
 *         if (self.step > 1 and
 *             ((self._nrow - self.start) % self.step > 0)):             # <<<<<<<<<<<<<<
 *           self.nextelement = self.nextelement + 1
 *           continue
 */
      __pyx_t_3 = (__pyx_v_self->_nrow - __pyx_v_self->start);
      if (unlikely(__pyx_v_self->step == 0)) {
        PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
        __PYX_ERR(0, 1007, __pyx_L1_error)
      }
      __pyx_t_11 = (((__pyx_t_3 % __pyx_v_self->step) > 0) != 0);
      __pyx_t_1 = __pyx_t_11;
      __pyx_L21_bool_binop_done:;

      /* "tables/tableextension.pyx":1006
 *           self.nextelement = self.nextelement + 1
 *           continue
 *         if (self.step > 1 and             # <<<<<<<<<<<<<<
 *             ((self._nrow - self.start) % self.step > 0)):
 *           self.nextelement = self.nextelement + 1
 */
      if (__pyx_t_1) {

        /* "tables/tableextension.pyx":1008
 *         if (self.step > 1 and
 *             ((self._nrow - self.start) % self.step > 0)):
 *           self.nextelement = self.nextelement + 1             # <<<<<<<<<<<<<<
 *           continue
 *       # Return this row
 */
        __pyx_v_self->nextelement = (__pyx_v_self->nextelement + 1);

        /* "tables/tableextension.pyx":1009
 *             ((self._nrow - self.start) % self.step > 0)):
 *           self.nextelement = self.nextelement + 1
 *           continue             # <<<<<<<<<<<<<<
 *       # Return this row
 *       self.nextelement = self._nrow + 1
 */
        goto __pyx_L3_continue;

        /* "tables/tableextension.pyx":1006
 *           self.nextelement = self.nextelement + 1
 *           continue
 *         if (self.step > 1 and             # <<<<<<<<<<<<<<
 *             ((self._nrow - self.start) % self.step > 0)):
 *           self.nextelement = self.nextelement + 1
 */
      }

      /* "tables/tableextension.pyx":1002
 *       self._nrow = self.index_values_data[self._row]
 *       # Check additional conditions on start, stop, step params
 *       if self.sss_on:             # <<<<<<<<<<<<<<
 *         if (self._nrow < self.start or self._nrow >= self.stop):
 *           self.nextelement = self.nextelement + 1
 */
    }

    /* "tables/tableextension.pyx":1011
 *           continue
 *       # Return this row
 *       self.nextelement = self._nrow + 1             # <<<<<<<<<<<<<<
 *       return self
 *     else:
 */
    __pyx_v_self->nextelement = (__pyx_v_self->_nrow + 1);

    /* "tables/tableextension.pyx":1012
 *       # Return this row
 *       self.nextelement = self._nrow + 1
 *       return self             # <<<<<<<<<<<<<<
 *     else:
 *       # All the elements have been read for this mode
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __pyx_r = ((PyObject *)__pyx_v_self);
    goto __pyx_L0;
    __pyx_L3_continue:;
  }

  /* "tables/tableextension.pyx":1015
 *     else:
 *       # All the elements have been read for this mode
 *       self._finish_riterator()             # <<<<<<<<<<<<<<
 * 
 *   cdef __next__coords(self):
 */
  /*else*/ {
    __pyx_t_10 = ((struct __pyx_vtabstruct_6tables_14tableextension_Row *)__pyx_v_self->__pyx_vtab)->_finish_riterator(__pyx_v_self); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1015, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  }

  /* "tables/tableextension.pyx":916
 *       return self.__next__general()
 * 
 *   cdef __next__indexed(self):             # <<<<<<<<<<<<<<
 *     """The version of next() for indexed columns and a chunkmap."""
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("tables.tableextension.Row.__next__indexed", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_tmp_range);
  __Pyx_XDECREF((PyObject *)__pyx_v_table);
  __Pyx_XDECREF((PyObject *)__pyx_v_iobuf);
  __Pyx_XDECREF(__pyx_v_seqcache);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":1017
 *       self._finish_riterator()
 * 
 *   cdef __next__coords(self):             # <<<<<<<<<<<<<<
 *     """The version of next() for user-required coordinates"""
 *     cdef int recout
 */

static PyObject *__pyx_f_6tables_14tableextension_3Row___next__coords(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self) {
  int __pyx_v_recout;
  PY_LONG_LONG __pyx_v_lenbuf;
  PyObject *__pyx_v_tmp = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  hsize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  PY_LONG_LONG __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__next__coords", 0);

  /* "tables/tableextension.pyx":1022
 *     cdef long long lenbuf, nextelement
 *     cdef object tmp
 *     if 0 < self.step:             # <<<<<<<<<<<<<<
 *       while self.nextelement < self.stop:
 *         if self.nextelement >= self.nrowsread:
 */
  __pyx_t_1 = ((0 < __pyx_v_self->step) != 0);
  if (__pyx_t_1) {

    /* "tables/tableextension.pyx":1023
 *     cdef object tmp
 *     if 0 < self.step:
 *       while self.nextelement < self.stop:             # <<<<<<<<<<<<<<
 *         if self.nextelement >= self.nrowsread:
 *           # Correction for avoiding reading past self.stop
 */
    while (1) {
      __pyx_t_1 = ((__pyx_v_self->nextelement < __pyx_v_self->stop) != 0);
      if (!__pyx_t_1) break;

      /* "tables/tableextension.pyx":1024
 *     if 0 < self.step:
 *       while self.nextelement < self.stop:
 *         if self.nextelement >= self.nrowsread:             # <<<<<<<<<<<<<<
 *           # Correction for avoiding reading past self.stop
 *           if self.nrowsread+self.nrowsinbuf > self.stop:
 */
      __pyx_t_1 = ((__pyx_v_self->nextelement >= __pyx_v_self->nrowsread) != 0);
      if (__pyx_t_1) {

        /* "tables/tableextension.pyx":1026
 *         if self.nextelement >= self.nrowsread:
 *           # Correction for avoiding reading past self.stop
 *           if self.nrowsread+self.nrowsinbuf > self.stop:             # <<<<<<<<<<<<<<
 *             lenbuf = self.stop-self.nrowsread
 *           else:
 */
        __pyx_t_1 = (((__pyx_v_self->nrowsread + __pyx_v_self->nrowsinbuf) > __pyx_v_self->stop) != 0);
        if (__pyx_t_1) {

          /* "tables/tableextension.pyx":1027
 *           # Correction for avoiding reading past self.stop
 *           if self.nrowsread+self.nrowsinbuf > self.stop:
 *             lenbuf = self.stop-self.nrowsread             # <<<<<<<<<<<<<<
 *           else:
 *             lenbuf = self.nrowsinbuf
 */
          __pyx_v_lenbuf = (__pyx_v_self->stop - __pyx_v_self->nrowsread);

          /* "tables/tableextension.pyx":1026
 *         if self.nextelement >= self.nrowsread:
 *           # Correction for avoiding reading past self.stop
 *           if self.nrowsread+self.nrowsinbuf > self.stop:             # <<<<<<<<<<<<<<
 *             lenbuf = self.stop-self.nrowsread
 *           else:
 */
          goto __pyx_L7;
        }

        /* "tables/tableextension.pyx":1029
 *             lenbuf = self.stop-self.nrowsread
 *           else:
 *             lenbuf = self.nrowsinbuf             # <<<<<<<<<<<<<<
 *           tmp = self.coords[self.nrowsread:self.nrowsread+lenbuf:self.step]
 *           # We have to get a contiguous buffer, so numpy.array is the way to go
 */
        /*else*/ {
          __pyx_t_2 = __pyx_v_self->nrowsinbuf;
          __pyx_v_lenbuf = __pyx_t_2;
        }
        __pyx_L7:;

        /* "tables/tableextension.pyx":1030
 *           else:
 *             lenbuf = self.nrowsinbuf
 *           tmp = self.coords[self.nrowsread:self.nrowsread+lenbuf:self.step]             # <<<<<<<<<<<<<<
 *           # We have to get a contiguous buffer, so numpy.array is the way to go
 *           self.bufcoords = numpy.array(tmp, dtype="uint64")
 */
        __pyx_t_3 = __Pyx_PyInt_From_hsize_t(__pyx_v_self->nrowsread); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1030, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = __Pyx_PyInt_From_hsize_t((__pyx_v_self->nrowsread + __pyx_v_lenbuf)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1030, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_5 = __Pyx_PyInt_From_PY_LONG_LONG(__pyx_v_self->step); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1030, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_6 = PySlice_New(__pyx_t_3, __pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1030, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_5 = __Pyx_PyObject_GetItem(__pyx_v_self->coords, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1030, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF_SET(__pyx_v_tmp, __pyx_t_5);
        __pyx_t_5 = 0;

        /* "tables/tableextension.pyx":1032
 *           tmp = self.coords[self.nrowsread:self.nrowsread+lenbuf:self.step]
 *           # We have to get a contiguous buffer, so numpy.array is the way to go
 *           self.bufcoords = numpy.array(tmp, dtype="uint64")             # <<<<<<<<<<<<<<
 *           self._row = -1
 *           if self.bufcoords.size > 0:
 */
        __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_numpy); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1032, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_array); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1032, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1032, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_INCREF(__pyx_v_tmp);
        __Pyx_GIVEREF(__pyx_v_tmp);
        PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_tmp);
        __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1032, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_n_s_uint64) < 0) __PYX_ERR(0, 1032, __pyx_L1_error)
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_5, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1032, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1032, __pyx_L1_error)
        __Pyx_GIVEREF(__pyx_t_3);
        __Pyx_GOTREF(__pyx_v_self->bufcoords);
        __Pyx_DECREF(((PyObject *)__pyx_v_self->bufcoords));
        __pyx_v_self->bufcoords = ((PyArrayObject *)__pyx_t_3);
        __pyx_t_3 = 0;

        /* "tables/tableextension.pyx":1033
 *           # We have to get a contiguous buffer, so numpy.array is the way to go
 *           self.bufcoords = numpy.array(tmp, dtype="uint64")
 *           self._row = -1             # <<<<<<<<<<<<<<
 *           if self.bufcoords.size > 0:
 *             recout = self.table._read_elements(self.bufcoords, self.iobuf)
 */
        __pyx_v_self->_row = -1L;

        /* "tables/tableextension.pyx":1034
 *           self.bufcoords = numpy.array(tmp, dtype="uint64")
 *           self._row = -1
 *           if self.bufcoords.size > 0:             # <<<<<<<<<<<<<<
 *             recout = self.table._read_elements(self.bufcoords, self.iobuf)
 *           else:
 */
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->bufcoords), __pyx_n_s_size); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1034, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = PyObject_RichCompare(__pyx_t_3, __pyx_int_0, Py_GT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1034, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 1034, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (__pyx_t_1) {

          /* "tables/tableextension.pyx":1035
 *           self._row = -1
 *           if self.bufcoords.size > 0:
 *             recout = self.table._read_elements(self.bufcoords, self.iobuf)             # <<<<<<<<<<<<<<
 *           else:
 *             recout = 0
 */
          __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_table); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1035, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_read_elements); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1035, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __pyx_t_3 = NULL;
          __pyx_t_7 = 0;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
            __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
            if (likely(__pyx_t_3)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
              __Pyx_INCREF(__pyx_t_3);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_5, function);
              __pyx_t_7 = 1;
            }
          }
          #if CYTHON_FAST_PYCALL
          if (PyFunction_Check(__pyx_t_5)) {
            PyObject *__pyx_temp[3] = {__pyx_t_3, ((PyObject *)__pyx_v_self->bufcoords), __pyx_v_self->iobuf};
            __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1035, __pyx_L1_error)
            __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
            __Pyx_GOTREF(__pyx_t_4);
          } else
          #endif
          #if CYTHON_FAST_PYCCALL
          if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
            PyObject *__pyx_temp[3] = {__pyx_t_3, ((PyObject *)__pyx_v_self->bufcoords), __pyx_v_self->iobuf};
            __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1035, __pyx_L1_error)
            __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
            __Pyx_GOTREF(__pyx_t_4);
          } else
          #endif
          {
            __pyx_t_6 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1035, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_6);
            if (__pyx_t_3) {
              __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3); __pyx_t_3 = NULL;
            }
            __Pyx_INCREF(((PyObject *)__pyx_v_self->bufcoords));
            __Pyx_GIVEREF(((PyObject *)__pyx_v_self->bufcoords));
            PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_7, ((PyObject *)__pyx_v_self->bufcoords));
            __Pyx_INCREF(__pyx_v_self->iobuf);
            __Pyx_GIVEREF(__pyx_v_self->iobuf);
            PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_7, __pyx_v_self->iobuf);
            __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_6, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1035, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
            __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          }
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __pyx_t_7 = __Pyx_PyInt_As_int(__pyx_t_4); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1035, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __pyx_v_recout = __pyx_t_7;

          /* "tables/tableextension.pyx":1034
 *           self.bufcoords = numpy.array(tmp, dtype="uint64")
 *           self._row = -1
 *           if self.bufcoords.size > 0:             # <<<<<<<<<<<<<<
 *             recout = self.table._read_elements(self.bufcoords, self.iobuf)
 *           else:
 */
          goto __pyx_L8;
        }

        /* "tables/tableextension.pyx":1037
 *             recout = self.table._read_elements(self.bufcoords, self.iobuf)
 *           else:
 *             recout = 0             # <<<<<<<<<<<<<<
 *           self.bufcoords_data = <hsize_t*>PyArray_DATA(self.bufcoords)
 *           self.nrowsread = self.nrowsread + lenbuf
 */
        /*else*/ {
          __pyx_v_recout = 0;
        }
        __pyx_L8:;

        /* "tables/tableextension.pyx":1038
 *           else:
 *             recout = 0
 *           self.bufcoords_data = <hsize_t*>PyArray_DATA(self.bufcoords)             # <<<<<<<<<<<<<<
 *           self.nrowsread = self.nrowsread + lenbuf
 *           if recout == 0:
 */
        __pyx_t_4 = ((PyObject *)__pyx_v_self->bufcoords);
        __Pyx_INCREF(__pyx_t_4);
        __pyx_v_self->bufcoords_data = ((hsize_t *)PyArray_DATA(((PyArrayObject *)__pyx_t_4)));
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

        /* "tables/tableextension.pyx":1039
 *             recout = 0
 *           self.bufcoords_data = <hsize_t*>PyArray_DATA(self.bufcoords)
 *           self.nrowsread = self.nrowsread + lenbuf             # <<<<<<<<<<<<<<
 *           if recout == 0:
 *             # no items were read, skip out
 */
        __pyx_v_self->nrowsread = (__pyx_v_self->nrowsread + __pyx_v_lenbuf);

        /* "tables/tableextension.pyx":1040
 *           self.bufcoords_data = <hsize_t*>PyArray_DATA(self.bufcoords)
 *           self.nrowsread = self.nrowsread + lenbuf
 *           if recout == 0:             # <<<<<<<<<<<<<<
 *             # no items were read, skip out
 *             continue
 */
        __pyx_t_1 = ((__pyx_v_recout == 0) != 0);
        if (__pyx_t_1) {

          /* "tables/tableextension.pyx":1042
 *           if recout == 0:
 *             # no items were read, skip out
 *             continue             # <<<<<<<<<<<<<<
 *         self._row = self._row + 1
 *         self._nrow = self.bufcoords_data[self._row]
 */
          goto __pyx_L4_continue;

          /* "tables/tableextension.pyx":1040
 *           self.bufcoords_data = <hsize_t*>PyArray_DATA(self.bufcoords)
 *           self.nrowsread = self.nrowsread + lenbuf
 *           if recout == 0:             # <<<<<<<<<<<<<<
 *             # no items were read, skip out
 *             continue
 */
        }

        /* "tables/tableextension.pyx":1024
 *     if 0 < self.step:
 *       while self.nextelement < self.stop:
 *         if self.nextelement >= self.nrowsread:             # <<<<<<<<<<<<<<
 *           # Correction for avoiding reading past self.stop
 *           if self.nrowsread+self.nrowsinbuf > self.stop:
 */
      }

      /* "tables/tableextension.pyx":1043
 *             # no items were read, skip out
 *             continue
 *         self._row = self._row + 1             # <<<<<<<<<<<<<<
 *         self._nrow = self.bufcoords_data[self._row]
 *         self.nextelement = self.nextelement + self.absstep
 */
      __pyx_v_self->_row = (__pyx_v_self->_row + 1);

      /* "tables/tableextension.pyx":1044
 *             continue
 *         self._row = self._row + 1
 *         self._nrow = self.bufcoords_data[self._row]             # <<<<<<<<<<<<<<
 *         self.nextelement = self.nextelement + self.absstep
 *         return self
 */
      __pyx_v_self->_nrow = (__pyx_v_self->bufcoords_data[__pyx_v_self->_row]);

      /* "tables/tableextension.pyx":1045
 *         self._row = self._row + 1
 *         self._nrow = self.bufcoords_data[self._row]
 *         self.nextelement = self.nextelement + self.absstep             # <<<<<<<<<<<<<<
 *         return self
 *       else:
 */
      __pyx_v_self->nextelement = (__pyx_v_self->nextelement + __pyx_v_self->absstep);

      /* "tables/tableextension.pyx":1046
 *         self._nrow = self.bufcoords_data[self._row]
 *         self.nextelement = self.nextelement + self.absstep
 *         return self             # <<<<<<<<<<<<<<
 *       else:
 *         # All the elements have been read for this mode
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(((PyObject *)__pyx_v_self));
      __pyx_r = ((PyObject *)__pyx_v_self);
      goto __pyx_L0;
      __pyx_L4_continue:;
    }

    /* "tables/tableextension.pyx":1049
 *       else:
 *         # All the elements have been read for this mode
 *         self._finish_riterator()             # <<<<<<<<<<<<<<
 *     elif 0 > self.step:
 *       #print("self.nextelement = ", self.nextelement, self.start, self.nrowsread, self.nextelement <  self.start - self.nrowsread + 1)
 */
    /*else*/ {
      __pyx_t_4 = ((struct __pyx_vtabstruct_6tables_14tableextension_Row *)__pyx_v_self->__pyx_vtab)->_finish_riterator(__pyx_v_self); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1049, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }

    /* "tables/tableextension.pyx":1022
 *     cdef long long lenbuf, nextelement
 *     cdef object tmp
 *     if 0 < self.step:             # <<<<<<<<<<<<<<
 *       while self.nextelement < self.stop:
 *         if self.nextelement >= self.nrowsread:
 */
    goto __pyx_L3;
  }

  /* "tables/tableextension.pyx":1050
 *         # All the elements have been read for this mode
 *         self._finish_riterator()
 *     elif 0 > self.step:             # <<<<<<<<<<<<<<
 *       #print("self.nextelement = ", self.nextelement, self.start, self.nrowsread, self.nextelement <  self.start - self.nrowsread + 1)
 *       while self.nextelement > self.stop:
 */
  __pyx_t_1 = ((0 > __pyx_v_self->step) != 0);
  if (__pyx_t_1) {

    /* "tables/tableextension.pyx":1052
 *     elif 0 > self.step:
 *       #print("self.nextelement = ", self.nextelement, self.start, self.nrowsread, self.nextelement <  self.start - self.nrowsread + 1)
 *       while self.nextelement > self.stop:             # <<<<<<<<<<<<<<
 *         if self.nextelement < self.start - (<long long> self.nrowsread) + 1:
 *           if 0 > self.nextelement - (<long long> self.nrowsinbuf) + 1:
 */
    while (1) {
      __pyx_t_1 = ((__pyx_v_self->nextelement > __pyx_v_self->stop) != 0);
      if (!__pyx_t_1) break;

      /* "tables/tableextension.pyx":1053
 *       #print("self.nextelement = ", self.nextelement, self.start, self.nrowsread, self.nextelement <  self.start - self.nrowsread + 1)
 *       while self.nextelement > self.stop:
 *         if self.nextelement < self.start - (<long long> self.nrowsread) + 1:             # <<<<<<<<<<<<<<
 *           if 0 > self.nextelement - (<long long> self.nrowsinbuf) + 1:
 *             tmp = self.coords[0:self.nextelement + 1]
 */
      __pyx_t_1 = ((__pyx_v_self->nextelement < ((__pyx_v_self->start - ((PY_LONG_LONG)__pyx_v_self->nrowsread)) + 1)) != 0);
      if (__pyx_t_1) {

        /* "tables/tableextension.pyx":1054
 *       while self.nextelement > self.stop:
 *         if self.nextelement < self.start - (<long long> self.nrowsread) + 1:
 *           if 0 > self.nextelement - (<long long> self.nrowsinbuf) + 1:             # <<<<<<<<<<<<<<
 *             tmp = self.coords[0:self.nextelement + 1]
 *           else:
 */
        __pyx_t_1 = ((0 > ((__pyx_v_self->nextelement - ((PY_LONG_LONG)__pyx_v_self->nrowsinbuf)) + 1)) != 0);
        if (__pyx_t_1) {

          /* "tables/tableextension.pyx":1055
 *         if self.nextelement < self.start - (<long long> self.nrowsread) + 1:
 *           if 0 > self.nextelement - (<long long> self.nrowsinbuf) + 1:
 *             tmp = self.coords[0:self.nextelement + 1]             # <<<<<<<<<<<<<<
 *           else:
 *             tmp = self.coords[self.nextelement - (<long long> self.nrowsinbuf) + 1:self.nextelement + 1]
 */
          __pyx_t_4 = __Pyx_PyObject_GetSlice(__pyx_v_self->coords, 0, (__pyx_v_self->nextelement + 1), NULL, NULL, NULL, 1, 1, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1055, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_v_tmp = __pyx_t_4;
          __pyx_t_4 = 0;

          /* "tables/tableextension.pyx":1054
 *       while self.nextelement > self.stop:
 *         if self.nextelement < self.start - (<long long> self.nrowsread) + 1:
 *           if 0 > self.nextelement - (<long long> self.nrowsinbuf) + 1:             # <<<<<<<<<<<<<<
 *             tmp = self.coords[0:self.nextelement + 1]
 *           else:
 */
          goto __pyx_L13;
        }

        /* "tables/tableextension.pyx":1057
 *             tmp = self.coords[0:self.nextelement + 1]
 *           else:
 *             tmp = self.coords[self.nextelement - (<long long> self.nrowsinbuf) + 1:self.nextelement + 1]             # <<<<<<<<<<<<<<
 *           self.bufcoords = numpy.array(tmp, dtype="uint64")
 *           recout = self.table._read_elements(self.bufcoords, self.iobuf)
 */
        /*else*/ {
          __pyx_t_4 = __Pyx_PyObject_GetSlice(__pyx_v_self->coords, ((__pyx_v_self->nextelement - ((PY_LONG_LONG)__pyx_v_self->nrowsinbuf)) + 1), (__pyx_v_self->nextelement + 1), NULL, NULL, NULL, 1, 1, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1057, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_v_tmp = __pyx_t_4;
          __pyx_t_4 = 0;
        }
        __pyx_L13:;

        /* "tables/tableextension.pyx":1058
 *           else:
 *             tmp = self.coords[self.nextelement - (<long long> self.nrowsinbuf) + 1:self.nextelement + 1]
 *           self.bufcoords = numpy.array(tmp, dtype="uint64")             # <<<<<<<<<<<<<<
 *           recout = self.table._read_elements(self.bufcoords, self.iobuf)
 *           self.bufcoords_data = <hsize_t*>PyArray_DATA(self.bufcoords)
 */
        __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_numpy); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1058, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_array); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1058, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1058, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_INCREF(__pyx_v_tmp);
        __Pyx_GIVEREF(__pyx_v_tmp);
        PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_tmp);
        __pyx_t_6 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1058, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_dtype, __pyx_n_s_uint64) < 0) __PYX_ERR(0, 1058, __pyx_L1_error)
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_4, __pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1058, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1058, __pyx_L1_error)
        __Pyx_GIVEREF(__pyx_t_3);
        __Pyx_GOTREF(__pyx_v_self->bufcoords);
        __Pyx_DECREF(((PyObject *)__pyx_v_self->bufcoords));
        __pyx_v_self->bufcoords = ((PyArrayObject *)__pyx_t_3);
        __pyx_t_3 = 0;

        /* "tables/tableextension.pyx":1059
 *             tmp = self.coords[self.nextelement - (<long long> self.nrowsinbuf) + 1:self.nextelement + 1]
 *           self.bufcoords = numpy.array(tmp, dtype="uint64")
 *           recout = self.table._read_elements(self.bufcoords, self.iobuf)             # <<<<<<<<<<<<<<
 *           self.bufcoords_data = <hsize_t*>PyArray_DATA(self.bufcoords)
 *           self.nrowsread = self.nrowsread + self.nrowsinbuf
 */
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_table); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1059, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_read_elements); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1059, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_t_6 = NULL;
        __pyx_t_7 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_6)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_6);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
            __pyx_t_7 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[3] = {__pyx_t_6, ((PyObject *)__pyx_v_self->bufcoords), __pyx_v_self->iobuf};
          __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1059, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_3);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[3] = {__pyx_t_6, ((PyObject *)__pyx_v_self->bufcoords), __pyx_v_self->iobuf};
          __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1059, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_3);
        } else
        #endif
        {
          __pyx_t_5 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1059, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          if (__pyx_t_6) {
            __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_6); __pyx_t_6 = NULL;
          }
          __Pyx_INCREF(((PyObject *)__pyx_v_self->bufcoords));
          __Pyx_GIVEREF(((PyObject *)__pyx_v_self->bufcoords));
          PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_7, ((PyObject *)__pyx_v_self->bufcoords));
          __Pyx_INCREF(__pyx_v_self->iobuf);
          __Pyx_GIVEREF(__pyx_v_self->iobuf);
          PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_7, __pyx_v_self->iobuf);
          __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_5, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1059, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        }
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_7 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1059, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_v_recout = __pyx_t_7;

        /* "tables/tableextension.pyx":1060
 *           self.bufcoords = numpy.array(tmp, dtype="uint64")
 *           recout = self.table._read_elements(self.bufcoords, self.iobuf)
 *           self.bufcoords_data = <hsize_t*>PyArray_DATA(self.bufcoords)             # <<<<<<<<<<<<<<
 *           self.nrowsread = self.nrowsread + self.nrowsinbuf
 *           self._row = len(self.bufcoords) - 1
 */
        __pyx_t_3 = ((PyObject *)__pyx_v_self->bufcoords);
        __Pyx_INCREF(__pyx_t_3);
        __pyx_v_self->bufcoords_data = ((hsize_t *)PyArray_DATA(((PyArrayObject *)__pyx_t_3)));
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

        /* "tables/tableextension.pyx":1061
 *           recout = self.table._read_elements(self.bufcoords, self.iobuf)
 *           self.bufcoords_data = <hsize_t*>PyArray_DATA(self.bufcoords)
 *           self.nrowsread = self.nrowsread + self.nrowsinbuf             # <<<<<<<<<<<<<<
 *           self._row = len(self.bufcoords) - 1
 *         else:
 */
        __pyx_v_self->nrowsread = (__pyx_v_self->nrowsread + __pyx_v_self->nrowsinbuf);

        /* "tables/tableextension.pyx":1062
 *           self.bufcoords_data = <hsize_t*>PyArray_DATA(self.bufcoords)
 *           self.nrowsread = self.nrowsread + self.nrowsinbuf
 *           self._row = len(self.bufcoords) - 1             # <<<<<<<<<<<<<<
 *         else:
 *           self._row = (self._row + self.step) % len(self.bufcoords)
 */
        __pyx_t_3 = ((PyObject *)__pyx_v_self->bufcoords);
        __Pyx_INCREF(__pyx_t_3);
        __pyx_t_8 = PyObject_Length(__pyx_t_3); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1062, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_v_self->_row = (__pyx_t_8 - 1);

        /* "tables/tableextension.pyx":1053
 *       #print("self.nextelement = ", self.nextelement, self.start, self.nrowsread, self.nextelement <  self.start - self.nrowsread + 1)
 *       while self.nextelement > self.stop:
 *         if self.nextelement < self.start - (<long long> self.nrowsread) + 1:             # <<<<<<<<<<<<<<
 *           if 0 > self.nextelement - (<long long> self.nrowsinbuf) + 1:
 *             tmp = self.coords[0:self.nextelement + 1]
 */
        goto __pyx_L12;
      }

      /* "tables/tableextension.pyx":1064
 *           self._row = len(self.bufcoords) - 1
 *         else:
 *           self._row = (self._row + self.step) % len(self.bufcoords)             # <<<<<<<<<<<<<<
 * 
 *         self._nrow = self.nextelement - self.step
 */
      /*else*/ {
        __pyx_t_9 = (__pyx_v_self->_row + __pyx_v_self->step);
        __pyx_t_3 = ((PyObject *)__pyx_v_self->bufcoords);
        __Pyx_INCREF(__pyx_t_3);
        __pyx_t_8 = PyObject_Length(__pyx_t_3); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1064, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(__pyx_t_8 == 0)) {
          PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
          __PYX_ERR(0, 1064, __pyx_L1_error)
        }
        __pyx_v_self->_row = __Pyx_mod_PY_LONG_LONG(__pyx_t_9, __pyx_t_8);
      }
      __pyx_L12:;

      /* "tables/tableextension.pyx":1066
 *           self._row = (self._row + self.step) % len(self.bufcoords)
 * 
 *         self._nrow = self.nextelement - self.step             # <<<<<<<<<<<<<<
 *         self.nextelement = self.nextelement + self.step
 *         # Return this value
 */
      __pyx_v_self->_nrow = (__pyx_v_self->nextelement - __pyx_v_self->step);

      /* "tables/tableextension.pyx":1067
 * 
 *         self._nrow = self.nextelement - self.step
 *         self.nextelement = self.nextelement + self.step             # <<<<<<<<<<<<<<
 *         # Return this value
 *         return self
 */
      __pyx_v_self->nextelement = (__pyx_v_self->nextelement + __pyx_v_self->step);

      /* "tables/tableextension.pyx":1069
 *         self.nextelement = self.nextelement + self.step
 *         # Return this value
 *         return self             # <<<<<<<<<<<<<<
 *       else:
 *         # All the elements have been read for this mode
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(((PyObject *)__pyx_v_self));
      __pyx_r = ((PyObject *)__pyx_v_self);
      goto __pyx_L0;
    }

    /* "tables/tableextension.pyx":1072
 *       else:
 *         # All the elements have been read for this mode
 *         self._finish_riterator()             # <<<<<<<<<<<<<<
 *     else:
 *       self._finish_riterator()
 */
    /*else*/ {
      __pyx_t_3 = ((struct __pyx_vtabstruct_6tables_14tableextension_Row *)__pyx_v_self->__pyx_vtab)->_finish_riterator(__pyx_v_self); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1072, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }

    /* "tables/tableextension.pyx":1050
 *         # All the elements have been read for this mode
 *         self._finish_riterator()
 *     elif 0 > self.step:             # <<<<<<<<<<<<<<
 *       #print("self.nextelement = ", self.nextelement, self.start, self.nrowsread, self.nextelement <  self.start - self.nrowsread + 1)
 *       while self.nextelement > self.stop:
 */
    goto __pyx_L3;
  }

  /* "tables/tableextension.pyx":1074
 *         self._finish_riterator()
 *     else:
 *       self._finish_riterator()             # <<<<<<<<<<<<<<
 * 
 *   cdef __next__inkernel(self):
 */
  /*else*/ {
    __pyx_t_3 = ((struct __pyx_vtabstruct_6tables_14tableextension_Row *)__pyx_v_self->__pyx_vtab)->_finish_riterator(__pyx_v_self); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1074, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __pyx_L3:;

  /* "tables/tableextension.pyx":1017
 *       self._finish_riterator()
 * 
 *   cdef __next__coords(self):             # <<<<<<<<<<<<<<
 *     """The version of next() for user-required coordinates"""
 *     cdef int recout
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("tables.tableextension.Row.__next__coords", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_tmp);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":1076
 *       self._finish_riterator()
 * 
 *   cdef __next__inkernel(self):             # <<<<<<<<<<<<<<
 *     """The version of next() in case of in-kernel conditions"""
 * 
 */

static PyObject *__pyx_f_6tables_14tableextension_3Row___next__inkernel(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self) {
  hsize_t __pyx_v_recout;
  hsize_t __pyx_v_correct;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  hsize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  PY_LONG_LONG __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__next__inkernel", 0);

  /* "tables/tableextension.pyx":1081
 *     cdef hsize_t recout, correct
 *     cdef object numexpr_locals, colvar, col
 *     self.nextelement = self._nrow + self.step             # <<<<<<<<<<<<<<
 *     while self.nextelement < self.stop:
 *       if self.nextelement >= self.nrowsread:
 */
  __pyx_v_self->nextelement = (__pyx_v_self->_nrow + __pyx_v_self->step);

  /* "tables/tableextension.pyx":1082
 *     cdef object numexpr_locals, colvar, col
 *     self.nextelement = self._nrow + self.step
 *     while self.nextelement < self.stop:             # <<<<<<<<<<<<<<
 *       if self.nextelement >= self.nrowsread:
 *         # Skip until there is interesting information
 */
  while (1) {
    __pyx_t_1 = ((__pyx_v_self->nextelement < __pyx_v_self->stop) != 0);
    if (!__pyx_t_1) break;

    /* "tables/tableextension.pyx":1083
 *     self.nextelement = self._nrow + self.step
 *     while self.nextelement < self.stop:
 *       if self.nextelement >= self.nrowsread:             # <<<<<<<<<<<<<<
 *         # Skip until there is interesting information
 *         while self.nextelement >= self.nrowsread + self.nrowsinbuf:
 */
    __pyx_t_1 = ((__pyx_v_self->nextelement >= __pyx_v_self->nrowsread) != 0);
    if (__pyx_t_1) {

      /* "tables/tableextension.pyx":1085
 *       if self.nextelement >= self.nrowsread:
 *         # Skip until there is interesting information
 *         while self.nextelement >= self.nrowsread + self.nrowsinbuf:             # <<<<<<<<<<<<<<
 *           self.nrowsread = self.nrowsread + self.nrowsinbuf
 *         # Compute the end for this iteration
 */
      while (1) {
        __pyx_t_1 = ((__pyx_v_self->nextelement >= (__pyx_v_self->nrowsread + __pyx_v_self->nrowsinbuf)) != 0);
        if (!__pyx_t_1) break;

        /* "tables/tableextension.pyx":1086
 *         # Skip until there is interesting information
 *         while self.nextelement >= self.nrowsread + self.nrowsinbuf:
 *           self.nrowsread = self.nrowsread + self.nrowsinbuf             # <<<<<<<<<<<<<<
 *         # Compute the end for this iteration
 *         self.stopb = self.stop - self.nrowsread
 */
        __pyx_v_self->nrowsread = (__pyx_v_self->nrowsread + __pyx_v_self->nrowsinbuf);
      }

      /* "tables/tableextension.pyx":1088
 *           self.nrowsread = self.nrowsread + self.nrowsinbuf
 *         # Compute the end for this iteration
 *         self.stopb = self.stop - self.nrowsread             # <<<<<<<<<<<<<<
 *         if self.stopb > self.nrowsinbuf:
 *           self.stopb = self.nrowsinbuf
 */
      __pyx_v_self->stopb = (__pyx_v_self->stop - __pyx_v_self->nrowsread);

      /* "tables/tableextension.pyx":1089
 *         # Compute the end for this iteration
 *         self.stopb = self.stop - self.nrowsread
 *         if self.stopb > self.nrowsinbuf:             # <<<<<<<<<<<<<<
 *           self.stopb = self.nrowsinbuf
 *         self._row = self.startb - self.step
 */
      __pyx_t_1 = ((__pyx_v_self->stopb > __pyx_v_self->nrowsinbuf) != 0);
      if (__pyx_t_1) {

        /* "tables/tableextension.pyx":1090
 *         self.stopb = self.stop - self.nrowsread
 *         if self.stopb > self.nrowsinbuf:
 *           self.stopb = self.nrowsinbuf             # <<<<<<<<<<<<<<
 *         self._row = self.startb - self.step
 *         # Read a chunk
 */
        __pyx_t_2 = __pyx_v_self->nrowsinbuf;
        __pyx_v_self->stopb = __pyx_t_2;

        /* "tables/tableextension.pyx":1089
 *         # Compute the end for this iteration
 *         self.stopb = self.stop - self.nrowsread
 *         if self.stopb > self.nrowsinbuf:             # <<<<<<<<<<<<<<
 *           self.stopb = self.nrowsinbuf
 *         self._row = self.startb - self.step
 */
      }

      /* "tables/tableextension.pyx":1091
 *         if self.stopb > self.nrowsinbuf:
 *           self.stopb = self.nrowsinbuf
 *         self._row = self.startb - self.step             # <<<<<<<<<<<<<<
 *         # Read a chunk
 *         recout = self.table._read_records(self.nextelement, self.nrowsinbuf,
 */
      __pyx_v_self->_row = (__pyx_v_self->startb - __pyx_v_self->step);

      /* "tables/tableextension.pyx":1093
 *         self._row = self.startb - self.step
 *         # Read a chunk
 *         recout = self.table._read_records(self.nextelement, self.nrowsinbuf,             # <<<<<<<<<<<<<<
 *                                           self.iobuf)
 *         self.nrowsread = self.nrowsread + recout
 */
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_table); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1093, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_read_records); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1093, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyInt_From_PY_LONG_LONG(__pyx_v_self->nextelement); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1093, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_6 = __Pyx_PyInt_From_hsize_t(__pyx_v_self->nrowsinbuf); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1093, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);

      /* "tables/tableextension.pyx":1094
 *         # Read a chunk
 *         recout = self.table._read_records(self.nextelement, self.nrowsinbuf,
 *                                           self.iobuf)             # <<<<<<<<<<<<<<
 *         self.nrowsread = self.nrowsread + recout
 *         self.indexchunk = -self.step
 */
      __pyx_t_7 = NULL;
      __pyx_t_8 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
          __pyx_t_8 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_t_4, __pyx_t_6, __pyx_v_self->iobuf};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1093, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_t_4, __pyx_t_6, __pyx_v_self->iobuf};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1093, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else
      #endif
      {
        __pyx_t_9 = PyTuple_New(3+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1093, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        if (__pyx_t_7) {
          __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
        }
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_8, __pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_6);
        PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_8, __pyx_t_6);
        __Pyx_INCREF(__pyx_v_self->iobuf);
        __Pyx_GIVEREF(__pyx_v_self->iobuf);
        PyTuple_SET_ITEM(__pyx_t_9, 2+__pyx_t_8, __pyx_v_self->iobuf);
        __pyx_t_4 = 0;
        __pyx_t_6 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_9, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1093, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      }
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "tables/tableextension.pyx":1093
 *         self._row = self.startb - self.step
 *         # Read a chunk
 *         recout = self.table._read_records(self.nextelement, self.nrowsinbuf,             # <<<<<<<<<<<<<<
 *                                           self.iobuf)
 *         self.nrowsread = self.nrowsread + recout
 */
      __pyx_t_2 = __Pyx_PyInt_As_hsize_t(__pyx_t_3); if (unlikely((__pyx_t_2 == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1093, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_v_recout = __pyx_t_2;

      /* "tables/tableextension.pyx":1095
 *         recout = self.table._read_records(self.nextelement, self.nrowsinbuf,
 *                                           self.iobuf)
 *         self.nrowsread = self.nrowsread + recout             # <<<<<<<<<<<<<<
 *         self.indexchunk = -self.step
 * 
 */
      __pyx_v_self->nrowsread = (__pyx_v_self->nrowsread + __pyx_v_recout);

      /* "tables/tableextension.pyx":1096
 *                                           self.iobuf)
 *         self.nrowsread = self.nrowsread + recout
 *         self.indexchunk = -self.step             # <<<<<<<<<<<<<<
 * 
 *         # Evaluate the condition on this table fragment.
 */
      __pyx_v_self->indexchunk = (-__pyx_v_self->step);

      /* "tables/tableextension.pyx":1099
 * 
 *         # Evaluate the condition on this table fragment.
 *         self.indexvalid = call_on_recarr(             # <<<<<<<<<<<<<<
 *           self.condfunc, self.condargs, self.iobuf[:recout], **self.condkwargs)
 *         self.index_valid_data = PyArray_BYTES(self.indexvalid)
 */
      __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_call_on_recarr); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1099, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);

      /* "tables/tableextension.pyx":1100
 *         # Evaluate the condition on this table fragment.
 *         self.indexvalid = call_on_recarr(
 *           self.condfunc, self.condargs, self.iobuf[:recout], **self.condkwargs)             # <<<<<<<<<<<<<<
 *         self.index_valid_data = PyArray_BYTES(self.indexvalid)
 * 
 */
      __pyx_t_5 = __Pyx_PyObject_GetSlice(__pyx_v_self->iobuf, 0, __pyx_v_recout, NULL, NULL, NULL, 0, 1, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1100, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);

      /* "tables/tableextension.pyx":1099
 * 
 *         # Evaluate the condition on this table fragment.
 *         self.indexvalid = call_on_recarr(             # <<<<<<<<<<<<<<
 *           self.condfunc, self.condargs, self.iobuf[:recout], **self.condkwargs)
 *         self.index_valid_data = PyArray_BYTES(self.indexvalid)
 */
      __pyx_t_9 = PyTuple_New(3); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1099, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(__pyx_v_self->condfunc);
      __Pyx_GIVEREF(__pyx_v_self->condfunc);
      PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_v_self->condfunc);
      __Pyx_INCREF(__pyx_v_self->condargs);
      __Pyx_GIVEREF(__pyx_v_self->condargs);
      PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_self->condargs);
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_9, 2, __pyx_t_5);
      __pyx_t_5 = 0;

      /* "tables/tableextension.pyx":1100
 *         # Evaluate the condition on this table fragment.
 *         self.indexvalid = call_on_recarr(
 *           self.condfunc, self.condargs, self.iobuf[:recout], **self.condkwargs)             # <<<<<<<<<<<<<<
 *         self.index_valid_data = PyArray_BYTES(self.indexvalid)
 * 
 */
      if (unlikely(__pyx_v_self->condkwargs == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "argument after ** must be a mapping, not NoneType");
        __PYX_ERR(0, 1100, __pyx_L1_error)
      }
      if (likely(PyDict_CheckExact(__pyx_v_self->condkwargs))) {
        __pyx_t_5 = PyDict_Copy(__pyx_v_self->condkwargs); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1100, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
      } else {
        __pyx_t_5 = PyObject_CallFunctionObjArgs((PyObject*)&PyDict_Type, __pyx_v_self->condkwargs, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1099, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
      }

      /* "tables/tableextension.pyx":1099
 * 
 *         # Evaluate the condition on this table fragment.
 *         self.indexvalid = call_on_recarr(             # <<<<<<<<<<<<<<
 *           self.condfunc, self.condargs, self.iobuf[:recout], **self.condkwargs)
 *         self.index_valid_data = PyArray_BYTES(self.indexvalid)
 */
      __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_9, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1099, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1099, __pyx_L1_error)
      __Pyx_GIVEREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_v_self->indexvalid);
      __Pyx_DECREF(((PyObject *)__pyx_v_self->indexvalid));
      __pyx_v_self->indexvalid = ((PyArrayObject *)__pyx_t_6);
      __pyx_t_6 = 0;

      /* "tables/tableextension.pyx":1101
 *         self.indexvalid = call_on_recarr(
 *           self.condfunc, self.condargs, self.iobuf[:recout], **self.condkwargs)
 *         self.index_valid_data = PyArray_BYTES(self.indexvalid)             # <<<<<<<<<<<<<<
 * 
 *         # Is there any interesting information in this buffer?
 */
      __pyx_t_6 = ((PyObject *)__pyx_v_self->indexvalid);
      __Pyx_INCREF(__pyx_t_6);
      __pyx_v_self->index_valid_data = PyArray_BYTES(((PyArrayObject *)__pyx_t_6));
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "tables/tableextension.pyx":1104
 * 
 *         # Is there any interesting information in this buffer?
 *         if not numpy.sometrue(self.indexvalid):             # <<<<<<<<<<<<<<
 *           # No, so take the next one
 *           if self.step >= self.nrowsinbuf:
 */
      __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_numpy); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1104, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_sometrue); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1104, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_9))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_9);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_9, function);
        }
      }
      __pyx_t_6 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_5, ((PyObject *)__pyx_v_self->indexvalid)) : __Pyx_PyObject_CallOneArg(__pyx_t_9, ((PyObject *)__pyx_v_self->indexvalid));
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1104, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 1104, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_10 = ((!__pyx_t_1) != 0);
      if (__pyx_t_10) {

        /* "tables/tableextension.pyx":1106
 *         if not numpy.sometrue(self.indexvalid):
 *           # No, so take the next one
 *           if self.step >= self.nrowsinbuf:             # <<<<<<<<<<<<<<
 *             self.nextelement = self.nextelement + self.step
 *           else:
 */
        __pyx_t_10 = ((__pyx_v_self->step >= __pyx_v_self->nrowsinbuf) != 0);
        if (__pyx_t_10) {

          /* "tables/tableextension.pyx":1107
 *           # No, so take the next one
 *           if self.step >= self.nrowsinbuf:
 *             self.nextelement = self.nextelement + self.step             # <<<<<<<<<<<<<<
 *           else:
 *             self.nextelement = self.nextelement + self.nrowsinbuf
 */
          __pyx_v_self->nextelement = (__pyx_v_self->nextelement + __pyx_v_self->step);

          /* "tables/tableextension.pyx":1106
 *         if not numpy.sometrue(self.indexvalid):
 *           # No, so take the next one
 *           if self.step >= self.nrowsinbuf:             # <<<<<<<<<<<<<<
 *             self.nextelement = self.nextelement + self.step
 *           else:
 */
          goto __pyx_L10;
        }

        /* "tables/tableextension.pyx":1109
 *             self.nextelement = self.nextelement + self.step
 *           else:
 *             self.nextelement = self.nextelement + self.nrowsinbuf             # <<<<<<<<<<<<<<
 *             # Correction for step size > 1
 *             if self.step > 1:
 */
        /*else*/ {
          __pyx_v_self->nextelement = (__pyx_v_self->nextelement + __pyx_v_self->nrowsinbuf);

          /* "tables/tableextension.pyx":1111
 *             self.nextelement = self.nextelement + self.nrowsinbuf
 *             # Correction for step size > 1
 *             if self.step > 1:             # <<<<<<<<<<<<<<
 *               correct = (self.nextelement - self.start) % self.step
 *               self.nextelement = self.nextelement - correct
 */
          __pyx_t_10 = ((__pyx_v_self->step > 1) != 0);
          if (__pyx_t_10) {

            /* "tables/tableextension.pyx":1112
 *             # Correction for step size > 1
 *             if self.step > 1:
 *               correct = (self.nextelement - self.start) % self.step             # <<<<<<<<<<<<<<
 *               self.nextelement = self.nextelement - correct
 *           continue
 */
            __pyx_t_2 = (__pyx_v_self->nextelement - __pyx_v_self->start);
            if (unlikely(__pyx_v_self->step == 0)) {
              PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
              __PYX_ERR(0, 1112, __pyx_L1_error)
            }
            __pyx_v_correct = (__pyx_t_2 % __pyx_v_self->step);

            /* "tables/tableextension.pyx":1113
 *             if self.step > 1:
 *               correct = (self.nextelement - self.start) % self.step
 *               self.nextelement = self.nextelement - correct             # <<<<<<<<<<<<<<
 *           continue
 * 
 */
            __pyx_v_self->nextelement = (__pyx_v_self->nextelement - __pyx_v_correct);

            /* "tables/tableextension.pyx":1111
 *             self.nextelement = self.nextelement + self.nrowsinbuf
 *             # Correction for step size > 1
 *             if self.step > 1:             # <<<<<<<<<<<<<<
 *               correct = (self.nextelement - self.start) % self.step
 *               self.nextelement = self.nextelement - correct
 */
          }
        }
        __pyx_L10:;

        /* "tables/tableextension.pyx":1114
 *               correct = (self.nextelement - self.start) % self.step
 *               self.nextelement = self.nextelement - correct
 *           continue             # <<<<<<<<<<<<<<
 * 
 *       self._row = self._row + self.step
 */
        goto __pyx_L3_continue;

        /* "tables/tableextension.pyx":1104
 * 
 *         # Is there any interesting information in this buffer?
 *         if not numpy.sometrue(self.indexvalid):             # <<<<<<<<<<<<<<
 *           # No, so take the next one
 *           if self.step >= self.nrowsinbuf:
 */
      }

      /* "tables/tableextension.pyx":1083
 *     self.nextelement = self._nrow + self.step
 *     while self.nextelement < self.stop:
 *       if self.nextelement >= self.nrowsread:             # <<<<<<<<<<<<<<
 *         # Skip until there is interesting information
 *         while self.nextelement >= self.nrowsread + self.nrowsinbuf:
 */
    }

    /* "tables/tableextension.pyx":1116
 *           continue
 * 
 *       self._row = self._row + self.step             # <<<<<<<<<<<<<<
 *       self._nrow = self.nextelement
 *       if self._row + self.step >= self.stopb:
 */
    __pyx_v_self->_row = (__pyx_v_self->_row + __pyx_v_self->step);

    /* "tables/tableextension.pyx":1117
 * 
 *       self._row = self._row + self.step
 *       self._nrow = self.nextelement             # <<<<<<<<<<<<<<
 *       if self._row + self.step >= self.stopb:
 *         # Compute the start row for the next buffer
 */
    __pyx_t_11 = __pyx_v_self->nextelement;
    __pyx_v_self->_nrow = __pyx_t_11;

    /* "tables/tableextension.pyx":1118
 *       self._row = self._row + self.step
 *       self._nrow = self.nextelement
 *       if self._row + self.step >= self.stopb:             # <<<<<<<<<<<<<<
 *         # Compute the start row for the next buffer
 *         self.startb = 0
 */
    __pyx_t_10 = (((__pyx_v_self->_row + __pyx_v_self->step) >= __pyx_v_self->stopb) != 0);
    if (__pyx_t_10) {

      /* "tables/tableextension.pyx":1120
 *       if self._row + self.step >= self.stopb:
 *         # Compute the start row for the next buffer
 *         self.startb = 0             # <<<<<<<<<<<<<<
 * 
 *       self.nextelement = self._nrow + self.step
 */
      __pyx_v_self->startb = 0;

      /* "tables/tableextension.pyx":1118
 *       self._row = self._row + self.step
 *       self._nrow = self.nextelement
 *       if self._row + self.step >= self.stopb:             # <<<<<<<<<<<<<<
 *         # Compute the start row for the next buffer
 *         self.startb = 0
 */
    }

    /* "tables/tableextension.pyx":1122
 *         self.startb = 0
 * 
 *       self.nextelement = self._nrow + self.step             # <<<<<<<<<<<<<<
 *       # Return only if this value is interesting
 *       self.indexchunk = self.indexchunk + self.step
 */
    __pyx_v_self->nextelement = (__pyx_v_self->_nrow + __pyx_v_self->step);

    /* "tables/tableextension.pyx":1124
 *       self.nextelement = self._nrow + self.step
 *       # Return only if this value is interesting
 *       self.indexchunk = self.indexchunk + self.step             # <<<<<<<<<<<<<<
 *       if self.index_valid_data[self.indexchunk]:
 *         return self
 */
    __pyx_v_self->indexchunk = (__pyx_v_self->indexchunk + __pyx_v_self->step);

    /* "tables/tableextension.pyx":1125
 *       # Return only if this value is interesting
 *       self.indexchunk = self.indexchunk + self.step
 *       if self.index_valid_data[self.indexchunk]:             # <<<<<<<<<<<<<<
 *         return self
 *     else:
 */
    __pyx_t_10 = ((__pyx_v_self->index_valid_data[__pyx_v_self->indexchunk]) != 0);
    if (__pyx_t_10) {

      /* "tables/tableextension.pyx":1126
 *       self.indexchunk = self.indexchunk + self.step
 *       if self.index_valid_data[self.indexchunk]:
 *         return self             # <<<<<<<<<<<<<<
 *     else:
 *       self._finish_riterator()
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(((PyObject *)__pyx_v_self));
      __pyx_r = ((PyObject *)__pyx_v_self);
      goto __pyx_L0;

      /* "tables/tableextension.pyx":1125
 *       # Return only if this value is interesting
 *       self.indexchunk = self.indexchunk + self.step
 *       if self.index_valid_data[self.indexchunk]:             # <<<<<<<<<<<<<<
 *         return self
 *     else:
 */
    }
    __pyx_L3_continue:;
  }

  /* "tables/tableextension.pyx":1128
 *         return self
 *     else:
 *       self._finish_riterator()             # <<<<<<<<<<<<<<
 * 
 *   cdef __next__general(self):
 */
  /*else*/ {
    __pyx_t_6 = ((struct __pyx_vtabstruct_6tables_14tableextension_Row *)__pyx_v_self->__pyx_vtab)->_finish_riterator(__pyx_v_self); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1128, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }

  /* "tables/tableextension.pyx":1076
 *       self._finish_riterator()
 * 
 *   cdef __next__inkernel(self):             # <<<<<<<<<<<<<<
 *     """The version of next() in case of in-kernel conditions"""
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("tables.tableextension.Row.__next__inkernel", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":1130
 *       self._finish_riterator()
 * 
 *   cdef __next__general(self):             # <<<<<<<<<<<<<<
 *     """The version of next() for the general cases"""
 *     cdef int recout
 */

static PyObject *__pyx_f_6tables_14tableextension_3Row___next__general(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self) {
  int __pyx_v_recout;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  hsize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PY_LONG_LONG __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__next__general", 0);

  /* "tables/tableextension.pyx":1133
 *     """The version of next() for the general cases"""
 *     cdef int recout
 *     if 0 < self.step:             # <<<<<<<<<<<<<<
 *       self.nextelement = self._nrow + self.step
 *       while self.nextelement < self.stop:
 */
  __pyx_t_1 = ((0 < __pyx_v_self->step) != 0);
  if (__pyx_t_1) {

    /* "tables/tableextension.pyx":1134
 *     cdef int recout
 *     if 0 < self.step:
 *       self.nextelement = self._nrow + self.step             # <<<<<<<<<<<<<<
 *       while self.nextelement < self.stop:
 *         if self.nextelement >= self.nrowsread:
 */
    __pyx_v_self->nextelement = (__pyx_v_self->_nrow + __pyx_v_self->step);

    /* "tables/tableextension.pyx":1135
 *     if 0 < self.step:
 *       self.nextelement = self._nrow + self.step
 *       while self.nextelement < self.stop:             # <<<<<<<<<<<<<<
 *         if self.nextelement >= self.nrowsread:
 *           # Skip until there is interesting information
 */
    while (1) {
      __pyx_t_1 = ((__pyx_v_self->nextelement < __pyx_v_self->stop) != 0);
      if (!__pyx_t_1) break;

      /* "tables/tableextension.pyx":1136
 *       self.nextelement = self._nrow + self.step
 *       while self.nextelement < self.stop:
 *         if self.nextelement >= self.nrowsread:             # <<<<<<<<<<<<<<
 *           # Skip until there is interesting information
 *           while self.nextelement >= self.nrowsread + self.nrowsinbuf:
 */
      __pyx_t_1 = ((__pyx_v_self->nextelement >= __pyx_v_self->nrowsread) != 0);
      if (__pyx_t_1) {

        /* "tables/tableextension.pyx":1138
 *         if self.nextelement >= self.nrowsread:
 *           # Skip until there is interesting information
 *           while self.nextelement >= self.nrowsread + self.nrowsinbuf:             # <<<<<<<<<<<<<<
 *             self.nrowsread = self.nrowsread + self.nrowsinbuf
 *           # Compute the end for this iteration
 */
        while (1) {
          __pyx_t_1 = ((__pyx_v_self->nextelement >= (__pyx_v_self->nrowsread + __pyx_v_self->nrowsinbuf)) != 0);
          if (!__pyx_t_1) break;

          /* "tables/tableextension.pyx":1139
 *           # Skip until there is interesting information
 *           while self.nextelement >= self.nrowsread + self.nrowsinbuf:
 *             self.nrowsread = self.nrowsread + self.nrowsinbuf             # <<<<<<<<<<<<<<
 *           # Compute the end for this iteration
 *           self.stopb = self.stop - self.nrowsread
 */
          __pyx_v_self->nrowsread = (__pyx_v_self->nrowsread + __pyx_v_self->nrowsinbuf);
        }

        /* "tables/tableextension.pyx":1141
 *             self.nrowsread = self.nrowsread + self.nrowsinbuf
 *           # Compute the end for this iteration
 *           self.stopb = self.stop - self.nrowsread             # <<<<<<<<<<<<<<
 *           if self.stopb > self.nrowsinbuf:
 *             self.stopb = self.nrowsinbuf
 */
        __pyx_v_self->stopb = (__pyx_v_self->stop - __pyx_v_self->nrowsread);

        /* "tables/tableextension.pyx":1142
 *           # Compute the end for this iteration
 *           self.stopb = self.stop - self.nrowsread
 *           if self.stopb > self.nrowsinbuf:             # <<<<<<<<<<<<<<
 *             self.stopb = self.nrowsinbuf
 *           self._row = self.startb - self.step
 */
        __pyx_t_1 = ((__pyx_v_self->stopb > __pyx_v_self->nrowsinbuf) != 0);
        if (__pyx_t_1) {

          /* "tables/tableextension.pyx":1143
 *           self.stopb = self.stop - self.nrowsread
 *           if self.stopb > self.nrowsinbuf:
 *             self.stopb = self.nrowsinbuf             # <<<<<<<<<<<<<<
 *           self._row = self.startb - self.step
 *           # Read a chunk
 */
          __pyx_t_2 = __pyx_v_self->nrowsinbuf;
          __pyx_v_self->stopb = __pyx_t_2;

          /* "tables/tableextension.pyx":1142
 *           # Compute the end for this iteration
 *           self.stopb = self.stop - self.nrowsread
 *           if self.stopb > self.nrowsinbuf:             # <<<<<<<<<<<<<<
 *             self.stopb = self.nrowsinbuf
 *           self._row = self.startb - self.step
 */
        }

        /* "tables/tableextension.pyx":1144
 *           if self.stopb > self.nrowsinbuf:
 *             self.stopb = self.nrowsinbuf
 *           self._row = self.startb - self.step             # <<<<<<<<<<<<<<
 *           # Read a chunk
 *           recout = self.table._read_records(self.nrowsread, self.nrowsinbuf,
 */
        __pyx_v_self->_row = (__pyx_v_self->startb - __pyx_v_self->step);

        /* "tables/tableextension.pyx":1146
 *           self._row = self.startb - self.step
 *           # Read a chunk
 *           recout = self.table._read_records(self.nrowsread, self.nrowsinbuf,             # <<<<<<<<<<<<<<
 *                                             self.iobuf)
 *           self.nrowsread = self.nrowsread + recout
 */
        __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_table); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1146, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_read_records); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1146, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_4 = __Pyx_PyInt_From_hsize_t(__pyx_v_self->nrowsread); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1146, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_6 = __Pyx_PyInt_From_hsize_t(__pyx_v_self->nrowsinbuf); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1146, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);

        /* "tables/tableextension.pyx":1147
 *           # Read a chunk
 *           recout = self.table._read_records(self.nrowsread, self.nrowsinbuf,
 *                                             self.iobuf)             # <<<<<<<<<<<<<<
 *           self.nrowsread = self.nrowsread + recout
 * 
 */
        __pyx_t_7 = NULL;
        __pyx_t_8 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
            __pyx_t_8 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_t_4, __pyx_t_6, __pyx_v_self->iobuf};
          __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1146, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_t_4, __pyx_t_6, __pyx_v_self->iobuf};
          __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1146, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        } else
        #endif
        {
          __pyx_t_9 = PyTuple_New(3+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1146, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_9);
          if (__pyx_t_7) {
            __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
          }
          __Pyx_GIVEREF(__pyx_t_4);
          PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_8, __pyx_t_4);
          __Pyx_GIVEREF(__pyx_t_6);
          PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_8, __pyx_t_6);
          __Pyx_INCREF(__pyx_v_self->iobuf);
          __Pyx_GIVEREF(__pyx_v_self->iobuf);
          PyTuple_SET_ITEM(__pyx_t_9, 2+__pyx_t_8, __pyx_v_self->iobuf);
          __pyx_t_4 = 0;
          __pyx_t_6 = 0;
          __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_9, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1146, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        }
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

        /* "tables/tableextension.pyx":1146
 *           self._row = self.startb - self.step
 *           # Read a chunk
 *           recout = self.table._read_records(self.nrowsread, self.nrowsinbuf,             # <<<<<<<<<<<<<<
 *                                             self.iobuf)
 *           self.nrowsread = self.nrowsread + recout
 */
        __pyx_t_8 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_8 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1146, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_v_recout = __pyx_t_8;

        /* "tables/tableextension.pyx":1148
 *           recout = self.table._read_records(self.nrowsread, self.nrowsinbuf,
 *                                             self.iobuf)
 *           self.nrowsread = self.nrowsread + recout             # <<<<<<<<<<<<<<
 * 
 *         self._row = self._row + self.step
 */
        __pyx_v_self->nrowsread = (__pyx_v_self->nrowsread + __pyx_v_recout);

        /* "tables/tableextension.pyx":1136
 *       self.nextelement = self._nrow + self.step
 *       while self.nextelement < self.stop:
 *         if self.nextelement >= self.nrowsread:             # <<<<<<<<<<<<<<
 *           # Skip until there is interesting information
 *           while self.nextelement >= self.nrowsread + self.nrowsinbuf:
 */
      }

      /* "tables/tableextension.pyx":1150
 *           self.nrowsread = self.nrowsread + recout
 * 
 *         self._row = self._row + self.step             # <<<<<<<<<<<<<<
 *         self._nrow = self.nextelement
 *         if self._row + self.step >= self.stopb:
 */
      __pyx_v_self->_row = (__pyx_v_self->_row + __pyx_v_self->step);

      /* "tables/tableextension.pyx":1151
 * 
 *         self._row = self._row + self.step
 *         self._nrow = self.nextelement             # <<<<<<<<<<<<<<
 *         if self._row + self.step >= self.stopb:
 *           # Compute the start row for the next buffer
 */
      __pyx_t_10 = __pyx_v_self->nextelement;
      __pyx_v_self->_nrow = __pyx_t_10;

      /* "tables/tableextension.pyx":1152
 *         self._row = self._row + self.step
 *         self._nrow = self.nextelement
 *         if self._row + self.step >= self.stopb:             # <<<<<<<<<<<<<<
 *           # Compute the start row for the next buffer
 *           self.startb = (self._row + self.step) % self.nrowsinbuf
 */
      __pyx_t_1 = (((__pyx_v_self->_row + __pyx_v_self->step) >= __pyx_v_self->stopb) != 0);
      if (__pyx_t_1) {

        /* "tables/tableextension.pyx":1154
 *         if self._row + self.step >= self.stopb:
 *           # Compute the start row for the next buffer
 *           self.startb = (self._row + self.step) % self.nrowsinbuf             # <<<<<<<<<<<<<<
 * 
 *         self.nextelement = self._nrow + self.step
 */
        __pyx_t_10 = (__pyx_v_self->_row + __pyx_v_self->step);
        if (unlikely(__pyx_v_self->nrowsinbuf == 0)) {
          PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
          __PYX_ERR(0, 1154, __pyx_L1_error)
        }
        __pyx_v_self->startb = (__pyx_t_10 % __pyx_v_self->nrowsinbuf);

        /* "tables/tableextension.pyx":1152
 *         self._row = self._row + self.step
 *         self._nrow = self.nextelement
 *         if self._row + self.step >= self.stopb:             # <<<<<<<<<<<<<<
 *           # Compute the start row for the next buffer
 *           self.startb = (self._row + self.step) % self.nrowsinbuf
 */
      }

      /* "tables/tableextension.pyx":1156
 *           self.startb = (self._row + self.step) % self.nrowsinbuf
 * 
 *         self.nextelement = self._nrow + self.step             # <<<<<<<<<<<<<<
 *         # Return this value
 *         return self
 */
      __pyx_v_self->nextelement = (__pyx_v_self->_nrow + __pyx_v_self->step);

      /* "tables/tableextension.pyx":1158
 *         self.nextelement = self._nrow + self.step
 *         # Return this value
 *         return self             # <<<<<<<<<<<<<<
 *       else:
 *         self._finish_riterator()
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(((PyObject *)__pyx_v_self));
      __pyx_r = ((PyObject *)__pyx_v_self);
      goto __pyx_L0;
    }

    /* "tables/tableextension.pyx":1160
 *         return self
 *       else:
 *         self._finish_riterator()             # <<<<<<<<<<<<<<
 *     elif 0 > self.step:
 *       self.stopb = -1
 */
    /*else*/ {
      __pyx_t_3 = ((struct __pyx_vtabstruct_6tables_14tableextension_Row *)__pyx_v_self->__pyx_vtab)->_finish_riterator(__pyx_v_self); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1160, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }

    /* "tables/tableextension.pyx":1133
 *     """The version of next() for the general cases"""
 *     cdef int recout
 *     if 0 < self.step:             # <<<<<<<<<<<<<<
 *       self.nextelement = self._nrow + self.step
 *       while self.nextelement < self.stop:
 */
    goto __pyx_L3;
  }

  /* "tables/tableextension.pyx":1161
 *       else:
 *         self._finish_riterator()
 *     elif 0 > self.step:             # <<<<<<<<<<<<<<
 *       self.stopb = -1
 *       while self.nextelement - 1 > self.stop:
 */
  __pyx_t_1 = ((0 > __pyx_v_self->step) != 0);
  if (__pyx_t_1) {

    /* "tables/tableextension.pyx":1162
 *         self._finish_riterator()
 *     elif 0 > self.step:
 *       self.stopb = -1             # <<<<<<<<<<<<<<
 *       while self.nextelement - 1 > self.stop:
 *         if self.nextelement < self.start - self.nrowsread + 1:
 */
    __pyx_v_self->stopb = -1LL;

    /* "tables/tableextension.pyx":1163
 *     elif 0 > self.step:
 *       self.stopb = -1
 *       while self.nextelement - 1 > self.stop:             # <<<<<<<<<<<<<<
 *         if self.nextelement < self.start - self.nrowsread + 1:
 *           # Read a chunk
 */
    while (1) {
      __pyx_t_1 = (((__pyx_v_self->nextelement - 1) > __pyx_v_self->stop) != 0);
      if (!__pyx_t_1) break;

      /* "tables/tableextension.pyx":1164
 *       self.stopb = -1
 *       while self.nextelement - 1 > self.stop:
 *         if self.nextelement < self.start - self.nrowsread + 1:             # <<<<<<<<<<<<<<
 *           # Read a chunk
 *           recout = self.table._read_records(self.nextelement - self.nrowsinbuf + 1,
 */
      __pyx_t_1 = ((__pyx_v_self->nextelement < ((__pyx_v_self->start - __pyx_v_self->nrowsread) + 1)) != 0);
      if (__pyx_t_1) {

        /* "tables/tableextension.pyx":1166
 *         if self.nextelement < self.start - self.nrowsread + 1:
 *           # Read a chunk
 *           recout = self.table._read_records(self.nextelement - self.nrowsinbuf + 1,             # <<<<<<<<<<<<<<
 *                                             self.nrowsinbuf, self.iobuf)
 *           self.nrowsread = self.nrowsread + self.nrowsinbuf
 */
        __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_table); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1166, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_read_records); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1166, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_5 = __Pyx_PyInt_From_hsize_t(((__pyx_v_self->nextelement - __pyx_v_self->nrowsinbuf) + 1)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1166, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);

        /* "tables/tableextension.pyx":1167
 *           # Read a chunk
 *           recout = self.table._read_records(self.nextelement - self.nrowsinbuf + 1,
 *                                             self.nrowsinbuf, self.iobuf)             # <<<<<<<<<<<<<<
 *           self.nrowsread = self.nrowsread + self.nrowsinbuf
 *           self._row = self.nrowsinbuf - 1
 */
        __pyx_t_6 = __Pyx_PyInt_From_hsize_t(__pyx_v_self->nrowsinbuf); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1167, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_4 = NULL;
        __pyx_t_8 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_9);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_9, function);
            __pyx_t_8 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_9)) {
          PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_t_5, __pyx_t_6, __pyx_v_self->iobuf};
          __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_9, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1166, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_9)) {
          PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_t_5, __pyx_t_6, __pyx_v_self->iobuf};
          __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_9, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1166, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        } else
        #endif
        {
          __pyx_t_7 = PyTuple_New(3+__pyx_t_8); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1166, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          if (__pyx_t_4) {
            __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_4); __pyx_t_4 = NULL;
          }
          __Pyx_GIVEREF(__pyx_t_5);
          PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_8, __pyx_t_5);
          __Pyx_GIVEREF(__pyx_t_6);
          PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_8, __pyx_t_6);
          __Pyx_INCREF(__pyx_v_self->iobuf);
          __Pyx_GIVEREF(__pyx_v_self->iobuf);
          PyTuple_SET_ITEM(__pyx_t_7, 2+__pyx_t_8, __pyx_v_self->iobuf);
          __pyx_t_5 = 0;
          __pyx_t_6 = 0;
          __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_7, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1166, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

        /* "tables/tableextension.pyx":1166
 *         if self.nextelement < self.start - self.nrowsread + 1:
 *           # Read a chunk
 *           recout = self.table._read_records(self.nextelement - self.nrowsinbuf + 1,             # <<<<<<<<<<<<<<
 *                                             self.nrowsinbuf, self.iobuf)
 *           self.nrowsread = self.nrowsread + self.nrowsinbuf
 */
        __pyx_t_8 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_8 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1166, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_v_recout = __pyx_t_8;

        /* "tables/tableextension.pyx":1168
 *           recout = self.table._read_records(self.nextelement - self.nrowsinbuf + 1,
 *                                             self.nrowsinbuf, self.iobuf)
 *           self.nrowsread = self.nrowsread + self.nrowsinbuf             # <<<<<<<<<<<<<<
 *           self._row = self.nrowsinbuf - 1
 *         else:
 */
        __pyx_v_self->nrowsread = (__pyx_v_self->nrowsread + __pyx_v_self->nrowsinbuf);

        /* "tables/tableextension.pyx":1169
 *                                             self.nrowsinbuf, self.iobuf)
 *           self.nrowsread = self.nrowsread + self.nrowsinbuf
 *           self._row = self.nrowsinbuf - 1             # <<<<<<<<<<<<<<
 *         else:
 *           self._row = (self._row + self.step) % self.nrowsinbuf
 */
        __pyx_v_self->_row = (__pyx_v_self->nrowsinbuf - 1);

        /* "tables/tableextension.pyx":1164
 *       self.stopb = -1
 *       while self.nextelement - 1 > self.stop:
 *         if self.nextelement < self.start - self.nrowsread + 1:             # <<<<<<<<<<<<<<
 *           # Read a chunk
 *           recout = self.table._read_records(self.nextelement - self.nrowsinbuf + 1,
 */
        goto __pyx_L13;
      }

      /* "tables/tableextension.pyx":1171
 *           self._row = self.nrowsinbuf - 1
 *         else:
 *           self._row = (self._row + self.step) % self.nrowsinbuf             # <<<<<<<<<<<<<<
 * 
 *         self._nrow = self.nextelement - self.step
 */
      /*else*/ {
        __pyx_t_10 = (__pyx_v_self->_row + __pyx_v_self->step);
        if (unlikely(__pyx_v_self->nrowsinbuf == 0)) {
          PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
          __PYX_ERR(0, 1171, __pyx_L1_error)
        }
        __pyx_v_self->_row = (__pyx_t_10 % __pyx_v_self->nrowsinbuf);
      }
      __pyx_L13:;

      /* "tables/tableextension.pyx":1173
 *           self._row = (self._row + self.step) % self.nrowsinbuf
 * 
 *         self._nrow = self.nextelement - self.step             # <<<<<<<<<<<<<<
 *         self.nextelement = self.nextelement + self.step
 *         # Return this value
 */
      __pyx_v_self->_nrow = (__pyx_v_self->nextelement - __pyx_v_self->step);

      /* "tables/tableextension.pyx":1174
 * 
 *         self._nrow = self.nextelement - self.step
 *         self.nextelement = self.nextelement + self.step             # <<<<<<<<<<<<<<
 *         # Return this value
 *         return self
 */
      __pyx_v_self->nextelement = (__pyx_v_self->nextelement + __pyx_v_self->step);

      /* "tables/tableextension.pyx":1176
 *         self.nextelement = self.nextelement + self.step
 *         # Return this value
 *         return self             # <<<<<<<<<<<<<<
 *       else:
 *         self._finish_riterator()
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(((PyObject *)__pyx_v_self));
      __pyx_r = ((PyObject *)__pyx_v_self);
      goto __pyx_L0;
    }

    /* "tables/tableextension.pyx":1178
 *         return self
 *       else:
 *         self._finish_riterator()             # <<<<<<<<<<<<<<
 * 
 *   cdef _finish_riterator(self):
 */
    /*else*/ {
      __pyx_t_3 = ((struct __pyx_vtabstruct_6tables_14tableextension_Row *)__pyx_v_self->__pyx_vtab)->_finish_riterator(__pyx_v_self); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1178, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }

    /* "tables/tableextension.pyx":1161
 *       else:
 *         self._finish_riterator()
 *     elif 0 > self.step:             # <<<<<<<<<<<<<<
 *       self.stopb = -1
 *       while self.nextelement - 1 > self.stop:
 */
  }
  __pyx_L3:;

  /* "tables/tableextension.pyx":1130
 *       self._finish_riterator()
 * 
 *   cdef __next__general(self):             # <<<<<<<<<<<<<<
 *     """The version of next() for the general cases"""
 *     cdef int recout
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("tables.tableextension.Row.__next__general", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":1180
 *         self._finish_riterator()
 * 
 *   cdef _finish_riterator(self):             # <<<<<<<<<<<<<<
 *     """Clean-up things after iterator has been done"""
 *     cdef ObjectCache seqcache
 */

static PyObject *__pyx_f_6tables_14tableextension_3Row__finish_riterator(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self) {
  struct __pyx_obj_6tables_17lrucacheextension_ObjectCache *__pyx_v_seqcache = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_finish_riterator", 0);

  /* "tables/tableextension.pyx":1184
 *     cdef ObjectCache seqcache
 * 
 *     self.rfieldscache = {}     # empty rfields cache             # <<<<<<<<<<<<<<
 *     self.wfieldscache = {}     # empty wfields cache
 *     # Make a copy of the last read row in the private record
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->rfieldscache);
  __Pyx_DECREF(__pyx_v_self->rfieldscache);
  __pyx_v_self->rfieldscache = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":1185
 * 
 *     self.rfieldscache = {}     # empty rfields cache
 *     self.wfieldscache = {}     # empty wfields cache             # <<<<<<<<<<<<<<
 *     # Make a copy of the last read row in the private record
 *     # (this is useful for accessing the last row after an iterator loop)
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1185, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->wfieldscache);
  __Pyx_DECREF(__pyx_v_self->wfieldscache);
  __pyx_v_self->wfieldscache = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":1188
 *     # Make a copy of the last read row in the private record
 *     # (this is useful for accessing the last row after an iterator loop)
 *     if self._row >= 0:             # <<<<<<<<<<<<<<
 *       self.wrec[:] = self.iobuf[self._row]
 *     if self._write_to_seqcache:
 */
  __pyx_t_2 = ((__pyx_v_self->_row >= 0) != 0);
  if (__pyx_t_2) {

    /* "tables/tableextension.pyx":1189
 *     # (this is useful for accessing the last row after an iterator loop)
 *     if self._row >= 0:
 *       self.wrec[:] = self.iobuf[self._row]             # <<<<<<<<<<<<<<
 *     if self._write_to_seqcache:
 *       seqcache = self.table._seqcache
 */
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_self->iobuf, __pyx_v_self->_row, long, 1, __Pyx_PyInt_From_long, 0, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1189, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (__Pyx_PyObject_SetSlice(__pyx_v_self->wrec, __pyx_t_1, 0, 0, NULL, NULL, &__pyx_slice__7, 0, 0, 1) < 0) __PYX_ERR(0, 1189, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "tables/tableextension.pyx":1188
 *     # Make a copy of the last read row in the private record
 *     # (this is useful for accessing the last row after an iterator loop)
 *     if self._row >= 0:             # <<<<<<<<<<<<<<
 *       self.wrec[:] = self.iobuf[self._row]
 *     if self._write_to_seqcache:
 */
  }

  /* "tables/tableextension.pyx":1190
 *     if self._row >= 0:
 *       self.wrec[:] = self.iobuf[self._row]
 *     if self._write_to_seqcache:             # <<<<<<<<<<<<<<
 *       seqcache = self.table._seqcache
 *       # Guessing iterseq size: Each element in self.iterseq should take at least 8 bytes
 */
  __pyx_t_2 = (__pyx_v_self->_write_to_seqcache != 0);
  if (__pyx_t_2) {

    /* "tables/tableextension.pyx":1191
 *       self.wrec[:] = self.iobuf[self._row]
 *     if self._write_to_seqcache:
 *       seqcache = self.table._seqcache             # <<<<<<<<<<<<<<
 *       # Guessing iterseq size: Each element in self.iterseq should take at least 8 bytes
 *       seqcache.setitem_(self.seqcache_key, self.iterseq, len(self.iterseq) * 8)
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_table); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1191, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_seqcache); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1191, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_6tables_17lrucacheextension_ObjectCache))))) __PYX_ERR(0, 1191, __pyx_L1_error)
    __pyx_v_seqcache = ((struct __pyx_obj_6tables_17lrucacheextension_ObjectCache *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "tables/tableextension.pyx":1193
 *       seqcache = self.table._seqcache
 *       # Guessing iterseq size: Each element in self.iterseq should take at least 8 bytes
 *       seqcache.setitem_(self.seqcache_key, self.iterseq, len(self.iterseq) * 8)             # <<<<<<<<<<<<<<
 *     self._riterator = 0        # out of iterator
 *     self.iterseq = None        # empty seqcache-related things
 */
    __pyx_t_3 = __pyx_v_self->seqcache_key;
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_1 = __pyx_v_self->iterseq;
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_4 = __pyx_v_self->iterseq;
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_5 = PyObject_Length(__pyx_t_4); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1193, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    (void)(((struct __pyx_vtabstruct_6tables_17lrucacheextension_ObjectCache *)__pyx_v_seqcache->__pyx_base.__pyx_vtab)->setitem_(__pyx_v_seqcache, __pyx_t_3, __pyx_t_1, (__pyx_t_5 * 8)));
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "tables/tableextension.pyx":1190
 *     if self._row >= 0:
 *       self.wrec[:] = self.iobuf[self._row]
 *     if self._write_to_seqcache:             # <<<<<<<<<<<<<<
 *       seqcache = self.table._seqcache
 *       # Guessing iterseq size: Each element in self.iterseq should take at least 8 bytes
 */
  }

  /* "tables/tableextension.pyx":1194
 *       # Guessing iterseq size: Each element in self.iterseq should take at least 8 bytes
 *       seqcache.setitem_(self.seqcache_key, self.iterseq, len(self.iterseq) * 8)
 *     self._riterator = 0        # out of iterator             # <<<<<<<<<<<<<<
 *     self.iterseq = None        # empty seqcache-related things
 *     self.seqcache_key = None
 */
  __pyx_v_self->_riterator = 0;

  /* "tables/tableextension.pyx":1195
 *       seqcache.setitem_(self.seqcache_key, self.iterseq, len(self.iterseq) * 8)
 *     self._riterator = 0        # out of iterator
 *     self.iterseq = None        # empty seqcache-related things             # <<<<<<<<<<<<<<
 *     self.seqcache_key = None
 *     if self._mod_nrows > 0:    # Check if there is some modified row
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->iterseq);
  __Pyx_DECREF(__pyx_v_self->iterseq);
  __pyx_v_self->iterseq = Py_None;

  /* "tables/tableextension.pyx":1196
 *     self._riterator = 0        # out of iterator
 *     self.iterseq = None        # empty seqcache-related things
 *     self.seqcache_key = None             # <<<<<<<<<<<<<<
 *     if self._mod_nrows > 0:    # Check if there is some modified row
 *       self._flush_mod_rows()     # Flush any possible modified row
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->seqcache_key);
  __Pyx_DECREF(__pyx_v_self->seqcache_key);
  __pyx_v_self->seqcache_key = Py_None;

  /* "tables/tableextension.pyx":1197
 *     self.iterseq = None        # empty seqcache-related things
 *     self.seqcache_key = None
 *     if self._mod_nrows > 0:    # Check if there is some modified row             # <<<<<<<<<<<<<<
 *       self._flush_mod_rows()     # Flush any possible modified row
 *     self.modified_fields = set()  # Empty the set of modified fields
 */
  __pyx_t_2 = ((__pyx_v_self->_mod_nrows > 0) != 0);
  if (__pyx_t_2) {

    /* "tables/tableextension.pyx":1198
 *     self.seqcache_key = None
 *     if self._mod_nrows > 0:    # Check if there is some modified row
 *       self._flush_mod_rows()     # Flush any possible modified row             # <<<<<<<<<<<<<<
 *     self.modified_fields = set()  # Empty the set of modified fields
 *     raise StopIteration        # end of iteration
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_flush_mod_rows); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1198, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1198, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "tables/tableextension.pyx":1197
 *     self.iterseq = None        # empty seqcache-related things
 *     self.seqcache_key = None
 *     if self._mod_nrows > 0:    # Check if there is some modified row             # <<<<<<<<<<<<<<
 *       self._flush_mod_rows()     # Flush any possible modified row
 *     self.modified_fields = set()  # Empty the set of modified fields
 */
  }

  /* "tables/tableextension.pyx":1199
 *     if self._mod_nrows > 0:    # Check if there is some modified row
 *       self._flush_mod_rows()     # Flush any possible modified row
 *     self.modified_fields = set()  # Empty the set of modified fields             # <<<<<<<<<<<<<<
 *     raise StopIteration        # end of iteration
 * 
 */
  __pyx_t_1 = PySet_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->modified_fields);
  __Pyx_DECREF(__pyx_v_self->modified_fields);
  __pyx_v_self->modified_fields = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":1200
 *       self._flush_mod_rows()     # Flush any possible modified row
 *     self.modified_fields = set()  # Empty the set of modified fields
 *     raise StopIteration        # end of iteration             # <<<<<<<<<<<<<<
 * 
 *   def _fill_col(self, result, start, stop, step, field):
 */
  __Pyx_Raise(__pyx_builtin_StopIteration, 0, 0, 0);
  __PYX_ERR(0, 1200, __pyx_L1_error)

  /* "tables/tableextension.pyx":1180
 *         self._finish_riterator()
 * 
 *   cdef _finish_riterator(self):             # <<<<<<<<<<<<<<
 *     """Clean-up things after iterator has been done"""
 *     cdef ObjectCache seqcache
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("tables.tableextension.Row._finish_riterator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __Pyx_XDECREF((PyObject *)__pyx_v_seqcache);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":1202
 *     raise StopIteration        # end of iteration
 * 
 *   def _fill_col(self, result, start, stop, step, field):             # <<<<<<<<<<<<<<
 *     """Read a field from a table on disk and put the result in result"""
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_14tableextension_3Row_9_fill_col(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6tables_14tableextension_3Row_8_fill_col[] = "Read a field from a table on disk and put the result in result";
static PyObject *__pyx_pw_6tables_14tableextension_3Row_9_fill_col(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_result = 0;
  PyObject *__pyx_v_start = 0;
  PyObject *__pyx_v_stop = 0;
  PyObject *__pyx_v_step = 0;
  PyObject *__pyx_v_field = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_fill_col (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_result,&__pyx_n_s_start,&__pyx_n_s_stop,&__pyx_n_s_step,&__pyx_n_s_field,0};
    PyObject* values[5] = {0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_result)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_start)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_fill_col", 1, 5, 5, 1); __PYX_ERR(0, 1202, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_stop)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_fill_col", 1, 5, 5, 2); __PYX_ERR(0, 1202, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_step)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_fill_col", 1, 5, 5, 3); __PYX_ERR(0, 1202, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_field)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_fill_col", 1, 5, 5, 4); __PYX_ERR(0, 1202, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_fill_col") < 0)) __PYX_ERR(0, 1202, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 5) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
    }
    __pyx_v_result = values[0];
    __pyx_v_start = values[1];
    __pyx_v_stop = values[2];
    __pyx_v_step = values[3];
    __pyx_v_field = values[4];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_fill_col", 1, 5, 5, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1202, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("tables.tableextension.Row._fill_col", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_14tableextension_3Row_8_fill_col(((struct __pyx_obj_6tables_14tableextension_Row *)__pyx_v_self), __pyx_v_result, __pyx_v_start, __pyx_v_stop, __pyx_v_step, __pyx_v_field);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_14tableextension_3Row_8_fill_col(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self, PyObject *__pyx_v_result, PyObject *__pyx_v_start, PyObject *__pyx_v_stop, PyObject *__pyx_v_step, PyObject *__pyx_v_field) {
  hsize_t __pyx_v_startr;
  hsize_t __pyx_v_istartb;
  hsize_t __pyx_v_istart;
  hsize_t __pyx_v_inrowsinbuf;
  hsize_t __pyx_v_inextelement;
  PY_LONG_LONG __pyx_v_stopr;
  PY_LONG_LONG __pyx_v_istopb;
  PY_LONG_LONG __pyx_v_i;
  PY_LONG_LONG __pyx_v_j;
  PY_LONG_LONG __pyx_v_inrowsread;
  PY_LONG_LONG __pyx_v_istop;
  PY_LONG_LONG __pyx_v_istep;
  PyObject *__pyx_v_fields = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  hsize_t __pyx_t_1;
  PY_LONG_LONG __pyx_t_2;
  PY_LONG_LONG __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  hsize_t __pyx_t_5;
  hsize_t __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_t_13;
  PyObject *__pyx_t_14 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_fill_col", 0);

  /* "tables/tableextension.pyx":1212
 * 
 *     # We can't reuse existing buffers in this context
 *     self._init_loop(start, stop, step, None, None)             # <<<<<<<<<<<<<<
 *     istart, istop, istep = self.start, self.stop, self.step
 *     inrowsinbuf, inextelement, inrowsread = self.nrowsinbuf, istart, istart
 */
  __pyx_t_1 = __Pyx_PyInt_As_hsize_t(__pyx_v_start); if (unlikely((__pyx_t_1 == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1212, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_PY_LONG_LONG(__pyx_v_stop); if (unlikely((__pyx_t_2 == (PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(0, 1212, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_PY_LONG_LONG(__pyx_v_step); if (unlikely((__pyx_t_3 == (PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(0, 1212, __pyx_L1_error)
  __pyx_t_4 = ((struct __pyx_vtabstruct_6tables_14tableextension_Row *)__pyx_v_self->__pyx_vtab)->_init_loop(__pyx_v_self, __pyx_t_1, __pyx_t_2, __pyx_t_3, Py_None, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1212, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "tables/tableextension.pyx":1213
 *     # We can't reuse existing buffers in this context
 *     self._init_loop(start, stop, step, None, None)
 *     istart, istop, istep = self.start, self.stop, self.step             # <<<<<<<<<<<<<<
 *     inrowsinbuf, inextelement, inrowsread = self.nrowsinbuf, istart, istart
 *     istartb, startr = self.startb, 0
 */
  __pyx_t_1 = __pyx_v_self->start;
  __pyx_t_3 = __pyx_v_self->stop;
  __pyx_t_2 = __pyx_v_self->step;
  __pyx_v_istart = __pyx_t_1;
  __pyx_v_istop = __pyx_t_3;
  __pyx_v_istep = __pyx_t_2;

  /* "tables/tableextension.pyx":1214
 *     self._init_loop(start, stop, step, None, None)
 *     istart, istop, istep = self.start, self.stop, self.step
 *     inrowsinbuf, inextelement, inrowsread = self.nrowsinbuf, istart, istart             # <<<<<<<<<<<<<<
 *     istartb, startr = self.startb, 0
 *     i = istart
 */
  __pyx_t_1 = __pyx_v_self->nrowsinbuf;
  __pyx_t_5 = __pyx_v_istart;
  __pyx_t_6 = __pyx_v_istart;
  __pyx_v_inrowsinbuf = __pyx_t_1;
  __pyx_v_inextelement = __pyx_t_5;
  __pyx_v_inrowsread = __pyx_t_6;

  /* "tables/tableextension.pyx":1215
 *     istart, istop, istep = self.start, self.stop, self.step
 *     inrowsinbuf, inextelement, inrowsread = self.nrowsinbuf, istart, istart
 *     istartb, startr = self.startb, 0             # <<<<<<<<<<<<<<
 *     i = istart
 *     if 0 < istep:
 */
  __pyx_t_6 = __pyx_v_self->startb;
  __pyx_t_5 = 0;
  __pyx_v_istartb = __pyx_t_6;
  __pyx_v_startr = __pyx_t_5;

  /* "tables/tableextension.pyx":1216
 *     inrowsinbuf, inextelement, inrowsread = self.nrowsinbuf, istart, istart
 *     istartb, startr = self.startb, 0
 *     i = istart             # <<<<<<<<<<<<<<
 *     if 0 < istep:
 *       while i < istop:
 */
  __pyx_v_i = __pyx_v_istart;

  /* "tables/tableextension.pyx":1217
 *     istartb, startr = self.startb, 0
 *     i = istart
 *     if 0 < istep:             # <<<<<<<<<<<<<<
 *       while i < istop:
 *         if (inextelement >= inrowsread + inrowsinbuf):
 */
  __pyx_t_7 = ((0 < __pyx_v_istep) != 0);
  if (__pyx_t_7) {

    /* "tables/tableextension.pyx":1218
 *     i = istart
 *     if 0 < istep:
 *       while i < istop:             # <<<<<<<<<<<<<<
 *         if (inextelement >= inrowsread + inrowsinbuf):
 *           inrowsread = inrowsread + inrowsinbuf
 */
    while (1) {
      __pyx_t_7 = ((__pyx_v_i < __pyx_v_istop) != 0);
      if (!__pyx_t_7) break;

      /* "tables/tableextension.pyx":1219
 *     if 0 < istep:
 *       while i < istop:
 *         if (inextelement >= inrowsread + inrowsinbuf):             # <<<<<<<<<<<<<<
 *           inrowsread = inrowsread + inrowsinbuf
 *           i = i + inrowsinbuf
 */
      __pyx_t_7 = ((__pyx_v_inextelement >= (__pyx_v_inrowsread + __pyx_v_inrowsinbuf)) != 0);
      if (__pyx_t_7) {

        /* "tables/tableextension.pyx":1220
 *       while i < istop:
 *         if (inextelement >= inrowsread + inrowsinbuf):
 *           inrowsread = inrowsread + inrowsinbuf             # <<<<<<<<<<<<<<
 *           i = i + inrowsinbuf
 *           continue
 */
        __pyx_v_inrowsread = (__pyx_v_inrowsread + __pyx_v_inrowsinbuf);

        /* "tables/tableextension.pyx":1221
 *         if (inextelement >= inrowsread + inrowsinbuf):
 *           inrowsread = inrowsread + inrowsinbuf
 *           i = i + inrowsinbuf             # <<<<<<<<<<<<<<
 *           continue
 *         # Compute the end for this iteration
 */
        __pyx_v_i = (__pyx_v_i + __pyx_v_inrowsinbuf);

        /* "tables/tableextension.pyx":1222
 *           inrowsread = inrowsread + inrowsinbuf
 *           i = i + inrowsinbuf
 *           continue             # <<<<<<<<<<<<<<
 *         # Compute the end for this iteration
 *         istopb = istop - inrowsread
 */
        goto __pyx_L4_continue;

        /* "tables/tableextension.pyx":1219
 *     if 0 < istep:
 *       while i < istop:
 *         if (inextelement >= inrowsread + inrowsinbuf):             # <<<<<<<<<<<<<<
 *           inrowsread = inrowsread + inrowsinbuf
 *           i = i + inrowsinbuf
 */
      }

      /* "tables/tableextension.pyx":1224
 *           continue
 *         # Compute the end for this iteration
 *         istopb = istop - inrowsread             # <<<<<<<<<<<<<<
 *         if istopb > inrowsinbuf:
 *           istopb = inrowsinbuf
 */
      __pyx_v_istopb = (__pyx_v_istop - __pyx_v_inrowsread);

      /* "tables/tableextension.pyx":1225
 *         # Compute the end for this iteration
 *         istopb = istop - inrowsread
 *         if istopb > inrowsinbuf:             # <<<<<<<<<<<<<<
 *           istopb = inrowsinbuf
 *         stopr = startr + ((istopb - istartb - 1) // istep) + 1
 */
      __pyx_t_7 = ((__pyx_v_istopb > __pyx_v_inrowsinbuf) != 0);
      if (__pyx_t_7) {

        /* "tables/tableextension.pyx":1226
 *         istopb = istop - inrowsread
 *         if istopb > inrowsinbuf:
 *           istopb = inrowsinbuf             # <<<<<<<<<<<<<<
 *         stopr = startr + ((istopb - istartb - 1) // istep) + 1
 *         # Read a chunk
 */
        __pyx_v_istopb = __pyx_v_inrowsinbuf;

        /* "tables/tableextension.pyx":1225
 *         # Compute the end for this iteration
 *         istopb = istop - inrowsread
 *         if istopb > inrowsinbuf:             # <<<<<<<<<<<<<<
 *           istopb = inrowsinbuf
 *         stopr = startr + ((istopb - istartb - 1) // istep) + 1
 */
      }

      /* "tables/tableextension.pyx":1227
 *         if istopb > inrowsinbuf:
 *           istopb = inrowsinbuf
 *         stopr = startr + ((istopb - istartb - 1) // istep) + 1             # <<<<<<<<<<<<<<
 *         # Read a chunk
 *         inrowsread = inrowsread + self.table._read_records(i, inrowsinbuf,
 */
      __pyx_t_5 = ((__pyx_v_istopb - __pyx_v_istartb) - 1);
      if (unlikely(__pyx_v_istep == 0)) {
        PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
        __PYX_ERR(0, 1227, __pyx_L1_error)
      }
      __pyx_v_stopr = ((__pyx_v_startr + (__pyx_t_5 / __pyx_v_istep)) + 1);

      /* "tables/tableextension.pyx":1229
 *         stopr = startr + ((istopb - istartb - 1) // istep) + 1
 *         # Read a chunk
 *         inrowsread = inrowsread + self.table._read_records(i, inrowsinbuf,             # <<<<<<<<<<<<<<
 *                                                            self.iobuf)
 *         # Assign the correct part to result
 */
      __pyx_t_4 = __Pyx_PyInt_From_PY_LONG_LONG(__pyx_v_inrowsread); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1229, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_table); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1229, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_read_records); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1229, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_9 = __Pyx_PyInt_From_PY_LONG_LONG(__pyx_v_i); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1229, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_11 = __Pyx_PyInt_From_hsize_t(__pyx_v_inrowsinbuf); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1229, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);

      /* "tables/tableextension.pyx":1230
 *         # Read a chunk
 *         inrowsread = inrowsread + self.table._read_records(i, inrowsinbuf,
 *                                                            self.iobuf)             # <<<<<<<<<<<<<<
 *         # Assign the correct part to result
 *         fields = self.iobuf
 */
      __pyx_t_12 = NULL;
      __pyx_t_13 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
        __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_10);
        if (likely(__pyx_t_12)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
          __Pyx_INCREF(__pyx_t_12);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_10, function);
          __pyx_t_13 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_10)) {
        PyObject *__pyx_temp[4] = {__pyx_t_12, __pyx_t_9, __pyx_t_11, __pyx_v_self->iobuf};
        __pyx_t_8 = __Pyx_PyFunction_FastCall(__pyx_t_10, __pyx_temp+1-__pyx_t_13, 3+__pyx_t_13); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1229, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_10)) {
        PyObject *__pyx_temp[4] = {__pyx_t_12, __pyx_t_9, __pyx_t_11, __pyx_v_self->iobuf};
        __pyx_t_8 = __Pyx_PyCFunction_FastCall(__pyx_t_10, __pyx_temp+1-__pyx_t_13, 3+__pyx_t_13); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1229, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      } else
      #endif
      {
        __pyx_t_14 = PyTuple_New(3+__pyx_t_13); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1229, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_14);
        if (__pyx_t_12) {
          __Pyx_GIVEREF(__pyx_t_12); PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_t_12); __pyx_t_12 = NULL;
        }
        __Pyx_GIVEREF(__pyx_t_9);
        PyTuple_SET_ITEM(__pyx_t_14, 0+__pyx_t_13, __pyx_t_9);
        __Pyx_GIVEREF(__pyx_t_11);
        PyTuple_SET_ITEM(__pyx_t_14, 1+__pyx_t_13, __pyx_t_11);
        __Pyx_INCREF(__pyx_v_self->iobuf);
        __Pyx_GIVEREF(__pyx_v_self->iobuf);
        PyTuple_SET_ITEM(__pyx_t_14, 2+__pyx_t_13, __pyx_v_self->iobuf);
        __pyx_t_9 = 0;
        __pyx_t_11 = 0;
        __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_t_14, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1229, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      }
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

      /* "tables/tableextension.pyx":1229
 *         stopr = startr + ((istopb - istartb - 1) // istep) + 1
 *         # Read a chunk
 *         inrowsread = inrowsread + self.table._read_records(i, inrowsinbuf,             # <<<<<<<<<<<<<<
 *                                                            self.iobuf)
 *         # Assign the correct part to result
 */
      __pyx_t_10 = PyNumber_Add(__pyx_t_4, __pyx_t_8); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1229, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_2 = __Pyx_PyInt_As_PY_LONG_LONG(__pyx_t_10); if (unlikely((__pyx_t_2 == (PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(0, 1229, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_v_inrowsread = __pyx_t_2;

      /* "tables/tableextension.pyx":1232
 *                                                            self.iobuf)
 *         # Assign the correct part to result
 *         fields = self.iobuf             # <<<<<<<<<<<<<<
 *         if field:
 *           fields = get_nested_field(fields, field)
 */
      __pyx_t_10 = __pyx_v_self->iobuf;
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_XDECREF_SET(__pyx_v_fields, __pyx_t_10);
      __pyx_t_10 = 0;

      /* "tables/tableextension.pyx":1233
 *         # Assign the correct part to result
 *         fields = self.iobuf
 *         if field:             # <<<<<<<<<<<<<<
 *           fields = get_nested_field(fields, field)
 *         result[startr:stopr] = fields[istartb:istopb:istep]
 */
      __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_v_field); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 1233, __pyx_L1_error)
      if (__pyx_t_7) {

        /* "tables/tableextension.pyx":1234
 *         fields = self.iobuf
 *         if field:
 *           fields = get_nested_field(fields, field)             # <<<<<<<<<<<<<<
 *         result[startr:stopr] = fields[istartb:istopb:istep]
 * 
 */
        __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_n_s_get_nested_field); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1234, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_4 = NULL;
        __pyx_t_13 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_8))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_8);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_8, function);
            __pyx_t_13 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_8)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_fields, __pyx_v_field};
          __pyx_t_10 = __Pyx_PyFunction_FastCall(__pyx_t_8, __pyx_temp+1-__pyx_t_13, 2+__pyx_t_13); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1234, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_10);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_8)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_fields, __pyx_v_field};
          __pyx_t_10 = __Pyx_PyCFunction_FastCall(__pyx_t_8, __pyx_temp+1-__pyx_t_13, 2+__pyx_t_13); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1234, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_10);
        } else
        #endif
        {
          __pyx_t_14 = PyTuple_New(2+__pyx_t_13); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1234, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_14);
          if (__pyx_t_4) {
            __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_t_4); __pyx_t_4 = NULL;
          }
          __Pyx_INCREF(__pyx_v_fields);
          __Pyx_GIVEREF(__pyx_v_fields);
          PyTuple_SET_ITEM(__pyx_t_14, 0+__pyx_t_13, __pyx_v_fields);
          __Pyx_INCREF(__pyx_v_field);
          __Pyx_GIVEREF(__pyx_v_field);
          PyTuple_SET_ITEM(__pyx_t_14, 1+__pyx_t_13, __pyx_v_field);
          __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_14, NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1234, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        }
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF_SET(__pyx_v_fields, __pyx_t_10);
        __pyx_t_10 = 0;

        /* "tables/tableextension.pyx":1233
 *         # Assign the correct part to result
 *         fields = self.iobuf
 *         if field:             # <<<<<<<<<<<<<<
 *           fields = get_nested_field(fields, field)
 *         result[startr:stopr] = fields[istartb:istopb:istep]
 */
      }

      /* "tables/tableextension.pyx":1235
 *         if field:
 *           fields = get_nested_field(fields, field)
 *         result[startr:stopr] = fields[istartb:istopb:istep]             # <<<<<<<<<<<<<<
 * 
 *         # Compute some indexes for the next iteration
 */
      __pyx_t_10 = __Pyx_PyInt_From_hsize_t(__pyx_v_istartb); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1235, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_8 = __Pyx_PyInt_From_PY_LONG_LONG(__pyx_v_istopb); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1235, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_14 = __Pyx_PyInt_From_PY_LONG_LONG(__pyx_v_istep); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1235, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_14);
      __pyx_t_4 = PySlice_New(__pyx_t_10, __pyx_t_8, __pyx_t_14); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1235, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      __pyx_t_14 = __Pyx_PyObject_GetItem(__pyx_v_fields, __pyx_t_4); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1235, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_14);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__Pyx_PyObject_SetSlice(__pyx_v_result, __pyx_t_14, __pyx_v_startr, __pyx_v_stopr, NULL, NULL, NULL, 1, 1, 1) < 0) __PYX_ERR(0, 1235, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;

      /* "tables/tableextension.pyx":1238
 * 
 *         # Compute some indexes for the next iteration
 *         startr = stopr             # <<<<<<<<<<<<<<
 *         j = istartb + ((istopb - istartb - 1) // istep) * istep
 *         istartb = (j+istep) % inrowsinbuf
 */
      __pyx_v_startr = __pyx_v_stopr;

      /* "tables/tableextension.pyx":1239
 *         # Compute some indexes for the next iteration
 *         startr = stopr
 *         j = istartb + ((istopb - istartb - 1) // istep) * istep             # <<<<<<<<<<<<<<
 *         istartb = (j+istep) % inrowsinbuf
 *         inextelement = inextelement + istep
 */
      __pyx_t_5 = ((__pyx_v_istopb - __pyx_v_istartb) - 1);
      if (unlikely(__pyx_v_istep == 0)) {
        PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
        __PYX_ERR(0, 1239, __pyx_L1_error)
      }
      __pyx_v_j = (__pyx_v_istartb + ((__pyx_t_5 / __pyx_v_istep) * __pyx_v_istep));

      /* "tables/tableextension.pyx":1240
 *         startr = stopr
 *         j = istartb + ((istopb - istartb - 1) // istep) * istep
 *         istartb = (j+istep) % inrowsinbuf             # <<<<<<<<<<<<<<
 *         inextelement = inextelement + istep
 *         i = i + inrowsinbuf
 */
      __pyx_t_2 = (__pyx_v_j + __pyx_v_istep);
      if (unlikely(__pyx_v_inrowsinbuf == 0)) {
        PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
        __PYX_ERR(0, 1240, __pyx_L1_error)
      }
      __pyx_v_istartb = (__pyx_t_2 % __pyx_v_inrowsinbuf);

      /* "tables/tableextension.pyx":1241
 *         j = istartb + ((istopb - istartb - 1) // istep) * istep
 *         istartb = (j+istep) % inrowsinbuf
 *         inextelement = inextelement + istep             # <<<<<<<<<<<<<<
 *         i = i + inrowsinbuf
 *     elif istep < 0:
 */
      __pyx_v_inextelement = (__pyx_v_inextelement + __pyx_v_istep);

      /* "tables/tableextension.pyx":1242
 *         istartb = (j+istep) % inrowsinbuf
 *         inextelement = inextelement + istep
 *         i = i + inrowsinbuf             # <<<<<<<<<<<<<<
 *     elif istep < 0:
 *       inrowsinbuf = self.nrowsinbuf
 */
      __pyx_v_i = (__pyx_v_i + __pyx_v_inrowsinbuf);
      __pyx_L4_continue:;
    }

    /* "tables/tableextension.pyx":1217
 *     istartb, startr = self.startb, 0
 *     i = istart
 *     if 0 < istep:             # <<<<<<<<<<<<<<
 *       while i < istop:
 *         if (inextelement >= inrowsread + inrowsinbuf):
 */
    goto __pyx_L3;
  }

  /* "tables/tableextension.pyx":1243
 *         inextelement = inextelement + istep
 *         i = i + inrowsinbuf
 *     elif istep < 0:             # <<<<<<<<<<<<<<
 *       inrowsinbuf = self.nrowsinbuf
 *       #istartb = self.startb
 */
  __pyx_t_7 = ((__pyx_v_istep < 0) != 0);
  if (__pyx_t_7) {

    /* "tables/tableextension.pyx":1244
 *         i = i + inrowsinbuf
 *     elif istep < 0:
 *       inrowsinbuf = self.nrowsinbuf             # <<<<<<<<<<<<<<
 *       #istartb = self.startb
 *       istartb = self.nrowsinbuf - 1
 */
    __pyx_t_5 = __pyx_v_self->nrowsinbuf;
    __pyx_v_inrowsinbuf = __pyx_t_5;

    /* "tables/tableextension.pyx":1246
 *       inrowsinbuf = self.nrowsinbuf
 *       #istartb = self.startb
 *       istartb = self.nrowsinbuf - 1             # <<<<<<<<<<<<<<
 *       #istopb = self.stopb - 1
 *       istopb = -1
 */
    __pyx_v_istartb = (__pyx_v_self->nrowsinbuf - 1);

    /* "tables/tableextension.pyx":1248
 *       istartb = self.nrowsinbuf - 1
 *       #istopb = self.stopb - 1
 *       istopb = -1             # <<<<<<<<<<<<<<
 *       startr = 0
 *       i = istart
 */
    __pyx_v_istopb = -1LL;

    /* "tables/tableextension.pyx":1249
 *       #istopb = self.stopb - 1
 *       istopb = -1
 *       startr = 0             # <<<<<<<<<<<<<<
 *       i = istart
 *       inextelement = istart
 */
    __pyx_v_startr = 0;

    /* "tables/tableextension.pyx":1250
 *       istopb = -1
 *       startr = 0
 *       i = istart             # <<<<<<<<<<<<<<
 *       inextelement = istart
 *       inrowsread = 0
 */
    __pyx_v_i = __pyx_v_istart;

    /* "tables/tableextension.pyx":1251
 *       startr = 0
 *       i = istart
 *       inextelement = istart             # <<<<<<<<<<<<<<
 *       inrowsread = 0
 *       while i-1 > istop:
 */
    __pyx_v_inextelement = __pyx_v_istart;

    /* "tables/tableextension.pyx":1252
 *       i = istart
 *       inextelement = istart
 *       inrowsread = 0             # <<<<<<<<<<<<<<
 *       while i-1 > istop:
 *         #if (inextelement <= inrowsread + inrowsinbuf):
 */
    __pyx_v_inrowsread = 0;

    /* "tables/tableextension.pyx":1253
 *       inextelement = istart
 *       inrowsread = 0
 *       while i-1 > istop:             # <<<<<<<<<<<<<<
 *         #if (inextelement <= inrowsread + inrowsinbuf):
 *         if (inextelement < i - inrowsinbuf):
 */
    while (1) {
      __pyx_t_7 = (((__pyx_v_i - 1) > __pyx_v_istop) != 0);
      if (!__pyx_t_7) break;

      /* "tables/tableextension.pyx":1255
 *       while i-1 > istop:
 *         #if (inextelement <= inrowsread + inrowsinbuf):
 *         if (inextelement < i - inrowsinbuf):             # <<<<<<<<<<<<<<
 *           inrowsread = inrowsread + inrowsinbuf
 *           i = i - inrowsinbuf
 */
      __pyx_t_7 = ((__pyx_v_inextelement < (__pyx_v_i - __pyx_v_inrowsinbuf)) != 0);
      if (__pyx_t_7) {

        /* "tables/tableextension.pyx":1256
 *         #if (inextelement <= inrowsread + inrowsinbuf):
 *         if (inextelement < i - inrowsinbuf):
 *           inrowsread = inrowsread + inrowsinbuf             # <<<<<<<<<<<<<<
 *           i = i - inrowsinbuf
 *           continue
 */
        __pyx_v_inrowsread = (__pyx_v_inrowsread + __pyx_v_inrowsinbuf);

        /* "tables/tableextension.pyx":1257
 *         if (inextelement < i - inrowsinbuf):
 *           inrowsread = inrowsread + inrowsinbuf
 *           i = i - inrowsinbuf             # <<<<<<<<<<<<<<
 *           continue
 *         # Compute the end for this iteration
 */
        __pyx_v_i = (__pyx_v_i - __pyx_v_inrowsinbuf);

        /* "tables/tableextension.pyx":1258
 *           inrowsread = inrowsread + inrowsinbuf
 *           i = i - inrowsinbuf
 *           continue             # <<<<<<<<<<<<<<
 *         # Compute the end for this iteration
 *         # (we know we are going backward so try to keep indices positive)
 */
        goto __pyx_L9_continue;

        /* "tables/tableextension.pyx":1255
 *       while i-1 > istop:
 *         #if (inextelement <= inrowsread + inrowsinbuf):
 *         if (inextelement < i - inrowsinbuf):             # <<<<<<<<<<<<<<
 *           inrowsread = inrowsread + inrowsinbuf
 *           i = i - inrowsinbuf
 */
      }

      /* "tables/tableextension.pyx":1261
 *         # Compute the end for this iteration
 *         # (we know we are going backward so try to keep indices positive)
 *         stopr = startr + (1 - istopb + istartb) // (-istep)             # <<<<<<<<<<<<<<
 *         # Read a chunk
 *         inrowsread = inrowsread + self.table._read_records(i - inrowsinbuf + 1,
 */
      __pyx_t_5 = ((1 - __pyx_v_istopb) + __pyx_v_istartb);
      __pyx_t_2 = (-__pyx_v_istep);
      if (unlikely(__pyx_t_2 == 0)) {
        PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
        __PYX_ERR(0, 1261, __pyx_L1_error)
      }
      __pyx_v_stopr = (__pyx_v_startr + (__pyx_t_5 / __pyx_t_2));

      /* "tables/tableextension.pyx":1263
 *         stopr = startr + (1 - istopb + istartb) // (-istep)
 *         # Read a chunk
 *         inrowsread = inrowsread + self.table._read_records(i - inrowsinbuf + 1,             # <<<<<<<<<<<<<<
 *                                                            inrowsinbuf, self.iobuf)
 *         # Assign the correct part to result
 */
      __pyx_t_14 = __Pyx_PyInt_From_PY_LONG_LONG(__pyx_v_inrowsread); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1263, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_14);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_table); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1263, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_read_records); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1263, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = __Pyx_PyInt_From_hsize_t(((__pyx_v_i - __pyx_v_inrowsinbuf) + 1)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1263, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);

      /* "tables/tableextension.pyx":1264
 *         # Read a chunk
 *         inrowsread = inrowsread + self.table._read_records(i - inrowsinbuf + 1,
 *                                                            inrowsinbuf, self.iobuf)             # <<<<<<<<<<<<<<
 *         # Assign the correct part to result
 *         fields = self.iobuf
 */
      __pyx_t_11 = __Pyx_PyInt_From_hsize_t(__pyx_v_inrowsinbuf); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1264, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_9 = NULL;
      __pyx_t_13 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
        __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_10);
        if (likely(__pyx_t_9)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
          __Pyx_INCREF(__pyx_t_9);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_10, function);
          __pyx_t_13 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_10)) {
        PyObject *__pyx_temp[4] = {__pyx_t_9, __pyx_t_8, __pyx_t_11, __pyx_v_self->iobuf};
        __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_10, __pyx_temp+1-__pyx_t_13, 3+__pyx_t_13); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1263, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_10)) {
        PyObject *__pyx_temp[4] = {__pyx_t_9, __pyx_t_8, __pyx_t_11, __pyx_v_self->iobuf};
        __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_10, __pyx_temp+1-__pyx_t_13, 3+__pyx_t_13); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1263, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      } else
      #endif
      {
        __pyx_t_12 = PyTuple_New(3+__pyx_t_13); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1263, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        if (__pyx_t_9) {
          __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_9); __pyx_t_9 = NULL;
        }
        __Pyx_GIVEREF(__pyx_t_8);
        PyTuple_SET_ITEM(__pyx_t_12, 0+__pyx_t_13, __pyx_t_8);
        __Pyx_GIVEREF(__pyx_t_11);
        PyTuple_SET_ITEM(__pyx_t_12, 1+__pyx_t_13, __pyx_t_11);
        __Pyx_INCREF(__pyx_v_self->iobuf);
        __Pyx_GIVEREF(__pyx_v_self->iobuf);
        PyTuple_SET_ITEM(__pyx_t_12, 2+__pyx_t_13, __pyx_v_self->iobuf);
        __pyx_t_8 = 0;
        __pyx_t_11 = 0;
        __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_t_12, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1263, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      }
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

      /* "tables/tableextension.pyx":1263
 *         stopr = startr + (1 - istopb + istartb) // (-istep)
 *         # Read a chunk
 *         inrowsread = inrowsread + self.table._read_records(i - inrowsinbuf + 1,             # <<<<<<<<<<<<<<
 *                                                            inrowsinbuf, self.iobuf)
 *         # Assign the correct part to result
 */
      __pyx_t_10 = PyNumber_Add(__pyx_t_14, __pyx_t_4); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1263, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_2 = __Pyx_PyInt_As_PY_LONG_LONG(__pyx_t_10); if (unlikely((__pyx_t_2 == (PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(0, 1263, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_v_inrowsread = __pyx_t_2;

      /* "tables/tableextension.pyx":1266
 *                                                            inrowsinbuf, self.iobuf)
 *         # Assign the correct part to result
 *         fields = self.iobuf             # <<<<<<<<<<<<<<
 *         if field:
 *           fields = get_nested_field(fields, field)
 */
      __pyx_t_10 = __pyx_v_self->iobuf;
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_XDECREF_SET(__pyx_v_fields, __pyx_t_10);
      __pyx_t_10 = 0;

      /* "tables/tableextension.pyx":1267
 *         # Assign the correct part to result
 *         fields = self.iobuf
 *         if field:             # <<<<<<<<<<<<<<
 *           fields = get_nested_field(fields, field)
 *         if istopb >= 0:
 */
      __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_v_field); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 1267, __pyx_L1_error)
      if (__pyx_t_7) {

        /* "tables/tableextension.pyx":1268
 *         fields = self.iobuf
 *         if field:
 *           fields = get_nested_field(fields, field)             # <<<<<<<<<<<<<<
 *         if istopb >= 0:
 *             result[startr:stopr] = fields[istartb:istopb:istep]
 */
        __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_get_nested_field); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1268, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_14 = NULL;
        __pyx_t_13 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_14)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_14);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
            __pyx_t_13 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[3] = {__pyx_t_14, __pyx_v_fields, __pyx_v_field};
          __pyx_t_10 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_13, 2+__pyx_t_13); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1268, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
          __Pyx_GOTREF(__pyx_t_10);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[3] = {__pyx_t_14, __pyx_v_fields, __pyx_v_field};
          __pyx_t_10 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_13, 2+__pyx_t_13); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1268, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
          __Pyx_GOTREF(__pyx_t_10);
        } else
        #endif
        {
          __pyx_t_12 = PyTuple_New(2+__pyx_t_13); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1268, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          if (__pyx_t_14) {
            __Pyx_GIVEREF(__pyx_t_14); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_14); __pyx_t_14 = NULL;
          }
          __Pyx_INCREF(__pyx_v_fields);
          __Pyx_GIVEREF(__pyx_v_fields);
          PyTuple_SET_ITEM(__pyx_t_12, 0+__pyx_t_13, __pyx_v_fields);
          __Pyx_INCREF(__pyx_v_field);
          __Pyx_GIVEREF(__pyx_v_field);
          PyTuple_SET_ITEM(__pyx_t_12, 1+__pyx_t_13, __pyx_v_field);
          __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_12, NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1268, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        }
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF_SET(__pyx_v_fields, __pyx_t_10);
        __pyx_t_10 = 0;

        /* "tables/tableextension.pyx":1267
 *         # Assign the correct part to result
 *         fields = self.iobuf
 *         if field:             # <<<<<<<<<<<<<<
 *           fields = get_nested_field(fields, field)
 *         if istopb >= 0:
 */
      }

      /* "tables/tableextension.pyx":1269
 *         if field:
 *           fields = get_nested_field(fields, field)
 *         if istopb >= 0:             # <<<<<<<<<<<<<<
 *             result[startr:stopr] = fields[istartb:istopb:istep]
 *         else:
 */
      __pyx_t_7 = ((__pyx_v_istopb >= 0) != 0);
      if (__pyx_t_7) {

        /* "tables/tableextension.pyx":1270
 *           fields = get_nested_field(fields, field)
 *         if istopb >= 0:
 *             result[startr:stopr] = fields[istartb:istopb:istep]             # <<<<<<<<<<<<<<
 *         else:
 *             result[startr:stopr] = fields[istartb::istep]
 */
        __pyx_t_10 = __Pyx_PyInt_From_hsize_t(__pyx_v_istartb); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1270, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_4 = __Pyx_PyInt_From_PY_LONG_LONG(__pyx_v_istopb); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1270, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_12 = __Pyx_PyInt_From_PY_LONG_LONG(__pyx_v_istep); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1270, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        __pyx_t_14 = PySlice_New(__pyx_t_10, __pyx_t_4, __pyx_t_12); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1270, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_14);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __pyx_t_12 = __Pyx_PyObject_GetItem(__pyx_v_fields, __pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1270, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        if (__Pyx_PyObject_SetSlice(__pyx_v_result, __pyx_t_12, __pyx_v_startr, __pyx_v_stopr, NULL, NULL, NULL, 1, 1, 1) < 0) __PYX_ERR(0, 1270, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

        /* "tables/tableextension.pyx":1269
 *         if field:
 *           fields = get_nested_field(fields, field)
 *         if istopb >= 0:             # <<<<<<<<<<<<<<
 *             result[startr:stopr] = fields[istartb:istopb:istep]
 *         else:
 */
        goto __pyx_L13;
      }

      /* "tables/tableextension.pyx":1272
 *             result[startr:stopr] = fields[istartb:istopb:istep]
 *         else:
 *             result[startr:stopr] = fields[istartb::istep]             # <<<<<<<<<<<<<<
 * 
 *         # Compute some indexes for the next iteration
 */
      /*else*/ {
        __pyx_t_12 = __Pyx_PyInt_From_hsize_t(__pyx_v_istartb); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1272, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        __pyx_t_14 = __Pyx_PyInt_From_PY_LONG_LONG(__pyx_v_istep); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1272, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_14);
        __pyx_t_4 = PySlice_New(__pyx_t_12, Py_None, __pyx_t_14); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1272, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        __pyx_t_14 = __Pyx_PyObject_GetItem(__pyx_v_fields, __pyx_t_4); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1272, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_14);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (__Pyx_PyObject_SetSlice(__pyx_v_result, __pyx_t_14, __pyx_v_startr, __pyx_v_stopr, NULL, NULL, NULL, 1, 1, 1) < 0) __PYX_ERR(0, 1272, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      }
      __pyx_L13:;

      /* "tables/tableextension.pyx":1275
 * 
 *         # Compute some indexes for the next iteration
 *         startr = stopr             # <<<<<<<<<<<<<<
 *         istartb = (i - istartb)%inrowsinbuf
 *         inextelement = inextelement + istep
 */
      __pyx_v_startr = __pyx_v_stopr;

      /* "tables/tableextension.pyx":1276
 *         # Compute some indexes for the next iteration
 *         startr = stopr
 *         istartb = (i - istartb)%inrowsinbuf             # <<<<<<<<<<<<<<
 *         inextelement = inextelement + istep
 *         i = i - inrowsinbuf
 */
      __pyx_t_5 = (__pyx_v_i - __pyx_v_istartb);
      if (unlikely(__pyx_v_inrowsinbuf == 0)) {
        PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
        __PYX_ERR(0, 1276, __pyx_L1_error)
      }
      __pyx_v_istartb = (__pyx_t_5 % __pyx_v_inrowsinbuf);

      /* "tables/tableextension.pyx":1277
 *         startr = stopr
 *         istartb = (i - istartb)%inrowsinbuf
 *         inextelement = inextelement + istep             # <<<<<<<<<<<<<<
 *         i = i - inrowsinbuf
 *     self._riterator = 0  # out of iterator
 */
      __pyx_v_inextelement = (__pyx_v_inextelement + __pyx_v_istep);

      /* "tables/tableextension.pyx":1278
 *         istartb = (i - istartb)%inrowsinbuf
 *         inextelement = inextelement + istep
 *         i = i - inrowsinbuf             # <<<<<<<<<<<<<<
 *     self._riterator = 0  # out of iterator
 *     return
 */
      __pyx_v_i = (__pyx_v_i - __pyx_v_inrowsinbuf);
      __pyx_L9_continue:;
    }

    /* "tables/tableextension.pyx":1243
 *         inextelement = inextelement + istep
 *         i = i + inrowsinbuf
 *     elif istep < 0:             # <<<<<<<<<<<<<<
 *       inrowsinbuf = self.nrowsinbuf
 *       #istartb = self.startb
 */
  }
  __pyx_L3:;

  /* "tables/tableextension.pyx":1279
 *         inextelement = inextelement + istep
 *         i = i - inrowsinbuf
 *     self._riterator = 0  # out of iterator             # <<<<<<<<<<<<<<
 *     return
 * 
 */
  __pyx_v_self->_riterator = 0;

  /* "tables/tableextension.pyx":1280
 *         i = i - inrowsinbuf
 *     self._riterator = 0  # out of iterator
 *     return             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "tables/tableextension.pyx":1202
 *     raise StopIteration        # end of iteration
 * 
 *   def _fill_col(self, result, start, stop, step, field):             # <<<<<<<<<<<<<<
 *     """Read a field from a table on disk and put the result in result"""
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_AddTraceback("tables.tableextension.Row._fill_col", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_fields);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":1283
 * 
 * 
 *   def append(self):             # <<<<<<<<<<<<<<
 *     """Add a new row of data to the end of the dataset.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_14tableextension_3Row_11append(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6tables_14tableextension_3Row_10append[] = "Add a new row of data to the end of the dataset.\n\n    Once you have filled the proper fields for the current\n    row, calling this method actually appends the new data to the\n    *output buffer* (which will eventually be\n    dumped to disk).  If you have not set the value of a field, the\n    default value of the column will be used.\n\n    .. warning::\n\n        After completion of the loop in which :meth:`Row.append` has\n        been called, it is always convenient to make a call to\n        :meth:`Table.flush` in order to avoid losing the last rows that\n        may still remain in internal buffers.\n\n    Examples\n    --------\n\n    ::\n\n        row = table.row\n        for i in xrange(nrows):\n            row['col1'] = i-1\n            row['col2'] = 'a'\n            row['col3'] = -1.0\n            row.append()\n        table.flush()\n\n    ";
static PyObject *__pyx_pw_6tables_14tableextension_3Row_11append(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("append (wrapper)", 0);
  __pyx_r = __pyx_pf_6tables_14tableextension_3Row_10append(((struct __pyx_obj_6tables_14tableextension_Row *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_14tableextension_3Row_10append(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self) {
  PyArrayObject *__pyx_v_iobuf = 0;
  PyArrayObject *__pyx_v_wrec = 0;
  PyArrayObject *__pyx_v_wreccpy = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("append", 0);

  /* "tables/tableextension.pyx":1315
 *     cdef ndarray iobuf, wrec, wreccpy
 * 
 *     if self.ro_filemode:             # <<<<<<<<<<<<<<
 *       raise IOError("Attempt to write over a file opened in read-only mode")
 * 
 */
  __pyx_t_1 = (__pyx_v_self->ro_filemode != 0);
  if (unlikely(__pyx_t_1)) {

    /* "tables/tableextension.pyx":1316
 * 
 *     if self.ro_filemode:
 *       raise IOError("Attempt to write over a file opened in read-only mode")             # <<<<<<<<<<<<<<
 * 
 *     if not self.chunked:
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_IOError, __pyx_tuple__8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1316, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1316, __pyx_L1_error)

    /* "tables/tableextension.pyx":1315
 *     cdef ndarray iobuf, wrec, wreccpy
 * 
 *     if self.ro_filemode:             # <<<<<<<<<<<<<<
 *       raise IOError("Attempt to write over a file opened in read-only mode")
 * 
 */
  }

  /* "tables/tableextension.pyx":1318
 *       raise IOError("Attempt to write over a file opened in read-only mode")
 * 
 *     if not self.chunked:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("You cannot append rows to a non-chunked table.",
 *                          h5tb=False)
 */
  __pyx_t_1 = ((!(__pyx_v_self->chunked != 0)) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "tables/tableextension.pyx":1319
 * 
 *     if not self.chunked:
 *       raise HDF5ExtError("You cannot append rows to a non-chunked table.",             # <<<<<<<<<<<<<<
 *                          h5tb=False)
 * 
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_HDF5ExtError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1319, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);

    /* "tables/tableextension.pyx":1320
 *     if not self.chunked:
 *       raise HDF5ExtError("You cannot append rows to a non-chunked table.",
 *                          h5tb=False)             # <<<<<<<<<<<<<<
 * 
 *     if self._riterator:
 */
    __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1320, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_h5tb, Py_False) < 0) __PYX_ERR(0, 1320, __pyx_L1_error)

    /* "tables/tableextension.pyx":1319
 * 
 *     if not self.chunked:
 *       raise HDF5ExtError("You cannot append rows to a non-chunked table.",             # <<<<<<<<<<<<<<
 *                          h5tb=False)
 * 
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__9, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1319, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 1319, __pyx_L1_error)

    /* "tables/tableextension.pyx":1318
 *       raise IOError("Attempt to write over a file opened in read-only mode")
 * 
 *     if not self.chunked:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("You cannot append rows to a non-chunked table.",
 *                          h5tb=False)
 */
  }

  /* "tables/tableextension.pyx":1322
 *                          h5tb=False)
 * 
 *     if self._riterator:             # <<<<<<<<<<<<<<
 *       raise NotImplementedError("You cannot append rows when in middle of a table iterator. If what you want is to update records, use Row.update() instead.")
 * 
 */
  __pyx_t_1 = (__pyx_v_self->_riterator != 0);
  if (unlikely(__pyx_t_1)) {

    /* "tables/tableextension.pyx":1323
 * 
 *     if self._riterator:
 *       raise NotImplementedError("You cannot append rows when in middle of a table iterator. If what you want is to update records, use Row.update() instead.")             # <<<<<<<<<<<<<<
 * 
 *     # Commit the private record into the write buffer
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_NotImplementedError, __pyx_tuple__10, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1323, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 1323, __pyx_L1_error)

    /* "tables/tableextension.pyx":1322
 *                          h5tb=False)
 * 
 *     if self._riterator:             # <<<<<<<<<<<<<<
 *       raise NotImplementedError("You cannot append rows when in middle of a table iterator. If what you want is to update records, use Row.update() instead.")
 * 
 */
  }

  /* "tables/tableextension.pyx":1328
 *     # self.iobuf[self._unsaved_nrows] = self.wrec
 *     # The next is faster
 *     iobuf = <ndarray>self.iobuf; wrec = <ndarray>self.wrec             # <<<<<<<<<<<<<<
 *     memcpy(PyArray_BYTES(iobuf) + self._unsaved_nrows * self._stride,
 *            PyArray_BYTES(wrec), self._rowsize)
 */
  __pyx_t_4 = __pyx_v_self->iobuf;
  __Pyx_INCREF(__pyx_t_4);
  __pyx_v_iobuf = ((PyArrayObject *)__pyx_t_4);
  __pyx_t_4 = 0;
  __pyx_t_4 = __pyx_v_self->wrec;
  __Pyx_INCREF(__pyx_t_4);
  __pyx_v_wrec = ((PyArrayObject *)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "tables/tableextension.pyx":1329
 *     # The next is faster
 *     iobuf = <ndarray>self.iobuf; wrec = <ndarray>self.wrec
 *     memcpy(PyArray_BYTES(iobuf) + self._unsaved_nrows * self._stride,             # <<<<<<<<<<<<<<
 *            PyArray_BYTES(wrec), self._rowsize)
 *     # Restore the defaults for the private record
 */
  (void)(memcpy((PyArray_BYTES(__pyx_v_iobuf) + (__pyx_v_self->_unsaved_nrows * __pyx_v_self->_stride)), PyArray_BYTES(__pyx_v_wrec), __pyx_v_self->_rowsize));

  /* "tables/tableextension.pyx":1334
 *     # self.wrec[:] = self.wreccpy
 *     # The next is faster
 *     wreccpy = <ndarray>self.wreccpy             # <<<<<<<<<<<<<<
 *     memcpy(PyArray_BYTES(wrec), PyArray_BYTES(wreccpy), self._rowsize)
 *     self._unsaved_nrows = self._unsaved_nrows + 1
 */
  __pyx_t_4 = __pyx_v_self->wreccpy;
  __Pyx_INCREF(__pyx_t_4);
  __pyx_v_wreccpy = ((PyArrayObject *)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "tables/tableextension.pyx":1335
 *     # The next is faster
 *     wreccpy = <ndarray>self.wreccpy
 *     memcpy(PyArray_BYTES(wrec), PyArray_BYTES(wreccpy), self._rowsize)             # <<<<<<<<<<<<<<
 *     self._unsaved_nrows = self._unsaved_nrows + 1
 *     # When the buffer is full, flush it
 */
  (void)(memcpy(PyArray_BYTES(__pyx_v_wrec), PyArray_BYTES(__pyx_v_wreccpy), __pyx_v_self->_rowsize));

  /* "tables/tableextension.pyx":1336
 *     wreccpy = <ndarray>self.wreccpy
 *     memcpy(PyArray_BYTES(wrec), PyArray_BYTES(wreccpy), self._rowsize)
 *     self._unsaved_nrows = self._unsaved_nrows + 1             # <<<<<<<<<<<<<<
 *     # When the buffer is full, flush it
 *     if self._unsaved_nrows == self.nrowsinbuf:
 */
  __pyx_v_self->_unsaved_nrows = (__pyx_v_self->_unsaved_nrows + 1);

  /* "tables/tableextension.pyx":1338
 *     self._unsaved_nrows = self._unsaved_nrows + 1
 *     # When the buffer is full, flush it
 *     if self._unsaved_nrows == self.nrowsinbuf:             # <<<<<<<<<<<<<<
 *       self._flush_buffered_rows()
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->_unsaved_nrows == __pyx_v_self->nrowsinbuf) != 0);
  if (__pyx_t_1) {

    /* "tables/tableextension.pyx":1339
 *     # When the buffer is full, flush it
 *     if self._unsaved_nrows == self.nrowsinbuf:
 *       self._flush_buffered_rows()             # <<<<<<<<<<<<<<
 * 
 *   def _flush_buffered_rows(self):
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_flush_buffered_rows); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1339, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_4 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1339, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "tables/tableextension.pyx":1338
 *     self._unsaved_nrows = self._unsaved_nrows + 1
 *     # When the buffer is full, flush it
 *     if self._unsaved_nrows == self.nrowsinbuf:             # <<<<<<<<<<<<<<
 *       self._flush_buffered_rows()
 * 
 */
  }

  /* "tables/tableextension.pyx":1283
 * 
 * 
 *   def append(self):             # <<<<<<<<<<<<<<
 *     """Add a new row of data to the end of the dataset.
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("tables.tableextension.Row.append", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_iobuf);
  __Pyx_XDECREF((PyObject *)__pyx_v_wrec);
  __Pyx_XDECREF((PyObject *)__pyx_v_wreccpy);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":1341
 *       self._flush_buffered_rows()
 * 
 *   def _flush_buffered_rows(self):             # <<<<<<<<<<<<<<
 *     if self._unsaved_nrows > 0:
 *       self.table._save_buffered_rows(self.iobuf, self._unsaved_nrows)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_14tableextension_3Row_13_flush_buffered_rows(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_6tables_14tableextension_3Row_13_flush_buffered_rows(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_flush_buffered_rows (wrapper)", 0);
  __pyx_r = __pyx_pf_6tables_14tableextension_3Row_12_flush_buffered_rows(((struct __pyx_obj_6tables_14tableextension_Row *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_14tableextension_3Row_12_flush_buffered_rows(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_flush_buffered_rows", 0);

  /* "tables/tableextension.pyx":1342
 * 
 *   def _flush_buffered_rows(self):
 *     if self._unsaved_nrows > 0:             # <<<<<<<<<<<<<<
 *       self.table._save_buffered_rows(self.iobuf, self._unsaved_nrows)
 *       # Reset the buffer unsaved counter
 */
  __pyx_t_1 = ((__pyx_v_self->_unsaved_nrows > 0) != 0);
  if (__pyx_t_1) {

    /* "tables/tableextension.pyx":1343
 *   def _flush_buffered_rows(self):
 *     if self._unsaved_nrows > 0:
 *       self.table._save_buffered_rows(self.iobuf, self._unsaved_nrows)             # <<<<<<<<<<<<<<
 *       # Reset the buffer unsaved counter
 *       self._unsaved_nrows = 0
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_table); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1343, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_save_buffered_rows); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1343, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyInt_From_long(__pyx_v_self->_unsaved_nrows); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1343, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = NULL;
    __pyx_t_6 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_6 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_self->iobuf, __pyx_t_3};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1343, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_self->iobuf, __pyx_t_3};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1343, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    {
      __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1343, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      if (__pyx_t_5) {
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
      }
      __Pyx_INCREF(__pyx_v_self->iobuf);
      __Pyx_GIVEREF(__pyx_v_self->iobuf);
      PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_v_self->iobuf);
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_3);
      __pyx_t_3 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_7, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1343, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "tables/tableextension.pyx":1345
 *       self.table._save_buffered_rows(self.iobuf, self._unsaved_nrows)
 *       # Reset the buffer unsaved counter
 *       self._unsaved_nrows = 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_v_self->_unsaved_nrows = 0;

    /* "tables/tableextension.pyx":1342
 * 
 *   def _flush_buffered_rows(self):
 *     if self._unsaved_nrows > 0:             # <<<<<<<<<<<<<<
 *       self.table._save_buffered_rows(self.iobuf, self._unsaved_nrows)
 *       # Reset the buffer unsaved counter
 */
  }

  /* "tables/tableextension.pyx":1341
 *       self._flush_buffered_rows()
 * 
 *   def _flush_buffered_rows(self):             # <<<<<<<<<<<<<<
 *     if self._unsaved_nrows > 0:
 *       self.table._save_buffered_rows(self.iobuf, self._unsaved_nrows)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("tables.tableextension.Row._flush_buffered_rows", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":1348
 * 
 * 
 *   def _get_unsaved_nrows(self):             # <<<<<<<<<<<<<<
 *     return self._unsaved_nrows
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_14tableextension_3Row_15_get_unsaved_nrows(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_6tables_14tableextension_3Row_15_get_unsaved_nrows(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_unsaved_nrows (wrapper)", 0);
  __pyx_r = __pyx_pf_6tables_14tableextension_3Row_14_get_unsaved_nrows(((struct __pyx_obj_6tables_14tableextension_Row *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_14tableextension_3Row_14_get_unsaved_nrows(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_unsaved_nrows", 0);

  /* "tables/tableextension.pyx":1349
 * 
 *   def _get_unsaved_nrows(self):
 *     return self._unsaved_nrows             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v_self->_unsaved_nrows); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1349, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "tables/tableextension.pyx":1348
 * 
 * 
 *   def _get_unsaved_nrows(self):             # <<<<<<<<<<<<<<
 *     return self._unsaved_nrows
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("tables.tableextension.Row._get_unsaved_nrows", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":1352
 * 
 * 
 *   def update(self):             # <<<<<<<<<<<<<<
 *     """Change the data of the current row in the dataset.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_14tableextension_3Row_17update(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6tables_14tableextension_3Row_16update[] = "Change the data of the current row in the dataset.\n\n    This method allows you to modify values in a table when you are in the\n    middle of a table iterator like :meth:`Table.iterrows` or\n    :meth:`Table.where`.\n\n    Once you have filled the proper fields for the current row, calling\n    this method actually changes data in the *output buffer* (which will\n    eventually be dumped to disk).  If you have not set the value of a\n    field, its original value will be used.\n\n    .. warning::\n\n        After completion of the loop in which :meth:`Row.update` has\n        been called, it is always convenient to make a call to\n        :meth:`Table.flush` in order to avoid losing changed rows that\n        may still remain in internal buffers.\n\n    Examples\n    --------\n\n    ::\n\n        for row in table.iterrows(step=10):\n            row['col1'] = row.nrow\n            row['col2'] = 'b'\n            row['col3'] = 0.0\n            row.update()\n        table.flush()\n\n    which modifies every tenth row in table.  Or::\n\n        for row in table.where('col1 > 3'):\n            row['col1'] = row.nrow\n            row['col2'] = 'b'\n            row['col3'] = 0.0\n            row.update()\n        table.flush()\n\n    which just updates the rows with values bigger than 3 in the first\n    column.\n\n    ";
static PyObject *__pyx_pw_6tables_14tableextension_3Row_17update(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("update (wrapper)", 0);
  __pyx_r = __pyx_pf_6tables_14tableextension_3Row_16update(((struct __pyx_obj_6tables_14tableextension_Row *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_14tableextension_3Row_16update(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self) {
  PyArrayObject *__pyx_v_iobufcpy = 0;
  PyArrayObject *__pyx_v_iobuf = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("update", 0);

  /* "tables/tableextension.pyx":1399
 *     cdef ndarray iobufcpy, iobuf
 * 
 *     if self.ro_filemode:             # <<<<<<<<<<<<<<
 *       raise IOError("Attempt to write over a file opened in read-only mode")
 * 
 */
  __pyx_t_1 = (__pyx_v_self->ro_filemode != 0);
  if (unlikely(__pyx_t_1)) {

    /* "tables/tableextension.pyx":1400
 * 
 *     if self.ro_filemode:
 *       raise IOError("Attempt to write over a file opened in read-only mode")             # <<<<<<<<<<<<<<
 * 
 *     if not self._riterator:
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_IOError, __pyx_tuple__8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1400, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1400, __pyx_L1_error)

    /* "tables/tableextension.pyx":1399
 *     cdef ndarray iobufcpy, iobuf
 * 
 *     if self.ro_filemode:             # <<<<<<<<<<<<<<
 *       raise IOError("Attempt to write over a file opened in read-only mode")
 * 
 */
  }

  /* "tables/tableextension.pyx":1402
 *       raise IOError("Attempt to write over a file opened in read-only mode")
 * 
 *     if not self._riterator:             # <<<<<<<<<<<<<<
 *       raise NotImplementedError("You are only allowed to update rows through the Row.update() method if you are in the middle of a table iterator.")
 * 
 */
  __pyx_t_1 = ((!(__pyx_v_self->_riterator != 0)) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "tables/tableextension.pyx":1403
 * 
 *     if not self._riterator:
 *       raise NotImplementedError("You are only allowed to update rows through the Row.update() method if you are in the middle of a table iterator.")             # <<<<<<<<<<<<<<
 * 
 *     if self.mod_elements is None:
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_NotImplementedError, __pyx_tuple__11, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1403, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1403, __pyx_L1_error)

    /* "tables/tableextension.pyx":1402
 *       raise IOError("Attempt to write over a file opened in read-only mode")
 * 
 *     if not self._riterator:             # <<<<<<<<<<<<<<
 *       raise NotImplementedError("You are only allowed to update rows through the Row.update() method if you are in the middle of a table iterator.")
 * 
 */
  }

  /* "tables/tableextension.pyx":1405
 *       raise NotImplementedError("You are only allowed to update rows through the Row.update() method if you are in the middle of a table iterator.")
 * 
 *     if self.mod_elements is None:             # <<<<<<<<<<<<<<
 *       # Initialize an array for keeping the modified elements
 *       # (just in case Row.update() would be used)
 */
  __pyx_t_1 = (__pyx_v_self->mod_elements == Py_None);
  __pyx_t_3 = (__pyx_t_1 != 0);
  if (__pyx_t_3) {

    /* "tables/tableextension.pyx":1408
 *       # Initialize an array for keeping the modified elements
 *       # (just in case Row.update() would be used)
 *       self.mod_elements = numpy.empty(shape=self.nrowsinbuf, dtype=SizeType)             # <<<<<<<<<<<<<<
 *       # We need a different copy for self.iobuf here
 *       self.iobufcpy = self.iobuf.copy()
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_numpy); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1408, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1408, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1408, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = __Pyx_PyInt_From_hsize_t(__pyx_v_self->nrowsinbuf); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1408, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_shape, __pyx_t_5) < 0) __PYX_ERR(0, 1408, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_SizeType); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1408, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_5) < 0) __PYX_ERR(0, 1408, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1408, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GIVEREF(__pyx_t_5);
    __Pyx_GOTREF(__pyx_v_self->mod_elements);
    __Pyx_DECREF(__pyx_v_self->mod_elements);
    __pyx_v_self->mod_elements = __pyx_t_5;
    __pyx_t_5 = 0;

    /* "tables/tableextension.pyx":1410
 *       self.mod_elements = numpy.empty(shape=self.nrowsinbuf, dtype=SizeType)
 *       # We need a different copy for self.iobuf here
 *       self.iobufcpy = self.iobuf.copy()             # <<<<<<<<<<<<<<
 * 
 *     # Add this row to the list of elements to be modified
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->iobuf, __pyx_n_s_copy); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1410, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_5 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1410, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GIVEREF(__pyx_t_5);
    __Pyx_GOTREF(__pyx_v_self->iobufcpy);
    __Pyx_DECREF(__pyx_v_self->iobufcpy);
    __pyx_v_self->iobufcpy = __pyx_t_5;
    __pyx_t_5 = 0;

    /* "tables/tableextension.pyx":1405
 *       raise NotImplementedError("You are only allowed to update rows through the Row.update() method if you are in the middle of a table iterator.")
 * 
 *     if self.mod_elements is None:             # <<<<<<<<<<<<<<
 *       # Initialize an array for keeping the modified elements
 *       # (just in case Row.update() would be used)
 */
  }

  /* "tables/tableextension.pyx":1413
 * 
 *     # Add this row to the list of elements to be modified
 *     self.mod_elements[self._mod_nrows] = self._nrow             # <<<<<<<<<<<<<<
 *     # Copy the current buffer row in input to the output buffer
 *     # self.iobufcpy[self._mod_nrows] = self.iobuf[self._row]
 */
  __pyx_t_5 = __Pyx_PyInt_From_PY_LONG_LONG(__pyx_v_self->_nrow); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1413, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (unlikely(__Pyx_SetItemInt(__pyx_v_self->mod_elements, __pyx_v_self->_mod_nrows, __pyx_t_5, long, 1, __Pyx_PyInt_From_long, 0, 1, 1) < 0)) __PYX_ERR(0, 1413, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "tables/tableextension.pyx":1417
 *     # self.iobufcpy[self._mod_nrows] = self.iobuf[self._row]
 *     # The next is faster
 *     iobufcpy = <ndarray>self.iobufcpy; iobuf = <ndarray>self.iobuf             # <<<<<<<<<<<<<<
 *     memcpy(PyArray_BYTES(iobufcpy) + self._mod_nrows * self._stride,
 *            PyArray_BYTES(iobuf) + self._row * self._stride, self._rowsize)
 */
  __pyx_t_5 = __pyx_v_self->iobufcpy;
  __Pyx_INCREF(__pyx_t_5);
  __pyx_v_iobufcpy = ((PyArrayObject *)__pyx_t_5);
  __pyx_t_5 = 0;
  __pyx_t_5 = __pyx_v_self->iobuf;
  __Pyx_INCREF(__pyx_t_5);
  __pyx_v_iobuf = ((PyArrayObject *)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "tables/tableextension.pyx":1418
 *     # The next is faster
 *     iobufcpy = <ndarray>self.iobufcpy; iobuf = <ndarray>self.iobuf
 *     memcpy(PyArray_BYTES(iobufcpy) + self._mod_nrows * self._stride,             # <<<<<<<<<<<<<<
 *            PyArray_BYTES(iobuf) + self._row * self._stride, self._rowsize)
 *     # Increase the modified buffer count by one
 */
  (void)(memcpy((PyArray_BYTES(__pyx_v_iobufcpy) + (__pyx_v_self->_mod_nrows * __pyx_v_self->_stride)), (PyArray_BYTES(__pyx_v_iobuf) + (__pyx_v_self->_row * __pyx_v_self->_stride)), __pyx_v_self->_rowsize));

  /* "tables/tableextension.pyx":1421
 *            PyArray_BYTES(iobuf) + self._row * self._stride, self._rowsize)
 *     # Increase the modified buffer count by one
 *     self._mod_nrows = self._mod_nrows + 1             # <<<<<<<<<<<<<<
 *     # No point writing seqcache -- Table.flush will invalidate it
 *     # since we no longer know whether this row will meet _where_condition
 */
  __pyx_v_self->_mod_nrows = (__pyx_v_self->_mod_nrows + 1);

  /* "tables/tableextension.pyx":1424
 *     # No point writing seqcache -- Table.flush will invalidate it
 *     # since we no longer know whether this row will meet _where_condition
 *     self._write_to_seqcache = 0             # <<<<<<<<<<<<<<
 *     # When the buffer is full, flush it
 *     if self._mod_nrows == self.nrowsinbuf:
 */
  __pyx_v_self->_write_to_seqcache = 0;

  /* "tables/tableextension.pyx":1426
 *     self._write_to_seqcache = 0
 *     # When the buffer is full, flush it
 *     if self._mod_nrows == self.nrowsinbuf:             # <<<<<<<<<<<<<<
 *       self._flush_mod_rows()
 * 
 */
  __pyx_t_3 = ((__pyx_v_self->_mod_nrows == __pyx_v_self->nrowsinbuf) != 0);
  if (__pyx_t_3) {

    /* "tables/tableextension.pyx":1427
 *     # When the buffer is full, flush it
 *     if self._mod_nrows == self.nrowsinbuf:
 *       self._flush_mod_rows()             # <<<<<<<<<<<<<<
 * 
 *   def _flush_mod_rows(self):
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_flush_mod_rows); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1427, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_5 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1427, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "tables/tableextension.pyx":1426
 *     self._write_to_seqcache = 0
 *     # When the buffer is full, flush it
 *     if self._mod_nrows == self.nrowsinbuf:             # <<<<<<<<<<<<<<
 *       self._flush_mod_rows()
 * 
 */
  }

  /* "tables/tableextension.pyx":1352
 * 
 * 
 *   def update(self):             # <<<<<<<<<<<<<<
 *     """Change the data of the current row in the dataset.
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("tables.tableextension.Row.update", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_iobufcpy);
  __Pyx_XDECREF((PyObject *)__pyx_v_iobuf);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":1429
 *       self._flush_mod_rows()
 * 
 *   def _flush_mod_rows(self):             # <<<<<<<<<<<<<<
 *     """Flush any possible modified row using Row.update()"""
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_14tableextension_3Row_19_flush_mod_rows(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6tables_14tableextension_3Row_18_flush_mod_rows[] = "Flush any possible modified row using Row.update()";
static PyObject *__pyx_pw_6tables_14tableextension_3Row_19_flush_mod_rows(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_flush_mod_rows (wrapper)", 0);
  __pyx_r = __pyx_pf_6tables_14tableextension_3Row_18_flush_mod_rows(((struct __pyx_obj_6tables_14tableextension_Row *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_14tableextension_3Row_18_flush_mod_rows(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self) {
  PyObject *__pyx_v_table = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_flush_mod_rows", 0);

  /* "tables/tableextension.pyx":1432
 *     """Flush any possible modified row using Row.update()"""
 * 
 *     table = self.table             # <<<<<<<<<<<<<<
 *     # Save the records on disk
 *     table._update_elements(self._mod_nrows, self.mod_elements, self.iobufcpy)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_table); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1432, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_table = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":1434
 *     table = self.table
 *     # Save the records on disk
 *     table._update_elements(self._mod_nrows, self.mod_elements, self.iobufcpy)             # <<<<<<<<<<<<<<
 *     # Reset the counter of modified rows to 0
 *     self._mod_nrows = 0
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_table, __pyx_n_s_update_elements); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1434, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_long(__pyx_v_self->_mod_nrows); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1434, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_t_3, __pyx_v_self->mod_elements, __pyx_v_self->iobufcpy};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1434, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_t_3, __pyx_v_self->mod_elements, __pyx_v_self->iobufcpy};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1434, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_6 = PyTuple_New(3+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1434, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_self->mod_elements);
    __Pyx_GIVEREF(__pyx_v_self->mod_elements);
    PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_self->mod_elements);
    __Pyx_INCREF(__pyx_v_self->iobufcpy);
    __Pyx_GIVEREF(__pyx_v_self->iobufcpy);
    PyTuple_SET_ITEM(__pyx_t_6, 2+__pyx_t_5, __pyx_v_self->iobufcpy);
    __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1434, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":1436
 *     table._update_elements(self._mod_nrows, self.mod_elements, self.iobufcpy)
 *     # Reset the counter of modified rows to 0
 *     self._mod_nrows = 0             # <<<<<<<<<<<<<<
 *     # Mark the modified fields' indexes as dirty.
 *     table._mark_columns_as_dirty(self.modified_fields)
 */
  __pyx_v_self->_mod_nrows = 0;

  /* "tables/tableextension.pyx":1438
 *     self._mod_nrows = 0
 *     # Mark the modified fields' indexes as dirty.
 *     table._mark_columns_as_dirty(self.modified_fields)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_table, __pyx_n_s_mark_columns_as_dirty); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1438, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_6, __pyx_v_self->modified_fields) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_self->modified_fields);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1438, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":1429
 *       self._flush_mod_rows()
 * 
 *   def _flush_mod_rows(self):             # <<<<<<<<<<<<<<
 *     """Flush any possible modified row using Row.update()"""
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("tables.tableextension.Row._flush_mod_rows", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_table);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":1441
 * 
 * 
 *   def __contains__(self, item):             # <<<<<<<<<<<<<<
 *     """__contains__(item)
 * 
 */

/* Python wrapper */
static int __pyx_pw_6tables_14tableextension_3Row_21__contains__(PyObject *__pyx_v_self, PyObject *__pyx_v_item); /*proto*/
static char __pyx_doc_6tables_14tableextension_3Row_20__contains__[] = "__contains__(item)\n\n    A true value is returned if item is found in current row, false\n    otherwise.\n\n    ";
#if CYTHON_COMPILING_IN_CPYTHON
struct wrapperbase __pyx_wrapperbase_6tables_14tableextension_3Row_20__contains__;
#endif
static int __pyx_pw_6tables_14tableextension_3Row_21__contains__(PyObject *__pyx_v_self, PyObject *__pyx_v_item) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__contains__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6tables_14tableextension_3Row_20__contains__(((struct __pyx_obj_6tables_14tableextension_Row *)__pyx_v_self), ((PyObject *)__pyx_v_item));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6tables_14tableextension_3Row_20__contains__(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self, PyObject *__pyx_v_item) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__contains__", 0);

  /* "tables/tableextension.pyx":1449
 *     """
 * 
 *     return item in self.fetch_all_fields()             # <<<<<<<<<<<<<<
 * 
 *   # This method is twice as faster than __getattr__ because there is
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_fetch_all_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1449, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1449, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = (__Pyx_PySequence_ContainsTF(__pyx_v_item, __pyx_t_1, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 1449, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_4;
  goto __pyx_L0;

  /* "tables/tableextension.pyx":1441
 * 
 * 
 *   def __contains__(self, item):             # <<<<<<<<<<<<<<
 *     """__contains__(item)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("tables.tableextension.Row.__contains__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":1453
 *   # This method is twice as faster than __getattr__ because there is
 *   # not a lookup in the local dictionary
 *   def __getitem__(self, key):             # <<<<<<<<<<<<<<
 *     """__getitem__(key)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_14tableextension_3Row_23__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key); /*proto*/
static char __pyx_doc_6tables_14tableextension_3Row_22__getitem__[] = "__getitem__(key)\n\n    Get the row field specified by the `key`.\n\n    The key can be a string (the name of the field), an integer (the\n    position of the field) or a slice (the range of field positions). When\n    key is a slice, the returned value is a *tuple* containing the values\n    of the specified fields.\n\n    Examples\n    --------\n\n    ::\n\n        res = [row['var3'] for row in table.where('var2 < 20')]\n\n    which selects the var3 field for all the rows that fulfil the\n    condition. Or::\n\n        res = [row[4] for row in table if row[1] < 20]\n\n    which selects the field in the *4th* position for all the rows that\n    fulfil the condition. Or::\n\n        res = [row[:] for row in table if row['var2'] < 20]\n\n    which selects the all the fields (in the form of a *tuple*) for all the\n    rows that fulfil the condition. Or::\n\n        res = [row[1::2] for row in table.iterrows(2, 3000, 3)]\n\n    which selects all the fields in even positions (in the form of a\n    *tuple*) for all the rows in the slice [2:3000:3].\n\n    ";
#if CYTHON_COMPILING_IN_CPYTHON
struct wrapperbase __pyx_wrapperbase_6tables_14tableextension_3Row_22__getitem__;
#endif
static PyObject *__pyx_pw_6tables_14tableextension_3Row_23__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6tables_14tableextension_3Row_22__getitem__(((struct __pyx_obj_6tables_14tableextension_Row *)__pyx_v_self), ((PyObject *)__pyx_v_key));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_14tableextension_3Row_22__getitem__(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self, PyObject *__pyx_v_key) {
  long __pyx_v_offset;
  PyArrayObject *__pyx_v_field = 0;
  PyObject *__pyx_v_row = 0;
  PyObject *__pyx_v_fields = 0;
  PyObject *__pyx_v_fieldscache = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "tables/tableextension.pyx":1494
 *     cdef object row, fields, fieldscache
 * 
 *     if self._riterator:             # <<<<<<<<<<<<<<
 *       # If in the middle of an iterator loop, the user probably wants to
 *       # access the read buffer
 */
  __pyx_t_1 = (__pyx_v_self->_riterator != 0);
  if (__pyx_t_1) {

    /* "tables/tableextension.pyx":1497
 *       # If in the middle of an iterator loop, the user probably wants to
 *       # access the read buffer
 *       fieldscache = self.rfieldscache; fields = self.rfields             # <<<<<<<<<<<<<<
 *       offset = <long>self._row
 *     else:
 */
    __pyx_t_2 = __pyx_v_self->rfieldscache;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v_fieldscache = __pyx_t_2;
    __pyx_t_2 = 0;
    __pyx_t_2 = __pyx_v_self->rfields;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v_fields = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "tables/tableextension.pyx":1498
 *       # access the read buffer
 *       fieldscache = self.rfieldscache; fields = self.rfields
 *       offset = <long>self._row             # <<<<<<<<<<<<<<
 *     else:
 *       # We are not in an iterator loop, so the user probably wants to access
 */
    __pyx_v_offset = ((long)__pyx_v_self->_row);

    /* "tables/tableextension.pyx":1494
 *     cdef object row, fields, fieldscache
 * 
 *     if self._riterator:             # <<<<<<<<<<<<<<
 *       # If in the middle of an iterator loop, the user probably wants to
 *       # access the read buffer
 */
    goto __pyx_L3;
  }

  /* "tables/tableextension.pyx":1502
 *       # We are not in an iterator loop, so the user probably wants to access
 *       # the write buffer
 *       fieldscache = self.wfieldscache; fields = self.wfields             # <<<<<<<<<<<<<<
 *       offset = 0
 * 
 */
  /*else*/ {
    __pyx_t_2 = __pyx_v_self->wfieldscache;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v_fieldscache = __pyx_t_2;
    __pyx_t_2 = 0;
    __pyx_t_2 = __pyx_v_self->wfields;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v_fields = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "tables/tableextension.pyx":1503
 *       # the write buffer
 *       fieldscache = self.wfieldscache; fields = self.wfields
 *       offset = 0             # <<<<<<<<<<<<<<
 * 
 *     try:
 */
    __pyx_v_offset = 0;
  }
  __pyx_L3:;

  /* "tables/tableextension.pyx":1505
 *       offset = 0
 * 
 *     try:             # <<<<<<<<<<<<<<
 *       # Check whether this object is in the cache dictionary
 *       field = fieldscache[key]
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    /*try:*/ {

      /* "tables/tableextension.pyx":1507
 *     try:
 *       # Check whether this object is in the cache dictionary
 *       field = fieldscache[key]             # <<<<<<<<<<<<<<
 *     except (KeyError, TypeError):
 *       try:
 */
      __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_fieldscache, __pyx_v_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1507, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1507, __pyx_L4_error)
      __pyx_v_field = ((PyArrayObject *)__pyx_t_2);
      __pyx_t_2 = 0;

      /* "tables/tableextension.pyx":1505
 *       offset = 0
 * 
 *     try:             # <<<<<<<<<<<<<<
 *       # Check whether this object is in the cache dictionary
 *       field = fieldscache[key]
 */
    }
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    goto __pyx_L9_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "tables/tableextension.pyx":1508
 *       # Check whether this object is in the cache dictionary
 *       field = fieldscache[key]
 *     except (KeyError, TypeError):             # <<<<<<<<<<<<<<
 *       try:
 *         # Try to get it from fields (str or int keys)
 */
    __pyx_t_6 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_KeyError) || __Pyx_PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_6) {
      __Pyx_AddTraceback("tables.tableextension.Row.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_7, &__pyx_t_8) < 0) __PYX_ERR(0, 1508, __pyx_L6_except_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_GOTREF(__pyx_t_8);

      /* "tables/tableextension.pyx":1509
 *       field = fieldscache[key]
 *     except (KeyError, TypeError):
 *       try:             # <<<<<<<<<<<<<<
 *         # Try to get it from fields (str or int keys)
 *         field = get_nested_field_cache(fields, key, fieldscache)
 */
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_9, &__pyx_t_10, &__pyx_t_11);
        __Pyx_XGOTREF(__pyx_t_9);
        __Pyx_XGOTREF(__pyx_t_10);
        __Pyx_XGOTREF(__pyx_t_11);
        /*try:*/ {

          /* "tables/tableextension.pyx":1511
 *       try:
 *         # Try to get it from fields (str or int keys)
 *         field = get_nested_field_cache(fields, key, fieldscache)             # <<<<<<<<<<<<<<
 *       except TypeError:
 *         # No luck yet. Still, the key can be a slice.
 */
          __pyx_t_12 = __pyx_f_6tables_14tableextension_get_nested_field_cache(__pyx_v_fields, __pyx_v_key, __pyx_v_fieldscache); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1511, __pyx_L12_error)
          __Pyx_GOTREF(__pyx_t_12);
          if (!(likely(((__pyx_t_12) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_12, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1511, __pyx_L12_error)
          __Pyx_XDECREF_SET(__pyx_v_field, ((PyArrayObject *)__pyx_t_12));
          __pyx_t_12 = 0;

          /* "tables/tableextension.pyx":1509
 *       field = fieldscache[key]
 *     except (KeyError, TypeError):
 *       try:             # <<<<<<<<<<<<<<
 *         # Try to get it from fields (str or int keys)
 *         field = get_nested_field_cache(fields, key, fieldscache)
 */
        }
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        goto __pyx_L19_try_end;
        __pyx_L12_error:;
        __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;

        /* "tables/tableextension.pyx":1512
 *         # Try to get it from fields (str or int keys)
 *         field = get_nested_field_cache(fields, key, fieldscache)
 *       except TypeError:             # <<<<<<<<<<<<<<
 *         # No luck yet. Still, the key can be a slice.
 *         # Fetch the complete row and convert it into a tuple
 */
        __pyx_t_6 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_TypeError);
        if (__pyx_t_6) {
          __Pyx_AddTraceback("tables.tableextension.Row.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_12, &__pyx_t_13, &__pyx_t_14) < 0) __PYX_ERR(0, 1512, __pyx_L14_except_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_GOTREF(__pyx_t_13);
          __Pyx_GOTREF(__pyx_t_14);

          /* "tables/tableextension.pyx":1515
 *         # No luck yet. Still, the key can be a slice.
 *         # Fetch the complete row and convert it into a tuple
 *         if self._riterator:             # <<<<<<<<<<<<<<
 *           row = self.iobuf[self._row].copy().item()
 *         else:
 */
          __pyx_t_1 = (__pyx_v_self->_riterator != 0);
          if (__pyx_t_1) {

            /* "tables/tableextension.pyx":1516
 *         # Fetch the complete row and convert it into a tuple
 *         if self._riterator:
 *           row = self.iobuf[self._row].copy().item()             # <<<<<<<<<<<<<<
 *         else:
 *           row = self.wrec[0].copy().item()
 */
            __pyx_t_17 = __Pyx_GetItemInt(__pyx_v_self->iobuf, __pyx_v_self->_row, long, 1, __Pyx_PyInt_From_long, 0, 1, 1); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1516, __pyx_L14_except_error)
            __Pyx_GOTREF(__pyx_t_17);
            __pyx_t_18 = __Pyx_PyObject_GetAttrStr(__pyx_t_17, __pyx_n_s_copy); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 1516, __pyx_L14_except_error)
            __Pyx_GOTREF(__pyx_t_18);
            __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
            __pyx_t_17 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_18))) {
              __pyx_t_17 = PyMethod_GET_SELF(__pyx_t_18);
              if (likely(__pyx_t_17)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_18);
                __Pyx_INCREF(__pyx_t_17);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_18, function);
              }
            }
            __pyx_t_16 = (__pyx_t_17) ? __Pyx_PyObject_CallOneArg(__pyx_t_18, __pyx_t_17) : __Pyx_PyObject_CallNoArg(__pyx_t_18);
            __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
            if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 1516, __pyx_L14_except_error)
            __Pyx_GOTREF(__pyx_t_16);
            __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
            __pyx_t_18 = __Pyx_PyObject_GetAttrStr(__pyx_t_16, __pyx_n_s_item); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 1516, __pyx_L14_except_error)
            __Pyx_GOTREF(__pyx_t_18);
            __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
            __pyx_t_16 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_18))) {
              __pyx_t_16 = PyMethod_GET_SELF(__pyx_t_18);
              if (likely(__pyx_t_16)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_18);
                __Pyx_INCREF(__pyx_t_16);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_18, function);
              }
            }
            __pyx_t_15 = (__pyx_t_16) ? __Pyx_PyObject_CallOneArg(__pyx_t_18, __pyx_t_16) : __Pyx_PyObject_CallNoArg(__pyx_t_18);
            __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
            if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 1516, __pyx_L14_except_error)
            __Pyx_GOTREF(__pyx_t_15);
            __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
            __pyx_v_row = __pyx_t_15;
            __pyx_t_15 = 0;

            /* "tables/tableextension.pyx":1515
 *         # No luck yet. Still, the key can be a slice.
 *         # Fetch the complete row and convert it into a tuple
 *         if self._riterator:             # <<<<<<<<<<<<<<
 *           row = self.iobuf[self._row].copy().item()
 *         else:
 */
            goto __pyx_L22;
          }

          /* "tables/tableextension.pyx":1518
 *           row = self.iobuf[self._row].copy().item()
 *         else:
 *           row = self.wrec[0].copy().item()             # <<<<<<<<<<<<<<
 *         # Try with __getitem__()
 *         return row[key]
 */
          /*else*/ {
            __pyx_t_16 = __Pyx_GetItemInt(__pyx_v_self->wrec, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 1518, __pyx_L14_except_error)
            __Pyx_GOTREF(__pyx_t_16);
            __pyx_t_17 = __Pyx_PyObject_GetAttrStr(__pyx_t_16, __pyx_n_s_copy); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1518, __pyx_L14_except_error)
            __Pyx_GOTREF(__pyx_t_17);
            __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
            __pyx_t_16 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_17))) {
              __pyx_t_16 = PyMethod_GET_SELF(__pyx_t_17);
              if (likely(__pyx_t_16)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_17);
                __Pyx_INCREF(__pyx_t_16);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_17, function);
              }
            }
            __pyx_t_18 = (__pyx_t_16) ? __Pyx_PyObject_CallOneArg(__pyx_t_17, __pyx_t_16) : __Pyx_PyObject_CallNoArg(__pyx_t_17);
            __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
            if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 1518, __pyx_L14_except_error)
            __Pyx_GOTREF(__pyx_t_18);
            __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
            __pyx_t_17 = __Pyx_PyObject_GetAttrStr(__pyx_t_18, __pyx_n_s_item); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1518, __pyx_L14_except_error)
            __Pyx_GOTREF(__pyx_t_17);
            __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
            __pyx_t_18 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_17))) {
              __pyx_t_18 = PyMethod_GET_SELF(__pyx_t_17);
              if (likely(__pyx_t_18)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_17);
                __Pyx_INCREF(__pyx_t_18);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_17, function);
              }
            }
            __pyx_t_15 = (__pyx_t_18) ? __Pyx_PyObject_CallOneArg(__pyx_t_17, __pyx_t_18) : __Pyx_PyObject_CallNoArg(__pyx_t_17);
            __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
            if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 1518, __pyx_L14_except_error)
            __Pyx_GOTREF(__pyx_t_15);
            __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
            __pyx_v_row = __pyx_t_15;
            __pyx_t_15 = 0;
          }
          __pyx_L22:;

          /* "tables/tableextension.pyx":1520
 *           row = self.wrec[0].copy().item()
 *         # Try with __getitem__()
 *         return row[key]             # <<<<<<<<<<<<<<
 * 
 *     if PyArray_NDIM(field) == 1:
 */
          __Pyx_XDECREF(__pyx_r);
          __pyx_t_15 = __Pyx_PyObject_GetItem(__pyx_v_row, __pyx_v_key); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 1520, __pyx_L14_except_error)
          __Pyx_GOTREF(__pyx_t_15);
          __pyx_r = __pyx_t_15;
          __pyx_t_15 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          goto __pyx_L15_except_return;
        }
        goto __pyx_L14_except_error;
        __pyx_L14_except_error:;

        /* "tables/tableextension.pyx":1509
 *       field = fieldscache[key]
 *     except (KeyError, TypeError):
 *       try:             # <<<<<<<<<<<<<<
 *         # Try to get it from fields (str or int keys)
 *         field = get_nested_field_cache(fields, key, fieldscache)
 */
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
        goto __pyx_L6_except_error;
        __pyx_L15_except_return:;
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
        goto __pyx_L7_except_return;
        __pyx_L19_try_end:;
      }
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;

    /* "tables/tableextension.pyx":1505
 *       offset = 0
 * 
 *     try:             # <<<<<<<<<<<<<<
 *       # Check whether this object is in the cache dictionary
 *       field = fieldscache[key]
 */
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
    goto __pyx_L1_error;
    __pyx_L7_except_return:;
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
    goto __pyx_L0;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
    __pyx_L9_try_end:;
  }

  /* "tables/tableextension.pyx":1522
 *         return row[key]
 * 
 *     if PyArray_NDIM(field) == 1:             # <<<<<<<<<<<<<<
 *       # For an scalar it is not needed a copy (immutable object)
 *       return PyArray_GETITEM(field, PyArray_BYTES(field) + offset * self._stride)
 */
  __pyx_t_1 = ((PyArray_NDIM(__pyx_v_field) == 1) != 0);
  if (__pyx_t_1) {

    /* "tables/tableextension.pyx":1524
 *     if PyArray_NDIM(field) == 1:
 *       # For an scalar it is not needed a copy (immutable object)
 *       return PyArray_GETITEM(field, PyArray_BYTES(field) + offset * self._stride)             # <<<<<<<<<<<<<<
 *     else:
 *       # Do a copy of the array, so that it can be overwritten by the user
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_8 = PyArray_GETITEM(__pyx_v_field, (PyArray_BYTES(__pyx_v_field) + (__pyx_v_offset * __pyx_v_self->_stride))); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1524, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;

    /* "tables/tableextension.pyx":1522
 *         return row[key]
 * 
 *     if PyArray_NDIM(field) == 1:             # <<<<<<<<<<<<<<
 *       # For an scalar it is not needed a copy (immutable object)
 *       return PyArray_GETITEM(field, PyArray_BYTES(field) + offset * self._stride)
 */
  }

  /* "tables/tableextension.pyx":1528
 *       # Do a copy of the array, so that it can be overwritten by the user
 *       # without damaging the internal self.rfields buffer
 *       return field[offset].copy()             # <<<<<<<<<<<<<<
 * 
 *   # This is slightly faster (around 3%) than __setattr__
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_7 = __Pyx_GetItemInt(((PyObject *)__pyx_v_field), __pyx_v_offset, long, 1, __Pyx_PyInt_From_long, 0, 1, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1528, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_copy); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1528, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_8 = (__pyx_t_7) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_7) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1528, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* "tables/tableextension.pyx":1453
 *   # This method is twice as faster than __getattr__ because there is
 *   # not a lookup in the local dictionary
 *   def __getitem__(self, key):             # <<<<<<<<<<<<<<
 *     """__getitem__(key)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("tables.tableextension.Row.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_field);
  __Pyx_XDECREF(__pyx_v_row);
  __Pyx_XDECREF(__pyx_v_fields);
  __Pyx_XDECREF(__pyx_v_fieldscache);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":1531
 * 
 *   # This is slightly faster (around 3%) than __setattr__
 *   def __setitem__(self, object key, object value):             # <<<<<<<<<<<<<<
 *     """__setitem__(key, value)
 * 
 */

/* Python wrapper */
static int __pyx_pw_6tables_14tableextension_3Row_25__setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value); /*proto*/
static char __pyx_doc_6tables_14tableextension_3Row_24__setitem__[] = "__setitem__(key, value)\n\n    Set the key row field to the specified value.\n\n    Differently from its __getitem__() counterpart, in this case key can\n    only be a string (the name of the field). The changes done via\n    __setitem__() will not take effect on the data on disk until any of the\n    :meth:`Row.append` or :meth:`Row.update` methods are called.\n\n    Examples\n    --------\n\n    ::\n\n        for row in table.iterrows(step=10):\n            row['col1'] = row.nrow\n            row['col2'] = 'b'\n            row['col3'] = 0.0\n            row.update()\n        table.flush()\n\n    which modifies every tenth row in the table.\n\n    ";
#if CYTHON_COMPILING_IN_CPYTHON
struct wrapperbase __pyx_wrapperbase_6tables_14tableextension_3Row_24__setitem__;
#endif
static int __pyx_pw_6tables_14tableextension_3Row_25__setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setitem__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6tables_14tableextension_3Row_24__setitem__(((struct __pyx_obj_6tables_14tableextension_Row *)__pyx_v_self), ((PyObject *)__pyx_v_key), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6tables_14tableextension_3Row_24__setitem__(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value) {
  int __pyx_v_ret;
  long __pyx_v_offset;
  PyArrayObject *__pyx_v_field = 0;
  PyObject *__pyx_v_fields = 0;
  PyObject *__pyx_v_fieldscache = 0;
  PyObject *__pyx_v_enum = NULL;
  PyObject *__pyx_v_cenval = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  PyObject *(*__pyx_t_7)(PyObject *);
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setitem__", 0);

  /* "tables/tableextension.pyx":1562
 *     cdef object fields, fieldscache
 * 
 *     if self.ro_filemode:             # <<<<<<<<<<<<<<
 *       raise IOError("attempt to write over a file opened in read-only mode")
 * 
 */
  __pyx_t_1 = (__pyx_v_self->ro_filemode != 0);
  if (unlikely(__pyx_t_1)) {

    /* "tables/tableextension.pyx":1563
 * 
 *     if self.ro_filemode:
 *       raise IOError("attempt to write over a file opened in read-only mode")             # <<<<<<<<<<<<<<
 * 
 *     if self._riterator:
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_IOError, __pyx_tuple__12, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1563, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1563, __pyx_L1_error)

    /* "tables/tableextension.pyx":1562
 *     cdef object fields, fieldscache
 * 
 *     if self.ro_filemode:             # <<<<<<<<<<<<<<
 *       raise IOError("attempt to write over a file opened in read-only mode")
 * 
 */
  }

  /* "tables/tableextension.pyx":1565
 *       raise IOError("attempt to write over a file opened in read-only mode")
 * 
 *     if self._riterator:             # <<<<<<<<<<<<<<
 *       # If in the middle of an iterator loop, or *after*, the user
 *       # probably wants to access the read buffer
 */
  __pyx_t_1 = (__pyx_v_self->_riterator != 0);
  if (__pyx_t_1) {

    /* "tables/tableextension.pyx":1568
 *       # If in the middle of an iterator loop, or *after*, the user
 *       # probably wants to access the read buffer
 *       fieldscache = self.rfieldscache; fields = self.rfields             # <<<<<<<<<<<<<<
 *       offset = <long>self._row
 *     else:
 */
    __pyx_t_2 = __pyx_v_self->rfieldscache;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v_fieldscache = __pyx_t_2;
    __pyx_t_2 = 0;
    __pyx_t_2 = __pyx_v_self->rfields;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v_fields = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "tables/tableextension.pyx":1569
 *       # probably wants to access the read buffer
 *       fieldscache = self.rfieldscache; fields = self.rfields
 *       offset = <long>self._row             # <<<<<<<<<<<<<<
 *     else:
 *       # We are not in an iterator loop, so the user probably wants to access
 */
    __pyx_v_offset = ((long)__pyx_v_self->_row);

    /* "tables/tableextension.pyx":1565
 *       raise IOError("attempt to write over a file opened in read-only mode")
 * 
 *     if self._riterator:             # <<<<<<<<<<<<<<
 *       # If in the middle of an iterator loop, or *after*, the user
 *       # probably wants to access the read buffer
 */
    goto __pyx_L4;
  }

  /* "tables/tableextension.pyx":1573
 *       # We are not in an iterator loop, so the user probably wants to access
 *       # the write buffer
 *       fieldscache = self.wfieldscache; fields = self.wfields             # <<<<<<<<<<<<<<
 *       offset = 0
 * 
 */
  /*else*/ {
    __pyx_t_2 = __pyx_v_self->wfieldscache;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v_fieldscache = __pyx_t_2;
    __pyx_t_2 = 0;
    __pyx_t_2 = __pyx_v_self->wfields;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v_fields = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "tables/tableextension.pyx":1574
 *       # the write buffer
 *       fieldscache = self.wfieldscache; fields = self.wfields
 *       offset = 0             # <<<<<<<<<<<<<<
 * 
 *     # Check validity of enumerated value.
 */
    __pyx_v_offset = 0;
  }
  __pyx_L4:;

  /* "tables/tableextension.pyx":1577
 * 
 *     # Check validity of enumerated value.
 *     if self.exist_enum_cols:             # <<<<<<<<<<<<<<
 *       if key in self.colenums:
 *         enum = self.colenums[key]
 */
  __pyx_t_1 = (__pyx_v_self->exist_enum_cols != 0);
  if (__pyx_t_1) {

    /* "tables/tableextension.pyx":1578
 *     # Check validity of enumerated value.
 *     if self.exist_enum_cols:
 *       if key in self.colenums:             # <<<<<<<<<<<<<<
 *         enum = self.colenums[key]
 *         for cenval in numpy.asarray(value).flat:
 */
    __pyx_t_1 = (__Pyx_PySequence_ContainsTF(__pyx_v_key, __pyx_v_self->colenums, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 1578, __pyx_L1_error)
    __pyx_t_3 = (__pyx_t_1 != 0);
    if (__pyx_t_3) {

      /* "tables/tableextension.pyx":1579
 *     if self.exist_enum_cols:
 *       if key in self.colenums:
 *         enum = self.colenums[key]             # <<<<<<<<<<<<<<
 *         for cenval in numpy.asarray(value).flat:
 *           enum(cenval)  # raises ``ValueError`` on invalid values
 */
      __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_self->colenums, __pyx_v_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1579, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_v_enum = __pyx_t_2;
      __pyx_t_2 = 0;

      /* "tables/tableextension.pyx":1580
 *       if key in self.colenums:
 *         enum = self.colenums[key]
 *         for cenval in numpy.asarray(value).flat:             # <<<<<<<<<<<<<<
 *           enum(cenval)  # raises ``ValueError`` on invalid values
 * 
 */
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_numpy); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1580, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_asarray); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1580, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_4, __pyx_v_value) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_value);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1580, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_flat); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1580, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (likely(PyList_CheckExact(__pyx_t_5)) || PyTuple_CheckExact(__pyx_t_5)) {
        __pyx_t_2 = __pyx_t_5; __Pyx_INCREF(__pyx_t_2); __pyx_t_6 = 0;
        __pyx_t_7 = NULL;
      } else {
        __pyx_t_6 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1580, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_7 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1580, __pyx_L1_error)
      }
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      for (;;) {
        if (likely(!__pyx_t_7)) {
          if (likely(PyList_CheckExact(__pyx_t_2))) {
            if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_2)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_5 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_6); __Pyx_INCREF(__pyx_t_5); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 1580, __pyx_L1_error)
            #else
            __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1580, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_5);
            #endif
          } else {
            if (__pyx_t_6 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_6); __Pyx_INCREF(__pyx_t_5); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 1580, __pyx_L1_error)
            #else
            __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1580, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_5);
            #endif
          }
        } else {
          __pyx_t_5 = __pyx_t_7(__pyx_t_2);
          if (unlikely(!__pyx_t_5)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
              else __PYX_ERR(0, 1580, __pyx_L1_error)
            }
            break;
          }
          __Pyx_GOTREF(__pyx_t_5);
        }
        __Pyx_XDECREF_SET(__pyx_v_cenval, __pyx_t_5);
        __pyx_t_5 = 0;

        /* "tables/tableextension.pyx":1581
 *         enum = self.colenums[key]
 *         for cenval in numpy.asarray(value).flat:
 *           enum(cenval)  # raises ``ValueError`` on invalid values             # <<<<<<<<<<<<<<
 * 
 *     # Get the field to be modified
 */
        __Pyx_INCREF(__pyx_v_enum);
        __pyx_t_4 = __pyx_v_enum; __pyx_t_8 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_8)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_8);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
          }
        }
        __pyx_t_5 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_8, __pyx_v_cenval) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_cenval);
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1581, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

        /* "tables/tableextension.pyx":1580
 *       if key in self.colenums:
 *         enum = self.colenums[key]
 *         for cenval in numpy.asarray(value).flat:             # <<<<<<<<<<<<<<
 *           enum(cenval)  # raises ``ValueError`` on invalid values
 * 
 */
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "tables/tableextension.pyx":1578
 *     # Check validity of enumerated value.
 *     if self.exist_enum_cols:
 *       if key in self.colenums:             # <<<<<<<<<<<<<<
 *         enum = self.colenums[key]
 *         for cenval in numpy.asarray(value).flat:
 */
    }

    /* "tables/tableextension.pyx":1577
 * 
 *     # Check validity of enumerated value.
 *     if self.exist_enum_cols:             # <<<<<<<<<<<<<<
 *       if key in self.colenums:
 *         enum = self.colenums[key]
 */
  }

  /* "tables/tableextension.pyx":1584
 * 
 *     # Get the field to be modified
 *     field = get_nested_field_cache(fields, key, fieldscache)             # <<<<<<<<<<<<<<
 *     if key not in self.modified_fields:
 *       self.modified_fields.add(key)
 */
  __pyx_t_2 = __pyx_f_6tables_14tableextension_get_nested_field_cache(__pyx_v_fields, __pyx_v_key, __pyx_v_fieldscache); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1584, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1584, __pyx_L1_error)
  __pyx_v_field = ((PyArrayObject *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "tables/tableextension.pyx":1585
 *     # Get the field to be modified
 *     field = get_nested_field_cache(fields, key, fieldscache)
 *     if key not in self.modified_fields:             # <<<<<<<<<<<<<<
 *       self.modified_fields.add(key)
 * 
 */
  __pyx_t_3 = (__Pyx_PySequence_ContainsTF(__pyx_v_key, __pyx_v_self->modified_fields, Py_NE)); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 1585, __pyx_L1_error)
  __pyx_t_1 = (__pyx_t_3 != 0);
  if (__pyx_t_1) {

    /* "tables/tableextension.pyx":1586
 *     field = get_nested_field_cache(fields, key, fieldscache)
 *     if key not in self.modified_fields:
 *       self.modified_fields.add(key)             # <<<<<<<<<<<<<<
 * 
 *     # Finally, try to set it to the value
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->modified_fields, __pyx_n_s_add); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1586, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_4, __pyx_v_key) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_key);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1586, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "tables/tableextension.pyx":1585
 *     # Get the field to be modified
 *     field = get_nested_field_cache(fields, key, fieldscache)
 *     if key not in self.modified_fields:             # <<<<<<<<<<<<<<
 *       self.modified_fields.add(key)
 * 
 */
  }

  /* "tables/tableextension.pyx":1589
 * 
 *     # Finally, try to set it to the value
 *     try:             # <<<<<<<<<<<<<<
 *       # Optimization for scalar values. This can optimize the writes
 *       # between a 10% and 100%, depending on the number of columns modified
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_9, &__pyx_t_10, &__pyx_t_11);
    __Pyx_XGOTREF(__pyx_t_9);
    __Pyx_XGOTREF(__pyx_t_10);
    __Pyx_XGOTREF(__pyx_t_11);
    /*try:*/ {

      /* "tables/tableextension.pyx":1592
 *       # Optimization for scalar values. This can optimize the writes
 *       # between a 10% and 100%, depending on the number of columns modified
 *       if PyArray_NDIM(field) == 1:             # <<<<<<<<<<<<<<
 *         ret = PyArray_SETITEM(field, PyArray_BYTES(field) + offset * self._stride, value)
 *         if ret < 0:
 */
      __pyx_t_1 = ((PyArray_NDIM(__pyx_v_field) == 1) != 0);
      if (__pyx_t_1) {

        /* "tables/tableextension.pyx":1593
 *       # between a 10% and 100%, depending on the number of columns modified
 *       if PyArray_NDIM(field) == 1:
 *         ret = PyArray_SETITEM(field, PyArray_BYTES(field) + offset * self._stride, value)             # <<<<<<<<<<<<<<
 *         if ret < 0:
 *           PyErr_Clear()
 */
        __pyx_v_ret = PyArray_SETITEM(__pyx_v_field, (PyArray_BYTES(__pyx_v_field) + (__pyx_v_offset * __pyx_v_self->_stride)), __pyx_v_value);

        /* "tables/tableextension.pyx":1594
 *       if PyArray_NDIM(field) == 1:
 *         ret = PyArray_SETITEM(field, PyArray_BYTES(field) + offset * self._stride, value)
 *         if ret < 0:             # <<<<<<<<<<<<<<
 *           PyErr_Clear()
 *           raise TypeError
 */
        __pyx_t_1 = ((__pyx_v_ret < 0) != 0);
        if (unlikely(__pyx_t_1)) {

          /* "tables/tableextension.pyx":1595
 *         ret = PyArray_SETITEM(field, PyArray_BYTES(field) + offset * self._stride, value)
 *         if ret < 0:
 *           PyErr_Clear()             # <<<<<<<<<<<<<<
 *           raise TypeError
 *       ##### End of optimization for scalar values
 */
          PyErr_Clear();

          /* "tables/tableextension.pyx":1596
 *         if ret < 0:
 *           PyErr_Clear()
 *           raise TypeError             # <<<<<<<<<<<<<<
 *       ##### End of optimization for scalar values
 *       else:
 */
          __Pyx_Raise(__pyx_builtin_TypeError, 0, 0, 0);
          __PYX_ERR(0, 1596, __pyx_L10_error)

          /* "tables/tableextension.pyx":1594
 *       if PyArray_NDIM(field) == 1:
 *         ret = PyArray_SETITEM(field, PyArray_BYTES(field) + offset * self._stride, value)
 *         if ret < 0:             # <<<<<<<<<<<<<<
 *           PyErr_Clear()
 *           raise TypeError
 */
        }

        /* "tables/tableextension.pyx":1592
 *       # Optimization for scalar values. This can optimize the writes
 *       # between a 10% and 100%, depending on the number of columns modified
 *       if PyArray_NDIM(field) == 1:             # <<<<<<<<<<<<<<
 *         ret = PyArray_SETITEM(field, PyArray_BYTES(field) + offset * self._stride, value)
 *         if ret < 0:
 */
        goto __pyx_L16;
      }

      /* "tables/tableextension.pyx":1599
 *       ##### End of optimization for scalar values
 *       else:
 *         field[offset] = value             # <<<<<<<<<<<<<<
 *     except TypeError:
 *       raise TypeError("invalid type (%s) for column ``%s``" % (type(value),
 */
      /*else*/ {
        if (unlikely(__Pyx_SetItemInt(((PyObject *)__pyx_v_field), __pyx_v_offset, __pyx_v_value, long, 1, __Pyx_PyInt_From_long, 0, 1, 1) < 0)) __PYX_ERR(0, 1599, __pyx_L10_error)
      }
      __pyx_L16:;

      /* "tables/tableextension.pyx":1589
 * 
 *     # Finally, try to set it to the value
 *     try:             # <<<<<<<<<<<<<<
 *       # Optimization for scalar values. This can optimize the writes
 *       # between a 10% and 100%, depending on the number of columns modified
 */
    }
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    goto __pyx_L15_try_end;
    __pyx_L10_error:;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "tables/tableextension.pyx":1600
 *       else:
 *         field[offset] = value
 *     except TypeError:             # <<<<<<<<<<<<<<
 *       raise TypeError("invalid type (%s) for column ``%s``" % (type(value),
 *                                                                key))
 */
    __pyx_t_12 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_12) {
      __Pyx_AddTraceback("tables.tableextension.Row.__setitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_5, &__pyx_t_4) < 0) __PYX_ERR(0, 1600, __pyx_L12_except_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_4);

      /* "tables/tableextension.pyx":1601
 *         field[offset] = value
 *     except TypeError:
 *       raise TypeError("invalid type (%s) for column ``%s``" % (type(value),             # <<<<<<<<<<<<<<
 *                                                                key))
 * 
 */
      __pyx_t_8 = PyTuple_New(2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1601, __pyx_L12_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_INCREF(((PyObject *)Py_TYPE(__pyx_v_value)));
      __Pyx_GIVEREF(((PyObject *)Py_TYPE(__pyx_v_value)));
      PyTuple_SET_ITEM(__pyx_t_8, 0, ((PyObject *)Py_TYPE(__pyx_v_value)));
      __Pyx_INCREF(__pyx_v_key);
      __Pyx_GIVEREF(__pyx_v_key);
      PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_v_key);
      __pyx_t_13 = __Pyx_PyString_Format(__pyx_kp_s_invalid_type_s_for_column_s, __pyx_t_8); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1601, __pyx_L12_except_error)
      __Pyx_GOTREF(__pyx_t_13);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = __Pyx_PyObject_CallOneArg(__pyx_builtin_TypeError, __pyx_t_13); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1601, __pyx_L12_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(0, 1601, __pyx_L12_except_error)
    }
    goto __pyx_L12_except_error;
    __pyx_L12_except_error:;

    /* "tables/tableextension.pyx":1589
 * 
 *     # Finally, try to set it to the value
 *     try:             # <<<<<<<<<<<<<<
 *       # Optimization for scalar values. This can optimize the writes
 *       # between a 10% and 100%, depending on the number of columns modified
 */
    __Pyx_XGIVEREF(__pyx_t_9);
    __Pyx_XGIVEREF(__pyx_t_10);
    __Pyx_XGIVEREF(__pyx_t_11);
    __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
    goto __pyx_L1_error;
    __pyx_L15_try_end:;
  }

  /* "tables/tableextension.pyx":1531
 * 
 *   # This is slightly faster (around 3%) than __setattr__
 *   def __setitem__(self, object key, object value):             # <<<<<<<<<<<<<<
 *     """__setitem__(key, value)
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("tables.tableextension.Row.__setitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_field);
  __Pyx_XDECREF(__pyx_v_fields);
  __Pyx_XDECREF(__pyx_v_fieldscache);
  __Pyx_XDECREF(__pyx_v_enum);
  __Pyx_XDECREF(__pyx_v_cenval);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":1604
 *                                                                key))
 * 
 *   def fetch_all_fields(self):             # <<<<<<<<<<<<<<
 *     """Retrieve all the fields in the current row.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_14tableextension_3Row_27fetch_all_fields(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6tables_14tableextension_3Row_26fetch_all_fields[] = "Retrieve all the fields in the current row.\n\n    Contrarily to row[:] (see :ref:`RowSpecialMethods`), this returns row\n    data as a NumPy void scalar.  For instance::\n\n        [row.fetch_all_fields() for row in table.where('col1 < 3')]\n\n    will select all the rows that fulfill the given condition\n    as a list of NumPy records.\n\n    ";
static PyObject *__pyx_pw_6tables_14tableextension_3Row_27fetch_all_fields(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("fetch_all_fields (wrapper)", 0);
  __pyx_r = __pyx_pf_6tables_14tableextension_3Row_26fetch_all_fields(((struct __pyx_obj_6tables_14tableextension_Row *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_14tableextension_3Row_26fetch_all_fields(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("fetch_all_fields", 0);

  /* "tables/tableextension.pyx":1618
 * 
 *     # We need to do a cast for recognizing negative row numbers!
 *     if <signed long long>self._nrow < 0:             # <<<<<<<<<<<<<<
 *       return ("Warning: Row iterator has not been initialized for table:\n"
 *               "  %s\n"
 */
  __pyx_t_1 = ((((PY_LONG_LONG)__pyx_v_self->_nrow) < 0) != 0);
  if (__pyx_t_1) {

    /* "tables/tableextension.pyx":1619
 *     # We need to do a cast for recognizing negative row numbers!
 *     if <signed long long>self._nrow < 0:
 *       return ("Warning: Row iterator has not been initialized for table:\n"             # <<<<<<<<<<<<<<
 *               "  %s\n"
 *               " You will normally want to use this method in iterator "
 */
    __Pyx_XDECREF(__pyx_r);

    /* "tables/tableextension.pyx":1622
 *               "  %s\n"
 *               " You will normally want to use this method in iterator "
 *               "contexts." % self.table)             # <<<<<<<<<<<<<<
 * 
 *     # Always return a copy of the row so that new data that is written
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_table); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1622, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyString_FormatSafe(__pyx_kp_s_Warning_Row_iterator_has_not_bee, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1622, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "tables/tableextension.pyx":1618
 * 
 *     # We need to do a cast for recognizing negative row numbers!
 *     if <signed long long>self._nrow < 0:             # <<<<<<<<<<<<<<
 *       return ("Warning: Row iterator has not been initialized for table:\n"
 *               "  %s\n"
 */
  }

  /* "tables/tableextension.pyx":1626
 *     # Always return a copy of the row so that new data that is written
 *     # in self.iobuf doesn't overwrite the original returned data.
 *     return self.iobuf[self._row].copy()             # <<<<<<<<<<<<<<
 * 
 *   def __str__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_self->iobuf, __pyx_v_self->_row, long, 1, __Pyx_PyInt_From_long, 0, 1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1626, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_copy); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1626, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_3 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1626, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "tables/tableextension.pyx":1604
 *                                                                key))
 * 
 *   def fetch_all_fields(self):             # <<<<<<<<<<<<<<
 *     """Retrieve all the fields in the current row.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("tables.tableextension.Row.fetch_all_fields", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":1628
 *     return self.iobuf[self._row].copy()
 * 
 *   def __str__(self):             # <<<<<<<<<<<<<<
 *     """Represent the record as an string"""
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_14tableextension_3Row_29__str__(PyObject *__pyx_v_self); /*proto*/
static char __pyx_doc_6tables_14tableextension_3Row_28__str__[] = "Represent the record as an string";
#if CYTHON_COMPILING_IN_CPYTHON
struct wrapperbase __pyx_wrapperbase_6tables_14tableextension_3Row_28__str__;
#endif
static PyObject *__pyx_pw_6tables_14tableextension_3Row_29__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6tables_14tableextension_3Row_28__str__(((struct __pyx_obj_6tables_14tableextension_Row *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_14tableextension_3Row_28__str__(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self) {
  PyObject *__pyx_v_tablepathname = NULL;
  PyObject *__pyx_v_classname = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "tables/tableextension.pyx":1632
 * 
 *     # We need to do a cast for recognizing negative row numbers!
 *     if <signed long long>self._nrow < 0:             # <<<<<<<<<<<<<<
 *       return ("Warning: Row iterator has not been initialized for table:\n"
 *               "  %s\n"
 */
  __pyx_t_1 = ((((PY_LONG_LONG)__pyx_v_self->_nrow) < 0) != 0);
  if (__pyx_t_1) {

    /* "tables/tableextension.pyx":1633
 *     # We need to do a cast for recognizing negative row numbers!
 *     if <signed long long>self._nrow < 0:
 *       return ("Warning: Row iterator has not been initialized for table:\n"             # <<<<<<<<<<<<<<
 *               "  %s\n"
 *               " You will normally want to use this object in iterator "
 */
    __Pyx_XDECREF(__pyx_r);

    /* "tables/tableextension.pyx":1636
 *               "  %s\n"
 *               " You will normally want to use this object in iterator "
 *               "contexts." % self.table)             # <<<<<<<<<<<<<<
 * 
 *     tablepathname = self.table._v_pathname
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_table); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1636, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyString_FormatSafe(__pyx_kp_s_Warning_Row_iterator_has_not_bee_2, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1636, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "tables/tableextension.pyx":1632
 * 
 *     # We need to do a cast for recognizing negative row numbers!
 *     if <signed long long>self._nrow < 0:             # <<<<<<<<<<<<<<
 *       return ("Warning: Row iterator has not been initialized for table:\n"
 *               "  %s\n"
 */
  }

  /* "tables/tableextension.pyx":1638
 *               "contexts." % self.table)
 * 
 *     tablepathname = self.table._v_pathname             # <<<<<<<<<<<<<<
 *     classname = self.__class__.__name__
 *     return "%s.row (%s), pointing to row #%d" %  (tablepathname, classname,
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_table); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1638, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_v_pathname); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1638, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_tablepathname = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "tables/tableextension.pyx":1639
 * 
 *     tablepathname = self.table._v_pathname
 *     classname = self.__class__.__name__             # <<<<<<<<<<<<<<
 *     return "%s.row (%s), pointing to row #%d" %  (tablepathname, classname,
 *                                                   self._nrow)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_class); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1639, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_name); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1639, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_classname = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "tables/tableextension.pyx":1640
 *     tablepathname = self.table._v_pathname
 *     classname = self.__class__.__name__
 *     return "%s.row (%s), pointing to row #%d" %  (tablepathname, classname,             # <<<<<<<<<<<<<<
 *                                                   self._nrow)
 * 
 */
  __Pyx_XDECREF(__pyx_r);

  /* "tables/tableextension.pyx":1641
 *     classname = self.__class__.__name__
 *     return "%s.row (%s), pointing to row #%d" %  (tablepathname, classname,
 *                                                   self._nrow)             # <<<<<<<<<<<<<<
 * 
 *   def __repr__(self):
 */
  __pyx_t_3 = __Pyx_PyInt_From_PY_LONG_LONG(__pyx_v_self->_nrow); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1641, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "tables/tableextension.pyx":1640
 *     tablepathname = self.table._v_pathname
 *     classname = self.__class__.__name__
 *     return "%s.row (%s), pointing to row #%d" %  (tablepathname, classname,             # <<<<<<<<<<<<<<
 *                                                   self._nrow)
 * 
 */
  __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1640, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_tablepathname);
  __Pyx_GIVEREF(__pyx_v_tablepathname);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_tablepathname);
  __Pyx_INCREF(__pyx_v_classname);
  __Pyx_GIVEREF(__pyx_v_classname);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_classname);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_s_row_s_pointing_to_row_d, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1640, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "tables/tableextension.pyx":1628
 *     return self.iobuf[self._row].copy()
 * 
 *   def __str__(self):             # <<<<<<<<<<<<<<
 *     """Represent the record as an string"""
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("tables.tableextension.Row.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_tablepathname);
  __Pyx_XDECREF(__pyx_v_classname);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/tableextension.pyx":1643
 *                                                   self._nrow)
 * 
 *   def __repr__(self):             # <<<<<<<<<<<<<<
 *     """Represent the record as an string"""
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_14tableextension_3Row_31__repr__(PyObject *__pyx_v_self); /*proto*/
static char __pyx_doc_6tables_14tableextension_3Row_30__repr__[] = "Represent the record as an string";
#if CYTHON_COMPILING_IN_CPYTHON
struct wrapperbase __pyx_wrapperbase_6tables_14tableextension_3Row_30__repr__;
#endif
static PyObject *__pyx_pw_6tables_14tableextension_3Row_31__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6tables_14tableextension_3Row_30__repr__(((struct __pyx_obj_6tables_14tableextension_Row *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_14tableextension_3Row_30__repr__(struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "tables/tableextension.pyx":1646
 *     """Represent the record as an string"""
 * 
 *     return str(self)             # <<<<<<<<<<<<<<
 * 
 * ## Local Variables:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1646, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "tables/tableextension.pyx":1643
 *                                                   self._nrow)
 * 
 *   def __repr__(self):             # <<<<<<<<<<<<<<
 *     """Represent the record as an string"""
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("tables.tableextension.Row.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_14tableextension_3Row_33__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_6tables_14tableextension_3Row_33__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6tables_14tableextension_3Row_32__reduce_cython__(((struct __pyx_obj_6tables_14tableextension_Row *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_14tableextension_3Row_32__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__13, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("tables.tableextension.Row.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6tables_14tableextension_3Row_35__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_6tables_14tableextension_3Row_35__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6tables_14tableextension_3Row_34__setstate_cython__(((struct __pyx_obj_6tables_14tableextension_Row *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_14tableextension_3Row_34__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_14tableextension_Row *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__14, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("tables.tableextension.Row.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":735
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew1(PyObject *__pyx_v_a) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew1", 0);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":736
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 *     return PyArray_MultiIterNew(1, <void*>a)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(1, ((void *)__pyx_v_a)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 736, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":735
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":738
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew2(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew2", 0);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":739
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(2, ((void *)__pyx_v_a), ((void *)__pyx_v_b)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 739, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":738
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew2", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":741
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew3(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew3", 0);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":742
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(3, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 742, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":741
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew3", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":744
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew4(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew4", 0);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":745
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(4, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 745, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":744
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew4", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":747
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew5(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d, PyObject *__pyx_v_e) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew5", 0);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":748
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)             # <<<<<<<<<<<<<<
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(5, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d), ((void *)__pyx_v_e)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 748, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":747
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew5", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":750
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyDataType_SHAPE(PyArray_Descr *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("PyDataType_SHAPE", 0);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":751
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */
  __pyx_t_1 = (PyDataType_HASSUBARRAY(__pyx_v_d) != 0);
  if (__pyx_t_1) {

    /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":752
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape             # <<<<<<<<<<<<<<
 *     else:
 *         return ()
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject*)__pyx_v_d->subarray->shape));
    __pyx_r = ((PyObject*)__pyx_v_d->subarray->shape);
    goto __pyx_L0;

    /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":751
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */
  }

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":754
 *         return <tuple>d.subarray.shape
 *     else:
 *         return ()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_empty_tuple);
    __pyx_r = __pyx_empty_tuple;
    goto __pyx_L0;
  }

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":750
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":929
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

static CYTHON_INLINE void __pyx_f_5numpy_set_array_base(PyArrayObject *__pyx_v_arr, PyObject *__pyx_v_base) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_array_base", 0);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":930
 * 
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!             # <<<<<<<<<<<<<<
 *     PyArray_SetBaseObject(arr, base)
 * 
 */
  Py_INCREF(__pyx_v_base);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":931
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object get_array_base(ndarray arr):
 */
  (void)(PyArray_SetBaseObject(__pyx_v_arr, __pyx_v_base));

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":929
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":933
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_get_array_base(PyArrayObject *__pyx_v_arr) {
  PyObject *__pyx_v_base;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("get_array_base", 0);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":934
 * 
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)             # <<<<<<<<<<<<<<
 *     if base is NULL:
 *         return None
 */
  __pyx_v_base = PyArray_BASE(__pyx_v_arr);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":935
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */
  __pyx_t_1 = ((__pyx_v_base == NULL) != 0);
  if (__pyx_t_1) {

    /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":936
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 *         return None             # <<<<<<<<<<<<<<
 *     return <object>base
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":935
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */
  }

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":937
 *     if base is NULL:
 *         return None
 *     return <object>base             # <<<<<<<<<<<<<<
 * 
 * # Versions of the import_* functions which are more suitable for
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_base));
  __pyx_r = ((PyObject *)__pyx_v_base);
  goto __pyx_L0;

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":933
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":941
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         __pyx_import_array()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_array(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_array", 0);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":942
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":943
 * cdef inline int import_array() except -1:
 *     try:
 *         __pyx_import_array()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")
 */
      __pyx_t_4 = _import_array(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 943, __pyx_L3_error)

      /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":942
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":944
 *     try:
 *         __pyx_import_array()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(2, 944, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":945
 *         __pyx_import_array()
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__15, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 945, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(2, 945, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":942
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":941
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         __pyx_import_array()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":947
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_umath(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_umath", 0);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":948
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":949
 * cdef inline int import_umath() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")
 */
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 949, __pyx_L3_error)

      /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":948
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":950
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(2, 950, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":951
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__16, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 951, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(2, 951, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":948
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":947
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":953
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_ufunc(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_ufunc", 0);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":954
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":955
 * cdef inline int import_ufunc() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")
 */
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 955, __pyx_L3_error)

      /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":954
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":956
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(2, 956, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":957
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef extern from *:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__16, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 957, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(2, 957, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":954
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":953
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":967
 * 
 * 
 * cdef inline bint is_timedelta64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.timedelta64)`
 */

static CYTHON_INLINE int __pyx_f_5numpy_is_timedelta64_object(PyObject *__pyx_v_obj) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_timedelta64_object", 0);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":979
 *     bool
 *     """
 *     return PyObject_TypeCheck(obj, &PyTimedeltaArrType_Type)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = PyObject_TypeCheck(__pyx_v_obj, (&PyTimedeltaArrType_Type));
  goto __pyx_L0;

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":967
 * 
 * 
 * cdef inline bint is_timedelta64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.timedelta64)`
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":982
 * 
 * 
 * cdef inline bint is_datetime64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.datetime64)`
 */

static CYTHON_INLINE int __pyx_f_5numpy_is_datetime64_object(PyObject *__pyx_v_obj) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_datetime64_object", 0);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":994
 *     bool
 *     """
 *     return PyObject_TypeCheck(obj, &PyDatetimeArrType_Type)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = PyObject_TypeCheck(__pyx_v_obj, (&PyDatetimeArrType_Type));
  goto __pyx_L0;

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":982
 * 
 * 
 * cdef inline bint is_datetime64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.datetime64)`
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":997
 * 
 * 
 * cdef inline npy_datetime get_datetime64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy datetime64 object
 */

static CYTHON_INLINE npy_datetime __pyx_f_5numpy_get_datetime64_value(PyObject *__pyx_v_obj) {
  npy_datetime __pyx_r;

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":1004
 *     also needed.  That can be found using `get_datetime64_unit`.
 *     """
 *     return (<PyDatetimeScalarObject*>obj).obval             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((PyDatetimeScalarObject *)__pyx_v_obj)->obval;
  goto __pyx_L0;

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":997
 * 
 * 
 * cdef inline npy_datetime get_datetime64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy datetime64 object
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":1007
 * 
 * 
 * cdef inline npy_timedelta get_timedelta64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy timedelta64 object
 */

static CYTHON_INLINE npy_timedelta __pyx_f_5numpy_get_timedelta64_value(PyObject *__pyx_v_obj) {
  npy_timedelta __pyx_r;

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":1011
 *     returns the int64 value underlying scalar numpy timedelta64 object
 *     """
 *     return (<PyTimedeltaScalarObject*>obj).obval             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((PyTimedeltaScalarObject *)__pyx_v_obj)->obval;
  goto __pyx_L0;

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":1007
 * 
 * 
 * cdef inline npy_timedelta get_timedelta64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy timedelta64 object
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":1014
 * 
 * 
 * cdef inline NPY_DATETIMEUNIT get_datetime64_unit(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the unit part of the dtype for a numpy datetime64 object.
 */

static CYTHON_INLINE NPY_DATETIMEUNIT __pyx_f_5numpy_get_datetime64_unit(PyObject *__pyx_v_obj) {
  NPY_DATETIMEUNIT __pyx_r;

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":1018
 *     returns the unit part of the dtype for a numpy datetime64 object.
 *     """
 *     return <NPY_DATETIMEUNIT>(<PyDatetimeScalarObject*>obj).obmeta.base             # <<<<<<<<<<<<<<
 */
  __pyx_r = ((NPY_DATETIMEUNIT)((PyDatetimeScalarObject *)__pyx_v_obj)->obmeta.base);
  goto __pyx_L0;

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":1014
 * 
 * 
 * cdef inline NPY_DATETIMEUNIT get_datetime64_unit(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the unit part of the dtype for a numpy datetime64 object.
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}
static struct __pyx_vtabstruct_6tables_14tableextension_Table __pyx_vtable_6tables_14tableextension_Table;

static PyObject *__pyx_tp_new_6tables_14tableextension_Table(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6tables_14tableextension_Table *p;
  PyObject *o = __pyx_ptype_6tables_13hdf5extension_Leaf->tp_new(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6tables_14tableextension_Table *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6tables_13hdf5extension_Leaf*)__pyx_vtabptr_6tables_14tableextension_Table;
  return o;
}

static void __pyx_tp_dealloc_6tables_14tableextension_Table(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  PyObject_GC_Track(o);
  if (likely(__pyx_ptype_6tables_13hdf5extension_Leaf)) __pyx_ptype_6tables_13hdf5extension_Leaf->tp_dealloc(o); else __Pyx_call_next_tp_dealloc(o, __pyx_tp_dealloc_6tables_14tableextension_Table);
}

static int __pyx_tp_traverse_6tables_14tableextension_Table(PyObject *o, visitproc v, void *a) {
  int e;
  e = ((likely(__pyx_ptype_6tables_13hdf5extension_Leaf)) ? ((__pyx_ptype_6tables_13hdf5extension_Leaf->tp_traverse) ? __pyx_ptype_6tables_13hdf5extension_Leaf->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_6tables_14tableextension_Table)); if (e) return e;
  return 0;
}

static int __pyx_tp_clear_6tables_14tableextension_Table(PyObject *o) {
  if (likely(__pyx_ptype_6tables_13hdf5extension_Leaf)) { if (__pyx_ptype_6tables_13hdf5extension_Leaf->tp_clear) __pyx_ptype_6tables_13hdf5extension_Leaf->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_6tables_14tableextension_Table);
  return 0;
}

static PyMethodDef __pyx_methods_6tables_14tableextension_Table[] = {
  {"_create_table", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6tables_14tableextension_5Table_1_create_table, METH_VARARGS|METH_KEYWORDS, 0},
  {"_get_info", (PyCFunction)__pyx_pw_6tables_14tableextension_5Table_3_get_info, METH_NOARGS, __pyx_doc_6tables_14tableextension_5Table_2_get_info},
  {"_convert_types", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6tables_14tableextension_5Table_5_convert_types, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6tables_14tableextension_5Table_4_convert_types},
  {"_open_append", (PyCFunction)__pyx_pw_6tables_14tableextension_5Table_7_open_append, METH_O, 0},
  {"_append_records", (PyCFunction)__pyx_pw_6tables_14tableextension_5Table_9_append_records, METH_O, 0},
  {"_close_append", (PyCFunction)__pyx_pw_6tables_14tableextension_5Table_11_close_append, METH_NOARGS, 0},
  {"_update_records", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6tables_14tableextension_5Table_13_update_records, METH_VARARGS|METH_KEYWORDS, 0},
  {"_update_elements", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6tables_14tableextension_5Table_15_update_elements, METH_VARARGS|METH_KEYWORDS, 0},
  {"_read_records", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6tables_14tableextension_5Table_17_read_records, METH_VARARGS|METH_KEYWORDS, 0},
  {"_read_elements", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6tables_14tableextension_5Table_19_read_elements, METH_VARARGS|METH_KEYWORDS, 0},
  {"_remove_rows", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6tables_14tableextension_5Table_21_remove_rows, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6tables_14tableextension_5Table_23__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6tables_14tableextension_5Table_25__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6tables_14tableextension_Table = {
  PyVarObject_HEAD_INIT(0, 0)
  "tables.tableextension.Table", /*tp_name*/
  sizeof(struct __pyx_obj_6tables_14tableextension_Table), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6tables_14tableextension_Table, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_6tables_14tableextension_Table, /*tp_traverse*/
  __pyx_tp_clear_6tables_14tableextension_Table, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6tables_14tableextension_Table, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6tables_14tableextension_Table, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM+0 >= 0x06000000
  0, /*tp_pypy_flags*/
  #endif
};
static struct __pyx_vtabstruct_6tables_14tableextension_Row __pyx_vtable_6tables_14tableextension_Row;

static PyObject *__pyx_tp_new_6tables_14tableextension_Row(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6tables_14tableextension_Row *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6tables_14tableextension_Row *)o);
  p->__pyx_vtab = __pyx_vtabptr_6tables_14tableextension_Row;
  p->bufcoords = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  p->indexvalid = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  p->indexvalues = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  p->chunkmap = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  p->dtype = Py_None; Py_INCREF(Py_None);
  p->iobuf = Py_None; Py_INCREF(Py_None);
  p->iobufcpy = Py_None; Py_INCREF(Py_None);
  p->wrec = Py_None; Py_INCREF(Py_None);
  p->wreccpy = Py_None; Py_INCREF(Py_None);
  p->wfields = Py_None; Py_INCREF(Py_None);
  p->rfields = Py_None; Py_INCREF(Py_None);
  p->coords = Py_None; Py_INCREF(Py_None);
  p->condfunc = Py_None; Py_INCREF(Py_None);
  p->condargs = Py_None; Py_INCREF(Py_None);
  p->condkwargs = Py_None; Py_INCREF(Py_None);
  p->mod_elements = Py_None; Py_INCREF(Py_None);
  p->colenums = Py_None; Py_INCREF(Py_None);
  p->rfieldscache = Py_None; Py_INCREF(Py_None);
  p->wfieldscache = Py_None; Py_INCREF(Py_None);
  p->iterseq = Py_None; Py_INCREF(Py_None);
  p->_table_file = Py_None; Py_INCREF(Py_None);
  p->_table_path = Py_None; Py_INCREF(Py_None);
  p->modified_fields = Py_None; Py_INCREF(Py_None);
  p->seqcache_key = Py_None; Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_6tables_14tableextension_3Row_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_6tables_14tableextension_Row(PyObject *o) {
  struct __pyx_obj_6tables_14tableextension_Row *p = (struct __pyx_obj_6tables_14tableextension_Row *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->bufcoords);
  Py_CLEAR(p->indexvalid);
  Py_CLEAR(p->indexvalues);
  Py_CLEAR(p->chunkmap);
  Py_CLEAR(p->dtype);
  Py_CLEAR(p->iobuf);
  Py_CLEAR(p->iobufcpy);
  Py_CLEAR(p->wrec);
  Py_CLEAR(p->wreccpy);
  Py_CLEAR(p->wfields);
  Py_CLEAR(p->rfields);
  Py_CLEAR(p->coords);
  Py_CLEAR(p->condfunc);
  Py_CLEAR(p->condargs);
  Py_CLEAR(p->condkwargs);
  Py_CLEAR(p->mod_elements);
  Py_CLEAR(p->colenums);
  Py_CLEAR(p->rfieldscache);
  Py_CLEAR(p->wfieldscache);
  Py_CLEAR(p->iterseq);
  Py_CLEAR(p->_table_file);
  Py_CLEAR(p->_table_path);
  Py_CLEAR(p->modified_fields);
  Py_CLEAR(p->seqcache_key);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_6tables_14tableextension_Row(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6tables_14tableextension_Row *p = (struct __pyx_obj_6tables_14tableextension_Row *)o;
  if (p->bufcoords) {
    e = (*v)(((PyObject *)p->bufcoords), a); if (e) return e;
  }
  if (p->indexvalid) {
    e = (*v)(((PyObject *)p->indexvalid), a); if (e) return e;
  }
  if (p->indexvalues) {
    e = (*v)(((PyObject *)p->indexvalues), a); if (e) return e;
  }
  if (p->chunkmap) {
    e = (*v)(((PyObject *)p->chunkmap), a); if (e) return e;
  }
  if (p->dtype) {
    e = (*v)(p->dtype, a); if (e) return e;
  }
  if (p->iobuf) {
    e = (*v)(p->iobuf, a); if (e) return e;
  }
  if (p->iobufcpy) {
    e = (*v)(p->iobufcpy, a); if (e) return e;
  }
  if (p->wrec) {
    e = (*v)(p->wrec, a); if (e) return e;
  }
  if (p->wreccpy) {
    e = (*v)(p->wreccpy, a); if (e) return e;
  }
  if (p->wfields) {
    e = (*v)(p->wfields, a); if (e) return e;
  }
  if (p->rfields) {
    e = (*v)(p->rfields, a); if (e) return e;
  }
  if (p->coords) {
    e = (*v)(p->coords, a); if (e) return e;
  }
  if (p->condfunc) {
    e = (*v)(p->condfunc, a); if (e) return e;
  }
  if (p->condargs) {
    e = (*v)(p->condargs, a); if (e) return e;
  }
  if (p->condkwargs) {
    e = (*v)(p->condkwargs, a); if (e) return e;
  }
  if (p->mod_elements) {
    e = (*v)(p->mod_elements, a); if (e) return e;
  }
  if (p->colenums) {
    e = (*v)(p->colenums, a); if (e) return e;
  }
  if (p->rfieldscache) {
    e = (*v)(p->rfieldscache, a); if (e) return e;
  }
  if (p->wfieldscache) {
    e = (*v)(p->wfieldscache, a); if (e) return e;
  }
  if (p->iterseq) {
    e = (*v)(p->iterseq, a); if (e) return e;
  }
  if (p->_table_file) {
    e = (*v)(p->_table_file, a); if (e) return e;
  }
  if (p->_table_path) {
    e = (*v)(p->_table_path, a); if (e) return e;
  }
  if (p->modified_fields) {
    e = (*v)(p->modified_fields, a); if (e) return e;
  }
  if (p->seqcache_key) {
    e = (*v)(p->seqcache_key, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6tables_14tableextension_Row(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6tables_14tableextension_Row *p = (struct __pyx_obj_6tables_14tableextension_Row *)o;
  tmp = ((PyObject*)p->bufcoords);
  p->bufcoords = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->indexvalid);
  p->indexvalid = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->indexvalues);
  p->indexvalues = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->chunkmap);
  p->chunkmap = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->dtype);
  p->dtype = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->iobuf);
  p->iobuf = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->iobufcpy);
  p->iobufcpy = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->wrec);
  p->wrec = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->wreccpy);
  p->wreccpy = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->wfields);
  p->wfields = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->rfields);
  p->rfields = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->coords);
  p->coords = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->condfunc);
  p->condfunc = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->condargs);
  p->condargs = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->condkwargs);
  p->condkwargs = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->mod_elements);
  p->mod_elements = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->colenums);
  p->colenums = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->rfieldscache);
  p->rfieldscache = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->wfieldscache);
  p->wfieldscache = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->iterseq);
  p->iterseq = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_table_file);
  p->_table_file = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_table_path);
  p->_table_path = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->modified_fields);
  p->modified_fields = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->seqcache_key);
  p->seqcache_key = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}
static PyObject *__pyx_sq_item_6tables_14tableextension_Row(PyObject *o, Py_ssize_t i) {
  PyObject *r;
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);
  Py_DECREF(x);
  return r;
}

static int __pyx_mp_ass_subscript_6tables_14tableextension_Row(PyObject *o, PyObject *i, PyObject *v) {
  if (v) {
    return __pyx_pw_6tables_14tableextension_3Row_25__setitem__(o, i, v);
  }
  else {
    PyErr_Format(PyExc_NotImplementedError,
      "Subscript deletion not supported by %.200s", Py_TYPE(o)->tp_name);
    return -1;
  }
}

static PyObject *__pyx_getprop_6tables_14tableextension_3Row_nrow(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6tables_14tableextension_3Row_4nrow_1__get__(o);
}

static PyObject *__pyx_getprop_6tables_14tableextension_3Row_table(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6tables_14tableextension_3Row_5table_1__get__(o);
}

static PyObject *__pyx_specialmethod___pyx_pw_6tables_14tableextension_3Row_7__next__(PyObject *self, CYTHON_UNUSED PyObject *arg) {return __pyx_pw_6tables_14tableextension_3Row_7__next__(self);}

static PyMethodDef __pyx_methods_6tables_14tableextension_Row[] = {
  {"_iter", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6tables_14tableextension_3Row_3_iter, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6tables_14tableextension_3Row_2_iter},
  {"__next__", (PyCFunction)__pyx_specialmethod___pyx_pw_6tables_14tableextension_3Row_7__next__, METH_NOARGS|METH_COEXIST, __pyx_doc_6tables_14tableextension_3Row_6__next__},
  {"_fill_col", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6tables_14tableextension_3Row_9_fill_col, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6tables_14tableextension_3Row_8_fill_col},
  {"append", (PyCFunction)__pyx_pw_6tables_14tableextension_3Row_11append, METH_NOARGS, __pyx_doc_6tables_14tableextension_3Row_10append},
  {"_flush_buffered_rows", (PyCFunction)__pyx_pw_6tables_14tableextension_3Row_13_flush_buffered_rows, METH_NOARGS, 0},
  {"_get_unsaved_nrows", (PyCFunction)__pyx_pw_6tables_14tableextension_3Row_15_get_unsaved_nrows, METH_NOARGS, 0},
  {"update", (PyCFunction)__pyx_pw_6tables_14tableextension_3Row_17update, METH_NOARGS, __pyx_doc_6tables_14tableextension_3Row_16update},
  {"_flush_mod_rows", (PyCFunction)__pyx_pw_6tables_14tableextension_3Row_19_flush_mod_rows, METH_NOARGS, __pyx_doc_6tables_14tableextension_3Row_18_flush_mod_rows},
  {"fetch_all_fields", (PyCFunction)__pyx_pw_6tables_14tableextension_3Row_27fetch_all_fields, METH_NOARGS, __pyx_doc_6tables_14tableextension_3Row_26fetch_all_fields},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6tables_14tableextension_3Row_33__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6tables_14tableextension_3Row_35__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6tables_14tableextension_Row[] = {
  {(char *)"nrow", __pyx_getprop_6tables_14tableextension_3Row_nrow, 0, (char *)"The current row number.\n\n    This property is useful for knowing which row is being dealt with in the\n    middle of a loop or iterator.\n\n    ", 0},
  {(char *)"table", __pyx_getprop_6tables_14tableextension_3Row_table, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PySequenceMethods __pyx_tp_as_sequence_Row = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  __pyx_sq_item_6tables_14tableextension_Row, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  __pyx_pw_6tables_14tableextension_3Row_21__contains__, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_Row = {
  0, /*mp_length*/
  __pyx_pw_6tables_14tableextension_3Row_23__getitem__, /*mp_subscript*/
  __pyx_mp_ass_subscript_6tables_14tableextension_Row, /*mp_ass_subscript*/
};

static PyTypeObject __pyx_type_6tables_14tableextension_Row = {
  PyVarObject_HEAD_INIT(0, 0)
  "tables.tableextension.Row", /*tp_name*/
  sizeof(struct __pyx_obj_6tables_14tableextension_Row), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6tables_14tableextension_Row, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_6tables_14tableextension_3Row_31__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  &__pyx_tp_as_sequence_Row, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_Row, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6tables_14tableextension_3Row_29__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "Table row iterator and field accessor.\n\n  Instances of this class are used to fetch and set the values\n  of individual table fields.  It works very much like a dictionary,\n  where keys are the pathnames or positions (extended slicing is\n  supported) of the fields in the associated table in a specific row.\n\n  This class provides an *iterator interface*\n  so that you can use the same Row instance to\n  access successive table rows one after the other.  There are also\n  some important methods that are useful for accessing, adding and\n  modifying values in tables.\n\n  .. rubric:: Row attributes\n\n  .. attribute:: nrow\n\n      The current row number.\n\n      This property is useful for knowing which row is being dealt with in the\n      middle of a loop or iterator.\n\n  ", /*tp_doc*/
  __pyx_tp_traverse_6tables_14tableextension_Row, /*tp_traverse*/
  __pyx_tp_clear_6tables_14tableextension_Row, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  __pyx_pw_6tables_14tableextension_3Row_5__iter__, /*tp_iter*/
  __pyx_pw_6tables_14tableextension_3Row_7__next__, /*tp_iternext*/
  __pyx_methods_6tables_14tableextension_Row, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6tables_14tableextension_Row, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6tables_14tableextension_Row, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM+0 >= 0x06000000
  0, /*tp_pypy_flags*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_tableextension(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_tableextension},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "tableextension",
    __pyx_k_Here_is_where_Table_and_Row_exte, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_s_, __pyx_k_, sizeof(__pyx_k_), 0, 0, 1, 0},
  {&__pyx_kp_s_Attempt_to_write_over_a_file_ope, __pyx_k_Attempt_to_write_over_a_file_ope, sizeof(__pyx_k_Attempt_to_write_over_a_file_ope), 0, 0, 1, 0},
  {&__pyx_n_s_CLASS, __pyx_k_CLASS, sizeof(__pyx_k_CLASS), 0, 0, 1, 1},
  {&__pyx_kp_s_Can_t_set_attribute_s_in_table_s, __pyx_k_Can_t_set_attribute_s_in_table_s, sizeof(__pyx_k_Can_t_set_attribute_s_in_table_s), 0, 0, 1, 0},
  {&__pyx_n_s_Col, __pyx_k_Col, sizeof(__pyx_k_Col), 0, 0, 1, 1},
  {&__pyx_n_s_H5T_STD_I64, __pyx_k_H5T_STD_I64, sizeof(__pyx_k_H5T_STD_I64), 0, 0, 1, 1},
  {&__pyx_n_s_HDF5ExtError, __pyx_k_HDF5ExtError, sizeof(__pyx_k_HDF5ExtError), 0, 0, 1, 1},
  {&__pyx_n_s_IOError, __pyx_k_IOError, sizeof(__pyx_k_IOError), 0, 0, 1, 1},
  {&__pyx_n_s_ITERSEQ_MAX_ELEMENTS, __pyx_k_ITERSEQ_MAX_ELEMENTS, sizeof(__pyx_k_ITERSEQ_MAX_ELEMENTS), 0, 0, 1, 1},
  {&__pyx_n_s_ImportError, __pyx_k_ImportError, sizeof(__pyx_k_ImportError), 0, 0, 1, 1},
  {&__pyx_n_s_KeyError, __pyx_k_KeyError, sizeof(__pyx_k_KeyError), 0, 0, 1, 1},
  {&__pyx_n_s_NROWS, __pyx_k_NROWS, sizeof(__pyx_k_NROWS), 0, 0, 1, 1},
  {&__pyx_kp_s_Node_s_is_not_a_Table_object, __pyx_k_Node_s_is_not_a_Table_object, sizeof(__pyx_k_Node_s_is_not_a_Table_object), 0, 0, 1, 0},
  {&__pyx_kp_s_Non_existing_node_s_under_s, __pyx_k_Non_existing_node_s_under_s, sizeof(__pyx_k_Non_existing_node_s_under_s), 0, 0, 1, 0},
  {&__pyx_n_s_NotImplementedError, __pyx_k_NotImplementedError, sizeof(__pyx_k_NotImplementedError), 0, 0, 1, 1},
  {&__pyx_n_s_PYTABLES_SYS_ATTRS, __pyx_k_PYTABLES_SYS_ATTRS, sizeof(__pyx_k_PYTABLES_SYS_ATTRS), 0, 0, 1, 1},
  {&__pyx_kp_s_Problems_appending_the_records, __pyx_k_Problems_appending_the_records, sizeof(__pyx_k_Problems_appending_the_records), 0, 0, 1, 0},
  {&__pyx_kp_s_Problems_creating_the_table, __pyx_k_Problems_creating_the_table, sizeof(__pyx_k_Problems_creating_the_table), 0, 0, 1, 0},
  {&__pyx_kp_s_Problems_deleting_records, __pyx_k_Problems_deleting_records, sizeof(__pyx_k_Problems_deleting_records), 0, 0, 1, 0},
  {&__pyx_kp_s_Problems_getting_desciption_for, __pyx_k_Problems_getting_desciption_for, sizeof(__pyx_k_Problems_getting_desciption_for), 0, 0, 1, 0},
  {&__pyx_kp_s_Problems_reading_chunk_records, __pyx_k_Problems_reading_chunk_records, sizeof(__pyx_k_Problems_reading_chunk_records), 0, 0, 1, 0},
  {&__pyx_kp_s_Problems_reading_records, __pyx_k_Problems_reading_records, sizeof(__pyx_k_Problems_reading_records), 0, 0, 1, 0},
  {&__pyx_kp_s_Problems_setting_the_NROWS_attri, __pyx_k_Problems_setting_the_NROWS_attri, sizeof(__pyx_k_Problems_setting_the_NROWS_attri), 0, 0, 1, 0},
  {&__pyx_kp_s_Problems_updating_the_records, __pyx_k_Problems_updating_the_records, sizeof(__pyx_k_Problems_updating_the_records), 0, 0, 1, 0},
  {&__pyx_n_s_Row, __pyx_k_Row, sizeof(__pyx_k_Row), 0, 0, 1, 1},
  {&__pyx_n_s_SizeType, __pyx_k_SizeType, sizeof(__pyx_k_SizeType), 0, 0, 1, 1},
  {&__pyx_n_s_StopIteration, __pyx_k_StopIteration, sizeof(__pyx_k_StopIteration), 0, 0, 1, 1},
  {&__pyx_n_s_TITLE, __pyx_k_TITLE, sizeof(__pyx_k_TITLE), 0, 0, 1, 1},
  {&__pyx_n_s_Table, __pyx_k_Table, sizeof(__pyx_k_Table), 0, 0, 1, 1},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_n_s_VERSION, __pyx_k_VERSION, sizeof(__pyx_k_VERSION), 0, 0, 1, 1},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_kp_s_Warning_Row_iterator_has_not_bee, __pyx_k_Warning_Row_iterator_has_not_bee, sizeof(__pyx_k_Warning_Row_iterator_has_not_bee), 0, 0, 1, 0},
  {&__pyx_kp_s_Warning_Row_iterator_has_not_bee_2, __pyx_k_Warning_Row_iterator_has_not_bee_2, sizeof(__pyx_k_Warning_Row_iterator_has_not_bee_2), 0, 0, 1, 0},
  {&__pyx_kp_s_You_are_only_allowed_to_update_r, __pyx_k_You_are_only_allowed_to_update_r, sizeof(__pyx_k_You_are_only_allowed_to_update_r), 0, 0, 1, 0},
  {&__pyx_kp_s_You_cannot_append_rows_to_a_non, __pyx_k_You_cannot_append_rows_to_a_non, sizeof(__pyx_k_You_cannot_append_rows_to_a_non), 0, 0, 1, 0},
  {&__pyx_kp_s_You_cannot_append_rows_when_in_m, __pyx_k_You_cannot_append_rows_when_in_m, sizeof(__pyx_k_You_cannot_append_rows_when_in_m), 0, 0, 1, 0},
  {&__pyx_kp_s__2, __pyx_k__2, sizeof(__pyx_k__2), 0, 0, 1, 0},
  {&__pyx_n_s_add, __pyx_k_add, sizeof(__pyx_k_add), 0, 0, 1, 1},
  {&__pyx_n_s_alltrue, __pyx_k_alltrue, sizeof(__pyx_k_alltrue), 0, 0, 1, 1},
  {&__pyx_n_s_append, __pyx_k_append, sizeof(__pyx_k_append), 0, 0, 1, 1},
  {&__pyx_n_s_arange, __pyx_k_arange, sizeof(__pyx_k_arange), 0, 0, 1, 1},
  {&__pyx_n_s_args, __pyx_k_args, sizeof(__pyx_k_args), 0, 0, 1, 1},
  {&__pyx_n_s_array, __pyx_k_array, sizeof(__pyx_k_array), 0, 0, 1, 1},
  {&__pyx_n_s_asarray, __pyx_k_asarray, sizeof(__pyx_k_asarray), 0, 0, 1, 1},
  {&__pyx_n_s_atom_from_hdf5_type, __pyx_k_atom_from_hdf5_type, sizeof(__pyx_k_atom_from_hdf5_type), 0, 0, 1, 1},
  {&__pyx_kp_s_attempt_to_write_over_a_file_ope, __pyx_k_attempt_to_write_over_a_file_ope, sizeof(__pyx_k_attempt_to_write_over_a_file_ope), 0, 0, 1, 0},
  {&__pyx_n_s_big, __pyx_k_big, sizeof(__pyx_k_big), 0, 0, 1, 1},
  {&__pyx_n_s_byteorder, __pyx_k_byteorder, sizeof(__pyx_k_byteorder), 0, 0, 1, 1},
  {&__pyx_n_s_byteorder_2, __pyx_k_byteorder_2, sizeof(__pyx_k_byteorder_2), 0, 0, 1, 1},
  {&__pyx_n_s_byteswap, __pyx_k_byteswap, sizeof(__pyx_k_byteswap), 0, 0, 1, 1},
  {&__pyx_n_s_c_classid, __pyx_k_c_classid, sizeof(__pyx_k_c_classid), 0, 0, 1, 1},
  {&__pyx_n_s_call_on_recarr, __pyx_k_call_on_recarr, sizeof(__pyx_k_call_on_recarr), 0, 0, 1, 1},
  {&__pyx_n_s_check_open, __pyx_k_check_open, sizeof(__pyx_k_check_open), 0, 0, 1, 1},
  {&__pyx_n_s_chunkcache, __pyx_k_chunkcache, sizeof(__pyx_k_chunkcache), 0, 0, 1, 1},
  {&__pyx_n_s_chunked, __pyx_k_chunked, sizeof(__pyx_k_chunked), 0, 0, 1, 1},
  {&__pyx_n_s_chunkmap, __pyx_k_chunkmap, sizeof(__pyx_k_chunkmap), 0, 0, 1, 1},
  {&__pyx_n_s_chunkshape, __pyx_k_chunkshape, sizeof(__pyx_k_chunkshape), 0, 0, 1, 1},
  {&__pyx_n_s_class, __pyx_k_class, sizeof(__pyx_k_class), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_coldescrs, __pyx_k_coldescrs, sizeof(__pyx_k_coldescrs), 0, 0, 1, 1},
  {&__pyx_n_s_colenums, __pyx_k_colenums, sizeof(__pyx_k_colenums), 0, 0, 1, 1},
  {&__pyx_n_s_colpathnames, __pyx_k_colpathnames, sizeof(__pyx_k_colpathnames), 0, 0, 1, 1},
  {&__pyx_n_s_coltypes, __pyx_k_coltypes, sizeof(__pyx_k_coltypes), 0, 0, 1, 1},
  {&__pyx_n_s_complevel, __pyx_k_complevel, sizeof(__pyx_k_complevel), 0, 0, 1, 1},
  {&__pyx_n_s_complex, __pyx_k_complex, sizeof(__pyx_k_complex), 0, 0, 1, 1},
  {&__pyx_n_s_complib, __pyx_k_complib, sizeof(__pyx_k_complib), 0, 0, 1, 1},
  {&__pyx_n_s_conditions, __pyx_k_conditions, sizeof(__pyx_k_conditions), 0, 0, 1, 1},
  {&__pyx_n_s_convert_types, __pyx_k_convert_types, sizeof(__pyx_k_convert_types), 0, 0, 1, 1},
  {&__pyx_n_s_coords, __pyx_k_coords, sizeof(__pyx_k_coords), 0, 0, 1, 1},
  {&__pyx_n_s_copy, __pyx_k_copy, sizeof(__pyx_k_copy), 0, 0, 1, 1},
  {&__pyx_n_s_create_nested_type, __pyx_k_create_nested_type, sizeof(__pyx_k_create_nested_type), 0, 0, 1, 1},
  {&__pyx_n_s_description, __pyx_k_description, sizeof(__pyx_k_description), 0, 0, 1, 1},
  {&__pyx_n_s_dirtycache, __pyx_k_dirtycache, sizeof(__pyx_k_dirtycache), 0, 0, 1, 1},
  {&__pyx_n_s_dtype, __pyx_k_dtype, sizeof(__pyx_k_dtype), 0, 0, 1, 1},
  {&__pyx_n_s_empty, __pyx_k_empty, sizeof(__pyx_k_empty), 0, 0, 1, 1},
  {&__pyx_n_s_encode, __pyx_k_encode, sizeof(__pyx_k_encode), 0, 0, 1, 1},
  {&__pyx_n_s_enum, __pyx_k_enum, sizeof(__pyx_k_enum), 0, 0, 1, 1},
  {&__pyx_n_s_enumerate, __pyx_k_enumerate, sizeof(__pyx_k_enumerate), 0, 0, 1, 1},
  {&__pyx_n_s_eq, __pyx_k_eq, sizeof(__pyx_k_eq), 0, 0, 1, 1},
  {&__pyx_n_s_exceptions, __pyx_k_exceptions, sizeof(__pyx_k_exceptions), 0, 0, 1, 1},
  {&__pyx_n_s_extend, __pyx_k_extend, sizeof(__pyx_k_extend), 0, 0, 1, 1},
  {&__pyx_n_s_fetch_all_fields, __pyx_k_fetch_all_fields, sizeof(__pyx_k_fetch_all_fields), 0, 0, 1, 1},
  {&__pyx_n_s_field, __pyx_k_field, sizeof(__pyx_k_field), 0, 0, 1, 1},
  {&__pyx_n_s_filters, __pyx_k_filters, sizeof(__pyx_k_filters), 0, 0, 1, 1},
  {&__pyx_n_s_flat, __pyx_k_flat, sizeof(__pyx_k_flat), 0, 0, 1, 1},
  {&__pyx_n_s_fletcher32, __pyx_k_fletcher32, sizeof(__pyx_k_fletcher32), 0, 0, 1, 1},
  {&__pyx_n_s_float, __pyx_k_float, sizeof(__pyx_k_float), 0, 0, 1, 1},
  {&__pyx_n_s_flush_buffered_rows, __pyx_k_flush_buffered_rows, sizeof(__pyx_k_flush_buffered_rows), 0, 0, 1, 1},
  {&__pyx_n_s_flush_mod_rows, __pyx_k_flush_mod_rows, sizeof(__pyx_k_flush_mod_rows), 0, 0, 1, 1},
  {&__pyx_n_s_from_atom, __pyx_k_from_atom, sizeof(__pyx_k_from_atom), 0, 0, 1, 1},
  {&__pyx_n_s_get_container, __pyx_k_get_container, sizeof(__pyx_k_get_container), 0, 0, 1, 1},
  {&__pyx_n_s_get_nested_field, __pyx_k_get_nested_field, sizeof(__pyx_k_get_nested_field), 0, 0, 1, 1},
  {&__pyx_n_s_get_node, __pyx_k_get_node, sizeof(__pyx_k_get_node), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_h5tb, __pyx_k_h5tb, sizeof(__pyx_k_h5tb), 0, 0, 1, 1},
  {&__pyx_n_s_hdf5_class_to_string, __pyx_k_hdf5_class_to_string, sizeof(__pyx_k_hdf5_class_to_string), 0, 0, 1, 1},
  {&__pyx_n_s_hdf5_to_np_ext_type, __pyx_k_hdf5_to_np_ext_type, sizeof(__pyx_k_hdf5_to_np_ext_type), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_int, __pyx_k_int, sizeof(__pyx_k_int), 0, 0, 1, 1},
  {&__pyx_n_s_int64, __pyx_k_int64, sizeof(__pyx_k_int64), 0, 0, 1, 1},
  {&__pyx_kp_s_invalid_type_s_for_column_s, __pyx_k_invalid_type_s_for_column_s, sizeof(__pyx_k_invalid_type_s_for_column_s), 0, 0, 1, 0},
  {&__pyx_n_s_irrelevant, __pyx_k_irrelevant, sizeof(__pyx_k_irrelevant), 0, 0, 1, 1},
  {&__pyx_n_s_item, __pyx_k_item, sizeof(__pyx_k_item), 0, 0, 1, 1},
  {&__pyx_n_s_itemsize, __pyx_k_itemsize, sizeof(__pyx_k_itemsize), 0, 0, 1, 1},
  {&__pyx_n_s_kind, __pyx_k_kind, sizeof(__pyx_k_kind), 0, 0, 1, 1},
  {&__pyx_n_s_little, __pyx_k_little, sizeof(__pyx_k_little), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_mark_columns_as_dirty, __pyx_k_mark_columns_as_dirty, sizeof(__pyx_k_mark_columns_as_dirty), 0, 0, 1, 1},
  {&__pyx_n_s_mixed, __pyx_k_mixed, sizeof(__pyx_k_mixed), 0, 0, 1, 1},
  {&__pyx_n_s_mode, __pyx_k_mode, sizeof(__pyx_k_mode), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_names, __pyx_k_names, sizeof(__pyx_k_names), 0, 0, 1, 1},
  {&__pyx_kp_s_no_default___reduce___due_to_non, __pyx_k_no_default___reduce___due_to_non, sizeof(__pyx_k_no_default___reduce___due_to_non), 0, 0, 1, 0},
  {&__pyx_n_s_npext_prefixes_to_ptkinds, __pyx_k_npext_prefixes_to_ptkinds, sizeof(__pyx_k_npext_prefixes_to_ptkinds), 0, 0, 1, 1},
  {&__pyx_n_s_nrecords, __pyx_k_nrecords, sizeof(__pyx_k_nrecords), 0, 0, 1, 1},
  {&__pyx_n_s_nrows, __pyx_k_nrows, sizeof(__pyx_k_nrows), 0, 0, 1, 1},
  {&__pyx_n_s_nrowsinbuf, __pyx_k_nrowsinbuf, sizeof(__pyx_k_nrowsinbuf), 0, 0, 1, 1},
  {&__pyx_n_s_numpy, __pyx_k_numpy, sizeof(__pyx_k_numpy), 0, 0, 1, 1},
  {&__pyx_kp_s_numpy_core_multiarray_failed_to, __pyx_k_numpy_core_multiarray_failed_to, sizeof(__pyx_k_numpy_core_multiarray_failed_to), 0, 0, 1, 0},
  {&__pyx_kp_s_numpy_core_umath_failed_to_impor, __pyx_k_numpy_core_umath_failed_to_impor, sizeof(__pyx_k_numpy_core_umath_failed_to_impor), 0, 0, 1, 0},
  {&__pyx_n_s_obversion, __pyx_k_obversion, sizeof(__pyx_k_obversion), 0, 0, 1, 1},
  {&__pyx_n_s_offset, __pyx_k_offset, sizeof(__pyx_k_offset), 0, 0, 1, 1},
  {&__pyx_n_s_params, __pyx_k_params, sizeof(__pyx_k_params), 0, 0, 1, 1},
  {&__pyx_n_s_platform_byteorder, __pyx_k_platform_byteorder, sizeof(__pyx_k_platform_byteorder), 0, 0, 1, 1},
  {&__pyx_n_s_pos, __pyx_k_pos, sizeof(__pyx_k_pos), 0, 0, 1, 1},
  {&__pyx_n_s_pt_special_kinds, __pyx_k_pt_special_kinds, sizeof(__pyx_k_pt_special_kinds), 0, 0, 1, 1},
  {&__pyx_n_s_pttype_to_hdf5, __pyx_k_pttype_to_hdf5, sizeof(__pyx_k_pttype_to_hdf5), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_r, __pyx_k_r, sizeof(__pyx_k_r), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_read_elements, __pyx_k_read_elements, sizeof(__pyx_k_read_elements), 0, 0, 1, 1},
  {&__pyx_n_s_read_records, __pyx_k_read_records, sizeof(__pyx_k_read_records), 0, 0, 1, 1},
  {&__pyx_n_s_recarr, __pyx_k_recarr, sizeof(__pyx_k_recarr), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_remove_rows, __pyx_k_remove_rows, sizeof(__pyx_k_remove_rows), 0, 0, 1, 1},
  {&__pyx_n_s_result, __pyx_k_result, sizeof(__pyx_k_result), 0, 0, 1, 1},
  {&__pyx_n_s_rowsize, __pyx_k_rowsize, sizeof(__pyx_k_rowsize), 0, 0, 1, 1},
  {&__pyx_kp_s_s_row_s_pointing_to_row_d, __pyx_k_s_row_s_pointing_to_row_d, sizeof(__pyx_k_s_row_s_pointing_to_row_d), 0, 0, 1, 0},
  {&__pyx_n_s_save_buffered_rows, __pyx_k_save_buffered_rows, sizeof(__pyx_k_save_buffered_rows), 0, 0, 1, 1},
  {&__pyx_kp_s_self_dims_self_wbuf_cannot_be_co, __pyx_k_self_dims_self_wbuf_cannot_be_co, sizeof(__pyx_k_self_dims_self_wbuf_cannot_be_co), 0, 0, 1, 0},
  {&__pyx_n_s_sense, __pyx_k_sense, sizeof(__pyx_k_sense), 0, 0, 1, 1},
  {&__pyx_n_s_seqcache, __pyx_k_seqcache, sizeof(__pyx_k_seqcache), 0, 0, 1, 1},
  {&__pyx_n_s_seqcache_key, __pyx_k_seqcache_key, sizeof(__pyx_k_seqcache_key), 0, 0, 1, 1},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_shape, __pyx_k_shape, sizeof(__pyx_k_shape), 0, 0, 1, 1},
  {&__pyx_n_s_shuffle_bitshuffle, __pyx_k_shuffle_bitshuffle, sizeof(__pyx_k_shuffle_bitshuffle), 0, 0, 1, 1},
  {&__pyx_n_s_size, __pyx_k_size, sizeof(__pyx_k_size), 0, 0, 1, 1},
  {&__pyx_n_s_sometrue, __pyx_k_sometrue, sizeof(__pyx_k_sometrue), 0, 0, 1, 1},
  {&__pyx_n_s_start, __pyx_k_start, sizeof(__pyx_k_start), 0, 0, 1, 1},
  {&__pyx_n_s_step, __pyx_k_step, sizeof(__pyx_k_step), 0, 0, 1, 1},
  {&__pyx_kp_s_step_size_may_not_be_0, __pyx_k_step_size_may_not_be_0, sizeof(__pyx_k_step_size_may_not_be_0), 0, 0, 1, 0},
  {&__pyx_n_s_stop, __pyx_k_stop, sizeof(__pyx_k_stop), 0, 0, 1, 1},
  {&__pyx_n_s_sys, __pyx_k_sys, sizeof(__pyx_k_sys), 0, 0, 1, 1},
  {&__pyx_n_s_table, __pyx_k_table, sizeof(__pyx_k_table), 0, 0, 1, 1},
  {&__pyx_kp_s_table_s_column_s_s, __pyx_k_table_s_column_s_s, sizeof(__pyx_k_table_s_column_s_s), 0, 0, 1, 0},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_time, __pyx_k_time, sizeof(__pyx_k_time), 0, 0, 1, 1},
  {&__pyx_n_s_time32, __pyx_k_time32, sizeof(__pyx_k_time32), 0, 0, 1, 1},
  {&__pyx_n_s_time64, __pyx_k_time64, sizeof(__pyx_k_time64), 0, 0, 1, 1},
  {&__pyx_n_s_time64colnames, __pyx_k_time64colnames, sizeof(__pyx_k_time64colnames), 0, 0, 1, 1},
  {&__pyx_n_s_title, __pyx_k_title, sizeof(__pyx_k_title), 0, 0, 1, 1},
  {&__pyx_n_s_uint, __pyx_k_uint, sizeof(__pyx_k_uint), 0, 0, 1, 1},
  {&__pyx_n_s_uint64, __pyx_k_uint64, sizeof(__pyx_k_uint64), 0, 0, 1, 1},
  {&__pyx_n_s_update_elements, __pyx_k_update_elements, sizeof(__pyx_k_update_elements), 0, 0, 1, 1},
  {&__pyx_n_s_use_index, __pyx_k_use_index, sizeof(__pyx_k_use_index), 0, 0, 1, 1},
  {&__pyx_kp_s_utf_8, __pyx_k_utf_8, sizeof(__pyx_k_utf_8), 0, 0, 1, 0},
  {&__pyx_n_s_utils, __pyx_k_utils, sizeof(__pyx_k_utils), 0, 0, 1, 1},
  {&__pyx_n_s_utilsextension, __pyx_k_utilsextension, sizeof(__pyx_k_utilsextension), 0, 0, 1, 1},
  {&__pyx_n_s_v_dtype, __pyx_k_v_dtype, sizeof(__pyx_k_v_dtype), 0, 0, 1, 1},
  {&__pyx_n_s_v_file, __pyx_k_v_file, sizeof(__pyx_k_v_file), 0, 0, 1, 1},
  {&__pyx_n_s_v_itemsize, __pyx_k_v_itemsize, sizeof(__pyx_k_v_itemsize), 0, 0, 1, 1},
  {&__pyx_n_s_v_leaves, __pyx_k_v_leaves, sizeof(__pyx_k_v_leaves), 0, 0, 1, 1},
  {&__pyx_n_s_v_names, __pyx_k_v_names, sizeof(__pyx_k_v_names), 0, 0, 1, 1},
  {&__pyx_n_s_v_offset, __pyx_k_v_offset, sizeof(__pyx_k_v_offset), 0, 0, 1, 1},
  {&__pyx_n_s_v_parent, __pyx_k_v_parent, sizeof(__pyx_k_v_parent), 0, 0, 1, 1},
  {&__pyx_n_s_v_pathname, __pyx_k_v_pathname, sizeof(__pyx_k_v_pathname), 0, 0, 1, 1},
  {&__pyx_n_s_v_pos, __pyx_k_v_pos, sizeof(__pyx_k_v_pos), 0, 0, 1, 1},
  {&__pyx_n_s_v_recarray, __pyx_k_v_recarray, sizeof(__pyx_k_v_recarray), 0, 0, 1, 1},
  {&__pyx_n_s_v_wdflts, __pyx_k_v_wdflts, sizeof(__pyx_k_v_wdflts), 0, 0, 1, 1},
  {&__pyx_n_s_want_track_times, __pyx_k_want_track_times, sizeof(__pyx_k_want_track_times), 0, 0, 1, 1},
  {&__pyx_n_s_where_condition, __pyx_k_where_condition, sizeof(__pyx_k_where_condition), 0, 0, 1, 1},
  {&__pyx_n_s_zeros, __pyx_k_zeros, sizeof(__pyx_k_zeros), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_KeyError = __Pyx_GetBuiltinName(__pyx_n_s_KeyError); if (!__pyx_builtin_KeyError) __PYX_ERR(0, 125, __pyx_L1_error)
  __pyx_builtin_enumerate = __Pyx_GetBuiltinName(__pyx_n_s_enumerate); if (!__pyx_builtin_enumerate) __PYX_ERR(0, 242, __pyx_L1_error)
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 283, __pyx_L1_error)
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(0, 317, __pyx_L1_error)
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(0, 388, __pyx_L1_error)
  __pyx_builtin_StopIteration = __Pyx_GetBuiltinName(__pyx_n_s_StopIteration); if (!__pyx_builtin_StopIteration) __PYX_ERR(0, 906, __pyx_L1_error)
  __pyx_builtin_IOError = __Pyx_GetBuiltinName(__pyx_n_s_IOError); if (!__pyx_builtin_IOError) __PYX_ERR(0, 1316, __pyx_L1_error)
  __pyx_builtin_NotImplementedError = __Pyx_GetBuiltinName(__pyx_n_s_NotImplementedError); if (!__pyx_builtin_NotImplementedError) __PYX_ERR(0, 1323, __pyx_L1_error)
  __pyx_builtin_ImportError = __Pyx_GetBuiltinName(__pyx_n_s_ImportError); if (!__pyx_builtin_ImportError) __PYX_ERR(2, 945, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "tables/tableextension.pyx":681
 *         nrecords += self._remove_rows(i, i+1, 1)
 *     else:
 *       raise ValueError("step size may not be 0.")             # <<<<<<<<<<<<<<
 * 
 *     # Return the number of records removed
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_s_step_size_may_not_be_0); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(0, 681, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.dims,self.wbuf cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.dims,self.wbuf cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_kp_s_self_dims_self_wbuf_cannot_be_co); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "(tree fragment)":4
 *     raise TypeError("self.dims,self.wbuf cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.dims,self.wbuf cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_kp_s_self_dims_self_wbuf_cannot_be_co); if (unlikely(!__pyx_tuple__5)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "tables/tableextension.pyx":804
 *     wdflts = table._v_wdflts
 *     if wdflts is None:
 *       self.wrec = numpy.zeros(1, dtype=self.dtype)  # Defaults are zero             # <<<<<<<<<<<<<<
 *     else:
 *       self.wrec = table._v_wdflts.copy()
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_int_1); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(0, 804, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "tables/tableextension.pyx":1189
 *     # (this is useful for accessing the last row after an iterator loop)
 *     if self._row >= 0:
 *       self.wrec[:] = self.iobuf[self._row]             # <<<<<<<<<<<<<<
 *     if self._write_to_seqcache:
 *       seqcache = self.table._seqcache
 */
  __pyx_slice__7 = PySlice_New(Py_None, Py_None, Py_None); if (unlikely(!__pyx_slice__7)) __PYX_ERR(0, 1189, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__7);
  __Pyx_GIVEREF(__pyx_slice__7);

  /* "tables/tableextension.pyx":1316
 * 
 *     if self.ro_filemode:
 *       raise IOError("Attempt to write over a file opened in read-only mode")             # <<<<<<<<<<<<<<
 * 
 *     if not self.chunked:
 */
  __pyx_tuple__8 = PyTuple_Pack(1, __pyx_kp_s_Attempt_to_write_over_a_file_ope); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(0, 1316, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);

  /* "tables/tableextension.pyx":1319
 * 
 *     if not self.chunked:
 *       raise HDF5ExtError("You cannot append rows to a non-chunked table.",             # <<<<<<<<<<<<<<
 *                          h5tb=False)
 * 
 */
  __pyx_tuple__9 = PyTuple_Pack(1, __pyx_kp_s_You_cannot_append_rows_to_a_non); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(0, 1319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);

  /* "tables/tableextension.pyx":1323
 * 
 *     if self._riterator:
 *       raise NotImplementedError("You cannot append rows when in middle of a table iterator. If what you want is to update records, use Row.update() instead.")             # <<<<<<<<<<<<<<
 * 
 *     # Commit the private record into the write buffer
 */
  __pyx_tuple__10 = PyTuple_Pack(1, __pyx_kp_s_You_cannot_append_rows_when_in_m); if (unlikely(!__pyx_tuple__10)) __PYX_ERR(0, 1323, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);

  /* "tables/tableextension.pyx":1403
 * 
 *     if not self._riterator:
 *       raise NotImplementedError("You are only allowed to update rows through the Row.update() method if you are in the middle of a table iterator.")             # <<<<<<<<<<<<<<
 * 
 *     if self.mod_elements is None:
 */
  __pyx_tuple__11 = PyTuple_Pack(1, __pyx_kp_s_You_are_only_allowed_to_update_r); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(0, 1403, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);

  /* "tables/tableextension.pyx":1563
 * 
 *     if self.ro_filemode:
 *       raise IOError("attempt to write over a file opened in read-only mode")             # <<<<<<<<<<<<<<
 * 
 *     if self._riterator:
 */
  __pyx_tuple__12 = PyTuple_Pack(1, __pyx_kp_s_attempt_to_write_over_a_file_ope); if (unlikely(!__pyx_tuple__12)) __PYX_ERR(0, 1563, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__13 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__14 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__14)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__14);
  __Pyx_GIVEREF(__pyx_tuple__14);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":945
 *         __pyx_import_array()
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */
  __pyx_tuple__15 = PyTuple_Pack(1, __pyx_kp_s_numpy_core_multiarray_failed_to); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(2, 945, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":951
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */
  __pyx_tuple__16 = PyTuple_Pack(1, __pyx_kp_s_numpy_core_umath_failed_to_impor); if (unlikely(!__pyx_tuple__16)) __PYX_ERR(2, 951, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__16);
  __Pyx_GIVEREF(__pyx_tuple__16);
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_t_1 = PyImport_ImportModule("tables.hdf5extension"); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_6tables_13hdf5extension_Leaf = __Pyx_ImportType(__pyx_t_1, "tables.hdf5extension", "Leaf", sizeof(struct __pyx_obj_6tables_13hdf5extension_Leaf), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6tables_13hdf5extension_Leaf) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_vtabptr_6tables_13hdf5extension_Leaf = (struct __pyx_vtabstruct_6tables_13hdf5extension_Leaf*)__Pyx_GetVtable(__pyx_ptype_6tables_13hdf5extension_Leaf->tp_dict); if (unlikely(!__pyx_vtabptr_6tables_13hdf5extension_Leaf)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_vtabptr_6tables_14tableextension_Table = &__pyx_vtable_6tables_14tableextension_Table;
  __pyx_vtable_6tables_14tableextension_Table.__pyx_base = *__pyx_vtabptr_6tables_13hdf5extension_Leaf;
  __pyx_vtable_6tables_14tableextension_Table.get_nested_type = (PyObject *(*)(struct __pyx_obj_6tables_14tableextension_Table *, hid_t, hid_t, PyObject *, PyObject *))__pyx_f_6tables_14tableextension_5Table_get_nested_type;
  __pyx_vtable_6tables_14tableextension_Table._convert_time64_ = (PyObject *(*)(struct __pyx_obj_6tables_14tableextension_Table *, PyArrayObject *, hsize_t, int))__pyx_f_6tables_14tableextension_5Table__convert_time64_;
  __pyx_vtable_6tables_14tableextension_Table._convert_types = (PyObject *(*)(struct __pyx_obj_6tables_14tableextension_Table *, PyArrayObject *, hsize_t, int, int __pyx_skip_dispatch))__pyx_f_6tables_14tableextension_5Table__convert_types;
  __pyx_vtable_6tables_14tableextension_Table._read_chunk = (hsize_t (*)(struct __pyx_obj_6tables_14tableextension_Table *, hsize_t, PyArrayObject *, long))__pyx_f_6tables_14tableextension_5Table__read_chunk;
  __pyx_type_6tables_14tableextension_Table.tp_base = __pyx_ptype_6tables_13hdf5extension_Leaf;
  if (PyType_Ready(&__pyx_type_6tables_14tableextension_Table) < 0) __PYX_ERR(0, 145, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6tables_14tableextension_Table.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6tables_14tableextension_Table.tp_dictoffset && __pyx_type_6tables_14tableextension_Table.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6tables_14tableextension_Table.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6tables_14tableextension_Table.tp_dict, __pyx_vtabptr_6tables_14tableextension_Table) < 0) __PYX_ERR(0, 145, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_Table, (PyObject *)&__pyx_type_6tables_14tableextension_Table) < 0) __PYX_ERR(0, 145, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6tables_14tableextension_Table) < 0) __PYX_ERR(0, 145, __pyx_L1_error)
  __pyx_ptype_6tables_14tableextension_Table = &__pyx_type_6tables_14tableextension_Table;
  __pyx_vtabptr_6tables_14tableextension_Row = &__pyx_vtable_6tables_14tableextension_Row;
  __pyx_vtable_6tables_14tableextension_Row._new_buffer = (PyObject *(*)(struct __pyx_obj_6tables_14tableextension_Row *, PyObject *))__pyx_f_6tables_14tableextension_3Row__new_buffer;
  __pyx_vtable_6tables_14tableextension_Row._init_loop = (PyObject *(*)(struct __pyx_obj_6tables_14tableextension_Row *, hsize_t, PY_LONG_LONG, PY_LONG_LONG, PyObject *, PyObject *))__pyx_f_6tables_14tableextension_3Row__init_loop;
  __pyx_vtable_6tables_14tableextension_Row.__pyx___next__indexed = (PyObject *(*)(struct __pyx_obj_6tables_14tableextension_Row *))__pyx_f_6tables_14tableextension_3Row___next__indexed;
  __pyx_vtable_6tables_14tableextension_Row.__pyx___next__coords = (PyObject *(*)(struct __pyx_obj_6tables_14tableextension_Row *))__pyx_f_6tables_14tableextension_3Row___next__coords;
  __pyx_vtable_6tables_14tableextension_Row.__pyx___next__inkernel = (PyObject *(*)(struct __pyx_obj_6tables_14tableextension_Row *))__pyx_f_6tables_14tableextension_3Row___next__inkernel;
  __pyx_vtable_6tables_14tableextension_Row.__pyx___next__general = (PyObject *(*)(struct __pyx_obj_6tables_14tableextension_Row *))__pyx_f_6tables_14tableextension_3Row___next__general;
  __pyx_vtable_6tables_14tableextension_Row._finish_riterator = (PyObject *(*)(struct __pyx_obj_6tables_14tableextension_Row *))__pyx_f_6tables_14tableextension_3Row__finish_riterator;
  if (PyType_Ready(&__pyx_type_6tables_14tableextension_Row) < 0) __PYX_ERR(0, 686, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6tables_14tableextension_Row.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6tables_14tableextension_Row.tp_dictoffset && __pyx_type_6tables_14tableextension_Row.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6tables_14tableextension_Row.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  #if CYTHON_COMPILING_IN_CPYTHON
  {
    PyObject *wrapper = PyObject_GetAttrString((PyObject *)&__pyx_type_6tables_14tableextension_Row, "__iter__"); if (unlikely(!wrapper)) __PYX_ERR(0, 686, __pyx_L1_error)
    if (Py_TYPE(wrapper) == &PyWrapperDescr_Type) {
      __pyx_wrapperbase_6tables_14tableextension_3Row_4__iter__ = *((PyWrapperDescrObject *)wrapper)->d_base;
      __pyx_wrapperbase_6tables_14tableextension_3Row_4__iter__.doc = __pyx_doc_6tables_14tableextension_3Row_4__iter__;
      ((PyWrapperDescrObject *)wrapper)->d_base = &__pyx_wrapperbase_6tables_14tableextension_3Row_4__iter__;
    }
  }
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON
  {
    PyObject *wrapper = PyObject_GetAttrString((PyObject *)&__pyx_type_6tables_14tableextension_Row, "__next__"); if (unlikely(!wrapper)) __PYX_ERR(0, 686, __pyx_L1_error)
    if (Py_TYPE(wrapper) == &PyWrapperDescr_Type) {
      __pyx_wrapperbase_6tables_14tableextension_3Row_6__next__ = *((PyWrapperDescrObject *)wrapper)->d_base;
      __pyx_wrapperbase_6tables_14tableextension_3Row_6__next__.doc = __pyx_doc_6tables_14tableextension_3Row_6__next__;
      ((PyWrapperDescrObject *)wrapper)->d_base = &__pyx_wrapperbase_6tables_14tableextension_3Row_6__next__;
    }
  }
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON
  {
    PyObject *wrapper = PyObject_GetAttrString((PyObject *)&__pyx_type_6tables_14tableextension_Row, "__contains__"); if (unlikely(!wrapper)) __PYX_ERR(0, 686, __pyx_L1_error)
    if (Py_TYPE(wrapper) == &PyWrapperDescr_Type) {
      __pyx_wrapperbase_6tables_14tableextension_3Row_20__contains__ = *((PyWrapperDescrObject *)wrapper)->d_base;
      __pyx_wrapperbase_6tables_14tableextension_3Row_20__contains__.doc = __pyx_doc_6tables_14tableextension_3Row_20__contains__;
      ((PyWrapperDescrObject *)wrapper)->d_base = &__pyx_wrapperbase_6tables_14tableextension_3Row_20__contains__;
    }
  }
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON
  {
    PyObject *wrapper = PyObject_GetAttrString((PyObject *)&__pyx_type_6tables_14tableextension_Row, "__getitem__"); if (unlikely(!wrapper)) __PYX_ERR(0, 686, __pyx_L1_error)
    if (Py_TYPE(wrapper) == &PyWrapperDescr_Type) {
      __pyx_wrapperbase_6tables_14tableextension_3Row_22__getitem__ = *((PyWrapperDescrObject *)wrapper)->d_base;
      __pyx_wrapperbase_6tables_14tableextension_3Row_22__getitem__.doc = __pyx_doc_6tables_14tableextension_3Row_22__getitem__;
      ((PyWrapperDescrObject *)wrapper)->d_base = &__pyx_wrapperbase_6tables_14tableextension_3Row_22__getitem__;
    }
  }
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON
  {
    PyObject *wrapper = PyObject_GetAttrString((PyObject *)&__pyx_type_6tables_14tableextension_Row, "__setitem__"); if (unlikely(!wrapper)) __PYX_ERR(0, 686, __pyx_L1_error)
    if (Py_TYPE(wrapper) == &PyWrapperDescr_Type) {
      __pyx_wrapperbase_6tables_14tableextension_3Row_24__setitem__ = *((PyWrapperDescrObject *)wrapper)->d_base;
      __pyx_wrapperbase_6tables_14tableextension_3Row_24__setitem__.doc = __pyx_doc_6tables_14tableextension_3Row_24__setitem__;
      ((PyWrapperDescrObject *)wrapper)->d_base = &__pyx_wrapperbase_6tables_14tableextension_3Row_24__setitem__;
    }
  }
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON
  {
    PyObject *wrapper = PyObject_GetAttrString((PyObject *)&__pyx_type_6tables_14tableextension_Row, "__str__"); if (unlikely(!wrapper)) __PYX_ERR(0, 686, __pyx_L1_error)
    if (Py_TYPE(wrapper) == &PyWrapperDescr_Type) {
      __pyx_wrapperbase_6tables_14tableextension_3Row_28__str__ = *((PyWrapperDescrObject *)wrapper)->d_base;
      __pyx_wrapperbase_6tables_14tableextension_3Row_28__str__.doc = __pyx_doc_6tables_14tableextension_3Row_28__str__;
      ((PyWrapperDescrObject *)wrapper)->d_base = &__pyx_wrapperbase_6tables_14tableextension_3Row_28__str__;
    }
  }
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON
  {
    PyObject *wrapper = PyObject_GetAttrString((PyObject *)&__pyx_type_6tables_14tableextension_Row, "__repr__"); if (unlikely(!wrapper)) __PYX_ERR(0, 686, __pyx_L1_error)
    if (Py_TYPE(wrapper) == &PyWrapperDescr_Type) {
      __pyx_wrapperbase_6tables_14tableextension_3Row_30__repr__ = *((PyWrapperDescrObject *)wrapper)->d_base;
      __pyx_wrapperbase_6tables_14tableextension_3Row_30__repr__.doc = __pyx_doc_6tables_14tableextension_3Row_30__repr__;
      ((PyWrapperDescrObject *)wrapper)->d_base = &__pyx_wrapperbase_6tables_14tableextension_3Row_30__repr__;
    }
  }
  #endif
  if (__Pyx_SetVtable(__pyx_type_6tables_14tableextension_Row.tp_dict, __pyx_vtabptr_6tables_14tableextension_Row) < 0) __PYX_ERR(0, 686, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_Row, (PyObject *)&__pyx_type_6tables_14tableextension_Row) < 0) __PYX_ERR(0, 686, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6tables_14tableextension_Row) < 0) __PYX_ERR(0, 686, __pyx_L1_error)
  __pyx_ptype_6tables_14tableextension_Row = &__pyx_type_6tables_14tableextension_Row;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type", 
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_4type_type) __PYX_ERR(3, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4bool_bool = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "bool", sizeof(PyBoolObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_4bool_bool) __PYX_ERR(4, 8, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_7complex_complex = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "complex", sizeof(PyComplexObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_7complex_complex) __PYX_ERR(5, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("numpy"); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_5numpy_dtype = __Pyx_ImportType(__pyx_t_1, "numpy", "dtype", sizeof(PyArray_Descr), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_dtype) __PYX_ERR(2, 200, __pyx_L1_error)
  __pyx_ptype_5numpy_flatiter = __Pyx_ImportType(__pyx_t_1, "numpy", "flatiter", sizeof(PyArrayIterObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_flatiter) __PYX_ERR(2, 223, __pyx_L1_error)
  __pyx_ptype_5numpy_broadcast = __Pyx_ImportType(__pyx_t_1, "numpy", "broadcast", sizeof(PyArrayMultiIterObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_broadcast) __PYX_ERR(2, 227, __pyx_L1_error)
  __pyx_ptype_5numpy_ndarray = __Pyx_ImportType(__pyx_t_1, "numpy", "ndarray", sizeof(PyArrayObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_ndarray) __PYX_ERR(2, 239, __pyx_L1_error)
  __pyx_ptype_5numpy_generic = __Pyx_ImportType(__pyx_t_1, "numpy", "generic", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_generic) __PYX_ERR(2, 771, __pyx_L1_error)
  __pyx_ptype_5numpy_number = __Pyx_ImportType(__pyx_t_1, "numpy", "number", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_number) __PYX_ERR(2, 773, __pyx_L1_error)
  __pyx_ptype_5numpy_integer = __Pyx_ImportType(__pyx_t_1, "numpy", "integer", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_integer) __PYX_ERR(2, 775, __pyx_L1_error)
  __pyx_ptype_5numpy_signedinteger = __Pyx_ImportType(__pyx_t_1, "numpy", "signedinteger", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_signedinteger) __PYX_ERR(2, 777, __pyx_L1_error)
  __pyx_ptype_5numpy_unsignedinteger = __Pyx_ImportType(__pyx_t_1, "numpy", "unsignedinteger", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_unsignedinteger) __PYX_ERR(2, 779, __pyx_L1_error)
  __pyx_ptype_5numpy_inexact = __Pyx_ImportType(__pyx_t_1, "numpy", "inexact", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_inexact) __PYX_ERR(2, 781, __pyx_L1_error)
  __pyx_ptype_5numpy_floating = __Pyx_ImportType(__pyx_t_1, "numpy", "floating", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_floating) __PYX_ERR(2, 783, __pyx_L1_error)
  __pyx_ptype_5numpy_complexfloating = __Pyx_ImportType(__pyx_t_1, "numpy", "complexfloating", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_complexfloating) __PYX_ERR(2, 785, __pyx_L1_error)
  __pyx_ptype_5numpy_flexible = __Pyx_ImportType(__pyx_t_1, "numpy", "flexible", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_flexible) __PYX_ERR(2, 787, __pyx_L1_error)
  __pyx_ptype_5numpy_character = __Pyx_ImportType(__pyx_t_1, "numpy", "character", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_character) __PYX_ERR(2, 789, __pyx_L1_error)
  __pyx_ptype_5numpy_ufunc = __Pyx_ImportType(__pyx_t_1, "numpy", "ufunc", sizeof(PyUFuncObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_ufunc) __PYX_ERR(2, 827, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("tables.hdf5extension"); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_6tables_13hdf5extension_Node = __Pyx_ImportType(__pyx_t_1, "tables.hdf5extension", "Node", sizeof(struct __pyx_obj_6tables_13hdf5extension_Node), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6tables_13hdf5extension_Node) __PYX_ERR(6, 16, __pyx_L1_error)
  __pyx_ptype_6tables_13hdf5extension_Array = __Pyx_ImportType(__pyx_t_1, "tables.hdf5extension", "Array", sizeof(struct __pyx_obj_6tables_13hdf5extension_Array), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6tables_13hdf5extension_Array) __PYX_ERR(6, 29, __pyx_L1_error)
  __pyx_vtabptr_6tables_13hdf5extension_Array = (struct __pyx_vtabstruct_6tables_13hdf5extension_Array*)__Pyx_GetVtable(__pyx_ptype_6tables_13hdf5extension_Array->tp_dict); if (unlikely(!__pyx_vtabptr_6tables_13hdf5extension_Array)) __PYX_ERR(6, 29, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("tables.lrucacheextension"); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_6tables_17lrucacheextension_NodeCache = __Pyx_ImportType(__pyx_t_1, "tables.lrucacheextension", "NodeCache", sizeof(struct __pyx_obj_6tables_17lrucacheextension_NodeCache), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6tables_17lrucacheextension_NodeCache) __PYX_ERR(7, 15, __pyx_L1_error)
  __pyx_vtabptr_6tables_17lrucacheextension_NodeCache = (struct __pyx_vtabstruct_6tables_17lrucacheextension_NodeCache*)__Pyx_GetVtable(__pyx_ptype_6tables_17lrucacheextension_NodeCache->tp_dict); if (unlikely(!__pyx_vtabptr_6tables_17lrucacheextension_NodeCache)) __PYX_ERR(7, 15, __pyx_L1_error)
  __pyx_ptype_6tables_17lrucacheextension_BaseCache = __Pyx_ImportType(__pyx_t_1, "tables.lrucacheextension", "BaseCache", sizeof(struct __pyx_obj_6tables_17lrucacheextension_BaseCache), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6tables_17lrucacheextension_BaseCache) __PYX_ERR(7, 25, __pyx_L1_error)
  __pyx_vtabptr_6tables_17lrucacheextension_BaseCache = (struct __pyx_vtabstruct_6tables_17lrucacheextension_BaseCache*)__Pyx_GetVtable(__pyx_ptype_6tables_17lrucacheextension_BaseCache->tp_dict); if (unlikely(!__pyx_vtabptr_6tables_17lrucacheextension_BaseCache)) __PYX_ERR(7, 25, __pyx_L1_error)
  __pyx_ptype_6tables_17lrucacheextension_ObjectNode = __Pyx_ImportType(__pyx_t_1, "tables.lrucacheextension", "ObjectNode", sizeof(struct __pyx_obj_6tables_17lrucacheextension_ObjectNode), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6tables_17lrucacheextension_ObjectNode) __PYX_ERR(7, 42, __pyx_L1_error)
  __pyx_ptype_6tables_17lrucacheextension_ObjectCache = __Pyx_ImportType(__pyx_t_1, "tables.lrucacheextension", "ObjectCache", sizeof(struct __pyx_obj_6tables_17lrucacheextension_ObjectCache), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6tables_17lrucacheextension_ObjectCache) __PYX_ERR(7, 48, __pyx_L1_error)
  __pyx_vtabptr_6tables_17lrucacheextension_ObjectCache = (struct __pyx_vtabstruct_6tables_17lrucacheextension_ObjectCache*)__Pyx_GetVtable(__pyx_ptype_6tables_17lrucacheextension_ObjectCache->tp_dict); if (unlikely(!__pyx_vtabptr_6tables_17lrucacheextension_ObjectCache)) __PYX_ERR(7, 48, __pyx_L1_error)
  __pyx_ptype_6tables_17lrucacheextension_NumCache = __Pyx_ImportType(__pyx_t_1, "tables.lrucacheextension", "NumCache", sizeof(struct __pyx_obj_6tables_17lrucacheextension_NumCache), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6tables_17lrucacheextension_NumCache) __PYX_ERR(7, 63, __pyx_L1_error)
  __pyx_vtabptr_6tables_17lrucacheextension_NumCache = (struct __pyx_vtabstruct_6tables_17lrucacheextension_NumCache*)__Pyx_GetVtable(__pyx_ptype_6tables_17lrucacheextension_NumCache->tp_dict); if (unlikely(!__pyx_vtabptr_6tables_17lrucacheextension_NumCache)) __PYX_ERR(7, 63, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __pyx_t_1 = PyImport_ImportModule("tables.utilsextension"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportFunction(__pyx_t_1, "get_native_type", (void (**)(void))&__pyx_f_6tables_14utilsextension_get_native_type, "hid_t (hid_t)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "cstr_to_pystr", (void (**)(void))&__pyx_f_6tables_14utilsextension_cstr_to_pystr, "PyObject *(char const *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}


#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC inittableextension(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC inittableextension(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_tableextension(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_tableextension(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_tableextension(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'tableextension' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_tableextension(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(WITH_THREAD) && PY_VERSION_HEX < 0x030700F0 && defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  PyEval_InitThreads();
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("tableextension", __pyx_methods, __pyx_k_Here_is_where_Table_and_Row_exte, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_b);
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_cython_runtime);
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_tables__tableextension) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "tables.tableextension")) {
      if (unlikely(PyDict_SetItemString(modules, "tables.tableextension", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  if (unlikely(__Pyx_modinit_type_init_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_type_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_variable_import_code();
  if (unlikely(__Pyx_modinit_function_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "tables/tableextension.pyx":24
 * """
 * 
 * import sys             # <<<<<<<<<<<<<<
 * import numpy
 * from time import time
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_sys, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_sys, __pyx_t_1) < 0) __PYX_ERR(0, 24, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":25
 * 
 * import sys
 * import numpy             # <<<<<<<<<<<<<<
 * from time import time
 * 
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_numpy, __pyx_t_1) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":26
 * import sys
 * import numpy
 * from time import time             # <<<<<<<<<<<<<<
 * 
 * from .description import Col
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_time);
  __Pyx_GIVEREF(__pyx_n_s_time);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_time);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_time, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_time); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_time, __pyx_t_1) < 0) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/tableextension.pyx":28
 * from time import time
 * 
 * from .description import Col             # <<<<<<<<<<<<<<
 * from .exceptions import HDF5ExtError
 * from .conditions import call_on_recarr
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_Col);
  __Pyx_GIVEREF(__pyx_n_s_Col);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_Col);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_description, __pyx_t_2, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_Col); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Col, __pyx_t_2) < 0) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":29
 * 
 * from .description import Col
 * from .exceptions import HDF5ExtError             # <<<<<<<<<<<<<<
 * from .conditions import call_on_recarr
 * from .utilsextension import (get_nested_field, atom_from_hdf5_type,
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_HDF5ExtError);
  __Pyx_GIVEREF(__pyx_n_s_HDF5ExtError);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_HDF5ExtError);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_exceptions, __pyx_t_1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_HDF5ExtError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_HDF5ExtError, __pyx_t_1) < 0) __PYX_ERR(0, 29, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/tableextension.pyx":30
 * from .description import Col
 * from .exceptions import HDF5ExtError
 * from .conditions import call_on_recarr             # <<<<<<<<<<<<<<
 * from .utilsextension import (get_nested_field, atom_from_hdf5_type,
 *   create_nested_type, hdf5_to_np_ext_type, create_nested_type, platform_byteorder,
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_call_on_recarr);
  __Pyx_GIVEREF(__pyx_n_s_call_on_recarr);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_call_on_recarr);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_conditions, __pyx_t_2, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_call_on_recarr); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_call_on_recarr, __pyx_t_2) < 0) __PYX_ERR(0, 30, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":31
 * from .exceptions import HDF5ExtError
 * from .conditions import call_on_recarr
 * from .utilsextension import (get_nested_field, atom_from_hdf5_type,             # <<<<<<<<<<<<<<
 *   create_nested_type, hdf5_to_np_ext_type, create_nested_type, platform_byteorder,
 *   pttype_to_hdf5, pt_special_kinds, npext_prefixes_to_ptkinds, hdf5_class_to_string,
 */
  __pyx_t_1 = PyList_New(11); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_get_nested_field);
  __Pyx_GIVEREF(__pyx_n_s_get_nested_field);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_get_nested_field);
  __Pyx_INCREF(__pyx_n_s_atom_from_hdf5_type);
  __Pyx_GIVEREF(__pyx_n_s_atom_from_hdf5_type);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_s_atom_from_hdf5_type);
  __Pyx_INCREF(__pyx_n_s_create_nested_type);
  __Pyx_GIVEREF(__pyx_n_s_create_nested_type);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_s_create_nested_type);
  __Pyx_INCREF(__pyx_n_s_hdf5_to_np_ext_type);
  __Pyx_GIVEREF(__pyx_n_s_hdf5_to_np_ext_type);
  PyList_SET_ITEM(__pyx_t_1, 3, __pyx_n_s_hdf5_to_np_ext_type);
  __Pyx_INCREF(__pyx_n_s_create_nested_type);
  __Pyx_GIVEREF(__pyx_n_s_create_nested_type);
  PyList_SET_ITEM(__pyx_t_1, 4, __pyx_n_s_create_nested_type);
  __Pyx_INCREF(__pyx_n_s_platform_byteorder);
  __Pyx_GIVEREF(__pyx_n_s_platform_byteorder);
  PyList_SET_ITEM(__pyx_t_1, 5, __pyx_n_s_platform_byteorder);
  __Pyx_INCREF(__pyx_n_s_pttype_to_hdf5);
  __Pyx_GIVEREF(__pyx_n_s_pttype_to_hdf5);
  PyList_SET_ITEM(__pyx_t_1, 6, __pyx_n_s_pttype_to_hdf5);
  __Pyx_INCREF(__pyx_n_s_pt_special_kinds);
  __Pyx_GIVEREF(__pyx_n_s_pt_special_kinds);
  PyList_SET_ITEM(__pyx_t_1, 7, __pyx_n_s_pt_special_kinds);
  __Pyx_INCREF(__pyx_n_s_npext_prefixes_to_ptkinds);
  __Pyx_GIVEREF(__pyx_n_s_npext_prefixes_to_ptkinds);
  PyList_SET_ITEM(__pyx_t_1, 8, __pyx_n_s_npext_prefixes_to_ptkinds);
  __Pyx_INCREF(__pyx_n_s_hdf5_class_to_string);
  __Pyx_GIVEREF(__pyx_n_s_hdf5_class_to_string);
  PyList_SET_ITEM(__pyx_t_1, 9, __pyx_n_s_hdf5_class_to_string);
  __Pyx_INCREF(__pyx_n_s_H5T_STD_I64);
  __Pyx_GIVEREF(__pyx_n_s_H5T_STD_I64);
  PyList_SET_ITEM(__pyx_t_1, 10, __pyx_n_s_H5T_STD_I64);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_utilsextension, __pyx_t_1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_get_nested_field); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_get_nested_field, __pyx_t_1) < 0) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_atom_from_hdf5_type); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_atom_from_hdf5_type, __pyx_t_1) < 0) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_create_nested_type); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_create_nested_type, __pyx_t_1) < 0) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_hdf5_to_np_ext_type); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_hdf5_to_np_ext_type, __pyx_t_1) < 0) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_create_nested_type); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_create_nested_type, __pyx_t_1) < 0) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_platform_byteorder); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_platform_byteorder, __pyx_t_1) < 0) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_pttype_to_hdf5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pttype_to_hdf5, __pyx_t_1) < 0) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_pt_special_kinds); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pt_special_kinds, __pyx_t_1) < 0) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_npext_prefixes_to_ptkinds); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_npext_prefixes_to_ptkinds, __pyx_t_1) < 0) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_hdf5_class_to_string); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_hdf5_class_to_string, __pyx_t_1) < 0) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_H5T_STD_I64); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_H5T_STD_I64, __pyx_t_1) < 0) __PYX_ERR(0, 34, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/tableextension.pyx":35
 *   pttype_to_hdf5, pt_special_kinds, npext_prefixes_to_ptkinds, hdf5_class_to_string,
 *   H5T_STD_I64)
 * from .utils import SizeType             # <<<<<<<<<<<<<<
 * 
 * from .utilsextension cimport get_native_type, cstr_to_pystr
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_SizeType);
  __Pyx_GIVEREF(__pyx_n_s_SizeType);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_SizeType);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_utils, __pyx_t_2, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_SizeType); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SizeType, __pyx_t_2) < 0) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "tables/tableextension.pyx":108
 * # The numpy API requires this function to be called before
 * # using any numpy facilities in an extension module.
 * import_array()             # <<<<<<<<<<<<<<
 * 
 * #-------------------------------------------------------------
 */
  __pyx_t_3 = __pyx_f_5numpy_import_array(); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 108, __pyx_L1_error)

  /* "tables/tableextension.pyx":1
 * ########################################################################             # <<<<<<<<<<<<<<
 * #
 * # License: BSD
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_1) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "../../../../../opt/hostedtoolcache/Python/3.7.12/x64/lib/python3.7/site-packages/numpy/__init__.pxd":1014
 * 
 * 
 * cdef inline NPY_DATETIMEUNIT get_datetime64_unit(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the unit part of the dtype for a numpy datetime64 object.
 */

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init tables.tableextension", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init tables.tableextension");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* ObjectGetItem */
#if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject* index) {
    PyObject *runerr;
    Py_ssize_t key_value;
    PySequenceMethods *m = Py_TYPE(obj)->tp_as_sequence;
    if (unlikely(!(m && m->sq_item))) {
        PyErr_Format(PyExc_TypeError, "'%.200s' object is not subscriptable", Py_TYPE(obj)->tp_name);
        return NULL;
    }
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError, "cannot fit '%.200s' into an index-sized integer", Py_TYPE(index)->tp_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key) {
    PyMappingMethods *m = Py_TYPE(obj)->tp_as_mapping;
    if (likely(m && m->mp_subscript)) {
        return m->mp_subscript(obj, key);
    }
    return __Pyx_PyObject_GetIndex(obj, key);
}
#endif

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_type == NULL || exc_info->exc_type == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* BytesEquals */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        #if PY_MAJOR_VERSION < 3
        Py_XDECREF(owned_ref);
        #endif
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* PyObjectCall2Args */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *result = NULL;
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyFunction_FastCall(function, args, 2);
    }
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyCFunction_FastCall(function, args, 2);
    }
    #endif
    args = PyTuple_New(2);
    if (unlikely(!args)) goto done;
    Py_INCREF(arg1);
    PyTuple_SET_ITEM(args, 0, arg1);
    Py_INCREF(arg2);
    PyTuple_SET_ITEM(args, 1, arg2);
    Py_INCREF(function);
    result = __Pyx_PyObject_Call(function, args, NULL);
    Py_DECREF(args);
    Py_DECREF(function);
done:
    return result;
}

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (__Pyx_PyFastCFunction_Check(func)) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* DictGetItem */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            if (unlikely(PyTuple_Check(key))) {
                PyObject* args = PyTuple_Pack(1, key);
                if (likely(args)) {
                    PyErr_SetObject(PyExc_KeyError, args);
                    Py_DECREF(args);
                }
            } else {
                PyErr_SetObject(PyExc_KeyError, key);
            }
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#endif

/* PyObjectSetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_setattr))
        return tp->tp_setattr(obj, PyString_AS_STRING(attr_name), value);
#endif
    return PyObject_SetAttr(obj, attr_name, value);
}
#endif

/* PyIntBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, int inplace, int zerodivision_check) {
    (void)inplace;
    (void)zerodivision_check;
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
            x = (long)((unsigned long)a + b);
            if (likely((x^a) >= 0 || (x^b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_add(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_add(op1, op2);
            }
        }
                x = a + b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla + llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            double result;
            PyFPE_START_PROTECT("add", return NULL)
            result = ((double)a) + (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceAdd : PyNumber_Add)(op1, op2);
}
#endif

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* IterFinish */
static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* UnpackItemEndCheck */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

/* PyObjectGetMethod */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method) {
    PyObject *attr;
#if CYTHON_UNPACK_METHODS && CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_PYTYPE_LOOKUP
    PyTypeObject *tp = Py_TYPE(obj);
    PyObject *descr;
    descrgetfunc f = NULL;
    PyObject **dictptr, *dict;
    int meth_found = 0;
    assert (*method == NULL);
    if (unlikely(tp->tp_getattro != PyObject_GenericGetAttr)) {
        attr = __Pyx_PyObject_GetAttrStr(obj, name);
        goto try_unpack;
    }
    if (unlikely(tp->tp_dict == NULL) && unlikely(PyType_Ready(tp) < 0)) {
        return 0;
    }
    descr = _PyType_Lookup(tp, name);
    if (likely(descr != NULL)) {
        Py_INCREF(descr);
#if PY_MAJOR_VERSION >= 3
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || (Py_TYPE(descr) == &PyMethodDescr_Type) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr) || (Py_TYPE(descr) == &PyMethodDescr_Type)))
        #endif
#else
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr)))
        #endif
#endif
        {
            meth_found = 1;
        } else {
            f = Py_TYPE(descr)->tp_descr_get;
            if (f != NULL && PyDescr_IsData(descr)) {
                attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
                Py_DECREF(descr);
                goto try_unpack;
            }
        }
    }
    dictptr = _PyObject_GetDictPtr(obj);
    if (dictptr != NULL && (dict = *dictptr) != NULL) {
        Py_INCREF(dict);
        attr = __Pyx_PyDict_GetItemStr(dict, name);
        if (attr != NULL) {
            Py_INCREF(attr);
            Py_DECREF(dict);
            Py_XDECREF(descr);
            goto try_unpack;
        }
        Py_DECREF(dict);
    }
    if (meth_found) {
        *method = descr;
        return 1;
    }
    if (f != NULL) {
        attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
        Py_DECREF(descr);
        goto try_unpack;
    }
    if (descr != NULL) {
        *method = descr;
        return 0;
    }
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(name));
#endif
    return 0;
#else
    attr = __Pyx_PyObject_GetAttrStr(obj, name);
    goto try_unpack;
#endif
try_unpack:
#if CYTHON_UNPACK_METHODS
    if (likely(attr) && PyMethod_Check(attr) && likely(PyMethod_GET_SELF(attr) == obj)) {
        PyObject *function = PyMethod_GET_FUNCTION(attr);
        Py_INCREF(function);
        Py_DECREF(attr);
        *method = function;
        return 1;
    }
#endif
    *method = attr;
    return 0;
}

/* PyObjectCallMethod1 */
static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg) {
    PyObject *result = __Pyx_PyObject_CallOneArg(method, arg);
    Py_DECREF(method);
    return result;
}
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
    PyObject *method = NULL, *result;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_Call2Args(method, obj, arg);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) return NULL;
    return __Pyx__PyObject_CallMethod1(method, arg);
}

/* append */
static CYTHON_INLINE int __Pyx_PyObject_Append(PyObject* L, PyObject* x) {
    if (likely(PyList_CheckExact(L))) {
        if (unlikely(__Pyx_PyList_Append(L, x) < 0)) return -1;
    } else {
        PyObject* retval = __Pyx_PyObject_CallMethod1(L, __pyx_n_s_append, x);
        if (unlikely(!retval))
            return -1;
        Py_DECREF(retval);
    }
    return 0;
}

/* GetAttr */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* HasAttr */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (unlikely(!r)) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* WriteUnraisableException */
static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_CyFunction_Check(func)))
#else
    if (likely(PyCFunction_Check(func)))
#endif
    {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* SliceObject */
static CYTHON_INLINE int __Pyx_PyObject_SetSlice(PyObject* obj, PyObject* value,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp;
#if PY_MAJOR_VERSION < 3
    PySequenceMethods* ms = Py_TYPE(obj)->tp_as_sequence;
    if (likely(ms && ms->sq_ass_slice)) {
        if (!has_cstart) {
            if (_py_start && (*_py_start != Py_None)) {
                cstart = __Pyx_PyIndex_AsSsize_t(*_py_start);
                if ((cstart == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstart = 0;
        }
        if (!has_cstop) {
            if (_py_stop && (*_py_stop != Py_None)) {
                cstop = __Pyx_PyIndex_AsSsize_t(*_py_stop);
                if ((cstop == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstop = PY_SSIZE_T_MAX;
        }
        if (wraparound && unlikely((cstart < 0) | (cstop < 0)) && likely(ms->sq_length)) {
            Py_ssize_t l = ms->sq_length(obj);
            if (likely(l >= 0)) {
                if (cstop < 0) {
                    cstop += l;
                    if (cstop < 0) cstop = 0;
                }
                if (cstart < 0) {
                    cstart += l;
                    if (cstart < 0) cstart = 0;
                }
            } else {
                if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                    goto bad;
                PyErr_Clear();
            }
        }
        return ms->sq_ass_slice(obj, cstart, cstop, value);
    }
#endif
    mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_ass_subscript))
#endif
    {
        int result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyInt_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyInt_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_ass_subscript(obj, py_slice, value);
#else
        result = value ? PyObject_SetItem(obj, py_slice, value) : PyObject_DelItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    PyErr_Format(PyExc_TypeError,
        "'%.200s' object does not support slice %.10s",
        Py_TYPE(obj)->tp_name, value ? "assignment" : "deletion");
bad:
    return -1;
}

/* SliceObject */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(PyObject* obj,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp;
#if PY_MAJOR_VERSION < 3
    PySequenceMethods* ms = Py_TYPE(obj)->tp_as_sequence;
    if (likely(ms && ms->sq_slice)) {
        if (!has_cstart) {
            if (_py_start && (*_py_start != Py_None)) {
                cstart = __Pyx_PyIndex_AsSsize_t(*_py_start);
                if ((cstart == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstart = 0;
        }
        if (!has_cstop) {
            if (_py_stop && (*_py_stop != Py_None)) {
                cstop = __Pyx_PyIndex_AsSsize_t(*_py_stop);
                if ((cstop == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstop = PY_SSIZE_T_MAX;
        }
        if (wraparound && unlikely((cstart < 0) | (cstop < 0)) && likely(ms->sq_length)) {
            Py_ssize_t l = ms->sq_length(obj);
            if (likely(l >= 0)) {
                if (cstop < 0) {
                    cstop += l;
                    if (cstop < 0) cstop = 0;
                }
                if (cstart < 0) {
                    cstart += l;
                    if (cstart < 0) cstart = 0;
                }
            } else {
                if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                    goto bad;
                PyErr_Clear();
            }
        }
        return ms->sq_slice(obj, cstart, cstop);
    }
#endif
    mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_subscript))
#endif
    {
        PyObject* result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyInt_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyInt_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_subscript(obj, py_slice);
#else
        result = PyObject_GetItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    PyErr_Format(PyExc_TypeError,
        "'%.200s' object is unsliceable", Py_TYPE(obj)->tp_name);
bad:
    return NULL;
}

/* None */
static CYTHON_INLINE PY_LONG_LONG __Pyx_mod_PY_LONG_LONG(PY_LONG_LONG a, PY_LONG_LONG b) {
    PY_LONG_LONG r = a % b;
    r += ((r != 0) & ((r ^ b) < 0)) * b;
    return r;
}

/* SetItemInt */
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v) {
    int r;
    if (!j) return -1;
    r = PyObject_SetItem(o, j, v);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v, int is_list,
                                               CYTHON_NCP_UNUSED int wraparound, CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = (!wraparound) ? i : ((likely(i >= 0)) ? i : i + PyList_GET_SIZE(o));
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o)))) {
            PyObject* old = PyList_GET_ITEM(o, n);
            Py_INCREF(v);
            PyList_SET_ITEM(o, n, v);
            Py_DECREF(old);
            return 1;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_ass_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return -1;
                    PyErr_Clear();
                }
            }
            return m->sq_ass_item(o, i, v);
        }
    }
#else
#if CYTHON_COMPILING_IN_PYPY
    if (is_list || (PySequence_Check(o) && !PyDict_Check(o)))
#else
    if (is_list || PySequence_Check(o))
#endif
    {
        return PySequence_SetItem(o, i, v);
    }
#endif
    return __Pyx_SetItemInt_Generic(o, PyInt_FromSsize_t(i), v);
}

/* CallNextTpDealloc */
static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_dealloc != current_tp_dealloc)
        type = type->tp_base;
    while (type && type->tp_dealloc == current_tp_dealloc)
        type = type->tp_base;
    if (type)
        type->tp_dealloc(obj);
}

/* CallNextTpTraverse */
static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_traverse != current_tp_traverse)
        type = type->tp_base;
    while (type && type->tp_traverse == current_tp_traverse)
        type = type->tp_base;
    if (type && type->tp_traverse)
        return type->tp_traverse(obj, v, a);
    return 0;
}

/* CallNextTpClear */
static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_clear) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_clear != current_tp_clear)
        type = type->tp_base;
    while (type && type->tp_clear == current_tp_clear)
        type = type->tp_base;
    if (type && type->tp_clear)
        type->tp_clear(obj);
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(PyObject *module, const char *module_name, const char *class_name,
    size_t size, enum __Pyx_ImportType_CheckSize check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if ((size_t)basicsize < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error && (size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* GetVTable */
static void* __Pyx_GetVtable(PyObject *dict) {
    void* ptr;
    PyObject *ob = PyObject_GetItem(dict, __pyx_n_s_pyx_vtable);
    if (!ob)
        goto bad;
#if PY_VERSION_HEX >= 0x02070000
    ptr = PyCapsule_GetPointer(ob, 0);
#else
    ptr = PyCObject_AsVoidPtr(ob);
#endif
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* PyObject_GenericGetAttrNoDict */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, attr_name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(attr_name));
#endif
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* SetVTable */
static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* PyObjectGetAttrStrNoError */
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS && PY_VERSION_HEX >= 0x030700B1
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
}

/* SetupReduce */
static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_reduce_cython);
            if (likely(reduce_cython)) {
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if ((1) && (strchr(__Pyx_MODULE_NAME, '.'))) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_NCP_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        (void) PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_srcfile = NULL;
    py_srcfile = PyString_FromString(filename);
    if (!py_srcfile) goto bad;
    #endif
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        if (!py_funcname) goto bad;
        #endif
    }
    #if PY_MAJOR_VERSION < 3
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    #else
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    #endif
    Py_XDECREF(py_funcname);  // XDECREF since it's only set on Py3 if cline
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_srcfile);
    #endif
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* Declarations */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return ::std::complex< float >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return x + y*(__pyx_t_float_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      __pyx_t_float_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabsf(b.real) >= fabsf(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                float r = b.imag / b.real;
                float s = (float)(1.0) / (b.real + b.imag * r);
                return __pyx_t_float_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            float r = b.real / b.imag;
            float s = (float)(1.0) / (b.imag + b.real * r);
            return __pyx_t_float_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            float denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_float_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrtf(z.real*z.real + z.imag*z.imag);
          #else
            return hypotf(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
            __pyx_t_float_complex z;
            float r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    float denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_float(a, a);
                    case 3:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, a);
                    case 4:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if (b.imag == 0) {
                    z.real = powf(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2f(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_float(a);
                theta = atan2f(a.imag, a.real);
            }
            lnr = logf(r);
            z_r = expf(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cosf(z_theta);
            z.imag = z_r * sinf(z_theta);
            return z;
        }
    #endif
#endif

/* Declarations */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return ::std::complex< double >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return x + y*(__pyx_t_double_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      __pyx_t_double_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabs(b.real) >= fabs(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                double r = b.imag / b.real;
                double s = (double)(1.0) / (b.real + b.imag * r);
                return __pyx_t_double_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            double r = b.real / b.imag;
            double s = (double)(1.0) / (b.imag + b.real * r);
            return __pyx_t_double_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            double denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_double_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrt(z.real*z.real + z.imag*z.imag);
          #else
            return hypot(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
            __pyx_t_double_complex z;
            double r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    double denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_double(a, a);
                    case 3:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, a);
                    case 4:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if (b.imag == 0) {
                    z.real = pow(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_double(a);
                theta = atan2(a.imag, a.real);
            }
            lnr = log(r);
            z_r = exp(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cos(z_theta);
            z.imag = z_r * sin(z_theta);
            return z;
        }
    #endif
#endif

/* CIntFromPy */
static CYTHON_INLINE hsize_t __Pyx_PyInt_As_hsize_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const hsize_t neg_one = (hsize_t) -1, const_zero = (hsize_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(hsize_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(hsize_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (hsize_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (hsize_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(hsize_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(hsize_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hsize_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hsize_t) >= 2 * PyLong_SHIFT) {
                            return (hsize_t) (((((hsize_t)digits[1]) << PyLong_SHIFT) | (hsize_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(hsize_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hsize_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hsize_t) >= 3 * PyLong_SHIFT) {
                            return (hsize_t) (((((((hsize_t)digits[2]) << PyLong_SHIFT) | (hsize_t)digits[1]) << PyLong_SHIFT) | (hsize_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(hsize_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hsize_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hsize_t) >= 4 * PyLong_SHIFT) {
                            return (hsize_t) (((((((((hsize_t)digits[3]) << PyLong_SHIFT) | (hsize_t)digits[2]) << PyLong_SHIFT) | (hsize_t)digits[1]) << PyLong_SHIFT) | (hsize_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (hsize_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(hsize_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(hsize_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(hsize_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(hsize_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (hsize_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(hsize_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(hsize_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(hsize_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hsize_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hsize_t) - 1 > 2 * PyLong_SHIFT) {
                            return (hsize_t) (((hsize_t)-1)*(((((hsize_t)digits[1]) << PyLong_SHIFT) | (hsize_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(hsize_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hsize_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hsize_t) - 1 > 2 * PyLong_SHIFT) {
                            return (hsize_t) ((((((hsize_t)digits[1]) << PyLong_SHIFT) | (hsize_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(hsize_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hsize_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hsize_t) - 1 > 3 * PyLong_SHIFT) {
                            return (hsize_t) (((hsize_t)-1)*(((((((hsize_t)digits[2]) << PyLong_SHIFT) | (hsize_t)digits[1]) << PyLong_SHIFT) | (hsize_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(hsize_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hsize_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hsize_t) - 1 > 3 * PyLong_SHIFT) {
                            return (hsize_t) ((((((((hsize_t)digits[2]) << PyLong_SHIFT) | (hsize_t)digits[1]) << PyLong_SHIFT) | (hsize_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(hsize_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hsize_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hsize_t) - 1 > 4 * PyLong_SHIFT) {
                            return (hsize_t) (((hsize_t)-1)*(((((((((hsize_t)digits[3]) << PyLong_SHIFT) | (hsize_t)digits[2]) << PyLong_SHIFT) | (hsize_t)digits[1]) << PyLong_SHIFT) | (hsize_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(hsize_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hsize_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hsize_t) - 1 > 4 * PyLong_SHIFT) {
                            return (hsize_t) ((((((((((hsize_t)digits[3]) << PyLong_SHIFT) | (hsize_t)digits[2]) << PyLong_SHIFT) | (hsize_t)digits[1]) << PyLong_SHIFT) | (hsize_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(hsize_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(hsize_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(hsize_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(hsize_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            hsize_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (hsize_t) -1;
        }
    } else {
        hsize_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (hsize_t) -1;
        val = __Pyx_PyInt_As_hsize_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to hsize_t");
    return (hsize_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to hsize_t");
    return (hsize_t) -1;
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
static CYTHON_INLINE hid_t __Pyx_PyInt_As_hid_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const hid_t neg_one = (hid_t) -1, const_zero = (hid_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(hid_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(hid_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (hid_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (hid_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(hid_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(hid_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hid_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hid_t) >= 2 * PyLong_SHIFT) {
                            return (hid_t) (((((hid_t)digits[1]) << PyLong_SHIFT) | (hid_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(hid_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hid_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hid_t) >= 3 * PyLong_SHIFT) {
                            return (hid_t) (((((((hid_t)digits[2]) << PyLong_SHIFT) | (hid_t)digits[1]) << PyLong_SHIFT) | (hid_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(hid_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hid_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hid_t) >= 4 * PyLong_SHIFT) {
                            return (hid_t) (((((((((hid_t)digits[3]) << PyLong_SHIFT) | (hid_t)digits[2]) << PyLong_SHIFT) | (hid_t)digits[1]) << PyLong_SHIFT) | (hid_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (hid_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(hid_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(hid_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(hid_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(hid_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (hid_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(hid_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(hid_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(hid_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hid_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hid_t) - 1 > 2 * PyLong_SHIFT) {
                            return (hid_t) (((hid_t)-1)*(((((hid_t)digits[1]) << PyLong_SHIFT) | (hid_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(hid_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hid_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hid_t) - 1 > 2 * PyLong_SHIFT) {
                            return (hid_t) ((((((hid_t)digits[1]) << PyLong_SHIFT) | (hid_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(hid_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hid_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hid_t) - 1 > 3 * PyLong_SHIFT) {
                            return (hid_t) (((hid_t)-1)*(((((((hid_t)digits[2]) << PyLong_SHIFT) | (hid_t)digits[1]) << PyLong_SHIFT) | (hid_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(hid_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hid_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hid_t) - 1 > 3 * PyLong_SHIFT) {
                            return (hid_t) ((((((((hid_t)digits[2]) << PyLong_SHIFT) | (hid_t)digits[1]) << PyLong_SHIFT) | (hid_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(hid_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hid_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hid_t) - 1 > 4 * PyLong_SHIFT) {
                            return (hid_t) (((hid_t)-1)*(((((((((hid_t)digits[3]) << PyLong_SHIFT) | (hid_t)digits[2]) << PyLong_SHIFT) | (hid_t)digits[1]) << PyLong_SHIFT) | (hid_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(hid_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hid_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hid_t) - 1 > 4 * PyLong_SHIFT) {
                            return (hid_t) ((((((((((hid_t)digits[3]) << PyLong_SHIFT) | (hid_t)digits[2]) << PyLong_SHIFT) | (hid_t)digits[1]) << PyLong_SHIFT) | (hid_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(hid_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(hid_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(hid_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(hid_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            hid_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (hid_t) -1;
        }
    } else {
        hid_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (hid_t) -1;
        val = __Pyx_PyInt_As_hid_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to hid_t");
    return (hid_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to hid_t");
    return (hid_t) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
static CYTHON_INLINE hbool_t __Pyx_PyInt_As_hbool_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const hbool_t neg_one = (hbool_t) -1, const_zero = (hbool_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(hbool_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(hbool_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (hbool_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (hbool_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(hbool_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(hbool_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hbool_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hbool_t) >= 2 * PyLong_SHIFT) {
                            return (hbool_t) (((((hbool_t)digits[1]) << PyLong_SHIFT) | (hbool_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(hbool_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hbool_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hbool_t) >= 3 * PyLong_SHIFT) {
                            return (hbool_t) (((((((hbool_t)digits[2]) << PyLong_SHIFT) | (hbool_t)digits[1]) << PyLong_SHIFT) | (hbool_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(hbool_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hbool_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hbool_t) >= 4 * PyLong_SHIFT) {
                            return (hbool_t) (((((((((hbool_t)digits[3]) << PyLong_SHIFT) | (hbool_t)digits[2]) << PyLong_SHIFT) | (hbool_t)digits[1]) << PyLong_SHIFT) | (hbool_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (hbool_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(hbool_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(hbool_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(hbool_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(hbool_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (hbool_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(hbool_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(hbool_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(hbool_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hbool_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hbool_t) - 1 > 2 * PyLong_SHIFT) {
                            return (hbool_t) (((hbool_t)-1)*(((((hbool_t)digits[1]) << PyLong_SHIFT) | (hbool_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(hbool_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hbool_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hbool_t) - 1 > 2 * PyLong_SHIFT) {
                            return (hbool_t) ((((((hbool_t)digits[1]) << PyLong_SHIFT) | (hbool_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(hbool_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hbool_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hbool_t) - 1 > 3 * PyLong_SHIFT) {
                            return (hbool_t) (((hbool_t)-1)*(((((((hbool_t)digits[2]) << PyLong_SHIFT) | (hbool_t)digits[1]) << PyLong_SHIFT) | (hbool_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(hbool_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hbool_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hbool_t) - 1 > 3 * PyLong_SHIFT) {
                            return (hbool_t) ((((((((hbool_t)digits[2]) << PyLong_SHIFT) | (hbool_t)digits[1]) << PyLong_SHIFT) | (hbool_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(hbool_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hbool_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hbool_t) - 1 > 4 * PyLong_SHIFT) {
                            return (hbool_t) (((hbool_t)-1)*(((((((((hbool_t)digits[3]) << PyLong_SHIFT) | (hbool_t)digits[2]) << PyLong_SHIFT) | (hbool_t)digits[1]) << PyLong_SHIFT) | (hbool_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(hbool_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(hbool_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(hbool_t) - 1 > 4 * PyLong_SHIFT) {
                            return (hbool_t) ((((((((((hbool_t)digits[3]) << PyLong_SHIFT) | (hbool_t)digits[2]) << PyLong_SHIFT) | (hbool_t)digits[1]) << PyLong_SHIFT) | (hbool_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(hbool_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(hbool_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(hbool_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(hbool_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            hbool_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (hbool_t) -1;
        }
    } else {
        hbool_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (hbool_t) -1;
        val = __Pyx_PyInt_As_hbool_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to hbool_t");
    return (hbool_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to hbool_t");
    return (hbool_t) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_hid_t(hid_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const hid_t neg_one = (hid_t) -1, const_zero = (hid_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(hid_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(hid_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(hid_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(hid_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(hid_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(hid_t),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_hsize_t(hsize_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const hsize_t neg_one = (hsize_t) -1, const_zero = (hsize_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(hsize_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(hsize_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(hsize_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(hsize_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(hsize_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(hsize_t),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const size_t neg_one = (size_t) -1, const_zero = (size_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(size_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(size_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (size_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(size_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 2 * PyLong_SHIFT) {
                            return (size_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 3 * PyLong_SHIFT) {
                            return (size_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 4 * PyLong_SHIFT) {
                            return (size_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (size_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(size_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(size_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(size_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(size_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) ((((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) ((((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) ((((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(size_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            size_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (size_t) -1;
        }
    } else {
        size_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (size_t) -1;
        val = __Pyx_PyInt_As_size_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to size_t");
    return (size_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to size_t");
    return (size_t) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__H5T_order_t(enum H5T_order_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const enum H5T_order_t neg_one = (enum H5T_order_t) -1, const_zero = (enum H5T_order_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(enum H5T_order_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enum H5T_order_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum H5T_order_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(enum H5T_order_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum H5T_order_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(enum H5T_order_t),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
static CYTHON_INLINE Py_intptr_t __Pyx_PyInt_As_Py_intptr_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const Py_intptr_t neg_one = (Py_intptr_t) -1, const_zero = (Py_intptr_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(Py_intptr_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(Py_intptr_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (Py_intptr_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (Py_intptr_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(Py_intptr_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(Py_intptr_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) >= 2 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((((Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(Py_intptr_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) >= 3 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((((((Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(Py_intptr_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) >= 4 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((((((((Py_intptr_t)digits[3]) << PyLong_SHIFT) | (Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (Py_intptr_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(Py_intptr_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(Py_intptr_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(Py_intptr_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(Py_intptr_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (Py_intptr_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(Py_intptr_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(Py_intptr_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(Py_intptr_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 2 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((Py_intptr_t)-1)*(((((Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(Py_intptr_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 2 * PyLong_SHIFT) {
                            return (Py_intptr_t) ((((((Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(Py_intptr_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 3 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((Py_intptr_t)-1)*(((((((Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(Py_intptr_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 3 * PyLong_SHIFT) {
                            return (Py_intptr_t) ((((((((Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(Py_intptr_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 4 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((Py_intptr_t)-1)*(((((((((Py_intptr_t)digits[3]) << PyLong_SHIFT) | (Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(Py_intptr_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 4 * PyLong_SHIFT) {
                            return (Py_intptr_t) ((((((((((Py_intptr_t)digits[3]) << PyLong_SHIFT) | (Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(Py_intptr_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(Py_intptr_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(Py_intptr_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(Py_intptr_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            Py_intptr_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (Py_intptr_t) -1;
        }
    } else {
        Py_intptr_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (Py_intptr_t) -1;
        val = __Pyx_PyInt_As_Py_intptr_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to Py_intptr_t");
    return (Py_intptr_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to Py_intptr_t");
    return (Py_intptr_t) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_PY_LONG_LONG(PY_LONG_LONG value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const PY_LONG_LONG neg_one = (PY_LONG_LONG) -1, const_zero = (PY_LONG_LONG) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(PY_LONG_LONG) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(PY_LONG_LONG) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(PY_LONG_LONG) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(PY_LONG_LONG) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(PY_LONG_LONG) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(PY_LONG_LONG),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
static CYTHON_INLINE PY_LONG_LONG __Pyx_PyInt_As_PY_LONG_LONG(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const PY_LONG_LONG neg_one = (PY_LONG_LONG) -1, const_zero = (PY_LONG_LONG) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(PY_LONG_LONG) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(PY_LONG_LONG, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (PY_LONG_LONG) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (PY_LONG_LONG) 0;
                case  1: __PYX_VERIFY_RETURN_INT(PY_LONG_LONG, digit, digits[0])
                case 2:
                    if (8 * sizeof(PY_LONG_LONG) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(PY_LONG_LONG, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(PY_LONG_LONG) >= 2 * PyLong_SHIFT) {
                            return (PY_LONG_LONG) (((((PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(PY_LONG_LONG) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(PY_LONG_LONG, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(PY_LONG_LONG) >= 3 * PyLong_SHIFT) {
                            return (PY_LONG_LONG) (((((((PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(PY_LONG_LONG) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(PY_LONG_LONG, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(PY_LONG_LONG) >= 4 * PyLong_SHIFT) {
                            return (PY_LONG_LONG) (((((((((PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (PY_LONG_LONG) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(PY_LONG_LONG) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(PY_LONG_LONG, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(PY_LONG_LONG) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(PY_LONG_LONG, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (PY_LONG_LONG) 0;
                case -1: __PYX_VERIFY_RETURN_INT(PY_LONG_LONG, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(PY_LONG_LONG,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(PY_LONG_LONG) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(PY_LONG_LONG, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                            return (PY_LONG_LONG) (((PY_LONG_LONG)-1)*(((((PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(PY_LONG_LONG) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(PY_LONG_LONG, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                            return (PY_LONG_LONG) ((((((PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(PY_LONG_LONG, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                            return (PY_LONG_LONG) (((PY_LONG_LONG)-1)*(((((((PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(PY_LONG_LONG) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(PY_LONG_LONG, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                            return (PY_LONG_LONG) ((((((((PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(PY_LONG_LONG, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                            return (PY_LONG_LONG) (((PY_LONG_LONG)-1)*(((((((((PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(PY_LONG_LONG) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(PY_LONG_LONG, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                            return (PY_LONG_LONG) ((((((((((PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(PY_LONG_LONG) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(PY_LONG_LONG, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(PY_LONG_LONG) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(PY_LONG_LONG, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            PY_LONG_LONG val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (PY_LONG_LONG) -1;
        }
    } else {
        PY_LONG_LONG val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (PY_LONG_LONG) -1;
        val = __Pyx_PyInt_As_PY_LONG_LONG(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to PY_LONG_LONG");
    return (PY_LONG_LONG) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to PY_LONG_LONG");
    return (PY_LONG_LONG) -1;
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* FunctionImport */
#ifndef __PYX_HAVE_RT_ImportFunction
#define __PYX_HAVE_RT_ImportFunction
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, funcname);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C function %.200s",
                PyModule_GetName(module), funcname);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    tmp.p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, desc);
        goto bad;
    }
    tmp.p = PyCObject_AsVoidPtr(cobj);}
#endif
    *f = tmp.fp;
    if (!(*f))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* InitStrings */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
#if PY_MAJOR_VERSION < 3
  } else if (likely(PyInt_CheckExact(o))) {
    return PyInt_AS_LONG(o);
#endif
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyInt_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
